/*  This file is part of the KDE project
    SPDX-FileCopyrightText: 2010 Dag Andersen <dag.andersen@kdemail.net>

    SPDX-License-Identifier: GPL-2.0-or-later
*/


// clazy:excludeall=qstring-arg
#include "kptpackage.h"

using namespace KPlato;

Package::Package()
    : task(nullptr)
    , toTask(nullptr)
{
    qInfo()<<Q_FUNC_INFO<<this;
}

Package::~Package()
{
    qInfo()<<Q_FUNC_INFO<<this;
}
