/********************************************************************************
** Form generated from reading UI file 'kptworkpackageconfigpanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTWORKPACKAGECONFIGPANEL_H
#define UI_KPTWORKPACKAGECONFIGPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kurlrequester.h"

namespace KPlato {

class Ui_WorkPackageConfigPanel
{
public:
    QVBoxLayout *verticalLayout_4;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QCheckBox *kcfg_CheckForWorkPackages;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    KUrlRequester *kcfg_RetrieveUrl;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QRadioButton *kcfg_DeleteFile;
    QRadioButton *kcfg_SaveFile;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_2;
    KUrlRequester *kcfg_SaveUrl;
    QGroupBox *ui_publishGroup;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_5;
    KUrlRequester *kcfg_PublishUrl;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPlato__WorkPackageConfigPanel)
    {
        if (KPlato__WorkPackageConfigPanel->objectName().isEmpty())
            KPlato__WorkPackageConfigPanel->setObjectName("KPlato__WorkPackageConfigPanel");
        KPlato__WorkPackageConfigPanel->resize(490, 405);
        verticalLayout_4 = new QVBoxLayout(KPlato__WorkPackageConfigPanel);
        verticalLayout_4->setObjectName("verticalLayout_4");
        groupBox_2 = new QGroupBox(KPlato__WorkPackageConfigPanel);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setObjectName("verticalLayout");
        kcfg_CheckForWorkPackages = new QCheckBox(groupBox_2);
        kcfg_CheckForWorkPackages->setObjectName("kcfg_CheckForWorkPackages");

        verticalLayout->addWidget(kcfg_CheckForWorkPackages);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(groupBox_2);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        kcfg_RetrieveUrl = new KUrlRequester(groupBox_2);
        kcfg_RetrieveUrl->setObjectName("kcfg_RetrieveUrl");

        horizontalLayout->addWidget(kcfg_RetrieveUrl);


        verticalLayout->addLayout(horizontalLayout);


        verticalLayout_4->addWidget(groupBox_2);

        groupBox = new QGroupBox(KPlato__WorkPackageConfigPanel);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        kcfg_DeleteFile = new QRadioButton(groupBox);
        kcfg_DeleteFile->setObjectName("kcfg_DeleteFile");

        verticalLayout_2->addWidget(kcfg_DeleteFile);

        kcfg_SaveFile = new QRadioButton(groupBox);
        kcfg_SaveFile->setObjectName("kcfg_SaveFile");

        verticalLayout_2->addWidget(kcfg_SaveFile);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");

        horizontalLayout_2->addWidget(label_2);

        kcfg_SaveUrl = new KUrlRequester(groupBox);
        kcfg_SaveUrl->setObjectName("kcfg_SaveUrl");

        horizontalLayout_2->addWidget(kcfg_SaveUrl);


        verticalLayout_2->addLayout(horizontalLayout_2);


        verticalLayout_4->addWidget(groupBox);

        ui_publishGroup = new QGroupBox(KPlato__WorkPackageConfigPanel);
        ui_publishGroup->setObjectName("ui_publishGroup");
        verticalLayout_3 = new QVBoxLayout(ui_publishGroup);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label_5 = new QLabel(ui_publishGroup);
        label_5->setObjectName("label_5");

        horizontalLayout_5->addWidget(label_5);

        kcfg_PublishUrl = new KUrlRequester(ui_publishGroup);
        kcfg_PublishUrl->setObjectName("kcfg_PublishUrl");

        horizontalLayout_5->addWidget(kcfg_PublishUrl);


        verticalLayout_3->addLayout(horizontalLayout_5);


        verticalLayout_4->addWidget(ui_publishGroup);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);


        retranslateUi(KPlato__WorkPackageConfigPanel);
        QObject::connect(kcfg_SaveFile, SIGNAL(toggled(bool)), kcfg_SaveUrl, SLOT(setEnabled(bool)));

        QMetaObject::connectSlotsByName(KPlato__WorkPackageConfigPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__WorkPackageConfigPanel)
    {
        groupBox_2->setTitle(tr2i18n("Retrieve", nullptr));
        kcfg_CheckForWorkPackages->setText(tr2i18n("Check for work packages", nullptr));
        label->setText(tr2i18n("Retrieve:", nullptr));
        groupBox->setTitle(tr2i18n("Archive", nullptr));
        kcfg_DeleteFile->setText(tr2i18n("Delete file after it has been retrieved", nullptr));
        kcfg_SaveFile->setText(tr2i18n("Archive file after it has been retrieved", nullptr));
        label_2->setText(tr2i18n("Archive:", nullptr));
        ui_publishGroup->setTitle(tr2i18n("Publish", nullptr));
        label_5->setText(tr2i18n("Publish:", nullptr));
        (void)KPlato__WorkPackageConfigPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class WorkPackageConfigPanel: public Ui_WorkPackageConfigPanel {};
} // namespace Ui
} // namespace KPlato

#endif // KPTWORKPACKAGECONFIGPANEL_H

