/****************************************************************************
** Meta object code from reading C++ file 'kptintervaledit.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/kptintervaledit.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptintervaledit.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato16IntervalEditImplE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::IntervalEditImpl::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato16IntervalEditImplE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::IntervalEditImpl",
        "changed",
        "",
        "slotClearClicked",
        "slotAddIntervalClicked",
        "slotRemoveIntervalClicked",
        "slotIntervalSelectionChanged",
        "enableButtons"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotClearClicked'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAddIntervalClicked'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRemoveIntervalClicked'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotIntervalSelectionChanged'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'enableButtons'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IntervalEditImpl, qt_meta_tag_ZN6KPlato16IntervalEditImplE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::IntervalEditImpl::staticMetaObject = { {
    QMetaObject::SuperData::link<IntervalEditBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16IntervalEditImplE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16IntervalEditImplE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato16IntervalEditImplE_t>.metaTypes,
    nullptr
} };

void KPlato::IntervalEditImpl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<IntervalEditImpl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: _t->slotClearClicked(); break;
        case 2: _t->slotAddIntervalClicked(); break;
        case 3: _t->slotRemoveIntervalClicked(); break;
        case 4: _t->slotIntervalSelectionChanged(); break;
        case 5: _t->enableButtons(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (IntervalEditImpl::*)()>(_a, &IntervalEditImpl::changed, 0))
            return;
    }
}

const QMetaObject *KPlato::IntervalEditImpl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::IntervalEditImpl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16IntervalEditImplE_t>.strings))
        return static_cast<void*>(this);
    return IntervalEditBase::qt_metacast(_clname);
}

int KPlato::IntervalEditImpl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = IntervalEditBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void KPlato::IntervalEditImpl::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN6KPlato12IntervalEditE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::IntervalEdit::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato12IntervalEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::IntervalEdit"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IntervalEdit, qt_meta_tag_ZN6KPlato12IntervalEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::IntervalEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<IntervalEditImpl::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12IntervalEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12IntervalEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato12IntervalEditE_t>.metaTypes,
    nullptr
} };

void KPlato::IntervalEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<IntervalEdit *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::IntervalEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::IntervalEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12IntervalEditE_t>.strings))
        return static_cast<void*>(this);
    return IntervalEditImpl::qt_metacast(_clname);
}

int KPlato::IntervalEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = IntervalEditImpl::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato18IntervalEditDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::IntervalEditDialog::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato18IntervalEditDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::IntervalEditDialog",
        "slotChanged",
        "",
        "slotCalendarRemoved",
        "const KPlato::Calendar*",
        "cal"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotChanged'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCalendarRemoved'
        QtMocHelpers::SlotData<void(const KPlato::Calendar *)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IntervalEditDialog, qt_meta_tag_ZN6KPlato18IntervalEditDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::IntervalEditDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<KoDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18IntervalEditDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18IntervalEditDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato18IntervalEditDialogE_t>.metaTypes,
    nullptr
} };

void KPlato::IntervalEditDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<IntervalEditDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotChanged(); break;
        case 1: _t->slotCalendarRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::Calendar*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::IntervalEditDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::IntervalEditDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18IntervalEditDialogE_t>.strings))
        return static_cast<void*>(this);
    return KoDialog::qt_metacast(_clname);
}

int KPlato::IntervalEditDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KoDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
