/****************************************************************************
** Meta object code from reading C++ file 'part.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../calligraplan-4.0.1/src/workpackage/part.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'part.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KPlatoWork13DocumentChildE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlatoWork::DocumentChild::qt_create_metaobjectdata<qt_meta_tag_ZN10KPlatoWork13DocumentChildE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlatoWork::DocumentChild",
        "modified",
        "",
        "fileModified",
        "setModified",
        "mod",
        "slotEditFinished",
        "QProcess::ExitStatus",
        "slotEditError",
        "QProcess::ProcessError",
        "status",
        "slotDirty",
        "file",
        "slotUpdateModified"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'modified'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'fileModified'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setModified'
        QtMocHelpers::SlotData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'slotEditFinished'
        QtMocHelpers::SlotData<void(int, QProcess::ExitStatus)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { 0x80000000 | 7, 2 },
        }}),
        // Slot 'slotEditError'
        QtMocHelpers::SlotData<void(QProcess::ProcessError)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotDirty'
        QtMocHelpers::SlotData<void(const QString &)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'slotUpdateModified'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DocumentChild, qt_meta_tag_ZN10KPlatoWork13DocumentChildE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlatoWork::DocumentChild::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork13DocumentChildE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork13DocumentChildE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KPlatoWork13DocumentChildE_t>.metaTypes,
    nullptr
} };

void KPlatoWork::DocumentChild::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DocumentChild *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->modified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->fileModified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->setModified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->slotEditFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QProcess::ExitStatus>>(_a[2]))); break;
        case 4: _t->slotEditError((*reinterpret_cast<std::add_pointer_t<QProcess::ProcessError>>(_a[1]))); break;
        case 5: _t->slotDirty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->slotUpdateModified(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DocumentChild::*)(bool )>(_a, &DocumentChild::modified, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentChild::*)(bool )>(_a, &DocumentChild::fileModified, 1))
            return;
    }
}

const QMetaObject *KPlatoWork::DocumentChild::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlatoWork::DocumentChild::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork13DocumentChildE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KPlatoWork::DocumentChild::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void KPlatoWork::DocumentChild::modified(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlatoWork::DocumentChild::fileModified(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN10KPlatoWork4PartE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlatoWork::Part::qt_create_metaobjectdata<qt_meta_tag_ZN10KPlatoWork4PartE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlatoWork::Part",
        "changed",
        "",
        "workPackageAdded",
        "KPlatoWork::WorkPackage*",
        "package",
        "index",
        "workPackageRemoved",
        "wp",
        "captionChanged",
        "setDocumentClean",
        "clean",
        "setModified",
        "mod",
        "saveModifiedWorkPackages",
        "saveWorkPackage",
        "addCommand",
        "KUndo2Command*",
        "cmd",
        "viewWorkpackageDocument",
        "KPlato::Document*",
        "doc"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'workPackageAdded'
        QtMocHelpers::SignalData<void(KPlatoWork::WorkPackage *, int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'workPackageRemoved'
        QtMocHelpers::SignalData<void(KPlatoWork::WorkPackage *, int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 8 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'captionChanged'
        QtMocHelpers::SignalData<void(const QString &, bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { QMetaType::Bool, 2 },
        }}),
        // Slot 'setDocumentClean'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'setModified'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'saveModifiedWorkPackages'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveWorkPackage'
        QtMocHelpers::SlotData<void(KPlatoWork::WorkPackage *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 8 },
        }}),
        // Slot 'addCommand'
        QtMocHelpers::SlotData<void(KUndo2Command *)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'viewWorkpackageDocument'
        QtMocHelpers::SlotData<void(KPlato::Document *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Part, qt_meta_tag_ZN10KPlatoWork4PartE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlatoWork::Part::staticMetaObject = { {
    QMetaObject::SuperData::link<KParts::ReadWritePart::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork4PartE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork4PartE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KPlatoWork4PartE_t>.metaTypes,
    nullptr
} };

void KPlatoWork::Part::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Part *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: _t->workPackageAdded((*reinterpret_cast<std::add_pointer_t<KPlatoWork::WorkPackage*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->workPackageRemoved((*reinterpret_cast<std::add_pointer_t<KPlatoWork::WorkPackage*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->captionChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->setDocumentClean((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->setModified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->saveModifiedWorkPackages(); break;
        case 7: _t->saveWorkPackage((*reinterpret_cast<std::add_pointer_t<KPlatoWork::WorkPackage*>>(_a[1]))); break;
        case 8: _t->addCommand((*reinterpret_cast<std::add_pointer_t<KUndo2Command*>>(_a[1]))); break;
        case 9: _t->viewWorkpackageDocument((*reinterpret_cast<std::add_pointer_t<KPlato::Document*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Part::*)()>(_a, &Part::changed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Part::*)(KPlatoWork::WorkPackage * , int )>(_a, &Part::workPackageAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Part::*)(KPlatoWork::WorkPackage * , int )>(_a, &Part::workPackageRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Part::*)(const QString & , bool )>(_a, &Part::captionChanged, 3))
            return;
    }
}

const QMetaObject *KPlatoWork::Part::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlatoWork::Part::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork4PartE_t>.strings))
        return static_cast<void*>(this);
    return KParts::ReadWritePart::qt_metacast(_clname);
}

int KPlatoWork::Part::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KParts::ReadWritePart::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void KPlatoWork::Part::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KPlatoWork::Part::workPackageAdded(KPlatoWork::WorkPackage * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KPlatoWork::Part::workPackageRemoved(KPlatoWork::WorkPackage * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void KPlatoWork::Part::captionChanged(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
QT_WARNING_POP
