/********************************************************************************
** Form generated from reading UI file 'PerformanceStatus.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PERFORMANCESTATUS_H
#define UI_PERFORMANCESTATUS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "performance/KPlatoChart.h"
#include "performance/PerformanceTableView.h"

namespace KPlato {

class Ui_PerformanceStatus
{
public:
    QVBoxLayout *verticalLayout_4;
    QStackedWidget *ui_stack;
    QWidget *page_7;
    QVBoxLayout *verticalLayout_5;
    PerformanceTableView *ui_performancetable;
    QFrame *ui_chartframe;
    QVBoxLayout *verticalLayout;
    KPlato::Chart *ui_chart;
    QWidget *for_debug;
    QVBoxLayout *verticalLayout_7;
    QTableView *ui_tableView;

    void setupUi(QWidget *KPlato__PerformanceStatus)
    {
        if (KPlato__PerformanceStatus->objectName().isEmpty())
            KPlato__PerformanceStatus->setObjectName("KPlato__PerformanceStatus");
        KPlato__PerformanceStatus->resize(600, 400);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(KPlato__PerformanceStatus->sizePolicy().hasHeightForWidth());
        KPlato__PerformanceStatus->setSizePolicy(sizePolicy);
        KPlato__PerformanceStatus->setFocusPolicy(Qt::NoFocus);
        KPlato__PerformanceStatus->setContextMenuPolicy(Qt::PreventContextMenu);
        verticalLayout_4 = new QVBoxLayout(KPlato__PerformanceStatus);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        ui_stack = new QStackedWidget(KPlato__PerformanceStatus);
        ui_stack->setObjectName("ui_stack");
        ui_stack->setContextMenuPolicy(Qt::CustomContextMenu);
        page_7 = new QWidget();
        page_7->setObjectName("page_7");
        page_7->setContextMenuPolicy(Qt::NoContextMenu);
        verticalLayout_5 = new QVBoxLayout(page_7);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        ui_performancetable = new PerformanceTableView(page_7);
        ui_performancetable->setObjectName("ui_performancetable");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(ui_performancetable->sizePolicy().hasHeightForWidth());
        ui_performancetable->setSizePolicy(sizePolicy1);
        ui_performancetable->setContextMenuPolicy(Qt::CustomContextMenu);
        ui_performancetable->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        ui_performancetable->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        ui_performancetable->setSelectionMode(QAbstractItemView::NoSelection);
        ui_performancetable->verticalHeader()->setDefaultSectionSize(30);

        verticalLayout_5->addWidget(ui_performancetable);

        ui_chartframe = new QFrame(page_7);
        ui_chartframe->setObjectName("ui_chartframe");
        ui_chartframe->setContextMenuPolicy(Qt::PreventContextMenu);
        ui_chartframe->setFrameShape(QFrame::StyledPanel);
        ui_chartframe->setFrameShadow(QFrame::Sunken);
        verticalLayout = new QVBoxLayout(ui_chartframe);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        ui_chart = new KPlato::Chart(ui_chartframe);
        ui_chart->setObjectName("ui_chart");
        ui_chart->setMinimumSize(QSize(0, 100));
        ui_chart->setContextMenuPolicy(Qt::DefaultContextMenu);

        verticalLayout->addWidget(ui_chart);


        verticalLayout_5->addWidget(ui_chartframe);

        ui_stack->addWidget(page_7);
        for_debug = new QWidget();
        for_debug->setObjectName("for_debug");
        verticalLayout_7 = new QVBoxLayout(for_debug);
        verticalLayout_7->setObjectName("verticalLayout_7");
        verticalLayout_7->setContentsMargins(0, 0, 0, 0);
        ui_tableView = new QTableView(for_debug);
        ui_tableView->setObjectName("ui_tableView");

        verticalLayout_7->addWidget(ui_tableView);

        ui_stack->addWidget(for_debug);

        verticalLayout_4->addWidget(ui_stack);


        retranslateUi(KPlato__PerformanceStatus);

        ui_stack->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KPlato__PerformanceStatus);
    } // setupUi

    void retranslateUi(QWidget *KPlato__PerformanceStatus)
    {
        (void)KPlato__PerformanceStatus;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class PerformanceStatus: public Ui_PerformanceStatus {};
} // namespace Ui
} // namespace KPlato

#endif // PERFORMANCESTATUS_H

