/********************************************************************************
** Form generated from reading UI file 'PerformanceStatusViewSettingsPanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PERFORMANCESTATUSVIEWSETTINGSPANEL_H
#define UI_PERFORMANCESTATUSVIEWSETTINGSPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_PerformanceStatusViewSettingsPanel
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout_2;
    QRadioButton *ui_linechart;
    QRadioButton *ui_barchart;
    QRadioButton *ui_table;
    QGroupBox *groupBox_2;
    QHBoxLayout *horizontalLayout_3;
    QRadioButton *ui_showbasevalues;
    QRadioButton *ui_showindices;
    QVBoxLayout *verticalLayout_4;
    QStackedWidget *ui_stackWidget;
    QWidget *page;
    QHBoxLayout *horizontalLayout;
    QGroupBox *ui_cost;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *ui_bcwsCost;
    QCheckBox *ui_bcwpCost;
    QCheckBox *ui_acwpCost;
    QGroupBox *ui_effort;
    QVBoxLayout *verticalLayout_6;
    QCheckBox *ui_bcwsEffort;
    QCheckBox *ui_bcwpEffort;
    QCheckBox *ui_acwpEffort;
    QWidget *page_2;
    QGridLayout *gridLayout;
    QCheckBox *ui_spicost;
    QCheckBox *ui_spieffort;
    QCheckBox *ui_cpieffort;
    QCheckBox *ui_cpicost;
    QSpacerItem *verticalSpacer_2;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPlato__PerformanceStatusViewSettingsPanel)
    {
        if (KPlato__PerformanceStatusViewSettingsPanel->objectName().isEmpty())
            KPlato__PerformanceStatusViewSettingsPanel->setObjectName("KPlato__PerformanceStatusViewSettingsPanel");
        KPlato__PerformanceStatusViewSettingsPanel->resize(312, 233);
        verticalLayout = new QVBoxLayout(KPlato__PerformanceStatusViewSettingsPanel);
        verticalLayout->setObjectName("verticalLayout");
        groupBox = new QGroupBox(KPlato__PerformanceStatusViewSettingsPanel);
        groupBox->setObjectName("groupBox");
        horizontalLayout_2 = new QHBoxLayout(groupBox);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        ui_linechart = new QRadioButton(groupBox);
        ui_linechart->setObjectName("ui_linechart");
        ui_linechart->setChecked(true);

        horizontalLayout_2->addWidget(ui_linechart);

        ui_barchart = new QRadioButton(groupBox);
        ui_barchart->setObjectName("ui_barchart");

        horizontalLayout_2->addWidget(ui_barchart);

        ui_table = new QRadioButton(groupBox);
        ui_table->setObjectName("ui_table");

        horizontalLayout_2->addWidget(ui_table);


        verticalLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(KPlato__PerformanceStatusViewSettingsPanel);
        groupBox_2->setObjectName("groupBox_2");
        groupBox_2->setFlat(true);
        horizontalLayout_3 = new QHBoxLayout(groupBox_2);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        ui_showbasevalues = new QRadioButton(groupBox_2);
        ui_showbasevalues->setObjectName("ui_showbasevalues");
        ui_showbasevalues->setChecked(true);

        horizontalLayout_3->addWidget(ui_showbasevalues);

        ui_showindices = new QRadioButton(groupBox_2);
        ui_showindices->setObjectName("ui_showindices");

        horizontalLayout_3->addWidget(ui_showindices);


        verticalLayout->addWidget(groupBox_2);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        ui_stackWidget = new QStackedWidget(KPlato__PerformanceStatusViewSettingsPanel);
        ui_stackWidget->setObjectName("ui_stackWidget");
        page = new QWidget();
        page->setObjectName("page");
        horizontalLayout = new QHBoxLayout(page);
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout->setObjectName("horizontalLayout");
        ui_cost = new QGroupBox(page);
        ui_cost->setObjectName("ui_cost");
        ui_cost->setFlat(false);
        ui_cost->setCheckable(true);
        ui_cost->setChecked(true);
        verticalLayout_3 = new QVBoxLayout(ui_cost);
        verticalLayout_3->setObjectName("verticalLayout_3");
        ui_bcwsCost = new QCheckBox(ui_cost);
        ui_bcwsCost->setObjectName("ui_bcwsCost");
        ui_bcwsCost->setChecked(true);

        verticalLayout_3->addWidget(ui_bcwsCost);

        ui_bcwpCost = new QCheckBox(ui_cost);
        ui_bcwpCost->setObjectName("ui_bcwpCost");
        ui_bcwpCost->setChecked(true);

        verticalLayout_3->addWidget(ui_bcwpCost);

        ui_acwpCost = new QCheckBox(ui_cost);
        ui_acwpCost->setObjectName("ui_acwpCost");
        ui_acwpCost->setChecked(true);

        verticalLayout_3->addWidget(ui_acwpCost);

        ui_acwpCost->raise();
        ui_bcwpCost->raise();
        ui_bcwsCost->raise();

        horizontalLayout->addWidget(ui_cost);

        ui_effort = new QGroupBox(page);
        ui_effort->setObjectName("ui_effort");
        ui_effort->setFlat(false);
        ui_effort->setCheckable(true);
        verticalLayout_6 = new QVBoxLayout(ui_effort);
        verticalLayout_6->setObjectName("verticalLayout_6");
        ui_bcwsEffort = new QCheckBox(ui_effort);
        ui_bcwsEffort->setObjectName("ui_bcwsEffort");
        ui_bcwsEffort->setChecked(true);

        verticalLayout_6->addWidget(ui_bcwsEffort);

        ui_bcwpEffort = new QCheckBox(ui_effort);
        ui_bcwpEffort->setObjectName("ui_bcwpEffort");
        ui_bcwpEffort->setChecked(true);

        verticalLayout_6->addWidget(ui_bcwpEffort);

        ui_acwpEffort = new QCheckBox(ui_effort);
        ui_acwpEffort->setObjectName("ui_acwpEffort");
        ui_acwpEffort->setChecked(true);

        verticalLayout_6->addWidget(ui_acwpEffort);

        ui_bcwpEffort->raise();
        ui_acwpEffort->raise();
        ui_bcwsEffort->raise();

        horizontalLayout->addWidget(ui_effort);

        ui_stackWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        gridLayout = new QGridLayout(page_2);
        gridLayout->setObjectName("gridLayout");
        ui_spicost = new QCheckBox(page_2);
        ui_spicost->setObjectName("ui_spicost");
        ui_spicost->setChecked(true);

        gridLayout->addWidget(ui_spicost, 0, 0, 1, 1);

        ui_spieffort = new QCheckBox(page_2);
        ui_spieffort->setObjectName("ui_spieffort");

        gridLayout->addWidget(ui_spieffort, 0, 1, 1, 1);

        ui_cpieffort = new QCheckBox(page_2);
        ui_cpieffort->setObjectName("ui_cpieffort");

        gridLayout->addWidget(ui_cpieffort, 1, 1, 1, 1);

        ui_cpicost = new QCheckBox(page_2);
        ui_cpicost->setObjectName("ui_cpicost");
        ui_cpicost->setChecked(true);

        gridLayout->addWidget(ui_cpicost, 1, 0, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer_2, 2, 0, 1, 1);

        ui_stackWidget->addWidget(page_2);
        ui_spicost->raise();
        ui_cpicost->raise();
        ui_spieffort->raise();
        ui_cpieffort->raise();

        verticalLayout_4->addWidget(ui_stackWidget);


        verticalLayout->addLayout(verticalLayout_4);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        QWidget::setTabOrder(ui_linechart, ui_barchart);
        QWidget::setTabOrder(ui_barchart, ui_table);
        QWidget::setTabOrder(ui_table, ui_showbasevalues);
        QWidget::setTabOrder(ui_showbasevalues, ui_showindices);
        QWidget::setTabOrder(ui_showindices, ui_spicost);
        QWidget::setTabOrder(ui_spicost, ui_cpicost);
        QWidget::setTabOrder(ui_cpicost, ui_spieffort);
        QWidget::setTabOrder(ui_spieffort, ui_cpieffort);
        QWidget::setTabOrder(ui_cpieffort, ui_cost);
        QWidget::setTabOrder(ui_cost, ui_effort);
        QWidget::setTabOrder(ui_effort, ui_bcwsCost);
        QWidget::setTabOrder(ui_bcwsCost, ui_bcwsEffort);
        QWidget::setTabOrder(ui_bcwsEffort, ui_bcwpCost);
        QWidget::setTabOrder(ui_bcwpCost, ui_bcwpEffort);
        QWidget::setTabOrder(ui_bcwpEffort, ui_acwpCost);
        QWidget::setTabOrder(ui_acwpCost, ui_acwpEffort);

        retranslateUi(KPlato__PerformanceStatusViewSettingsPanel);

        ui_stackWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KPlato__PerformanceStatusViewSettingsPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__PerformanceStatusViewSettingsPanel)
    {
        groupBox->setTitle(tr2i18n("Chart type", nullptr));
        ui_linechart->setText(tr2i18n("Line chart", nullptr));
        ui_barchart->setText(tr2i18n("Bar chart", nullptr));
        ui_table->setText(tr2i18n("Table", nullptr));
        groupBox_2->setTitle(QString());
        ui_showbasevalues->setText(tr2i18n("Base values", nullptr));
        ui_showindices->setText(tr2i18n("Indices", nullptr));
        ui_cost->setTitle(tr2i18n("Cost", nullptr));
        ui_bcwsCost->setText(tr2i18n("Show BCWS plot", nullptr));
        ui_bcwpCost->setText(tr2i18n("Show BCWP plot", nullptr));
        ui_acwpCost->setText(tr2i18n("Show ACWP plot", nullptr));
        ui_effort->setTitle(tr2i18n("Effort", nullptr));
        ui_bcwsEffort->setText(tr2i18n("Show BCWS plot", nullptr));
        ui_bcwpEffort->setText(tr2i18n("Show BCWP plot", nullptr));
        ui_acwpEffort->setText(tr2i18n("Show ACWP plot", nullptr));
        ui_spicost->setText(tr2i18n("SPI Cost", nullptr));
        ui_spieffort->setText(tr2i18n("SPI Effort", nullptr));
        ui_cpieffort->setText(tr2i18n("CPI Effort", nullptr));
        ui_cpicost->setText(tr2i18n("CPI Cost", nullptr));
        (void)KPlato__PerformanceStatusViewSettingsPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class PerformanceStatusViewSettingsPanel: public Ui_PerformanceStatusViewSettingsPanel {};
} // namespace Ui
} // namespace KPlato

#endif // PERFORMANCESTATUSVIEWSETTINGSPANEL_H

