/****************************************************************************
** Meta object code from reading C++ file 'kptviewlist.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../calligraplan-4.0.1/src/plan/kptviewlist.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptviewlist.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato18ViewListTreeWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ViewListTreeWidget::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato18ViewListTreeWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ViewListTreeWidget",
        "updateViewInfo",
        "",
        "KPlato::ViewListItem*",
        "itm",
        "modified",
        "handleMousePress",
        "QTreeWidgetItem*",
        "item"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updateViewInfo'
        QtMocHelpers::SignalData<void(KPlato::ViewListItem *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'modified'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleMousePress'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ViewListTreeWidget, qt_meta_tag_ZN6KPlato18ViewListTreeWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ViewListTreeWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18ViewListTreeWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18ViewListTreeWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato18ViewListTreeWidgetE_t>.metaTypes,
    nullptr
} };

void KPlato::ViewListTreeWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ViewListTreeWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updateViewInfo((*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[1]))); break;
        case 1: _t->modified(); break;
        case 2: _t->handleMousePress((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ViewListTreeWidget::*)(KPlato::ViewListItem * )>(_a, &ViewListTreeWidget::updateViewInfo, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewListTreeWidget::*)()>(_a, &ViewListTreeWidget::modified, 1))
            return;
    }
}

const QMetaObject *KPlato::ViewListTreeWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ViewListTreeWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18ViewListTreeWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QTreeWidget::qt_metacast(_clname);
}

int KPlato::ViewListTreeWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ViewListTreeWidget::updateViewInfo(KPlato::ViewListItem * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::ViewListTreeWidget::modified()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN6KPlato14ViewListWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ViewListWidget::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato14ViewListWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ViewListWidget",
        "activated",
        "",
        "KPlato::ViewListItem*",
        "createView",
        "viewListItemRemoved",
        "item",
        "viewListItemInserted",
        "parent",
        "index",
        "selectionChanged",
        "KPlato::ScheduleManager*",
        "updateViewInfo",
        "itm",
        "modified",
        "setProject",
        "KPlato::Project*",
        "project",
        "setSelectedSchedule",
        "sm",
        "setModified",
        "slotActivated",
        "QTreeWidgetItem*",
        "prev",
        "slotItemChanged",
        "col",
        "renameCategory",
        "slotAddView",
        "slotRemoveCategory",
        "slotRemoveView",
        "slotEditViewTitle",
        "slotEditDocumentTitle",
        "slotConfigureItem",
        "slotCurrentScheduleChanged",
        "slotScheduleManagerAdded",
        "slotDialogFinished",
        "result"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activated'
        QtMocHelpers::SignalData<void(KPlato::ViewListItem *, KPlato::ViewListItem *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 3, 2 },
        }}),
        // Signal 'createView'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewListItemRemoved'
        QtMocHelpers::SignalData<void(KPlato::ViewListItem *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 },
        }}),
        // Signal 'viewListItemInserted'
        QtMocHelpers::SignalData<void(KPlato::ViewListItem *, KPlato::ViewListItem *, int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 }, { 0x80000000 | 3, 8 }, { QMetaType::Int, 9 },
        }}),
        // Signal 'selectionChanged'
        QtMocHelpers::SignalData<void(KPlato::ScheduleManager *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 2 },
        }}),
        // Signal 'updateViewInfo'
        QtMocHelpers::SignalData<void(KPlato::ViewListItem *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 13 },
        }}),
        // Signal 'modified'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setProject'
        QtMocHelpers::SlotData<void(KPlato::Project *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'setSelectedSchedule'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 19 },
        }}),
        // Slot 'setModified'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotActivated'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, QTreeWidgetItem *)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 22, 6 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotItemChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, int)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 22, 6 }, { QMetaType::Int, 25 },
        }}),
        // Slot 'renameCategory'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAddView'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRemoveCategory'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRemoveView'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEditViewTitle'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEditDocumentTitle'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotConfigureItem'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCurrentScheduleChanged'
        QtMocHelpers::SlotData<void(int)>(33, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotScheduleManagerAdded'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(34, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 11, 2 },
        }}),
        // Slot 'slotDialogFinished'
        QtMocHelpers::SlotData<void(int)>(35, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 36 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ViewListWidget, qt_meta_tag_ZN6KPlato14ViewListWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ViewListWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14ViewListWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14ViewListWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato14ViewListWidgetE_t>.metaTypes,
    nullptr
} };

void KPlato::ViewListWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ViewListWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activated((*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[2]))); break;
        case 1: _t->createView(); break;
        case 2: _t->viewListItemRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[1]))); break;
        case 3: _t->viewListItemInserted((*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 4: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 5: _t->updateViewInfo((*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[1]))); break;
        case 6: _t->modified(); break;
        case 7: _t->setProject((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1]))); break;
        case 8: _t->setSelectedSchedule((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 9: _t->setModified(); break;
        case 10: _t->slotActivated((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[2]))); break;
        case 11: _t->slotItemChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 12: _t->renameCategory(); break;
        case 13: _t->slotAddView(); break;
        case 14: _t->slotRemoveCategory(); break;
        case 15: _t->slotRemoveView(); break;
        case 16: _t->slotEditViewTitle(); break;
        case 17: _t->slotEditDocumentTitle(); break;
        case 18: _t->slotConfigureItem(); break;
        case 19: _t->slotCurrentScheduleChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->slotScheduleManagerAdded((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 21: _t->slotDialogFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ViewListWidget::*)(KPlato::ViewListItem * , KPlato::ViewListItem * )>(_a, &ViewListWidget::activated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewListWidget::*)()>(_a, &ViewListWidget::createView, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewListWidget::*)(KPlato::ViewListItem * )>(_a, &ViewListWidget::viewListItemRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewListWidget::*)(KPlato::ViewListItem * , KPlato::ViewListItem * , int )>(_a, &ViewListWidget::viewListItemInserted, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewListWidget::*)(KPlato::ScheduleManager * )>(_a, &ViewListWidget::selectionChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewListWidget::*)(KPlato::ViewListItem * )>(_a, &ViewListWidget::updateViewInfo, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewListWidget::*)()>(_a, &ViewListWidget::modified, 6))
            return;
    }
}

const QMetaObject *KPlato::ViewListWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ViewListWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14ViewListWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int KPlato::ViewListWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ViewListWidget::activated(KPlato::ViewListItem * _t1, KPlato::ViewListItem * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KPlato::ViewListWidget::createView()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KPlato::ViewListWidget::viewListItemRemoved(KPlato::ViewListItem * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlato::ViewListWidget::viewListItemInserted(KPlato::ViewListItem * _t1, KPlato::ViewListItem * _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void KPlato::ViewListWidget::selectionChanged(KPlato::ScheduleManager * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KPlato::ViewListWidget::updateViewInfo(KPlato::ViewListItem * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KPlato::ViewListWidget::modified()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
