/********************************************************************************
** Form generated from reading UI file 'SharedResourcesDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SHAREDRESOURCESDIALOG_H
#define UI_SHAREDRESOURCESDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SharedResourcesDialog
{
public:
    QHBoxLayout *horizontalLayout_2;
    QWidget *mainWidget;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QPushButton *markAsRemove;
    QPushButton *markAsConvert;
    QPushButton *markAsKeep;
    QSpacerItem *horizontalSpacer;
    QTreeView *resourceView;

    void setupUi(QWidget *SharedResourcesDialog)
    {
        if (SharedResourcesDialog->objectName().isEmpty())
            SharedResourcesDialog->setObjectName("SharedResourcesDialog");
        SharedResourcesDialog->resize(572, 284);
        SharedResourcesDialog->setWindowTitle(QString::fromUtf8(""));
        horizontalLayout_2 = new QHBoxLayout(SharedResourcesDialog);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        mainWidget = new QWidget(SharedResourcesDialog);
        mainWidget->setObjectName("mainWidget");
        verticalLayout = new QVBoxLayout(mainWidget);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        markAsRemove = new QPushButton(mainWidget);
        markAsRemove->setObjectName("markAsRemove");

        horizontalLayout->addWidget(markAsRemove);

        markAsConvert = new QPushButton(mainWidget);
        markAsConvert->setObjectName("markAsConvert");

        horizontalLayout->addWidget(markAsConvert);

        markAsKeep = new QPushButton(mainWidget);
        markAsKeep->setObjectName("markAsKeep");

        horizontalLayout->addWidget(markAsKeep);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        resourceView = new QTreeView(mainWidget);
        resourceView->setObjectName("resourceView");
        resourceView->setSelectionMode(QAbstractItemView::ExtendedSelection);
        resourceView->setRootIsDecorated(true);
        resourceView->header()->setDefaultSectionSize(300);
        resourceView->header()->setHighlightSections(false);
        resourceView->header()->setProperty("showSortIndicator", QVariant(false));
        resourceView->header()->setStretchLastSection(true);

        verticalLayout->addWidget(resourceView);


        horizontalLayout_2->addWidget(mainWidget);


        retranslateUi(SharedResourcesDialog);

        QMetaObject::connectSlotsByName(SharedResourcesDialog);
    } // setupUi

    void retranslateUi(QWidget *SharedResourcesDialog)
    {
#if QT_CONFIG(whatsthis)
        mainWidget->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        markAsRemove->setToolTip(tr2i18n("Mark all selected items to be removed from your project", nullptr));
#endif // QT_CONFIG(tooltip)
        markAsRemove->setText(tr2i18n("Mark as: Remove", nullptr));
#if QT_CONFIG(tooltip)
        markAsConvert->setToolTip(tr2i18n("Mark all selected items to be converted to local resources", nullptr));
#endif // QT_CONFIG(tooltip)
        markAsConvert->setText(tr2i18n("Mark as: Convert", nullptr));
#if QT_CONFIG(tooltip)
        markAsKeep->setToolTip(tr2i18n("Mark all selected items to be kept as shared resources", nullptr));
#endif // QT_CONFIG(tooltip)
        markAsKeep->setText(tr2i18n("Mark as: Keep", nullptr));
        (void)SharedResourcesDialog;
    } // retranslateUi

};

namespace Ui {
    class SharedResourcesDialog: public Ui_SharedResourcesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SHAREDRESOURCESDIALOG_H

