/****************************************************************************
** Meta object code from reading C++ file 'SchedulingView.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../calligraplan-4.0.1/src/portfolio/SchedulingView.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SchedulingView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14SchedulingViewE_t {};
} // unnamed namespace

template <> constexpr inline auto SchedulingView::qt_create_metaobjectdata<qt_meta_tag_ZN14SchedulingViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SchedulingView",
        "projectCalculated",
        "",
        "KPlato::Project*",
        "project",
        "KPlato::ScheduleManager*",
        "sm",
        "selectionChanged",
        "QItemSelection",
        "selected",
        "deselected",
        "calculate",
        "updateSchedulingProperties",
        "slotSchedulersComboChanged",
        "idx",
        "slotGranularitiesChanged",
        "slotSequentialChanged",
        "state",
        "slotTodayToggled",
        "slotTomorrowToggled",
        "slotTimeToggled",
        "updateLogFilter",
        "itemDoubleClicked",
        "QPersistentModelIndex",
        "slotContextMenuRequested",
        "QPoint",
        "pos",
        "slotDescription",
        "calculateFromChanged",
        "portfolioChanged",
        "saveSettings",
        "QDomElement&",
        "settings",
        "loadSettings",
        "KoXmlElement&"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'projectCalculated'
        QtMocHelpers::SignalData<void(KPlato::Project *, KPlato::ScheduleManager *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'selectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 8, 10 },
        }}),
        // Slot 'calculate'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateSchedulingProperties'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSchedulersComboChanged'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'slotGranularitiesChanged'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'slotSequentialChanged'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'slotTodayToggled'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'slotTomorrowToggled'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'slotTimeToggled'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'updateLogFilter'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'itemDoubleClicked'
        QtMocHelpers::SlotData<void(const QPersistentModelIndex &)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 23, 14 },
        }}),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'slotDescription'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'calculateFromChanged'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'portfolioChanged'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Method 'saveSettings'
        QtMocHelpers::MethodData<void(QDomElement &) const>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Method 'loadSettings'
        QtMocHelpers::MethodData<void(KoXmlElement &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 32 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SchedulingView, qt_meta_tag_ZN14SchedulingViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SchedulingView::staticMetaObject = { {
    QMetaObject::SuperData::link<KoView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SchedulingViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SchedulingViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14SchedulingViewE_t>.metaTypes,
    nullptr
} };

void SchedulingView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SchedulingView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->projectCalculated((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[2]))); break;
        case 1: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 2: _t->calculate(); break;
        case 3: _t->updateSchedulingProperties(); break;
        case 4: _t->slotSchedulersComboChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->slotGranularitiesChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->slotSequentialChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->slotTodayToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->slotTomorrowToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->slotTimeToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->updateLogFilter(); break;
        case 11: _t->itemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QPersistentModelIndex>>(_a[1]))); break;
        case 12: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 13: _t->slotDescription(); break;
        case 14: _t->calculateFromChanged(); break;
        case 15: _t->portfolioChanged(); break;
        case 16: _t->saveSettings((*reinterpret_cast<std::add_pointer_t<QDomElement&>>(_a[1]))); break;
        case 17: _t->loadSettings((*reinterpret_cast<std::add_pointer_t<KoXmlElement&>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SchedulingView::*)(KPlato::Project * , KPlato::ScheduleManager * )>(_a, &SchedulingView::projectCalculated, 0))
            return;
    }
}

const QMetaObject *SchedulingView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SchedulingView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SchedulingViewE_t>.strings))
        return static_cast<void*>(this);
    return KoView::qt_metacast(_clname);
}

int SchedulingView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KoView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void SchedulingView::projectCalculated(KPlato::Project * _t1, KPlato::ScheduleManager * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
