/********************************************************************************
** Form generated from reading UI file 'kptworkpackagesendpanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTWORKPACKAGESENDPANEL_H
#define UI_KPTWORKPACKAGESENDPANEL_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_WorkPackageSendPanel
{
public:
    QHBoxLayout *horizontalLayout;
    QTreeView *ui_treeView;
    QVBoxLayout *verticalLayout;
    QSpacerItem *verticalSpacer;
    QPushButton *ui_publish;

    void setupUi(QWidget *KPlato__WorkPackageSendPanel)
    {
        if (KPlato__WorkPackageSendPanel->objectName().isEmpty())
            KPlato__WorkPackageSendPanel->setObjectName("KPlato__WorkPackageSendPanel");
        KPlato__WorkPackageSendPanel->resize(470, 279);
        horizontalLayout = new QHBoxLayout(KPlato__WorkPackageSendPanel);
        horizontalLayout->setObjectName("horizontalLayout");
        ui_treeView = new QTreeView(KPlato__WorkPackageSendPanel);
        ui_treeView->setObjectName("ui_treeView");
        ui_treeView->setRootIsDecorated(false);
        ui_treeView->setItemsExpandable(false);

        horizontalLayout->addWidget(ui_treeView);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        ui_publish = new QPushButton(KPlato__WorkPackageSendPanel);
        ui_publish->setObjectName("ui_publish");
        ui_publish->setEnabled(false);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("mail-send")));
        ui_publish->setIcon(icon);

        verticalLayout->addWidget(ui_publish);


        horizontalLayout->addLayout(verticalLayout);


        retranslateUi(KPlato__WorkPackageSendPanel);

        QMetaObject::connectSlotsByName(KPlato__WorkPackageSendPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__WorkPackageSendPanel)
    {
        ui_publish->setText(tr2i18n("Publish...", nullptr));
        (void)KPlato__WorkPackageSendPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class WorkPackageSendPanel: public Ui_WorkPackageSendPanel {};
} // namespace Ui
} // namespace KPlato

#endif // KPTWORKPACKAGESENDPANEL_H

