/********************************************************************************
** Form generated from reading UI file 'koDocumentInfoAuthorWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KODOCUMENTINFOAUTHORWIDGET_H
#define UI_KODOCUMENTINFOAUTHORWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KoDocumentInfoAuthorWidget
{
public:
    QGridLayout *gridLayout;
    QLabel *fullNameLabel;
    QLabel *fullName;
    QLabel *lblInitialsDesc;
    QLabel *initials;
    QLabel *lblTitleDesc;
    QLabel *title;
    QLabel *lblPositionDesc;
    QLabel *position;
    QLabel *lblCompanyDesc;
    QLabel *lblEmailDesc;
    QLabel *lblPhoneHome;
    QLabel *lblPhoneWork;
    QLabel *lblFaxDesc;
    QLabel *lblStreetDesc;
    QLabel *lblPostalDesc;
    QLabel *lblCityDesc;
    QLabel *lblCountryDesc;
    QSpacerItem *spacerItem;
    QLabel *company;
    QLabel *email;
    QLabel *phoneHome;
    QLabel *phoneWork;
    QLabel *fax;
    QLabel *street;
    QLabel *postal;
    QLabel *city;
    QLabel *country;

    void setupUi(QWidget *KoDocumentInfoAuthorWidget)
    {
        if (KoDocumentInfoAuthorWidget->objectName().isEmpty())
            KoDocumentInfoAuthorWidget->setObjectName("KoDocumentInfoAuthorWidget");
        KoDocumentInfoAuthorWidget->resize(450, 307);
        gridLayout = new QGridLayout(KoDocumentInfoAuthorWidget);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setVerticalSpacing(8);
        fullNameLabel = new QLabel(KoDocumentInfoAuthorWidget);
        fullNameLabel->setObjectName("fullNameLabel");
        fullNameLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(fullNameLabel, 0, 0, 1, 1);

        fullName = new QLabel(KoDocumentInfoAuthorWidget);
        fullName->setObjectName("fullName");

        gridLayout->addWidget(fullName, 0, 2, 1, 1);

        lblInitialsDesc = new QLabel(KoDocumentInfoAuthorWidget);
        lblInitialsDesc->setObjectName("lblInitialsDesc");
        lblInitialsDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblInitialsDesc, 1, 0, 1, 1);

        initials = new QLabel(KoDocumentInfoAuthorWidget);
        initials->setObjectName("initials");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(initials->sizePolicy().hasHeightForWidth());
        initials->setSizePolicy(sizePolicy);

        gridLayout->addWidget(initials, 1, 2, 1, 1);

        lblTitleDesc = new QLabel(KoDocumentInfoAuthorWidget);
        lblTitleDesc->setObjectName("lblTitleDesc");
        lblTitleDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblTitleDesc, 2, 0, 1, 1);

        title = new QLabel(KoDocumentInfoAuthorWidget);
        title->setObjectName("title");

        gridLayout->addWidget(title, 2, 2, 1, 1);

        lblPositionDesc = new QLabel(KoDocumentInfoAuthorWidget);
        lblPositionDesc->setObjectName("lblPositionDesc");
        lblPositionDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblPositionDesc, 3, 0, 1, 1);

        position = new QLabel(KoDocumentInfoAuthorWidget);
        position->setObjectName("position");

        gridLayout->addWidget(position, 3, 2, 1, 1);

        lblCompanyDesc = new QLabel(KoDocumentInfoAuthorWidget);
        lblCompanyDesc->setObjectName("lblCompanyDesc");
        lblCompanyDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblCompanyDesc, 4, 0, 1, 1);

        lblEmailDesc = new QLabel(KoDocumentInfoAuthorWidget);
        lblEmailDesc->setObjectName("lblEmailDesc");
        lblEmailDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblEmailDesc, 5, 0, 1, 1);

        lblPhoneHome = new QLabel(KoDocumentInfoAuthorWidget);
        lblPhoneHome->setObjectName("lblPhoneHome");
        lblPhoneHome->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblPhoneHome, 6, 0, 1, 1);

        lblPhoneWork = new QLabel(KoDocumentInfoAuthorWidget);
        lblPhoneWork->setObjectName("lblPhoneWork");
        lblPhoneWork->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblPhoneWork, 7, 0, 1, 1);

        lblFaxDesc = new QLabel(KoDocumentInfoAuthorWidget);
        lblFaxDesc->setObjectName("lblFaxDesc");
        lblFaxDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblFaxDesc, 8, 0, 1, 1);

        lblStreetDesc = new QLabel(KoDocumentInfoAuthorWidget);
        lblStreetDesc->setObjectName("lblStreetDesc");
        lblStreetDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblStreetDesc, 9, 0, 1, 1);

        lblPostalDesc = new QLabel(KoDocumentInfoAuthorWidget);
        lblPostalDesc->setObjectName("lblPostalDesc");
        lblPostalDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblPostalDesc, 10, 0, 1, 1);

        lblCityDesc = new QLabel(KoDocumentInfoAuthorWidget);
        lblCityDesc->setObjectName("lblCityDesc");
        lblCityDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblCityDesc, 11, 0, 1, 1);

        lblCountryDesc = new QLabel(KoDocumentInfoAuthorWidget);
        lblCountryDesc->setObjectName("lblCountryDesc");
        lblCountryDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblCountryDesc, 12, 0, 1, 1);

        spacerItem = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 13, 2, 1, 1);

        company = new QLabel(KoDocumentInfoAuthorWidget);
        company->setObjectName("company");

        gridLayout->addWidget(company, 4, 2, 1, 1);

        email = new QLabel(KoDocumentInfoAuthorWidget);
        email->setObjectName("email");

        gridLayout->addWidget(email, 5, 2, 1, 1);

        phoneHome = new QLabel(KoDocumentInfoAuthorWidget);
        phoneHome->setObjectName("phoneHome");

        gridLayout->addWidget(phoneHome, 6, 2, 1, 1);

        phoneWork = new QLabel(KoDocumentInfoAuthorWidget);
        phoneWork->setObjectName("phoneWork");

        gridLayout->addWidget(phoneWork, 7, 2, 1, 1);

        fax = new QLabel(KoDocumentInfoAuthorWidget);
        fax->setObjectName("fax");

        gridLayout->addWidget(fax, 8, 2, 1, 1);

        street = new QLabel(KoDocumentInfoAuthorWidget);
        street->setObjectName("street");

        gridLayout->addWidget(street, 9, 2, 1, 1);

        postal = new QLabel(KoDocumentInfoAuthorWidget);
        postal->setObjectName("postal");

        gridLayout->addWidget(postal, 10, 2, 1, 1);

        city = new QLabel(KoDocumentInfoAuthorWidget);
        city->setObjectName("city");

        gridLayout->addWidget(city, 11, 2, 1, 1);

        country = new QLabel(KoDocumentInfoAuthorWidget);
        country->setObjectName("country");

        gridLayout->addWidget(country, 12, 2, 1, 1);


        retranslateUi(KoDocumentInfoAuthorWidget);

        QMetaObject::connectSlotsByName(KoDocumentInfoAuthorWidget);
    } // setupUi

    void retranslateUi(QWidget *KoDocumentInfoAuthorWidget)
    {
        fullNameLabel->setText(tr2i18n("Name:", nullptr));
        fullName->setText(QString());
        lblInitialsDesc->setText(tr2i18n("Initials:", nullptr));
        initials->setText(QString());
        lblTitleDesc->setText(tr2i18n("Title:", nullptr));
        title->setText(QString());
        lblPositionDesc->setText(tr2i18n("Position:", nullptr));
        position->setText(QString());
        lblCompanyDesc->setText(tr2i18n("Company:", nullptr));
        lblEmailDesc->setText(tr2i18n("Email:", nullptr));
        lblPhoneHome->setText(tr2i18n("Telephone (home):", nullptr));
        lblPhoneWork->setText(tr2i18n("Telephone (work):", nullptr));
        lblFaxDesc->setText(tr2i18n("Fax:", nullptr));
        lblStreetDesc->setText(tr2i18n("Street:", nullptr));
        lblPostalDesc->setText(tr2i18n("Postal code:", nullptr));
        lblCityDesc->setText(tr2i18n("City:", nullptr));
        lblCountryDesc->setText(tr2i18n("Country:", nullptr));
        company->setText(QString());
        email->setText(QString());
        phoneHome->setText(QString());
        phoneWork->setText(QString());
        fax->setText(QString());
        street->setText(QString());
        postal->setText(QString());
        city->setText(QString());
        country->setText(QString());
        (void)KoDocumentInfoAuthorWidget;
    } // retranslateUi

};

namespace Ui {
    class KoDocumentInfoAuthorWidget: public Ui_KoDocumentInfoAuthorWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KODOCUMENTINFOAUTHORWIDGET_H

