/********************************************************************************
** Form generated from reading UI file 'kpttaskstatusviewsettingspanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTTASKSTATUSVIEWSETTINGSPANEL_H
#define UI_KPTTASKSTATUSVIEWSETTINGSPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_TaskStatusViewSettingsPanel
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *periodGroupBox;
    QGridLayout *gridLayout;
    QLabel *label;
    QSpinBox *period;
    QSpacerItem *horizontalSpacer;
    QRadioButton *useCurrentDate;
    QSpacerItem *horizontalSpacer_3;
    QRadioButton *useWeekday;
    QComboBox *weekdays;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPlato__TaskStatusViewSettingsPanel)
    {
        if (KPlato__TaskStatusViewSettingsPanel->objectName().isEmpty())
            KPlato__TaskStatusViewSettingsPanel->setObjectName("KPlato__TaskStatusViewSettingsPanel");
        KPlato__TaskStatusViewSettingsPanel->resize(412, 243);
        verticalLayout = new QVBoxLayout(KPlato__TaskStatusViewSettingsPanel);
        verticalLayout->setObjectName("verticalLayout");
        periodGroupBox = new QGroupBox(KPlato__TaskStatusViewSettingsPanel);
        periodGroupBox->setObjectName("periodGroupBox");
        periodGroupBox->setFlat(true);
        gridLayout = new QGridLayout(periodGroupBox);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(periodGroupBox);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        period = new QSpinBox(periodGroupBox);
        period->setObjectName("period");

        gridLayout->addWidget(period, 0, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(176, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 2, 1, 2);

        useCurrentDate = new QRadioButton(periodGroupBox);
        useCurrentDate->setObjectName("useCurrentDate");

        gridLayout->addWidget(useCurrentDate, 1, 0, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(238, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_3, 1, 1, 1, 3);

        useWeekday = new QRadioButton(periodGroupBox);
        useWeekday->setObjectName("useWeekday");

        gridLayout->addWidget(useWeekday, 2, 0, 1, 1);

        weekdays = new QComboBox(periodGroupBox);
        weekdays->setObjectName("weekdays");

        gridLayout->addWidget(weekdays, 2, 1, 1, 2);

        horizontalSpacer_2 = new QSpacerItem(158, 17, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_2, 2, 3, 1, 1);


        verticalLayout->addWidget(periodGroupBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(KPlato__TaskStatusViewSettingsPanel);

        QMetaObject::connectSlotsByName(KPlato__TaskStatusViewSettingsPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__TaskStatusViewSettingsPanel)
    {
        periodGroupBox->setTitle(tr2i18n("Period Definition", nullptr));
        label->setText(tr2i18n("Period length (days):", nullptr));
        useCurrentDate->setText(tr2i18n("Use current date", nullptr));
        useWeekday->setText(tr2i18n("Use weekday:", nullptr));
        (void)KPlato__TaskStatusViewSettingsPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class TaskStatusViewSettingsPanel: public Ui_TaskStatusViewSettingsPanel {};
} // namespace Ui
} // namespace KPlato

#endif // KPTTASKSTATUSVIEWSETTINGSPANEL_H

