/****************************************************************************
** Meta object code from reading C++ file 'KoDialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/widgets/KoDialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KoDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8KoDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto KoDialog::qt_create_metaobjectdata<qt_meta_tag_ZN8KoDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KoDialog",
        "layoutHintChanged",
        "",
        "helpClicked",
        "defaultClicked",
        "resetClicked",
        "user3Clicked",
        "user2Clicked",
        "user1Clicked",
        "applyClicked",
        "tryClicked",
        "okClicked",
        "cancelClicked",
        "closeClicked",
        "buttonClicked",
        "KoDialog::ButtonCode",
        "button",
        "hidden",
        "aboutToShowDetails",
        "setCaption",
        "caption",
        "modified",
        "setPlainCaption",
        "enableButton",
        "id",
        "state",
        "enableButtonOk",
        "enableButtonApply",
        "enableButtonCancel",
        "enableLinkedHelp",
        "setHelpLinkText",
        "text",
        "setHelp",
        "anchor",
        "appname",
        "setDetailsWidgetVisible",
        "visible",
        "setDetailsWidget",
        "QWidget*",
        "detailsWidget",
        "delayedDestruct",
        "slotButtonClicked",
        "updateGeometry",
        "queuedLayoutUpdate",
        "helpLinkClicked",
        "ButtonCode",
        "None",
        "Help",
        "Default",
        "Ok",
        "Apply",
        "Try",
        "Cancel",
        "Close",
        "Reset",
        "Details",
        "User1",
        "User2",
        "User3",
        "NoDefault"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'layoutHintChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'helpClicked'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'defaultClicked'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resetClicked'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'user3Clicked'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'user2Clicked'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'user1Clicked'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'applyClicked'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tryClicked'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'okClicked'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cancelClicked'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'closeClicked'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'buttonClicked'
        QtMocHelpers::SignalData<void(KoDialog::ButtonCode)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'hidden'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aboutToShowDetails'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setCaption'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'setCaption'
        QtMocHelpers::SlotData<void(const QString &, bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'setPlainCaption'
        QtMocHelpers::SlotData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'enableButton'
        QtMocHelpers::SlotData<void(KoDialog::ButtonCode, bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 24 }, { QMetaType::Bool, 25 },
        }}),
        // Slot 'enableButtonOk'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'enableButtonApply'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'enableButtonCancel'
        QtMocHelpers::SlotData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'enableLinkedHelp'
        QtMocHelpers::SlotData<void(bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'setHelpLinkText'
        QtMocHelpers::SlotData<void(const QString &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 31 },
        }}),
        // Slot 'setHelp'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 }, { QMetaType::QString, 34 },
        }}),
        // Slot 'setHelp'
        QtMocHelpers::SlotData<void(const QString &)>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDetailsWidgetVisible'
        QtMocHelpers::SlotData<void(bool)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 36 },
        }}),
        // Slot 'setDetailsWidget'
        QtMocHelpers::SlotData<void(QWidget *)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Slot 'delayedDestruct'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotButtonClicked'
        QtMocHelpers::SlotData<void(int)>(41, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'updateGeometry'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'queuedLayoutUpdate'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'helpLinkClicked'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ButtonCode'
        QtMocHelpers::EnumData<enum ButtonCode>(45, 45, QMC::EnumFlags{}).add({
            {   46, ButtonCode::None },
            {   47, ButtonCode::Help },
            {   48, ButtonCode::Default },
            {   49, ButtonCode::Ok },
            {   50, ButtonCode::Apply },
            {   51, ButtonCode::Try },
            {   52, ButtonCode::Cancel },
            {   53, ButtonCode::Close },
            {   54, ButtonCode::Reset },
            {   55, ButtonCode::Details },
            {   56, ButtonCode::User1 },
            {   57, ButtonCode::User2 },
            {   58, ButtonCode::User3 },
            {   59, ButtonCode::NoDefault },
        }),
    };
    return QtMocHelpers::metaObjectData<KoDialog, qt_meta_tag_ZN8KoDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KoDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KoDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KoDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8KoDialogE_t>.metaTypes,
    nullptr
} };

void KoDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KoDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->layoutHintChanged(); break;
        case 1: _t->helpClicked(); break;
        case 2: _t->defaultClicked(); break;
        case 3: _t->resetClicked(); break;
        case 4: _t->user3Clicked(); break;
        case 5: _t->user2Clicked(); break;
        case 6: _t->user1Clicked(); break;
        case 7: _t->applyClicked(); break;
        case 8: _t->tryClicked(); break;
        case 9: _t->okClicked(); break;
        case 10: _t->cancelClicked(); break;
        case 11: _t->closeClicked(); break;
        case 12: _t->buttonClicked((*reinterpret_cast<std::add_pointer_t<KoDialog::ButtonCode>>(_a[1]))); break;
        case 13: _t->hidden(); break;
        case 14: _t->aboutToShowDetails(); break;
        case 15: _t->setCaption((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->setCaption((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 17: _t->setPlainCaption((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->enableButton((*reinterpret_cast<std::add_pointer_t<KoDialog::ButtonCode>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 19: _t->enableButtonOk((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->enableButtonApply((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->enableButtonCancel((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->enableLinkedHelp((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->setHelpLinkText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->setHelp((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 25: _t->setHelp((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 26: _t->setDetailsWidgetVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->setDetailsWidget((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 28: _t->delayedDestruct(); break;
        case 29: _t->slotButtonClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 30: _t->updateGeometry(); break;
        case 31: _t->d_ptr->queuedLayoutUpdate(); break;
        case 32: _t->d_ptr->helpLinkClicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 27:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::layoutHintChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::helpClicked, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::defaultClicked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::resetClicked, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::user3Clicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::user2Clicked, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::user1Clicked, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::applyClicked, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::tryClicked, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::okClicked, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::cancelClicked, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::closeClicked, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)(KoDialog::ButtonCode )>(_a, &KoDialog::buttonClicked, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::hidden, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDialog::*)()>(_a, &KoDialog::aboutToShowDetails, 14))
            return;
    }
}

const QMetaObject *KoDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KoDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KoDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int KoDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 33)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 33)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    }
    return _id;
}

// SIGNAL 0
void KoDialog::layoutHintChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KoDialog::helpClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KoDialog::defaultClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KoDialog::resetClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KoDialog::user3Clicked()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KoDialog::user2Clicked()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void KoDialog::user1Clicked()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KoDialog::applyClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void KoDialog::tryClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void KoDialog::okClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void KoDialog::cancelClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void KoDialog::closeClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void KoDialog::buttonClicked(KoDialog::ButtonCode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void KoDialog::hidden()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void KoDialog::aboutToShowDetails()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}
QT_WARNING_POP
