/********************************************************************************
** Form generated from reading UI file 'kptaccountsviewconfigurepanelbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTACCOUNTSVIEWCONFIGUREPANELBASE_H
#define UI_KPTACCOUNTSVIEWCONFIGUREPANELBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_AccountsviewConfigurePanelBase
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout_3;
    QLabel *label;
    QLabel *textLabel1_2;
    QComboBox *ui_periodBox;
    QCheckBox *ui_cumulative;
    QComboBox *ui_showBox;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QRadioButton *ui_projectstartBtn;
    QRadioButton *ui_startdateBtn;
    QDateEdit *ui_startdate;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout_2;
    QRadioButton *ui_currentdateBtn;
    QRadioButton *ui_enddateBtn;
    QDateEdit *ui_enddate;
    QRadioButton *ui_projectendBtn;

    void setupUi(QWidget *KPlato__AccountsviewConfigurePanelBase)
    {
        if (KPlato__AccountsviewConfigurePanelBase->objectName().isEmpty())
            KPlato__AccountsviewConfigurePanelBase->setObjectName("KPlato__AccountsviewConfigurePanelBase");
        KPlato__AccountsviewConfigurePanelBase->resize(233, 336);
        verticalLayout_2 = new QVBoxLayout(KPlato__AccountsviewConfigurePanelBase);
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox_3 = new QGroupBox(KPlato__AccountsviewConfigurePanelBase);
        groupBox_3->setObjectName("groupBox_3");
        gridLayout_3 = new QGridLayout(groupBox_3);
        gridLayout_3->setObjectName("gridLayout_3");
        label = new QLabel(groupBox_3);
        label->setObjectName("label");

        gridLayout_3->addWidget(label, 0, 0, 1, 1);

        textLabel1_2 = new QLabel(groupBox_3);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        gridLayout_3->addWidget(textLabel1_2, 1, 0, 1, 1);

        ui_periodBox = new QComboBox(groupBox_3);
        ui_periodBox->addItem(QString());
        ui_periodBox->addItem(QString());
        ui_periodBox->addItem(QString());
        ui_periodBox->setObjectName("ui_periodBox");

        gridLayout_3->addWidget(ui_periodBox, 1, 1, 1, 1);

        ui_cumulative = new QCheckBox(groupBox_3);
        ui_cumulative->setObjectName("ui_cumulative");

        gridLayout_3->addWidget(ui_cumulative, 2, 0, 1, 2);

        ui_showBox = new QComboBox(groupBox_3);
        ui_showBox->addItem(QString());
        ui_showBox->addItem(QString());
        ui_showBox->addItem(QString());
        ui_showBox->addItem(QString());
        ui_showBox->setObjectName("ui_showBox");

        gridLayout_3->addWidget(ui_showBox, 0, 1, 1, 1);

        label->raise();
        textLabel1_2->raise();
        ui_periodBox->raise();
        ui_cumulative->raise();
        ui_showBox->raise();

        verticalLayout_2->addWidget(groupBox_3);

        groupBox = new QGroupBox(KPlato__AccountsviewConfigurePanelBase);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        ui_projectstartBtn = new QRadioButton(groupBox);
        ui_projectstartBtn->setObjectName("ui_projectstartBtn");

        gridLayout->addWidget(ui_projectstartBtn, 0, 0, 1, 1);

        ui_startdateBtn = new QRadioButton(groupBox);
        ui_startdateBtn->setObjectName("ui_startdateBtn");

        gridLayout->addWidget(ui_startdateBtn, 1, 0, 1, 1);

        ui_startdate = new QDateEdit(groupBox);
        ui_startdate->setObjectName("ui_startdate");
        ui_startdate->setCalendarPopup(true);

        gridLayout->addWidget(ui_startdate, 1, 1, 1, 1);


        verticalLayout_2->addWidget(groupBox);

        groupBox_2 = new QGroupBox(KPlato__AccountsviewConfigurePanelBase);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout_2 = new QGridLayout(groupBox_2);
        gridLayout_2->setObjectName("gridLayout_2");
        ui_currentdateBtn = new QRadioButton(groupBox_2);
        ui_currentdateBtn->setObjectName("ui_currentdateBtn");

        gridLayout_2->addWidget(ui_currentdateBtn, 2, 0, 1, 1);

        ui_enddateBtn = new QRadioButton(groupBox_2);
        ui_enddateBtn->setObjectName("ui_enddateBtn");

        gridLayout_2->addWidget(ui_enddateBtn, 3, 0, 1, 1);

        ui_enddate = new QDateEdit(groupBox_2);
        ui_enddate->setObjectName("ui_enddate");
        ui_enddate->setCalendarPopup(true);

        gridLayout_2->addWidget(ui_enddate, 3, 1, 1, 1);

        ui_projectendBtn = new QRadioButton(groupBox_2);
        ui_projectendBtn->setObjectName("ui_projectendBtn");

        gridLayout_2->addWidget(ui_projectendBtn, 1, 0, 1, 1);


        verticalLayout_2->addWidget(groupBox_2);


        retranslateUi(KPlato__AccountsviewConfigurePanelBase);

        QMetaObject::connectSlotsByName(KPlato__AccountsviewConfigurePanelBase);
    } // setupUi

    void retranslateUi(QWidget *KPlato__AccountsviewConfigurePanelBase)
    {
        groupBox_3->setTitle(QString());
        label->setText(tr2i18n("Show:", nullptr));
        textLabel1_2->setText(tr2i18n("Periodicity:", nullptr));
        ui_periodBox->setItemText(0, tr2i18n("Day", nullptr));
        ui_periodBox->setItemText(1, tr2i18n("Week", nullptr));
        ui_periodBox->setItemText(2, tr2i18n("Month", nullptr));

        ui_cumulative->setText(tr2i18n("Cumulative", nullptr));
        ui_showBox->setItemText(0, tr2i18n("Actual cost", nullptr));
        ui_showBox->setItemText(1, tr2i18n("Planned cost", nullptr));
        ui_showBox->setItemText(2, tr2i18n("Actual and planned", nullptr));
        ui_showBox->setItemText(3, tr2i18n("Deviation", nullptr));

        groupBox->setTitle(tr2i18n("Start date", nullptr));
        ui_projectstartBtn->setText(tr2i18n("Project start", nullptr));
        ui_startdateBtn->setText(tr2i18n("Date", nullptr));
        groupBox_2->setTitle(tr2i18n("End date", nullptr));
        ui_currentdateBtn->setText(tr2i18n("Current date", nullptr));
        ui_enddateBtn->setText(tr2i18n("Date", nullptr));
        ui_projectendBtn->setText(tr2i18n("Project end", nullptr));
        (void)KPlato__AccountsviewConfigurePanelBase;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class AccountsviewConfigurePanelBase: public Ui_AccountsviewConfigurePanelBase {};
} // namespace Ui
} // namespace KPlato

#endif // KPTACCOUNTSVIEWCONFIGUREPANELBASE_H

