/****************************************************************************
** Meta object code from reading C++ file 'kptdocumentseditor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/kptdocumentseditor.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptdocumentseditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato16DocumentTreeViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::DocumentTreeView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato16DocumentTreeViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::DocumentTreeView",
        "selectedIndexesChanged",
        "",
        "QModelIndexList",
        "slotSelectionChanged",
        "QItemSelection",
        "selected"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selectedIndexesChanged'
        QtMocHelpers::SignalData<void(const QModelIndexList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DocumentTreeView, qt_meta_tag_ZN6KPlato16DocumentTreeViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::DocumentTreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<TreeViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16DocumentTreeViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16DocumentTreeViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato16DocumentTreeViewE_t>.metaTypes,
    nullptr
} };

void KPlato::DocumentTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DocumentTreeView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectedIndexesChanged((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 1: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DocumentTreeView::*)(const QModelIndexList & )>(_a, &DocumentTreeView::selectedIndexesChanged, 0))
            return;
    }
}

const QMetaObject *KPlato::DocumentTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::DocumentTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16DocumentTreeViewE_t>.strings))
        return static_cast<void*>(this);
    return TreeViewBase::qt_metacast(_clname);
}

int KPlato::DocumentTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TreeViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KPlato::DocumentTreeView::selectedIndexesChanged(const QModelIndexList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN6KPlato15DocumentsEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::DocumentsEditor::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato15DocumentsEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::DocumentsEditor",
        "addDocument",
        "",
        "deleteDocumentList",
        "QList<KPlato::Document*>",
        "editDocument",
        "KPlato::Document*",
        "doc",
        "viewDocument",
        "setGuiActive",
        "activate",
        "slotOptions",
        "slotContextMenuRequested",
        "QModelIndex",
        "index",
        "QPoint",
        "pos",
        "slotHeaderContextMenuRequested",
        "slotSelectionChanged",
        "QModelIndexList",
        "slotCurrentChanged",
        "slotEnableActions",
        "on",
        "slotEditDocument",
        "slotViewDocument",
        "slotAddDocument",
        "slotDeleteSelection"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'addDocument'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'deleteDocumentList'
        QtMocHelpers::SignalData<void(const QList<KPlato::Document*> &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Signal 'editDocument'
        QtMocHelpers::SignalData<void(KPlato::Document *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'viewDocument'
        QtMocHelpers::SignalData<void(KPlato::Document *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setGuiActive'
        QtMocHelpers::SlotData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'slotOptions'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QPoint &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'slotHeaderContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QModelIndexList &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 2 },
        }}),
        // Slot 'slotCurrentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 2 },
        }}),
        // Slot 'slotEnableActions'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotEditDocument'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotViewDocument'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddDocument'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDeleteSelection'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DocumentsEditor, qt_meta_tag_ZN6KPlato15DocumentsEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::DocumentsEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<ViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15DocumentsEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15DocumentsEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato15DocumentsEditorE_t>.metaTypes,
    nullptr
} };

void KPlato::DocumentsEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DocumentsEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addDocument(); break;
        case 1: _t->deleteDocumentList((*reinterpret_cast<std::add_pointer_t<QList<KPlato::Document*>>>(_a[1]))); break;
        case 2: _t->editDocument((*reinterpret_cast<std::add_pointer_t<KPlato::Document*>>(_a[1]))); break;
        case 3: _t->viewDocument((*reinterpret_cast<std::add_pointer_t<KPlato::Document*>>(_a[1]))); break;
        case 4: _t->setGuiActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->slotOptions(); break;
        case 6: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 7: _t->slotHeaderContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 8: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 9: _t->slotCurrentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 10: _t->slotEnableActions((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->slotEditDocument(); break;
        case 12: _t->slotViewDocument(); break;
        case 13: _t->slotAddDocument(); break;
        case 14: _t->slotDeleteSelection(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DocumentsEditor::*)()>(_a, &DocumentsEditor::addDocument, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentsEditor::*)(const QList<KPlato::Document*> & )>(_a, &DocumentsEditor::deleteDocumentList, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentsEditor::*)(KPlato::Document * )>(_a, &DocumentsEditor::editDocument, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentsEditor::*)(KPlato::Document * )>(_a, &DocumentsEditor::viewDocument, 3))
            return;
    }
}

const QMetaObject *KPlato::DocumentsEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::DocumentsEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15DocumentsEditorE_t>.strings))
        return static_cast<void*>(this);
    return ViewBase::qt_metacast(_clname);
}

int KPlato::DocumentsEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void KPlato::DocumentsEditor::addDocument()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KPlato::DocumentsEditor::deleteDocumentList(const QList<KPlato::Document*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KPlato::DocumentsEditor::editDocument(KPlato::Document * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlato::DocumentsEditor::viewDocument(KPlato::Document * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
