// This file is generated by kconfig_compiler_kf6 from calligraplanworksettings.kcfg.
// All changes you do to this file will be lost.
#ifndef PLANWORKSETTINGS_H
#define PLANWORKSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class PlanWorkSettings : public KConfigSkeleton
{
  public:

    static PlanWorkSettings *self();
    ~PlanWorkSettings() override;

    /**
      Set CurrentView
    */
    static
    void setCurrentView( int v )
    {
      if (!self()->isCurrentViewImmutable())
        self()->mCurrentView = v;
    }

    /**
      Get CurrentView
    */
    static
    int currentView()
    {
      return self()->mCurrentView;
    }

    /**
      Is CurrentView Immutable
    */
    static
    bool isCurrentViewImmutable()
    {
      return self()->isImmutable( QStringLiteral( "CurrentView" ) );
    }

    /**
      Set TaskWorkPackageView
    */
    static
    void setTaskWorkPackageView( const QString & v )
    {
      if (!self()->isTaskWorkPackageViewImmutable())
        self()->mTaskWorkPackageView = v;
    }

    /**
      Get TaskWorkPackageView
    */
    static
    QString taskWorkPackageView()
    {
      return self()->mTaskWorkPackageView;
    }

    /**
      Is TaskWorkPackageView Immutable
    */
    static
    bool isTaskWorkPackageViewImmutable()
    {
      return self()->isImmutable( QStringLiteral( "TaskWorkPackageView" ) );
    }

    /**
      Set TaskWPGanttView
    */
    static
    void setTaskWPGanttView( const QString & v )
    {
      if (!self()->isTaskWPGanttViewImmutable())
        self()->mTaskWPGanttView = v;
    }

    /**
      Get TaskWPGanttView
    */
    static
    QString taskWPGanttView()
    {
      return self()->mTaskWPGanttView;
    }

    /**
      Is TaskWPGanttView Immutable
    */
    static
    bool isTaskWPGanttViewImmutable()
    {
      return self()->isImmutable( QStringLiteral( "TaskWPGanttView" ) );
    }

  protected:
    PlanWorkSettings();
    friend class PlanWorkSettingsHelper;


    // Views
    int mCurrentView;
    QString mTaskWorkPackageView;
    QString mTaskWPGanttView;
};

#endif

