/****************************************************************************
** Meta object code from reading C++ file 'Resource.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/kernel/Resource.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Resource.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato8ResourceE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::Resource::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato8ResourceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::Resource",
        "dataChanged",
        "",
        "KPlato::Resource*",
        "resource",
        "resourceGroupToBeAdded",
        "row",
        "resourceGroupAdded",
        "KPlato::ResourceGroup*",
        "group",
        "resourceGroupToBeRemoved",
        "resourceGroupRemoved",
        "externalAppointmentToBeAdded",
        "r",
        "externalAppointmentAdded",
        "KPlato::Appointment*",
        "externalAppointmentToBeRemoved",
        "externalAppointmentRemoved",
        "externalAppointmentChanged",
        "a",
        "teamToBeAdded",
        "teamAdded",
        "teamToBeRemoved",
        "team",
        "teamRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dataChanged'
        QtMocHelpers::SignalData<void(KPlato::Resource *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'resourceGroupToBeAdded'
        QtMocHelpers::SignalData<void(KPlato::Resource *, int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'resourceGroupAdded'
        QtMocHelpers::SignalData<void(KPlato::ResourceGroup *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'resourceGroupToBeRemoved'
        QtMocHelpers::SignalData<void(KPlato::Resource *, int, KPlato::ResourceGroup *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 6 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'resourceGroupRemoved'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'externalAppointmentToBeAdded'
        QtMocHelpers::SignalData<void(KPlato::Resource *, int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 13 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'externalAppointmentAdded'
        QtMocHelpers::SignalData<void(KPlato::Resource *, KPlato::Appointment *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 15, 2 },
        }}),
        // Signal 'externalAppointmentToBeRemoved'
        QtMocHelpers::SignalData<void(KPlato::Resource *, int)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 13 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'externalAppointmentRemoved'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'externalAppointmentChanged'
        QtMocHelpers::SignalData<void(KPlato::Resource *, KPlato::Appointment *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 13 }, { 0x80000000 | 15, 19 },
        }}),
        // Signal 'teamToBeAdded'
        QtMocHelpers::SignalData<void(KPlato::Resource *, int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 13 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'teamAdded'
        QtMocHelpers::SignalData<void(KPlato::Resource *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'teamToBeRemoved'
        QtMocHelpers::SignalData<void(KPlato::Resource *, int, KPlato::Resource *)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 13 }, { QMetaType::Int, 6 }, { 0x80000000 | 3, 23 },
        }}),
        // Signal 'teamRemoved'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Resource, qt_meta_tag_ZN6KPlato8ResourceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::Resource::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8ResourceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8ResourceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato8ResourceE_t>.metaTypes,
    nullptr
} };

void KPlato::Resource::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Resource *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dataChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 1: _t->resourceGroupToBeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->resourceGroupAdded((*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[1]))); break;
        case 3: _t->resourceGroupToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[3]))); break;
        case 4: _t->resourceGroupRemoved(); break;
        case 5: _t->externalAppointmentToBeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->externalAppointmentAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Appointment*>>(_a[2]))); break;
        case 7: _t->externalAppointmentToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->externalAppointmentRemoved(); break;
        case 9: _t->externalAppointmentChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Appointment*>>(_a[2]))); break;
        case 10: _t->teamToBeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 11: _t->teamAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 12: _t->teamToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[3]))); break;
        case 13: _t->teamRemoved(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Resource* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Resource* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Resource* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Resource* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Resource* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Resource* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Resource* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Resource* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Resource* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Resource* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Resource::*)(KPlato::Resource * )>(_a, &Resource::dataChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Resource::*)(KPlato::Resource * , int )>(_a, &Resource::resourceGroupToBeAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Resource::*)(KPlato::ResourceGroup * )>(_a, &Resource::resourceGroupAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Resource::*)(KPlato::Resource * , int , KPlato::ResourceGroup * )>(_a, &Resource::resourceGroupToBeRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Resource::*)()>(_a, &Resource::resourceGroupRemoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Resource::*)(KPlato::Resource * , int )>(_a, &Resource::externalAppointmentToBeAdded, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Resource::*)(KPlato::Resource * , KPlato::Appointment * )>(_a, &Resource::externalAppointmentAdded, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Resource::*)(KPlato::Resource * , int )>(_a, &Resource::externalAppointmentToBeRemoved, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Resource::*)()>(_a, &Resource::externalAppointmentRemoved, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Resource::*)(KPlato::Resource * , KPlato::Appointment * )>(_a, &Resource::externalAppointmentChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Resource::*)(KPlato::Resource * , int )>(_a, &Resource::teamToBeAdded, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Resource::*)(KPlato::Resource * )>(_a, &Resource::teamAdded, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Resource::*)(KPlato::Resource * , int , KPlato::Resource * )>(_a, &Resource::teamToBeRemoved, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Resource::*)()>(_a, &Resource::teamRemoved, 13))
            return;
    }
}

const QMetaObject *KPlato::Resource::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::Resource::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8ResourceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KPlato::Resource::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void KPlato::Resource::dataChanged(KPlato::Resource * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::Resource::resourceGroupToBeAdded(KPlato::Resource * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KPlato::Resource::resourceGroupAdded(KPlato::ResourceGroup * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlato::Resource::resourceGroupToBeRemoved(KPlato::Resource * _t1, int _t2, KPlato::ResourceGroup * _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void KPlato::Resource::resourceGroupRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KPlato::Resource::externalAppointmentToBeAdded(KPlato::Resource * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void KPlato::Resource::externalAppointmentAdded(KPlato::Resource * _t1, KPlato::Appointment * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void KPlato::Resource::externalAppointmentToBeRemoved(KPlato::Resource * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void KPlato::Resource::externalAppointmentRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void KPlato::Resource::externalAppointmentChanged(KPlato::Resource * _t1, KPlato::Appointment * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void KPlato::Resource::teamToBeAdded(KPlato::Resource * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void KPlato::Resource::teamAdded(KPlato::Resource * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void KPlato::Resource::teamToBeRemoved(KPlato::Resource * _t1, int _t2, KPlato::Resource * _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2, _t3);
}

// SIGNAL 13
void KPlato::Resource::teamRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}
QT_WARNING_POP
