/********************************************************************************
** Form generated from reading UI file 'kptitemviewsettings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTITEMVIEWSETTINGS_H
#define UI_KPTITEMVIEWSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kactionselector.h"

namespace KPlato {

class Ui_ItemViewSettings
{
public:
    QVBoxLayout *vboxLayout;
    QCheckBox *stretchLastSection;
    QGroupBox *groupBox;
    QVBoxLayout *vboxLayout1;
    KActionSelector *selector;

    void setupUi(QWidget *KPlato__ItemViewSettings)
    {
        if (KPlato__ItemViewSettings->objectName().isEmpty())
            KPlato__ItemViewSettings->setObjectName("KPlato__ItemViewSettings");
        KPlato__ItemViewSettings->resize(600, 278);
        vboxLayout = new QVBoxLayout(KPlato__ItemViewSettings);
        vboxLayout->setSpacing(0);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        stretchLastSection = new QCheckBox(KPlato__ItemViewSettings);
        stretchLastSection->setObjectName("stretchLastSection");

        vboxLayout->addWidget(stretchLastSection);

        groupBox = new QGroupBox(KPlato__ItemViewSettings);
        groupBox->setObjectName("groupBox");
        groupBox->setFlat(true);
        vboxLayout1 = new QVBoxLayout(groupBox);
        vboxLayout1->setSpacing(0);
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        vboxLayout1->setObjectName("vboxLayout1");
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        selector = new KActionSelector(groupBox);
        selector->setObjectName("selector");

        vboxLayout1->addWidget(selector);


        vboxLayout->addWidget(groupBox);


        retranslateUi(KPlato__ItemViewSettings);

        QMetaObject::connectSlotsByName(KPlato__ItemViewSettings);
    } // setupUi

    void retranslateUi(QWidget *KPlato__ItemViewSettings)
    {
        stretchLastSection->setText(tr2i18n("Stretch last section", nullptr));
        groupBox->setTitle(tr2i18n("Select columns to be shown:", nullptr));
        selector->setAvailableLabel(tr2i18n("&Hidden columns:", nullptr));
        selector->setSelectedLabel(tr2i18n("&Visible columns:", nullptr));
        (void)KPlato__ItemViewSettings;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class ItemViewSettings: public Ui_ItemViewSettings {};
} // namespace Ui
} // namespace KPlato

#endif // KPTITEMVIEWSETTINGS_H

