/********************************************************************************
** Form generated from reading UI file 'kptganttchartdisplayoptions.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTGANTTCHARTDISPLAYOPTIONS_H
#define UI_KPTGANTTCHARTDISPLAYOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcolorcombo.h"

namespace KPlato {

class Ui_GanttChartDisplayOptions
{
public:
    QVBoxLayout *verticalLayout_3;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout_4;
    QRadioButton *ui_timeLineHide;
    QRadioButton *ui_timeLineForeground;
    QRadioButton *ui_timeLineBackground;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_3;
    QSpinBox *ui_timeLineInterval;
    QFrame *line;
    QGridLayout *gridLayout_3;
    QCheckBox *ui_timeLineUseCustom;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QSpinBox *ui_timeLineStroke;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    KColorCombo *ui_timeLineColor;
    QGroupBox *groupBox_4;
    QVBoxLayout *verticalLayout;
    QCheckBox *ui_showRowSeparators;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_4;
    QComboBox *freedays;
    QSpacerItem *horizontalSpacer;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QCheckBox *ui_showTaskName;
    QCheckBox *ui_showResourceNames;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout;
    QCheckBox *ui_showCriticalPath;
    QCheckBox *ui_showDependencies;
    QCheckBox *ui_showNegativeFloat;
    QCheckBox *ui_showCriticalTasks;
    QCheckBox *ui_showCompletion;
    QCheckBox *ui_showPositiveFloat;
    QCheckBox *ui_showSchedulingError;
    QCheckBox *ui_showTimeConstraint;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPlato__GanttChartDisplayOptions)
    {
        if (KPlato__GanttChartDisplayOptions->objectName().isEmpty())
            KPlato__GanttChartDisplayOptions->setObjectName("KPlato__GanttChartDisplayOptions");
        KPlato__GanttChartDisplayOptions->resize(349, 515);
        verticalLayout_3 = new QVBoxLayout(KPlato__GanttChartDisplayOptions);
        verticalLayout_3->setObjectName("verticalLayout_3");
        groupBox_3 = new QGroupBox(KPlato__GanttChartDisplayOptions);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_2 = new QVBoxLayout(groupBox_3);
        verticalLayout_2->setObjectName("verticalLayout_2");
        gridLayout_4 = new QGridLayout();
        gridLayout_4->setObjectName("gridLayout_4");
        ui_timeLineHide = new QRadioButton(groupBox_3);
        ui_timeLineHide->setObjectName("ui_timeLineHide");

        gridLayout_4->addWidget(ui_timeLineHide, 0, 0, 1, 1);

        ui_timeLineForeground = new QRadioButton(groupBox_3);
        ui_timeLineForeground->setObjectName("ui_timeLineForeground");

        gridLayout_4->addWidget(ui_timeLineForeground, 0, 1, 1, 1);

        ui_timeLineBackground = new QRadioButton(groupBox_3);
        ui_timeLineBackground->setObjectName("ui_timeLineBackground");

        gridLayout_4->addWidget(ui_timeLineBackground, 0, 2, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_3 = new QLabel(groupBox_3);
        label_3->setObjectName("label_3");

        horizontalLayout_3->addWidget(label_3);

        ui_timeLineInterval = new QSpinBox(groupBox_3);
        ui_timeLineInterval->setObjectName("ui_timeLineInterval");

        horizontalLayout_3->addWidget(ui_timeLineInterval);


        gridLayout_4->addLayout(horizontalLayout_3, 1, 0, 1, 2);


        verticalLayout_2->addLayout(gridLayout_4);

        line = new QFrame(groupBox_3);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line);

        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName("gridLayout_3");
        ui_timeLineUseCustom = new QCheckBox(groupBox_3);
        ui_timeLineUseCustom->setObjectName("ui_timeLineUseCustom");

        gridLayout_3->addWidget(ui_timeLineUseCustom, 0, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label_2 = new QLabel(groupBox_3);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        ui_timeLineStroke = new QSpinBox(groupBox_3);
        ui_timeLineStroke->setObjectName("ui_timeLineStroke");

        horizontalLayout->addWidget(ui_timeLineStroke);


        gridLayout_3->addLayout(horizontalLayout, 1, 0, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label = new QLabel(groupBox_3);
        label->setObjectName("label");

        horizontalLayout_2->addWidget(label);

        ui_timeLineColor = new KColorCombo(groupBox_3);
        ui_timeLineColor->setObjectName("ui_timeLineColor");

        horizontalLayout_2->addWidget(ui_timeLineColor);


        gridLayout_3->addLayout(horizontalLayout_2, 1, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout_3);


        verticalLayout_3->addWidget(groupBox_3);

        groupBox_4 = new QGroupBox(KPlato__GanttChartDisplayOptions);
        groupBox_4->setObjectName("groupBox_4");
        verticalLayout = new QVBoxLayout(groupBox_4);
        verticalLayout->setObjectName("verticalLayout");
        ui_showRowSeparators = new QCheckBox(groupBox_4);
        ui_showRowSeparators->setObjectName("ui_showRowSeparators");

        verticalLayout->addWidget(ui_showRowSeparators);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_4 = new QLabel(groupBox_4);
        label_4->setObjectName("label_4");

        horizontalLayout_4->addWidget(label_4);

        freedays = new QComboBox(groupBox_4);
        freedays->setObjectName("freedays");
        freedays->setMinimumSize(QSize(200, 0));

        horizontalLayout_4->addWidget(freedays);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_4);


        verticalLayout_3->addWidget(groupBox_4);

        groupBox = new QGroupBox(KPlato__GanttChartDisplayOptions);
        groupBox->setObjectName("groupBox");
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        ui_showTaskName = new QCheckBox(groupBox);
        ui_showTaskName->setObjectName("ui_showTaskName");

        gridLayout_2->addWidget(ui_showTaskName, 0, 0, 1, 1);

        ui_showResourceNames = new QCheckBox(groupBox);
        ui_showResourceNames->setObjectName("ui_showResourceNames");

        gridLayout_2->addWidget(ui_showResourceNames, 0, 1, 1, 1);


        verticalLayout_3->addWidget(groupBox);

        groupBox_2 = new QGroupBox(KPlato__GanttChartDisplayOptions);
        groupBox_2->setObjectName("groupBox_2");
        groupBox_2->setFlat(false);
        gridLayout = new QGridLayout(groupBox_2);
        gridLayout->setObjectName("gridLayout");
        ui_showCriticalPath = new QCheckBox(groupBox_2);
        ui_showCriticalPath->setObjectName("ui_showCriticalPath");

        gridLayout->addWidget(ui_showCriticalPath, 2, 0, 1, 1);

        ui_showDependencies = new QCheckBox(groupBox_2);
        ui_showDependencies->setObjectName("ui_showDependencies");

        gridLayout->addWidget(ui_showDependencies, 0, 0, 1, 1);

        ui_showNegativeFloat = new QCheckBox(groupBox_2);
        ui_showNegativeFloat->setObjectName("ui_showNegativeFloat");

        gridLayout->addWidget(ui_showNegativeFloat, 2, 1, 1, 1);

        ui_showCriticalTasks = new QCheckBox(groupBox_2);
        ui_showCriticalTasks->setObjectName("ui_showCriticalTasks");

        gridLayout->addWidget(ui_showCriticalTasks, 3, 0, 1, 1);

        ui_showCompletion = new QCheckBox(groupBox_2);
        ui_showCompletion->setObjectName("ui_showCompletion");

        gridLayout->addWidget(ui_showCompletion, 1, 0, 1, 1);

        ui_showPositiveFloat = new QCheckBox(groupBox_2);
        ui_showPositiveFloat->setObjectName("ui_showPositiveFloat");

        gridLayout->addWidget(ui_showPositiveFloat, 1, 1, 1, 1);

        ui_showSchedulingError = new QCheckBox(groupBox_2);
        ui_showSchedulingError->setObjectName("ui_showSchedulingError");

        gridLayout->addWidget(ui_showSchedulingError, 0, 1, 1, 1);

        ui_showTimeConstraint = new QCheckBox(groupBox_2);
        ui_showTimeConstraint->setObjectName("ui_showTimeConstraint");

        gridLayout->addWidget(ui_showTimeConstraint, 3, 1, 1, 1);


        verticalLayout_3->addWidget(groupBox_2);

        verticalSpacer = new QSpacerItem(20, 15, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        label_3->setBuddy(ui_timeLineInterval);
        label_2->setBuddy(ui_timeLineStroke);
        label->setBuddy(ui_timeLineColor);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__GanttChartDisplayOptions);

        QMetaObject::connectSlotsByName(KPlato__GanttChartDisplayOptions);
    } // setupUi

    void retranslateUi(QWidget *KPlato__GanttChartDisplayOptions)
    {
#if QT_CONFIG(tooltip)
        KPlato__GanttChartDisplayOptions->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        groupBox_3->setTitle(tr2i18n("Timeline", nullptr));
#if QT_CONFIG(tooltip)
        ui_timeLineHide->setToolTip(tr2i18n("Do not show the timeline", nullptr));
#endif // QT_CONFIG(tooltip)
        ui_timeLineHide->setText(tr2i18n("Hide", nullptr));
#if QT_CONFIG(tooltip)
        ui_timeLineForeground->setToolTip(tr2i18n("Show timeline in the foreground", nullptr));
#endif // QT_CONFIG(tooltip)
        ui_timeLineForeground->setText(tr2i18n("Foreground", nullptr));
#if QT_CONFIG(tooltip)
        ui_timeLineBackground->setToolTip(tr2i18n("Show timeline in the background", nullptr));
#endif // QT_CONFIG(tooltip)
        ui_timeLineBackground->setText(tr2i18n("Background", nullptr));
#if QT_CONFIG(tooltip)
        label_3->setToolTip(tr2i18n("Update interval in minutes, 0 disables update", nullptr));
#endif // QT_CONFIG(tooltip)
        label_3->setText(tr2i18n("Interval:", nullptr));
#if QT_CONFIG(tooltip)
        ui_timeLineInterval->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        ui_timeLineUseCustom->setText(tr2i18n("Use custom line", nullptr));
#if QT_CONFIG(tooltip)
        label_2->setToolTip(tr2i18n("Line width", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(tr2i18n("Stroke:", nullptr));
#if QT_CONFIG(tooltip)
        ui_timeLineStroke->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label->setToolTip(tr2i18n("Line color", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(tr2i18n("Color:", nullptr));
#if QT_CONFIG(tooltip)
        ui_timeLineColor->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        groupBox_4->setTitle(tr2i18n("Decorations", nullptr));
        ui_showRowSeparators->setText(tr2i18n("Show Row Separators", nullptr));
        label_4->setText(tr2i18n("Freedays:", nullptr));
        groupBox->setTitle(tr2i18n("Additional text information", nullptr));
        ui_showTaskName->setText(tr2i18n("Task name", nullptr));
        ui_showResourceNames->setText(tr2i18n("Resource names", nullptr));
        groupBox_2->setTitle(tr2i18n("Additional graphics information", nullptr));
        ui_showCriticalPath->setText(tr2i18n("Critical path", nullptr));
        ui_showDependencies->setText(tr2i18n("Task dependencies", nullptr));
        ui_showNegativeFloat->setText(tr2i18n("Negative float", nullptr));
        ui_showCriticalTasks->setText(tr2i18n("Critical tasks", nullptr));
        ui_showCompletion->setText(tr2i18n("Task completion", nullptr));
        ui_showPositiveFloat->setText(tr2i18n("Positive float", nullptr));
        ui_showSchedulingError->setText(tr2i18n("Scheduling errors", nullptr));
        ui_showTimeConstraint->setText(tr2i18n("Time constraints", nullptr));
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class GanttChartDisplayOptions: public Ui_GanttChartDisplayOptions {};
} // namespace Ui
} // namespace KPlato

#endif // KPTGANTTCHARTDISPLAYOPTIONS_H

