/********************************************************************************
** Form generated from reading UI file 'ConfigReportTemplatesPanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGREPORTTEMPLATESPANEL_H
#define UI_CONFIGREPORTTEMPLATESPANEL_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_ConfigReportTemplatesPanel
{
public:
    QGridLayout *gridLayout;
    QToolButton *insertModule;
    QSpacerItem *verticalSpacer_3;
    QToolButton *removeModule;
    QTreeView *reportTemplatesView;

    void setupUi(QWidget *KPlato__ConfigReportTemplatesPanel)
    {
        if (KPlato__ConfigReportTemplatesPanel->objectName().isEmpty())
            KPlato__ConfigReportTemplatesPanel->setObjectName("KPlato__ConfigReportTemplatesPanel");
        KPlato__ConfigReportTemplatesPanel->resize(400, 300);
        gridLayout = new QGridLayout(KPlato__ConfigReportTemplatesPanel);
        gridLayout->setObjectName("gridLayout");
        insertModule = new QToolButton(KPlato__ConfigReportTemplatesPanel);
        insertModule->setObjectName("insertModule");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("libs/ui"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        insertModule->setIcon(icon);

        gridLayout->addWidget(insertModule, 0, 1, 1, 1);

        verticalSpacer_3 = new QSpacerItem(20, 181, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer_3, 2, 1, 1, 1);

        removeModule = new QToolButton(KPlato__ConfigReportTemplatesPanel);
        removeModule->setObjectName("removeModule");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("libs/ui"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        removeModule->setIcon(icon1);

        gridLayout->addWidget(removeModule, 1, 1, 1, 1);

        reportTemplatesView = new QTreeView(KPlato__ConfigReportTemplatesPanel);
        reportTemplatesView->setObjectName("reportTemplatesView");
        reportTemplatesView->setRootIsDecorated(false);
        reportTemplatesView->setItemsExpandable(false);
        reportTemplatesView->header()->setVisible(false);

        gridLayout->addWidget(reportTemplatesView, 0, 0, 3, 1);


        retranslateUi(KPlato__ConfigReportTemplatesPanel);

        QMetaObject::connectSlotsByName(KPlato__ConfigReportTemplatesPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__ConfigReportTemplatesPanel)
    {
        insertModule->setText(tr2i18n("Insert", nullptr));
        removeModule->setText(tr2i18n("Remove", nullptr));
        (void)KPlato__ConfigReportTemplatesPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class ConfigReportTemplatesPanel: public Ui_ConfigReportTemplatesPanel {};
} // namespace Ui
} // namespace KPlato

#endif // CONFIGREPORTTEMPLATESPANEL_H

