/********************************************************************************
** Form generated from reading UI file 'kptworkpackagemergepanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTWORKPACKAGEMERGEPANEL_H
#define UI_KPTWORKPACKAGEMERGEPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_WorkPackageMergePanel
{
public:
    QVBoxLayout *layout_2;
    QStackedWidget *ui_stackedWidget;
    QWidget *ui_packagePage;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *ui_ownerName;
    QLabel *label;
    QLabel *label_2;
    QLabel *label_3;
    QLabel *ui_packageTime;
    QLabel *ui_taskName;
    QFrame *line;
    QHBoxLayout *horizontalLayout_5;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout;
    QCheckBox *ui_updateStarted;
    QLabel *ui_started;
    QGroupBox *groupBox_2;
    QHBoxLayout *horizontalLayout_3;
    QCheckBox *ui_updateFinished;
    QLabel *ui_finished;
    QSpacerItem *horizontalSpacer;
    QGroupBox *groupBox_3;
    QHBoxLayout *horizontalLayout_4;
    QCheckBox *ui_updateDocuments;
    QLabel *ui_documents;
    QFrame *line_2;
    QHBoxLayout *horizontalLayout_2;
    QTreeView *ui_completionView;
    QWidget *ui_progressPage;
    QHBoxLayout *ui_progressLayout;
    QWidget *ui_finishPage;
    QHBoxLayout *horizontalLayout_7;
    QLabel *label_4;

    void setupUi(QWidget *KPlato__WorkPackageMergePanel)
    {
        if (KPlato__WorkPackageMergePanel->objectName().isEmpty())
            KPlato__WorkPackageMergePanel->setObjectName("KPlato__WorkPackageMergePanel");
        KPlato__WorkPackageMergePanel->resize(619, 356);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(KPlato__WorkPackageMergePanel->sizePolicy().hasHeightForWidth());
        KPlato__WorkPackageMergePanel->setSizePolicy(sizePolicy);
        KPlato__WorkPackageMergePanel->setMinimumSize(QSize(0, 0));
        layout_2 = new QVBoxLayout(KPlato__WorkPackageMergePanel);
        layout_2->setObjectName("layout_2");
        ui_stackedWidget = new QStackedWidget(KPlato__WorkPackageMergePanel);
        ui_stackedWidget->setObjectName("ui_stackedWidget");
        ui_packagePage = new QWidget();
        ui_packagePage->setObjectName("ui_packagePage");
        verticalLayout = new QVBoxLayout(ui_packagePage);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        ui_ownerName = new QLabel(ui_packagePage);
        ui_ownerName->setObjectName("ui_ownerName");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(ui_ownerName->sizePolicy().hasHeightForWidth());
        ui_ownerName->setSizePolicy(sizePolicy1);
        ui_ownerName->setFrameShape(QFrame::StyledPanel);
        ui_ownerName->setFrameShadow(QFrame::Sunken);
        ui_ownerName->setText(QString::fromUtf8("<owner name>"));

        gridLayout->addWidget(ui_ownerName, 7, 1, 1, 1);

        label = new QLabel(ui_packagePage);
        label->setObjectName("label");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(label, 5, 0, 1, 1);

        label_2 = new QLabel(ui_packagePage);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 7, 2, 1, 1);

        label_3 = new QLabel(ui_packagePage);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 7, 0, 1, 1);

        ui_packageTime = new QLabel(ui_packagePage);
        ui_packageTime->setObjectName("ui_packageTime");
        sizePolicy2.setHeightForWidth(ui_packageTime->sizePolicy().hasHeightForWidth());
        ui_packageTime->setSizePolicy(sizePolicy2);
        ui_packageTime->setFrameShape(QFrame::StyledPanel);
        ui_packageTime->setFrameShadow(QFrame::Sunken);

        gridLayout->addWidget(ui_packageTime, 7, 3, 1, 1);

        ui_taskName = new QLabel(ui_packagePage);
        ui_taskName->setObjectName("ui_taskName");
        sizePolicy1.setHeightForWidth(ui_taskName->sizePolicy().hasHeightForWidth());
        ui_taskName->setSizePolicy(sizePolicy1);
        ui_taskName->setFrameShape(QFrame::StyledPanel);
        ui_taskName->setFrameShadow(QFrame::Sunken);
        ui_taskName->setText(QString::fromUtf8("<task name>"));

        gridLayout->addWidget(ui_taskName, 5, 1, 1, 3);


        verticalLayout->addLayout(gridLayout);

        line = new QFrame(ui_packagePage);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        groupBox = new QGroupBox(ui_packagePage);
        groupBox->setObjectName("groupBox");
        horizontalLayout = new QHBoxLayout(groupBox);
        horizontalLayout->setObjectName("horizontalLayout");
        ui_updateStarted = new QCheckBox(groupBox);
        ui_updateStarted->setObjectName("ui_updateStarted");

        horizontalLayout->addWidget(ui_updateStarted);

        ui_started = new QLabel(groupBox);
        ui_started->setObjectName("ui_started");
        ui_started->setText(QString::fromUtf8("<started>"));

        horizontalLayout->addWidget(ui_started);


        horizontalLayout_5->addWidget(groupBox);

        groupBox_2 = new QGroupBox(ui_packagePage);
        groupBox_2->setObjectName("groupBox_2");
        horizontalLayout_3 = new QHBoxLayout(groupBox_2);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        ui_updateFinished = new QCheckBox(groupBox_2);
        ui_updateFinished->setObjectName("ui_updateFinished");

        horizontalLayout_3->addWidget(ui_updateFinished);

        ui_finished = new QLabel(groupBox_2);
        ui_finished->setObjectName("ui_finished");
        ui_finished->setText(QString::fromUtf8("<finished>"));

        horizontalLayout_3->addWidget(ui_finished);


        horizontalLayout_5->addWidget(groupBox_2);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer);

        groupBox_3 = new QGroupBox(ui_packagePage);
        groupBox_3->setObjectName("groupBox_3");
        horizontalLayout_4 = new QHBoxLayout(groupBox_3);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        ui_updateDocuments = new QCheckBox(groupBox_3);
        ui_updateDocuments->setObjectName("ui_updateDocuments");

        horizontalLayout_4->addWidget(ui_updateDocuments);

        ui_documents = new QLabel(groupBox_3);
        ui_documents->setObjectName("ui_documents");
        ui_documents->setText(QString::fromUtf8("<documents>l"));

        horizontalLayout_4->addWidget(ui_documents);


        horizontalLayout_5->addWidget(groupBox_3);


        verticalLayout->addLayout(horizontalLayout_5);

        line_2 = new QFrame(ui_packagePage);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::VLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_2);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        ui_completionView = new QTreeView(ui_packagePage);
        ui_completionView->setObjectName("ui_completionView");
        sizePolicy2.setHeightForWidth(ui_completionView->sizePolicy().hasHeightForWidth());
        ui_completionView->setSizePolicy(sizePolicy2);

        horizontalLayout_2->addWidget(ui_completionView);


        verticalLayout->addLayout(horizontalLayout_2);

        ui_stackedWidget->addWidget(ui_packagePage);
        ui_progressPage = new QWidget();
        ui_progressPage->setObjectName("ui_progressPage");
        ui_progressLayout = new QHBoxLayout(ui_progressPage);
        ui_progressLayout->setObjectName("ui_progressLayout");
        ui_stackedWidget->addWidget(ui_progressPage);
        ui_finishPage = new QWidget();
        ui_finishPage->setObjectName("ui_finishPage");
        horizontalLayout_7 = new QHBoxLayout(ui_finishPage);
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        label_4 = new QLabel(ui_finishPage);
        label_4->setObjectName("label_4");
        label_4->setAlignment(Qt::AlignCenter);

        horizontalLayout_7->addWidget(label_4);

        ui_stackedWidget->addWidget(ui_finishPage);

        layout_2->addWidget(ui_stackedWidget);


        retranslateUi(KPlato__WorkPackageMergePanel);

        ui_stackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KPlato__WorkPackageMergePanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__WorkPackageMergePanel)
    {
        label->setText(tr2i18n("Task:", nullptr));
        label_2->setText(tr2i18n("Generated:", nullptr));
        label_3->setText(tr2i18n("From:", nullptr));
        ui_packageTime->setText(tr2i18n("<package time>", nullptr));
        groupBox->setTitle(tr2i18n("Started", nullptr));
        ui_updateStarted->setText(tr2i18n("Update", nullptr));
        groupBox_2->setTitle(tr2i18n("Finished", nullptr));
        ui_updateFinished->setText(tr2i18n("Update", nullptr));
        groupBox_3->setTitle(tr2i18n("Documents", nullptr));
        ui_updateDocuments->setText(tr2i18n("Update", nullptr));
        label_4->setText(tr2i18n("No more data. Press Close to finish.", nullptr));
        (void)KPlato__WorkPackageMergePanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class WorkPackageMergePanel: public Ui_WorkPackageMergePanel {};
} // namespace Ui
} // namespace KPlato

#endif // KPTWORKPACKAGEMERGEPANEL_H

