/********************************************************************************
** Form generated from reading UI file 'ParameterSubstitutionPanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PARAMETERSUBSTITUTIONPANEL_H
#define UI_PARAMETERSUBSTITUTIONPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_ParameterSubstitutionPanel
{
public:
    QHBoxLayout *horizontalLayout;
    QTreeView *treeView;

    void setupUi(QWidget *KPlato__ParameterSubstitutionPanel)
    {
        if (KPlato__ParameterSubstitutionPanel->objectName().isEmpty())
            KPlato__ParameterSubstitutionPanel->setObjectName("KPlato__ParameterSubstitutionPanel");
        KPlato__ParameterSubstitutionPanel->resize(609, 254);
        horizontalLayout = new QHBoxLayout(KPlato__ParameterSubstitutionPanel);
        horizontalLayout->setObjectName("horizontalLayout");
        treeView = new QTreeView(KPlato__ParameterSubstitutionPanel);
        treeView->setObjectName("treeView");
        treeView->setTabKeyNavigation(true);
        treeView->setSelectionBehavior(QAbstractItemView::SelectItems);
        treeView->setRootIsDecorated(false);
        treeView->setItemsExpandable(false);

        horizontalLayout->addWidget(treeView);


        retranslateUi(KPlato__ParameterSubstitutionPanel);

        QMetaObject::connectSlotsByName(KPlato__ParameterSubstitutionPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__ParameterSubstitutionPanel)
    {
        (void)KPlato__ParameterSubstitutionPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class ParameterSubstitutionPanel: public Ui_ParameterSubstitutionPanel {};
} // namespace Ui
} // namespace KPlato

#endif // PARAMETERSUBSTITUTIONPANEL_H

