/****************************************************************************
** Meta object code from reading C++ file 'WelcomeView.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/main/WelcomeView.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'WelcomeView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11WelcomeViewE_t {};
} // unnamed namespace

template <> constexpr inline auto WelcomeView::qt_create_metaobjectdata<qt_meta_tag_ZN11WelcomeViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WelcomeView",
        "newProject",
        "",
        "openProject",
        "recentProject",
        "QUrl",
        "file",
        "KoPart*",
        "part",
        "selectDefaultView",
        "loadSharedResources",
        "url",
        "projects",
        "openExistingFile",
        "projectCreated",
        "finished",
        "openTemplate",
        "slotContextMenuRequested",
        "QModelIndex",
        "index",
        "QPoint",
        "pos",
        "slotRecentFileSelected",
        "idx",
        "slotEnableActions",
        "on",
        "slotNewProject",
        "slotOpenProject",
        "slotLoadSharedResources",
        "loadProjectsAtStartup",
        "slotOpenProjectTemplate",
        "slotCreateResourceFile",
        "slotTemplateDocumentLoaded"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newProject'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openProject'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'recentProject'
        QtMocHelpers::SignalData<void(const QUrl &, KoPart *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'selectDefaultView'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loadSharedResources'
        QtMocHelpers::SignalData<void(const QUrl &, const QUrl &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 11 }, { 0x80000000 | 5, 12 },
        }}),
        // Signal 'openExistingFile'
        QtMocHelpers::SignalData<void(const QUrl &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 11 },
        }}),
        // Signal 'projectCreated'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openTemplate'
        QtMocHelpers::SignalData<void(QUrl)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QPoint &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'slotRecentFileSelected'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 23 },
        }}),
        // Slot 'slotEnableActions'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'slotNewProject'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenProject'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotLoadSharedResources'
        QtMocHelpers::SlotData<void(const QString &, const QUrl &, bool)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 5, 12 }, { QMetaType::Bool, 29 },
        }}),
        // Slot 'slotOpenProjectTemplate'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 23 },
        }}),
        // Slot 'slotCreateResourceFile'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTemplateDocumentLoaded'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WelcomeView, qt_meta_tag_ZN11WelcomeViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WelcomeView::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11WelcomeViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11WelcomeViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11WelcomeViewE_t>.metaTypes,
    nullptr
} };

void WelcomeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WelcomeView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newProject(); break;
        case 1: _t->openProject(); break;
        case 2: _t->recentProject((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KoPart*>>(_a[2]))); break;
        case 3: _t->selectDefaultView(); break;
        case 4: _t->loadSharedResources((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 5: _t->openExistingFile((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 6: _t->projectCreated(); break;
        case 7: _t->finished(); break;
        case 8: _t->openTemplate((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 9: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 10: _t->slotRecentFileSelected((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 11: _t->slotEnableActions((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->slotNewProject(); break;
        case 13: _t->slotOpenProject(); break;
        case 14: _t->slotLoadSharedResources((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 15: _t->slotOpenProjectTemplate((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 16: _t->slotCreateResourceFile(); break;
        case 17: _t->slotTemplateDocumentLoaded(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WelcomeView::*)()>(_a, &WelcomeView::newProject, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WelcomeView::*)()>(_a, &WelcomeView::openProject, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WelcomeView::*)(const QUrl & , KoPart * )>(_a, &WelcomeView::recentProject, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WelcomeView::*)()>(_a, &WelcomeView::selectDefaultView, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WelcomeView::*)(const QUrl & , const QUrl & )>(_a, &WelcomeView::loadSharedResources, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WelcomeView::*)(const QUrl & )>(_a, &WelcomeView::openExistingFile, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WelcomeView::*)()>(_a, &WelcomeView::projectCreated, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WelcomeView::*)()>(_a, &WelcomeView::finished, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WelcomeView::*)(QUrl )>(_a, &WelcomeView::openTemplate, 8))
            return;
    }
}

const QMetaObject *WelcomeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WelcomeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11WelcomeViewE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int WelcomeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void WelcomeView::newProject()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void WelcomeView::openProject()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void WelcomeView::recentProject(const QUrl & _t1, KoPart * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void WelcomeView::selectDefaultView()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void WelcomeView::loadSharedResources(const QUrl & _t1, const QUrl & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void WelcomeView::openExistingFile(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void WelcomeView::projectCreated()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void WelcomeView::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void WelcomeView::openTemplate(QUrl _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
