/****************************************************************************
** Meta object code from reading C++ file 'kptcalendarmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/models/kptcalendarmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptcalendarmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato24CalendarDayItemModelBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::CalendarDayItemModelBase::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato24CalendarDayItemModelBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::CalendarDayItemModelBase",
        "slotCalendarToBeRemoved",
        "",
        "const KPlato::Calendar*",
        "calendar"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotCalendarToBeRemoved'
        QtMocHelpers::SlotData<void(const KPlato::Calendar *)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CalendarDayItemModelBase, qt_meta_tag_ZN6KPlato24CalendarDayItemModelBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::CalendarDayItemModelBase::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato24CalendarDayItemModelBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato24CalendarDayItemModelBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato24CalendarDayItemModelBaseE_t>.metaTypes,
    nullptr
} };

void KPlato::CalendarDayItemModelBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CalendarDayItemModelBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotCalendarToBeRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::Calendar*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::CalendarDayItemModelBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::CalendarDayItemModelBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato24CalendarDayItemModelBaseE_t>.strings))
        return static_cast<void*>(this);
    return ItemModelBase::qt_metacast(_clname);
}

int KPlato::CalendarDayItemModelBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato17CalendarItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::CalendarItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato17CalendarItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::CalendarItemModel",
        "slotCalendarChanged",
        "",
        "KPlato::Calendar*",
        "slotCalendarToBeInserted",
        "const KPlato::Calendar*",
        "parent",
        "row",
        "slotCalendarInserted",
        "calendar",
        "slotCalendarToBeRemoved",
        "slotCalendarRemoved",
        "Properties",
        "Name",
        "Origin",
        "TimeZone",
        "HolidayRegion"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotCalendarChanged'
        QtMocHelpers::SlotData<void(KPlato::Calendar *)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotCalendarToBeInserted'
        QtMocHelpers::SlotData<void(const KPlato::Calendar *, int)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::Int, 7 },
        }}),
        // Slot 'slotCalendarInserted'
        QtMocHelpers::SlotData<void(const KPlato::Calendar *)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
        // Slot 'slotCalendarToBeRemoved'
        QtMocHelpers::SlotData<void(const KPlato::Calendar *)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
        // Slot 'slotCalendarRemoved'
        QtMocHelpers::SlotData<void(const KPlato::Calendar *)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Properties'
        QtMocHelpers::EnumData<enum Properties>(12, 12, QMC::EnumFlags{}).add({
            {   13, Properties::Name },
            {   14, Properties::Origin },
            {   15, Properties::TimeZone },
            {   16, Properties::HolidayRegion },
        }),
    };
    return QtMocHelpers::metaObjectData<CalendarItemModel, qt_meta_tag_ZN6KPlato17CalendarItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::CalendarItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17CalendarItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17CalendarItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato17CalendarItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::CalendarItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CalendarItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotCalendarChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Calendar*>>(_a[1]))); break;
        case 1: _t->slotCalendarToBeInserted((*reinterpret_cast<std::add_pointer_t<const KPlato::Calendar*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->slotCalendarInserted((*reinterpret_cast<std::add_pointer_t<const KPlato::Calendar*>>(_a[1]))); break;
        case 3: _t->slotCalendarToBeRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::Calendar*>>(_a[1]))); break;
        case 4: _t->slotCalendarRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::Calendar*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Calendar* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KPlato::CalendarItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::CalendarItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17CalendarItemModelE_t>.strings))
        return static_cast<void*>(this);
    return ItemModelBase::qt_metacast(_clname);
}

int KPlato::CalendarItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato20CalendarDayItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::CalendarDayItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato20CalendarDayItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::CalendarDayItemModel",
        "slotDayChanged",
        "",
        "KPlato::CalendarDay*",
        "day",
        "slotTimeIntervalChanged",
        "KPlato::TimeInterval*",
        "ti",
        "slotWorkIntervalAdded",
        "slotWorkIntervalRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotDayChanged'
        QtMocHelpers::SlotData<void(KPlato::CalendarDay *)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotTimeIntervalChanged'
        QtMocHelpers::SlotData<void(KPlato::TimeInterval *)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotWorkIntervalAdded'
        QtMocHelpers::SlotData<void(KPlato::CalendarDay *, KPlato::TimeInterval *)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotWorkIntervalRemoved'
        QtMocHelpers::SlotData<void(KPlato::CalendarDay *, KPlato::TimeInterval *)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 6, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CalendarDayItemModel, qt_meta_tag_ZN6KPlato20CalendarDayItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::CalendarDayItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<CalendarDayItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20CalendarDayItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20CalendarDayItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato20CalendarDayItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::CalendarDayItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CalendarDayItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotDayChanged((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1]))); break;
        case 1: _t->slotTimeIntervalChanged((*reinterpret_cast<std::add_pointer_t<KPlato::TimeInterval*>>(_a[1]))); break;
        case 2: _t->slotWorkIntervalAdded((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::TimeInterval*>>(_a[2]))); break;
        case 3: _t->slotWorkIntervalRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::TimeInterval*>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::CalendarDayItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::CalendarDayItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20CalendarDayItemModelE_t>.strings))
        return static_cast<void*>(this);
    return CalendarDayItemModelBase::qt_metacast(_clname);
}

int KPlato::CalendarDayItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CalendarDayItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato18DateTableDataModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::DateTableDataModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato18DateTableDataModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::DateTableDataModel",
        "setCalendar",
        "",
        "KPlato::Calendar*",
        "calendar"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setCalendar'
        QtMocHelpers::SlotData<void(KPlato::Calendar *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DateTableDataModel, qt_meta_tag_ZN6KPlato18DateTableDataModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::DateTableDataModel::staticMetaObject = { {
    QMetaObject::SuperData::link<KDateTableDataModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18DateTableDataModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18DateTableDataModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato18DateTableDataModelE_t>.metaTypes,
    nullptr
} };

void KPlato::DateTableDataModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DateTableDataModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setCalendar((*reinterpret_cast<std::add_pointer_t<KPlato::Calendar*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Calendar* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KPlato::DateTableDataModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::DateTableDataModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18DateTableDataModelE_t>.strings))
        return static_cast<void*>(this);
    return KDateTableDataModel::qt_metacast(_clname);
}

int KPlato::DateTableDataModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KDateTableDataModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato21DateTableDateDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::DateTableDateDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato21DateTableDateDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::DateTableDateDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DateTableDateDelegate, qt_meta_tag_ZN6KPlato21DateTableDateDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::DateTableDateDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<KDateTableDateDelegate::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato21DateTableDateDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato21DateTableDateDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato21DateTableDateDelegateE_t>.metaTypes,
    nullptr
} };

void KPlato::DateTableDateDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DateTableDateDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::DateTableDateDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::DateTableDateDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato21DateTableDateDelegateE_t>.strings))
        return static_cast<void*>(this);
    return KDateTableDateDelegate::qt_metacast(_clname);
}

int KPlato::DateTableDateDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KDateTableDateDelegate::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato25CalendarExtendedItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::CalendarExtendedItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato25CalendarExtendedItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::CalendarExtendedItemModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CalendarExtendedItemModel, qt_meta_tag_ZN6KPlato25CalendarExtendedItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::CalendarExtendedItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<CalendarItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato25CalendarExtendedItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato25CalendarExtendedItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato25CalendarExtendedItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::CalendarExtendedItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CalendarExtendedItemModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::CalendarExtendedItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::CalendarExtendedItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato25CalendarExtendedItemModelE_t>.strings))
        return static_cast<void*>(this);
    return CalendarItemModel::qt_metacast(_clname);
}

int KPlato::CalendarExtendedItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CalendarItemModel::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
