/****************************************************************************
** Meta object code from reading C++ file 'kptmainprojectpanel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/kptmainprojectpanel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptmainprojectpanel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato16MainProjectPanelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::MainProjectPanel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato16MainProjectPanelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::MainProjectPanel",
        "obligatedFieldsFilled",
        "",
        "changed",
        "loadResourceAssignments",
        "QUrl",
        "url",
        "slotCheckAllFieldsFilled",
        "slotChooseLeader",
        "slotStartDateClicked",
        "slotEndDateClicked",
        "enableDateTime",
        "insertTaskModuleClicked",
        "removeTaskModuleClicked",
        "taskModulesSelectionChanged",
        "openResourcesFile"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'obligatedFieldsFilled'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'changed'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loadResourceAssignments'
        QtMocHelpers::SignalData<void(QUrl)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotCheckAllFieldsFilled'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotChooseLeader'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotStartDateClicked'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEndDateClicked'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enableDateTime'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertTaskModuleClicked'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'removeTaskModuleClicked'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'taskModulesSelectionChanged'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openResourcesFile'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainProjectPanel, qt_meta_tag_ZN6KPlato16MainProjectPanelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::MainProjectPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16MainProjectPanelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16MainProjectPanelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato16MainProjectPanelE_t>.metaTypes,
    nullptr
} };

void KPlato::MainProjectPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainProjectPanel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->obligatedFieldsFilled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->changed(); break;
        case 2: _t->loadResourceAssignments((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 3: _t->slotCheckAllFieldsFilled(); break;
        case 4: _t->slotChooseLeader(); break;
        case 5: _t->slotStartDateClicked(); break;
        case 6: _t->slotEndDateClicked(); break;
        case 7: _t->enableDateTime(); break;
        case 8: _t->insertTaskModuleClicked(); break;
        case 9: _t->removeTaskModuleClicked(); break;
        case 10: _t->taskModulesSelectionChanged(); break;
        case 11: _t->openResourcesFile(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainProjectPanel::*)(bool )>(_a, &MainProjectPanel::obligatedFieldsFilled, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainProjectPanel::*)()>(_a, &MainProjectPanel::changed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainProjectPanel::*)(QUrl )>(_a, &MainProjectPanel::loadResourceAssignments, 2))
            return;
    }
}

const QMetaObject *KPlato::MainProjectPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::MainProjectPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16MainProjectPanelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui_MainProjectPanelBase"))
        return static_cast< Ui_MainProjectPanelBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int KPlato::MainProjectPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void KPlato::MainProjectPanel::obligatedFieldsFilled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::MainProjectPanel::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KPlato::MainProjectPanel::loadResourceAssignments(QUrl _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
