/********************************************************************************
** Form generated from reading UI file 'ResourceUsageView.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RESOURCEUSAGEVIEW_H
#define UI_RESOURCEUSAGEVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QWidget>
#include <ScrollableChart.h>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ResourceUsageView
{
public:
    QHBoxLayout *horizontalLayout;
    QSplitter *splitter;
    QTreeView *resourceView;
    KPlato::ScrollableChart *chart;

    void setupUi(QWidget *ResourceUsageView)
    {
        if (ResourceUsageView->objectName().isEmpty())
            ResourceUsageView->setObjectName("ResourceUsageView");
        ResourceUsageView->resize(848, 392);
        ResourceUsageView->setFocusPolicy(Qt::StrongFocus);
        horizontalLayout = new QHBoxLayout(ResourceUsageView);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        splitter = new QSplitter(ResourceUsageView);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Horizontal);
        resourceView = new QTreeView(splitter);
        resourceView->setObjectName("resourceView");
        splitter->addWidget(resourceView);
        chart = new KPlato::ScrollableChart(splitter);
        chart->setObjectName("chart");
        chart->setMinimumSize(QSize(100, 100));
        chart->setAutoFillBackground(true);
        splitter->addWidget(chart);

        horizontalLayout->addWidget(splitter);


        retranslateUi(ResourceUsageView);

        QMetaObject::connectSlotsByName(ResourceUsageView);
    } // setupUi

    void retranslateUi(QWidget *ResourceUsageView)
    {
        (void)ResourceUsageView;
    } // retranslateUi

};

namespace Ui {
    class ResourceUsageView: public Ui_ResourceUsageView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // RESOURCEUSAGEVIEW_H

