/****************************************************************************
** Meta object code from reading C++ file 'kptresourceappointmentsmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/models/kptresourceappointmentsmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptresourceappointmentsmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato29ResourceAppointmentsItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ResourceAppointmentsItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato29ResourceAppointmentsItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ResourceAppointmentsItemModel",
        "refreshed",
        "",
        "appointmentInserted",
        "KPlato::Resource*",
        "KPlato::Appointment*",
        "setScheduleManager",
        "KPlato::ScheduleManager*",
        "sm",
        "refresh",
        "slotResourceGroupInserted",
        "KPlato::ResourceGroup*",
        "group",
        "slotResourceGroupToBeRemoved",
        "KPlato::Project*",
        "project",
        "parent",
        "row",
        "slotResourceChanged",
        "slotResourceToBeInserted",
        "slotResourceInserted",
        "resource",
        "slotResourceToBeRemoved",
        "slotResourceRemoved",
        "slotCalendarChanged",
        "KPlato::Calendar*",
        "cal",
        "slotProjectCalculated",
        "slotAppointmentToBeInserted",
        "r",
        "slotAppointmentInserted",
        "slotAppointmentToBeRemoved",
        "slotAppointmentRemoved",
        "slotAppointmentChanged",
        "a"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'refreshed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'appointmentInserted'
        QtMocHelpers::SignalData<void(KPlato::Resource *, KPlato::Appointment *)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 }, { 0x80000000 | 5, 2 },
        }}),
        // Slot 'setScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'refresh'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotResourceGroupInserted'
        QtMocHelpers::SlotData<void(KPlato::ResourceGroup *)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'slotResourceGroupToBeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Project *, KPlato::ResourceGroup *, int, KPlato::ResourceGroup *)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 11, 16 }, { QMetaType::Int, 17 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'slotResourceChanged'
        QtMocHelpers::SlotData<void(KPlato::Resource *)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Slot 'slotResourceToBeInserted'
        QtMocHelpers::SlotData<void(KPlato::Project *, int)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'slotResourceInserted'
        QtMocHelpers::SlotData<void(KPlato::Resource *)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 21 },
        }}),
        // Slot 'slotResourceToBeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Project *, int, KPlato::Resource *)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { QMetaType::Int, 17 }, { 0x80000000 | 4, 21 },
        }}),
        // Slot 'slotResourceRemoved'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCalendarChanged'
        QtMocHelpers::SlotData<void(KPlato::Calendar *)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'slotProjectCalculated'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotAppointmentToBeInserted'
        QtMocHelpers::SlotData<void(KPlato::Resource *, int)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 29 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'slotAppointmentInserted'
        QtMocHelpers::SlotData<void(KPlato::Resource *, KPlato::Appointment *)>(30, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 2 }, { 0x80000000 | 5, 2 },
        }}),
        // Slot 'slotAppointmentToBeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Resource *, int)>(31, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 29 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'slotAppointmentRemoved'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAppointmentChanged'
        QtMocHelpers::SlotData<void(KPlato::Resource *, KPlato::Appointment *)>(33, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 29 }, { 0x80000000 | 5, 34 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ResourceAppointmentsItemModel, qt_meta_tag_ZN6KPlato29ResourceAppointmentsItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ResourceAppointmentsItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato29ResourceAppointmentsItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato29ResourceAppointmentsItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato29ResourceAppointmentsItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::ResourceAppointmentsItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResourceAppointmentsItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->refreshed(); break;
        case 1: _t->appointmentInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Appointment*>>(_a[2]))); break;
        case 2: _t->setScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 3: _t->refresh(); break;
        case 4: _t->slotResourceGroupInserted((*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[1]))); break;
        case 5: _t->slotResourceGroupToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[4]))); break;
        case 6: _t->slotResourceChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 7: _t->slotResourceToBeInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->slotResourceInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 9: _t->slotResourceToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[3]))); break;
        case 10: _t->slotResourceRemoved(); break;
        case 11: _t->slotCalendarChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Calendar*>>(_a[1]))); break;
        case 12: _t->slotProjectCalculated((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 13: _t->slotAppointmentToBeInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 14: _t->slotAppointmentInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Appointment*>>(_a[2]))); break;
        case 15: _t->slotAppointmentToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 16: _t->slotAppointmentRemoved(); break;
        case 17: _t->slotAppointmentChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Appointment*>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ResourceAppointmentsItemModel::*)()>(_a, &ResourceAppointmentsItemModel::refreshed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResourceAppointmentsItemModel::*)(KPlato::Resource * , KPlato::Appointment * )>(_a, &ResourceAppointmentsItemModel::appointmentInserted, 1))
            return;
    }
}

const QMetaObject *KPlato::ResourceAppointmentsItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ResourceAppointmentsItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato29ResourceAppointmentsItemModelE_t>.strings))
        return static_cast<void*>(this);
    return ItemModelBase::qt_metacast(_clname);
}

int KPlato::ResourceAppointmentsItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ResourceAppointmentsItemModel::refreshed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KPlato::ResourceAppointmentsItemModel::appointmentInserted(KPlato::Resource * _t1, KPlato::Appointment * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN6KPlato28ResourceAppointmentsRowModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ResourceAppointmentsRowModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato28ResourceAppointmentsRowModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ResourceAppointmentsRowModel",
        "setScheduleManager",
        "",
        "KPlato::ScheduleManager*",
        "sm",
        "slotResourceToBeInserted",
        "KPlato::Project*",
        "project",
        "row",
        "slotResourceInserted",
        "KPlato::Resource*",
        "resource",
        "slotResourceToBeRemoved",
        "slotResourceRemoved",
        "slotAppointmentToBeInserted",
        "r",
        "slotAppointmentInserted",
        "KPlato::Appointment*",
        "a",
        "slotAppointmentToBeRemoved",
        "slotAppointmentRemoved",
        "slotAppointmentChanged",
        "slotProjectCalculated",
        "Properties",
        "Name",
        "Type",
        "StartTime",
        "EndTime",
        "Load"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotResourceToBeInserted'
        QtMocHelpers::SlotData<void(KPlato::Project *, int)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::Int, 8 },
        }}),
        // Slot 'slotResourceInserted'
        QtMocHelpers::SlotData<void(KPlato::Resource *)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotResourceToBeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Project *, int, KPlato::Resource *)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::Int, 8 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotResourceRemoved'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAppointmentToBeInserted'
        QtMocHelpers::SlotData<void(KPlato::Resource *, int)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 15 }, { QMetaType::Int, 8 },
        }}),
        // Slot 'slotAppointmentInserted'
        QtMocHelpers::SlotData<void(KPlato::Resource *, KPlato::Appointment *)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 15 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotAppointmentToBeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Resource *, int)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 15 }, { QMetaType::Int, 8 },
        }}),
        // Slot 'slotAppointmentRemoved'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAppointmentChanged'
        QtMocHelpers::SlotData<void(KPlato::Resource *, KPlato::Appointment *)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 15 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotProjectCalculated'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Properties'
        QtMocHelpers::EnumData<enum Properties>(23, 23, QMC::EnumFlags{}).add({
            {   24, Properties::Name },
            {   25, Properties::Type },
            {   26, Properties::StartTime },
            {   27, Properties::EndTime },
            {   28, Properties::Load },
        }),
    };
    return QtMocHelpers::metaObjectData<ResourceAppointmentsRowModel, qt_meta_tag_ZN6KPlato28ResourceAppointmentsRowModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ResourceAppointmentsRowModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato28ResourceAppointmentsRowModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato28ResourceAppointmentsRowModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato28ResourceAppointmentsRowModelE_t>.metaTypes,
    nullptr
} };

void KPlato::ResourceAppointmentsRowModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResourceAppointmentsRowModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 1: _t->slotResourceToBeInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->slotResourceInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 3: _t->slotResourceToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[3]))); break;
        case 4: _t->slotResourceRemoved(); break;
        case 5: _t->slotAppointmentToBeInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->slotAppointmentInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Appointment*>>(_a[2]))); break;
        case 7: _t->slotAppointmentToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->slotAppointmentRemoved(); break;
        case 9: _t->slotAppointmentChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Appointment*>>(_a[2]))); break;
        case 10: _t->slotProjectCalculated((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::ResourceAppointmentsRowModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ResourceAppointmentsRowModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato28ResourceAppointmentsRowModelE_t>.strings))
        return static_cast<void*>(this);
    return ItemModelBase::qt_metacast(_clname);
}

int KPlato::ResourceAppointmentsRowModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato30ResourceAppointmentsGanttModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ResourceAppointmentsGanttModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato30ResourceAppointmentsGanttModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ResourceAppointmentsGanttModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ResourceAppointmentsGanttModel, qt_meta_tag_ZN6KPlato30ResourceAppointmentsGanttModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ResourceAppointmentsGanttModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ResourceAppointmentsRowModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato30ResourceAppointmentsGanttModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato30ResourceAppointmentsGanttModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato30ResourceAppointmentsGanttModelE_t>.metaTypes,
    nullptr
} };

void KPlato::ResourceAppointmentsGanttModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResourceAppointmentsGanttModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::ResourceAppointmentsGanttModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ResourceAppointmentsGanttModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato30ResourceAppointmentsGanttModelE_t>.strings))
        return static_cast<void*>(this);
    return ResourceAppointmentsRowModel::qt_metacast(_clname);
}

int KPlato::ResourceAppointmentsGanttModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ResourceAppointmentsRowModel::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
