/* This file is part of the KDE project
   SPDX-FileCopyrightText: 2010 Thomas Zander <zander@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

// clazy:excludeall=qstring-arg
#include "KoDockFactoryBase.h"

KoDockFactoryBase::KoDockFactoryBase()
{
}

KoDockFactoryBase::~KoDockFactoryBase()
{
}
