/********************************************************************************
** Form generated from reading UI file 'koDocumentInfoAboutWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KODOCUMENTINFOABOUTWIDGET_H
#define UI_KODOCUMENTINFOABOUTWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KoDocumentInfoAboutWidget
{
public:
    QGridLayout *gridLayout;
    QPushButton *pbEncrypt;
    QHBoxLayout *hboxLayout;
    QLabel *lblEncryptedPic;
    QLabel *lblEncrypted;
    QSpacerItem *spacerItem;
    QLabel *lblKeywordsDesc;
    QLabel *lblCreated;
    QFrame *line_2;
    QLabel *lblRevision;
    QLabel *filePathLabel;
    QLineEdit *leTitle;
    QLineEdit *leSubject;
    QLabel *lblSubjectDesc;
    QLabel *lblLastPrintedDesc;
    QLabel *lblType;
    QLabel *lblTypeDesc;
    QFrame *filePathSeparatorLine;
    QLabel *lblEditingDesc;
    QLabel *lblLastPrinted;
    QLabel *lblEncryptedDesc;
    QLabel *lblTitleDesc;
    QLabel *lblRevisionDesc;
    QLabel *lblModifiedDesc;
    QTextEdit *meComments;
    QLabel *lblModified;
    QLabel *lbCommentsDesc;
    QLineEdit *leKeywords;
    QLabel *lblCreatedDesc;
    QLabel *filePathInfoLabel;
    QPushButton *pbReset;
    QLabel *lblEditing;
    QComboBox *cbLanguage;
    QLabel *lbLanguageDesc;

    void setupUi(QWidget *KoDocumentInfoAboutWidget)
    {
        if (KoDocumentInfoAboutWidget->objectName().isEmpty())
            KoDocumentInfoAboutWidget->setObjectName("KoDocumentInfoAboutWidget");
        KoDocumentInfoAboutWidget->resize(443, 546);
        gridLayout = new QGridLayout(KoDocumentInfoAboutWidget);
        gridLayout->setObjectName("gridLayout");
        pbEncrypt = new QPushButton(KoDocumentInfoAboutWidget);
        pbEncrypt->setObjectName("pbEncrypt");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pbEncrypt->sizePolicy().hasHeightForWidth());
        pbEncrypt->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbEncrypt, 14, 2, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        lblEncryptedPic = new QLabel(KoDocumentInfoAboutWidget);
        lblEncryptedPic->setObjectName("lblEncryptedPic");

        hboxLayout->addWidget(lblEncryptedPic);

        lblEncrypted = new QLabel(KoDocumentInfoAboutWidget);
        lblEncrypted->setObjectName("lblEncrypted");

        hboxLayout->addWidget(lblEncrypted);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);


        gridLayout->addLayout(hboxLayout, 14, 1, 1, 1);

        lblKeywordsDesc = new QLabel(KoDocumentInfoAboutWidget);
        lblKeywordsDesc->setObjectName("lblKeywordsDesc");
        lblKeywordsDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblKeywordsDesc, 4, 0, 1, 1);

        lblCreated = new QLabel(KoDocumentInfoAboutWidget);
        lblCreated->setObjectName("lblCreated");

        gridLayout->addWidget(lblCreated, 9, 1, 1, 1);

        line_2 = new QFrame(KoDocumentInfoAboutWidget);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line_2, 7, 0, 1, 3);

        lblRevision = new QLabel(KoDocumentInfoAboutWidget);
        lblRevision->setObjectName("lblRevision");

        gridLayout->addWidget(lblRevision, 13, 1, 1, 1);

        filePathLabel = new QLabel(KoDocumentInfoAboutWidget);
        filePathLabel->setObjectName("filePathLabel");

        gridLayout->addWidget(filePathLabel, 0, 1, 1, 2);

        leTitle = new QLineEdit(KoDocumentInfoAboutWidget);
        leTitle->setObjectName("leTitle");

        gridLayout->addWidget(leTitle, 2, 1, 1, 2);

        leSubject = new QLineEdit(KoDocumentInfoAboutWidget);
        leSubject->setObjectName("leSubject");

        gridLayout->addWidget(leSubject, 3, 1, 1, 2);

        lblSubjectDesc = new QLabel(KoDocumentInfoAboutWidget);
        lblSubjectDesc->setObjectName("lblSubjectDesc");
        lblSubjectDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblSubjectDesc, 3, 0, 1, 1);

        lblLastPrintedDesc = new QLabel(KoDocumentInfoAboutWidget);
        lblLastPrintedDesc->setObjectName("lblLastPrintedDesc");
        lblLastPrintedDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblLastPrintedDesc, 11, 0, 1, 1);

        lblType = new QLabel(KoDocumentInfoAboutWidget);
        lblType->setObjectName("lblType");

        gridLayout->addWidget(lblType, 8, 1, 1, 1);

        lblTypeDesc = new QLabel(KoDocumentInfoAboutWidget);
        lblTypeDesc->setObjectName("lblTypeDesc");
        lblTypeDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblTypeDesc, 8, 0, 1, 1);

        filePathSeparatorLine = new QFrame(KoDocumentInfoAboutWidget);
        filePathSeparatorLine->setObjectName("filePathSeparatorLine");
        filePathSeparatorLine->setFrameShape(QFrame::Shape::HLine);
        filePathSeparatorLine->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(filePathSeparatorLine, 1, 0, 1, 3);

        lblEditingDesc = new QLabel(KoDocumentInfoAboutWidget);
        lblEditingDesc->setObjectName("lblEditingDesc");
        lblEditingDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblEditingDesc, 12, 0, 1, 1);

        lblLastPrinted = new QLabel(KoDocumentInfoAboutWidget);
        lblLastPrinted->setObjectName("lblLastPrinted");

        gridLayout->addWidget(lblLastPrinted, 11, 1, 1, 1);

        lblEncryptedDesc = new QLabel(KoDocumentInfoAboutWidget);
        lblEncryptedDesc->setObjectName("lblEncryptedDesc");
        lblEncryptedDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblEncryptedDesc, 14, 0, 1, 1);

        lblTitleDesc = new QLabel(KoDocumentInfoAboutWidget);
        lblTitleDesc->setObjectName("lblTitleDesc");
        lblTitleDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblTitleDesc, 2, 0, 1, 1);

        lblRevisionDesc = new QLabel(KoDocumentInfoAboutWidget);
        lblRevisionDesc->setObjectName("lblRevisionDesc");
        lblRevisionDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblRevisionDesc, 13, 0, 1, 1);

        lblModifiedDesc = new QLabel(KoDocumentInfoAboutWidget);
        lblModifiedDesc->setObjectName("lblModifiedDesc");
        lblModifiedDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblModifiedDesc, 10, 0, 1, 1);

        meComments = new QTextEdit(KoDocumentInfoAboutWidget);
        meComments->setObjectName("meComments");

        gridLayout->addWidget(meComments, 6, 1, 1, 2);

        lblModified = new QLabel(KoDocumentInfoAboutWidget);
        lblModified->setObjectName("lblModified");

        gridLayout->addWidget(lblModified, 10, 1, 1, 1);

        lbCommentsDesc = new QLabel(KoDocumentInfoAboutWidget);
        lbCommentsDesc->setObjectName("lbCommentsDesc");
        lbCommentsDesc->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);

        gridLayout->addWidget(lbCommentsDesc, 6, 0, 1, 1);

        leKeywords = new QLineEdit(KoDocumentInfoAboutWidget);
        leKeywords->setObjectName("leKeywords");

        gridLayout->addWidget(leKeywords, 4, 1, 1, 2);

        lblCreatedDesc = new QLabel(KoDocumentInfoAboutWidget);
        lblCreatedDesc->setObjectName("lblCreatedDesc");
        lblCreatedDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblCreatedDesc, 9, 0, 1, 1);

        filePathInfoLabel = new QLabel(KoDocumentInfoAboutWidget);
        filePathInfoLabel->setObjectName("filePathInfoLabel");
        filePathInfoLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(filePathInfoLabel, 0, 0, 1, 1);

        pbReset = new QPushButton(KoDocumentInfoAboutWidget);
        pbReset->setObjectName("pbReset");
        sizePolicy.setHeightForWidth(pbReset->sizePolicy().hasHeightForWidth());
        pbReset->setSizePolicy(sizePolicy);
        pbReset->setLayoutDirection(Qt::RightToLeft);

        gridLayout->addWidget(pbReset, 12, 2, 1, 1);

        lblEditing = new QLabel(KoDocumentInfoAboutWidget);
        lblEditing->setObjectName("lblEditing");

        gridLayout->addWidget(lblEditing, 12, 1, 1, 1);

        cbLanguage = new QComboBox(KoDocumentInfoAboutWidget);
        cbLanguage->setObjectName("cbLanguage");

        gridLayout->addWidget(cbLanguage, 5, 1, 1, 2);

        lbLanguageDesc = new QLabel(KoDocumentInfoAboutWidget);
        lbLanguageDesc->setObjectName("lbLanguageDesc");
        lbLanguageDesc->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lbLanguageDesc, 5, 0, 1, 1);

#if QT_CONFIG(shortcut)
        lblKeywordsDesc->setBuddy(leKeywords);
        lblSubjectDesc->setBuddy(leSubject);
        lblEditingDesc->setBuddy(pbReset);
        lblTitleDesc->setBuddy(leTitle);
        lbCommentsDesc->setBuddy(meComments);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KoDocumentInfoAboutWidget);

        QMetaObject::connectSlotsByName(KoDocumentInfoAboutWidget);
    } // setupUi

    void retranslateUi(QWidget *KoDocumentInfoAboutWidget)
    {
        pbEncrypt->setText(tr2i18n("&Encrypt", nullptr));
        lblEncryptedPic->setText(QString());
        lblEncrypted->setText(QString());
        lblKeywordsDesc->setText(tr2i18n("Keywords:", nullptr));
        lblCreated->setText(QString());
        lblRevision->setText(QString());
        filePathLabel->setText(QString());
        lblSubjectDesc->setText(tr2i18n("Subject:", nullptr));
        lblLastPrintedDesc->setText(tr2i18n("Last printed:", nullptr));
        lblType->setText(QString());
        lblTypeDesc->setText(tr2i18n("Type:", nullptr));
        lblEditingDesc->setText(tr2i18n("Total editing time:", nullptr));
        lblLastPrinted->setText(QString());
        lblEncryptedDesc->setText(tr2i18n("Encryption:", nullptr));
        lblTitleDesc->setText(tr2i18n("Title:", "Document title"));
        lblRevisionDesc->setText(tr2i18n("Revision number:", nullptr));
        lblModifiedDesc->setText(tr2i18n("Modified:", nullptr));
        lblModified->setText(QString());
        lbCommentsDesc->setText(tr2i18n("Comments:", nullptr));
        leKeywords->setText(QString());
        lblCreatedDesc->setText(tr2i18n("Created:", nullptr));
        filePathInfoLabel->setText(tr2i18n("Path:", nullptr));
        pbReset->setText(tr2i18n("&Reset", nullptr));
        lblEditing->setText(QString());
        lbLanguageDesc->setText(tr2i18n("Language:", nullptr));
        (void)KoDocumentInfoAboutWidget;
    } // retranslateUi

};

namespace Ui {
    class KoDocumentInfoAboutWidget: public Ui_KoDocumentInfoAboutWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KODOCUMENTINFOABOUTWIDGET_H

