/****************************************************************************
** Meta object code from reading C++ file 'kptmaindocument.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../calligraplan-4.0.1/src/plan/kptmaindocument.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptmaindocument.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato12MainDocumentE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::MainDocument::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato12MainDocumentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::MainDocument",
        "changed",
        "",
        "workPackageLoaded",
        "viewlistModified",
        "viewListItemAdded",
        "const KPlato::ViewListItem*",
        "item",
        "parent",
        "index",
        "viewListItemRemoved",
        "insertSharedProject",
        "setModified",
        "mod",
        "insertViewListItem",
        "KPlato::View*",
        "view",
        "removeViewListItem",
        "slotViewlistModified",
        "checkForWorkPackages",
        "keep",
        "terminateWorkPackage",
        "const KPlato::Package*",
        "package",
        "setLoadingTemplate",
        "setLoadingSharedResourcesTemplate",
        "setSavingTemplate",
        "setSkipSharedResourcesAndProjects",
        "skip",
        "insertResourcesFile",
        "QUrl",
        "url",
        "slotProjectCreated",
        "insertSharedProjects",
        "QList<QUrl>",
        "urls",
        "slotInsertSharedProject",
        "insertSharedProjectCompleted",
        "insertSharedProjectCancelled",
        "clearResourceAssignments",
        "loadResourceAssignments",
        "setIsTaskModule",
        "value",
        "autoCheckForWorkPackages",
        "slotViewDestroyed",
        "addSchedulerPlugin",
        "KPlato::SchedulerPlugin*",
        "plugin",
        "checkForWorkPackage",
        "insertFileCompleted",
        "insertFileCancelled",
        "slotInsertResourceFile",
        "insertResourcesFileCompleted",
        "insertResourcesFileCancelled",
        "slotNodeChanged",
        "KPlato::Node*",
        "slotScheduleManagerChanged",
        "KPlato::ScheduleManager*",
        "sm",
        "property",
        "setCalculationNeeded",
        "slotCalculationFinished",
        "KPlato::Project*",
        "project",
        "slotStartCalculation",
        "setTaskModulesWatch",
        "taskModuleDirChanged",
        "insertSharedResourceAssignments",
        "const KPlato::Project*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'workPackageLoaded'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewlistModified'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'viewListItemAdded'
        QtMocHelpers::SignalData<void(const KPlato::ViewListItem *, const KPlato::ViewListItem *, int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 6, 8 }, { QMetaType::Int, 9 },
        }}),
        // Signal 'viewListItemRemoved'
        QtMocHelpers::SignalData<void(const KPlato::ViewListItem *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'insertSharedProject'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setModified'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'insertViewListItem'
        QtMocHelpers::SlotData<void(KPlato::View *, const KPlato::ViewListItem *, const KPlato::ViewListItem *, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 6, 7 }, { 0x80000000 | 6, 8 }, { QMetaType::Int, 9 },
        }}),
        // Slot 'removeViewListItem'
        QtMocHelpers::SlotData<void(KPlato::View *, const KPlato::ViewListItem *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotViewlistModified'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'checkForWorkPackages'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Slot 'terminateWorkPackage'
        QtMocHelpers::SlotData<void(const KPlato::Package *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'setLoadingTemplate'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setLoadingSharedResourcesTemplate'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setSavingTemplate'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setSkipSharedResourcesAndProjects'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Slot 'insertResourcesFile'
        QtMocHelpers::SlotData<void(const QUrl &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'slotProjectCreated'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertSharedProjects'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Slot 'insertSharedProjects'
        QtMocHelpers::SlotData<void(const QUrl &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'slotInsertSharedProject'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertSharedProjectCompleted'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertSharedProjectCancelled'
        QtMocHelpers::SlotData<void(const QString &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'clearResourceAssignments'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadResourceAssignments'
        QtMocHelpers::SlotData<void(QUrl)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'setIsTaskModule'
        QtMocHelpers::SlotData<void(bool)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 42 },
        }}),
        // Slot 'autoCheckForWorkPackages'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotViewDestroyed'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addSchedulerPlugin'
        QtMocHelpers::SlotData<void(const QString &, KPlato::SchedulerPlugin *)>(45, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { 0x80000000 | 46, 47 },
        }}),
        // Slot 'checkForWorkPackage'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'insertFileCompleted'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'insertFileCancelled'
        QtMocHelpers::SlotData<void(const QString &)>(50, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotInsertResourceFile'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'insertResourcesFileCompleted'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'insertResourcesFileCancelled'
        QtMocHelpers::SlotData<void(const QString &)>(53, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotNodeChanged'
        QtMocHelpers::SlotData<void(KPlato::Node *, int)>(54, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 55, 2 }, { QMetaType::Int, 2 },
        }}),
        // Slot 'slotScheduleManagerChanged'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *, int)>(56, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 57, 58 }, { QMetaType::Int, 59 },
        }}),
        // Slot 'setCalculationNeeded'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCalculationFinished'
        QtMocHelpers::SlotData<void(KPlato::Project *, KPlato::ScheduleManager *)>(61, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 62, 63 }, { 0x80000000 | 57, 58 },
        }}),
        // Slot 'slotStartCalculation'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'setTaskModulesWatch'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'taskModuleDirChanged'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessProtected, QMetaType::Void),
        // Method 'insertSharedResourceAssignments'
        QtMocHelpers::MethodData<void(const KPlato::Project *)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 68, 63 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainDocument, qt_meta_tag_ZN6KPlato12MainDocumentE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::MainDocument::staticMetaObject = { {
    QMetaObject::SuperData::link<KoDocument::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12MainDocumentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12MainDocumentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato12MainDocumentE_t>.metaTypes,
    nullptr
} };

void KPlato::MainDocument::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainDocument *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: _t->workPackageLoaded(); break;
        case 2: _t->viewlistModified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->viewListItemAdded((*reinterpret_cast<std::add_pointer_t<const KPlato::ViewListItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<const KPlato::ViewListItem*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 4: _t->viewListItemRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::ViewListItem*>>(_a[1]))); break;
        case 5: _t->insertSharedProject(); break;
        case 6: _t->setModified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->insertViewListItem((*reinterpret_cast<std::add_pointer_t<KPlato::View*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<const KPlato::ViewListItem*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<const KPlato::ViewListItem*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 8: _t->removeViewListItem((*reinterpret_cast<std::add_pointer_t<KPlato::View*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<const KPlato::ViewListItem*>>(_a[2]))); break;
        case 9: _t->slotViewlistModified(); break;
        case 10: _t->checkForWorkPackages((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->terminateWorkPackage((*reinterpret_cast<std::add_pointer_t<const KPlato::Package*>>(_a[1]))); break;
        case 12: _t->setLoadingTemplate((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->setLoadingSharedResourcesTemplate((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->setSavingTemplate((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->setSkipSharedResourcesAndProjects((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->insertResourcesFile((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 17: _t->slotProjectCreated(); break;
        case 18: _t->insertSharedProjects((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 19: _t->insertSharedProjects((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 20: _t->slotInsertSharedProject(); break;
        case 21: _t->insertSharedProjectCompleted(); break;
        case 22: _t->insertSharedProjectCancelled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->clearResourceAssignments(); break;
        case 24: _t->loadResourceAssignments((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 25: _t->setIsTaskModule((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->autoCheckForWorkPackages(); break;
        case 27: _t->slotViewDestroyed(); break;
        case 28: _t->addSchedulerPlugin((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::SchedulerPlugin*>>(_a[2]))); break;
        case 29: _t->checkForWorkPackage(); break;
        case 30: _t->insertFileCompleted(); break;
        case 31: _t->insertFileCancelled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 32: _t->slotInsertResourceFile(); break;
        case 33: _t->insertResourcesFileCompleted(); break;
        case 34: _t->insertResourcesFileCancelled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 35: _t->slotNodeChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 36: _t->slotScheduleManagerChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 37: _t->setCalculationNeeded(); break;
        case 38: _t->slotCalculationFinished((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[2]))); break;
        case 39: _t->slotStartCalculation(); break;
        case 40: _t->setTaskModulesWatch(); break;
        case 41: _t->taskModuleDirChanged(); break;
        case 42: _t->insertSharedResourceAssignments((*reinterpret_cast<std::add_pointer_t<const KPlato::Project*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 35:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        case 36:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Project* >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)()>(_a, &MainDocument::changed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)()>(_a, &MainDocument::workPackageLoaded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)(bool )>(_a, &MainDocument::viewlistModified, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)(const KPlato::ViewListItem * , const KPlato::ViewListItem * , int )>(_a, &MainDocument::viewListItemAdded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)(const KPlato::ViewListItem * )>(_a, &MainDocument::viewListItemRemoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)()>(_a, &MainDocument::insertSharedProject, 5))
            return;
    }
}

const QMetaObject *KPlato::MainDocument::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::MainDocument::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12MainDocumentE_t>.strings))
        return static_cast<void*>(this);
    return KoDocument::qt_metacast(_clname);
}

int KPlato::MainDocument::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KoDocument::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    }
    return _id;
}

// SIGNAL 0
void KPlato::MainDocument::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KPlato::MainDocument::workPackageLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KPlato::MainDocument::viewlistModified(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlato::MainDocument::viewListItemAdded(const KPlato::ViewListItem * _t1, const KPlato::ViewListItem * _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void KPlato::MainDocument::viewListItemRemoved(const KPlato::ViewListItem * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KPlato::MainDocument::insertSharedProject()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
