/********************************************************************************
** Form generated from reading UI file 'localemon.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOCALEMON_H
#define UI_LOCALEMON_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_LocaleConfigMoney
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *m_labMonCurSym;
    QLineEdit *m_edMonCurSym;
    QLabel *m_labMonFraDig;
    QSpinBox *m_inMonFraDig;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPlato__LocaleConfigMoney)
    {
        if (KPlato__LocaleConfigMoney->objectName().isEmpty())
            KPlato__LocaleConfigMoney->setObjectName("KPlato__LocaleConfigMoney");
        KPlato__LocaleConfigMoney->resize(567, 208);
        verticalLayout = new QVBoxLayout(KPlato__LocaleConfigMoney);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        m_labMonCurSym = new QLabel(KPlato__LocaleConfigMoney);
        m_labMonCurSym->setObjectName("m_labMonCurSym");
        m_labMonCurSym->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(m_labMonCurSym, 0, 0, 1, 1);

        m_edMonCurSym = new QLineEdit(KPlato__LocaleConfigMoney);
        m_edMonCurSym->setObjectName("m_edMonCurSym");

        gridLayout->addWidget(m_edMonCurSym, 0, 1, 1, 1);

        m_labMonFraDig = new QLabel(KPlato__LocaleConfigMoney);
        m_labMonFraDig->setObjectName("m_labMonFraDig");
        m_labMonFraDig->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(m_labMonFraDig, 1, 0, 1, 1);

        m_inMonFraDig = new QSpinBox(KPlato__LocaleConfigMoney);
        m_inMonFraDig->setObjectName("m_inMonFraDig");

        gridLayout->addWidget(m_inMonFraDig, 1, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(KPlato__LocaleConfigMoney);

        QMetaObject::connectSlotsByName(KPlato__LocaleConfigMoney);
    } // setupUi

    void retranslateUi(QWidget *KPlato__LocaleConfigMoney)
    {
        m_labMonCurSym->setText(tr2i18n("Currency symbol:", nullptr));
        m_labMonFraDig->setText(tr2i18n("Fractional digits:", nullptr));
        (void)KPlato__LocaleConfigMoney;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class LocaleConfigMoney: public Ui_LocaleConfigMoney {};
} // namespace Ui
} // namespace KPlato

#endif // LOCALEMON_H

