/****************************************************************************
** Meta object code from reading C++ file 'ResourceGroupItemModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/models/ResourceGroupItemModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ResourceGroupItemModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato22ResourceGroupItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ResourceGroupItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato22ResourceGroupItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ResourceGroupItemModel",
        "resourceAdded",
        "",
        "KPlato::Resource*",
        "resource",
        "resourceRemoved",
        "slotResourceGroupChanged",
        "KPlato::ResourceGroup*",
        "group",
        "slotResourceGroupToBeAdded",
        "KPlato::Project*",
        "project",
        "parent",
        "row",
        "slotResourceGroupAdded",
        "slotResourceGroupToBeRemoved",
        "slotResourceGroupRemoved",
        "slotResourceChanged",
        "slotResourceToBeAdded",
        "slotResourceAdded",
        "slotResourceToBeRemoved",
        "slotResourceRemoved",
        "slotDataArrived",
        "KIO::Job*",
        "job",
        "data",
        "slotJobFinished",
        "KJob*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'resourceAdded'
        QtMocHelpers::SignalData<void(KPlato::Resource *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'resourceRemoved'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotResourceGroupChanged'
        QtMocHelpers::SlotData<void(KPlato::ResourceGroup *)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotResourceGroupToBeAdded'
        QtMocHelpers::SlotData<void(KPlato::Project *, KPlato::ResourceGroup *, int)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 7, 12 }, { QMetaType::Int, 13 },
        }}),
        // Slot 'slotResourceGroupAdded'
        QtMocHelpers::SlotData<void(KPlato::ResourceGroup *)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotResourceGroupToBeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Project *, KPlato::ResourceGroup *, int, KPlato::ResourceGroup *)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 7, 12 }, { QMetaType::Int, 13 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotResourceGroupRemoved'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotResourceChanged'
        QtMocHelpers::SlotData<void(KPlato::Resource *)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotResourceToBeAdded'
        QtMocHelpers::SlotData<void(KPlato::ResourceGroup *, int)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { QMetaType::Int, 13 },
        }}),
        // Slot 'slotResourceAdded'
        QtMocHelpers::SlotData<void(KPlato::Resource *)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotResourceToBeRemoved'
        QtMocHelpers::SlotData<void(KPlato::ResourceGroup *, int, KPlato::Resource *)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { QMetaType::Int, 13 }, { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotResourceRemoved'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDataArrived'
        QtMocHelpers::SlotData<void(KIO::Job *, const QByteArray &)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 23, 24 }, { QMetaType::QByteArray, 25 },
        }}),
        // Slot 'slotJobFinished'
        QtMocHelpers::SlotData<void(KJob *)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 27, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ResourceGroupItemModel, qt_meta_tag_ZN6KPlato22ResourceGroupItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ResourceGroupItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22ResourceGroupItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22ResourceGroupItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato22ResourceGroupItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::ResourceGroupItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResourceGroupItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->resourceAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 1: _t->resourceRemoved(); break;
        case 2: _t->slotResourceGroupChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[1]))); break;
        case 3: _t->slotResourceGroupToBeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 4: _t->slotResourceGroupAdded((*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[1]))); break;
        case 5: _t->slotResourceGroupToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[4]))); break;
        case 6: _t->slotResourceGroupRemoved(); break;
        case 7: _t->slotResourceChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 8: _t->slotResourceToBeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 9: _t->slotResourceAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 10: _t->slotResourceToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[3]))); break;
        case 11: _t->slotResourceRemoved(); break;
        case 12: _t->slotDataArrived((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 13: _t->slotJobFinished((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ResourceGroupItemModel::*)(KPlato::Resource * )>(_a, &ResourceGroupItemModel::resourceAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResourceGroupItemModel::*)()>(_a, &ResourceGroupItemModel::resourceRemoved, 1))
            return;
    }
}

const QMetaObject *KPlato::ResourceGroupItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ResourceGroupItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22ResourceGroupItemModelE_t>.strings))
        return static_cast<void*>(this);
    return ItemModelBase::qt_metacast(_clname);
}

int KPlato::ResourceGroupItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ResourceGroupItemModel::resourceAdded(KPlato::Resource * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::ResourceGroupItemModel::resourceRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN6KPlato20ParentGroupItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ParentGroupItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato20ParentGroupItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ParentGroupItemModel",
        "executeCommand",
        "",
        "KUndo2Command*",
        "cmd",
        "setProject",
        "KPlato::Project*",
        "project",
        "setResource",
        "KPlato::Resource*",
        "resource",
        "slotResourceAdded",
        "KPlato::ResourceGroup*",
        "group",
        "slotResourceRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'executeCommand'
        QtMocHelpers::SignalData<void(KUndo2Command *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setProject'
        QtMocHelpers::SlotData<void(KPlato::Project *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setResource'
        QtMocHelpers::SlotData<void(KPlato::Resource *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotResourceAdded'
        QtMocHelpers::SlotData<void(KPlato::ResourceGroup *)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'slotResourceRemoved'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ParentGroupItemModel, qt_meta_tag_ZN6KPlato20ParentGroupItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ParentGroupItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QSortFilterProxyModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20ParentGroupItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20ParentGroupItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato20ParentGroupItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::ParentGroupItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ParentGroupItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->executeCommand((*reinterpret_cast<std::add_pointer_t<KUndo2Command*>>(_a[1]))); break;
        case 1: _t->setProject((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1]))); break;
        case 2: _t->setResource((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 3: _t->slotResourceAdded((*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[1]))); break;
        case 4: _t->slotResourceRemoved(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ParentGroupItemModel::*)(KUndo2Command * )>(_a, &ParentGroupItemModel::executeCommand, 0))
            return;
    }
}

const QMetaObject *KPlato::ParentGroupItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ParentGroupItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20ParentGroupItemModelE_t>.strings))
        return static_cast<void*>(this);
    return QSortFilterProxyModel::qt_metacast(_clname);
}

int KPlato::ParentGroupItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSortFilterProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ParentGroupItemModel::executeCommand(KUndo2Command * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
