/****************************************************************************
** Meta object code from reading C++ file 'kptnodeitemmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/models/kptnodeitemmodel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptnodeitemmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato9NodeModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::NodeModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato9NodeModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::NodeModel",
        "Properties",
        "NodeName",
        "NodeType",
        "NodePriority",
        "NodeResponsible",
        "NodeAllocation",
        "NodeEstimateType",
        "NodeEstimateCalendar",
        "NodeEstimate",
        "NodeOptimisticRatio",
        "NodePessimisticRatio",
        "NodeRisk",
        "NodeConstraint",
        "NodeConstraintStart",
        "NodeConstraintEnd",
        "NodeRunningAccount",
        "NodeStartupAccount",
        "NodeStartupCost",
        "NodeShutdownAccount",
        "NodeShutdownCost",
        "NodeDescription",
        "NodeExpected",
        "NodeVarianceEstimate",
        "NodeOptimistic",
        "NodePessimistic",
        "NodeStartTime",
        "NodeEndTime",
        "NodeEarlyStart",
        "NodeEarlyFinish",
        "NodeLateStart",
        "NodeLateFinish",
        "NodePositiveFloat",
        "NodeFreeFloat",
        "NodeNegativeFloat",
        "NodeStartFloat",
        "NodeFinishFloat",
        "NodeAssignments",
        "NodeDuration",
        "NodeVarianceDuration",
        "NodeOptimisticDuration",
        "NodePessimisticDuration",
        "NodeStatus",
        "NodeCompleted",
        "NodePlannedEffort",
        "NodeActualEffort",
        "NodeRemainingEffort",
        "NodePlannedCost",
        "NodeActualCost",
        "NodeActualStart",
        "NodeStarted",
        "NodeActualFinish",
        "NodeFinished",
        "NodeStatusNote",
        "NodeSchedulingStatus",
        "NodeNotScheduled",
        "NodeAssignmentMissing",
        "NodeResourceOverbooked",
        "NodeResourceUnavailable",
        "NodeConstraintsError",
        "NodeEffortNotMet",
        "NodeSchedulingError",
        "NodeWBSCode",
        "NodeLevel",
        "NodeBCWS",
        "NodeBCWP",
        "NodeACWP",
        "NodePerformanceIndex",
        "NodeCritical",
        "NodeCriticalPath",
        "WPOwnerName",
        "WPTransmitionStatus",
        "WPTransmitionTime"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Properties'
        QtMocHelpers::EnumData<enum Properties>(1, 1, QMC::EnumFlags{}).add({
            {    2, Properties::NodeName },
            {    3, Properties::NodeType },
            {    4, Properties::NodePriority },
            {    5, Properties::NodeResponsible },
            {    6, Properties::NodeAllocation },
            {    7, Properties::NodeEstimateType },
            {    8, Properties::NodeEstimateCalendar },
            {    9, Properties::NodeEstimate },
            {   10, Properties::NodeOptimisticRatio },
            {   11, Properties::NodePessimisticRatio },
            {   12, Properties::NodeRisk },
            {   13, Properties::NodeConstraint },
            {   14, Properties::NodeConstraintStart },
            {   15, Properties::NodeConstraintEnd },
            {   16, Properties::NodeRunningAccount },
            {   17, Properties::NodeStartupAccount },
            {   18, Properties::NodeStartupCost },
            {   19, Properties::NodeShutdownAccount },
            {   20, Properties::NodeShutdownCost },
            {   21, Properties::NodeDescription },
            {   22, Properties::NodeExpected },
            {   23, Properties::NodeVarianceEstimate },
            {   24, Properties::NodeOptimistic },
            {   25, Properties::NodePessimistic },
            {   26, Properties::NodeStartTime },
            {   27, Properties::NodeEndTime },
            {   28, Properties::NodeEarlyStart },
            {   29, Properties::NodeEarlyFinish },
            {   30, Properties::NodeLateStart },
            {   31, Properties::NodeLateFinish },
            {   32, Properties::NodePositiveFloat },
            {   33, Properties::NodeFreeFloat },
            {   34, Properties::NodeNegativeFloat },
            {   35, Properties::NodeStartFloat },
            {   36, Properties::NodeFinishFloat },
            {   37, Properties::NodeAssignments },
            {   38, Properties::NodeDuration },
            {   39, Properties::NodeVarianceDuration },
            {   40, Properties::NodeOptimisticDuration },
            {   41, Properties::NodePessimisticDuration },
            {   42, Properties::NodeStatus },
            {   43, Properties::NodeCompleted },
            {   44, Properties::NodePlannedEffort },
            {   45, Properties::NodeActualEffort },
            {   46, Properties::NodeRemainingEffort },
            {   47, Properties::NodePlannedCost },
            {   48, Properties::NodeActualCost },
            {   49, Properties::NodeActualStart },
            {   50, Properties::NodeStarted },
            {   51, Properties::NodeActualFinish },
            {   52, Properties::NodeFinished },
            {   53, Properties::NodeStatusNote },
            {   54, Properties::NodeSchedulingStatus },
            {   55, Properties::NodeNotScheduled },
            {   56, Properties::NodeAssignmentMissing },
            {   57, Properties::NodeResourceOverbooked },
            {   58, Properties::NodeResourceUnavailable },
            {   59, Properties::NodeConstraintsError },
            {   60, Properties::NodeEffortNotMet },
            {   61, Properties::NodeSchedulingError },
            {   62, Properties::NodeWBSCode },
            {   63, Properties::NodeLevel },
            {   64, Properties::NodeBCWS },
            {   65, Properties::NodeBCWP },
            {   66, Properties::NodeACWP },
            {   67, Properties::NodePerformanceIndex },
            {   68, Properties::NodeCritical },
            {   69, Properties::NodeCriticalPath },
            {   70, Properties::WPOwnerName },
            {   71, Properties::WPTransmitionStatus },
            {   72, Properties::WPTransmitionTime },
        }),
    };
    return QtMocHelpers::metaObjectData<NodeModel, qt_meta_tag_ZN6KPlato9NodeModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::NodeModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato9NodeModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato9NodeModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato9NodeModelE_t>.metaTypes,
    nullptr
} };

void KPlato::NodeModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NodeModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::NodeModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::NodeModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato9NodeModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KPlato::NodeModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato13NodeItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::NodeItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato13NodeItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::NodeItemModel",
        "nodeInserted",
        "",
        "KPlato::Node*",
        "node",
        "projectShownChanged",
        "setProject",
        "KPlato::Project*",
        "project",
        "setScheduleManager",
        "KPlato::ScheduleManager*",
        "sm",
        "setShowProject",
        "on",
        "slotWbsDefinitionChanged",
        "slotNodeChanged",
        "slotNodeToBeInserted",
        "row",
        "slotNodeInserted",
        "slotNodeToBeRemoved",
        "slotNodeRemoved",
        "slotNodeToBeMoved",
        "pos",
        "newParent",
        "newPos",
        "slotNodeMoved",
        "slotLayoutChanged",
        "slotProjectCalculated"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'nodeInserted'
        QtMocHelpers::SignalData<void(KPlato::Node *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'projectShownChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setProject'
        QtMocHelpers::SlotData<void(KPlato::Project *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'setScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setShowProject'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'slotWbsDefinitionChanged'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotNodeChanged'
        QtMocHelpers::SlotData<void(KPlato::Node *, int)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::Int, 2 },
        }}),
        // Slot 'slotNodeToBeInserted'
        QtMocHelpers::SlotData<void(KPlato::Node *, int)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'slotNodeInserted'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotNodeToBeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotNodeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotNodeToBeMoved'
        QtMocHelpers::SlotData<void(KPlato::Node *, int, KPlato::Node *, int)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 22 }, { 0x80000000 | 3, 23 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'slotNodeMoved'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(25, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotLayoutChanged'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotProjectCalculated'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NodeItemModel, qt_meta_tag_ZN6KPlato13NodeItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::NodeItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato13NodeItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato13NodeItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato13NodeItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::NodeItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NodeItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->nodeInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 1: _t->projectShownChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->setProject((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1]))); break;
        case 3: _t->setScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 4: _t->setShowProject((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->slotWbsDefinitionChanged(); break;
        case 6: _t->slotNodeChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 7: _t->slotNodeToBeInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->slotNodeInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 9: _t->slotNodeToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 10: _t->slotNodeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 11: _t->slotNodeToBeMoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 12: _t->slotNodeMoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 13: _t->slotLayoutChanged(); break;
        case 14: _t->slotProjectCalculated((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (NodeItemModel::*)(KPlato::Node * )>(_a, &NodeItemModel::nodeInserted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeItemModel::*)(bool )>(_a, &NodeItemModel::projectShownChanged, 1))
            return;
    }
}

const QMetaObject *KPlato::NodeItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::NodeItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato13NodeItemModelE_t>.strings))
        return static_cast<void*>(this);
    return ItemModelBase::qt_metacast(_clname);
}

int KPlato::NodeItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void KPlato::NodeItemModel::nodeInserted(KPlato::Node * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::NodeItemModel::projectShownChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN6KPlato14GanttItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::GanttItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato14GanttItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::GanttItemModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GanttItemModel, qt_meta_tag_ZN6KPlato14GanttItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::GanttItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<NodeItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14GanttItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14GanttItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato14GanttItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::GanttItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GanttItemModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::GanttItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::GanttItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14GanttItemModelE_t>.strings))
        return static_cast<void*>(this);
    return NodeItemModel::qt_metacast(_clname);
}

int KPlato::GanttItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = NodeItemModel::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato18MilestoneItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::MilestoneItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato18MilestoneItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::MilestoneItemModel",
        "setProject",
        "",
        "KPlato::Project*",
        "project",
        "setScheduleManager",
        "KPlato::ScheduleManager*",
        "sm",
        "slotNodeChanged",
        "KPlato::Node*",
        "slotNodeToBeInserted",
        "node",
        "row",
        "slotNodeInserted",
        "slotNodeToBeRemoved",
        "slotNodeRemoved",
        "slotNodeToBeMoved",
        "pos",
        "newParent",
        "newPos",
        "slotNodeMoved",
        "slotLayoutChanged",
        "slotWbsDefinitionChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setProject'
        QtMocHelpers::SlotData<void(KPlato::Project *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotNodeChanged'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Slot 'slotNodeToBeInserted'
        QtMocHelpers::SlotData<void(KPlato::Node *, int)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'slotNodeInserted'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 11 },
        }}),
        // Slot 'slotNodeToBeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 11 },
        }}),
        // Slot 'slotNodeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 11 },
        }}),
        // Slot 'slotNodeToBeMoved'
        QtMocHelpers::SlotData<void(KPlato::Node *, int, KPlato::Node *, int)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 11 }, { QMetaType::Int, 17 }, { 0x80000000 | 9, 18 }, { QMetaType::Int, 19 },
        }}),
        // Slot 'slotNodeMoved'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 11 },
        }}),
        // Slot 'slotLayoutChanged'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotWbsDefinitionChanged'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MilestoneItemModel, qt_meta_tag_ZN6KPlato18MilestoneItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::MilestoneItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18MilestoneItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18MilestoneItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato18MilestoneItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::MilestoneItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MilestoneItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setProject((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1]))); break;
        case 1: _t->setScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 2: _t->slotNodeChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 3: _t->slotNodeToBeInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->slotNodeInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 5: _t->slotNodeToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 6: _t->slotNodeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 7: _t->slotNodeToBeMoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 8: _t->slotNodeMoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 9: _t->slotLayoutChanged(); break;
        case 10: _t->slotWbsDefinitionChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KPlato::MilestoneItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::MilestoneItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18MilestoneItemModelE_t>.strings))
        return static_cast<void*>(this);
    return ItemModelBase::qt_metacast(_clname);
}

int KPlato::MilestoneItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato24NodeSortFilterProxyModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::NodeSortFilterProxyModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato24NodeSortFilterProxyModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::NodeSortFilterProxyModel",
        "setShowSummarytasks",
        "",
        "on",
        "setTasksAndMilestonesGroupEnabled",
        "setShowProject",
        "setShowUnscheduled",
        "setShowTasks",
        "setShowMilestones",
        "setPeriodGroupEnabled",
        "setShowRunning",
        "setShowFinished",
        "setShowNotStarted",
        "setPeriodStart",
        "dt",
        "setPeriodEnd"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setShowSummarytasks'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setTasksAndMilestonesGroupEnabled'
        QtMocHelpers::SlotData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setShowProject'
        QtMocHelpers::SlotData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setShowUnscheduled'
        QtMocHelpers::SlotData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setShowTasks'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setShowMilestones'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setPeriodGroupEnabled'
        QtMocHelpers::SlotData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setShowRunning'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setShowFinished'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setShowNotStarted'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setPeriodStart'
        QtMocHelpers::SlotData<void(const QDate &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDate, 14 },
        }}),
        // Slot 'setPeriodEnd'
        QtMocHelpers::SlotData<void(const QDate &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDate, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NodeSortFilterProxyModel, qt_meta_tag_ZN6KPlato24NodeSortFilterProxyModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::NodeSortFilterProxyModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QSortFilterProxyModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato24NodeSortFilterProxyModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato24NodeSortFilterProxyModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato24NodeSortFilterProxyModelE_t>.metaTypes,
    nullptr
} };

void KPlato::NodeSortFilterProxyModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NodeSortFilterProxyModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setShowSummarytasks((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->setTasksAndMilestonesGroupEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->setShowProject((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->setShowUnscheduled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->setShowTasks((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->setShowMilestones((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->setPeriodGroupEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setShowRunning((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->setShowFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->setShowNotStarted((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->setPeriodStart((*reinterpret_cast<std::add_pointer_t<QDate>>(_a[1]))); break;
        case 11: _t->setPeriodEnd((*reinterpret_cast<std::add_pointer_t<QDate>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::NodeSortFilterProxyModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::NodeSortFilterProxyModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato24NodeSortFilterProxyModelE_t>.strings))
        return static_cast<void*>(this);
    return QSortFilterProxyModel::qt_metacast(_clname);
}

int KPlato::NodeSortFilterProxyModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSortFilterProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato15TaskModuleModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::TaskModuleModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato15TaskModuleModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::TaskModuleModel",
        "executeCommand",
        "",
        "KUndo2Command*",
        "cmd",
        "saveTaskModule",
        "QUrl",
        "url",
        "KPlato::Project*",
        "project",
        "removeTaskModule",
        "setProject",
        "loadTaskModules",
        "files",
        "slotTaskModulesChanged",
        "QList<QUrl>",
        "modules",
        "slotReset"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'executeCommand'
        QtMocHelpers::SignalData<void(KUndo2Command *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'saveTaskModule'
        QtMocHelpers::SignalData<void(const QUrl &, KPlato::Project *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'removeTaskModule'
        QtMocHelpers::SignalData<void(const QUrl &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setProject'
        QtMocHelpers::SlotData<void(KPlato::Project *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'loadTaskModules'
        QtMocHelpers::SlotData<void(const QStringList &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 13 },
        }}),
        // Slot 'slotTaskModulesChanged'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'slotReset'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaskModuleModel, qt_meta_tag_ZN6KPlato15TaskModuleModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::TaskModuleModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15TaskModuleModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15TaskModuleModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato15TaskModuleModelE_t>.metaTypes,
    nullptr
} };

void KPlato::TaskModuleModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskModuleModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->executeCommand((*reinterpret_cast<std::add_pointer_t<KUndo2Command*>>(_a[1]))); break;
        case 1: _t->saveTaskModule((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[2]))); break;
        case 2: _t->removeTaskModule((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 3: _t->setProject((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1]))); break;
        case 4: _t->loadTaskModules((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 5: _t->slotTaskModulesChanged((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 6: _t->slotReset(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TaskModuleModel::*)(KUndo2Command * )>(_a, &TaskModuleModel::executeCommand, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskModuleModel::*)(const QUrl & , KPlato::Project * )>(_a, &TaskModuleModel::saveTaskModule, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskModuleModel::*)(const QUrl & )>(_a, &TaskModuleModel::removeTaskModule, 2))
            return;
    }
}

const QMetaObject *KPlato::TaskModuleModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::TaskModuleModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15TaskModuleModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int KPlato::TaskModuleModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void KPlato::TaskModuleModel::executeCommand(KUndo2Command * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::TaskModuleModel::saveTaskModule(const QUrl & _t1, KPlato::Project * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KPlato::TaskModuleModel::removeTaskModule(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
