/********************************************************************************
** Form generated from reading UI file 'kptinsertfilepanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTINSERTFILEPANEL_H
#define UI_KPTINSERTFILEPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kurlrequester.h"

namespace KPlato {

class Ui_InsertFilePanel
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QLabel *ui_name;
    QRadioButton *ui_useProject;
    QRadioButton *ui_isParent;
    QRadioButton *ui_isAfter;
    QFrame *line;
    KUrlRequester *ui_url;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPlato__InsertFilePanel)
    {
        if (KPlato__InsertFilePanel->objectName().isEmpty())
            KPlato__InsertFilePanel->setObjectName("KPlato__InsertFilePanel");
        KPlato__InsertFilePanel->resize(350, 145);
        KPlato__InsertFilePanel->setMinimumSize(QSize(350, 0));
        verticalLayout_2 = new QVBoxLayout(KPlato__InsertFilePanel);
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox = new QGroupBox(KPlato__InsertFilePanel);
        groupBox->setObjectName("groupBox");
        groupBox->setFlat(true);
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(groupBox);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        ui_name = new QLabel(groupBox);
        ui_name->setObjectName("ui_name");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ui_name->sizePolicy().hasHeightForWidth());
        ui_name->setSizePolicy(sizePolicy);
        ui_name->setFrameShape(QFrame::Panel);
        ui_name->setFrameShadow(QFrame::Sunken);

        horizontalLayout->addWidget(ui_name);


        verticalLayout->addLayout(horizontalLayout);

        ui_useProject = new QRadioButton(groupBox);
        ui_useProject->setObjectName("ui_useProject");

        verticalLayout->addWidget(ui_useProject);

        ui_isParent = new QRadioButton(groupBox);
        ui_isParent->setObjectName("ui_isParent");

        verticalLayout->addWidget(ui_isParent);

        ui_isAfter = new QRadioButton(groupBox);
        ui_isAfter->setObjectName("ui_isAfter");

        verticalLayout->addWidget(ui_isAfter);

        line = new QFrame(groupBox);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        ui_url = new KUrlRequester(groupBox);
        ui_url->setObjectName("ui_url");

        verticalLayout->addWidget(ui_url);

        verticalSpacer = new QSpacerItem(20, 5, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        verticalLayout_2->addWidget(groupBox);


        retranslateUi(KPlato__InsertFilePanel);

        QMetaObject::connectSlotsByName(KPlato__InsertFilePanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__InsertFilePanel)
    {
        groupBox->setTitle(QString());
        label->setText(tr2i18n("Current node:", nullptr));
        ui_name->setText(tr2i18n("TextLabel", "KDE::DoNotExtract"));
        ui_useProject->setText(tr2i18n("Insert on top level", nullptr));
        ui_isParent->setText(tr2i18n("Insert as children to current node", nullptr));
        ui_isAfter->setText(tr2i18n("Insert after current node", nullptr));
        (void)KPlato__InsertFilePanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class InsertFilePanel: public Ui_InsertFilePanel {};
} // namespace Ui
} // namespace KPlato

#endif // KPTINSERTFILEPANEL_H

