/****************************************************************************
** Meta object code from reading C++ file 'taskworkpackagemodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../calligraplan-4.0.1/src/workpackage/taskworkpackagemodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'taskworkpackagemodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KPlatoWork20TaskWorkPackageModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlatoWork::TaskWorkPackageModel::qt_create_metaobjectdata<qt_meta_tag_ZN10KPlatoWork20TaskWorkPackageModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlatoWork::TaskWorkPackageModel",
        "addWorkPackage",
        "",
        "KPlatoWork::WorkPackage*",
        "package",
        "row",
        "removeWorkPackage",
        "slotNodeChanged",
        "KPlato::Node*",
        "slotNodeToBeInserted",
        "node",
        "slotNodeInserted",
        "slotNodeToBeRemoved",
        "slotNodeRemoved",
        "slotDocumentAdded",
        "KPlato::Document*",
        "doc",
        "index",
        "slotDocumentRemoved",
        "slotDocumentChanged",
        "Properties",
        "NodeName",
        "NodeType",
        "NodeResponsible",
        "NodeDescription",
        "NodeStartTime",
        "NodeEndTime",
        "NodeAssignments",
        "NodeCompleted",
        "NodeActualEffort",
        "NodeRemainingEffort",
        "NodePlannedEffort",
        "NodeActualStart",
        "NodeStarted",
        "NodeActualFinish",
        "NodeFinished",
        "NodeStatus",
        "NodeStatusNote",
        "ProjectName",
        "ProjectManager"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'addWorkPackage'
        QtMocHelpers::SlotData<void(KPlatoWork::WorkPackage *, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Slot 'removeWorkPackage'
        QtMocHelpers::SlotData<void(KPlatoWork::WorkPackage *, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Slot 'slotNodeChanged'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Slot 'slotNodeToBeInserted'
        QtMocHelpers::SlotData<void(KPlato::Node *, int)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 10 }, { QMetaType::Int, 5 },
        }}),
        // Slot 'slotNodeInserted'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 10 },
        }}),
        // Slot 'slotNodeToBeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 10 },
        }}),
        // Slot 'slotNodeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 10 },
        }}),
        // Slot 'slotDocumentAdded'
        QtMocHelpers::SlotData<void(KPlato::Node *, KPlato::Document *, int)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 10 }, { 0x80000000 | 15, 16 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'slotDocumentRemoved'
        QtMocHelpers::SlotData<void(KPlato::Node *, KPlato::Document *, int)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 10 }, { 0x80000000 | 15, 16 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'slotDocumentChanged'
        QtMocHelpers::SlotData<void(KPlato::Node *, KPlato::Document *, int)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 10 }, { 0x80000000 | 15, 16 }, { QMetaType::Int, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Properties'
        QtMocHelpers::EnumData<enum Properties>(20, 20, QMC::EnumFlags{}).add({
            {   21, Properties::NodeName },
            {   22, Properties::NodeType },
            {   23, Properties::NodeResponsible },
            {   24, Properties::NodeDescription },
            {   25, Properties::NodeStartTime },
            {   26, Properties::NodeEndTime },
            {   27, Properties::NodeAssignments },
            {   28, Properties::NodeCompleted },
            {   29, Properties::NodeActualEffort },
            {   30, Properties::NodeRemainingEffort },
            {   31, Properties::NodePlannedEffort },
            {   32, Properties::NodeActualStart },
            {   33, Properties::NodeStarted },
            {   34, Properties::NodeActualFinish },
            {   35, Properties::NodeFinished },
            {   36, Properties::NodeStatus },
            {   37, Properties::NodeStatusNote },
            {   38, Properties::ProjectName },
            {   39, Properties::ProjectManager },
        }),
    };
    return QtMocHelpers::metaObjectData<TaskWorkPackageModel, qt_meta_tag_ZN10KPlatoWork20TaskWorkPackageModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlatoWork::TaskWorkPackageModel::staticMetaObject = { {
    QMetaObject::SuperData::link<KPlato::ItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork20TaskWorkPackageModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork20TaskWorkPackageModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KPlatoWork20TaskWorkPackageModelE_t>.metaTypes,
    nullptr
} };

void KPlatoWork::TaskWorkPackageModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskWorkPackageModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addWorkPackage((*reinterpret_cast<std::add_pointer_t<KPlatoWork::WorkPackage*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->removeWorkPackage((*reinterpret_cast<std::add_pointer_t<KPlatoWork::WorkPackage*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->slotNodeChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 3: _t->slotNodeToBeInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->slotNodeInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 5: _t->slotNodeToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 6: _t->slotNodeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 7: _t->slotDocumentAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Document*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 8: _t->slotDocumentRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Document*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 9: _t->slotDocumentChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Document*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlatoWork::TaskWorkPackageModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlatoWork::TaskWorkPackageModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork20TaskWorkPackageModelE_t>.strings))
        return static_cast<void*>(this);
    return KPlato::ItemModelBase::qt_metacast(_clname);
}

int KPlatoWork::TaskWorkPackageModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPlato::ItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
