/********************************************************************************
** Form generated from reading UI file 'kptrecalculatedialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTRECALCULATEDIALOG_H
#define UI_KPTRECALCULATEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_RecalculateDialog
{
public:
    QHBoxLayout *horizontalLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QRadioButton *btnCurrent;
    QRadioButton *btnFrom;
    QDateTimeEdit *dateTimeEdit;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPlato__RecalculateDialog)
    {
        if (KPlato__RecalculateDialog->objectName().isEmpty())
            KPlato__RecalculateDialog->setObjectName("KPlato__RecalculateDialog");
        KPlato__RecalculateDialog->resize(296, 113);
        horizontalLayout = new QHBoxLayout(KPlato__RecalculateDialog);
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout->setObjectName("horizontalLayout");
        groupBox = new QGroupBox(KPlato__RecalculateDialog);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        btnCurrent = new QRadioButton(groupBox);
        btnCurrent->setObjectName("btnCurrent");

        gridLayout->addWidget(btnCurrent, 0, 0, 1, 2);

        btnFrom = new QRadioButton(groupBox);
        btnFrom->setObjectName("btnFrom");

        gridLayout->addWidget(btnFrom, 1, 0, 1, 1);

        dateTimeEdit = new QDateTimeEdit(groupBox);
        dateTimeEdit->setObjectName("dateTimeEdit");
        dateTimeEdit->setCalendarPopup(true);

        gridLayout->addWidget(dateTimeEdit, 1, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 2, 0, 1, 1);


        horizontalLayout->addWidget(groupBox);


        retranslateUi(KPlato__RecalculateDialog);

        QMetaObject::connectSlotsByName(KPlato__RecalculateDialog);
    } // setupUi

    void retranslateUi(QWidget *KPlato__RecalculateDialog)
    {
        groupBox->setTitle(QString());
        btnCurrent->setText(tr2i18n("Re-calculate from current date/time", nullptr));
        btnFrom->setText(tr2i18n("Re-calculate from:", nullptr));
        (void)KPlato__RecalculateDialog;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class RecalculateDialog: public Ui_RecalculateDialog {};
} // namespace Ui
} // namespace KPlato

#endif // KPTRECALCULATEDIALOG_H

