/****************************************************************************
** Meta object code from reading C++ file 'kptaccountsmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/models/kptaccountsmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptaccountsmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato12AccountModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::AccountModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato12AccountModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::AccountModel",
        "Properties",
        "Name",
        "Description"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Properties'
        QtMocHelpers::EnumData<enum Properties>(1, 1, QMC::EnumFlags{}).add({
            {    2, Properties::Name },
            {    3, Properties::Description },
        }),
    };
    return QtMocHelpers::metaObjectData<AccountModel, qt_meta_tag_ZN6KPlato12AccountModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::AccountModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12AccountModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12AccountModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato12AccountModelE_t>.metaTypes,
    nullptr
} };

void KPlato::AccountModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::AccountModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::AccountModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12AccountModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KPlato::AccountModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato16AccountItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::AccountItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato16AccountItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::AccountItemModel",
        "slotAccountChanged",
        "",
        "KPlato::Account*",
        "slotAccountToBeInserted",
        "const KPlato::Account*",
        "parent",
        "row",
        "slotAccountInserted",
        "account",
        "slotAccountToBeRemoved",
        "slotAccountRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotAccountChanged'
        QtMocHelpers::SlotData<void(KPlato::Account *)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotAccountToBeInserted'
        QtMocHelpers::SlotData<void(const KPlato::Account *, int)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::Int, 7 },
        }}),
        // Slot 'slotAccountInserted'
        QtMocHelpers::SlotData<void(const KPlato::Account *)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
        // Slot 'slotAccountToBeRemoved'
        QtMocHelpers::SlotData<void(const KPlato::Account *)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
        // Slot 'slotAccountRemoved'
        QtMocHelpers::SlotData<void(const KPlato::Account *)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AccountItemModel, qt_meta_tag_ZN6KPlato16AccountItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::AccountItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16AccountItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16AccountItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato16AccountItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::AccountItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotAccountChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Account*>>(_a[1]))); break;
        case 1: _t->slotAccountToBeInserted((*reinterpret_cast<std::add_pointer_t<const KPlato::Account*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->slotAccountInserted((*reinterpret_cast<std::add_pointer_t<const KPlato::Account*>>(_a[1]))); break;
        case 3: _t->slotAccountToBeRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::Account*>>(_a[1]))); break;
        case 4: _t->slotAccountRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::Account*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::AccountItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::AccountItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16AccountItemModelE_t>.strings))
        return static_cast<void*>(this);
    return ItemModelBase::qt_metacast(_clname);
}

int KPlato::AccountItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato22CostBreakdownItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::CostBreakdownItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato22CostBreakdownItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::CostBreakdownItemModel",
        "slotAccountChanged",
        "",
        "KPlato::Account*",
        "slotAccountToBeInserted",
        "const KPlato::Account*",
        "parent",
        "row",
        "slotAccountInserted",
        "account",
        "slotAccountToBeRemoved",
        "slotAccountRemoved",
        "slotDataChanged",
        "Properties",
        "Name",
        "Description",
        "Total",
        "Planned",
        "Actual"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotAccountChanged'
        QtMocHelpers::SlotData<void(KPlato::Account *)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotAccountToBeInserted'
        QtMocHelpers::SlotData<void(const KPlato::Account *, int)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::Int, 7 },
        }}),
        // Slot 'slotAccountInserted'
        QtMocHelpers::SlotData<void(const KPlato::Account *)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
        // Slot 'slotAccountToBeRemoved'
        QtMocHelpers::SlotData<void(const KPlato::Account *)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
        // Slot 'slotAccountRemoved'
        QtMocHelpers::SlotData<void(const KPlato::Account *)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
        // Slot 'slotDataChanged'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Properties'
        QtMocHelpers::EnumData<enum Properties>(13, 13, QMC::EnumFlags{}).add({
            {   14, Properties::Name },
            {   15, Properties::Description },
            {   16, Properties::Total },
            {   17, Properties::Planned },
            {   18, Properties::Actual },
        }),
    };
    return QtMocHelpers::metaObjectData<CostBreakdownItemModel, qt_meta_tag_ZN6KPlato22CostBreakdownItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::CostBreakdownItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22CostBreakdownItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22CostBreakdownItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato22CostBreakdownItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::CostBreakdownItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CostBreakdownItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotAccountChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Account*>>(_a[1]))); break;
        case 1: _t->slotAccountToBeInserted((*reinterpret_cast<std::add_pointer_t<const KPlato::Account*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->slotAccountInserted((*reinterpret_cast<std::add_pointer_t<const KPlato::Account*>>(_a[1]))); break;
        case 3: _t->slotAccountToBeRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::Account*>>(_a[1]))); break;
        case 4: _t->slotAccountRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::Account*>>(_a[1]))); break;
        case 5: _t->slotDataChanged(); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::CostBreakdownItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::CostBreakdownItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22CostBreakdownItemModelE_t>.strings))
        return static_cast<void*>(this);
    return ItemModelBase::qt_metacast(_clname);
}

int KPlato::CostBreakdownItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}
QT_WARNING_POP
