/****************************************************************************
** Meta object code from reading C++ file 'taskworkpackageview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../calligraplan-4.0.1/src/workpackage/taskworkpackageview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'taskworkpackageview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KPlatoWork23TaskWorkPackageTreeViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlatoWork::TaskWorkPackageTreeView::qt_create_metaobjectdata<qt_meta_tag_ZN10KPlatoWork23TaskWorkPackageTreeViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlatoWork::TaskWorkPackageTreeView",
        "sectionsMoved",
        "",
        "slotActivated",
        "QModelIndex",
        "index",
        "setSortOrder",
        "col",
        "Qt::SortOrder",
        "order"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sectionsMoved'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotActivated'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'setSortOrder'
        QtMocHelpers::SlotData<void(int, Qt::SortOrder)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { 0x80000000 | 8, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaskWorkPackageTreeView, qt_meta_tag_ZN10KPlatoWork23TaskWorkPackageTreeViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlatoWork::TaskWorkPackageTreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<KPlato::DoubleTreeViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork23TaskWorkPackageTreeViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork23TaskWorkPackageTreeViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KPlatoWork23TaskWorkPackageTreeViewE_t>.metaTypes,
    nullptr
} };

void KPlatoWork::TaskWorkPackageTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskWorkPackageTreeView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sectionsMoved(); break;
        case 1: _t->slotActivated((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 2: _t->setSortOrder((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TaskWorkPackageTreeView::*)()>(_a, &TaskWorkPackageTreeView::sectionsMoved, 0))
            return;
    }
}

const QMetaObject *KPlatoWork::TaskWorkPackageTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlatoWork::TaskWorkPackageTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork23TaskWorkPackageTreeViewE_t>.strings))
        return static_cast<void*>(this);
    return KPlato::DoubleTreeViewBase::qt_metacast(_clname);
}

int KPlatoWork::TaskWorkPackageTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPlato::DoubleTreeViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void KPlatoWork::TaskWorkPackageTreeView::sectionsMoved()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN10KPlatoWork12AbstractViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlatoWork::AbstractView::qt_create_metaobjectdata<qt_meta_tag_ZN10KPlatoWork12AbstractViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlatoWork::AbstractView",
        "requestPopupMenu",
        "",
        "name",
        "QPoint",
        "pos",
        "selectionChanged",
        "slotHeaderContextMenuRequested",
        "slotContextMenuRequested",
        "QModelIndex",
        "index",
        "KPlato::Node*",
        "node",
        "KPlato::Document*",
        "doc",
        "sectionsMoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestPopupMenu'
        QtMocHelpers::SignalData<void(const QString &, const QPoint &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'selectionChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHeaderContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QPoint &)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(KPlato::Node *, const QPoint &)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(KPlato::Document *, const QPoint &)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'sectionsMoved'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AbstractView, qt_meta_tag_ZN10KPlatoWork12AbstractViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlatoWork::AbstractView::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork12AbstractViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork12AbstractViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KPlatoWork12AbstractViewE_t>.metaTypes,
    nullptr
} };

void KPlatoWork::AbstractView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AbstractView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestPopupMenu((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 1: _t->selectionChanged(); break;
        case 2: _t->slotHeaderContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 3: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 4: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 5: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<KPlato::Document*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 6: _t->sectionsMoved(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AbstractView::*)(const QString & , const QPoint & )>(_a, &AbstractView::requestPopupMenu, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractView::*)()>(_a, &AbstractView::selectionChanged, 1))
            return;
    }
}

const QMetaObject *KPlatoWork::AbstractView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlatoWork::AbstractView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork12AbstractViewE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "KPlato::ViewActionLists"))
        return static_cast< KPlato::ViewActionLists*>(this);
    return QWidget::qt_metacast(_clname);
}

int KPlatoWork::AbstractView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void KPlatoWork::AbstractView::requestPopupMenu(const QString & _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KPlatoWork::AbstractView::selectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN10KPlatoWork19TaskWorkPackageViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlatoWork::TaskWorkPackageView::qt_create_metaobjectdata<qt_meta_tag_ZN10KPlatoWork19TaskWorkPackageViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlatoWork::TaskWorkPackageView",
        "slotOptions",
        "",
        "slotSplitView",
        "slotContextMenuRequested",
        "QModelIndex",
        "index",
        "QPoint",
        "pos",
        "slotSelectionChanged",
        "QModelIndexList",
        "lst"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotOptions'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSplitView'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QPoint &)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QModelIndexList &)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaskWorkPackageView, qt_meta_tag_ZN10KPlatoWork19TaskWorkPackageViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlatoWork::TaskWorkPackageView::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork19TaskWorkPackageViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork19TaskWorkPackageViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KPlatoWork19TaskWorkPackageViewE_t>.metaTypes,
    nullptr
} };

void KPlatoWork::TaskWorkPackageView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskWorkPackageView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotOptions(); break;
        case 1: _t->slotSplitView(); break;
        case 2: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 3: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KPlatoWork::TaskWorkPackageView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlatoWork::TaskWorkPackageView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork19TaskWorkPackageViewE_t>.strings))
        return static_cast<void*>(this);
    return AbstractView::qt_metacast(_clname);
}

int KPlatoWork::TaskWorkPackageView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN10KPlatoWork17GanttItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlatoWork::GanttItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN10KPlatoWork17GanttItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlatoWork::GanttItemDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GanttItemDelegate, qt_meta_tag_ZN10KPlatoWork17GanttItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlatoWork::GanttItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<KPlato::GanttItemDelegate::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork17GanttItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork17GanttItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KPlatoWork17GanttItemDelegateE_t>.metaTypes,
    nullptr
} };

void KPlatoWork::GanttItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GanttItemDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlatoWork::GanttItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlatoWork::GanttItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork17GanttItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return KPlato::GanttItemDelegate::qt_metacast(_clname);
}

int KPlatoWork::GanttItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPlato::GanttItemDelegate::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN10KPlatoWork9GanttViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlatoWork::GanttView::qt_create_metaobjectdata<qt_meta_tag_ZN10KPlatoWork9GanttViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlatoWork::GanttView",
        "headerContextMenuRequested",
        "",
        "QPoint",
        "selectionChanged",
        "QModelIndexList",
        "sectionsMoved",
        "slotSelectionChanged",
        "QItemSelection",
        "selected",
        "deselected",
        "slotRowsInserted",
        "QModelIndex",
        "parent",
        "start",
        "end",
        "slotRowsRemoved",
        "updateDateTimeGrid",
        "KPlatoWork::WorkPackage*",
        "wp"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'headerContextMenuRequested'
        QtMocHelpers::SignalData<void(const QPoint &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'selectionChanged'
        QtMocHelpers::SignalData<void(const QModelIndexList &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'sectionsMoved'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 8, 10 },
        }}),
        // Slot 'slotRowsInserted'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { QMetaType::Int, 14 }, { QMetaType::Int, 15 },
        }}),
        // Slot 'slotRowsRemoved'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { QMetaType::Int, 14 }, { QMetaType::Int, 15 },
        }}),
        // Slot 'updateDateTimeGrid'
        QtMocHelpers::SlotData<void(KPlatoWork::WorkPackage *)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GanttView, qt_meta_tag_ZN10KPlatoWork9GanttViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlatoWork::GanttView::staticMetaObject = { {
    QMetaObject::SuperData::link<KPlato::GanttViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork9GanttViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork9GanttViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KPlatoWork9GanttViewE_t>.metaTypes,
    nullptr
} };

void KPlatoWork::GanttView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GanttView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->headerContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 1: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 2: _t->sectionsMoved(); break;
        case 3: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 4: _t->slotRowsInserted((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 5: _t->slotRowsRemoved((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 6: _t->updateDateTimeGrid((*reinterpret_cast<std::add_pointer_t<KPlatoWork::WorkPackage*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GanttView::*)(const QPoint & )>(_a, &GanttView::headerContextMenuRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GanttView::*)(const QModelIndexList & )>(_a, &GanttView::selectionChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (GanttView::*)()>(_a, &GanttView::sectionsMoved, 2))
            return;
    }
}

const QMetaObject *KPlatoWork::GanttView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlatoWork::GanttView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork9GanttViewE_t>.strings))
        return static_cast<void*>(this);
    return KPlato::GanttViewBase::qt_metacast(_clname);
}

int KPlatoWork::GanttView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPlato::GanttViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void KPlatoWork::GanttView::headerContextMenuRequested(const QPoint & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlatoWork::GanttView::selectionChanged(const QModelIndexList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KPlatoWork::GanttView::sectionsMoved()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN10KPlatoWork15TaskWPGanttViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlatoWork::TaskWPGanttView::qt_create_metaobjectdata<qt_meta_tag_ZN10KPlatoWork15TaskWPGanttViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlatoWork::TaskWPGanttView",
        "slotContextMenuRequested",
        "",
        "QModelIndex",
        "index",
        "QPoint",
        "pos",
        "slotSelectionChanged",
        "QModelIndexList",
        "lst",
        "slotOptions"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QPoint &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QModelIndexList &)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotOptions'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaskWPGanttView, qt_meta_tag_ZN10KPlatoWork15TaskWPGanttViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlatoWork::TaskWPGanttView::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork15TaskWPGanttViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork15TaskWPGanttViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KPlatoWork15TaskWPGanttViewE_t>.metaTypes,
    nullptr
} };

void KPlatoWork::TaskWPGanttView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskWPGanttView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 1: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 2: _t->slotOptions(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KPlatoWork::TaskWPGanttView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlatoWork::TaskWPGanttView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork15TaskWPGanttViewE_t>.strings))
        return static_cast<void*>(this);
    return AbstractView::qt_metacast(_clname);
}

int KPlatoWork::TaskWPGanttView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
