/********************************************************************************
** Form generated from reading UI file 'ICalExportDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ICALEXPORTDIALOG_H
#define UI_ICALEXPORTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kptscheduleeditor.h"

namespace KPlato {

class Ui_ICalExportDialog
{
public:
    QHBoxLayout *horizontalLayout_2;
    QWidget *ui_mainWidget;
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout;
    QCheckBox *ui_project;
    QCheckBox *ui_summarytasks;
    KPlato::ScheduleTreeView *ui_scheduleView;

    void setupUi(QWidget *KPlato__ICalExportDialog)
    {
        if (KPlato__ICalExportDialog->objectName().isEmpty())
            KPlato__ICalExportDialog->setObjectName("KPlato__ICalExportDialog");
        KPlato__ICalExportDialog->resize(333, 260);
        horizontalLayout_2 = new QHBoxLayout(KPlato__ICalExportDialog);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        ui_mainWidget = new QWidget(KPlato__ICalExportDialog);
        ui_mainWidget->setObjectName("ui_mainWidget");
        verticalLayout = new QVBoxLayout(ui_mainWidget);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(ui_mainWidget);
        groupBox->setObjectName("groupBox");
        horizontalLayout = new QHBoxLayout(groupBox);
        horizontalLayout->setObjectName("horizontalLayout");
        ui_project = new QCheckBox(groupBox);
        ui_project->setObjectName("ui_project");
        ui_project->setChecked(true);

        horizontalLayout->addWidget(ui_project);

        ui_summarytasks = new QCheckBox(groupBox);
        ui_summarytasks->setObjectName("ui_summarytasks");
        ui_summarytasks->setChecked(true);

        horizontalLayout->addWidget(ui_summarytasks);


        verticalLayout->addWidget(groupBox);

        ui_scheduleView = new KPlato::ScheduleTreeView(ui_mainWidget);
        ui_scheduleView->setObjectName("ui_scheduleView");

        verticalLayout->addWidget(ui_scheduleView);


        horizontalLayout_2->addWidget(ui_mainWidget);


        retranslateUi(KPlato__ICalExportDialog);

        QMetaObject::connectSlotsByName(KPlato__ICalExportDialog);
    } // setupUi

    void retranslateUi(QWidget *KPlato__ICalExportDialog)
    {
        groupBox->setTitle(QString());
        ui_project->setText(tr2i18n("Include Project", nullptr));
        ui_summarytasks->setText(tr2i18n("Include Summarytasks", nullptr));
        (void)KPlato__ICalExportDialog;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class ICalExportDialog: public Ui_ICalExportDialog {};
} // namespace Ui
} // namespace KPlato

#endif // ICALEXPORTDIALOG_H

