/********************************************************************************
** Form generated from reading UI file 'kptperteditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTPERTEDITOR_H
#define UI_KPTPERTEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kptrelationeditor.h"

namespace KPlato {

class Ui_PertEditor
{
public:
    QHBoxLayout *hboxLayout;
    QTreeWidget *taskList;
    QTreeWidget *available;
    QVBoxLayout *vboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *addBtn;
    QPushButton *removeBtn;
    QSpacerItem *spacerItem1;
    RelationTreeView *required;

    void setupUi(QWidget *KPlato__PertEditor)
    {
        if (KPlato__PertEditor->objectName().isEmpty())
            KPlato__PertEditor->setObjectName("KPlato__PertEditor");
        KPlato__PertEditor->resize(809, 200);
        hboxLayout = new QHBoxLayout(KPlato__PertEditor);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        taskList = new QTreeWidget(KPlato__PertEditor);
        taskList->setObjectName("taskList");

        hboxLayout->addWidget(taskList);

        available = new QTreeWidget(KPlato__PertEditor);
        available->setObjectName("available");

        hboxLayout->addWidget(available);

        vboxLayout = new QVBoxLayout();
        vboxLayout->setObjectName("vboxLayout");
        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem);

        addBtn = new QPushButton(KPlato__PertEditor);
        addBtn->setObjectName("addBtn");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(addBtn->sizePolicy().hasHeightForWidth());
        addBtn->setSizePolicy(sizePolicy);
        addBtn->setMaximumSize(QSize(25, 25));

        vboxLayout->addWidget(addBtn);

        removeBtn = new QPushButton(KPlato__PertEditor);
        removeBtn->setObjectName("removeBtn");
        removeBtn->setMaximumSize(QSize(25, 25));

        vboxLayout->addWidget(removeBtn);

        spacerItem1 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem1);


        hboxLayout->addLayout(vboxLayout);

        required = new RelationTreeView(KPlato__PertEditor);
        required->setObjectName("required");

        hboxLayout->addWidget(required);

        QWidget::setTabOrder(taskList, available);
        QWidget::setTabOrder(available, addBtn);
        QWidget::setTabOrder(addBtn, removeBtn);
        QWidget::setTabOrder(removeBtn, required);

        retranslateUi(KPlato__PertEditor);

        QMetaObject::connectSlotsByName(KPlato__PertEditor);
    } // setupUi

    void retranslateUi(QWidget *KPlato__PertEditor)
    {
        QTreeWidgetItem *___qtreewidgetitem = taskList->headerItem();
        ___qtreewidgetitem->setText(0, tr2i18n("Tasks", nullptr));
        QTreeWidgetItem *___qtreewidgetitem1 = available->headerItem();
        ___qtreewidgetitem1->setText(0, tr2i18n("Available tasks", nullptr));
#if QT_CONFIG(tooltip)
        addBtn->setToolTip(tr2i18n("Add required task", nullptr));
#endif // QT_CONFIG(tooltip)
        addBtn->setText(QString());
#if QT_CONFIG(tooltip)
        removeBtn->setToolTip(tr2i18n("Remove required task", nullptr));
#endif // QT_CONFIG(tooltip)
        removeBtn->setText(QString());
        (void)KPlato__PertEditor;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class PertEditor: public Ui_PertEditor {};
} // namespace Ui
} // namespace KPlato

#endif // KPTPERTEDITOR_H

