/********************************************************************************
** Form generated from reading UI file 'RelationEditorDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RELATIONEDITORDIALOG_H
#define UI_RELATIONEDITORDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_RelationEditorDialog
{
public:
    QHBoxLayout *horizontalLayout_2;
    QWidget *mainWidget;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QLabel *taskName;
    QHBoxLayout *horizontalLayout_3;
    QTreeView *view;
    QVBoxLayout *verticalLayout;
    QSpacerItem *verticalSpacer;
    QToolButton *addBtn;
    QToolButton *removeBtn;
    QSpacerItem *verticalSpacer_2;

    void setupUi(QWidget *KPlato__RelationEditorDialog)
    {
        if (KPlato__RelationEditorDialog->objectName().isEmpty())
            KPlato__RelationEditorDialog->setObjectName("KPlato__RelationEditorDialog");
        KPlato__RelationEditorDialog->resize(550, 172);
        horizontalLayout_2 = new QHBoxLayout(KPlato__RelationEditorDialog);
        horizontalLayout_2->setSpacing(0);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        mainWidget = new QWidget(KPlato__RelationEditorDialog);
        mainWidget->setObjectName("mainWidget");
        verticalLayout_2 = new QVBoxLayout(mainWidget);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(mainWidget);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(label);

        taskName = new QLabel(mainWidget);
        taskName->setObjectName("taskName");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(taskName->sizePolicy().hasHeightForWidth());
        taskName->setSizePolicy(sizePolicy1);
        taskName->setFrameShape(QFrame::StyledPanel);
        taskName->setFrameShadow(QFrame::Sunken);
        taskName->setText(QString::fromUtf8(""));
        taskName->setWordWrap(true);

        horizontalLayout->addWidget(taskName);


        verticalLayout_2->addLayout(horizontalLayout);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        view = new QTreeView(mainWidget);
        view->setObjectName("view");
        view->setTabKeyNavigation(true);
        view->setProperty("showDropIndicator", QVariant(false));
        view->setAlternatingRowColors(true);
        view->setSelectionMode(QAbstractItemView::ExtendedSelection);
        view->setSelectionBehavior(QAbstractItemView::SelectItems);
        view->setRootIsDecorated(false);
        view->setItemsExpandable(false);

        horizontalLayout_3->addWidget(view);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        addBtn = new QToolButton(mainWidget);
        addBtn->setObjectName("addBtn");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        addBtn->setIcon(icon);

        verticalLayout->addWidget(addBtn);

        removeBtn = new QToolButton(mainWidget);
        removeBtn->setObjectName("removeBtn");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        removeBtn->setIcon(icon1);

        verticalLayout->addWidget(removeBtn);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);


        horizontalLayout_3->addLayout(verticalLayout);


        verticalLayout_2->addLayout(horizontalLayout_3);


        horizontalLayout_2->addWidget(mainWidget);


        retranslateUi(KPlato__RelationEditorDialog);

        QMetaObject::connectSlotsByName(KPlato__RelationEditorDialog);
    } // setupUi

    void retranslateUi(QWidget *KPlato__RelationEditorDialog)
    {
        label->setText(tr2i18n("Task:", nullptr));
        addBtn->setText(tr2i18n("Insert", "Insert task dependency"));
        removeBtn->setText(QString());
        (void)KPlato__RelationEditorDialog;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class RelationEditorDialog: public Ui_RelationEditorDialog {};
} // namespace Ui
} // namespace KPlato

#endif // RELATIONEDITORDIALOG_H

