/********************************************************************************
** Form generated from reading UI file 'SchedulingView.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCHEDULINGVIEW_H
#define UI_SCHEDULINGVIEW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SchedulingView
{
public:
    QHBoxLayout *horizontalLayout;
    QTreeView *schedulingView;
    QGroupBox *schedulingProperties;
    QGridLayout *gridLayout;
    QLabel *label;
    QPushButton *calculate;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout_3;
    QRadioButton *sequential;
    QLabel *label_4;
    QRadioButton *parallel;
    QComboBox *granularities;
    QSpacerItem *horizontalSpacer_2;
    QComboBox *schedulersCombo;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QRadioButton *todayRB;
    QDateTimeEdit *calculationDateTime;
    QRadioButton *timeRB;
    QRadioButton *tomorrowRB;

    void setupUi(QWidget *SchedulingView)
    {
        if (SchedulingView->objectName().isEmpty())
            SchedulingView->setObjectName("SchedulingView");
        SchedulingView->resize(865, 374);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SchedulingView->sizePolicy().hasHeightForWidth());
        SchedulingView->setSizePolicy(sizePolicy);
        horizontalLayout = new QHBoxLayout(SchedulingView);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        schedulingView = new QTreeView(SchedulingView);
        schedulingView->setObjectName("schedulingView");
        schedulingView->setContextMenuPolicy(Qt::CustomContextMenu);
        schedulingView->setRootIsDecorated(false);

        horizontalLayout->addWidget(schedulingView);

        schedulingProperties = new QGroupBox(SchedulingView);
        schedulingProperties->setObjectName("schedulingProperties");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(schedulingProperties->sizePolicy().hasHeightForWidth());
        schedulingProperties->setSizePolicy(sizePolicy1);
        gridLayout = new QGridLayout(schedulingProperties);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(schedulingProperties);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        calculate = new QPushButton(schedulingProperties);
        calculate->setObjectName("calculate");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("view-time-schedule-calculus");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        calculate->setIcon(icon);

        gridLayout->addWidget(calculate, 3, 1, 2, 1);

        groupBox_2 = new QGroupBox(schedulingProperties);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout_3 = new QGridLayout(groupBox_2);
        gridLayout_3->setObjectName("gridLayout_3");
        sequential = new QRadioButton(groupBox_2);
        sequential->setObjectName("sequential");

        gridLayout_3->addWidget(sequential, 0, 0, 1, 1);

        label_4 = new QLabel(groupBox_2);
        label_4->setObjectName("label_4");

        gridLayout_3->addWidget(label_4, 2, 0, 1, 1);

        parallel = new QRadioButton(groupBox_2);
        parallel->setObjectName("parallel");

        gridLayout_3->addWidget(parallel, 0, 1, 1, 1);

        granularities = new QComboBox(groupBox_2);
        granularities->setObjectName("granularities");

        gridLayout_3->addWidget(granularities, 2, 1, 1, 1);


        gridLayout->addWidget(groupBox_2, 1, 0, 1, 3);

        horizontalSpacer_2 = new QSpacerItem(58, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_2, 4, 2, 1, 1);

        schedulersCombo = new QComboBox(schedulingProperties);
        schedulersCombo->setObjectName("schedulersCombo");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(schedulersCombo->sizePolicy().hasHeightForWidth());
        schedulersCombo->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(schedulersCombo, 0, 1, 1, 2);

        horizontalSpacer = new QSpacerItem(62, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 4, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 5, 1, 1, 1);

        groupBox = new QGroupBox(schedulingProperties);
        groupBox->setObjectName("groupBox");
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        todayRB = new QRadioButton(groupBox);
        todayRB->setObjectName("todayRB");
        todayRB->setChecked(true);

        gridLayout_2->addWidget(todayRB, 0, 0, 1, 1);

        calculationDateTime = new QDateTimeEdit(groupBox);
        calculationDateTime->setObjectName("calculationDateTime");
        calculationDateTime->setCalendarPopup(true);

        gridLayout_2->addWidget(calculationDateTime, 2, 1, 1, 1);

        timeRB = new QRadioButton(groupBox);
        timeRB->setObjectName("timeRB");

        gridLayout_2->addWidget(timeRB, 2, 0, 1, 1);

        tomorrowRB = new QRadioButton(groupBox);
        tomorrowRB->setObjectName("tomorrowRB");

        gridLayout_2->addWidget(tomorrowRB, 0, 1, 1, 1);


        gridLayout->addWidget(groupBox, 2, 0, 1, 3);


        horizontalLayout->addWidget(schedulingProperties);

#if QT_CONFIG(shortcut)
        label_4->setBuddy(granularities);
#endif // QT_CONFIG(shortcut)

        retranslateUi(SchedulingView);

        QMetaObject::connectSlotsByName(SchedulingView);
    } // setupUi

    void retranslateUi(QWidget *SchedulingView)
    {
        SchedulingView->setWindowTitle(QString());
        schedulingProperties->setTitle(tr2i18n("Scheduling Control", nullptr));
        label->setText(tr2i18n("Scheduler:", nullptr));
        calculate->setText(tr2i18n("Calculate", nullptr));
        groupBox_2->setTitle(tr2i18n("Options", nullptr));
        sequential->setText(tr2i18n("Sequential", nullptr));
        label_4->setText(tr2i18n("Granularity:", nullptr));
        parallel->setText(tr2i18n("Parallel", nullptr));
        groupBox->setTitle(tr2i18n("Calculate From", nullptr));
        todayRB->setText(tr2i18n("Today", nullptr));
        timeRB->setText(tr2i18n("Time:", nullptr));
        tomorrowRB->setText(tr2i18n("Tomorrow", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SchedulingView: public Ui_SchedulingView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SCHEDULINGVIEW_H

