/****************************************************************************
** Meta object code from reading C++ file 'KoMainWindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/main/KoMainWindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KoMainWindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12KoMainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto KoMainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN12KoMainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KoMainWindow",
        "documentSaved",
        "",
        "saveDialogShown",
        "restoringDone",
        "loadCompleted",
        "loadCanceled",
        "themeChanged",
        "keyBindingsChanged",
        "configure",
        "KoMainWindow*",
        "mw",
        "updateCaption",
        "slotEmailFile",
        "slotFileNew",
        "slotFileOpen",
        "slotFileOpenRecent",
        "QUrl",
        "KoPart*",
        "part",
        "slotFileSave",
        "slotFileSaveAs",
        "slotFilePrint",
        "slotFilePrintPreview",
        "printPreviewToPdf",
        "KoPrintJob*",
        "exportToPdf",
        "pdfFileName",
        "slotDocumentInfo",
        "slotFileClose",
        "slotFileQuit",
        "slotConfigureKeys",
        "slotConfigureToolbars",
        "slotNewToolbarConfig",
        "slotToolbarToggled",
        "toggle",
        "viewFullscreen",
        "fullScreen",
        "showDockerTitleBars",
        "show",
        "slotReloadFile",
        "slotImportFile",
        "slotExportFile",
        "slotEncryptDocument",
        "slotUncompressToDir",
        "slotProgress",
        "value",
        "toggleDockersVisibility",
        "visible",
        "saveDocument",
        "saveas",
        "silent",
        "specialOutputFlag",
        "slotConfigure",
        "saveRecentFiles",
        "slotLoadCompleted",
        "slotLoadCanceled",
        "slotSaveCompleted",
        "slotSaveCanceled",
        "forceDockTabFonts",
        "newView",
        "slotPrintPreviewPaintRequest",
        "QPrinter*",
        "printer",
        "slotWidgetDestroyed",
        "slotDocumentTitleModified",
        "caption",
        "mod"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'documentSaved'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'saveDialogShown'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'restoringDone'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loadCompleted'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loadCanceled'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'themeChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'keyBindingsChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'configure'
        QtMocHelpers::SignalData<void(KoMainWindow *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'updateCaption'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEmailFile'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileNew'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileOpen'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileOpenRecent'
        QtMocHelpers::SlotData<void(const QUrl &, KoPart *)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 2 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'slotFileOpenRecent'
        QtMocHelpers::SlotData<void(const QUrl &)>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 17, 2 },
        }}),
        // Slot 'slotFileSave'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileSaveAs'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFilePrint'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFilePrintPreview'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'printPreviewToPdf'
        QtMocHelpers::SlotData<KoPrintJob *()>(24, 2, QMC::AccessPublic, 0x80000000 | 25),
        // Slot 'exportToPdf'
        QtMocHelpers::SlotData<KoPrintJob *(const QString &)>(26, 2, QMC::AccessPublic, 0x80000000 | 25, {{
            { QMetaType::QString, 27 },
        }}),
        // Slot 'exportToPdf'
        QtMocHelpers::SlotData<KoPrintJob *()>(26, 2, QMC::AccessPublic, 0x80000000 | 25),
        // Slot 'slotDocumentInfo'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileClose'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileQuit'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotConfigureKeys'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotConfigureToolbars'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNewToolbarConfig'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotToolbarToggled'
        QtMocHelpers::SlotData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'viewFullscreen'
        QtMocHelpers::SlotData<void(bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 37 },
        }}),
        // Slot 'showDockerTitleBars'
        QtMocHelpers::SlotData<void(bool)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 39 },
        }}),
        // Slot 'slotReloadFile'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotImportFile'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotExportFile'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEncryptDocument'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUncompressToDir'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotProgress'
        QtMocHelpers::SlotData<void(int)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 46 },
        }}),
        // Slot 'toggleDockersVisibility'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 48 },
        }}),
        // Slot 'saveDocument'
        QtMocHelpers::SlotData<bool(bool, bool, int)>(49, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Bool, 50 }, { QMetaType::Bool, 51 }, { QMetaType::Int, 52 },
        }}),
        // Slot 'saveDocument'
        QtMocHelpers::SlotData<bool(bool, bool)>(49, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::Bool, 50 }, { QMetaType::Bool, 51 },
        }}),
        // Slot 'saveDocument'
        QtMocHelpers::SlotData<bool(bool)>(49, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::Bool, 50 },
        }}),
        // Slot 'saveDocument'
        QtMocHelpers::SlotData<bool()>(49, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'slotConfigure'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveRecentFiles'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotLoadCompleted'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotLoadCanceled'
        QtMocHelpers::SlotData<void(const QString &)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotSaveCompleted'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSaveCanceled'
        QtMocHelpers::SlotData<void(const QString &)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'forceDockTabFonts'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newView'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPrintPreviewPaintRequest'
        QtMocHelpers::SlotData<void(QPrinter *)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 62, 63 },
        }}),
        // Slot 'slotWidgetDestroyed'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDocumentTitleModified'
        QtMocHelpers::SlotData<void(const QString &, bool)>(65, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 66 }, { QMetaType::Bool, 67 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KoMainWindow, qt_meta_tag_ZN12KoMainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KoMainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<KXmlGuiWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KoMainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KoMainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12KoMainWindowE_t>.metaTypes,
    nullptr
} };

void KoMainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KoMainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->documentSaved(); break;
        case 1: _t->saveDialogShown(); break;
        case 2: _t->restoringDone(); break;
        case 3: _t->loadCompleted(); break;
        case 4: _t->loadCanceled(); break;
        case 5: _t->themeChanged(); break;
        case 6: _t->keyBindingsChanged(); break;
        case 7: _t->configure((*reinterpret_cast<std::add_pointer_t<KoMainWindow*>>(_a[1]))); break;
        case 8: _t->updateCaption(); break;
        case 9: _t->slotEmailFile(); break;
        case 10: _t->slotFileNew(); break;
        case 11: _t->slotFileOpen(); break;
        case 12: _t->slotFileOpenRecent((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KoPart*>>(_a[2]))); break;
        case 13: _t->slotFileOpenRecent((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 14: _t->slotFileSave(); break;
        case 15: _t->slotFileSaveAs(); break;
        case 16: _t->slotFilePrint(); break;
        case 17: _t->slotFilePrintPreview(); break;
        case 18: { KoPrintJob* _r = _t->printPreviewToPdf();
            if (_a[0]) *reinterpret_cast<KoPrintJob**>(_a[0]) = std::move(_r); }  break;
        case 19: { KoPrintJob* _r = _t->exportToPdf((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KoPrintJob**>(_a[0]) = std::move(_r); }  break;
        case 20: { KoPrintJob* _r = _t->exportToPdf();
            if (_a[0]) *reinterpret_cast<KoPrintJob**>(_a[0]) = std::move(_r); }  break;
        case 21: _t->slotDocumentInfo(); break;
        case 22: _t->slotFileClose(); break;
        case 23: _t->slotFileQuit(); break;
        case 24: _t->slotConfigureKeys(); break;
        case 25: _t->slotConfigureToolbars(); break;
        case 26: _t->slotNewToolbarConfig(); break;
        case 27: _t->slotToolbarToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->viewFullscreen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->showDockerTitleBars((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->slotReloadFile(); break;
        case 31: _t->slotImportFile(); break;
        case 32: _t->slotExportFile(); break;
        case 33: _t->slotEncryptDocument(); break;
        case 34: _t->slotUncompressToDir(); break;
        case 35: _t->slotProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 36: _t->toggleDockersVisibility((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: { bool _r = _t->saveDocument((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 38: { bool _r = _t->saveDocument((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 39: { bool _r = _t->saveDocument((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 40: { bool _r = _t->saveDocument();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 41: _t->slotConfigure(); break;
        case 42: _t->saveRecentFiles(); break;
        case 43: _t->slotLoadCompleted(); break;
        case 44: _t->slotLoadCanceled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 45: _t->slotSaveCompleted(); break;
        case 46: _t->slotSaveCanceled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 47: _t->forceDockTabFonts(); break;
        case 48: _t->newView(); break;
        case 49: _t->slotPrintPreviewPaintRequest((*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[1]))); break;
        case 50: _t->slotWidgetDestroyed(); break;
        case 51: _t->slotDocumentTitleModified((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KoMainWindow* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KoMainWindow::*)()>(_a, &KoMainWindow::documentSaved, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoMainWindow::*)()>(_a, &KoMainWindow::saveDialogShown, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoMainWindow::*)()>(_a, &KoMainWindow::restoringDone, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoMainWindow::*)()>(_a, &KoMainWindow::loadCompleted, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoMainWindow::*)()>(_a, &KoMainWindow::loadCanceled, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoMainWindow::*)()>(_a, &KoMainWindow::themeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoMainWindow::*)()>(_a, &KoMainWindow::keyBindingsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoMainWindow::*)(KoMainWindow * )>(_a, &KoMainWindow::configure, 7))
            return;
    }
}

const QMetaObject *KoMainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KoMainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KoMainWindowE_t>.strings))
        return static_cast<void*>(this);
    return KXmlGuiWindow::qt_metacast(_clname);
}

int KoMainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KXmlGuiWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 52)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 52;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 52)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 52;
    }
    return _id;
}

// SIGNAL 0
void KoMainWindow::documentSaved()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KoMainWindow::saveDialogShown()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KoMainWindow::restoringDone()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KoMainWindow::loadCompleted()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KoMainWindow::loadCanceled()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KoMainWindow::themeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void KoMainWindow::keyBindingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KoMainWindow::configure(KoMainWindow * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
