/********************************************************************************
** Form generated from reading UI file 'ConfigWorkVacationPanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGWORKVACATIONPANEL_H
#define UI_CONFIGWORKVACATIONPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTimeEdit>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_ConfigWorkVacationPanel
{
public:
    QHBoxLayout *horizontalLayout_3;
    QTabWidget *tabWidget;
    QWidget *tab_2;
    QHBoxLayout *horizontalLayout_2;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout_4;
    QCheckBox *kcfg_GenerateWeek;
    QComboBox *kcfg_GenerateWeekChoice;
    QWidget *holidaysWidget;
    QGridLayout *gridLayout_3;
    QComboBox *region;
    QCheckBox *kcfg_GenerateHolidays;
    QComboBox *kcfg_GenerateHolidaysChoice;
    QLabel *label;
    QLineEdit *kcfg_Region;
    QSpacerItem *verticalSpacer_2;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout_2;
    QTimeEdit *kcfg_MondayEnd;
    QTimeEdit *kcfg_ThursdayStart;
    QTimeEdit *kcfg_FridayEnd;
    QTimeEdit *kcfg_TuesdayStart;
    QTimeEdit *kcfg_FridayStart;
    QTimeEdit *kcfg_SundayEnd;
    QTimeEdit *kcfg_ThursdayEnd;
    QTimeEdit *kcfg_SaturdayStart;
    QTimeEdit *kcfg_TuesdayEnd;
    QTimeEdit *kcfg_MondayStart;
    QTimeEdit *kcfg_WednesdayStart;
    QTimeEdit *kcfg_SundayStart;
    QTimeEdit *kcfg_WednesdayEnd;
    QTimeEdit *kcfg_SaturdayEnd;
    QSpacerItem *horizontalSpacer_2;
    QCheckBox *kcfg_Tuesday;
    QCheckBox *kcfg_Wednesday;
    QCheckBox *kcfg_Thursday;
    QCheckBox *kcfg_Friday;
    QCheckBox *kcfg_Saturday;
    QCheckBox *kcfg_Sunday;
    QCheckBox *kcfg_Monday;
    QWidget *tab;
    QHBoxLayout *horizontalLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *textLabel1;
    QDoubleSpinBox *kcfg_HoursPrYear;
    QLabel *textLabel2;
    QDoubleSpinBox *kcfg_HoursPrMonth;
    QSpacerItem *horizontalSpacer;
    QLabel *textLabe3;
    QDoubleSpinBox *kcfg_HoursPrWeek;
    QLabel *textLabel4;
    QDoubleSpinBox *kcfg_HoursPrDay;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPlato__ConfigWorkVacationPanel)
    {
        if (KPlato__ConfigWorkVacationPanel->objectName().isEmpty())
            KPlato__ConfigWorkVacationPanel->setObjectName("KPlato__ConfigWorkVacationPanel");
        KPlato__ConfigWorkVacationPanel->resize(624, 362);
        horizontalLayout_3 = new QHBoxLayout(KPlato__ConfigWorkVacationPanel);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        tabWidget = new QTabWidget(KPlato__ConfigWorkVacationPanel);
        tabWidget->setObjectName("tabWidget");
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        horizontalLayout_2 = new QHBoxLayout(tab_2);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        groupBox_3 = new QGroupBox(tab_2);
        groupBox_3->setObjectName("groupBox_3");
        gridLayout_4 = new QGridLayout(groupBox_3);
        gridLayout_4->setObjectName("gridLayout_4");
        kcfg_GenerateWeek = new QCheckBox(groupBox_3);
        kcfg_GenerateWeek->setObjectName("kcfg_GenerateWeek");

        gridLayout_4->addWidget(kcfg_GenerateWeek, 0, 0, 1, 1);

        kcfg_GenerateWeekChoice = new QComboBox(groupBox_3);
        kcfg_GenerateWeekChoice->addItem(QString());
        kcfg_GenerateWeekChoice->addItem(QString());
        kcfg_GenerateWeekChoice->setObjectName("kcfg_GenerateWeekChoice");

        gridLayout_4->addWidget(kcfg_GenerateWeekChoice, 0, 1, 1, 1);

        holidaysWidget = new QWidget(groupBox_3);
        holidaysWidget->setObjectName("holidaysWidget");
        gridLayout_3 = new QGridLayout(holidaysWidget);
        gridLayout_3->setSpacing(0);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, -1, 0, -1);
        region = new QComboBox(holidaysWidget);
        region->setObjectName("region");

        gridLayout_3->addWidget(region, 1, 1, 1, 1);

        kcfg_GenerateHolidays = new QCheckBox(holidaysWidget);
        kcfg_GenerateHolidays->setObjectName("kcfg_GenerateHolidays");

        gridLayout_3->addWidget(kcfg_GenerateHolidays, 0, 0, 1, 1);

        kcfg_GenerateHolidaysChoice = new QComboBox(holidaysWidget);
        kcfg_GenerateHolidaysChoice->addItem(QString());
        kcfg_GenerateHolidaysChoice->addItem(QString());
        kcfg_GenerateHolidaysChoice->addItem(QString());
        kcfg_GenerateHolidaysChoice->setObjectName("kcfg_GenerateHolidaysChoice");

        gridLayout_3->addWidget(kcfg_GenerateHolidaysChoice, 0, 1, 1, 1);

        label = new QLabel(holidaysWidget);
        label->setObjectName("label");

        gridLayout_3->addWidget(label, 1, 0, 1, 1);

        kcfg_Region = new QLineEdit(holidaysWidget);
        kcfg_Region->setObjectName("kcfg_Region");

        gridLayout_3->addWidget(kcfg_Region, 2, 1, 1, 1);


        gridLayout_4->addWidget(holidaysWidget, 1, 0, 1, 2);

        verticalSpacer_2 = new QSpacerItem(119, 106, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_4->addItem(verticalSpacer_2, 2, 1, 1, 1);


        horizontalLayout_2->addWidget(groupBox_3);

        groupBox_2 = new QGroupBox(tab_2);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout_2 = new QGridLayout(groupBox_2);
        gridLayout_2->setObjectName("gridLayout_2");
        kcfg_MondayEnd = new QTimeEdit(groupBox_2);
        kcfg_MondayEnd->setObjectName("kcfg_MondayEnd");

        gridLayout_2->addWidget(kcfg_MondayEnd, 0, 2, 1, 1);

        kcfg_ThursdayStart = new QTimeEdit(groupBox_2);
        kcfg_ThursdayStart->setObjectName("kcfg_ThursdayStart");

        gridLayout_2->addWidget(kcfg_ThursdayStart, 3, 1, 1, 1);

        kcfg_FridayEnd = new QTimeEdit(groupBox_2);
        kcfg_FridayEnd->setObjectName("kcfg_FridayEnd");

        gridLayout_2->addWidget(kcfg_FridayEnd, 4, 2, 1, 1);

        kcfg_TuesdayStart = new QTimeEdit(groupBox_2);
        kcfg_TuesdayStart->setObjectName("kcfg_TuesdayStart");

        gridLayout_2->addWidget(kcfg_TuesdayStart, 1, 1, 1, 1);

        kcfg_FridayStart = new QTimeEdit(groupBox_2);
        kcfg_FridayStart->setObjectName("kcfg_FridayStart");

        gridLayout_2->addWidget(kcfg_FridayStart, 4, 1, 1, 1);

        kcfg_SundayEnd = new QTimeEdit(groupBox_2);
        kcfg_SundayEnd->setObjectName("kcfg_SundayEnd");

        gridLayout_2->addWidget(kcfg_SundayEnd, 6, 2, 1, 1);

        kcfg_ThursdayEnd = new QTimeEdit(groupBox_2);
        kcfg_ThursdayEnd->setObjectName("kcfg_ThursdayEnd");

        gridLayout_2->addWidget(kcfg_ThursdayEnd, 3, 2, 1, 1);

        kcfg_SaturdayStart = new QTimeEdit(groupBox_2);
        kcfg_SaturdayStart->setObjectName("kcfg_SaturdayStart");

        gridLayout_2->addWidget(kcfg_SaturdayStart, 5, 1, 1, 1);

        kcfg_TuesdayEnd = new QTimeEdit(groupBox_2);
        kcfg_TuesdayEnd->setObjectName("kcfg_TuesdayEnd");

        gridLayout_2->addWidget(kcfg_TuesdayEnd, 1, 2, 1, 1);

        kcfg_MondayStart = new QTimeEdit(groupBox_2);
        kcfg_MondayStart->setObjectName("kcfg_MondayStart");

        gridLayout_2->addWidget(kcfg_MondayStart, 0, 1, 1, 1);

        kcfg_WednesdayStart = new QTimeEdit(groupBox_2);
        kcfg_WednesdayStart->setObjectName("kcfg_WednesdayStart");

        gridLayout_2->addWidget(kcfg_WednesdayStart, 2, 1, 1, 1);

        kcfg_SundayStart = new QTimeEdit(groupBox_2);
        kcfg_SundayStart->setObjectName("kcfg_SundayStart");

        gridLayout_2->addWidget(kcfg_SundayStart, 6, 1, 1, 1);

        kcfg_WednesdayEnd = new QTimeEdit(groupBox_2);
        kcfg_WednesdayEnd->setObjectName("kcfg_WednesdayEnd");

        gridLayout_2->addWidget(kcfg_WednesdayEnd, 2, 2, 1, 1);

        kcfg_SaturdayEnd = new QTimeEdit(groupBox_2);
        kcfg_SaturdayEnd->setObjectName("kcfg_SaturdayEnd");

        gridLayout_2->addWidget(kcfg_SaturdayEnd, 5, 2, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_2, 2, 3, 1, 1);

        kcfg_Tuesday = new QCheckBox(groupBox_2);
        kcfg_Tuesday->setObjectName("kcfg_Tuesday");

        gridLayout_2->addWidget(kcfg_Tuesday, 1, 0, 1, 1);

        kcfg_Wednesday = new QCheckBox(groupBox_2);
        kcfg_Wednesday->setObjectName("kcfg_Wednesday");

        gridLayout_2->addWidget(kcfg_Wednesday, 2, 0, 1, 1);

        kcfg_Thursday = new QCheckBox(groupBox_2);
        kcfg_Thursday->setObjectName("kcfg_Thursday");

        gridLayout_2->addWidget(kcfg_Thursday, 3, 0, 1, 1);

        kcfg_Friday = new QCheckBox(groupBox_2);
        kcfg_Friday->setObjectName("kcfg_Friday");

        gridLayout_2->addWidget(kcfg_Friday, 4, 0, 1, 1);

        kcfg_Saturday = new QCheckBox(groupBox_2);
        kcfg_Saturday->setObjectName("kcfg_Saturday");

        gridLayout_2->addWidget(kcfg_Saturday, 5, 0, 1, 1);

        kcfg_Sunday = new QCheckBox(groupBox_2);
        kcfg_Sunday->setObjectName("kcfg_Sunday");

        gridLayout_2->addWidget(kcfg_Sunday, 6, 0, 1, 1);

        kcfg_Monday = new QCheckBox(groupBox_2);
        kcfg_Monday->setObjectName("kcfg_Monday");

        gridLayout_2->addWidget(kcfg_Monday, 0, 0, 1, 1);


        horizontalLayout_2->addWidget(groupBox_2);

        tabWidget->addTab(tab_2, QString());
        tab = new QWidget();
        tab->setObjectName("tab");
        horizontalLayout = new QHBoxLayout(tab);
        horizontalLayout->setObjectName("horizontalLayout");
        groupBox = new QGroupBox(tab);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        textLabel1 = new QLabel(groupBox);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        kcfg_HoursPrYear = new QDoubleSpinBox(groupBox);
        kcfg_HoursPrYear->setObjectName("kcfg_HoursPrYear");
        kcfg_HoursPrYear->setMinimum(1.000000000000000);
        kcfg_HoursPrYear->setMaximum(8784.000000000000000);
        kcfg_HoursPrYear->setValue(1760.000000000000000);

        gridLayout->addWidget(kcfg_HoursPrYear, 0, 1, 1, 1);

        textLabel2 = new QLabel(groupBox);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 1, 0, 1, 1);

        kcfg_HoursPrMonth = new QDoubleSpinBox(groupBox);
        kcfg_HoursPrMonth->setObjectName("kcfg_HoursPrMonth");
        kcfg_HoursPrMonth->setMinimum(1.000000000000000);
        kcfg_HoursPrMonth->setMaximum(744.000000000000000);
        kcfg_HoursPrMonth->setSingleStep(1.000000000000000);
        kcfg_HoursPrMonth->setValue(176.000000000000000);

        gridLayout->addWidget(kcfg_HoursPrMonth, 1, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 2, 1, 1);

        textLabe3 = new QLabel(groupBox);
        textLabe3->setObjectName("textLabe3");
        textLabe3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabe3->setWordWrap(false);

        gridLayout->addWidget(textLabe3, 2, 0, 1, 1);

        kcfg_HoursPrWeek = new QDoubleSpinBox(groupBox);
        kcfg_HoursPrWeek->setObjectName("kcfg_HoursPrWeek");
        kcfg_HoursPrWeek->setMinimum(1.000000000000000);
        kcfg_HoursPrWeek->setMaximum(168.000000000000000);
        kcfg_HoursPrWeek->setValue(40.000000000000000);

        gridLayout->addWidget(kcfg_HoursPrWeek, 2, 1, 1, 1);

        textLabel4 = new QLabel(groupBox);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 3, 0, 1, 1);

        kcfg_HoursPrDay = new QDoubleSpinBox(groupBox);
        kcfg_HoursPrDay->setObjectName("kcfg_HoursPrDay");
        kcfg_HoursPrDay->setMinimum(1.000000000000000);
        kcfg_HoursPrDay->setMaximum(24.000000000000000);
        kcfg_HoursPrDay->setValue(8.000000000000000);

        gridLayout->addWidget(kcfg_HoursPrDay, 3, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 4, 1, 1, 1);


        horizontalLayout->addWidget(groupBox);

        tabWidget->addTab(tab, QString());

        horizontalLayout_3->addWidget(tabWidget);

#if QT_CONFIG(shortcut)
        label->setBuddy(region);
        textLabel1->setBuddy(kcfg_HoursPrYear);
        textLabel2->setBuddy(kcfg_HoursPrMonth);
        textLabe3->setBuddy(kcfg_HoursPrWeek);
        textLabel4->setBuddy(kcfg_HoursPrDay);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__ConfigWorkVacationPanel);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KPlato__ConfigWorkVacationPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__ConfigWorkVacationPanel)
    {
#if QT_CONFIG(whatsthis)
        groupBox_3->setWhatsThis(tr2i18n("<html><head/><body><p>Values used for generating default calendars for working weeks and holidays.</p><p>The <span style=\" font-style:italic;\">Week</span> calendar uses the values defined in <span style=\" font-style:italic;\">Working week</span>.</p><p>The <span style=\" font-style:italic;\">Week</span> calendar can be <span style=\" font-style:italic;\">Always</span> generated or only generated <span style=\" font-style:italic;\">If none exists</span>. The latter choice is the most common as you often have calendars defined as part of templates or shared resources.</p><p><span style=\" font-style:italic;\">Holidays</span> can be generated for your region (default) or any region you choose. The holidays can be added into the week calendar, as sub-calendar to the week calendar or as a separate calendar.</p><p>If a week calendar is not generated, the holidays are put into a separate calendar.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        groupBox_3->setTitle(tr2i18n("Calendar generation", nullptr));
        kcfg_GenerateWeek->setText(tr2i18n("Week", nullptr));
        kcfg_GenerateWeekChoice->setItemText(0, tr2i18n("If none exists", nullptr));
        kcfg_GenerateWeekChoice->setItemText(1, tr2i18n("Always", nullptr));

        kcfg_GenerateHolidays->setText(tr2i18n("Holidays", nullptr));
        kcfg_GenerateHolidaysChoice->setItemText(0, tr2i18n("In week calendar", nullptr));
        kcfg_GenerateHolidaysChoice->setItemText(1, tr2i18n("As sub-calendar", nullptr));
        kcfg_GenerateHolidaysChoice->setItemText(2, tr2i18n("As separate calendar", nullptr));

        label->setText(tr2i18n("Holida&y region:", nullptr));
        groupBox_2->setTitle(tr2i18n("Working week", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_Tuesday->setToolTip(tr2i18n("Tick  if working day", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_Tuesday->setText(tr2i18n("Tuesday:", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_Wednesday->setToolTip(tr2i18n("Tick if working day", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_Wednesday->setText(tr2i18n("Wednesday:", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_Thursday->setToolTip(tr2i18n("Tick if working day", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_Thursday->setText(tr2i18n("Thursday:", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_Friday->setToolTip(tr2i18n("Tick if working day", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_Friday->setText(tr2i18n("Friday:", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_Saturday->setToolTip(tr2i18n("Tick if working day", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_Saturday->setText(tr2i18n("Saturday:", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_Sunday->setToolTip(tr2i18n("Tick if working day", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_Sunday->setText(tr2i18n("Sunday:", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_Monday->setToolTip(tr2i18n("Tick  if working day", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_Monday->setText(tr2i18n("Monday:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), tr2i18n("Working Hours", nullptr));
#if QT_CONFIG(whatsthis)
        groupBox->setWhatsThis(tr2i18n("These values are used if your estimate is not in hours.\n"
"Let us say Hours per day is 8. If you estimate that a task needs 3 days effort to complete, this is converted to 24 hours when the task is scheduled. The actual time it will take to complete is of course dependent on the availability of the person (or persons) that is assigned to the task.", nullptr));
#endif // QT_CONFIG(whatsthis)
        groupBox->setTitle(QString());
        textLabel1->setText(tr2i18n("Hours &per year:", nullptr));
        textLabel2->setText(tr2i18n("Hours per &month:", nullptr));
        textLabe3->setText(tr2i18n("Hours per wee&k:", nullptr));
#if QT_CONFIG(whatsthis)
        textLabel4->setWhatsThis(tr2i18n("These values are used when you estimate the effort needed to complete a task.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel4->setText(tr2i18n("Hours per &day:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), tr2i18n("Estimate Conversions", nullptr));
        (void)KPlato__ConfigWorkVacationPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class ConfigWorkVacationPanel: public Ui_ConfigWorkVacationPanel {};
} // namespace Ui
} // namespace KPlato

#endif // CONFIGWORKVACATIONPANEL_H

