/********************************************************************************
** Form generated from reading UI file 'kptmainprojectpanelbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTMAINPROJECTPANELBASE_H
#define UI_KPTMAINPROJECTPANELBASE_H

#include <QtCore/QLocale>
#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTimeEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "klineedit.h"
#include "kurlrequester.h"

namespace KPlato {

class Ui_MainProjectPanelBase
{
public:
    QGridLayout *gridLayout_5;
    QTabWidget *tabWidget;
    QWidget *ui_general;
    QVBoxLayout *verticalLayout_7;
    QGridLayout *gridLayout;
    QLabel *wbslabel;
    QLabel *wbs;
    QLabel *textLabel1;
    KLineEdit *namefield;
    QLabel *textLabel2;
    KLineEdit *leaderfield;
    QPushButton *chooseLeader;
    QGroupBox *schedulingGroup;
    QHBoxLayout *horizontalLayout;
    QGridLayout *gridLayout_2;
    QDateEdit *startDate;
    QDateEdit *endDate;
    QLabel *label;
    QTimeEdit *endTime;
    QLabel *label_2;
    QTimeEdit *startTime;
    QLabel *label_6;
    QComboBox *ui_timezone;
    QSpacerItem *horizontalSpacer;
    QGroupBox *useSharedResources;
    QVBoxLayout *verticalLayout_3;
    QWidget *resourcesWidget;
    QGridLayout *gridLayout_3;
    QLineEdit *resourcesFile;
    QPushButton *resourcesBrowseBtn;
    QGroupBox *groupBox_3;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_7;
    QComboBox *freedays;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer;
    QWidget *ui_description;
    QVBoxLayout *verticalLayout;
    QWidget *ui_documents;
    QVBoxLayout *verticalLayout_8;
    QWidget *widget;
    QWidget *ui_workpackage;
    QVBoxLayout *verticalLayout_9;
    QGroupBox *ui_publishGroup;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_5;
    KUrlRequester *ui_PublishUrl;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_5;
    QCheckBox *ui_CheckForWorkPackages;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_3;
    KUrlRequester *ui_RetrieveUrl;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_4;
    QRadioButton *ui_DeleteFile;
    QRadioButton *ui_ArchiveFile;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_4;
    KUrlRequester *ui_ArchiveUrl;
    QSpacerItem *verticalSpacer_2;
    QWidget *ui_taskModules;
    QGridLayout *gridLayout_4;
    QToolButton *ui_removeModule;
    QSpacerItem *verticalSpacer_3;
    QToolButton *ui_insertModule;
    QTreeView *ui_taskModulesView;
    QCheckBox *ui_useLocalModules;

    void setupUi(QWidget *KPlato__MainProjectPanelBase)
    {
        if (KPlato__MainProjectPanelBase->objectName().isEmpty())
            KPlato__MainProjectPanelBase->setObjectName("KPlato__MainProjectPanelBase");
        KPlato__MainProjectPanelBase->resize(495, 523);
        KPlato__MainProjectPanelBase->setLocale(QLocale(QLocale::C, QLocale::AnyCountry));
        gridLayout_5 = new QGridLayout(KPlato__MainProjectPanelBase);
        gridLayout_5->setObjectName("gridLayout_5");
        tabWidget = new QTabWidget(KPlato__MainProjectPanelBase);
        tabWidget->setObjectName("tabWidget");
        ui_general = new QWidget();
        ui_general->setObjectName("ui_general");
        verticalLayout_7 = new QVBoxLayout(ui_general);
        verticalLayout_7->setObjectName("verticalLayout_7");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        wbslabel = new QLabel(ui_general);
        wbslabel->setObjectName("wbslabel");
        wbslabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        wbslabel->setWordWrap(false);

        gridLayout->addWidget(wbslabel, 0, 0, 1, 1);

        wbs = new QLabel(ui_general);
        wbs->setObjectName("wbs");
        wbs->setMinimumSize(QSize(20, 0));
        wbs->setFrameShape(QFrame::StyledPanel);
        wbs->setFrameShadow(QFrame::Sunken);
        wbs->setWordWrap(false);

        gridLayout->addWidget(wbs, 0, 1, 1, 2);

        textLabel1 = new QLabel(ui_general);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 1, 0, 1, 1);

        namefield = new KLineEdit(ui_general);
        namefield->setObjectName("namefield");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(namefield->sizePolicy().hasHeightForWidth());
        namefield->setSizePolicy(sizePolicy);

        gridLayout->addWidget(namefield, 1, 1, 1, 2);

        textLabel2 = new QLabel(ui_general);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 2, 0, 1, 1);

        leaderfield = new KLineEdit(ui_general);
        leaderfield->setObjectName("leaderfield");
        sizePolicy.setHeightForWidth(leaderfield->sizePolicy().hasHeightForWidth());
        leaderfield->setSizePolicy(sizePolicy);

        gridLayout->addWidget(leaderfield, 2, 1, 1, 1);

        chooseLeader = new QPushButton(ui_general);
        chooseLeader->setObjectName("chooseLeader");

        gridLayout->addWidget(chooseLeader, 2, 2, 1, 1);


        verticalLayout_7->addLayout(gridLayout);

        schedulingGroup = new QGroupBox(ui_general);
        schedulingGroup->setObjectName("schedulingGroup");
        horizontalLayout = new QHBoxLayout(schedulingGroup);
        horizontalLayout->setObjectName("horizontalLayout");
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        startDate = new QDateEdit(schedulingGroup);
        startDate->setObjectName("startDate");
        startDate->setCalendarPopup(true);

        gridLayout_2->addWidget(startDate, 0, 1, 1, 1);

        endDate = new QDateEdit(schedulingGroup);
        endDate->setObjectName("endDate");
        endDate->setCalendarPopup(true);

        gridLayout_2->addWidget(endDate, 1, 1, 1, 1);

        label = new QLabel(schedulingGroup);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label, 0, 0, 1, 1);

        endTime = new QTimeEdit(schedulingGroup);
        endTime->setObjectName("endTime");

        gridLayout_2->addWidget(endTime, 1, 2, 1, 1);

        label_2 = new QLabel(schedulingGroup);
        label_2->setObjectName("label_2");
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_2, 1, 0, 1, 1);

        startTime = new QTimeEdit(schedulingGroup);
        startTime->setObjectName("startTime");

        gridLayout_2->addWidget(startTime, 0, 2, 1, 1);

        label_6 = new QLabel(schedulingGroup);
        label_6->setObjectName("label_6");

        gridLayout_2->addWidget(label_6, 3, 0, 1, 1);

        ui_timezone = new QComboBox(schedulingGroup);
        ui_timezone->setObjectName("ui_timezone");

        gridLayout_2->addWidget(ui_timezone, 3, 1, 1, 2);


        horizontalLayout->addLayout(gridLayout_2);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_7->addWidget(schedulingGroup);

        useSharedResources = new QGroupBox(ui_general);
        useSharedResources->setObjectName("useSharedResources");
        useSharedResources->setCheckable(true);
        useSharedResources->setChecked(false);
        verticalLayout_3 = new QVBoxLayout(useSharedResources);
        verticalLayout_3->setObjectName("verticalLayout_3");
        resourcesWidget = new QWidget(useSharedResources);
        resourcesWidget->setObjectName("resourcesWidget");
        gridLayout_3 = new QGridLayout(resourcesWidget);
        gridLayout_3->setObjectName("gridLayout_3");
        resourcesFile = new QLineEdit(resourcesWidget);
        resourcesFile->setObjectName("resourcesFile");

        gridLayout_3->addWidget(resourcesFile, 0, 0, 1, 1);

        resourcesBrowseBtn = new QPushButton(resourcesWidget);
        resourcesBrowseBtn->setObjectName("resourcesBrowseBtn");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        resourcesBrowseBtn->setIcon(icon);

        gridLayout_3->addWidget(resourcesBrowseBtn, 0, 1, 1, 1);


        verticalLayout_3->addWidget(resourcesWidget);


        verticalLayout_7->addWidget(useSharedResources);

        groupBox_3 = new QGroupBox(ui_general);
        groupBox_3->setObjectName("groupBox_3");
        horizontalLayout_2 = new QHBoxLayout(groupBox_3);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_7 = new QLabel(groupBox_3);
        label_7->setObjectName("label_7");

        horizontalLayout_2->addWidget(label_7);

        freedays = new QComboBox(groupBox_3);
        freedays->addItem(QString());
        freedays->setObjectName("freedays");
        freedays->setMinimumSize(QSize(200, 0));

        horizontalLayout_2->addWidget(freedays);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout_7->addWidget(groupBox_3);

        verticalSpacer = new QSpacerItem(20, 18, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(verticalSpacer);

        tabWidget->addTab(ui_general, QString());
        ui_description = new QWidget();
        ui_description->setObjectName("ui_description");
        verticalLayout = new QVBoxLayout(ui_description);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        tabWidget->addTab(ui_description, QString());
        ui_documents = new QWidget();
        ui_documents->setObjectName("ui_documents");
        verticalLayout_8 = new QVBoxLayout(ui_documents);
        verticalLayout_8->setObjectName("verticalLayout_8");
        widget = new QWidget(ui_documents);
        widget->setObjectName("widget");
        widget->setEnabled(false);

        verticalLayout_8->addWidget(widget);

        tabWidget->addTab(ui_documents, QString());
        ui_workpackage = new QWidget();
        ui_workpackage->setObjectName("ui_workpackage");
        verticalLayout_9 = new QVBoxLayout(ui_workpackage);
        verticalLayout_9->setObjectName("verticalLayout_9");
        ui_publishGroup = new QGroupBox(ui_workpackage);
        ui_publishGroup->setObjectName("ui_publishGroup");
        verticalLayout_6 = new QVBoxLayout(ui_publishGroup);
        verticalLayout_6->setObjectName("verticalLayout_6");
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label_5 = new QLabel(ui_publishGroup);
        label_5->setObjectName("label_5");

        horizontalLayout_5->addWidget(label_5);

        ui_PublishUrl = new KUrlRequester(ui_publishGroup);
        ui_PublishUrl->setObjectName("ui_PublishUrl");

        horizontalLayout_5->addWidget(ui_PublishUrl);


        verticalLayout_6->addLayout(horizontalLayout_5);


        verticalLayout_9->addWidget(ui_publishGroup);

        groupBox_2 = new QGroupBox(ui_workpackage);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_5 = new QVBoxLayout(groupBox_2);
        verticalLayout_5->setObjectName("verticalLayout_5");
        ui_CheckForWorkPackages = new QCheckBox(groupBox_2);
        ui_CheckForWorkPackages->setObjectName("ui_CheckForWorkPackages");

        verticalLayout_5->addWidget(ui_CheckForWorkPackages);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_3 = new QLabel(groupBox_2);
        label_3->setObjectName("label_3");

        horizontalLayout_4->addWidget(label_3);

        ui_RetrieveUrl = new KUrlRequester(groupBox_2);
        ui_RetrieveUrl->setObjectName("ui_RetrieveUrl");

        horizontalLayout_4->addWidget(ui_RetrieveUrl);


        verticalLayout_5->addLayout(horizontalLayout_4);


        verticalLayout_9->addWidget(groupBox_2);

        groupBox = new QGroupBox(ui_workpackage);
        groupBox->setObjectName("groupBox");
        verticalLayout_4 = new QVBoxLayout(groupBox);
        verticalLayout_4->setObjectName("verticalLayout_4");
        ui_DeleteFile = new QRadioButton(groupBox);
        ui_DeleteFile->setObjectName("ui_DeleteFile");

        verticalLayout_4->addWidget(ui_DeleteFile);

        ui_ArchiveFile = new QRadioButton(groupBox);
        ui_ArchiveFile->setObjectName("ui_ArchiveFile");

        verticalLayout_4->addWidget(ui_ArchiveFile);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");

        horizontalLayout_3->addWidget(label_4);

        ui_ArchiveUrl = new KUrlRequester(groupBox);
        ui_ArchiveUrl->setObjectName("ui_ArchiveUrl");

        horizontalLayout_3->addWidget(ui_ArchiveUrl);


        verticalLayout_4->addLayout(horizontalLayout_3);


        verticalLayout_9->addWidget(groupBox);

        verticalSpacer_2 = new QSpacerItem(20, 170, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_9->addItem(verticalSpacer_2);

        tabWidget->addTab(ui_workpackage, QString());
        ui_taskModules = new QWidget();
        ui_taskModules->setObjectName("ui_taskModules");
        gridLayout_4 = new QGridLayout(ui_taskModules);
        gridLayout_4->setObjectName("gridLayout_4");
        gridLayout_4->setHorizontalSpacing(0);
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        ui_removeModule = new QToolButton(ui_taskModules);
        ui_removeModule->setObjectName("ui_removeModule");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        ui_removeModule->setIcon(icon1);

        gridLayout_4->addWidget(ui_removeModule, 2, 1, 1, 1);

        verticalSpacer_3 = new QSpacerItem(20, 394, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_4->addItem(verticalSpacer_3, 3, 1, 1, 1);

        ui_insertModule = new QToolButton(ui_taskModules);
        ui_insertModule->setObjectName("ui_insertModule");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        ui_insertModule->setIcon(icon2);

        gridLayout_4->addWidget(ui_insertModule, 1, 1, 1, 1);

        ui_taskModulesView = new QTreeView(ui_taskModules);
        ui_taskModulesView->setObjectName("ui_taskModulesView");
        ui_taskModulesView->setRootIsDecorated(false);
        ui_taskModulesView->setItemsExpandable(false);
        ui_taskModulesView->header()->setVisible(false);

        gridLayout_4->addWidget(ui_taskModulesView, 1, 0, 3, 1);

        ui_useLocalModules = new QCheckBox(ui_taskModules);
        ui_useLocalModules->setObjectName("ui_useLocalModules");

        gridLayout_4->addWidget(ui_useLocalModules, 0, 0, 1, 1);

        tabWidget->addTab(ui_taskModules, QString());

        gridLayout_5->addWidget(tabWidget, 0, 0, 1, 1);

#if QT_CONFIG(shortcut)
        textLabel1->setBuddy(namefield);
        textLabel2->setBuddy(leaderfield);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(namefield, leaderfield);
        QWidget::setTabOrder(leaderfield, chooseLeader);
        QWidget::setTabOrder(chooseLeader, startTime);
        QWidget::setTabOrder(startTime, endDate);
        QWidget::setTabOrder(endDate, endTime);
        QWidget::setTabOrder(endTime, tabWidget);
        QWidget::setTabOrder(tabWidget, resourcesFile);
        QWidget::setTabOrder(resourcesFile, resourcesBrowseBtn);
        QWidget::setTabOrder(resourcesBrowseBtn, ui_PublishUrl);
        QWidget::setTabOrder(ui_PublishUrl, ui_CheckForWorkPackages);
        QWidget::setTabOrder(ui_CheckForWorkPackages, ui_RetrieveUrl);
        QWidget::setTabOrder(ui_RetrieveUrl, ui_DeleteFile);
        QWidget::setTabOrder(ui_DeleteFile, ui_ArchiveFile);
        QWidget::setTabOrder(ui_ArchiveFile, ui_ArchiveUrl);
        QWidget::setTabOrder(ui_ArchiveUrl, startDate);
        QWidget::setTabOrder(startDate, ui_useLocalModules);
        QWidget::setTabOrder(ui_useLocalModules, ui_taskModulesView);
        QWidget::setTabOrder(ui_taskModulesView, ui_insertModule);
        QWidget::setTabOrder(ui_insertModule, ui_removeModule);

        retranslateUi(KPlato__MainProjectPanelBase);
        QObject::connect(useSharedResources, &QGroupBox::toggled, resourcesWidget, &QWidget::setEnabled);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KPlato__MainProjectPanelBase);
    } // setupUi

    void retranslateUi(QWidget *KPlato__MainProjectPanelBase)
    {
        wbslabel->setText(tr2i18n("WBS:", nullptr));
        wbs->setText(QString());
#if QT_CONFIG(whatsthis)
        textLabel1->setWhatsThis(tr2i18n("The project name.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1->setText(tr2i18n("&Name:", nullptr));
#if QT_CONFIG(whatsthis)
        namefield->setWhatsThis(tr2i18n("The project name.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        textLabel2->setWhatsThis(tr2i18n("The project leader.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel2->setText(tr2i18n("&Manager:", nullptr));
#if QT_CONFIG(whatsthis)
        leaderfield->setWhatsThis(tr2i18n("The project leader.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        chooseLeader->setWhatsThis(tr2i18n("Choose a project leader from your address book.", nullptr));
#endif // QT_CONFIG(whatsthis)
        chooseLeader->setText(tr2i18n("&Choose...", nullptr));
        schedulingGroup->setTitle(tr2i18n("Scheduling Range", nullptr));
        label->setText(tr2i18n("Earliest start:", nullptr));
        label_2->setText(tr2i18n("Latest finish:", nullptr));
        label_6->setText(tr2i18n("<html><head/><body><p align=\"right\">Time zone:</p></body></html>", nullptr));
        useSharedResources->setTitle(tr2i18n("&Use shared resources", nullptr));
        resourcesBrowseBtn->setText(tr2i18n("Browse...", nullptr));
        groupBox_3->setTitle(tr2i18n("Freedays", nullptr));
        label_7->setText(tr2i18n("Freedays:", nullptr));
        freedays->setItemText(0, tr2i18n("None", nullptr));

        tabWidget->setTabText(tabWidget->indexOf(ui_general), tr2i18n("General", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(ui_description), tr2i18n("Description", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(ui_documents), tr2i18n("Documents", nullptr));
        ui_publishGroup->setTitle(tr2i18n("Publish", nullptr));
        label_5->setText(tr2i18n("Publish:", nullptr));
        groupBox_2->setTitle(tr2i18n("Retrieve", nullptr));
        ui_CheckForWorkPackages->setText(tr2i18n("Check for work packages", nullptr));
        label_3->setText(tr2i18n("Retrieve:", nullptr));
        groupBox->setTitle(tr2i18n("Archive", nullptr));
        ui_DeleteFile->setText(tr2i18n("Delete file after it has been retrieved", nullptr));
        ui_ArchiveFile->setText(tr2i18n("Archive file after it has been retrieved", nullptr));
        label_4->setText(tr2i18n("Archive:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(ui_workpackage), tr2i18n("Work Package", nullptr));
        ui_removeModule->setText(tr2i18n("Remove", nullptr));
        ui_insertModule->setText(tr2i18n("Insert", nullptr));
        ui_useLocalModules->setText(tr2i18n("Use local task modules", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(ui_taskModules), tr2i18n("Task Modules", nullptr));
        (void)KPlato__MainProjectPanelBase;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class MainProjectPanelBase: public Ui_MainProjectPanelBase {};
} // namespace Ui
} // namespace KPlato

#endif // KPTMAINPROJECTPANELBASE_H

