/****************************************************************************
** Meta object code from reading C++ file 'kptresourcedialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/kptresourcedialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptresourcedialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato18ResourceDialogImplE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ResourceDialogImpl::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato18ResourceDialogImplE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ResourceDialogImpl",
        "changed",
        "",
        "calculate",
        "slotChanged",
        "slotTypeChanged",
        "slotCalculationNeeded",
        "slotChooseResource",
        "setCurrentIndexes",
        "QModelIndexList",
        "lst",
        "slotAvailableFromChanged",
        "dt",
        "slotAvailableUntilChanged",
        "slotUseRequiredChanged",
        "state",
        "slotTeamChanged",
        "QModelIndex",
        "index"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'calculate'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotChanged'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotTypeChanged'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotCalculationNeeded'
        QtMocHelpers::SlotData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotChooseResource'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setCurrentIndexes'
        QtMocHelpers::SlotData<void(const QModelIndexList &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotAvailableFromChanged'
        QtMocHelpers::SlotData<void(const QDateTime &)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QDateTime, 12 },
        }}),
        // Slot 'slotAvailableUntilChanged'
        QtMocHelpers::SlotData<void(const QDateTime &)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QDateTime, 12 },
        }}),
        // Slot 'slotUseRequiredChanged'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'slotTeamChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ResourceDialogImpl, qt_meta_tag_ZN6KPlato18ResourceDialogImplE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ResourceDialogImpl::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18ResourceDialogImplE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18ResourceDialogImplE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato18ResourceDialogImplE_t>.metaTypes,
    nullptr
} };

void KPlato::ResourceDialogImpl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResourceDialogImpl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: _t->calculate(); break;
        case 2: _t->slotChanged(); break;
        case 3: _t->slotTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->slotCalculationNeeded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->slotChooseResource(); break;
        case 6: _t->setCurrentIndexes((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 7: _t->slotAvailableFromChanged((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1]))); break;
        case 8: _t->slotAvailableUntilChanged((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1]))); break;
        case 9: _t->slotUseRequiredChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->slotTeamChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ResourceDialogImpl::*)()>(_a, &ResourceDialogImpl::changed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResourceDialogImpl::*)()>(_a, &ResourceDialogImpl::calculate, 1))
            return;
    }
}

const QMetaObject *KPlato::ResourceDialogImpl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ResourceDialogImpl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18ResourceDialogImplE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui_ResourceDialogBase"))
        return static_cast< Ui_ResourceDialogBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int KPlato::ResourceDialogImpl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ResourceDialogImpl::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KPlato::ResourceDialogImpl::calculate()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN6KPlato14ResourceDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ResourceDialog::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato14ResourceDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ResourceDialog",
        "enableButtonOk",
        "",
        "slotCalculationNeeded",
        "slotOk",
        "slotCalendarChanged",
        "slotButtonClicked",
        "button",
        "slotAccountChanged",
        "name",
        "slotResourceRemoved",
        "KPlato::Project*",
        "project",
        "row",
        "KPlato::Resource*",
        "resource"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'enableButtonOk'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCalculationNeeded'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotOk'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCalendarChanged'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotButtonClicked'
        QtMocHelpers::SlotData<void(int)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'slotAccountChanged'
        QtMocHelpers::SlotData<void(const QString &)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Slot 'slotResourceRemoved'
        QtMocHelpers::SlotData<void(KPlato::Project *, int, KPlato::Resource *)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Int, 13 }, { 0x80000000 | 14, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ResourceDialog, qt_meta_tag_ZN6KPlato14ResourceDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ResourceDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<KoDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14ResourceDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14ResourceDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato14ResourceDialogE_t>.metaTypes,
    nullptr
} };

void KPlato::ResourceDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResourceDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->enableButtonOk(); break;
        case 1: _t->slotCalculationNeeded(); break;
        case 2: _t->slotOk(); break;
        case 3: _t->slotCalendarChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->slotButtonClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->slotAccountChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->slotResourceRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Resource* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KPlato::ResourceDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ResourceDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14ResourceDialogE_t>.strings))
        return static_cast<void*>(this);
    return KoDialog::qt_metacast(_clname);
}

int KPlato::ResourceDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KoDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
