/****************************************************************************
** Meta object code from reading C++ file 'NodeGanttViewBase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/gantt/NodeGanttViewBase.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'NodeGanttViewBase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato17NodeGanttViewBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::NodeGanttViewBase::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato17NodeGanttViewBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::NodeGanttViewBase"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NodeGanttViewBase, qt_meta_tag_ZN6KPlato17NodeGanttViewBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::NodeGanttViewBase::staticMetaObject = { {
    QMetaObject::SuperData::link<GanttViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17NodeGanttViewBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17NodeGanttViewBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato17NodeGanttViewBaseE_t>.metaTypes,
    nullptr
} };

void KPlato::NodeGanttViewBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NodeGanttViewBase *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::NodeGanttViewBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::NodeGanttViewBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17NodeGanttViewBaseE_t>.strings))
        return static_cast<void*>(this);
    return GanttViewBase::qt_metacast(_clname);
}

int KPlato::NodeGanttViewBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GanttViewBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato12MyKGanttViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::MyKGanttView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato12MyKGanttViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::MyKGanttView",
        "clearDependencies",
        "",
        "createDependencies",
        "addDependency",
        "KPlato::Relation*",
        "rel",
        "removeDependency",
        "slotProjectCalculated",
        "KPlato::ScheduleManager*",
        "sm",
        "slotNodeInserted",
        "KPlato::Node*",
        "node"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'clearDependencies'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'createDependencies'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addDependency'
        QtMocHelpers::SlotData<void(KPlato::Relation *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'removeDependency'
        QtMocHelpers::SlotData<void(KPlato::Relation *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotProjectCalculated'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotNodeInserted'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MyKGanttView, qt_meta_tag_ZN6KPlato12MyKGanttViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::MyKGanttView::staticMetaObject = { {
    QMetaObject::SuperData::link<NodeGanttViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12MyKGanttViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12MyKGanttViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato12MyKGanttViewE_t>.metaTypes,
    nullptr
} };

void KPlato::MyKGanttView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MyKGanttView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clearDependencies(); break;
        case 1: _t->createDependencies(); break;
        case 2: _t->addDependency((*reinterpret_cast<std::add_pointer_t<KPlato::Relation*>>(_a[1]))); break;
        case 3: _t->removeDependency((*reinterpret_cast<std::add_pointer_t<KPlato::Relation*>>(_a[1]))); break;
        case 4: _t->slotProjectCalculated((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 5: _t->slotNodeInserted((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KPlato::MyKGanttView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::MyKGanttView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12MyKGanttViewE_t>.strings))
        return static_cast<void*>(this);
    return NodeGanttViewBase::qt_metacast(_clname);
}

int KPlato::MyKGanttView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = NodeGanttViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}
QT_WARNING_POP
