/****************************************************************************
** Meta object code from reading C++ file 'ResourceGroup.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/kernel/ResourceGroup.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ResourceGroup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato13ResourceGroupE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ResourceGroup::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato13ResourceGroupE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ResourceGroup",
        "dataChanged",
        "",
        "KPlato::ResourceGroup*",
        "group",
        "groupToBeAdded",
        "KPlato::Project*",
        "project",
        "parent",
        "row",
        "groupAdded",
        "child",
        "groupToBeRemoved",
        "groupRemoved",
        "resourceToBeAdded",
        "resourceAdded",
        "KPlato::Resource*",
        "resource",
        "resourceToBeRemoved",
        "resourceRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dataChanged'
        QtMocHelpers::SignalData<void(KPlato::ResourceGroup *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'groupToBeAdded'
        QtMocHelpers::SignalData<void(KPlato::Project *, KPlato::ResourceGroup *, int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 3, 8 }, { QMetaType::Int, 9 },
        }}),
        // Signal 'groupAdded'
        QtMocHelpers::SignalData<void(KPlato::ResourceGroup *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 11 },
        }}),
        // Signal 'groupToBeRemoved'
        QtMocHelpers::SignalData<void(KPlato::Project *, KPlato::ResourceGroup *, int, KPlato::ResourceGroup *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 3, 8 }, { QMetaType::Int, 9 }, { 0x80000000 | 3, 4 },
        }}),
        // Signal 'groupRemoved'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resourceToBeAdded'
        QtMocHelpers::SignalData<void(KPlato::ResourceGroup *, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 9 },
        }}),
        // Signal 'resourceAdded'
        QtMocHelpers::SignalData<void(KPlato::Resource *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'resourceToBeRemoved'
        QtMocHelpers::SignalData<void(KPlato::ResourceGroup *, int, KPlato::Resource *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 9 }, { 0x80000000 | 16, 17 },
        }}),
        // Signal 'resourceRemoved'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ResourceGroup, qt_meta_tag_ZN6KPlato13ResourceGroupE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ResourceGroup::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato13ResourceGroupE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato13ResourceGroupE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato13ResourceGroupE_t>.metaTypes,
    nullptr
} };

void KPlato::ResourceGroup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResourceGroup *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dataChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[1]))); break;
        case 1: _t->groupToBeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 2: _t->groupAdded((*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[1]))); break;
        case 3: _t->groupToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[4]))); break;
        case 4: _t->groupRemoved(); break;
        case 5: _t->resourceToBeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->resourceAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 7: _t->resourceToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[3]))); break;
        case 8: _t->resourceRemoved(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ResourceGroup* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ResourceGroup* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ResourceGroup* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 3:
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ResourceGroup* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ResourceGroup* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ResourceGroup* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ResourceGroup::*)(KPlato::ResourceGroup * )>(_a, &ResourceGroup::dataChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResourceGroup::*)(KPlato::Project * , KPlato::ResourceGroup * , int )>(_a, &ResourceGroup::groupToBeAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResourceGroup::*)(KPlato::ResourceGroup * )>(_a, &ResourceGroup::groupAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResourceGroup::*)(KPlato::Project * , KPlato::ResourceGroup * , int , KPlato::ResourceGroup * )>(_a, &ResourceGroup::groupToBeRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResourceGroup::*)()>(_a, &ResourceGroup::groupRemoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResourceGroup::*)(KPlato::ResourceGroup * , int )>(_a, &ResourceGroup::resourceToBeAdded, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResourceGroup::*)(KPlato::Resource * )>(_a, &ResourceGroup::resourceAdded, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResourceGroup::*)(KPlato::ResourceGroup * , int , KPlato::Resource * )>(_a, &ResourceGroup::resourceToBeRemoved, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResourceGroup::*)()>(_a, &ResourceGroup::resourceRemoved, 8))
            return;
    }
}

const QMetaObject *KPlato::ResourceGroup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ResourceGroup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato13ResourceGroupE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KPlato::ResourceGroup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ResourceGroup::dataChanged(KPlato::ResourceGroup * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::ResourceGroup::groupToBeAdded(KPlato::Project * _t1, KPlato::ResourceGroup * _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void KPlato::ResourceGroup::groupAdded(KPlato::ResourceGroup * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlato::ResourceGroup::groupToBeRemoved(KPlato::Project * _t1, KPlato::ResourceGroup * _t2, int _t3, KPlato::ResourceGroup * _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 4
void KPlato::ResourceGroup::groupRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KPlato::ResourceGroup::resourceToBeAdded(KPlato::ResourceGroup * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void KPlato::ResourceGroup::resourceAdded(KPlato::Resource * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void KPlato::ResourceGroup::resourceToBeRemoved(KPlato::ResourceGroup * _t1, int _t2, KPlato::Resource * _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3);
}

// SIGNAL 8
void KPlato::ResourceGroup::resourceRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
