/****************************************************************************
** Meta object code from reading C++ file 'ResourceUsageView.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../calligraplan-4.0.1/src/portfolio/ResourceUsageView.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ResourceUsageView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11LineDiagramE_t {};
} // unnamed namespace

template <> constexpr inline auto LineDiagram::qt_create_metaobjectdata<qt_meta_tag_ZN11LineDiagramE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LineDiagram",
        "sizeChanged",
        "",
        "QSizeF",
        "size"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sizeChanged'
        QtMocHelpers::SignalData<void(const QSizeF &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LineDiagram, qt_meta_tag_ZN11LineDiagramE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LineDiagram::staticMetaObject = { {
    QMetaObject::SuperData::link<KChart::LineDiagram::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11LineDiagramE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11LineDiagramE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11LineDiagramE_t>.metaTypes,
    nullptr
} };

void LineDiagram::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LineDiagram *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sizeChanged((*reinterpret_cast<std::add_pointer_t<QSizeF>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LineDiagram::*)(const QSizeF & )>(_a, &LineDiagram::sizeChanged, 0))
            return;
    }
}

const QMetaObject *LineDiagram::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LineDiagram::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11LineDiagramE_t>.strings))
        return static_cast<void*>(this);
    return KChart::LineDiagram::qt_metacast(_clname);
}

int LineDiagram::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KChart::LineDiagram::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void LineDiagram::sizeChanged(const QSizeF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN17ResourceUsageViewE_t {};
} // unnamed namespace

template <> constexpr inline auto ResourceUsageView::qt_create_metaobjectdata<qt_meta_tag_ZN17ResourceUsageViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ResourceUsageView",
        "setNumDays",
        "",
        "days",
        "slotCurrentIndexChanged",
        "QModelIndex",
        "current",
        "previous",
        "slotUpdateNumDays",
        "slotNumDaysChanged",
        "value",
        "updateMarker",
        "slotRestoreOverrideCursor",
        "saveSettings",
        "QDomElement&",
        "settings",
        "loadSettings",
        "KoXmlElement&"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setNumDays'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'slotCurrentIndexChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 5, 7 },
        }}),
        // Slot 'slotUpdateNumDays'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotNumDaysChanged'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'updateMarker'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRestoreOverrideCursor'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'saveSettings'
        QtMocHelpers::MethodData<void(QDomElement &) const>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Method 'loadSettings'
        QtMocHelpers::MethodData<void(KoXmlElement &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ResourceUsageView, qt_meta_tag_ZN17ResourceUsageViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ResourceUsageView::staticMetaObject = { {
    QMetaObject::SuperData::link<KoView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17ResourceUsageViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17ResourceUsageViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17ResourceUsageViewE_t>.metaTypes,
    nullptr
} };

void ResourceUsageView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResourceUsageView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setNumDays((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->slotCurrentIndexChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 2: _t->slotUpdateNumDays(); break;
        case 3: _t->slotNumDaysChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->updateMarker(); break;
        case 5: _t->slotRestoreOverrideCursor(); break;
        case 6: _t->saveSettings((*reinterpret_cast<std::add_pointer_t<QDomElement&>>(_a[1]))); break;
        case 7: _t->loadSettings((*reinterpret_cast<std::add_pointer_t<KoXmlElement&>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *ResourceUsageView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ResourceUsageView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17ResourceUsageViewE_t>.strings))
        return static_cast<void*>(this);
    return KoView::qt_metacast(_clname);
}

int ResourceUsageView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KoView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
