/********************************************************************************
** Form generated from reading UI file 'TaskSplitDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TASKSPLITDIALOG_H
#define UI_TASKSPLITDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_TaskSplitDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QSpinBox *parts;
    QSpacerItem *horizontalSpacer;
    QCheckBox *keep;
    QCheckBox *migrateEstimate;
    QCheckBox *migrateAllocations;
    QCheckBox *createDependencies;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KPlato__TaskSplitDialog)
    {
        if (KPlato__TaskSplitDialog->objectName().isEmpty())
            KPlato__TaskSplitDialog->setObjectName("KPlato__TaskSplitDialog");
        KPlato__TaskSplitDialog->resize(400, 300);
        verticalLayout = new QVBoxLayout(KPlato__TaskSplitDialog);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(KPlato__TaskSplitDialog);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        parts = new QSpinBox(KPlato__TaskSplitDialog);
        parts->setObjectName("parts");
        parts->setMinimum(2);
        parts->setValue(4);

        horizontalLayout->addWidget(parts);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        keep = new QCheckBox(KPlato__TaskSplitDialog);
        keep->setObjectName("keep");
        keep->setChecked(true);

        verticalLayout->addWidget(keep);

        migrateEstimate = new QCheckBox(KPlato__TaskSplitDialog);
        migrateEstimate->setObjectName("migrateEstimate");
        migrateEstimate->setChecked(true);

        verticalLayout->addWidget(migrateEstimate);

        migrateAllocations = new QCheckBox(KPlato__TaskSplitDialog);
        migrateAllocations->setObjectName("migrateAllocations");
        migrateAllocations->setChecked(true);

        verticalLayout->addWidget(migrateAllocations);

        createDependencies = new QCheckBox(KPlato__TaskSplitDialog);
        createDependencies->setObjectName("createDependencies");
        createDependencies->setChecked(true);

        verticalLayout->addWidget(createDependencies);

        verticalSpacer = new QSpacerItem(20, 93, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(KPlato__TaskSplitDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(KPlato__TaskSplitDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KPlato__TaskSplitDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KPlato__TaskSplitDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KPlato__TaskSplitDialog);
    } // setupUi

    void retranslateUi(QDialog *KPlato__TaskSplitDialog)
    {
        KPlato__TaskSplitDialog->setWindowTitle(tr2i18n("Dialog", nullptr));
        label->setText(tr2i18n("Number of tasks:", nullptr));
        keep->setText(tr2i18n("Keep task as summary task", nullptr));
        migrateEstimate->setText(tr2i18n("Migrate estimate", nullptr));
        migrateAllocations->setText(tr2i18n("Migrate resource allocations", nullptr));
        createDependencies->setText(tr2i18n("Create dependencies", nullptr));
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class TaskSplitDialog: public Ui_TaskSplitDialog {};
} // namespace Ui
} // namespace KPlato

#endif // TASKSPLITDIALOG_H

