/********************************************************************************
** Form generated from reading UI file 'kptcolorsconfigpanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTCOLORSCONFIGPANEL_H
#define UI_KPTCOLORSCONFIGPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "KComboBox"
#include "kcolorcombo.h"

namespace KPlato {

class Ui_ColorsConfigPanel
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label_9;
    KComboBox *kcfg_ColorGradientType;
    QSpacerItem *horizontalSpacer;
    QTabWidget *ui_colors;
    QWidget *tab;
    QGridLayout *gridLayout_4;
    QLabel *label;
    KColorCombo *kcfg_SummaryTaskDefaultColor;
    QGroupBox *kcfg_SummaryTaskLevelColorsEnabled;
    QGridLayout *gridLayout;
    QLabel *label_3;
    KColorCombo *kcfg_SummaryTaskLevelColor_1;
    QLabel *label_4;
    KColorCombo *kcfg_SummaryTaskLevelColor_2;
    QLabel *label_5;
    KColorCombo *kcfg_SummaryTaskLevelColor_3;
    QLabel *label_10;
    KColorCombo *kcfg_SummaryTaskLevelColor_4;
    QFrame *line_2;
    QWidget *tab_2;
    QGridLayout *gridLayout_2;
    QLabel *label_2;
    KColorCombo *kcfg_TaskNormalColor;
    QLabel *label_8;
    KColorCombo *kcfg_TaskCriticalColor;
    QLabel *label_7;
    KColorCombo *kcfg_TaskFinishedColor;
    QLabel *label_6;
    KColorCombo *kcfg_TaskErrorColor;
    QFrame *line;
    QWidget *tab_3;
    QGridLayout *gridLayout_3;
    QLabel *label_12;
    KColorCombo *kcfg_MilestoneNormalColor;
    QLabel *label_14;
    KColorCombo *kcfg_MilestoneCriticalColor;
    QLabel *label_13;
    KColorCombo *kcfg_MilestoneFinishedColor;
    QLabel *label_11;
    KColorCombo *kcfg_MilestoneErrorColor;
    QFrame *line_3;

    void setupUi(QWidget *KPlato__ColorsConfigPanel)
    {
        if (KPlato__ColorsConfigPanel->objectName().isEmpty())
            KPlato__ColorsConfigPanel->setObjectName("KPlato__ColorsConfigPanel");
        KPlato__ColorsConfigPanel->resize(402, 289);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(KPlato__ColorsConfigPanel->sizePolicy().hasHeightForWidth());
        KPlato__ColorsConfigPanel->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(KPlato__ColorsConfigPanel);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label_9 = new QLabel(KPlato__ColorsConfigPanel);
        label_9->setObjectName("label_9");

        horizontalLayout->addWidget(label_9);

        kcfg_ColorGradientType = new KComboBox(KPlato__ColorsConfigPanel);
        kcfg_ColorGradientType->setObjectName("kcfg_ColorGradientType");

        horizontalLayout->addWidget(kcfg_ColorGradientType);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        ui_colors = new QTabWidget(KPlato__ColorsConfigPanel);
        ui_colors->setObjectName("ui_colors");
        tab = new QWidget();
        tab->setObjectName("tab");
        gridLayout_4 = new QGridLayout(tab);
        gridLayout_4->setObjectName("gridLayout_4");
        label = new QLabel(tab);
        label->setObjectName("label");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);

        gridLayout_4->addWidget(label, 0, 0, 1, 1);

        kcfg_SummaryTaskDefaultColor = new KColorCombo(tab);
        kcfg_SummaryTaskDefaultColor->setObjectName("kcfg_SummaryTaskDefaultColor");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(kcfg_SummaryTaskDefaultColor->sizePolicy().hasHeightForWidth());
        kcfg_SummaryTaskDefaultColor->setSizePolicy(sizePolicy2);
        kcfg_SummaryTaskDefaultColor->setProperty("kcfg_property", QVariant(QByteArray("color")));

        gridLayout_4->addWidget(kcfg_SummaryTaskDefaultColor, 0, 1, 1, 1);

        kcfg_SummaryTaskLevelColorsEnabled = new QGroupBox(tab);
        kcfg_SummaryTaskLevelColorsEnabled->setObjectName("kcfg_SummaryTaskLevelColorsEnabled");
        kcfg_SummaryTaskLevelColorsEnabled->setCheckable(true);
        kcfg_SummaryTaskLevelColorsEnabled->setChecked(false);
        gridLayout = new QGridLayout(kcfg_SummaryTaskLevelColorsEnabled);
        gridLayout->setObjectName("gridLayout");
        label_3 = new QLabel(kcfg_SummaryTaskLevelColorsEnabled);
        label_3->setObjectName("label_3");
        sizePolicy1.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        kcfg_SummaryTaskLevelColor_1 = new KColorCombo(kcfg_SummaryTaskLevelColorsEnabled);
        kcfg_SummaryTaskLevelColor_1->setObjectName("kcfg_SummaryTaskLevelColor_1");
        sizePolicy2.setHeightForWidth(kcfg_SummaryTaskLevelColor_1->sizePolicy().hasHeightForWidth());
        kcfg_SummaryTaskLevelColor_1->setSizePolicy(sizePolicy2);
        kcfg_SummaryTaskLevelColor_1->setProperty("kcfg_property", QVariant(QByteArray("color")));

        gridLayout->addWidget(kcfg_SummaryTaskLevelColor_1, 0, 1, 1, 1);

        label_4 = new QLabel(kcfg_SummaryTaskLevelColorsEnabled);
        label_4->setObjectName("label_4");
        sizePolicy1.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(label_4, 1, 0, 1, 1);

        kcfg_SummaryTaskLevelColor_2 = new KColorCombo(kcfg_SummaryTaskLevelColorsEnabled);
        kcfg_SummaryTaskLevelColor_2->setObjectName("kcfg_SummaryTaskLevelColor_2");
        sizePolicy2.setHeightForWidth(kcfg_SummaryTaskLevelColor_2->sizePolicy().hasHeightForWidth());
        kcfg_SummaryTaskLevelColor_2->setSizePolicy(sizePolicy2);
        kcfg_SummaryTaskLevelColor_2->setProperty("kcfg_property", QVariant(QByteArray("color")));

        gridLayout->addWidget(kcfg_SummaryTaskLevelColor_2, 1, 1, 1, 1);

        label_5 = new QLabel(kcfg_SummaryTaskLevelColorsEnabled);
        label_5->setObjectName("label_5");
        sizePolicy1.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(label_5, 2, 0, 1, 1);

        kcfg_SummaryTaskLevelColor_3 = new KColorCombo(kcfg_SummaryTaskLevelColorsEnabled);
        kcfg_SummaryTaskLevelColor_3->setObjectName("kcfg_SummaryTaskLevelColor_3");
        sizePolicy2.setHeightForWidth(kcfg_SummaryTaskLevelColor_3->sizePolicy().hasHeightForWidth());
        kcfg_SummaryTaskLevelColor_3->setSizePolicy(sizePolicy2);
        kcfg_SummaryTaskLevelColor_3->setProperty("kcfg_property", QVariant(QByteArray("color")));

        gridLayout->addWidget(kcfg_SummaryTaskLevelColor_3, 2, 1, 1, 1);

        label_10 = new QLabel(kcfg_SummaryTaskLevelColorsEnabled);
        label_10->setObjectName("label_10");
        sizePolicy1.setHeightForWidth(label_10->sizePolicy().hasHeightForWidth());
        label_10->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(label_10, 3, 0, 1, 1);

        kcfg_SummaryTaskLevelColor_4 = new KColorCombo(kcfg_SummaryTaskLevelColorsEnabled);
        kcfg_SummaryTaskLevelColor_4->setObjectName("kcfg_SummaryTaskLevelColor_4");
        sizePolicy2.setHeightForWidth(kcfg_SummaryTaskLevelColor_4->sizePolicy().hasHeightForWidth());
        kcfg_SummaryTaskLevelColor_4->setSizePolicy(sizePolicy2);
        kcfg_SummaryTaskLevelColor_4->setProperty("kcfg_property", QVariant(QByteArray("color")));

        gridLayout->addWidget(kcfg_SummaryTaskLevelColor_4, 3, 1, 1, 1);


        gridLayout_4->addWidget(kcfg_SummaryTaskLevelColorsEnabled, 1, 0, 1, 2);

        line_2 = new QFrame(tab);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::VLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout_4->addWidget(line_2, 2, 0, 1, 1);

        ui_colors->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        gridLayout_2 = new QGridLayout(tab_2);
        gridLayout_2->setObjectName("gridLayout_2");
        label_2 = new QLabel(tab_2);
        label_2->setObjectName("label_2");
        sizePolicy1.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy1);
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_2, 0, 0, 1, 1);

        kcfg_TaskNormalColor = new KColorCombo(tab_2);
        kcfg_TaskNormalColor->setObjectName("kcfg_TaskNormalColor");
        sizePolicy2.setHeightForWidth(kcfg_TaskNormalColor->sizePolicy().hasHeightForWidth());
        kcfg_TaskNormalColor->setSizePolicy(sizePolicy2);
        kcfg_TaskNormalColor->setProperty("kcfg_property", QVariant(QByteArray("color")));

        gridLayout_2->addWidget(kcfg_TaskNormalColor, 0, 1, 1, 1);

        label_8 = new QLabel(tab_2);
        label_8->setObjectName("label_8");
        sizePolicy1.setHeightForWidth(label_8->sizePolicy().hasHeightForWidth());
        label_8->setSizePolicy(sizePolicy1);
        label_8->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_8, 1, 0, 1, 1);

        kcfg_TaskCriticalColor = new KColorCombo(tab_2);
        kcfg_TaskCriticalColor->setObjectName("kcfg_TaskCriticalColor");
        sizePolicy2.setHeightForWidth(kcfg_TaskCriticalColor->sizePolicy().hasHeightForWidth());
        kcfg_TaskCriticalColor->setSizePolicy(sizePolicy2);
        kcfg_TaskCriticalColor->setProperty("kcfg_property", QVariant(QByteArray("color")));

        gridLayout_2->addWidget(kcfg_TaskCriticalColor, 1, 1, 1, 1);

        label_7 = new QLabel(tab_2);
        label_7->setObjectName("label_7");
        sizePolicy1.setHeightForWidth(label_7->sizePolicy().hasHeightForWidth());
        label_7->setSizePolicy(sizePolicy1);
        label_7->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_7, 2, 0, 1, 1);

        kcfg_TaskFinishedColor = new KColorCombo(tab_2);
        kcfg_TaskFinishedColor->setObjectName("kcfg_TaskFinishedColor");
        sizePolicy2.setHeightForWidth(kcfg_TaskFinishedColor->sizePolicy().hasHeightForWidth());
        kcfg_TaskFinishedColor->setSizePolicy(sizePolicy2);
        kcfg_TaskFinishedColor->setProperty("kcfg_property", QVariant(QByteArray("color")));

        gridLayout_2->addWidget(kcfg_TaskFinishedColor, 2, 1, 1, 1);

        label_6 = new QLabel(tab_2);
        label_6->setObjectName("label_6");
        sizePolicy1.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy1);
        label_6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_6, 3, 0, 1, 1);

        kcfg_TaskErrorColor = new KColorCombo(tab_2);
        kcfg_TaskErrorColor->setObjectName("kcfg_TaskErrorColor");
        sizePolicy2.setHeightForWidth(kcfg_TaskErrorColor->sizePolicy().hasHeightForWidth());
        kcfg_TaskErrorColor->setSizePolicy(sizePolicy2);
        kcfg_TaskErrorColor->setProperty("kcfg_property", QVariant(QByteArray("color")));

        gridLayout_2->addWidget(kcfg_TaskErrorColor, 3, 1, 1, 1);

        line = new QFrame(tab_2);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::VLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout_2->addWidget(line, 4, 1, 1, 1);

        ui_colors->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        gridLayout_3 = new QGridLayout(tab_3);
        gridLayout_3->setObjectName("gridLayout_3");
        label_12 = new QLabel(tab_3);
        label_12->setObjectName("label_12");
        sizePolicy1.setHeightForWidth(label_12->sizePolicy().hasHeightForWidth());
        label_12->setSizePolicy(sizePolicy1);
        label_12->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(label_12, 0, 0, 1, 1);

        kcfg_MilestoneNormalColor = new KColorCombo(tab_3);
        kcfg_MilestoneNormalColor->setObjectName("kcfg_MilestoneNormalColor");
        sizePolicy2.setHeightForWidth(kcfg_MilestoneNormalColor->sizePolicy().hasHeightForWidth());
        kcfg_MilestoneNormalColor->setSizePolicy(sizePolicy2);
        kcfg_MilestoneNormalColor->setProperty("kcfg_property", QVariant(QByteArray("color")));

        gridLayout_3->addWidget(kcfg_MilestoneNormalColor, 0, 1, 1, 1);

        label_14 = new QLabel(tab_3);
        label_14->setObjectName("label_14");
        sizePolicy1.setHeightForWidth(label_14->sizePolicy().hasHeightForWidth());
        label_14->setSizePolicy(sizePolicy1);
        label_14->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(label_14, 1, 0, 1, 1);

        kcfg_MilestoneCriticalColor = new KColorCombo(tab_3);
        kcfg_MilestoneCriticalColor->setObjectName("kcfg_MilestoneCriticalColor");
        sizePolicy2.setHeightForWidth(kcfg_MilestoneCriticalColor->sizePolicy().hasHeightForWidth());
        kcfg_MilestoneCriticalColor->setSizePolicy(sizePolicy2);
        kcfg_MilestoneCriticalColor->setProperty("kcfg_property", QVariant(QByteArray("color")));

        gridLayout_3->addWidget(kcfg_MilestoneCriticalColor, 1, 1, 1, 1);

        label_13 = new QLabel(tab_3);
        label_13->setObjectName("label_13");
        sizePolicy1.setHeightForWidth(label_13->sizePolicy().hasHeightForWidth());
        label_13->setSizePolicy(sizePolicy1);
        label_13->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(label_13, 2, 0, 1, 1);

        kcfg_MilestoneFinishedColor = new KColorCombo(tab_3);
        kcfg_MilestoneFinishedColor->setObjectName("kcfg_MilestoneFinishedColor");
        sizePolicy2.setHeightForWidth(kcfg_MilestoneFinishedColor->sizePolicy().hasHeightForWidth());
        kcfg_MilestoneFinishedColor->setSizePolicy(sizePolicy2);
        kcfg_MilestoneFinishedColor->setProperty("kcfg_property", QVariant(QByteArray("color")));

        gridLayout_3->addWidget(kcfg_MilestoneFinishedColor, 2, 1, 1, 1);

        label_11 = new QLabel(tab_3);
        label_11->setObjectName("label_11");
        sizePolicy1.setHeightForWidth(label_11->sizePolicy().hasHeightForWidth());
        label_11->setSizePolicy(sizePolicy1);
        label_11->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(label_11, 3, 0, 1, 1);

        kcfg_MilestoneErrorColor = new KColorCombo(tab_3);
        kcfg_MilestoneErrorColor->setObjectName("kcfg_MilestoneErrorColor");
        sizePolicy2.setHeightForWidth(kcfg_MilestoneErrorColor->sizePolicy().hasHeightForWidth());
        kcfg_MilestoneErrorColor->setSizePolicy(sizePolicy2);
        kcfg_MilestoneErrorColor->setProperty("kcfg_property", QVariant(QByteArray("color")));

        gridLayout_3->addWidget(kcfg_MilestoneErrorColor, 3, 1, 1, 1);

        line_3 = new QFrame(tab_3);
        line_3->setObjectName("line_3");
        line_3->setFrameShape(QFrame::Shape::VLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout_3->addWidget(line_3, 4, 1, 1, 1);

        ui_colors->addTab(tab_3, QString());

        verticalLayout->addWidget(ui_colors);

#if QT_CONFIG(shortcut)
        label->setBuddy(kcfg_SummaryTaskDefaultColor);
        label_3->setBuddy(kcfg_SummaryTaskDefaultColor);
        label_4->setBuddy(kcfg_SummaryTaskDefaultColor);
        label_5->setBuddy(kcfg_SummaryTaskDefaultColor);
        label_10->setBuddy(kcfg_SummaryTaskDefaultColor);
        label_2->setBuddy(kcfg_TaskNormalColor);
        label_8->setBuddy(kcfg_TaskCriticalColor);
        label_7->setBuddy(kcfg_TaskFinishedColor);
        label_6->setBuddy(kcfg_TaskErrorColor);
        label_12->setBuddy(kcfg_MilestoneNormalColor);
        label_14->setBuddy(kcfg_TaskCriticalColor);
        label_13->setBuddy(kcfg_TaskFinishedColor);
        label_11->setBuddy(kcfg_TaskErrorColor);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__ColorsConfigPanel);

        ui_colors->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KPlato__ColorsConfigPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__ColorsConfigPanel)
    {
        label_9->setText(tr2i18n("Gradient:", nullptr));
        label->setText(tr2i18n("Default:", "@label:chooser"));
        kcfg_SummaryTaskLevelColorsEnabled->setTitle(tr2i18n("Use level colors", "@title:group"));
        label_3->setText(tr2i18n("Level 1", "@label:chooser"));
        label_4->setText(tr2i18n("Level 2", "@label:chooser"));
        label_5->setText(tr2i18n("Level 3", "@label:chooser"));
        label_10->setText(tr2i18n("Level 4", "@label:chooser"));
        ui_colors->setTabText(ui_colors->indexOf(tab), tr2i18n("Summary task", "@title:tab"));
        label_2->setText(tr2i18n("Normal:", "@label:chooser"));
        label_8->setText(tr2i18n("Critical:", "@label:chooser"));
        label_7->setText(tr2i18n("Finished:", "@label:chooser"));
        label_6->setText(tr2i18n("Error:", "@label:chooser"));
        ui_colors->setTabText(ui_colors->indexOf(tab_2), tr2i18n("Task", "@title:tab"));
        label_12->setText(tr2i18n("Normal:", "@label:chooser"));
        label_14->setText(tr2i18n("Critical:", "@label:chooser"));
        label_13->setText(tr2i18n("Finished:", "@label:chooser"));
        label_11->setText(tr2i18n("Error:", "@label:chooser"));
        ui_colors->setTabText(ui_colors->indexOf(tab_3), tr2i18n("Milestone", "@title:tab"));
        (void)KPlato__ColorsConfigPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class ColorsConfigPanel: public Ui_ColorsConfigPanel {};
} // namespace Ui
} // namespace KPlato

#endif // KPTCOLORSCONFIGPANEL_H

