/********************************************************************************
** Form generated from reading UI file 'kptviewlistaddview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTVIEWLISTADDVIEW_H
#define UI_KPTVIEWLISTADDVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "KComboBox"

namespace KPlato {

class Ui_AddViewPanel
{
public:
    QGridLayout *gridLayout;
    QLabel *categoryLabel;
    KComboBox *category;
    QLabel *afterLabel;
    KComboBox *insertAfter;
    QLabel *viewtypeLabel;
    KComboBox *viewtype;
    QLabel *viewnameLabel;
    QLineEdit *viewname;
    QLabel *tooltipLabel;
    QLineEdit *tooltip;

    void setupUi(QWidget *KPlato__AddViewPanel)
    {
        if (KPlato__AddViewPanel->objectName().isEmpty())
            KPlato__AddViewPanel->setObjectName("KPlato__AddViewPanel");
        KPlato__AddViewPanel->resize(400, 184);
        gridLayout = new QGridLayout(KPlato__AddViewPanel);
        gridLayout->setObjectName("gridLayout");
        categoryLabel = new QLabel(KPlato__AddViewPanel);
        categoryLabel->setObjectName("categoryLabel");

        gridLayout->addWidget(categoryLabel, 0, 0, 1, 1);

        category = new KComboBox(KPlato__AddViewPanel);
        category->setObjectName("category");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(category->sizePolicy().hasHeightForWidth());
        category->setSizePolicy(sizePolicy);
        category->setEditable(true);
        category->setInsertPolicy(QComboBox::InsertAlphabetically);
        category->setAutoCompletion(false);

        gridLayout->addWidget(category, 0, 1, 1, 1);

        afterLabel = new QLabel(KPlato__AddViewPanel);
        afterLabel->setObjectName("afterLabel");

        gridLayout->addWidget(afterLabel, 1, 0, 1, 1);

        insertAfter = new KComboBox(KPlato__AddViewPanel);
        insertAfter->setObjectName("insertAfter");
        insertAfter->setInsertPolicy(QComboBox::InsertAlphabetically);
        insertAfter->setAutoCompletion(false);

        gridLayout->addWidget(insertAfter, 1, 1, 1, 1);

        viewtypeLabel = new QLabel(KPlato__AddViewPanel);
        viewtypeLabel->setObjectName("viewtypeLabel");

        gridLayout->addWidget(viewtypeLabel, 2, 0, 1, 1);

        viewtype = new KComboBox(KPlato__AddViewPanel);
        viewtype->setObjectName("viewtype");
        viewtype->setAutoCompletion(false);

        gridLayout->addWidget(viewtype, 2, 1, 1, 1);

        viewnameLabel = new QLabel(KPlato__AddViewPanel);
        viewnameLabel->setObjectName("viewnameLabel");

        gridLayout->addWidget(viewnameLabel, 3, 0, 1, 1);

        viewname = new QLineEdit(KPlato__AddViewPanel);
        viewname->setObjectName("viewname");

        gridLayout->addWidget(viewname, 3, 1, 1, 1);

        tooltipLabel = new QLabel(KPlato__AddViewPanel);
        tooltipLabel->setObjectName("tooltipLabel");

        gridLayout->addWidget(tooltipLabel, 4, 0, 1, 1);

        tooltip = new QLineEdit(KPlato__AddViewPanel);
        tooltip->setObjectName("tooltip");

        gridLayout->addWidget(tooltip, 4, 1, 1, 1);

#if QT_CONFIG(shortcut)
        categoryLabel->setBuddy(category);
        afterLabel->setBuddy(insertAfter);
        viewtypeLabel->setBuddy(viewtype);
        viewnameLabel->setBuddy(viewname);
        tooltipLabel->setBuddy(tooltip);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__AddViewPanel);

        QMetaObject::connectSlotsByName(KPlato__AddViewPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__AddViewPanel)
    {
#if QT_CONFIG(tooltip)
        categoryLabel->setToolTip(tr2i18n("The category of the view", nullptr));
#endif // QT_CONFIG(tooltip)
        categoryLabel->setText(tr2i18n("Category:", nullptr));
#if QT_CONFIG(tooltip)
        afterLabel->setToolTip(tr2i18n("The view is placed after this view", nullptr));
#endif // QT_CONFIG(tooltip)
        afterLabel->setText(tr2i18n("After", nullptr));
#if QT_CONFIG(tooltip)
        viewtypeLabel->setToolTip(tr2i18n("The type of view", nullptr));
#endif // QT_CONFIG(tooltip)
        viewtypeLabel->setText(tr2i18n("View type:", nullptr));
#if QT_CONFIG(tooltip)
        viewnameLabel->setToolTip(tr2i18n("The name of the view", nullptr));
#endif // QT_CONFIG(tooltip)
        viewnameLabel->setText(tr2i18n("Name:", nullptr));
#if QT_CONFIG(tooltip)
        tooltipLabel->setToolTip(tr2i18n("The views description", nullptr));
#endif // QT_CONFIG(tooltip)
        tooltipLabel->setText(tr2i18n("Description:", nullptr));
        (void)KPlato__AddViewPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class AddViewPanel: public Ui_AddViewPanel {};
} // namespace Ui
} // namespace KPlato

#endif // KPTVIEWLISTADDVIEW_H

