/****************************************************************************
** Meta object code from reading C++ file 'kptcalendar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/kernel/kptcalendar.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptcalendar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato8CalendarE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::Calendar::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato8CalendarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::Calendar",
        "calendarChanged",
        "",
        "KPlato::Calendar*",
        "calendarDayChanged",
        "KPlato::CalendarDay*",
        "timeIntervalChanged",
        "KPlato::TimeInterval*",
        "weekdayToBeAdded",
        "day",
        "index",
        "weekdayAdded",
        "weekdayToBeRemoved",
        "weekdayRemoved",
        "dayToBeAdded",
        "dayAdded",
        "dayToBeRemoved",
        "dayRemoved",
        "workIntervalToBeAdded",
        "workIntervalAdded",
        "workIntervalToBeRemoved",
        "workIntervalRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'calendarChanged'
        QtMocHelpers::SignalData<void(KPlato::Calendar *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'calendarDayChanged'
        QtMocHelpers::SignalData<void(KPlato::CalendarDay *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'timeIntervalChanged'
        QtMocHelpers::SignalData<void(KPlato::TimeInterval *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'weekdayToBeAdded'
        QtMocHelpers::SignalData<void(KPlato::CalendarDay *, int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 9 }, { QMetaType::Int, 10 },
        }}),
        // Signal 'weekdayAdded'
        QtMocHelpers::SignalData<void(KPlato::CalendarDay *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
        // Signal 'weekdayToBeRemoved'
        QtMocHelpers::SignalData<void(KPlato::CalendarDay *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
        // Signal 'weekdayRemoved'
        QtMocHelpers::SignalData<void(KPlato::CalendarDay *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
        // Signal 'dayToBeAdded'
        QtMocHelpers::SignalData<void(KPlato::CalendarDay *, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 9 }, { QMetaType::Int, 10 },
        }}),
        // Signal 'dayAdded'
        QtMocHelpers::SignalData<void(KPlato::CalendarDay *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
        // Signal 'dayToBeRemoved'
        QtMocHelpers::SignalData<void(KPlato::CalendarDay *)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
        // Signal 'dayRemoved'
        QtMocHelpers::SignalData<void(KPlato::CalendarDay *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 9 },
        }}),
        // Signal 'workIntervalToBeAdded'
        QtMocHelpers::SignalData<void(KPlato::CalendarDay *, KPlato::TimeInterval *, int)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 }, { 0x80000000 | 7, 2 }, { QMetaType::Int, 10 },
        }}),
        // Signal 'workIntervalAdded'
        QtMocHelpers::SignalData<void(KPlato::CalendarDay *, KPlato::TimeInterval *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 }, { 0x80000000 | 7, 2 },
        }}),
        // Signal 'workIntervalToBeRemoved'
        QtMocHelpers::SignalData<void(KPlato::CalendarDay *, KPlato::TimeInterval *)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 }, { 0x80000000 | 7, 2 },
        }}),
        // Signal 'workIntervalRemoved'
        QtMocHelpers::SignalData<void(KPlato::CalendarDay *, KPlato::TimeInterval *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 }, { 0x80000000 | 7, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Calendar, qt_meta_tag_ZN6KPlato8CalendarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::Calendar::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8CalendarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8CalendarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato8CalendarE_t>.metaTypes,
    nullptr
} };

void KPlato::Calendar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Calendar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->calendarChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Calendar*>>(_a[1]))); break;
        case 1: _t->calendarDayChanged((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1]))); break;
        case 2: _t->timeIntervalChanged((*reinterpret_cast<std::add_pointer_t<KPlato::TimeInterval*>>(_a[1]))); break;
        case 3: _t->weekdayToBeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->weekdayAdded((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1]))); break;
        case 5: _t->weekdayToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1]))); break;
        case 6: _t->weekdayRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1]))); break;
        case 7: _t->dayToBeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->dayAdded((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1]))); break;
        case 9: _t->dayToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1]))); break;
        case 10: _t->dayRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1]))); break;
        case 11: _t->workIntervalToBeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::TimeInterval*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 12: _t->workIntervalAdded((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::TimeInterval*>>(_a[2]))); break;
        case 13: _t->workIntervalToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::TimeInterval*>>(_a[2]))); break;
        case 14: _t->workIntervalRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::CalendarDay*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::TimeInterval*>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Calendar* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::Calendar * )>(_a, &Calendar::calendarChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::CalendarDay * )>(_a, &Calendar::calendarDayChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::TimeInterval * )>(_a, &Calendar::timeIntervalChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::CalendarDay * , int )>(_a, &Calendar::weekdayToBeAdded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::CalendarDay * )>(_a, &Calendar::weekdayAdded, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::CalendarDay * )>(_a, &Calendar::weekdayToBeRemoved, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::CalendarDay * )>(_a, &Calendar::weekdayRemoved, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::CalendarDay * , int )>(_a, &Calendar::dayToBeAdded, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::CalendarDay * )>(_a, &Calendar::dayAdded, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::CalendarDay * )>(_a, &Calendar::dayToBeRemoved, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::CalendarDay * )>(_a, &Calendar::dayRemoved, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::CalendarDay * , KPlato::TimeInterval * , int )>(_a, &Calendar::workIntervalToBeAdded, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::CalendarDay * , KPlato::TimeInterval * )>(_a, &Calendar::workIntervalAdded, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::CalendarDay * , KPlato::TimeInterval * )>(_a, &Calendar::workIntervalToBeRemoved, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendar::*)(KPlato::CalendarDay * , KPlato::TimeInterval * )>(_a, &Calendar::workIntervalRemoved, 14))
            return;
    }
}

const QMetaObject *KPlato::Calendar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::Calendar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8CalendarE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KPlato::Calendar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void KPlato::Calendar::calendarChanged(KPlato::Calendar * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::Calendar::calendarDayChanged(KPlato::CalendarDay * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KPlato::Calendar::timeIntervalChanged(KPlato::TimeInterval * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlato::Calendar::weekdayToBeAdded(KPlato::CalendarDay * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void KPlato::Calendar::weekdayAdded(KPlato::CalendarDay * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KPlato::Calendar::weekdayToBeRemoved(KPlato::CalendarDay * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KPlato::Calendar::weekdayRemoved(KPlato::CalendarDay * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void KPlato::Calendar::dayToBeAdded(KPlato::CalendarDay * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void KPlato::Calendar::dayAdded(KPlato::CalendarDay * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void KPlato::Calendar::dayToBeRemoved(KPlato::CalendarDay * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void KPlato::Calendar::dayRemoved(KPlato::CalendarDay * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void KPlato::Calendar::workIntervalToBeAdded(KPlato::CalendarDay * _t1, KPlato::TimeInterval * _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2, _t3);
}

// SIGNAL 12
void KPlato::Calendar::workIntervalAdded(KPlato::CalendarDay * _t1, KPlato::TimeInterval * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2);
}

// SIGNAL 13
void KPlato::Calendar::workIntervalToBeRemoved(KPlato::CalendarDay * _t1, KPlato::TimeInterval * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2);
}

// SIGNAL 14
void KPlato::Calendar::workIntervalRemoved(KPlato::CalendarDay * _t1, KPlato::TimeInterval * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1, _t2);
}
QT_WARNING_POP
