/********************************************************************************
** Form generated from reading UI file 'kptcpmwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTCPMWIDGET_H
#define UI_KPTCPMWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kptitemmodelbase.h"

namespace KPlato {

class Ui_CpmWidget
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *label_1;
    QLabel *scheduleName;
    QFrame *probabilityFrame;
    QHBoxLayout *hboxLayout1;
    QLabel *label;
    QDateTimeEdit *finishTime;
    QLabel *label_2;
    QSpinBox *probability;
    QSpacerItem *spacerItem;
    DoubleTreeViewBase *cpmTable;

    void setupUi(QWidget *KPlato__CpmWidget)
    {
        if (KPlato__CpmWidget->objectName().isEmpty())
            KPlato__CpmWidget->setObjectName("KPlato__CpmWidget");
        KPlato__CpmWidget->resize(484, 150);
        vboxLayout = new QVBoxLayout(KPlato__CpmWidget);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        label_1 = new QLabel(KPlato__CpmWidget);
        label_1->setObjectName("label_1");
        label_1->setFrameShape(QFrame::NoFrame);
        label_1->setFrameShadow(QFrame::Plain);
        label_1->setMargin(1);
        label_1->setIndent(0);

        hboxLayout->addWidget(label_1);

        scheduleName = new QLabel(KPlato__CpmWidget);
        scheduleName->setObjectName("scheduleName");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(scheduleName->sizePolicy().hasHeightForWidth());
        scheduleName->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(scheduleName);

        probabilityFrame = new QFrame(KPlato__CpmWidget);
        probabilityFrame->setObjectName("probabilityFrame");
        probabilityFrame->setFrameShape(QFrame::NoFrame);
        probabilityFrame->setFrameShadow(QFrame::Raised);
        hboxLayout1 = new QHBoxLayout(probabilityFrame);
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(probabilityFrame);
        label->setObjectName("label");

        hboxLayout1->addWidget(label);

        finishTime = new QDateTimeEdit(probabilityFrame);
        finishTime->setObjectName("finishTime");
        finishTime->setCalendarPopup(true);

        hboxLayout1->addWidget(finishTime);

        label_2 = new QLabel(probabilityFrame);
        label_2->setObjectName("label_2");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy1);

        hboxLayout1->addWidget(label_2);

        probability = new QSpinBox(probabilityFrame);
        probability->setObjectName("probability");
        probability->setMinimum(1);

        hboxLayout1->addWidget(probability);


        hboxLayout->addWidget(probabilityFrame);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);


        vboxLayout->addLayout(hboxLayout);

        cpmTable = new DoubleTreeViewBase(KPlato__CpmWidget);
        cpmTable->setObjectName("cpmTable");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(cpmTable->sizePolicy().hasHeightForWidth());
        cpmTable->setSizePolicy(sizePolicy2);
        cpmTable->setFrameShape(QFrame::NoFrame);
        cpmTable->setFrameShadow(QFrame::Raised);

        vboxLayout->addWidget(cpmTable);


        retranslateUi(KPlato__CpmWidget);

        QMetaObject::connectSlotsByName(KPlato__CpmWidget);
    } // setupUi

    void retranslateUi(QWidget *KPlato__CpmWidget)
    {
        label_1->setText(tr2i18n("Project Schedule:", nullptr));
        scheduleName->setText(tr2i18n("...", nullptr));
        label->setText(tr2i18n("Probability of finishing before", nullptr));
        label_2->setText(tr2i18n("is", nullptr));
        probability->setSuffix(tr2i18n("%", nullptr));
        (void)KPlato__CpmWidget;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class CpmWidget: public Ui_CpmWidget {};
} // namespace Ui
} // namespace KPlato

#endif // KPTCPMWIDGET_H

