/********************************************************************************
** Form generated from reading UI file 'resourcedialogbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RESOURCEDIALOGBASE_H
#define UI_RESOURCEDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "KComboBox"
#include "kpttreecombobox.h"

namespace KPlato {

class Ui_ResourceDialogBase
{
public:
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout_2;
    QHBoxLayout *hboxLayout;
    QLineEdit *initialsEdit;
    QLabel *ui_emaillabel;
    QLineEdit *emailEdit;
    QLabel *ui_namelabel;
    QLabel *ui_initialslabel;
    QHBoxLayout *hboxLayout1;
    QLineEdit *nameEdit;
    QPushButton *chooseBtn;
    QFrame *line;
    QHBoxLayout *horizontalLayout_2;
    QLabel *textLabel7;
    QComboBox *type;
    QFrame *line_3;
    QStackedWidget *ui_stackedWidget;
    QWidget *page;
    QVBoxLayout *verticalLayout_5;
    QGridLayout *gridLayout;
    QLabel *ui_unitslabel;
    QSpinBox *units;
    QLabel *ui_calendarlabel;
    QComboBox *calendarList;
    QGroupBox *ui_fromgroup;
    QVBoxLayout *verticalLayout_4;
    QRadioButton *ui_rbfromunlimited;
    QHBoxLayout *horizontalLayout_4;
    QRadioButton *ui_rbfrom;
    QDateTimeEdit *availableFrom;
    QGroupBox *ui_untilgroup;
    QVBoxLayout *verticalLayout_3;
    QRadioButton *ui_rbuntilunlimited;
    QHBoxLayout *horizontalLayout_3;
    QRadioButton *ui_rbuntil;
    QDateTimeEdit *availableUntil;
    QFrame *line_2;
    QHBoxLayout *horizontalLayout;
    QCheckBox *useRequired;
    KPlato::TreeComboBox *required;
    QGroupBox *ui_costgroup;
    QGridLayout *gridLayout1;
    QLabel *textLabel3_2;
    QLineEdit *rateEdit;
    QLabel *textLabel4_2;
    QLineEdit *overtimeEdit;
    QSpacerItem *spacerItem;
    KComboBox *account;
    QLabel *label;
    QSpacerItem *verticalSpacer;
    QWidget *page_2;
    QVBoxLayout *verticalLayout;
    QTreeView *ui_teamView;

    void setupUi(QWidget *KPlato__ResourceDialogBase)
    {
        if (KPlato__ResourceDialogBase->objectName().isEmpty())
            KPlato__ResourceDialogBase->setObjectName("KPlato__ResourceDialogBase");
        KPlato__ResourceDialogBase->resize(514, 535);
        verticalLayout_2 = new QVBoxLayout(KPlato__ResourceDialogBase);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        initialsEdit = new QLineEdit(KPlato__ResourceDialogBase);
        initialsEdit->setObjectName("initialsEdit");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(initialsEdit->sizePolicy().hasHeightForWidth());
        initialsEdit->setSizePolicy(sizePolicy);
        initialsEdit->setMaximumSize(QSize(100, 32767));

        hboxLayout->addWidget(initialsEdit);

        ui_emaillabel = new QLabel(KPlato__ResourceDialogBase);
        ui_emaillabel->setObjectName("ui_emaillabel");
        ui_emaillabel->setWordWrap(false);

        hboxLayout->addWidget(ui_emaillabel);

        emailEdit = new QLineEdit(KPlato__ResourceDialogBase);
        emailEdit->setObjectName("emailEdit");

        hboxLayout->addWidget(emailEdit);


        gridLayout_2->addLayout(hboxLayout, 1, 1, 1, 1);

        ui_namelabel = new QLabel(KPlato__ResourceDialogBase);
        ui_namelabel->setObjectName("ui_namelabel");
        ui_namelabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ui_namelabel->setWordWrap(false);

        gridLayout_2->addWidget(ui_namelabel, 0, 0, 1, 1);

        ui_initialslabel = new QLabel(KPlato__ResourceDialogBase);
        ui_initialslabel->setObjectName("ui_initialslabel");
        ui_initialslabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ui_initialslabel->setWordWrap(false);

        gridLayout_2->addWidget(ui_initialslabel, 1, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        nameEdit = new QLineEdit(KPlato__ResourceDialogBase);
        nameEdit->setObjectName("nameEdit");
        nameEdit->setFocusPolicy(Qt::WheelFocus);

        hboxLayout1->addWidget(nameEdit);

        chooseBtn = new QPushButton(KPlato__ResourceDialogBase);
        chooseBtn->setObjectName("chooseBtn");

        hboxLayout1->addWidget(chooseBtn);


        gridLayout_2->addLayout(hboxLayout1, 0, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout_2);

        line = new QFrame(KPlato__ResourceDialogBase);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        textLabel7 = new QLabel(KPlato__ResourceDialogBase);
        textLabel7->setObjectName("textLabel7");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textLabel7->sizePolicy().hasHeightForWidth());
        textLabel7->setSizePolicy(sizePolicy1);
        textLabel7->setWordWrap(false);

        horizontalLayout_2->addWidget(textLabel7);

        type = new QComboBox(KPlato__ResourceDialogBase);
        type->addItem(QString());
        type->addItem(QString());
        type->addItem(QString());
        type->setObjectName("type");

        horizontalLayout_2->addWidget(type);


        verticalLayout_2->addLayout(horizontalLayout_2);

        line_3 = new QFrame(KPlato__ResourceDialogBase);
        line_3->setObjectName("line_3");
        line_3->setFrameShape(QFrame::Shape::HLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line_3);

        ui_stackedWidget = new QStackedWidget(KPlato__ResourceDialogBase);
        ui_stackedWidget->setObjectName("ui_stackedWidget");
        page = new QWidget();
        page->setObjectName("page");
        verticalLayout_5 = new QVBoxLayout(page);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        ui_unitslabel = new QLabel(page);
        ui_unitslabel->setObjectName("ui_unitslabel");
        sizePolicy1.setHeightForWidth(ui_unitslabel->sizePolicy().hasHeightForWidth());
        ui_unitslabel->setSizePolicy(sizePolicy1);
        ui_unitslabel->setWordWrap(false);

        gridLayout->addWidget(ui_unitslabel, 0, 0, 1, 1);

        units = new QSpinBox(page);
        units->setObjectName("units");
        units->setEnabled(true);
        units->setFocusPolicy(Qt::WheelFocus);
        units->setMaximum(9999);

        gridLayout->addWidget(units, 0, 1, 1, 1);

        ui_calendarlabel = new QLabel(page);
        ui_calendarlabel->setObjectName("ui_calendarlabel");
        sizePolicy1.setHeightForWidth(ui_calendarlabel->sizePolicy().hasHeightForWidth());
        ui_calendarlabel->setSizePolicy(sizePolicy1);
        ui_calendarlabel->setWordWrap(false);

        gridLayout->addWidget(ui_calendarlabel, 0, 2, 1, 1);

        calendarList = new QComboBox(page);
        calendarList->setObjectName("calendarList");

        gridLayout->addWidget(calendarList, 0, 3, 1, 1);

        ui_fromgroup = new QGroupBox(page);
        ui_fromgroup->setObjectName("ui_fromgroup");
        verticalLayout_4 = new QVBoxLayout(ui_fromgroup);
        verticalLayout_4->setObjectName("verticalLayout_4");
        ui_rbfromunlimited = new QRadioButton(ui_fromgroup);
        ui_rbfromunlimited->setObjectName("ui_rbfromunlimited");
        sizePolicy.setHeightForWidth(ui_rbfromunlimited->sizePolicy().hasHeightForWidth());
        ui_rbfromunlimited->setSizePolicy(sizePolicy);

        verticalLayout_4->addWidget(ui_rbfromunlimited);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        ui_rbfrom = new QRadioButton(ui_fromgroup);
        ui_rbfrom->setObjectName("ui_rbfrom");
        sizePolicy.setHeightForWidth(ui_rbfrom->sizePolicy().hasHeightForWidth());
        ui_rbfrom->setSizePolicy(sizePolicy);

        horizontalLayout_4->addWidget(ui_rbfrom);

        availableFrom = new QDateTimeEdit(ui_fromgroup);
        availableFrom->setObjectName("availableFrom");
        availableFrom->setCalendarPopup(true);

        horizontalLayout_4->addWidget(availableFrom);


        verticalLayout_4->addLayout(horizontalLayout_4);


        gridLayout->addWidget(ui_fromgroup, 1, 0, 1, 2);

        ui_untilgroup = new QGroupBox(page);
        ui_untilgroup->setObjectName("ui_untilgroup");
        ui_untilgroup->setFlat(false);
        verticalLayout_3 = new QVBoxLayout(ui_untilgroup);
        verticalLayout_3->setObjectName("verticalLayout_3");
        ui_rbuntilunlimited = new QRadioButton(ui_untilgroup);
        ui_rbuntilunlimited->setObjectName("ui_rbuntilunlimited");
        sizePolicy.setHeightForWidth(ui_rbuntilunlimited->sizePolicy().hasHeightForWidth());
        ui_rbuntilunlimited->setSizePolicy(sizePolicy);

        verticalLayout_3->addWidget(ui_rbuntilunlimited);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        ui_rbuntil = new QRadioButton(ui_untilgroup);
        ui_rbuntil->setObjectName("ui_rbuntil");
        sizePolicy.setHeightForWidth(ui_rbuntil->sizePolicy().hasHeightForWidth());
        ui_rbuntil->setSizePolicy(sizePolicy);

        horizontalLayout_3->addWidget(ui_rbuntil);

        availableUntil = new QDateTimeEdit(ui_untilgroup);
        availableUntil->setObjectName("availableUntil");
        availableUntil->setCalendarPopup(true);

        horizontalLayout_3->addWidget(availableUntil);


        verticalLayout_3->addLayout(horizontalLayout_3);


        gridLayout->addWidget(ui_untilgroup, 1, 2, 1, 2);


        verticalLayout_5->addLayout(gridLayout);

        line_2 = new QFrame(page);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_5->addWidget(line_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        useRequired = new QCheckBox(page);
        useRequired->setObjectName("useRequired");
        sizePolicy.setHeightForWidth(useRequired->sizePolicy().hasHeightForWidth());
        useRequired->setSizePolicy(sizePolicy);
        useRequired->setMinimumSize(QSize(23, 0));

        horizontalLayout->addWidget(useRequired);

        required = new KPlato::TreeComboBox(page);
        required->setObjectName("required");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(100);
        sizePolicy2.setHeightForWidth(required->sizePolicy().hasHeightForWidth());
        required->setSizePolicy(sizePolicy2);
        required->setMinimumSize(QSize(200, 0));

        horizontalLayout->addWidget(required);


        verticalLayout_5->addLayout(horizontalLayout);

        ui_costgroup = new QGroupBox(page);
        ui_costgroup->setObjectName("ui_costgroup");
        gridLayout1 = new QGridLayout(ui_costgroup);
        gridLayout1->setObjectName("gridLayout1");
        textLabel3_2 = new QLabel(ui_costgroup);
        textLabel3_2->setObjectName("textLabel3_2");
        textLabel3_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3_2->setWordWrap(false);

        gridLayout1->addWidget(textLabel3_2, 0, 0, 1, 1);

        rateEdit = new QLineEdit(ui_costgroup);
        rateEdit->setObjectName("rateEdit");
        rateEdit->setFocusPolicy(Qt::WheelFocus);

        gridLayout1->addWidget(rateEdit, 0, 1, 1, 1);

        textLabel4_2 = new QLabel(ui_costgroup);
        textLabel4_2->setObjectName("textLabel4_2");
        textLabel4_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel4_2->setWordWrap(false);

        gridLayout1->addWidget(textLabel4_2, 1, 0, 1, 1);

        overtimeEdit = new QLineEdit(ui_costgroup);
        overtimeEdit->setObjectName("overtimeEdit");
        overtimeEdit->setFocusPolicy(Qt::WheelFocus);

        gridLayout1->addWidget(overtimeEdit, 1, 1, 1, 1);

        spacerItem = new QSpacerItem(0, 23, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout1->addItem(spacerItem, 1, 2, 1, 1);

        account = new KComboBox(ui_costgroup);
        account->setObjectName("account");
        account->setAutoCompletion(false);

        gridLayout1->addWidget(account, 2, 1, 1, 2);

        label = new QLabel(ui_costgroup);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout1->addWidget(label, 2, 0, 1, 1);


        verticalLayout_5->addWidget(ui_costgroup);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer);

        ui_stackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        verticalLayout = new QVBoxLayout(page_2);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        ui_teamView = new QTreeView(page_2);
        ui_teamView->setObjectName("ui_teamView");
        ui_teamView->setRootIsDecorated(false);
        ui_teamView->setItemsExpandable(false);
        ui_teamView->setSortingEnabled(true);
        ui_teamView->setExpandsOnDoubleClick(false);

        verticalLayout->addWidget(ui_teamView);

        ui_stackedWidget->addWidget(page_2);

        verticalLayout_2->addWidget(ui_stackedWidget);

#if QT_CONFIG(shortcut)
        ui_emaillabel->setBuddy(emailEdit);
        ui_namelabel->setBuddy(nameEdit);
        ui_initialslabel->setBuddy(initialsEdit);
        textLabel7->setBuddy(type);
        ui_unitslabel->setBuddy(units);
        ui_calendarlabel->setBuddy(calendarList);
        textLabel3_2->setBuddy(rateEdit);
        textLabel4_2->setBuddy(overtimeEdit);
        label->setBuddy(account);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(nameEdit, chooseBtn);
        QWidget::setTabOrder(chooseBtn, initialsEdit);
        QWidget::setTabOrder(initialsEdit, emailEdit);
        QWidget::setTabOrder(emailEdit, type);
        QWidget::setTabOrder(type, calendarList);
        QWidget::setTabOrder(calendarList, units);
        QWidget::setTabOrder(units, availableFrom);
        QWidget::setTabOrder(availableFrom, availableUntil);
        QWidget::setTabOrder(availableUntil, required);
        QWidget::setTabOrder(required, rateEdit);
        QWidget::setTabOrder(rateEdit, overtimeEdit);

        retranslateUi(KPlato__ResourceDialogBase);

        ui_stackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KPlato__ResourceDialogBase);
    } // setupUi

    void retranslateUi(QWidget *KPlato__ResourceDialogBase)
    {
        initialsEdit->setText(QString());
        ui_emaillabel->setText(tr2i18n("Email:", nullptr));
        emailEdit->setText(QString());
        ui_namelabel->setText(tr2i18n("Name:", nullptr));
        ui_initialslabel->setText(tr2i18n("Initials:", nullptr));
        nameEdit->setText(QString());
#if QT_CONFIG(tooltip)
        chooseBtn->setToolTip(tr2i18n("Choose resource from address book", nullptr));
#endif // QT_CONFIG(tooltip)
        chooseBtn->setText(tr2i18n("Choose...", nullptr));
        textLabel7->setText(tr2i18n("Resource type:", nullptr));
        type->setItemText(0, tr2i18n("Work", nullptr));
        type->setItemText(1, tr2i18n("Material", nullptr));
        type->setItemText(2, tr2i18n("Team", nullptr));

        ui_unitslabel->setText(tr2i18n("Available:", nullptr));
        units->setSuffix(tr2i18n("%", nullptr));
        ui_calendarlabel->setText(tr2i18n("Calendar:", nullptr));
        ui_fromgroup->setTitle(tr2i18n("From", nullptr));
        ui_rbfromunlimited->setText(tr2i18n("Unlimited", nullptr));
        ui_rbfrom->setText(QString());
        ui_untilgroup->setTitle(tr2i18n("Until", nullptr));
        ui_rbuntilunlimited->setText(tr2i18n("Unlimited", nullptr));
        ui_rbuntil->setText(QString());
        useRequired->setText(tr2i18n("Required resources:", nullptr));
        ui_costgroup->setTitle(tr2i18n("Cost", nullptr));
        textLabel3_2->setText(tr2i18n("Hourly rate:", nullptr));
        textLabel4_2->setText(tr2i18n("Overtime rate:", nullptr));
        overtimeEdit->setText(QString());
        label->setText(tr2i18n("Account:", nullptr));
        (void)KPlato__ResourceDialogBase;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class ResourceDialogBase: public Ui_ResourceDialogBase {};
} // namespace Ui
} // namespace KPlato

#endif // RESOURCEDIALOGBASE_H

