/****************************************************************************
** Meta object code from reading C++ file 'PlanTJScheduler.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../calligraplan-4.0.1/src/plugins/schedulers/tj/PlanTJScheduler.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PlanTJScheduler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15PlanTJSchedulerE_t {};
} // unnamed namespace

template <> constexpr inline auto PlanTJScheduler::qt_create_metaobjectdata<qt_meta_tag_ZN15PlanTJSchedulerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlanTJScheduler",
        "sigCalculationStarted",
        "",
        "KPlato::Project*",
        "KPlato::ScheduleManager*",
        "sigCalculationFinished",
        "taskname",
        "const char*",
        "slotMessage",
        "type",
        "msg",
        "TJ::CoreAttributes*",
        "object",
        "cancelScheduling",
        "SchedulingContext&",
        "context"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sigCalculationStarted'
        QtMocHelpers::SignalData<void(KPlato::Project *, KPlato::ScheduleManager *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 },
        }}),
        // Signal 'sigCalculationFinished'
        QtMocHelpers::SignalData<void(KPlato::Project *, KPlato::ScheduleManager *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 },
        }}),
        // Signal 'taskname'
        QtMocHelpers::SignalData<const char *()>(6, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'slotMessage'
        QtMocHelpers::SlotData<void(int, const QString &, TJ::CoreAttributes *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { QMetaType::QString, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'cancelScheduling'
        QtMocHelpers::SlotData<void(SchedulingContext &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlanTJScheduler, qt_meta_tag_ZN15PlanTJSchedulerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlanTJScheduler::staticMetaObject = { {
    QMetaObject::SuperData::link<KPlato::SchedulerThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlanTJSchedulerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlanTJSchedulerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15PlanTJSchedulerE_t>.metaTypes,
    nullptr
} };

void PlanTJScheduler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlanTJScheduler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigCalculationStarted((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[2]))); break;
        case 1: _t->sigCalculationFinished((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[2]))); break;
        case 2: { const char* _r = _t->taskname();
            if (_a[0]) *reinterpret_cast<const char**>(_a[0]) = std::move(_r); }  break;
        case 3: _t->slotMessage((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<TJ::CoreAttributes*>>(_a[3]))); break;
        case 4: _t->cancelScheduling((*reinterpret_cast<std::add_pointer_t<SchedulingContext&>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlanTJScheduler::*)(KPlato::Project * , KPlato::ScheduleManager * )>(_a, &PlanTJScheduler::sigCalculationStarted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlanTJScheduler::*)(KPlato::Project * , KPlato::ScheduleManager * )>(_a, &PlanTJScheduler::sigCalculationFinished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<const char * (PlanTJScheduler::*)()>(_a, &PlanTJScheduler::taskname, 2))
            return;
    }
}

const QMetaObject *PlanTJScheduler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlanTJScheduler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlanTJSchedulerE_t>.strings))
        return static_cast<void*>(this);
    return KPlato::SchedulerThread::qt_metacast(_clname);
}

int PlanTJScheduler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPlato::SchedulerThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void PlanTJScheduler::sigCalculationStarted(KPlato::Project * _t1, KPlato::ScheduleManager * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void PlanTJScheduler::sigCalculationFinished(KPlato::Project * _t1, KPlato::ScheduleManager * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
const char * PlanTJScheduler::taskname()
{
    const char* _t0{};
    QMetaObject::activate<const char*>(this, &staticMetaObject, 2, std::addressof(_t0));
    return _t0;
}
QT_WARNING_POP
