/********************************************************************************
** Form generated from reading UI file 'DocumentsSaveDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DOCUMENTSSAVEDIALOG_H
#define UI_DOCUMENTSSAVEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_DocumentsSaveDialog
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *mainWidget;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QCheckBox *saveMain;
    KUrlRequester *mainUrl;
    QTreeView *subdocsView;

    void setupUi(QWidget *DocumentsSaveDialog)
    {
        if (DocumentsSaveDialog->objectName().isEmpty())
            DocumentsSaveDialog->setObjectName("DocumentsSaveDialog");
        DocumentsSaveDialog->resize(280, 256);
        verticalLayout = new QVBoxLayout(DocumentsSaveDialog);
        verticalLayout->setObjectName("verticalLayout");
        mainWidget = new QWidget(DocumentsSaveDialog);
        mainWidget->setObjectName("mainWidget");
        verticalLayout_2 = new QVBoxLayout(mainWidget);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        saveMain = new QCheckBox(mainWidget);
        saveMain->setObjectName("saveMain");

        horizontalLayout->addWidget(saveMain);

        mainUrl = new KUrlRequester(mainWidget);
        mainUrl->setObjectName("mainUrl");
        mainUrl->setMode(KFile::File);

        horizontalLayout->addWidget(mainUrl);


        verticalLayout_2->addLayout(horizontalLayout);

        subdocsView = new QTreeView(mainWidget);
        subdocsView->setObjectName("subdocsView");

        verticalLayout_2->addWidget(subdocsView);


        verticalLayout->addWidget(mainWidget);


        retranslateUi(DocumentsSaveDialog);

        QMetaObject::connectSlotsByName(DocumentsSaveDialog);
    } // setupUi

    void retranslateUi(QWidget *DocumentsSaveDialog)
    {
        DocumentsSaveDialog->setWindowTitle(QString());
        saveMain->setText(tr2i18n("Save", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DocumentsSaveDialog: public Ui_DocumentsSaveDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // DOCUMENTSSAVEDIALOG_H

