/********************************************************************************
** Form generated from reading UI file 'relationpanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RELATIONPANEL_H
#define UI_RELATIONPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kptdurationspinbox.h"

namespace KPlato {

class Ui_RelationPanel
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QGroupBox *groupBox;
    QHBoxLayout *hboxLayout1;
    QLabel *fromName;
    QGroupBox *groupBox_2;
    QHBoxLayout *hboxLayout2;
    QLabel *toName;
    QGroupBox *relationType;
    QVBoxLayout *vboxLayout1;
    QRadioButton *bFinishStart;
    QRadioButton *bFinishFinish;
    QRadioButton *bStartStart;
    QHBoxLayout *hboxLayout3;
    QLabel *lagLabel;
    DurationSpinBox *lag;

    void setupUi(QWidget *KPlato__RelationPanel)
    {
        if (KPlato__RelationPanel->objectName().isEmpty())
            KPlato__RelationPanel->setObjectName("KPlato__RelationPanel");
        KPlato__RelationPanel->setEnabled(true);
        KPlato__RelationPanel->resize(302, 225);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(KPlato__RelationPanel->sizePolicy().hasHeightForWidth());
        KPlato__RelationPanel->setSizePolicy(sizePolicy);
        KPlato__RelationPanel->setMinimumSize(QSize(0, 0));
        vboxLayout = new QVBoxLayout(KPlato__RelationPanel);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName("hboxLayout");
        groupBox = new QGroupBox(KPlato__RelationPanel);
        groupBox->setObjectName("groupBox");
        hboxLayout1 = new QHBoxLayout(groupBox);
        hboxLayout1->setObjectName("hboxLayout1");
        fromName = new QLabel(groupBox);
        fromName->setObjectName("fromName");
        fromName->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        fromName->setWordWrap(true);

        hboxLayout1->addWidget(fromName);


        hboxLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(KPlato__RelationPanel);
        groupBox_2->setObjectName("groupBox_2");
        hboxLayout2 = new QHBoxLayout(groupBox_2);
        hboxLayout2->setObjectName("hboxLayout2");
        toName = new QLabel(groupBox_2);
        toName->setObjectName("toName");
        toName->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        toName->setWordWrap(true);

        hboxLayout2->addWidget(toName);


        hboxLayout->addWidget(groupBox_2);


        vboxLayout->addLayout(hboxLayout);

        relationType = new QGroupBox(KPlato__RelationPanel);
        relationType->setObjectName("relationType");
        vboxLayout1 = new QVBoxLayout(relationType);
        vboxLayout1->setObjectName("vboxLayout1");
        bFinishStart = new QRadioButton(relationType);
        bFinishStart->setObjectName("bFinishStart");

        vboxLayout1->addWidget(bFinishStart);

        bFinishFinish = new QRadioButton(relationType);
        bFinishFinish->setObjectName("bFinishFinish");

        vboxLayout1->addWidget(bFinishFinish);

        bStartStart = new QRadioButton(relationType);
        bStartStart->setObjectName("bStartStart");

        vboxLayout1->addWidget(bStartStart);


        vboxLayout->addWidget(relationType);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setObjectName("hboxLayout3");
        lagLabel = new QLabel(KPlato__RelationPanel);
        lagLabel->setObjectName("lagLabel");
        lagLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        lagLabel->setWordWrap(true);

        hboxLayout3->addWidget(lagLabel);

        lag = new DurationSpinBox(KPlato__RelationPanel);
        lag->setObjectName("lag");

        hboxLayout3->addWidget(lag);


        vboxLayout->addLayout(hboxLayout3);

#if QT_CONFIG(shortcut)
        lagLabel->setBuddy(lag);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__RelationPanel);

        QMetaObject::connectSlotsByName(KPlato__RelationPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__RelationPanel)
    {
        groupBox->setTitle(tr2i18n("From", nullptr));
        fromName->setText(QString());
        groupBox_2->setTitle(tr2i18n("To", nullptr));
        toName->setText(QString());
        relationType->setTitle(tr2i18n("Dependency Type", nullptr));
        bFinishStart->setText(tr2i18n("Finish-Start", nullptr));
        bFinishFinish->setText(tr2i18n("Finish-Finish", nullptr));
        bStartStart->setText(tr2i18n("Start-Start", nullptr));
        lagLabel->setText(QString());
        (void)KPlato__RelationPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class RelationPanel: public Ui_RelationPanel {};
} // namespace Ui
} // namespace KPlato

#endif // RELATIONPANEL_H

