/****************************************************************************
** Meta object code from reading C++ file 'kptschedulemodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/models/kptschedulemodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptschedulemodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato13ScheduleModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ScheduleModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato13ScheduleModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ScheduleModel",
        "Properties",
        "ScheduleName",
        "ScheduleState",
        "ScheduleDirection",
        "ScheduleOverbooking",
        "ScheduleDistribution",
        "SchedulePlannedStart",
        "SchedulePlannedFinish",
        "ScheduleScheduler",
        "ScheduleGranularity",
        "ScheduleScheduled",
        "ScheduleMode"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Properties'
        QtMocHelpers::EnumData<enum Properties>(1, 1, QMC::EnumFlags{}).add({
            {    2, Properties::ScheduleName },
            {    3, Properties::ScheduleState },
            {    4, Properties::ScheduleDirection },
            {    5, Properties::ScheduleOverbooking },
            {    6, Properties::ScheduleDistribution },
            {    7, Properties::SchedulePlannedStart },
            {    8, Properties::SchedulePlannedFinish },
            {    9, Properties::ScheduleScheduler },
            {   10, Properties::ScheduleGranularity },
            {   11, Properties::ScheduleScheduled },
            {   12, Properties::ScheduleMode },
        }),
    };
    return QtMocHelpers::metaObjectData<ScheduleModel, qt_meta_tag_ZN6KPlato13ScheduleModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ScheduleModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato13ScheduleModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato13ScheduleModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato13ScheduleModelE_t>.metaTypes,
    nullptr
} };

void KPlato::ScheduleModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScheduleModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::ScheduleModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ScheduleModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato13ScheduleModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KPlato::ScheduleModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato17ScheduleItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ScheduleItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato17ScheduleItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ScheduleItemModel",
        "scheduleManagerAdded",
        "",
        "KPlato::ScheduleManager*",
        "slotManagerChanged",
        "sch",
        "slotScheduleChanged",
        "KPlato::MainSchedule*",
        "slotScheduleManagerToBeInserted",
        "const KPlato::ScheduleManager*",
        "manager",
        "row",
        "slotScheduleManagerInserted",
        "slotScheduleManagerToBeRemoved",
        "slotScheduleManagerRemoved",
        "slotScheduleManagerToBeMoved",
        "slotScheduleManagerMoved",
        "index",
        "slotScheduleToBeInserted",
        "slotScheduleInserted",
        "const KPlato::MainSchedule*",
        "schedule",
        "slotScheduleToBeRemoved",
        "slotScheduleRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'scheduleManagerAdded'
        QtMocHelpers::SignalData<void(KPlato::ScheduleManager *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotManagerChanged'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 5 },
        }}),
        // Slot 'slotScheduleChanged'
        QtMocHelpers::SlotData<void(KPlato::MainSchedule *)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 5 },
        }}),
        // Slot 'slotScheduleManagerToBeInserted'
        QtMocHelpers::SlotData<void(const KPlato::ScheduleManager *, int)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Int, 11 },
        }}),
        // Slot 'slotScheduleManagerInserted'
        QtMocHelpers::SlotData<void(const KPlato::ScheduleManager *)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotScheduleManagerToBeRemoved'
        QtMocHelpers::SlotData<void(const KPlato::ScheduleManager *)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotScheduleManagerRemoved'
        QtMocHelpers::SlotData<void(const KPlato::ScheduleManager *)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotScheduleManagerToBeMoved'
        QtMocHelpers::SlotData<void(const KPlato::ScheduleManager *)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotScheduleManagerMoved'
        QtMocHelpers::SlotData<void(const KPlato::ScheduleManager *, int)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'slotScheduleToBeInserted'
        QtMocHelpers::SlotData<void(const KPlato::ScheduleManager *, int)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Int, 11 },
        }}),
        // Slot 'slotScheduleInserted'
        QtMocHelpers::SlotData<void(const KPlato::MainSchedule *)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'slotScheduleToBeRemoved'
        QtMocHelpers::SlotData<void(const KPlato::MainSchedule *)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'slotScheduleRemoved'
        QtMocHelpers::SlotData<void(const KPlato::MainSchedule *)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ScheduleItemModel, qt_meta_tag_ZN6KPlato17ScheduleItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ScheduleItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17ScheduleItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17ScheduleItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato17ScheduleItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::ScheduleItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScheduleItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->scheduleManagerAdded((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 1: _t->slotManagerChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 2: _t->slotScheduleChanged((*reinterpret_cast<std::add_pointer_t<KPlato::MainSchedule*>>(_a[1]))); break;
        case 3: _t->slotScheduleManagerToBeInserted((*reinterpret_cast<std::add_pointer_t<const KPlato::ScheduleManager*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->slotScheduleManagerInserted((*reinterpret_cast<std::add_pointer_t<const KPlato::ScheduleManager*>>(_a[1]))); break;
        case 5: _t->slotScheduleManagerToBeRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::ScheduleManager*>>(_a[1]))); break;
        case 6: _t->slotScheduleManagerRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::ScheduleManager*>>(_a[1]))); break;
        case 7: _t->slotScheduleManagerToBeMoved((*reinterpret_cast<std::add_pointer_t<const KPlato::ScheduleManager*>>(_a[1]))); break;
        case 8: _t->slotScheduleManagerMoved((*reinterpret_cast<std::add_pointer_t<const KPlato::ScheduleManager*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 9: _t->slotScheduleToBeInserted((*reinterpret_cast<std::add_pointer_t<const KPlato::ScheduleManager*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 10: _t->slotScheduleInserted((*reinterpret_cast<std::add_pointer_t<const KPlato::MainSchedule*>>(_a[1]))); break;
        case 11: _t->slotScheduleToBeRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::MainSchedule*>>(_a[1]))); break;
        case 12: _t->slotScheduleRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::MainSchedule*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ScheduleItemModel::*)(KPlato::ScheduleManager * )>(_a, &ScheduleItemModel::scheduleManagerAdded, 0))
            return;
    }
}

const QMetaObject *KPlato::ScheduleItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ScheduleItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17ScheduleItemModelE_t>.strings))
        return static_cast<void*>(this);
    return ItemModelBase::qt_metacast(_clname);
}

int KPlato::ScheduleItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ScheduleItemModel::scheduleManagerAdded(KPlato::ScheduleManager * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN6KPlato23ScheduleSortFilterModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ScheduleSortFilterModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato23ScheduleSortFilterModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ScheduleSortFilterModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ScheduleSortFilterModel, qt_meta_tag_ZN6KPlato23ScheduleSortFilterModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ScheduleSortFilterModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QSortFilterProxyModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato23ScheduleSortFilterModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato23ScheduleSortFilterModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato23ScheduleSortFilterModelE_t>.metaTypes,
    nullptr
} };

void KPlato::ScheduleSortFilterModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScheduleSortFilterModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::ScheduleSortFilterModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ScheduleSortFilterModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato23ScheduleSortFilterModelE_t>.strings))
        return static_cast<void*>(this);
    return QSortFilterProxyModel::qt_metacast(_clname);
}

int KPlato::ScheduleSortFilterModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSortFilterProxyModel::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato20ScheduleLogItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ScheduleLogItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato20ScheduleLogItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ScheduleLogItemModel",
        "slotManagerChanged",
        "",
        "KPlato::ScheduleManager*",
        "sch",
        "slotScheduleChanged",
        "KPlato::MainSchedule*",
        "slotScheduleManagerToBeRemoved",
        "const KPlato::ScheduleManager*",
        "manager",
        "slotScheduleManagerRemoved",
        "slotScheduleToBeInserted",
        "row",
        "slotScheduleInserted",
        "const KPlato::MainSchedule*",
        "schedule",
        "slotScheduleToBeRemoved",
        "slotScheduleRemoved",
        "slotLogInserted",
        "firstrow",
        "lastrow",
        "projectDeleted"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotManagerChanged'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotScheduleChanged'
        QtMocHelpers::SlotData<void(KPlato::MainSchedule *)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 6, 4 },
        }}),
        // Slot 'slotScheduleManagerToBeRemoved'
        QtMocHelpers::SlotData<void(const KPlato::ScheduleManager *)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotScheduleManagerRemoved'
        QtMocHelpers::SlotData<void(const KPlato::ScheduleManager *)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotScheduleToBeInserted'
        QtMocHelpers::SlotData<void(const KPlato::ScheduleManager *, int)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'slotScheduleInserted'
        QtMocHelpers::SlotData<void(const KPlato::MainSchedule *)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotScheduleToBeRemoved'
        QtMocHelpers::SlotData<void(const KPlato::MainSchedule *)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotScheduleRemoved'
        QtMocHelpers::SlotData<void(const KPlato::MainSchedule *)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotLogInserted'
        QtMocHelpers::SlotData<void(KPlato::MainSchedule *, int, int)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 6, 2 }, { QMetaType::Int, 19 }, { QMetaType::Int, 20 },
        }}),
        // Slot 'projectDeleted'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ScheduleLogItemModel, qt_meta_tag_ZN6KPlato20ScheduleLogItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ScheduleLogItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QStandardItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20ScheduleLogItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20ScheduleLogItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato20ScheduleLogItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::ScheduleLogItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScheduleLogItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotManagerChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 1: _t->slotScheduleChanged((*reinterpret_cast<std::add_pointer_t<KPlato::MainSchedule*>>(_a[1]))); break;
        case 2: _t->slotScheduleManagerToBeRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::ScheduleManager*>>(_a[1]))); break;
        case 3: _t->slotScheduleManagerRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::ScheduleManager*>>(_a[1]))); break;
        case 4: _t->slotScheduleToBeInserted((*reinterpret_cast<std::add_pointer_t<const KPlato::ScheduleManager*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->slotScheduleInserted((*reinterpret_cast<std::add_pointer_t<const KPlato::MainSchedule*>>(_a[1]))); break;
        case 6: _t->slotScheduleToBeRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::MainSchedule*>>(_a[1]))); break;
        case 7: _t->slotScheduleRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::MainSchedule*>>(_a[1]))); break;
        case 8: _t->slotLogInserted((*reinterpret_cast<std::add_pointer_t<KPlato::MainSchedule*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 9: _t->projectDeleted(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KPlato::ScheduleLogItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ScheduleLogItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20ScheduleLogItemModelE_t>.strings))
        return static_cast<void*>(this);
    return QStandardItemModel::qt_metacast(_clname);
}

int KPlato::ScheduleLogItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStandardItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
