/********************************************************************************
** Form generated from reading UI file 'taskcompletionpanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TASKCOMPLETIONPANEL_H
#define UI_TASKCOMPLETIONPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kptusedefforteditor.h"

namespace KPlatoWork {

class Ui_TaskCompletionPanel
{
public:
    QVBoxLayout *verticalLayout;
    QFrame *frame;
    QHBoxLayout *horizontalLayout;
    QCheckBox *started;
    QDateTimeEdit *startTime;
    QCheckBox *finished;
    QDateTimeEdit *finishTime;
    QSpacerItem *spacer;
    QHBoxLayout *horizontalLayout_7;
    QFrame *ui_plannedFrame;
    QHBoxLayout *horizontalLayout_6;
    QLabel *ui_plannedLabel;
    QSpacerItem *spacerItem;
    KPlato::CompletionEntryEditor *entryTable;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *addEntryBtn;
    QPushButton *removeEntryBtn;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *KPlatoWork__TaskCompletionPanel)
    {
        if (KPlatoWork__TaskCompletionPanel->objectName().isEmpty())
            KPlatoWork__TaskCompletionPanel->setObjectName("KPlatoWork__TaskCompletionPanel");
        KPlatoWork__TaskCompletionPanel->resize(620, 323);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(KPlatoWork__TaskCompletionPanel->sizePolicy().hasHeightForWidth());
        KPlatoWork__TaskCompletionPanel->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(KPlatoWork__TaskCompletionPanel);
        verticalLayout->setObjectName("verticalLayout");
        frame = new QFrame(KPlatoWork__TaskCompletionPanel);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        horizontalLayout = new QHBoxLayout(frame);
        horizontalLayout->setObjectName("horizontalLayout");
        started = new QCheckBox(frame);
        started->setObjectName("started");

        horizontalLayout->addWidget(started);

        startTime = new QDateTimeEdit(frame);
        startTime->setObjectName("startTime");
        startTime->setCalendarPopup(true);

        horizontalLayout->addWidget(startTime);

        finished = new QCheckBox(frame);
        finished->setObjectName("finished");

        horizontalLayout->addWidget(finished);

        finishTime = new QDateTimeEdit(frame);
        finishTime->setObjectName("finishTime");
        finishTime->setCalendarPopup(true);

        horizontalLayout->addWidget(finishTime);

        spacer = new QSpacerItem(16, 25, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(spacer);


        verticalLayout->addWidget(frame);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        ui_plannedFrame = new QFrame(KPlatoWork__TaskCompletionPanel);
        ui_plannedFrame->setObjectName("ui_plannedFrame");
        ui_plannedFrame->setFrameShape(QFrame::StyledPanel);
        ui_plannedFrame->setFrameShadow(QFrame::Raised);
        horizontalLayout_6 = new QHBoxLayout(ui_plannedFrame);
        horizontalLayout_6->setSpacing(10);
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        horizontalLayout_6->setContentsMargins(-1, 0, -1, 0);
        ui_plannedLabel = new QLabel(ui_plannedFrame);
        ui_plannedLabel->setObjectName("ui_plannedLabel");
        ui_plannedLabel->setText(QString::fromUtf8("Planned effort:"));
        ui_plannedLabel->setAlignment(Qt::AlignCenter);

        horizontalLayout_6->addWidget(ui_plannedLabel);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(spacerItem);


        horizontalLayout_7->addWidget(ui_plannedFrame);


        verticalLayout->addLayout(horizontalLayout_7);

        entryTable = new KPlato::CompletionEntryEditor(KPlatoWork__TaskCompletionPanel);
        entryTable->setObjectName("entryTable");
        entryTable->setTabKeyNavigation(false);
        entryTable->verticalHeader()->setVisible(false);

        verticalLayout->addWidget(entryTable);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        addEntryBtn = new QPushButton(KPlatoWork__TaskCompletionPanel);
        addEntryBtn->setObjectName("addEntryBtn");

        horizontalLayout_3->addWidget(addEntryBtn);

        removeEntryBtn = new QPushButton(KPlatoWork__TaskCompletionPanel);
        removeEntryBtn->setObjectName("removeEntryBtn");

        horizontalLayout_3->addWidget(removeEntryBtn);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_3);

        QWidget::setTabOrder(started, startTime);
        QWidget::setTabOrder(startTime, finished);
        QWidget::setTabOrder(finished, finishTime);
        QWidget::setTabOrder(finishTime, entryTable);

        retranslateUi(KPlatoWork__TaskCompletionPanel);

        QMetaObject::connectSlotsByName(KPlatoWork__TaskCompletionPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlatoWork__TaskCompletionPanel)
    {
        started->setText(tr2i18n("Started:", nullptr));
#if QT_CONFIG(shortcut)
        started->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        finished->setText(tr2i18n("Finished:", nullptr));
#if QT_CONFIG(shortcut)
        finished->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        addEntryBtn->setText(tr2i18n("Add Entry", nullptr));
        removeEntryBtn->setText(tr2i18n("Remove Entry", nullptr));
        (void)KPlatoWork__TaskCompletionPanel;
    } // retranslateUi

};

} // namespace KPlatoWork

namespace KPlatoWork {
namespace Ui {
    class TaskCompletionPanel: public Ui_TaskCompletionPanel {};
} // namespace Ui
} // namespace KPlatoWork

#endif // TASKCOMPLETIONPANEL_H

