/********************************************************************************
** Form generated from reading UI file 'PortfolioView.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PORTFOLIOVIEW_H
#define UI_PORTFOLIOVIEW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_PortfolioView
{
public:
    QVBoxLayout *verticalLayout;
    QStackedWidget *stackedWidget;
    QWidget *page;
    QHBoxLayout *horizontalLayout;
    QTreeView *recentPortfolios;
    QFrame *frame_6;
    QVBoxLayout *verticalLayout_8;
    QWidget *informationWidget;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_5;
    QPushButton *introduction;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer_2;
    QWidget *widget_3;
    QVBoxLayout *verticalLayout_7;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_6;
    QLabel *contextHelp;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QLabel *onlineLabel;
    QLabel *calligraPlanLabel;
    QLabel *userDocLabel;
    QLabel *forumLabel;
    QLabel *commuityLabel;
    QSpacerItem *verticalSpacer;
    QWidget *page_2;
    QVBoxLayout *verticalLayout_3;
    QTreeView *treeView;

    void setupUi(QWidget *PortfolioView)
    {
        if (PortfolioView->objectName().isEmpty())
            PortfolioView->setObjectName("PortfolioView");
        PortfolioView->resize(888, 442);
        PortfolioView->setFocusPolicy(Qt::StrongFocus);
        verticalLayout = new QVBoxLayout(PortfolioView);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        stackedWidget = new QStackedWidget(PortfolioView);
        stackedWidget->setObjectName("stackedWidget");
        page = new QWidget();
        page->setObjectName("page");
        horizontalLayout = new QHBoxLayout(page);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        recentPortfolios = new QTreeView(page);
        recentPortfolios->setObjectName("recentPortfolios");
        recentPortfolios->setEditTriggers(QAbstractItemView::NoEditTriggers);
        recentPortfolios->setAlternatingRowColors(true);
        recentPortfolios->setTextElideMode(Qt::ElideMiddle);
        recentPortfolios->setRootIsDecorated(false);
        recentPortfolios->setItemsExpandable(false);
        recentPortfolios->setHeaderHidden(false);
        recentPortfolios->header()->setVisible(true);

        horizontalLayout->addWidget(recentPortfolios);

        frame_6 = new QFrame(page);
        frame_6->setObjectName("frame_6");
        frame_6->setMaximumSize(QSize(300, 16777215));
        frame_6->setFrameShape(QFrame::StyledPanel);
        verticalLayout_8 = new QVBoxLayout(frame_6);
        verticalLayout_8->setObjectName("verticalLayout_8");
        informationWidget = new QWidget(frame_6);
        informationWidget->setObjectName("informationWidget");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(informationWidget->sizePolicy().hasHeightForWidth());
        informationWidget->setSizePolicy(sizePolicy);
        informationWidget->setMinimumSize(QSize(0, 0));
        verticalLayout_5 = new QVBoxLayout(informationWidget);
        verticalLayout_5->setSpacing(0);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_5);

        introduction = new QPushButton(informationWidget);
        introduction->setObjectName("introduction");
        introduction->setFocusPolicy(Qt::NoFocus);
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("emblem-information");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("../libs/main"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        introduction->setIcon(icon);
        introduction->setFlat(true);

        horizontalLayout_3->addWidget(introduction);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);


        verticalLayout_5->addLayout(horizontalLayout_3);

        verticalSpacer_2 = new QSpacerItem(20, 22, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_5->addItem(verticalSpacer_2);


        verticalLayout_8->addWidget(informationWidget);

        widget_3 = new QWidget(frame_6);
        widget_3->setObjectName("widget_3");
        verticalLayout_7 = new QVBoxLayout(widget_3);
        verticalLayout_7->setObjectName("verticalLayout_7");
        verticalLayout_7->setContentsMargins(0, 0, 0, 0);
        groupBox_2 = new QGroupBox(widget_3);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_6 = new QVBoxLayout(groupBox_2);
        verticalLayout_6->setObjectName("verticalLayout_6");
        contextHelp = new QLabel(groupBox_2);
        contextHelp->setObjectName("contextHelp");
        contextHelp->setScaledContents(true);
        contextHelp->setAlignment(Qt::AlignHCenter|Qt::AlignTop);
        contextHelp->setWordWrap(true);

        verticalLayout_6->addWidget(contextHelp);


        verticalLayout_7->addWidget(groupBox_2);

        groupBox = new QGroupBox(widget_3);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        onlineLabel = new QLabel(groupBox);
        onlineLabel->setObjectName("onlineLabel");
        onlineLabel->setAlignment(Qt::AlignCenter);

        verticalLayout_2->addWidget(onlineLabel);

        calligraPlanLabel = new QLabel(groupBox);
        calligraPlanLabel->setObjectName("calligraPlanLabel");
        calligraPlanLabel->setAlignment(Qt::AlignCenter);

        verticalLayout_2->addWidget(calligraPlanLabel);

        userDocLabel = new QLabel(groupBox);
        userDocLabel->setObjectName("userDocLabel");
        userDocLabel->setAlignment(Qt::AlignCenter);
        userDocLabel->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(userDocLabel);

        forumLabel = new QLabel(groupBox);
        forumLabel->setObjectName("forumLabel");
        forumLabel->setAlignment(Qt::AlignCenter);
        forumLabel->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(forumLabel);

        commuityLabel = new QLabel(groupBox);
        commuityLabel->setObjectName("commuityLabel");
        commuityLabel->setAlignment(Qt::AlignCenter);
        commuityLabel->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(commuityLabel);


        verticalLayout_7->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(20, 175, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(verticalSpacer);


        verticalLayout_8->addWidget(widget_3);


        horizontalLayout->addWidget(frame_6);

        stackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        verticalLayout_3 = new QVBoxLayout(page_2);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        treeView = new QTreeView(page_2);
        treeView->setObjectName("treeView");
        treeView->setRootIsDecorated(false);

        verticalLayout_3->addWidget(treeView);

        stackedWidget->addWidget(page_2);

        verticalLayout->addWidget(stackedWidget);


        retranslateUi(PortfolioView);

        stackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(PortfolioView);
    } // setupUi

    void retranslateUi(QWidget *PortfolioView)
    {
        introduction->setText(tr2i18n("Information", nullptr));
        groupBox_2->setTitle(QString());
        contextHelp->setText(QString());
        groupBox->setTitle(QString());
        onlineLabel->setText(QString());
#if QT_CONFIG(tooltip)
        calligraPlanLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        calligraPlanLabel->setStatusTip(tr2i18n("https://calligra.org/plan", "@info:status"));
#endif // QT_CONFIG(statustip)
        calligraPlanLabel->setText(tr2i18n("<html><head/><body><p><a href=\"https://calligra.org/plan\"><span style=\" text-decoration: underline; color:#2980b9;\">Home</span></a></p></body></html>", nullptr));
#if QT_CONFIG(tooltip)
        userDocLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        userDocLabel->setStatusTip(tr2i18n("https://docs.kde.org", "@info:status"));
#endif // QT_CONFIG(statustip)
        userDocLabel->setText(tr2i18n("<a href=\"https://docs.kde.org/index.php?application=calligraplanportfolio\">User documentation</a>", "@action:button"));
#if QT_CONFIG(tooltip)
        forumLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        forumLabel->setStatusTip(tr2i18n("https://forum.kde.org/viewforum.php?f=203", "@info:status"));
#endif // QT_CONFIG(statustip)
        forumLabel->setText(tr2i18n("<a href=\"https://forum.kde.org/viewforum.php?f=203\">Community forum</a>", "@action:button"));
#if QT_CONFIG(tooltip)
        commuityLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        commuityLabel->setStatusTip(tr2i18n("https://userbase.kde.org/Plan", "@info:status"));
#endif // QT_CONFIG(statustip)
        commuityLabel->setText(tr2i18n("<a href=\"https://userbase.kde.org/Plan\">Community</a>", "@action:button"));
        (void)PortfolioView;
    } // retranslateUi

};

namespace Ui {
    class PortfolioView: public Ui_PortfolioView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PORTFOLIOVIEW_H

