/********************************************************************************
** Form generated from reading UI file 'ConfigDocumentationPanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGDOCUMENTATIONPANEL_H
#define UI_CONFIGDOCUMENTATIONPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "KLineEdit"

namespace KPlato {

class Ui_ConfigDocumentationPanel
{
public:
    QHBoxLayout *horizontalLayout;
    QWidget *widget;
    QGridLayout *gridLayout;
    QLabel *leaderlabel;
    KLineEdit *kcfg_DocumentationPath;
    QFrame *line;
    QLabel *contextlabel;
    QLineEdit *kcfg_ContextPath;
    QLabel *languageLabel;
    QLineEdit *kcfg_ContextLanguage;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPlato__ConfigDocumentationPanel)
    {
        if (KPlato__ConfigDocumentationPanel->objectName().isEmpty())
            KPlato__ConfigDocumentationPanel->setObjectName("KPlato__ConfigDocumentationPanel");
        KPlato__ConfigDocumentationPanel->resize(547, 362);
        horizontalLayout = new QHBoxLayout(KPlato__ConfigDocumentationPanel);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        widget = new QWidget(KPlato__ConfigDocumentationPanel);
        widget->setObjectName("widget");
        gridLayout = new QGridLayout(widget);
        gridLayout->setObjectName("gridLayout");
        leaderlabel = new QLabel(widget);
        leaderlabel->setObjectName("leaderlabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(leaderlabel->sizePolicy().hasHeightForWidth());
        leaderlabel->setSizePolicy(sizePolicy);
        leaderlabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        leaderlabel->setWordWrap(false);

        gridLayout->addWidget(leaderlabel, 0, 0, 1, 1);

        kcfg_DocumentationPath = new KLineEdit(widget);
        kcfg_DocumentationPath->setObjectName("kcfg_DocumentationPath");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(kcfg_DocumentationPath->sizePolicy().hasHeightForWidth());
        kcfg_DocumentationPath->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(kcfg_DocumentationPath, 0, 1, 1, 1);

        line = new QFrame(widget);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line, 1, 0, 1, 2);

        contextlabel = new QLabel(widget);
        contextlabel->setObjectName("contextlabel");
        contextlabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(contextlabel, 2, 0, 1, 1);

        kcfg_ContextPath = new QLineEdit(widget);
        kcfg_ContextPath->setObjectName("kcfg_ContextPath");

        gridLayout->addWidget(kcfg_ContextPath, 2, 1, 1, 1);

        languageLabel = new QLabel(widget);
        languageLabel->setObjectName("languageLabel");
        languageLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(languageLabel, 3, 0, 1, 1);

        kcfg_ContextLanguage = new QLineEdit(widget);
        kcfg_ContextLanguage->setObjectName("kcfg_ContextLanguage");
        kcfg_ContextLanguage->setEnabled(true);

        gridLayout->addWidget(kcfg_ContextLanguage, 3, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 287, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 4, 0, 1, 2);


        horizontalLayout->addWidget(widget);

#if QT_CONFIG(shortcut)
        leaderlabel->setBuddy(kcfg_DocumentationPath);
        contextlabel->setBuddy(kcfg_ContextPath);
        languageLabel->setBuddy(kcfg_ContextLanguage);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__ConfigDocumentationPanel);

        QMetaObject::connectSlotsByName(KPlato__ConfigDocumentationPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__ConfigDocumentationPanel)
    {
#if QT_CONFIG(whatsthis)
        leaderlabel->setWhatsThis(tr2i18n("The person responsible for this task.\n"
"\n"
"This is not limited to persons available in a resource group but can be anyone. You can even directly access your address book with the Choose button.", nullptr));
#endif // QT_CONFIG(whatsthis)
        leaderlabel->setText(tr2i18n("Documentation:", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_DocumentationPath->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
        contextlabel->setText(tr2i18n("Context help:", nullptr));
        languageLabel->setText(tr2i18n("Context language", nullptr));
        (void)KPlato__ConfigDocumentationPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class ConfigDocumentationPanel: public Ui_ConfigDocumentationPanel {};
} // namespace Ui
} // namespace KPlato

#endif // CONFIGDOCUMENTATIONPANEL_H

