/********************************************************************************
** Form generated from reading UI file 'kptwbsdefinitionpanelbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTWBSDEFINITIONPANELBASE_H
#define UI_KPTWBSDEFINITIONPANELBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_WBSDefinitionPanelBase
{
public:
    QVBoxLayout *verticalLayout_3;
    QFrame *frame_2;
    QVBoxLayout *verticalLayout_2;
    QLabel *label_2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *textLabel1_2;
    QLineEdit *projectCode;
    QLabel *textLabel1_3_3;
    QLineEdit *projectSeparator;
    QFrame *line_2;
    QFrame *frame;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QLabel *textLabel1;
    QComboBox *defaultCode;
    QLabel *textLabel1_3_2;
    QLineEdit *defaultSeparator;
    QGroupBox *levelsGroup;
    QGridLayout *gridLayout;
    QTableWidget *levelsTable;
    QPushButton *removeBtn;
    QPushButton *addBtn;
    QSpinBox *level;

    void setupUi(QWidget *KPlato__WBSDefinitionPanelBase)
    {
        if (KPlato__WBSDefinitionPanelBase->objectName().isEmpty())
            KPlato__WBSDefinitionPanelBase->setObjectName("KPlato__WBSDefinitionPanelBase");
        KPlato__WBSDefinitionPanelBase->resize(362, 416);
        verticalLayout_3 = new QVBoxLayout(KPlato__WBSDefinitionPanelBase);
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        verticalLayout_3->setObjectName("verticalLayout_3");
        frame_2 = new QFrame(KPlato__WBSDefinitionPanelBase);
        frame_2->setObjectName("frame_2");
        frame_2->setFrameShape(QFrame::StyledPanel);
        frame_2->setFrameShadow(QFrame::Raised);
        verticalLayout_2 = new QVBoxLayout(frame_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        label_2 = new QLabel(frame_2);
        label_2->setObjectName("label_2");

        verticalLayout_2->addWidget(label_2);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        textLabel1_2 = new QLabel(frame_2);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        horizontalLayout_2->addWidget(textLabel1_2);

        projectCode = new QLineEdit(frame_2);
        projectCode->setObjectName("projectCode");

        horizontalLayout_2->addWidget(projectCode);

        textLabel1_3_3 = new QLabel(frame_2);
        textLabel1_3_3->setObjectName("textLabel1_3_3");
        textLabel1_3_3->setWordWrap(false);

        horizontalLayout_2->addWidget(textLabel1_3_3);

        projectSeparator = new QLineEdit(frame_2);
        projectSeparator->setObjectName("projectSeparator");

        horizontalLayout_2->addWidget(projectSeparator);


        verticalLayout_2->addLayout(horizontalLayout_2);


        verticalLayout_3->addWidget(frame_2);

        line_2 = new QFrame(KPlato__WBSDefinitionPanelBase);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_3->addWidget(line_2);

        frame = new QFrame(KPlato__WBSDefinitionPanelBase);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        verticalLayout = new QVBoxLayout(frame);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(frame);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        textLabel1 = new QLabel(frame);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        horizontalLayout->addWidget(textLabel1);

        defaultCode = new QComboBox(frame);
        defaultCode->setObjectName("defaultCode");
        defaultCode->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        horizontalLayout->addWidget(defaultCode);

        textLabel1_3_2 = new QLabel(frame);
        textLabel1_3_2->setObjectName("textLabel1_3_2");
        textLabel1_3_2->setWordWrap(false);

        horizontalLayout->addWidget(textLabel1_3_2);

        defaultSeparator = new QLineEdit(frame);
        defaultSeparator->setObjectName("defaultSeparator");

        horizontalLayout->addWidget(defaultSeparator);


        verticalLayout->addLayout(horizontalLayout);


        verticalLayout_3->addWidget(frame);

        levelsGroup = new QGroupBox(KPlato__WBSDefinitionPanelBase);
        levelsGroup->setObjectName("levelsGroup");
        levelsGroup->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        levelsGroup->setCheckable(true);
        gridLayout = new QGridLayout(levelsGroup);
        gridLayout->setObjectName("gridLayout");
        levelsTable = new QTableWidget(levelsGroup);
        if (levelsTable->columnCount() < 2)
            levelsTable->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        levelsTable->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        levelsTable->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        levelsTable->setObjectName("levelsTable");
        levelsTable->setColumnCount(2);

        gridLayout->addWidget(levelsTable, 0, 0, 1, 3);

        removeBtn = new QPushButton(levelsGroup);
        removeBtn->setObjectName("removeBtn");

        gridLayout->addWidget(removeBtn, 1, 0, 1, 1);

        addBtn = new QPushButton(levelsGroup);
        addBtn->setObjectName("addBtn");

        gridLayout->addWidget(addBtn, 1, 1, 1, 1);

        level = new QSpinBox(levelsGroup);
        level->setObjectName("level");
        level->setMinimum(1);
        level->setValue(1);

        gridLayout->addWidget(level, 1, 2, 1, 1);


        verticalLayout_3->addWidget(levelsGroup);

#if QT_CONFIG(shortcut)
        textLabel1_2->setBuddy(projectCode);
        textLabel1_3_3->setBuddy(projectSeparator);
        textLabel1->setBuddy(defaultCode);
        textLabel1_3_2->setBuddy(defaultSeparator);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__WBSDefinitionPanelBase);

        QMetaObject::connectSlotsByName(KPlato__WBSDefinitionPanelBase);
    } // setupUi

    void retranslateUi(QWidget *KPlato__WBSDefinitionPanelBase)
    {
#if QT_CONFIG(tooltip)
        frame_2->setToolTip(tr2i18n("Define WBS code for the project", "@info:tooltip Work Breakdown Structure code"));
#endif // QT_CONFIG(tooltip)
        label_2->setText(tr2i18n("Project Code Definition", nullptr));
        textLabel1_2->setText(tr2i18n("Code:", nullptr));
        textLabel1_3_3->setText(tr2i18n("Separator:", nullptr));
#if QT_CONFIG(tooltip)
        frame->setToolTip(tr2i18n("Define default WBS code", "@info:tooltip Work Breakdown Structure code"));
#endif // QT_CONFIG(tooltip)
        label->setText(tr2i18n("Default Definition", nullptr));
        textLabel1->setText(tr2i18n("Code:", nullptr));
        textLabel1_3_2->setText(tr2i18n("Separator:", nullptr));
#if QT_CONFIG(tooltip)
        levelsGroup->setToolTip(tr2i18n("Define separate WBS codes for individual levels", "@info:tooltip Work Breakdown Structure code"));
#endif // QT_CONFIG(tooltip)
        levelsGroup->setTitle(tr2i18n("Use Levels Definition", nullptr));
        QTableWidgetItem *___qtablewidgetitem = levelsTable->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(tr2i18n("Code", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = levelsTable->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(tr2i18n("Separator", nullptr));
        removeBtn->setText(tr2i18n("Remove", nullptr));
        addBtn->setText(tr2i18n("Add Level", nullptr));
        (void)KPlato__WBSDefinitionPanelBase;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class WBSDefinitionPanelBase: public Ui_WBSDefinitionPanelBase {};
} // namespace Ui
} // namespace KPlato

#endif // KPTWBSDEFINITIONPANELBASE_H

