/********************************************************************************
** Form generated from reading UI file 'kptganttprintingoptions.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTGANTTPRINTINGOPTIONS_H
#define UI_KPTGANTTPRINTINGOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_GanttPrintingOptionsWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox_3;
    QHBoxLayout *horizontalLayout_3;
    QCheckBox *ui_printRowLabels;
    QCheckBox *ui_printColumnLabels;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QRadioButton *ui_noFitting;
    QRadioButton *ui_fitVertical;
    QRadioButton *ui_fitSingle;
    QGroupBox *groupBox_5;
    QHBoxLayout *horizontalLayout;
    QFrame *frame;
    QGridLayout *gridLayout_2;
    QRadioButton *ui_projectStartBtn;
    QDateTimeEdit *ui_chartStartTime;
    QRadioButton *ui_startTimeBtn;
    QDateTimeEdit *ui_startTime;
    QFrame *frame_2;
    QGridLayout *gridLayout;
    QRadioButton *ui_projectEndBtn;
    QDateTimeEdit *ui_chartEndTime;
    QRadioButton *ui_endTimeBtn;
    QDateTimeEdit *ui_endTime;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPlato__GanttPrintingOptionsWidget)
    {
        if (KPlato__GanttPrintingOptionsWidget->objectName().isEmpty())
            KPlato__GanttPrintingOptionsWidget->setObjectName("KPlato__GanttPrintingOptionsWidget");
        KPlato__GanttPrintingOptionsWidget->resize(547, 382);
        verticalLayout_2 = new QVBoxLayout(KPlato__GanttPrintingOptionsWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox_3 = new QGroupBox(KPlato__GanttPrintingOptionsWidget);
        groupBox_3->setObjectName("groupBox_3");
        horizontalLayout_3 = new QHBoxLayout(groupBox_3);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        ui_printRowLabels = new QCheckBox(groupBox_3);
        ui_printRowLabels->setObjectName("ui_printRowLabels");

        horizontalLayout_3->addWidget(ui_printRowLabels);

        ui_printColumnLabels = new QCheckBox(groupBox_3);
        ui_printColumnLabels->setObjectName("ui_printColumnLabels");

        horizontalLayout_3->addWidget(ui_printColumnLabels);


        verticalLayout_2->addWidget(groupBox_3);

        groupBox_2 = new QGroupBox(KPlato__GanttPrintingOptionsWidget);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setObjectName("verticalLayout");
        ui_noFitting = new QRadioButton(groupBox_2);
        ui_noFitting->setObjectName("ui_noFitting");

        verticalLayout->addWidget(ui_noFitting);

        ui_fitVertical = new QRadioButton(groupBox_2);
        ui_fitVertical->setObjectName("ui_fitVertical");

        verticalLayout->addWidget(ui_fitVertical);

        ui_fitSingle = new QRadioButton(groupBox_2);
        ui_fitSingle->setObjectName("ui_fitSingle");
        ui_fitSingle->setChecked(true);

        verticalLayout->addWidget(ui_fitSingle);


        verticalLayout_2->addWidget(groupBox_2);

        groupBox_5 = new QGroupBox(KPlato__GanttPrintingOptionsWidget);
        groupBox_5->setObjectName("groupBox_5");
        horizontalLayout = new QHBoxLayout(groupBox_5);
        horizontalLayout->setObjectName("horizontalLayout");
        frame = new QFrame(groupBox_5);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        gridLayout_2 = new QGridLayout(frame);
        gridLayout_2->setObjectName("gridLayout_2");
        ui_projectStartBtn = new QRadioButton(frame);
        ui_projectStartBtn->setObjectName("ui_projectStartBtn");

        gridLayout_2->addWidget(ui_projectStartBtn, 0, 0, 1, 1);

        ui_chartStartTime = new QDateTimeEdit(frame);
        ui_chartStartTime->setObjectName("ui_chartStartTime");
        ui_chartStartTime->setEnabled(true);
        ui_chartStartTime->setButtonSymbols(QAbstractSpinBox::NoButtons);

        gridLayout_2->addWidget(ui_chartStartTime, 0, 1, 1, 1);

        ui_startTimeBtn = new QRadioButton(frame);
        ui_startTimeBtn->setObjectName("ui_startTimeBtn");

        gridLayout_2->addWidget(ui_startTimeBtn, 1, 0, 1, 1);

        ui_startTime = new QDateTimeEdit(frame);
        ui_startTime->setObjectName("ui_startTime");
        ui_startTime->setEnabled(true);
        ui_startTime->setReadOnly(false);
        ui_startTime->setCalendarPopup(true);

        gridLayout_2->addWidget(ui_startTime, 1, 1, 1, 1);


        horizontalLayout->addWidget(frame);

        frame_2 = new QFrame(groupBox_5);
        frame_2->setObjectName("frame_2");
        frame_2->setFrameShape(QFrame::StyledPanel);
        frame_2->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(frame_2);
        gridLayout->setObjectName("gridLayout");
        ui_projectEndBtn = new QRadioButton(frame_2);
        ui_projectEndBtn->setObjectName("ui_projectEndBtn");

        gridLayout->addWidget(ui_projectEndBtn, 0, 0, 1, 1);

        ui_chartEndTime = new QDateTimeEdit(frame_2);
        ui_chartEndTime->setObjectName("ui_chartEndTime");
        ui_chartEndTime->setEnabled(true);
        ui_chartEndTime->setButtonSymbols(QAbstractSpinBox::NoButtons);

        gridLayout->addWidget(ui_chartEndTime, 0, 1, 1, 1);

        ui_endTimeBtn = new QRadioButton(frame_2);
        ui_endTimeBtn->setObjectName("ui_endTimeBtn");

        gridLayout->addWidget(ui_endTimeBtn, 1, 0, 1, 1);

        ui_endTime = new QDateTimeEdit(frame_2);
        ui_endTime->setObjectName("ui_endTime");
        ui_endTime->setCalendarPopup(true);

        gridLayout->addWidget(ui_endTime, 1, 1, 1, 1);


        horizontalLayout->addWidget(frame_2);


        verticalLayout_2->addWidget(groupBox_5);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        retranslateUi(KPlato__GanttPrintingOptionsWidget);

        QMetaObject::connectSlotsByName(KPlato__GanttPrintingOptionsWidget);
    } // setupUi

    void retranslateUi(QWidget *KPlato__GanttPrintingOptionsWidget)
    {
        groupBox_3->setTitle(tr2i18n("Headers", "@title:group"));
        ui_printRowLabels->setText(tr2i18n("Print task names", "@option:check"));
        ui_printColumnLabels->setText(tr2i18n("Print time scale", "@option:check"));
        groupBox_2->setTitle(tr2i18n("Scaling", "@title:group"));
        ui_noFitting->setText(tr2i18n("Divide chart on multiple pages", "@option:radio"));
        ui_fitVertical->setText(tr2i18n("Fit chart to page height", "@option:radio"));
        ui_fitSingle->setText(tr2i18n("Fit chart to a single page", "@option:radio"));
        groupBox_5->setTitle(tr2i18n("Range", "@title:group"));
        ui_projectStartBtn->setText(tr2i18n("Chart start:", "@option:radio"));
        ui_startTimeBtn->setText(tr2i18n("Start:", "@option:radio"));
        ui_projectEndBtn->setText(tr2i18n("Chart end:", "@option:radio"));
        ui_endTimeBtn->setText(tr2i18n("End:", "@option:radio"));
        (void)KPlato__GanttPrintingOptionsWidget;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class GanttPrintingOptionsWidget: public Ui_GanttPrintingOptionsWidget {};
} // namespace Ui
} // namespace KPlato

#endif // KPTGANTTPRINTINGOPTIONS_H

