/********************************************************************************
** Form generated from reading UI file 'ScrollableChart.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCROLLABLECHART_H
#define UI_SCROLLABLECHART_H

#include <KChartChart.h>
#include <KChartLegend>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QScrollBar>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_ScrollableChart
{
public:
    QHBoxLayout *horizontalLayout;
    QFrame *frame;
    QVBoxLayout *verticalLayout;
    KChart::Chart *chart;
    QScrollBar *scrollBar;
    KChart::Legend *legend;

    void setupUi(QWidget *KPlato__ScrollableChart)
    {
        if (KPlato__ScrollableChart->objectName().isEmpty())
            KPlato__ScrollableChart->setObjectName("KPlato__ScrollableChart");
        KPlato__ScrollableChart->resize(848, 392);
        KPlato__ScrollableChart->setFocusPolicy(Qt::StrongFocus);
        horizontalLayout = new QHBoxLayout(KPlato__ScrollableChart);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        frame = new QFrame(KPlato__ScrollableChart);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        verticalLayout = new QVBoxLayout(frame);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        chart = new KChart::Chart(frame);
        chart->setObjectName("chart");
        chart->setMinimumSize(QSize(100, 100));
        chart->setAutoFillBackground(false);

        verticalLayout->addWidget(chart);

        scrollBar = new QScrollBar(frame);
        scrollBar->setObjectName("scrollBar");
        scrollBar->setOrientation(Qt::Horizontal);

        verticalLayout->addWidget(scrollBar);


        horizontalLayout->addWidget(frame);

        legend = new KChart::Legend(KPlato__ScrollableChart);
        legend->setObjectName("legend");
        legend->setMinimumSize(QSize(0, 50));

        horizontalLayout->addWidget(legend);


        retranslateUi(KPlato__ScrollableChart);

        QMetaObject::connectSlotsByName(KPlato__ScrollableChart);
    } // setupUi

    void retranslateUi(QWidget *KPlato__ScrollableChart)
    {
        (void)KPlato__ScrollableChart;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class ScrollableChart: public Ui_ScrollableChart {};
} // namespace Ui
} // namespace KPlato

#endif // SCROLLABLECHART_H

