/****************************************************************************
** Meta object code from reading C++ file 'KoDocument.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/main/KoDocument.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KoDocument.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KoDocumentE_t {};
} // unnamed namespace

template <> constexpr inline auto KoDocument::qt_create_metaobjectdata<qt_meta_tag_ZN10KoDocumentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KoDocument",
        "unitChanged",
        "",
        "KoUnit",
        "unit",
        "sigProgress",
        "value",
        "statusBarMessage",
        "text",
        "clearStatusBarMessage",
        "modified",
        "titleModified",
        "caption",
        "isModified",
        "backupFileChanged",
        "completed",
        "canceled",
        "scheduleManagerChanged",
        "KPlato::ScheduleManager*",
        "sm",
        "addCommand",
        "KUndo2Command*",
        "command",
        "beginMacro",
        "KUndo2MagicString",
        "endMacro",
        "save",
        "waitSaveComplete",
        "slotAutoSave",
        "slotUndoStackIndexChanged",
        "idx",
        "_k_slotJobFinished",
        "KJob*",
        "job",
        "_k_slotStatJobFinished",
        "_k_slotGotMimeType",
        "KIO::Job*",
        "mime",
        "_k_slotUploadFinished",
        "setModified",
        "_mod",
        "backupFile",
        "pageCount"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'unitChanged'
        QtMocHelpers::SignalData<void(const KoUnit &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'sigProgress'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'statusBarMessage'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'clearStatusBarMessage'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'modified'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'titleModified'
        QtMocHelpers::SignalData<void(const QString &, bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { QMetaType::Bool, 13 },
        }}),
        // Signal 'backupFileChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'completed'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'canceled'
        QtMocHelpers::SignalData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'scheduleManagerChanged'
        QtMocHelpers::SignalData<void(KPlato::ScheduleManager *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'addCommand'
        QtMocHelpers::SlotData<void(KUndo2Command *)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'beginMacro'
        QtMocHelpers::SlotData<void(const KUndo2MagicString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 8 },
        }}),
        // Slot 'endMacro'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'save'
        QtMocHelpers::SlotData<bool()>(26, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'waitSaveComplete'
        QtMocHelpers::SlotData<bool()>(27, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'slotAutoSave'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUndoStackIndexChanged'
        QtMocHelpers::SlotData<void(int)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 30 },
        }}),
        // Slot '_k_slotJobFinished'
        QtMocHelpers::SlotData<void(KJob *)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot '_k_slotStatJobFinished'
        QtMocHelpers::SlotData<void(KJob *)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 2 },
        }}),
        // Slot '_k_slotGotMimeType'
        QtMocHelpers::SlotData<void(KIO::Job *, const QString &)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 33 }, { QMetaType::QString, 37 },
        }}),
        // Slot '_k_slotUploadFinished'
        QtMocHelpers::SlotData<void(KJob *)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'isModified'
        QtMocHelpers::MethodData<bool() const>(13, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'setModified'
        QtMocHelpers::MethodData<void(bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 40 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'backupFile'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'pageCount'
        QtMocHelpers::PropertyData<int>(42, QMetaType::Int, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KoDocument, qt_meta_tag_ZN10KoDocumentE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KoDocument::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KoDocumentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KoDocumentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KoDocumentE_t>.metaTypes,
    nullptr
} };

void KoDocument::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KoDocument *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->unitChanged((*reinterpret_cast<std::add_pointer_t<KoUnit>>(_a[1]))); break;
        case 1: _t->sigProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->statusBarMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->clearStatusBarMessage(); break;
        case 4: _t->modified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->titleModified((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->backupFileChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->completed(); break;
        case 8: _t->canceled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->scheduleManagerChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 10: _t->addCommand((*reinterpret_cast<std::add_pointer_t<KUndo2Command*>>(_a[1]))); break;
        case 11: _t->beginMacro((*reinterpret_cast<std::add_pointer_t<KUndo2MagicString>>(_a[1]))); break;
        case 12: _t->endMacro(); break;
        case 13: { bool _r = _t->save();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->waitSaveComplete();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->slotAutoSave(); break;
        case 16: _t->slotUndoStackIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->d->_k_slotJobFinished((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 18: _t->d->_k_slotStatJobFinished((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 19: _t->d->_k_slotGotMimeType((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 20: _t->d->_k_slotUploadFinished((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 21: { bool _r = _t->isModified();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: _t->setModified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KoDocument::*)(const KoUnit & )>(_a, &KoDocument::unitChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDocument::*)(int )>(_a, &KoDocument::sigProgress, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDocument::*)(const QString & )>(_a, &KoDocument::statusBarMessage, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDocument::*)()>(_a, &KoDocument::clearStatusBarMessage, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDocument::*)(bool )>(_a, &KoDocument::modified, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDocument::*)(const QString & , bool )>(_a, &KoDocument::titleModified, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDocument::*)(bool )>(_a, &KoDocument::backupFileChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDocument::*)()>(_a, &KoDocument::completed, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDocument::*)(const QString & )>(_a, &KoDocument::canceled, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoDocument::*)(KPlato::ScheduleManager * )>(_a, &KoDocument::scheduleManagerChanged, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->backupFile(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->pageCount(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBackupFile(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KoDocument::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KoDocument::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KoDocumentE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "KoDocumentBase"))
        return static_cast< KoDocumentBase*>(this);
    return QObject::qt_metacast(_clname);
}

int KoDocument::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KoDocument::unitChanged(const KoUnit & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KoDocument::sigProgress(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KoDocument::statusBarMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KoDocument::clearStatusBarMessage()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KoDocument::modified(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KoDocument::titleModified(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void KoDocument::backupFileChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void KoDocument::completed()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void KoDocument::canceled(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void KoDocument::scheduleManagerChanged(KPlato::ScheduleManager * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
