/********************************************************************************
** Form generated from reading UI file 'kptdocumentspanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTDOCUMENTSPANEL_H
#define UI_KPTDOCUMENTSPANEL_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_DocumentsPanel
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *nodeNameHolder;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QLineEdit *nodeName;
    QWidget *itemViewHolder;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *pbAdd;
    QPushButton *pbChange;
    QPushButton *pbRemove;
    QPushButton *pbView;

    void setupUi(QWidget *KPlato__DocumentsPanel)
    {
        if (KPlato__DocumentsPanel->objectName().isEmpty())
            KPlato__DocumentsPanel->setObjectName("KPlato__DocumentsPanel");
        KPlato__DocumentsPanel->resize(380, 244);
        verticalLayout = new QVBoxLayout(KPlato__DocumentsPanel);
        verticalLayout->setObjectName("verticalLayout");
        nodeNameHolder = new QWidget(KPlato__DocumentsPanel);
        nodeNameHolder->setObjectName("nodeNameHolder");
        nodeNameHolder->setEnabled(true);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(nodeNameHolder->sizePolicy().hasHeightForWidth());
        nodeNameHolder->setSizePolicy(sizePolicy);
        horizontalLayout = new QHBoxLayout(nodeNameHolder);
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(nodeNameHolder);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        nodeName = new QLineEdit(nodeNameHolder);
        nodeName->setObjectName("nodeName");

        horizontalLayout->addWidget(nodeName);


        verticalLayout->addWidget(nodeNameHolder);

        itemViewHolder = new QWidget(KPlato__DocumentsPanel);
        itemViewHolder->setObjectName("itemViewHolder");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(itemViewHolder->sizePolicy().hasHeightForWidth());
        itemViewHolder->setSizePolicy(sizePolicy1);

        verticalLayout->addWidget(itemViewHolder);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        pbAdd = new QPushButton(KPlato__DocumentsPanel);
        pbAdd->setObjectName("pbAdd");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        pbAdd->setIcon(icon);

        horizontalLayout_2->addWidget(pbAdd);

        pbChange = new QPushButton(KPlato__DocumentsPanel);
        pbChange->setObjectName("pbChange");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("document-edit");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        pbChange->setIcon(icon1);

        horizontalLayout_2->addWidget(pbChange);

        pbRemove = new QPushButton(KPlato__DocumentsPanel);
        pbRemove->setObjectName("pbRemove");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        pbRemove->setIcon(icon2);

        horizontalLayout_2->addWidget(pbRemove);

        pbView = new QPushButton(KPlato__DocumentsPanel);
        pbView->setObjectName("pbView");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        pbView->setIcon(icon3);

        horizontalLayout_2->addWidget(pbView);


        verticalLayout->addLayout(horizontalLayout_2);


        retranslateUi(KPlato__DocumentsPanel);

        QMetaObject::connectSlotsByName(KPlato__DocumentsPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__DocumentsPanel)
    {
        label->setText(tr2i18n("Name:", nullptr));
        pbAdd->setText(tr2i18n("Add...", nullptr));
#if QT_CONFIG(shortcut)
        pbAdd->setShortcut(tr2i18n("Ctrl+I", nullptr));
#endif // QT_CONFIG(shortcut)
        pbChange->setText(tr2i18n("Change...", nullptr));
#if QT_CONFIG(shortcut)
        pbChange->setShortcut(tr2i18n("Ctrl+M", nullptr));
#endif // QT_CONFIG(shortcut)
        pbRemove->setText(tr2i18n("Remove", nullptr));
#if QT_CONFIG(shortcut)
        pbRemove->setShortcut(tr2i18n("Del", nullptr));
#endif // QT_CONFIG(shortcut)
        pbView->setText(tr2i18n("Open", nullptr));
#if QT_CONFIG(shortcut)
        pbView->setShortcut(tr2i18n("Ctrl+O", nullptr));
#endif // QT_CONFIG(shortcut)
        (void)KPlato__DocumentsPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class DocumentsPanel: public Ui_DocumentsPanel {};
} // namespace Ui
} // namespace KPlato

#endif // KPTDOCUMENTSPANEL_H

