/****************************************************************************
** Meta object code from reading C++ file 'kpttaskeditor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/kpttaskeditor.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kpttaskeditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato19TaskEditorItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::TaskEditorItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato19TaskEditorItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::TaskEditorItemModel",
        "setScheduleManager",
        "",
        "KPlato::ScheduleManager*",
        "sm"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaskEditorItemModel, qt_meta_tag_ZN6KPlato19TaskEditorItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::TaskEditorItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<NodeItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19TaskEditorItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19TaskEditorItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato19TaskEditorItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::TaskEditorItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskEditorItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KPlato::TaskEditorItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::TaskEditorItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19TaskEditorItemModelE_t>.strings))
        return static_cast<void*>(this);
    return NodeItemModel::qt_metacast(_clname);
}

int KPlato::TaskEditorItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = NodeItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato18TaskEditorTreeViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::TaskEditorTreeView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato18TaskEditorTreeViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::TaskEditorTreeView",
        "currentColumnChanged",
        "",
        "QModelIndex",
        "slotDropAllowed",
        "index",
        "dropIndicatorPosition",
        "QDragMoveEvent*",
        "event"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentColumnChanged'
        QtMocHelpers::SignalData<void(const QModelIndex &, const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotDropAllowed'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, QDragMoveEvent *)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 5 }, { QMetaType::Int, 6 }, { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaskEditorTreeView, qt_meta_tag_ZN6KPlato18TaskEditorTreeViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::TaskEditorTreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<DoubleTreeViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18TaskEditorTreeViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18TaskEditorTreeViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato18TaskEditorTreeViewE_t>.metaTypes,
    nullptr
} };

void KPlato::TaskEditorTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskEditorTreeView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentColumnChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 1: _t->slotDropAllowed((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDragMoveEvent*>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TaskEditorTreeView::*)(const QModelIndex & , const QModelIndex & )>(_a, &TaskEditorTreeView::currentColumnChanged, 0))
            return;
    }
}

const QMetaObject *KPlato::TaskEditorTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::TaskEditorTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18TaskEditorTreeViewE_t>.strings))
        return static_cast<void*>(this);
    return DoubleTreeViewBase::qt_metacast(_clname);
}

int KPlato::TaskEditorTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DoubleTreeViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KPlato::TaskEditorTreeView::currentColumnChanged(const QModelIndex & _t1, const QModelIndex & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN6KPlato12NodeTreeViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::NodeTreeView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato12NodeTreeViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::NodeTreeView",
        "currentColumnChanged",
        "",
        "QModelIndex",
        "slotDropAllowed",
        "index",
        "dropIndicatorPosition",
        "QDragMoveEvent*",
        "event"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentColumnChanged'
        QtMocHelpers::SignalData<void(const QModelIndex &, const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotDropAllowed'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, QDragMoveEvent *)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 5 }, { QMetaType::Int, 6 }, { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NodeTreeView, qt_meta_tag_ZN6KPlato12NodeTreeViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::NodeTreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<DoubleTreeViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12NodeTreeViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12NodeTreeViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato12NodeTreeViewE_t>.metaTypes,
    nullptr
} };

void KPlato::NodeTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NodeTreeView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentColumnChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 1: _t->slotDropAllowed((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDragMoveEvent*>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (NodeTreeView::*)(const QModelIndex & , const QModelIndex & )>(_a, &NodeTreeView::currentColumnChanged, 0))
            return;
    }
}

const QMetaObject *KPlato::NodeTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::NodeTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12NodeTreeViewE_t>.strings))
        return static_cast<void*>(this);
    return DoubleTreeViewBase::qt_metacast(_clname);
}

int KPlato::NodeTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DoubleTreeViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KPlato::NodeTreeView::currentColumnChanged(const QModelIndex & _t1, const QModelIndex & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN6KPlato10TaskEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::TaskEditor::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato10TaskEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::TaskEditor",
        "taskSelected",
        "",
        "KPlato::Task*",
        "task",
        "openNode",
        "addTask",
        "addMilestone",
        "addSubtask",
        "addSubMilestone",
        "moveTaskUp",
        "moveTaskDown",
        "indentTask",
        "unindentTask",
        "saveTaskModule",
        "QUrl",
        "url",
        "KPlato::Project*",
        "project",
        "removeTaskModule",
        "setGuiActive",
        "activate",
        "setScheduleManager",
        "KPlato::ScheduleManager*",
        "sm",
        "slotEditCopy",
        "slotEditPaste",
        "slotOptions",
        "itemDoubleClicked",
        "QPersistentModelIndex",
        "idx",
        "slotSelectionChanged",
        "QModelIndexList",
        "slotCurrentChanged",
        "QModelIndex",
        "slotContextMenuRequested",
        "index",
        "QPoint",
        "pos",
        "rows",
        "slotEnableActions",
        "slotAddTask",
        "slotAddSubtask",
        "slotAddMilestone",
        "slotAddSubMilestone",
        "slotDeleteTask",
        "slotLinkTask",
        "slotIndentTask",
        "slotUnindentTask",
        "slotMoveTaskUp",
        "slotMoveTaskDown",
        "slotTaskSplit",
        "slotSplitView",
        "slotProjectShown",
        "taskModuleDoubleClicked",
        "slotOpenCurrentSelection",
        "slotOpenNode",
        "KPlato::Node*",
        "node",
        "slotTaskProgress",
        "slotOpenProjectDescription",
        "slotTaskDescription",
        "slotOpenTaskDescription",
        "slotDocuments",
        "slotTaskEditFinished",
        "result",
        "slotSummaryTaskEditFinished",
        "slotTaskProgressFinished",
        "slotMilestoneProgressFinished",
        "slotTaskDescriptionFinished",
        "slotDocumentsFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'taskSelected'
        QtMocHelpers::SignalData<void(KPlato::Task *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'openNode'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'addTask'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'addMilestone'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'addSubtask'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'addSubMilestone'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'moveTaskUp'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'moveTaskDown'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'indentTask'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'unindentTask'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'saveTaskModule'
        QtMocHelpers::SignalData<void(const QUrl &, KPlato::Project *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Signal 'removeTaskModule'
        QtMocHelpers::SignalData<void(const QUrl &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'setGuiActive'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Slot 'setScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'slotEditCopy'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditPaste'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOptions'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'itemDoubleClicked'
        QtMocHelpers::SlotData<void(const QPersistentModelIndex &)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QModelIndexList &)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 2 },
        }}),
        // Slot 'slotCurrentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 34, 2 }, { 0x80000000 | 34, 2 },
        }}),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QPoint &, const QModelIndexList &)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 34, 36 }, { 0x80000000 | 37, 38 }, { 0x80000000 | 32, 39 },
        }}),
        // Slot 'slotEnableActions'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddTask'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddSubtask'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddMilestone'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddSubMilestone'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDeleteTask'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotLinkTask'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotIndentTask'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUnindentTask'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMoveTaskUp'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMoveTaskDown'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTaskSplit'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSplitView'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotProjectShown'
        QtMocHelpers::SlotData<void(bool)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'taskModuleDoubleClicked'
        QtMocHelpers::SlotData<void(QModelIndex)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 34, 30 },
        }}),
        // Slot 'slotOpenCurrentSelection'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenNode'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 57, 58 },
        }}),
        // Slot 'slotTaskProgress'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenProjectDescription'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTaskDescription'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenTaskDescription'
        QtMocHelpers::SlotData<void(bool)>(62, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotDocuments'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTaskEditFinished'
        QtMocHelpers::SlotData<void(int)>(64, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 65 },
        }}),
        // Slot 'slotSummaryTaskEditFinished'
        QtMocHelpers::SlotData<void(int)>(66, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 65 },
        }}),
        // Slot 'slotTaskProgressFinished'
        QtMocHelpers::SlotData<void(int)>(67, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 65 },
        }}),
        // Slot 'slotMilestoneProgressFinished'
        QtMocHelpers::SlotData<void(int)>(68, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 65 },
        }}),
        // Slot 'slotTaskDescriptionFinished'
        QtMocHelpers::SlotData<void(int)>(69, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 65 },
        }}),
        // Slot 'slotDocumentsFinished'
        QtMocHelpers::SlotData<void(int)>(70, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 65 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaskEditor, qt_meta_tag_ZN6KPlato10TaskEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::TaskEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<ViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato10TaskEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato10TaskEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato10TaskEditorE_t>.metaTypes,
    nullptr
} };

void KPlato::TaskEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->taskSelected((*reinterpret_cast<std::add_pointer_t<KPlato::Task*>>(_a[1]))); break;
        case 1: _t->openNode(); break;
        case 2: _t->addTask(); break;
        case 3: _t->addMilestone(); break;
        case 4: _t->addSubtask(); break;
        case 5: _t->addSubMilestone(); break;
        case 6: _t->moveTaskUp(); break;
        case 7: _t->moveTaskDown(); break;
        case 8: _t->indentTask(); break;
        case 9: _t->unindentTask(); break;
        case 10: _t->saveTaskModule((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[2]))); break;
        case 11: _t->removeTaskModule((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 12: _t->setGuiActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->setScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 14: _t->slotEditCopy(); break;
        case 15: _t->slotEditPaste(); break;
        case 16: _t->slotOptions(); break;
        case 17: _t->itemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QPersistentModelIndex>>(_a[1]))); break;
        case 18: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 19: _t->slotCurrentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 20: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[3]))); break;
        case 21: _t->slotEnableActions(); break;
        case 22: _t->slotAddTask(); break;
        case 23: _t->slotAddSubtask(); break;
        case 24: _t->slotAddMilestone(); break;
        case 25: _t->slotAddSubMilestone(); break;
        case 26: _t->slotDeleteTask(); break;
        case 27: _t->slotLinkTask(); break;
        case 28: _t->slotIndentTask(); break;
        case 29: _t->slotUnindentTask(); break;
        case 30: _t->slotMoveTaskUp(); break;
        case 31: _t->slotMoveTaskDown(); break;
        case 32: _t->slotTaskSplit(); break;
        case 33: _t->slotSplitView(); break;
        case 34: _t->slotProjectShown((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: _t->taskModuleDoubleClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 36: _t->slotOpenCurrentSelection(); break;
        case 37: _t->slotOpenNode((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 38: _t->slotTaskProgress(); break;
        case 39: _t->slotOpenProjectDescription(); break;
        case 40: _t->slotTaskDescription(); break;
        case 41: _t->slotOpenTaskDescription((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 42: _t->slotDocuments(); break;
        case 43: _t->slotTaskEditFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 44: _t->slotSummaryTaskEditFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 45: _t->slotTaskProgressFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 46: _t->slotMilestoneProgressFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 47: _t->slotTaskDescriptionFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 48: _t->slotDocumentsFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        case 37:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TaskEditor::*)(KPlato::Task * )>(_a, &TaskEditor::taskSelected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskEditor::*)()>(_a, &TaskEditor::openNode, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskEditor::*)()>(_a, &TaskEditor::addTask, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskEditor::*)()>(_a, &TaskEditor::addMilestone, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskEditor::*)()>(_a, &TaskEditor::addSubtask, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskEditor::*)()>(_a, &TaskEditor::addSubMilestone, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskEditor::*)()>(_a, &TaskEditor::moveTaskUp, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskEditor::*)()>(_a, &TaskEditor::moveTaskDown, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskEditor::*)()>(_a, &TaskEditor::indentTask, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskEditor::*)()>(_a, &TaskEditor::unindentTask, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskEditor::*)(const QUrl & , KPlato::Project * )>(_a, &TaskEditor::saveTaskModule, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskEditor::*)(const QUrl & )>(_a, &TaskEditor::removeTaskModule, 11))
            return;
    }
}

const QMetaObject *KPlato::TaskEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::TaskEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato10TaskEditorE_t>.strings))
        return static_cast<void*>(this);
    return ViewBase::qt_metacast(_clname);
}

int KPlato::TaskEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 49)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 49;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 49)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 49;
    }
    return _id;
}

// SIGNAL 0
void KPlato::TaskEditor::taskSelected(KPlato::Task * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::TaskEditor::openNode()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KPlato::TaskEditor::addTask()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KPlato::TaskEditor::addMilestone()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KPlato::TaskEditor::addSubtask()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KPlato::TaskEditor::addSubMilestone()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void KPlato::TaskEditor::moveTaskUp()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KPlato::TaskEditor::moveTaskDown()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void KPlato::TaskEditor::indentTask()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void KPlato::TaskEditor::unindentTask()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void KPlato::TaskEditor::saveTaskModule(const QUrl & _t1, KPlato::Project * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void KPlato::TaskEditor::removeTaskModule(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN6KPlato8TaskViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::TaskView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato8TaskViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::TaskView",
        "openNode",
        "",
        "setGuiActive",
        "activate",
        "setScheduleManager",
        "KPlato::ScheduleManager*",
        "sm",
        "slotEditCopy",
        "slotOptions",
        "itemDoubleClicked",
        "QPersistentModelIndex",
        "idx",
        "slotSelectionChanged",
        "QModelIndexList",
        "slotCurrentChanged",
        "QModelIndex",
        "slotContextMenuRequested",
        "index",
        "QPoint",
        "pos",
        "slotEnableActions",
        "slotSplitView",
        "slotOpenCurrentNode",
        "slotOpenNode",
        "KPlato::Node*",
        "node",
        "slotTaskProgress",
        "slotOpenProjectDescription",
        "slotTaskDescription",
        "slotOpenTaskDescription",
        "slotDocuments",
        "slotTaskEditFinished",
        "result",
        "slotSummaryTaskEditFinished",
        "slotTaskProgressFinished",
        "slotMilestoneProgressFinished",
        "slotTaskDescriptionFinished",
        "slotDocumentsFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'openNode'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setGuiActive'
        QtMocHelpers::SlotData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'setScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotEditCopy'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOptions'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'itemDoubleClicked'
        QtMocHelpers::SlotData<void(const QPersistentModelIndex &)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QModelIndexList &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Slot 'slotCurrentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 2 }, { 0x80000000 | 16, 2 },
        }}),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QPoint &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 18 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotEnableActions'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSplitView'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenCurrentNode'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenNode'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'slotTaskProgress'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenProjectDescription'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTaskDescription'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenTaskDescription'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotDocuments'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTaskEditFinished'
        QtMocHelpers::SlotData<void(int)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 33 },
        }}),
        // Slot 'slotSummaryTaskEditFinished'
        QtMocHelpers::SlotData<void(int)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 33 },
        }}),
        // Slot 'slotTaskProgressFinished'
        QtMocHelpers::SlotData<void(int)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 33 },
        }}),
        // Slot 'slotMilestoneProgressFinished'
        QtMocHelpers::SlotData<void(int)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 33 },
        }}),
        // Slot 'slotTaskDescriptionFinished'
        QtMocHelpers::SlotData<void(int)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 33 },
        }}),
        // Slot 'slotDocumentsFinished'
        QtMocHelpers::SlotData<void(int)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 33 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaskView, qt_meta_tag_ZN6KPlato8TaskViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::TaskView::staticMetaObject = { {
    QMetaObject::SuperData::link<ViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8TaskViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8TaskViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato8TaskViewE_t>.metaTypes,
    nullptr
} };

void KPlato::TaskView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->openNode(); break;
        case 1: _t->setGuiActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->setScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 3: _t->slotEditCopy(); break;
        case 4: _t->slotOptions(); break;
        case 5: _t->itemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QPersistentModelIndex>>(_a[1]))); break;
        case 6: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 7: _t->slotCurrentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 8: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 9: _t->slotEnableActions(); break;
        case 10: _t->slotSplitView(); break;
        case 11: _t->slotOpenCurrentNode(); break;
        case 12: _t->slotOpenNode((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 13: _t->slotTaskProgress(); break;
        case 14: _t->slotOpenProjectDescription(); break;
        case 15: _t->slotTaskDescription(); break;
        case 16: _t->slotOpenTaskDescription((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->slotDocuments(); break;
        case 18: _t->slotTaskEditFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->slotSummaryTaskEditFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->slotTaskProgressFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->slotMilestoneProgressFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 22: _t->slotTaskDescriptionFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->slotDocumentsFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TaskView::*)()>(_a, &TaskView::openNode, 0))
            return;
    }
}

const QMetaObject *KPlato::TaskView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::TaskView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8TaskViewE_t>.strings))
        return static_cast<void*>(this);
    return ViewBase::qt_metacast(_clname);
}

int KPlato::TaskView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void KPlato::TaskView::openNode()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN6KPlato19WorkPackageTreeViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::WorkPackageTreeView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato19WorkPackageTreeViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::WorkPackageTreeView",
        "currentColumnChanged",
        "",
        "QModelIndex",
        "slotDropAllowed",
        "index",
        "dropIndicatorPosition",
        "QDragMoveEvent*",
        "event"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentColumnChanged'
        QtMocHelpers::SignalData<void(const QModelIndex &, const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotDropAllowed'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, QDragMoveEvent *)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 5 }, { QMetaType::Int, 6 }, { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WorkPackageTreeView, qt_meta_tag_ZN6KPlato19WorkPackageTreeViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::WorkPackageTreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<DoubleTreeViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19WorkPackageTreeViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19WorkPackageTreeViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato19WorkPackageTreeViewE_t>.metaTypes,
    nullptr
} };

void KPlato::WorkPackageTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WorkPackageTreeView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentColumnChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 1: _t->slotDropAllowed((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDragMoveEvent*>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WorkPackageTreeView::*)(const QModelIndex & , const QModelIndex & )>(_a, &WorkPackageTreeView::currentColumnChanged, 0))
            return;
    }
}

const QMetaObject *KPlato::WorkPackageTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::WorkPackageTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19WorkPackageTreeViewE_t>.strings))
        return static_cast<void*>(this);
    return DoubleTreeViewBase::qt_metacast(_clname);
}

int KPlato::WorkPackageTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DoubleTreeViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KPlato::WorkPackageTreeView::currentColumnChanged(const QModelIndex & _t1, const QModelIndex & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN6KPlato19TaskWorkPackageViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::TaskWorkPackageView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato19TaskWorkPackageViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::TaskWorkPackageView",
        "publishWorkpackages",
        "",
        "QList<KPlato::Node*>",
        "nodes",
        "KPlato::Resource*",
        "r",
        "mailTo",
        "openWorkpackages",
        "checkForWorkPackages",
        "loadWorkPackageUrl",
        "KPlato::Project*",
        "project",
        "QList<QUrl>&",
        "urls",
        "setGuiActive",
        "activate",
        "slotRefreshView",
        "setScheduleManager",
        "KPlato::ScheduleManager*",
        "sm",
        "slotEditCopy",
        "slotWorkpackagesAvailable",
        "value",
        "slotOptions",
        "slotMailWorkpackage",
        "slotWorkPackageSent",
        "resource",
        "slotLoadWorkPackage",
        "slotSelectionChanged",
        "QModelIndexList",
        "slotCurrentChanged",
        "QModelIndex",
        "slotContextMenuRequested",
        "index",
        "QPoint",
        "pos",
        "itemDoubleClicked",
        "QPersistentModelIndex",
        "idx",
        "slotEnableActions",
        "slotSplitView",
        "slotTaskProgress",
        "slotOpenProjectDescription",
        "slotTaskDescription",
        "slotOpenTaskDescription",
        "slotDocuments",
        "slotTaskProgressFinished",
        "result",
        "slotMilestoneProgressFinished",
        "slotTaskDescriptionFinished",
        "slotDocumentsFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'publishWorkpackages'
        QtMocHelpers::SignalData<void(const QList<KPlato::Node*> &, KPlato::Resource *, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Signal 'openWorkpackages'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'checkForWorkPackages'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'loadWorkPackageUrl'
        QtMocHelpers::SignalData<void(KPlato::Project *, QList<QUrl> &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot 'setGuiActive'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'slotRefreshView'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotEditCopy'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotWorkpackagesAvailable'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Slot 'slotOptions'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotMailWorkpackage'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotWorkPackageSent'
        QtMocHelpers::SlotData<void(const QList<KPlato::Node*> &, KPlato::Resource *)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 27 },
        }}),
        // Slot 'slotLoadWorkPackage'
        QtMocHelpers::SlotData<void(QList<QString>)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QStringList, 2 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QModelIndexList &)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 30, 2 },
        }}),
        // Slot 'slotCurrentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 2 }, { 0x80000000 | 32, 2 },
        }}),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QPoint &)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 34 }, { 0x80000000 | 35, 36 },
        }}),
        // Slot 'itemDoubleClicked'
        QtMocHelpers::SlotData<void(const QPersistentModelIndex &)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Slot 'slotEnableActions'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSplitView'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTaskProgress'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenProjectDescription'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTaskDescription'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenTaskDescription'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotDocuments'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTaskProgressFinished'
        QtMocHelpers::SlotData<void(int)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 48 },
        }}),
        // Slot 'slotMilestoneProgressFinished'
        QtMocHelpers::SlotData<void(int)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 48 },
        }}),
        // Slot 'slotTaskDescriptionFinished'
        QtMocHelpers::SlotData<void(int)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 48 },
        }}),
        // Slot 'slotDocumentsFinished'
        QtMocHelpers::SlotData<void(int)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 48 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaskWorkPackageView, qt_meta_tag_ZN6KPlato19TaskWorkPackageViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::TaskWorkPackageView::staticMetaObject = { {
    QMetaObject::SuperData::link<ViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19TaskWorkPackageViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19TaskWorkPackageViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato19TaskWorkPackageViewE_t>.metaTypes,
    nullptr
} };

void KPlato::TaskWorkPackageView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskWorkPackageView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->publishWorkpackages((*reinterpret_cast<std::add_pointer_t<QList<KPlato::Node*>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 1: _t->openWorkpackages(); break;
        case 2: _t->checkForWorkPackages((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->loadWorkPackageUrl((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>&>>(_a[2]))); break;
        case 4: _t->setGuiActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->slotRefreshView(); break;
        case 6: _t->setScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 7: _t->slotEditCopy(); break;
        case 8: _t->slotWorkpackagesAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->slotOptions(); break;
        case 10: _t->slotMailWorkpackage(); break;
        case 11: _t->slotWorkPackageSent((*reinterpret_cast<std::add_pointer_t<QList<KPlato::Node*>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[2]))); break;
        case 12: _t->slotLoadWorkPackage((*reinterpret_cast<std::add_pointer_t<QList<QString>>>(_a[1]))); break;
        case 13: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 14: _t->slotCurrentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 15: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 16: _t->itemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QPersistentModelIndex>>(_a[1]))); break;
        case 17: _t->slotEnableActions(); break;
        case 18: _t->slotSplitView(); break;
        case 19: _t->slotTaskProgress(); break;
        case 20: _t->slotOpenProjectDescription(); break;
        case 21: _t->slotTaskDescription(); break;
        case 22: _t->slotOpenTaskDescription((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->slotDocuments(); break;
        case 24: _t->slotTaskProgressFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->slotMilestoneProgressFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->slotTaskDescriptionFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->slotDocumentsFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Resource* >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<KPlato::Node*> >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Resource* >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<KPlato::Node*> >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TaskWorkPackageView::*)(const QList<KPlato::Node*> & , KPlato::Resource * , bool )>(_a, &TaskWorkPackageView::publishWorkpackages, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskWorkPackageView::*)()>(_a, &TaskWorkPackageView::openWorkpackages, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskWorkPackageView::*)(bool )>(_a, &TaskWorkPackageView::checkForWorkPackages, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskWorkPackageView::*)(KPlato::Project * , QList<QUrl> & )>(_a, &TaskWorkPackageView::loadWorkPackageUrl, 3))
            return;
    }
}

const QMetaObject *KPlato::TaskWorkPackageView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::TaskWorkPackageView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19TaskWorkPackageViewE_t>.strings))
        return static_cast<void*>(this);
    return ViewBase::qt_metacast(_clname);
}

int KPlato::TaskWorkPackageView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void KPlato::TaskWorkPackageView::publishWorkpackages(const QList<KPlato::Node*> & _t1, KPlato::Resource * _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void KPlato::TaskWorkPackageView::openWorkpackages()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KPlato::TaskWorkPackageView::checkForWorkPackages(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlato::TaskWorkPackageView::loadWorkPackageUrl(KPlato::Project * _t1, QList<QUrl> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
QT_WARNING_POP
