/****************************************************************************
** Meta object code from reading C++ file 'kptnodechartmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/models/kptnodechartmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptnodechartmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato15ChartProxyModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ChartProxyModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato15ChartProxyModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ChartProxyModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ChartProxyModel, qt_meta_tag_ZN6KPlato15ChartProxyModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ChartProxyModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QSortFilterProxyModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15ChartProxyModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15ChartProxyModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato15ChartProxyModelE_t>.metaTypes,
    nullptr
} };

void KPlato::ChartProxyModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ChartProxyModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::ChartProxyModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ChartProxyModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15ChartProxyModelE_t>.strings))
        return static_cast<void*>(this);
    return QSortFilterProxyModel::qt_metacast(_clname);
}

int KPlato::ChartProxyModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSortFilterProxyModel::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato14ChartItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ChartItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato14ChartItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ChartItemModel",
        "setScheduleManager",
        "",
        "KPlato::ScheduleManager*",
        "sm",
        "slotNodeRemoved",
        "KPlato::Node*",
        "node",
        "slotNodeChanged",
        "slotResourceChanged",
        "slotResourceRemoved",
        "slotSetScheduleManager",
        "Properties",
        "BCWSCost",
        "BCWPCost",
        "ACWPCost",
        "BCWSEffort",
        "BCWPEffort",
        "ACWPEffort",
        "SPICost",
        "CPICost",
        "SPIEffort",
        "CPIEffort"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotNodeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotNodeChanged'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotResourceChanged'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotResourceRemoved'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Properties'
        QtMocHelpers::EnumData<enum Properties>(12, 12, QMC::EnumFlags{}).add({
            {   13, Properties::BCWSCost },
            {   14, Properties::BCWPCost },
            {   15, Properties::ACWPCost },
            {   16, Properties::BCWSEffort },
            {   17, Properties::BCWPEffort },
            {   18, Properties::ACWPEffort },
            {   19, Properties::SPICost },
            {   20, Properties::CPICost },
            {   21, Properties::SPIEffort },
            {   22, Properties::CPIEffort },
        }),
    };
    return QtMocHelpers::metaObjectData<ChartItemModel, qt_meta_tag_ZN6KPlato14ChartItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ChartItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14ChartItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14ChartItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato14ChartItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::ChartItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ChartItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 1: _t->slotNodeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 2: _t->slotNodeChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 3: _t->slotResourceChanged(); break;
        case 4: _t->slotResourceRemoved(); break;
        case 5: _t->slotSetScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::ChartItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ChartItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14ChartItemModelE_t>.strings))
        return static_cast<void*>(this);
    return ItemModelBase::qt_metacast(_clname);
}

int KPlato::ChartItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato31PerformanceDataCurrentDateModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::PerformanceDataCurrentDateModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato31PerformanceDataCurrentDateModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::PerformanceDataCurrentDateModel",
        "setProject",
        "",
        "KPlato::Project*",
        "project",
        "setScheduleManager",
        "KPlato::ScheduleManager*",
        "sm",
        "setReadWrite",
        "rw"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setProject'
        QtMocHelpers::SlotData<void(KPlato::Project *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setReadWrite'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PerformanceDataCurrentDateModel, qt_meta_tag_ZN6KPlato31PerformanceDataCurrentDateModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::PerformanceDataCurrentDateModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractProxyModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato31PerformanceDataCurrentDateModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato31PerformanceDataCurrentDateModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato31PerformanceDataCurrentDateModelE_t>.metaTypes,
    nullptr
} };

void KPlato::PerformanceDataCurrentDateModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PerformanceDataCurrentDateModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setProject((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1]))); break;
        case 1: _t->setScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 2: _t->setReadWrite((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::PerformanceDataCurrentDateModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::PerformanceDataCurrentDateModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato31PerformanceDataCurrentDateModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractProxyModel::qt_metacast(_clname);
}

int KPlato::PerformanceDataCurrentDateModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
