/****************************************************************************
** Meta object code from reading C++ file 'kptschedule.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/kernel/kptschedule.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptschedule.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato15ScheduleManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ScheduleManager::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato15ScheduleManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ScheduleManager",
        "maxProgressChanged",
        "",
        "progressChanged",
        "logInserted",
        "KPlato::MainSchedule*",
        "firstrow",
        "lastrow",
        "sigLogAdded",
        "KPlato::Schedule::Log",
        "log",
        "setMaxProgress",
        "value",
        "setProgress",
        "slotAddLog",
        "QList<KPlato::Schedule::Log>",
        "setPhaseNames",
        "QMap<int,QString>",
        "phasenames",
        "CalculationResult",
        "CalculationRunning",
        "CalculationDone",
        "CalculationStopped",
        "CalculationCanceled",
        "CalculationError",
        "SchedulingMode",
        "ManualMode",
        "AutoMode",
        "Properties",
        "NameProperty",
        "DirectionProperty",
        "OverbookProperty",
        "DistributionProperty",
        "SchedulingModeProperty",
        "GranularityIndexProperty"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'maxProgressChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'progressChanged'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'logInserted'
        QtMocHelpers::SignalData<void(KPlato::MainSchedule *, int, int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 }, { QMetaType::Int, 6 }, { QMetaType::Int, 7 },
        }}),
        // Signal 'sigLogAdded'
        QtMocHelpers::SignalData<void(const KPlato::Schedule::Log &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'setMaxProgress'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'setProgress'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'slotAddLog'
        QtMocHelpers::SlotData<void(const QVector<KPlato::Schedule::Log> &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 10 },
        }}),
        // Slot 'setPhaseNames'
        QtMocHelpers::SlotData<void(const QMap<int,QString> &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'CalculationResult'
        QtMocHelpers::EnumData<enum CalculationResult>(19, 19, QMC::EnumFlags{}).add({
            {   20, CalculationResult::CalculationRunning },
            {   21, CalculationResult::CalculationDone },
            {   22, CalculationResult::CalculationStopped },
            {   23, CalculationResult::CalculationCanceled },
            {   24, CalculationResult::CalculationError },
        }),
        // enum 'SchedulingMode'
        QtMocHelpers::EnumData<enum SchedulingMode>(25, 25, QMC::EnumFlags{}).add({
            {   26, SchedulingMode::ManualMode },
            {   27, SchedulingMode::AutoMode },
        }),
        // enum 'Properties'
        QtMocHelpers::EnumData<enum Properties>(28, 28, QMC::EnumFlags{}).add({
            {   29, Properties::NameProperty },
            {   30, Properties::DirectionProperty },
            {   31, Properties::OverbookProperty },
            {   32, Properties::DistributionProperty },
            {   33, Properties::SchedulingModeProperty },
            {   34, Properties::GranularityIndexProperty },
        }),
    };
    return QtMocHelpers::metaObjectData<ScheduleManager, qt_meta_tag_ZN6KPlato15ScheduleManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ScheduleManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15ScheduleManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15ScheduleManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato15ScheduleManagerE_t>.metaTypes,
    nullptr
} };

void KPlato::ScheduleManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScheduleManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->maxProgressChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->progressChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->logInserted((*reinterpret_cast<std::add_pointer_t<KPlato::MainSchedule*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 3: _t->sigLogAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Schedule::Log>>(_a[1]))); break;
        case 4: _t->setMaxProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->setProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->slotAddLog((*reinterpret_cast<std::add_pointer_t<QList<KPlato::Schedule::Log>>>(_a[1]))); break;
        case 7: _t->setPhaseNames((*reinterpret_cast<std::add_pointer_t<QMap<int,QString>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Schedule::Log >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<KPlato::Schedule::Log> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ScheduleManager::*)(int )>(_a, &ScheduleManager::maxProgressChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleManager::*)(int )>(_a, &ScheduleManager::progressChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleManager::*)(KPlato::MainSchedule * , int , int )>(_a, &ScheduleManager::logInserted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleManager::*)(const KPlato::Schedule::Log & )>(_a, &ScheduleManager::sigLogAdded, 3))
            return;
    }
}

const QMetaObject *KPlato::ScheduleManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ScheduleManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15ScheduleManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KPlato::ScheduleManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ScheduleManager::maxProgressChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::ScheduleManager::progressChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KPlato::ScheduleManager::logInserted(KPlato::MainSchedule * _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void KPlato::ScheduleManager::sigLogAdded(const KPlato::Schedule::Log & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
