/****************************************************************************
** Meta object code from reading C++ file 'MainDocument.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../calligraplan-4.0.1/src/portfolio/MainDocument.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MainDocument.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12MainDocumentE_t {};
} // unnamed namespace

template <> constexpr inline auto MainDocument::qt_create_metaobjectdata<qt_meta_tag_ZN12MainDocumentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainDocument",
        "changed",
        "",
        "documentChanged",
        "KoDocument*",
        "doc",
        "index",
        "projectChanged",
        "documentAboutToBeInserted",
        "row",
        "documentInserted",
        "documentAboutToBeRemoved",
        "documentRemoved",
        "documentModified",
        "saveSettings",
        "QDomDocument&",
        "xml",
        "setModified",
        "mod",
        "slotProjectChanged",
        "slotProjectDocumentLoaded",
        "slotProjectDocumentCanceled",
        "slotDocumentModified"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'documentChanged'
        QtMocHelpers::SignalData<void(KoDocument *, int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'projectChanged'
        QtMocHelpers::SignalData<void(KoDocument *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'documentAboutToBeInserted'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Signal 'documentInserted'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'documentAboutToBeRemoved'
        QtMocHelpers::SignalData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Signal 'documentRemoved'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'documentModified'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'saveSettings'
        QtMocHelpers::SignalData<void(QDomDocument &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'setModified'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Slot 'slotProjectChanged'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotProjectDocumentLoaded'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotProjectDocumentCanceled'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDocumentModified'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainDocument, qt_meta_tag_ZN12MainDocumentE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainDocument::staticMetaObject = { {
    QMetaObject::SuperData::link<KoDocument::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MainDocumentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MainDocumentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12MainDocumentE_t>.metaTypes,
    nullptr
} };

void MainDocument::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainDocument *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: _t->documentChanged((*reinterpret_cast<std::add_pointer_t<KoDocument*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->projectChanged((*reinterpret_cast<std::add_pointer_t<KoDocument*>>(_a[1]))); break;
        case 3: _t->documentAboutToBeInserted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->documentInserted(); break;
        case 5: _t->documentAboutToBeRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->documentRemoved(); break;
        case 7: _t->documentModified(); break;
        case 8: _t->saveSettings((*reinterpret_cast<std::add_pointer_t<QDomDocument&>>(_a[1]))); break;
        case 9: _t->setModified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->slotProjectChanged(); break;
        case 11: _t->slotProjectDocumentLoaded(); break;
        case 12: _t->slotProjectDocumentCanceled(); break;
        case 13: _t->slotDocumentModified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KoDocument* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KoDocument* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)()>(_a, &MainDocument::changed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)(KoDocument * , int )>(_a, &MainDocument::documentChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)(KoDocument * )>(_a, &MainDocument::projectChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)(int )>(_a, &MainDocument::documentAboutToBeInserted, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)()>(_a, &MainDocument::documentInserted, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)(int )>(_a, &MainDocument::documentAboutToBeRemoved, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)()>(_a, &MainDocument::documentRemoved, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)()>(_a, &MainDocument::documentModified, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainDocument::*)(QDomDocument & )>(_a, &MainDocument::saveSettings, 8))
            return;
    }
}

const QMetaObject *MainDocument::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainDocument::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MainDocumentE_t>.strings))
        return static_cast<void*>(this);
    return KoDocument::qt_metacast(_clname);
}

int MainDocument::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KoDocument::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void MainDocument::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MainDocument::documentChanged(KoDocument * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void MainDocument::projectChanged(KoDocument * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MainDocument::documentAboutToBeInserted(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void MainDocument::documentInserted()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MainDocument::documentAboutToBeRemoved(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void MainDocument::documentRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void MainDocument::documentModified()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void MainDocument::saveSettings(QDomDocument & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
