/****************************************************************************
** Meta object code from reading C++ file 'kptdependencyeditor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/kptdependencyeditor.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptdependencyeditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato27DependecyViewPrintingDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::DependecyViewPrintingDialog::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato27DependecyViewPrintingDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::DependecyViewPrintingDialog"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DependecyViewPrintingDialog, qt_meta_tag_ZN6KPlato27DependecyViewPrintingDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::DependecyViewPrintingDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<PrintingDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato27DependecyViewPrintingDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato27DependecyViewPrintingDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato27DependecyViewPrintingDialogE_t>.metaTypes,
    nullptr
} };

void KPlato::DependecyViewPrintingDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DependecyViewPrintingDialog *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::DependecyViewPrintingDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::DependecyViewPrintingDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato27DependecyViewPrintingDialogE_t>.strings))
        return static_cast<void*>(this);
    return PrintingDialog::qt_metacast(_clname);
}

int KPlato::DependecyViewPrintingDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PrintingDialog::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato15DependencySceneE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::DependencyScene::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato15DependencySceneE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::DependencyScene",
        "connectorClicked",
        "",
        "KPlato::DependencyConnectorItem*",
        "item",
        "connectItems",
        "pred",
        "succ",
        "itemDoubleClicked",
        "QGraphicsItem*",
        "contextMenuRequested",
        "QPoint",
        "dependencyContextMenuRequested",
        "KPlato::DependencyLinkItem*",
        "update"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connectorClicked'
        QtMocHelpers::SignalData<void(KPlato::DependencyConnectorItem *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'connectItems'
        QtMocHelpers::SignalData<void(KPlato::DependencyConnectorItem *, KPlato::DependencyConnectorItem *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 }, { 0x80000000 | 3, 7 },
        }}),
        // Signal 'itemDoubleClicked'
        QtMocHelpers::SignalData<void(QGraphicsItem *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Signal 'contextMenuRequested'
        QtMocHelpers::SignalData<void(QGraphicsItem *, const QPoint &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 }, { 0x80000000 | 11, 2 },
        }}),
        // Signal 'dependencyContextMenuRequested'
        QtMocHelpers::SignalData<void(KPlato::DependencyLinkItem *, KPlato::DependencyConnectorItem *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 2 }, { 0x80000000 | 3, 2 },
        }}),
        // Slot 'update'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DependencyScene, qt_meta_tag_ZN6KPlato15DependencySceneE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::DependencyScene::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsScene::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15DependencySceneE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15DependencySceneE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato15DependencySceneE_t>.metaTypes,
    nullptr
} };

void KPlato::DependencyScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DependencyScene *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connectorClicked((*reinterpret_cast<std::add_pointer_t<KPlato::DependencyConnectorItem*>>(_a[1]))); break;
        case 1: _t->connectItems((*reinterpret_cast<std::add_pointer_t<KPlato::DependencyConnectorItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::DependencyConnectorItem*>>(_a[2]))); break;
        case 2: _t->itemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QGraphicsItem*>>(_a[1]))); break;
        case 3: _t->contextMenuRequested((*reinterpret_cast<std::add_pointer_t<QGraphicsItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 4: _t->dependencyContextMenuRequested((*reinterpret_cast<std::add_pointer_t<KPlato::DependencyLinkItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::DependencyConnectorItem*>>(_a[2]))); break;
        case 5: _t->update(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphicsItem* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphicsItem* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DependencyScene::*)(KPlato::DependencyConnectorItem * )>(_a, &DependencyScene::connectorClicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DependencyScene::*)(KPlato::DependencyConnectorItem * , KPlato::DependencyConnectorItem * )>(_a, &DependencyScene::connectItems, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DependencyScene::*)(QGraphicsItem * )>(_a, &DependencyScene::itemDoubleClicked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DependencyScene::*)(QGraphicsItem * , const QPoint & )>(_a, &DependencyScene::contextMenuRequested, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DependencyScene::*)(KPlato::DependencyLinkItem * , KPlato::DependencyConnectorItem * )>(_a, &DependencyScene::dependencyContextMenuRequested, 4))
            return;
    }
}

const QMetaObject *KPlato::DependencyScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::DependencyScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15DependencySceneE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsScene::qt_metacast(_clname);
}

int KPlato::DependencyScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsScene::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void KPlato::DependencyScene::connectorClicked(KPlato::DependencyConnectorItem * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::DependencyScene::connectItems(KPlato::DependencyConnectorItem * _t1, KPlato::DependencyConnectorItem * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KPlato::DependencyScene::itemDoubleClicked(QGraphicsItem * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlato::DependencyScene::contextMenuRequested(QGraphicsItem * _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void KPlato::DependencyScene::dependencyContextMenuRequested(KPlato::DependencyLinkItem * _t1, KPlato::DependencyConnectorItem * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN6KPlato14DependencyViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::DependencyView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato14DependencyViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::DependencyView",
        "selectionChanged",
        "",
        "QList<QGraphicsItem*>",
        "focusItemChanged",
        "QGraphicsItem*",
        "item",
        "makeConnection",
        "KPlato::DependencyConnectorItem*",
        "pred",
        "succ",
        "contextMenuRequested",
        "QPoint",
        "slotNodeAdded",
        "KPlato::Node*",
        "node",
        "slotNodeRemoved",
        "slotNodeChanged",
        "slotNodeMoved",
        "slotSelectionChanged",
        "slotRelationAdded",
        "KPlato::Relation*",
        "rel",
        "slotRelationRemoved",
        "slotRelationModified",
        "slotWbsCodeChanged",
        "slotSelectedItems",
        "slotConnectorClicked",
        "slotContextMenuRequested",
        "pos",
        "slotDependencyContextMenuRequested",
        "KPlato::DependencyLinkItem*",
        "connector",
        "slotFocusItemChanged",
        "slotAutoScroll"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selectionChanged'
        QtMocHelpers::SignalData<void(const QList<QGraphicsItem*> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'focusItemChanged'
        QtMocHelpers::SignalData<void(QGraphicsItem *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'makeConnection'
        QtMocHelpers::SignalData<void(KPlato::DependencyConnectorItem *, KPlato::DependencyConnectorItem *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 8, 10 },
        }}),
        // Signal 'contextMenuRequested'
        QtMocHelpers::SignalData<void(QGraphicsItem *, const QPoint &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 }, { 0x80000000 | 12, 2 },
        }}),
        // Slot 'slotNodeAdded'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotNodeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotNodeChanged'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotNodeMoved'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRelationAdded'
        QtMocHelpers::SlotData<void(KPlato::Relation *)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'slotRelationRemoved'
        QtMocHelpers::SlotData<void(KPlato::Relation *)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'slotRelationModified'
        QtMocHelpers::SlotData<void(KPlato::Relation *)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'slotWbsCodeChanged'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSelectedItems'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotConnectorClicked'
        QtMocHelpers::SlotData<void(KPlato::DependencyConnectorItem *)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 6 },
        }}),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(QGraphicsItem *, const QPoint &)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 12, 29 },
        }}),
        // Slot 'slotDependencyContextMenuRequested'
        QtMocHelpers::SlotData<void(KPlato::DependencyLinkItem *, KPlato::DependencyConnectorItem *)>(30, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 31, 6 }, { 0x80000000 | 8, 32 },
        }}),
        // Slot 'slotFocusItemChanged'
        QtMocHelpers::SlotData<void(QGraphicsItem *)>(33, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Slot 'slotAutoScroll'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DependencyView, qt_meta_tag_ZN6KPlato14DependencyViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::DependencyView::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14DependencyViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14DependencyViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato14DependencyViewE_t>.metaTypes,
    nullptr
} };

void KPlato::DependencyView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DependencyView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<QList<QGraphicsItem*>>>(_a[1]))); break;
        case 1: _t->focusItemChanged((*reinterpret_cast<std::add_pointer_t<QGraphicsItem*>>(_a[1]))); break;
        case 2: _t->makeConnection((*reinterpret_cast<std::add_pointer_t<KPlato::DependencyConnectorItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::DependencyConnectorItem*>>(_a[2]))); break;
        case 3: _t->contextMenuRequested((*reinterpret_cast<std::add_pointer_t<QGraphicsItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 4: _t->slotNodeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 5: _t->slotNodeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 6: _t->slotNodeChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 7: _t->slotNodeMoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 8: _t->slotSelectionChanged(); break;
        case 9: _t->slotRelationAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Relation*>>(_a[1]))); break;
        case 10: _t->slotRelationRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Relation*>>(_a[1]))); break;
        case 11: _t->slotRelationModified((*reinterpret_cast<std::add_pointer_t<KPlato::Relation*>>(_a[1]))); break;
        case 12: _t->slotWbsCodeChanged(); break;
        case 13: _t->slotSelectedItems(); break;
        case 14: _t->slotConnectorClicked((*reinterpret_cast<std::add_pointer_t<KPlato::DependencyConnectorItem*>>(_a[1]))); break;
        case 15: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QGraphicsItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 16: _t->slotDependencyContextMenuRequested((*reinterpret_cast<std::add_pointer_t<KPlato::DependencyLinkItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::DependencyConnectorItem*>>(_a[2]))); break;
        case 17: _t->slotFocusItemChanged((*reinterpret_cast<std::add_pointer_t<QGraphicsItem*>>(_a[1]))); break;
        case 18: _t->slotAutoScroll(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QGraphicsItem*> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphicsItem* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphicsItem* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphicsItem* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphicsItem* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DependencyView::*)(const QList<QGraphicsItem*> & )>(_a, &DependencyView::selectionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DependencyView::*)(QGraphicsItem * )>(_a, &DependencyView::focusItemChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DependencyView::*)(KPlato::DependencyConnectorItem * , KPlato::DependencyConnectorItem * )>(_a, &DependencyView::makeConnection, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DependencyView::*)(QGraphicsItem * , const QPoint & )>(_a, &DependencyView::contextMenuRequested, 3))
            return;
    }
}

const QMetaObject *KPlato::DependencyView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::DependencyView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14DependencyViewE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int KPlato::DependencyView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void KPlato::DependencyView::selectionChanged(const QList<QGraphicsItem*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::DependencyView::focusItemChanged(QGraphicsItem * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KPlato::DependencyView::makeConnection(KPlato::DependencyConnectorItem * _t1, KPlato::DependencyConnectorItem * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void KPlato::DependencyView::contextMenuRequested(QGraphicsItem * _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN6KPlato28DependencyeditorConfigDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::DependencyeditorConfigDialog::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato28DependencyeditorConfigDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::DependencyeditorConfigDialog",
        "slotOk",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotOk'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DependencyeditorConfigDialog, qt_meta_tag_ZN6KPlato28DependencyeditorConfigDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::DependencyeditorConfigDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<KPageDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato28DependencyeditorConfigDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato28DependencyeditorConfigDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato28DependencyeditorConfigDialogE_t>.metaTypes,
    nullptr
} };

void KPlato::DependencyeditorConfigDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DependencyeditorConfigDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotOk(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *KPlato::DependencyeditorConfigDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::DependencyeditorConfigDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato28DependencyeditorConfigDialogE_t>.strings))
        return static_cast<void*>(this);
    return KPageDialog::qt_metacast(_clname);
}

int KPlato::DependencyeditorConfigDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPageDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato16DependencyEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::DependencyEditor::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato16DependencyEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::DependencyEditor",
        "setGuiActive",
        "",
        "activate",
        "slotCreateRelation",
        "KPlato::DependencyConnectorItem*",
        "pred",
        "succ",
        "setScheduleManager",
        "KPlato::ScheduleManager*",
        "sm",
        "slotOptions",
        "slotItemDoubleClicked",
        "QGraphicsItem*",
        "item",
        "slotCurrentChanged",
        "QModelIndex",
        "slotSelectionChanged",
        "QList<QGraphicsItem*>",
        "lst",
        "slotFocusItemChanged",
        "slotContextMenuRequested",
        "QPoint",
        "pos",
        "slotEnableActions",
        "slotAddTask",
        "slotAddSubtask",
        "slotAddMilestone",
        "slotAddSubMilestone",
        "slotDeleteTask",
        "slotOpenCurrentNode",
        "slotOpenNode",
        "KPlato::Node*",
        "node",
        "slotTaskProgress",
        "slotTaskDescription",
        "slotOpenProjectDescription",
        "slotOpenTaskDescription",
        "ro",
        "slotDocuments",
        "slotLinkTask",
        "slotModifyCurrentRelation",
        "slotDeleteRelation",
        "slotAddRelation",
        "par",
        "child",
        "linkType",
        "slotModifyRelation",
        "KPlato::Relation*",
        "rel",
        "slotAddTaskFinished",
        "result",
        "slotAddMilestoneFinished",
        "slotAddSubMilestoneFinished",
        "slotAddSubTaskFinished",
        "slotTaskEditFinished",
        "slotSummaryTaskEditFinished",
        "slotTaskProgressFinished",
        "slotMilestoneProgressFinished",
        "slotTaskDescriptionFinished",
        "slotDocumentsFinished",
        "slotAddRelationFinished",
        "slotModifyRelationFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setGuiActive'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'slotCreateRelation'
        QtMocHelpers::SlotData<void(KPlato::DependencyConnectorItem *, KPlato::DependencyConnectorItem *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 5, 7 },
        }}),
        // Slot 'setScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotOptions'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotItemDoubleClicked'
        QtMocHelpers::SlotData<void(QGraphicsItem *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'slotCurrentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 2 }, { 0x80000000 | 16, 2 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QList<QGraphicsItem*> &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'slotFocusItemChanged'
        QtMocHelpers::SlotData<void(QGraphicsItem *)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(QGraphicsItem *, const QPoint &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotEnableActions'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddTask'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddSubtask'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddMilestone'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddSubMilestone'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDeleteTask'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenCurrentNode'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenNode'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'slotTaskProgress'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTaskDescription'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenProjectDescription'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenTaskDescription'
        QtMocHelpers::SlotData<void(bool)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'slotDocuments'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotLinkTask'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotModifyCurrentRelation'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDeleteRelation'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddRelation'
        QtMocHelpers::SlotData<void(KPlato::Node *, KPlato::Node *, int)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 44 }, { 0x80000000 | 32, 45 }, { QMetaType::Int, 46 },
        }}),
        // Slot 'slotModifyRelation'
        QtMocHelpers::SlotData<void(KPlato::Relation *, int)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 }, { QMetaType::Int, 46 },
        }}),
        // Slot 'slotAddTaskFinished'
        QtMocHelpers::SlotData<void(int)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
        // Slot 'slotAddMilestoneFinished'
        QtMocHelpers::SlotData<void(int)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
        // Slot 'slotAddSubMilestoneFinished'
        QtMocHelpers::SlotData<void(int)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
        // Slot 'slotAddSubTaskFinished'
        QtMocHelpers::SlotData<void(int)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
        // Slot 'slotTaskEditFinished'
        QtMocHelpers::SlotData<void(int)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
        // Slot 'slotSummaryTaskEditFinished'
        QtMocHelpers::SlotData<void(int)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
        // Slot 'slotTaskProgressFinished'
        QtMocHelpers::SlotData<void(int)>(57, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
        // Slot 'slotMilestoneProgressFinished'
        QtMocHelpers::SlotData<void(int)>(58, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
        // Slot 'slotTaskDescriptionFinished'
        QtMocHelpers::SlotData<void(int)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
        // Slot 'slotDocumentsFinished'
        QtMocHelpers::SlotData<void(int)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
        // Slot 'slotAddRelationFinished'
        QtMocHelpers::SlotData<void(int)>(61, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
        // Slot 'slotModifyRelationFinished'
        QtMocHelpers::SlotData<void(int)>(62, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DependencyEditor, qt_meta_tag_ZN6KPlato16DependencyEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::DependencyEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<ViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16DependencyEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16DependencyEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato16DependencyEditorE_t>.metaTypes,
    nullptr
} };

void KPlato::DependencyEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DependencyEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setGuiActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->slotCreateRelation((*reinterpret_cast<std::add_pointer_t<KPlato::DependencyConnectorItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::DependencyConnectorItem*>>(_a[2]))); break;
        case 2: _t->setScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 3: _t->slotOptions(); break;
        case 4: _t->slotItemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QGraphicsItem*>>(_a[1]))); break;
        case 5: _t->slotCurrentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 6: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QList<QGraphicsItem*>>>(_a[1]))); break;
        case 7: _t->slotFocusItemChanged((*reinterpret_cast<std::add_pointer_t<QGraphicsItem*>>(_a[1]))); break;
        case 8: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QGraphicsItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 9: _t->slotEnableActions(); break;
        case 10: _t->slotAddTask(); break;
        case 11: _t->slotAddSubtask(); break;
        case 12: _t->slotAddMilestone(); break;
        case 13: _t->slotAddSubMilestone(); break;
        case 14: _t->slotDeleteTask(); break;
        case 15: _t->slotOpenCurrentNode(); break;
        case 16: _t->slotOpenNode((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 17: _t->slotTaskProgress(); break;
        case 18: _t->slotTaskDescription(); break;
        case 19: _t->slotOpenProjectDescription(); break;
        case 20: _t->slotOpenTaskDescription((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->slotDocuments(); break;
        case 22: _t->slotLinkTask(); break;
        case 23: _t->slotModifyCurrentRelation(); break;
        case 24: _t->slotDeleteRelation(); break;
        case 25: _t->slotAddRelation((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 26: _t->slotModifyRelation((*reinterpret_cast<std::add_pointer_t<KPlato::Relation*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 27: _t->slotAddTaskFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->slotAddMilestoneFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 29: _t->slotAddSubMilestoneFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 30: _t->slotAddSubTaskFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 31: _t->slotTaskEditFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 32: _t->slotSummaryTaskEditFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 33: _t->slotTaskProgressFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 34: _t->slotMilestoneProgressFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 35: _t->slotTaskDescriptionFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 36: _t->slotDocumentsFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 37: _t->slotAddRelationFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 38: _t->slotModifyRelationFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphicsItem* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QGraphicsItem*> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphicsItem* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphicsItem* >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KPlato::DependencyEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::DependencyEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16DependencyEditorE_t>.strings))
        return static_cast<void*>(this);
    return ViewBase::qt_metacast(_clname);
}

int KPlato::DependencyEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 39)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 39;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 39)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 39;
    }
    return _id;
}
QT_WARNING_POP
