/********************************************************************************
** Form generated from reading UI file 'GanttFilterOptionsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GANTTFILTEROPTIONSWIDGET_H
#define UI_GANTTFILTEROPTIONSWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_GanttFilterOptionsWidget
{
public:
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QPushButton *closeButton;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_5;
    QCheckBox *show_project;
    QGroupBox *tasks_and_milestones_group;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *show_tasks;
    QCheckBox *show_milestones;
    QCheckBox *show_unscheduled_tasks;
    QGroupBox *period_group;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *show_running;
    QCheckBox *show_finished;
    QCheckBox *show_not_started;
    QHBoxLayout *horizontalLayout_2;
    QRadioButton *enable_period;
    QSpinBox *period;
    QSpacerItem *horizontalSpacer;
    QRadioButton *enable_spesific_period;
    QDateEdit *start_date;
    QDateEdit *end_date;
    QSpacerItem *verticalSpacer;

    void setupUi(QFrame *KPlato__GanttFilterOptionsWidget)
    {
        if (KPlato__GanttFilterOptionsWidget->objectName().isEmpty())
            KPlato__GanttFilterOptionsWidget->setObjectName("KPlato__GanttFilterOptionsWidget");
        KPlato__GanttFilterOptionsWidget->resize(195, 480);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(KPlato__GanttFilterOptionsWidget->sizePolicy().hasHeightForWidth());
        KPlato__GanttFilterOptionsWidget->setSizePolicy(sizePolicy);
        KPlato__GanttFilterOptionsWidget->setMinimumSize(QSize(195, 0));
        KPlato__GanttFilterOptionsWidget->setMaximumSize(QSize(16777215, 16777215));
        KPlato__GanttFilterOptionsWidget->setWindowTitle(QString::fromUtf8(""));
        verticalLayout_4 = new QVBoxLayout(KPlato__GanttFilterOptionsWidget);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, -1, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(KPlato__GanttFilterOptionsWidget);
        label->setObjectName("label");
        label->setScaledContents(true);
        label->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(label);

        closeButton = new QPushButton(KPlato__GanttFilterOptionsWidget);
        closeButton->setObjectName("closeButton");
        closeButton->setMaximumSize(QSize(16, 16));
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("dialog-close");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        closeButton->setIcon(icon);

        horizontalLayout->addWidget(closeButton);


        verticalLayout_4->addLayout(horizontalLayout);

        scrollArea = new QScrollArea(KPlato__GanttFilterOptionsWidget);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 191, 444));
        verticalLayout = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(scrollAreaWidgetContents);
        groupBox->setObjectName("groupBox");
        verticalLayout_5 = new QVBoxLayout(groupBox);
        verticalLayout_5->setObjectName("verticalLayout_5");
        show_project = new QCheckBox(groupBox);
        show_project->setObjectName("show_project");

        verticalLayout_5->addWidget(show_project);


        verticalLayout->addWidget(groupBox);

        tasks_and_milestones_group = new QGroupBox(scrollAreaWidgetContents);
        tasks_and_milestones_group->setObjectName("tasks_and_milestones_group");
        tasks_and_milestones_group->setCheckable(true);
        verticalLayout_2 = new QVBoxLayout(tasks_and_milestones_group);
        verticalLayout_2->setObjectName("verticalLayout_2");
        show_tasks = new QCheckBox(tasks_and_milestones_group);
        show_tasks->setObjectName("show_tasks");

        verticalLayout_2->addWidget(show_tasks);

        show_milestones = new QCheckBox(tasks_and_milestones_group);
        show_milestones->setObjectName("show_milestones");

        verticalLayout_2->addWidget(show_milestones);

        show_unscheduled_tasks = new QCheckBox(tasks_and_milestones_group);
        show_unscheduled_tasks->setObjectName("show_unscheduled_tasks");

        verticalLayout_2->addWidget(show_unscheduled_tasks);


        verticalLayout->addWidget(tasks_and_milestones_group);

        period_group = new QGroupBox(scrollAreaWidgetContents);
        period_group->setObjectName("period_group");
        period_group->setCheckable(true);
        verticalLayout_3 = new QVBoxLayout(period_group);
        verticalLayout_3->setObjectName("verticalLayout_3");
        show_running = new QCheckBox(period_group);
        show_running->setObjectName("show_running");

        verticalLayout_3->addWidget(show_running);

        show_finished = new QCheckBox(period_group);
        show_finished->setObjectName("show_finished");

        verticalLayout_3->addWidget(show_finished);

        show_not_started = new QCheckBox(period_group);
        show_not_started->setObjectName("show_not_started");

        verticalLayout_3->addWidget(show_not_started);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        enable_period = new QRadioButton(period_group);
        enable_period->setObjectName("enable_period");

        horizontalLayout_2->addWidget(enable_period);

        period = new QSpinBox(period_group);
        period->setObjectName("period");
        period->setMinimum(1);
        period->setValue(7);

        horizontalLayout_2->addWidget(period);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout_3->addLayout(horizontalLayout_2);

        enable_spesific_period = new QRadioButton(period_group);
        enable_spesific_period->setObjectName("enable_spesific_period");

        verticalLayout_3->addWidget(enable_spesific_period);

        start_date = new QDateEdit(period_group);
        start_date->setObjectName("start_date");
        start_date->setCalendarPopup(true);

        verticalLayout_3->addWidget(start_date);

        end_date = new QDateEdit(period_group);
        end_date->setObjectName("end_date");
        end_date->setCalendarPopup(true);

        verticalLayout_3->addWidget(end_date);

        verticalSpacer = new QSpacerItem(20, 124, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        verticalLayout->addWidget(period_group);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_4->addWidget(scrollArea);


        retranslateUi(KPlato__GanttFilterOptionsWidget);

        QMetaObject::connectSlotsByName(KPlato__GanttFilterOptionsWidget);
    } // setupUi

    void retranslateUi(QFrame *KPlato__GanttFilterOptionsWidget)
    {
        label->setText(tr2i18n("Filter Options", nullptr));
        closeButton->setText(QString());
        groupBox->setTitle(QString());
        show_project->setText(tr2i18n("Show project", nullptr));
        tasks_and_milestones_group->setTitle(tr2i18n("Tasks and Milestones", nullptr));
        show_tasks->setText(tr2i18n("Tasks", nullptr));
        show_milestones->setText(tr2i18n("Milestones", nullptr));
        show_unscheduled_tasks->setText(tr2i18n("Unscheduled tasks", nullptr));
        period_group->setTitle(tr2i18n("Period", nullptr));
        show_running->setText(tr2i18n("Running tasks", nullptr));
        show_finished->setText(tr2i18n("Finished tasks", nullptr));
        show_not_started->setText(tr2i18n("Not started tasks", nullptr));
#if QT_CONFIG(tooltip)
        enable_period->setToolTip(tr2i18n("Number of days around current datetime", nullptr));
#endif // QT_CONFIG(tooltip)
        enable_period->setText(tr2i18n("Period:", nullptr));
#if QT_CONFIG(tooltip)
        period->setToolTip(tr2i18n("Number of days around current datetime", nullptr));
#endif // QT_CONFIG(tooltip)
        period->setSuffix(QString());
        enable_spesific_period->setText(tr2i18n("Specific period:", nullptr));
        (void)KPlato__GanttFilterOptionsWidget;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class GanttFilterOptionsWidget: public Ui_GanttFilterOptionsWidget {};
} // namespace Ui
} // namespace KPlato

#endif // GANTTFILTEROPTIONSWIDGET_H

