/****************************************************************************
** Meta object code from reading C++ file 'kptscheduleeditor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/kptscheduleeditor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptscheduleeditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato16ScheduleTreeViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ScheduleTreeView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato16ScheduleTreeViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ScheduleTreeView",
        "currentIndexChanged",
        "",
        "QModelIndex",
        "currentColumnChanged",
        "selectedIndexesChanged",
        "QModelIndexList",
        "selectionChanged",
        "QItemSelection",
        "selected",
        "deselected",
        "currentChanged",
        "current",
        "previous"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentIndexChanged'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'currentColumnChanged'
        QtMocHelpers::SignalData<void(const QModelIndex &, const QModelIndex &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 3, 2 },
        }}),
        // Signal 'selectedIndexesChanged'
        QtMocHelpers::SignalData<void(const QModelIndexList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Slot 'selectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 8, 10 },
        }}),
        // Slot 'currentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 12 }, { 0x80000000 | 3, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ScheduleTreeView, qt_meta_tag_ZN6KPlato16ScheduleTreeViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ScheduleTreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<TreeViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16ScheduleTreeViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16ScheduleTreeViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato16ScheduleTreeViewE_t>.metaTypes,
    nullptr
} };

void KPlato::ScheduleTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScheduleTreeView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentIndexChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 1: _t->currentColumnChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 2: _t->selectedIndexesChanged((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 3: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 4: _t->currentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ScheduleTreeView::*)(const QModelIndex & )>(_a, &ScheduleTreeView::currentIndexChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleTreeView::*)(const QModelIndex & , const QModelIndex & )>(_a, &ScheduleTreeView::currentColumnChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleTreeView::*)(const QModelIndexList & )>(_a, &ScheduleTreeView::selectedIndexesChanged, 2))
            return;
    }
}

const QMetaObject *KPlato::ScheduleTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ScheduleTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16ScheduleTreeViewE_t>.strings))
        return static_cast<void*>(this);
    return TreeViewBase::qt_metacast(_clname);
}

int KPlato::ScheduleTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TreeViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ScheduleTreeView::currentIndexChanged(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::ScheduleTreeView::currentColumnChanged(const QModelIndex & _t1, const QModelIndex & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KPlato::ScheduleTreeView::selectedIndexesChanged(const QModelIndexList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN6KPlato14ScheduleEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ScheduleEditor::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato14ScheduleEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ScheduleEditor",
        "calculateSchedule",
        "",
        "KPlato::Project*",
        "KPlato::ScheduleManager*",
        "baselineSchedule",
        "addScheduleManager",
        "deleteScheduleManager",
        "SelectionScheduleChanged",
        "scheduleSelectionChanged",
        "sm",
        "moveScheduleManager",
        "newparent",
        "index",
        "setGuiActive",
        "activate",
        "slotOptions",
        "slotContextMenuRequested",
        "QModelIndex",
        "QPoint",
        "pos",
        "slotSelectionChanged",
        "QModelIndexList",
        "slotCurrentChanged",
        "updateActionsEnabled",
        "slotEnableActions",
        "slotCalculateSchedule",
        "slotBaselineSchedule",
        "slotAddSchedule",
        "slotAddSubSchedule",
        "slotDeleteSelection",
        "slotMoveLeft",
        "slotClaimSchedule"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'calculateSchedule'
        QtMocHelpers::SignalData<void(KPlato::Project *, KPlato::ScheduleManager *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 },
        }}),
        // Signal 'baselineSchedule'
        QtMocHelpers::SignalData<void(KPlato::Project *, KPlato::ScheduleManager *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 },
        }}),
        // Signal 'addScheduleManager'
        QtMocHelpers::SignalData<void(KPlato::Project *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'deleteScheduleManager'
        QtMocHelpers::SignalData<void(KPlato::Project *, KPlato::ScheduleManager *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 },
        }}),
        // Signal 'SelectionScheduleChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'scheduleSelectionChanged'
        QtMocHelpers::SignalData<void(KPlato::ScheduleManager *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 10 },
        }}),
        // Signal 'moveScheduleManager'
        QtMocHelpers::SignalData<void(KPlato::ScheduleManager *, KPlato::ScheduleManager *, int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 10 }, { 0x80000000 | 4, 12 }, { QMetaType::Int, 13 },
        }}),
        // Slot 'setGuiActive'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'slotOptions'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QPoint &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 13 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QModelIndexList &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 2 },
        }}),
        // Slot 'slotCurrentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 2 },
        }}),
        // Slot 'updateActionsEnabled'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 13 },
        }}),
        // Slot 'slotEnableActions'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCalculateSchedule'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotBaselineSchedule'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddSchedule'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddSubSchedule'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDeleteSelection'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMoveLeft'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotClaimSchedule'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ScheduleEditor, qt_meta_tag_ZN6KPlato14ScheduleEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ScheduleEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<ViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14ScheduleEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14ScheduleEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato14ScheduleEditorE_t>.metaTypes,
    nullptr
} };

void KPlato::ScheduleEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScheduleEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->calculateSchedule((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[2]))); break;
        case 1: _t->baselineSchedule((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[2]))); break;
        case 2: _t->addScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1]))); break;
        case 3: _t->deleteScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[2]))); break;
        case 4: _t->SelectionScheduleChanged(); break;
        case 5: _t->scheduleSelectionChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 6: _t->moveScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 7: _t->setGuiActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->slotOptions(); break;
        case 9: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 10: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 11: _t->slotCurrentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 12: _t->updateActionsEnabled((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 13: _t->slotEnableActions(); break;
        case 14: _t->slotCalculateSchedule(); break;
        case 15: _t->slotBaselineSchedule(); break;
        case 16: _t->slotAddSchedule(); break;
        case 17: _t->slotAddSubSchedule(); break;
        case 18: _t->slotDeleteSelection(); break;
        case 19: _t->slotMoveLeft(); break;
        case 20: _t->slotClaimSchedule(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ScheduleEditor::*)(KPlato::Project * , KPlato::ScheduleManager * )>(_a, &ScheduleEditor::calculateSchedule, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleEditor::*)(KPlato::Project * , KPlato::ScheduleManager * )>(_a, &ScheduleEditor::baselineSchedule, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleEditor::*)(KPlato::Project * )>(_a, &ScheduleEditor::addScheduleManager, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleEditor::*)(KPlato::Project * , KPlato::ScheduleManager * )>(_a, &ScheduleEditor::deleteScheduleManager, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleEditor::*)()>(_a, &ScheduleEditor::SelectionScheduleChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleEditor::*)(KPlato::ScheduleManager * )>(_a, &ScheduleEditor::scheduleSelectionChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleEditor::*)(KPlato::ScheduleManager * , KPlato::ScheduleManager * , int )>(_a, &ScheduleEditor::moveScheduleManager, 6))
            return;
    }
}

const QMetaObject *KPlato::ScheduleEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ScheduleEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14ScheduleEditorE_t>.strings))
        return static_cast<void*>(this);
    return ViewBase::qt_metacast(_clname);
}

int KPlato::ScheduleEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ScheduleEditor::calculateSchedule(KPlato::Project * _t1, KPlato::ScheduleManager * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KPlato::ScheduleEditor::baselineSchedule(KPlato::Project * _t1, KPlato::ScheduleManager * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KPlato::ScheduleEditor::addScheduleManager(KPlato::Project * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlato::ScheduleEditor::deleteScheduleManager(KPlato::Project * _t1, KPlato::ScheduleManager * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void KPlato::ScheduleEditor::SelectionScheduleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KPlato::ScheduleEditor::scheduleSelectionChanged(KPlato::ScheduleManager * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KPlato::ScheduleEditor::moveScheduleManager(KPlato::ScheduleManager * _t1, KPlato::ScheduleManager * _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3);
}
namespace {
struct qt_meta_tag_ZN6KPlato19ScheduleLogTreeViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ScheduleLogTreeView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato19ScheduleLogTreeViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ScheduleLogTreeView",
        "currentIndexChanged",
        "",
        "QModelIndex",
        "currentColumnChanged",
        "selectedIndexesChanged",
        "QModelIndexList",
        "contextMenuRequested",
        "QPoint",
        "slotEditCopy",
        "contextMenuEvent",
        "QContextMenuEvent*",
        "e",
        "headerContextMenuRequested",
        "pos",
        "selectionChanged",
        "QItemSelection",
        "selected",
        "deselected",
        "currentChanged",
        "current",
        "previous",
        "slotShowDebug"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentIndexChanged'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'currentColumnChanged'
        QtMocHelpers::SignalData<void(const QModelIndex &, const QModelIndex &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 3, 2 },
        }}),
        // Signal 'selectedIndexesChanged'
        QtMocHelpers::SignalData<void(const QModelIndexList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Signal 'contextMenuRequested'
        QtMocHelpers::SignalData<void(const QModelIndex &, const QPoint &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 8, 2 },
        }}),
        // Slot 'slotEditCopy'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'contextMenuEvent'
        QtMocHelpers::SlotData<void(QContextMenuEvent *)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'headerContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 14 },
        }}),
        // Slot 'selectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { 0x80000000 | 16, 18 },
        }}),
        // Slot 'currentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 20 }, { 0x80000000 | 3, 21 },
        }}),
        // Slot 'slotShowDebug'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ScheduleLogTreeView, qt_meta_tag_ZN6KPlato19ScheduleLogTreeViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ScheduleLogTreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19ScheduleLogTreeViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19ScheduleLogTreeViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato19ScheduleLogTreeViewE_t>.metaTypes,
    nullptr
} };

void KPlato::ScheduleLogTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScheduleLogTreeView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentIndexChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 1: _t->currentColumnChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 2: _t->selectedIndexesChanged((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 3: _t->contextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 4: _t->slotEditCopy(); break;
        case 5: _t->contextMenuEvent((*reinterpret_cast<std::add_pointer_t<QContextMenuEvent*>>(_a[1]))); break;
        case 6: _t->headerContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 7: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 8: _t->currentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 9: _t->slotShowDebug((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ScheduleLogTreeView::*)(const QModelIndex & )>(_a, &ScheduleLogTreeView::currentIndexChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleLogTreeView::*)(const QModelIndex & , const QModelIndex & )>(_a, &ScheduleLogTreeView::currentColumnChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleLogTreeView::*)(const QModelIndexList & )>(_a, &ScheduleLogTreeView::selectedIndexesChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleLogTreeView::*)(const QModelIndex & , const QPoint & )>(_a, &ScheduleLogTreeView::contextMenuRequested, 3))
            return;
    }
}

const QMetaObject *KPlato::ScheduleLogTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ScheduleLogTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19ScheduleLogTreeViewE_t>.strings))
        return static_cast<void*>(this);
    return QTreeView::qt_metacast(_clname);
}

int KPlato::ScheduleLogTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ScheduleLogTreeView::currentIndexChanged(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::ScheduleLogTreeView::currentColumnChanged(const QModelIndex & _t1, const QModelIndex & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KPlato::ScheduleLogTreeView::selectedIndexesChanged(const QModelIndexList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlato::ScheduleLogTreeView::contextMenuRequested(const QModelIndex & _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN6KPlato15ScheduleLogViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ScheduleLogView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato15ScheduleLogViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ScheduleLogView",
        "editNode",
        "",
        "KPlato::Node*",
        "node",
        "editResource",
        "KPlato::Resource*",
        "resource",
        "setGuiActive",
        "activate",
        "slotEditCopy",
        "slotEdit",
        "slotOptions",
        "slotContextMenuRequested",
        "QModelIndex",
        "index",
        "QPoint",
        "pos",
        "slotScheduleSelectionChanged",
        "KPlato::ScheduleManager*",
        "sm",
        "slotEnableActions",
        "const KPlato::ScheduleManager*",
        "slotSelectionChanged",
        "QModelIndexList",
        "slotCurrentChanged",
        "updateActionsEnabled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'editNode'
        QtMocHelpers::SignalData<void(KPlato::Node *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'editResource'
        QtMocHelpers::SignalData<void(KPlato::Resource *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setGuiActive'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'slotEditCopy'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEdit'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOptions'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QPoint &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotScheduleSelectionChanged'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotEnableActions'
        QtMocHelpers::SlotData<void(const KPlato::ScheduleManager *)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 20 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QModelIndexList &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 2 },
        }}),
        // Slot 'slotCurrentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Slot 'updateActionsEnabled'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ScheduleLogView, qt_meta_tag_ZN6KPlato15ScheduleLogViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ScheduleLogView::staticMetaObject = { {
    QMetaObject::SuperData::link<ViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15ScheduleLogViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15ScheduleLogViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato15ScheduleLogViewE_t>.metaTypes,
    nullptr
} };

void KPlato::ScheduleLogView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScheduleLogView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->editNode((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 1: _t->editResource((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 2: _t->setGuiActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->slotEditCopy(); break;
        case 4: _t->slotEdit(); break;
        case 5: _t->slotOptions(); break;
        case 6: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 7: _t->slotScheduleSelectionChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 8: _t->slotEnableActions((*reinterpret_cast<std::add_pointer_t<const KPlato::ScheduleManager*>>(_a[1]))); break;
        case 9: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 10: _t->slotCurrentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 11: _t->updateActionsEnabled((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ScheduleLogView::*)(KPlato::Node * )>(_a, &ScheduleLogView::editNode, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScheduleLogView::*)(KPlato::Resource * )>(_a, &ScheduleLogView::editResource, 1))
            return;
    }
}

const QMetaObject *KPlato::ScheduleLogView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ScheduleLogView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15ScheduleLogViewE_t>.strings))
        return static_cast<void*>(this);
    return ViewBase::qt_metacast(_clname);
}

int KPlato::ScheduleLogView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ScheduleLogView::editNode(KPlato::Node * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::ScheduleLogView::editResource(KPlato::Resource * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN6KPlato19ScheduleHandlerViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ScheduleHandlerView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato19ScheduleHandlerViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ScheduleHandlerView",
        "currentScheduleManagerChanged",
        "",
        "KPlato::ScheduleManager*",
        "setGuiActive",
        "activate",
        "slotGuiActivated",
        "KPlato::ViewBase*",
        "v",
        "active",
        "currentTabChanged",
        "i",
        "slotOpenNode",
        "KPlato::Node*",
        "node",
        "slotTaskEditFinished",
        "result",
        "slotSummaryTaskEditFinished",
        "slotEditResource",
        "KPlato::Resource*",
        "resource",
        "slotEditResourceFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentScheduleManagerChanged'
        QtMocHelpers::SignalData<void(KPlato::ScheduleManager *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'setGuiActive'
        QtMocHelpers::SlotData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'slotGuiActivated'
        QtMocHelpers::SlotData<void(KPlato::ViewBase *, bool)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'currentTabChanged'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'slotOpenNode'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'slotTaskEditFinished'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'slotSummaryTaskEditFinished'
        QtMocHelpers::SlotData<void(int)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'slotEditResource'
        QtMocHelpers::SlotData<void(KPlato::Resource *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotEditResourceFinished'
        QtMocHelpers::SlotData<void(int)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ScheduleHandlerView, qt_meta_tag_ZN6KPlato19ScheduleHandlerViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ScheduleHandlerView::staticMetaObject = { {
    QMetaObject::SuperData::link<SplitterView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19ScheduleHandlerViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19ScheduleHandlerViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato19ScheduleHandlerViewE_t>.metaTypes,
    nullptr
} };

void KPlato::ScheduleHandlerView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScheduleHandlerView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentScheduleManagerChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 1: _t->setGuiActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->slotGuiActivated((*reinterpret_cast<std::add_pointer_t<KPlato::ViewBase*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->currentTabChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->slotOpenNode((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 5: _t->slotTaskEditFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->slotSummaryTaskEditFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->slotEditResource((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 8: _t->slotEditResourceFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ViewBase* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ScheduleHandlerView::*)(KPlato::ScheduleManager * )>(_a, &ScheduleHandlerView::currentScheduleManagerChanged, 0))
            return;
    }
}

const QMetaObject *KPlato::ScheduleHandlerView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ScheduleHandlerView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19ScheduleHandlerViewE_t>.strings))
        return static_cast<void*>(this);
    return SplitterView::qt_metacast(_clname);
}

int KPlato::ScheduleHandlerView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SplitterView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ScheduleHandlerView::currentScheduleManagerChanged(KPlato::ScheduleManager * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN6KPlato15SchedulingRangeE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::SchedulingRange::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato15SchedulingRangeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::SchedulingRange",
        "setProject",
        "",
        "KPlato::Project*",
        "project",
        "slotProjectChanged",
        "KPlato::Node*",
        "slotStartChanged",
        "slotEndChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setProject'
        QtMocHelpers::SlotData<void(KPlato::Project *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotProjectChanged'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Slot 'slotStartChanged'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEndChanged'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SchedulingRange, qt_meta_tag_ZN6KPlato15SchedulingRangeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::SchedulingRange::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15SchedulingRangeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15SchedulingRangeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato15SchedulingRangeE_t>.metaTypes,
    nullptr
} };

void KPlato::SchedulingRange::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SchedulingRange *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setProject((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1]))); break;
        case 1: _t->slotProjectChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 2: _t->slotStartChanged(); break;
        case 3: _t->slotEndChanged(); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::SchedulingRange::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::SchedulingRange::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15SchedulingRangeE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::SchedulingRange"))
        return static_cast< Ui::SchedulingRange*>(this);
    return QWidget::qt_metacast(_clname);
}

int KPlato::SchedulingRange::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
QT_WARNING_POP
