/* This file is part of the KDE project
 *  SPDX-FileCopyrightText: 2021 Dag Andersen <dag.andersen@kdemail.net>
 * 
 *  SPDX-License-Identifier: LGPL-2.0-or-later
 */

#ifndef PLAN_MIMETYPES_H
#define PLAN_MIMETYPES_H

#include <QLatin1String>

static const QLatin1String PLAN_MIME_TYPE("application/x-vnd.kde.plan");
static const QLatin1String PLANPORTFOLIO_MIME_TYPE("application/x-vnd.kde.plan.portfolio");
static const QLatin1String KPLATO_MIME_TYPE("application/x-vnd.kde.kplato");
static const QLatin1String KPLATOWORK_MIME_TYPE("application/x-vnd.kde.kplato.work");
static const QLatin1String PLANWORK_MIME_TYPE("application/x-vnd.kde.plan.work");

#endif
