/********************************************************************************
** Form generated from reading UI file 'kptintervaleditbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTINTERVALEDITBASE_H
#define UI_KPTINTERVALEDITBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTimeEdit>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_IntervalEditBase
{
public:
    QVBoxLayout *verticalLayout;
    QTreeWidget *intervalList;
    QHBoxLayout *horizontalLayout;
    QLabel *textLabel1;
    QTimeEdit *startTime;
    QLabel *textLabel2;
    QDoubleSpinBox *length;
    QPushButton *bAddInterval;
    QPushButton *bRemoveInterval;
    QSpacerItem *spacerItem;
    QPushButton *bClear;

    void setupUi(QWidget *KPlato__IntervalEditBase)
    {
        if (KPlato__IntervalEditBase->objectName().isEmpty())
            KPlato__IntervalEditBase->setObjectName("KPlato__IntervalEditBase");
        KPlato__IntervalEditBase->resize(502, 225);
        verticalLayout = new QVBoxLayout(KPlato__IntervalEditBase);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        intervalList = new QTreeWidget(KPlato__IntervalEditBase);
        intervalList->setObjectName("intervalList");

        verticalLayout->addWidget(intervalList);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        textLabel1 = new QLabel(KPlato__IntervalEditBase);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        horizontalLayout->addWidget(textLabel1);

        startTime = new QTimeEdit(KPlato__IntervalEditBase);
        startTime->setObjectName("startTime");

        horizontalLayout->addWidget(startTime);

        textLabel2 = new QLabel(KPlato__IntervalEditBase);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        horizontalLayout->addWidget(textLabel2);

        length = new QDoubleSpinBox(KPlato__IntervalEditBase);
        length->setObjectName("length");
        length->setMaximum(24.000000000000000);
        length->setValue(8.000000000000000);

        horizontalLayout->addWidget(length);

        bAddInterval = new QPushButton(KPlato__IntervalEditBase);
        bAddInterval->setObjectName("bAddInterval");

        horizontalLayout->addWidget(bAddInterval);

        bRemoveInterval = new QPushButton(KPlato__IntervalEditBase);
        bRemoveInterval->setObjectName("bRemoveInterval");

        horizontalLayout->addWidget(bRemoveInterval);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(spacerItem);

        bClear = new QPushButton(KPlato__IntervalEditBase);
        bClear->setObjectName("bClear");

        horizontalLayout->addWidget(bClear);


        verticalLayout->addLayout(horizontalLayout);

#if QT_CONFIG(shortcut)
        textLabel1->setBuddy(startTime);
        textLabel2->setBuddy(length);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(intervalList, startTime);
        QWidget::setTabOrder(startTime, length);
        QWidget::setTabOrder(length, bAddInterval);
        QWidget::setTabOrder(bAddInterval, bRemoveInterval);
        QWidget::setTabOrder(bRemoveInterval, bClear);

        retranslateUi(KPlato__IntervalEditBase);

        QMetaObject::connectSlotsByName(KPlato__IntervalEditBase);
    } // setupUi

    void retranslateUi(QWidget *KPlato__IntervalEditBase)
    {
        QTreeWidgetItem *___qtreewidgetitem = intervalList->headerItem();
        ___qtreewidgetitem->setText(0, tr2i18n("1", nullptr));
        textLabel1->setText(tr2i18n("From:", "Start time"));
        textLabel2->setText(tr2i18n("Length", nullptr));
        bAddInterval->setText(tr2i18n("Add", nullptr));
        bRemoveInterval->setText(tr2i18n("Remove", nullptr));
        bClear->setText(tr2i18n("Clear", nullptr));
        (void)KPlato__IntervalEditBase;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class IntervalEditBase: public Ui_IntervalEditBase {};
} // namespace Ui
} // namespace KPlato

#endif // KPTINTERVALEDITBASE_H

