/********************************************************************************
** Form generated from reading UI file 'kptprintingheaderfooter.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTPRINTINGHEADERFOOTER_H
#define UI_KPTPRINTINGHEADERFOOTER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_PrintingHeaderFooter
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *ui_header;
    QGridLayout *gridLayout_2;
    QCheckBox *ui_headerProject;
    QCheckBox *ui_headerPage;
    QCheckBox *ui_headerManager;
    QCheckBox *ui_headerDate;
    QGroupBox *ui_footer;
    QGridLayout *gridLayout;
    QCheckBox *ui_footerProject;
    QCheckBox *ui_footerPage;
    QCheckBox *ui_footerManager;
    QCheckBox *ui_footerDate;

    void setupUi(QWidget *KPlato__PrintingHeaderFooter)
    {
        if (KPlato__PrintingHeaderFooter->objectName().isEmpty())
            KPlato__PrintingHeaderFooter->setObjectName("KPlato__PrintingHeaderFooter");
        KPlato__PrintingHeaderFooter->resize(254, 194);
        verticalLayout = new QVBoxLayout(KPlato__PrintingHeaderFooter);
        verticalLayout->setObjectName("verticalLayout");
        ui_header = new QGroupBox(KPlato__PrintingHeaderFooter);
        ui_header->setObjectName("ui_header");
        ui_header->setCheckable(true);
        gridLayout_2 = new QGridLayout(ui_header);
        gridLayout_2->setObjectName("gridLayout_2");
        ui_headerProject = new QCheckBox(ui_header);
        ui_headerProject->setObjectName("ui_headerProject");

        gridLayout_2->addWidget(ui_headerProject, 0, 0, 1, 1);

        ui_headerPage = new QCheckBox(ui_header);
        ui_headerPage->setObjectName("ui_headerPage");

        gridLayout_2->addWidget(ui_headerPage, 0, 1, 1, 1);

        ui_headerManager = new QCheckBox(ui_header);
        ui_headerManager->setObjectName("ui_headerManager");

        gridLayout_2->addWidget(ui_headerManager, 1, 0, 1, 1);

        ui_headerDate = new QCheckBox(ui_header);
        ui_headerDate->setObjectName("ui_headerDate");

        gridLayout_2->addWidget(ui_headerDate, 1, 1, 1, 1);


        verticalLayout->addWidget(ui_header);

        ui_footer = new QGroupBox(KPlato__PrintingHeaderFooter);
        ui_footer->setObjectName("ui_footer");
        ui_footer->setCheckable(true);
        gridLayout = new QGridLayout(ui_footer);
        gridLayout->setObjectName("gridLayout");
        ui_footerProject = new QCheckBox(ui_footer);
        ui_footerProject->setObjectName("ui_footerProject");

        gridLayout->addWidget(ui_footerProject, 0, 0, 1, 1);

        ui_footerPage = new QCheckBox(ui_footer);
        ui_footerPage->setObjectName("ui_footerPage");

        gridLayout->addWidget(ui_footerPage, 0, 1, 1, 1);

        ui_footerManager = new QCheckBox(ui_footer);
        ui_footerManager->setObjectName("ui_footerManager");

        gridLayout->addWidget(ui_footerManager, 1, 0, 1, 1);

        ui_footerDate = new QCheckBox(ui_footer);
        ui_footerDate->setObjectName("ui_footerDate");

        gridLayout->addWidget(ui_footerDate, 1, 1, 1, 1);


        verticalLayout->addWidget(ui_footer);


        retranslateUi(KPlato__PrintingHeaderFooter);

        QMetaObject::connectSlotsByName(KPlato__PrintingHeaderFooter);
    } // setupUi

    void retranslateUi(QWidget *KPlato__PrintingHeaderFooter)
    {
        ui_header->setTitle(tr2i18n("Header", nullptr));
        ui_headerProject->setText(tr2i18n("Project name", nullptr));
        ui_headerPage->setText(tr2i18n("Page number", nullptr));
        ui_headerManager->setText(tr2i18n("Project manager", nullptr));
        ui_headerDate->setText(tr2i18n("Date", nullptr));
        ui_footer->setTitle(tr2i18n("Footer", nullptr));
        ui_footerProject->setText(tr2i18n("Project name", nullptr));
        ui_footerPage->setText(tr2i18n("Page number", nullptr));
        ui_footerManager->setText(tr2i18n("Project manager", nullptr));
        ui_footerDate->setText(tr2i18n("Date", nullptr));
        (void)KPlato__PrintingHeaderFooter;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class PrintingHeaderFooter: public Ui_PrintingHeaderFooter {};
} // namespace Ui
} // namespace KPlato

#endif // KPTPRINTINGHEADERFOOTER_H

