/********************************************************************************
** Form generated from reading UI file 'ConfigProjectTemplatesPanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGPROJECTTEMPLATESPANEL_H
#define UI_CONFIGPROJECTTEMPLATESPANEL_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_ConfigProjectTemplatesPanel
{
public:
    QGridLayout *gridLayout;
    QToolButton *insertTemplatePath;
    QSpacerItem *verticalSpacer_3;
    QToolButton *removeTemplatePath;
    QTreeView *projectTemplatesView;

    void setupUi(QWidget *KPlato__ConfigProjectTemplatesPanel)
    {
        if (KPlato__ConfigProjectTemplatesPanel->objectName().isEmpty())
            KPlato__ConfigProjectTemplatesPanel->setObjectName("KPlato__ConfigProjectTemplatesPanel");
        KPlato__ConfigProjectTemplatesPanel->resize(400, 300);
        gridLayout = new QGridLayout(KPlato__ConfigProjectTemplatesPanel);
        gridLayout->setObjectName("gridLayout");
        insertTemplatePath = new QToolButton(KPlato__ConfigProjectTemplatesPanel);
        insertTemplatePath->setObjectName("insertTemplatePath");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("libs/ui"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        insertTemplatePath->setIcon(icon);

        gridLayout->addWidget(insertTemplatePath, 0, 1, 1, 1);

        verticalSpacer_3 = new QSpacerItem(20, 181, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer_3, 2, 1, 1, 1);

        removeTemplatePath = new QToolButton(KPlato__ConfigProjectTemplatesPanel);
        removeTemplatePath->setObjectName("removeTemplatePath");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("libs/ui"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        removeTemplatePath->setIcon(icon1);

        gridLayout->addWidget(removeTemplatePath, 1, 1, 1, 1);

        projectTemplatesView = new QTreeView(KPlato__ConfigProjectTemplatesPanel);
        projectTemplatesView->setObjectName("projectTemplatesView");
        projectTemplatesView->setRootIsDecorated(false);
        projectTemplatesView->setItemsExpandable(false);
        projectTemplatesView->header()->setVisible(false);

        gridLayout->addWidget(projectTemplatesView, 0, 0, 3, 1);


        retranslateUi(KPlato__ConfigProjectTemplatesPanel);

        QMetaObject::connectSlotsByName(KPlato__ConfigProjectTemplatesPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__ConfigProjectTemplatesPanel)
    {
        insertTemplatePath->setText(tr2i18n("Insert", nullptr));
        removeTemplatePath->setText(tr2i18n("Remove", nullptr));
        (void)KPlato__ConfigProjectTemplatesPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class ConfigProjectTemplatesPanel: public Ui_ConfigProjectTemplatesPanel {};
} // namespace Ui
} // namespace KPlato

#endif // CONFIGPROJECTTEMPLATESPANEL_H

