/****************************************************************************
** Meta object code from reading C++ file 'PortfolioView.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../calligraplan-4.0.1/src/portfolio/PortfolioView.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PortfolioView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13PortfolioViewE_t {};
} // unnamed namespace

template <> constexpr inline auto PortfolioView::qt_create_metaobjectdata<qt_meta_tag_ZN13PortfolioViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PortfolioView",
        "openUrl",
        "",
        "QUrl",
        "url",
        "slotRecentFileActivated",
        "QModelIndex",
        "idx",
        "selectionChanged",
        "QItemSelection",
        "selected",
        "deselected",
        "slotAddProject",
        "slotRemoveSelected",
        "slotLoadCompleted",
        "slotLoadCanceled",
        "slotUpdateView",
        "saveSettings",
        "QDomElement&",
        "settings",
        "loadSettings",
        "KoXmlElement&"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'openUrl'
        QtMocHelpers::SignalData<void(const QUrl &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotRecentFileActivated'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'selectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 9, 11 },
        }}),
        // Slot 'slotAddProject'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRemoveSelected'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotLoadCompleted'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotLoadCanceled'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUpdateView'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'saveSettings'
        QtMocHelpers::MethodData<void(QDomElement &) const>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Method 'loadSettings'
        QtMocHelpers::MethodData<void(KoXmlElement &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PortfolioView, qt_meta_tag_ZN13PortfolioViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PortfolioView::staticMetaObject = { {
    QMetaObject::SuperData::link<KoView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13PortfolioViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13PortfolioViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13PortfolioViewE_t>.metaTypes,
    nullptr
} };

void PortfolioView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PortfolioView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->openUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 1: _t->slotRecentFileActivated((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 2: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 3: _t->slotAddProject(); break;
        case 4: _t->slotRemoveSelected(); break;
        case 5: _t->slotLoadCompleted(); break;
        case 6: _t->slotLoadCanceled(); break;
        case 7: _t->slotUpdateView(); break;
        case 8: _t->saveSettings((*reinterpret_cast<std::add_pointer_t<QDomElement&>>(_a[1]))); break;
        case 9: _t->loadSettings((*reinterpret_cast<std::add_pointer_t<KoXmlElement&>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PortfolioView::*)(const QUrl & )>(_a, &PortfolioView::openUrl, 0))
            return;
    }
}

const QMetaObject *PortfolioView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PortfolioView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13PortfolioViewE_t>.strings))
        return static_cast<void*>(this);
    return KoView::qt_metacast(_clname);
}

int PortfolioView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KoView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void PortfolioView::openUrl(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
