/*
 *  SPDX-FileCopyrightText: 2015 Dmitry Kazakov <dimula73@gmail.com>
 *
 *  SPDX-License-Identifier: GPL-2.0-or-later
 */

#ifndef __KUNDO2COMMANDEXTRADATA_H
#define __KUNDO2COMMANDEXTRADATA_H

#include "kundo2_export.h"


class KUNDO2_EXPORT KUndo2CommandExtraData
{
public:
    virtual ~KUndo2CommandExtraData();
};

#endif /* __KUNDO2COMMANDEXTRADATA_H */
