/********************************************************************************
** Form generated from reading UI file 'kptsummarytaskgeneralpanelbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTSUMMARYTASKGENERALPANELBASE_H
#define UI_KPTSUMMARYTASKGENERALPANELBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "KLineEdit"

namespace KPlato {

class Ui_SummaryTaskGeneralPanelBase
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *wbslabel;
    QLabel *wbsfield;
    QLabel *namelabel;
    KLineEdit *namefield;
    QLabel *leaderlabel;
    QHBoxLayout *hboxLayout;
    KLineEdit *leaderfield;
    QPushButton *chooseLeader;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QSpinBox *ui_priority;
    QSpacerItem *horizontalSpacer;
    QFrame *line;

    void setupUi(QWidget *KPlato__SummaryTaskGeneralPanelBase)
    {
        if (KPlato__SummaryTaskGeneralPanelBase->objectName().isEmpty())
            KPlato__SummaryTaskGeneralPanelBase->setObjectName("KPlato__SummaryTaskGeneralPanelBase");
        KPlato__SummaryTaskGeneralPanelBase->resize(400, 165);
        KPlato__SummaryTaskGeneralPanelBase->setMinimumSize(QSize(400, 0));
        verticalLayout = new QVBoxLayout(KPlato__SummaryTaskGeneralPanelBase);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        wbslabel = new QLabel(KPlato__SummaryTaskGeneralPanelBase);
        wbslabel->setObjectName("wbslabel");
        wbslabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        wbslabel->setWordWrap(false);

        gridLayout->addWidget(wbslabel, 0, 0, 1, 1);

        wbsfield = new QLabel(KPlato__SummaryTaskGeneralPanelBase);
        wbsfield->setObjectName("wbsfield");
        wbsfield->setFrameShape(QFrame::StyledPanel);
        wbsfield->setFrameShadow(QFrame::Sunken);
        wbsfield->setWordWrap(false);

        gridLayout->addWidget(wbsfield, 0, 1, 1, 1);

        namelabel = new QLabel(KPlato__SummaryTaskGeneralPanelBase);
        namelabel->setObjectName("namelabel");
        namelabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        namelabel->setWordWrap(false);

        gridLayout->addWidget(namelabel, 1, 0, 1, 1);

        namefield = new KLineEdit(KPlato__SummaryTaskGeneralPanelBase);
        namefield->setObjectName("namefield");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(namefield->sizePolicy().hasHeightForWidth());
        namefield->setSizePolicy(sizePolicy);
        namefield->setFocusPolicy(Qt::WheelFocus);

        gridLayout->addWidget(namefield, 1, 1, 1, 2);

        leaderlabel = new QLabel(KPlato__SummaryTaskGeneralPanelBase);
        leaderlabel->setObjectName("leaderlabel");
        leaderlabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        leaderlabel->setWordWrap(false);

        gridLayout->addWidget(leaderlabel, 2, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        leaderfield = new KLineEdit(KPlato__SummaryTaskGeneralPanelBase);
        leaderfield->setObjectName("leaderfield");
        leaderfield->setFocusPolicy(Qt::WheelFocus);

        hboxLayout->addWidget(leaderfield);

        chooseLeader = new QPushButton(KPlato__SummaryTaskGeneralPanelBase);
        chooseLeader->setObjectName("chooseLeader");

        hboxLayout->addWidget(chooseLeader);


        gridLayout->addLayout(hboxLayout, 2, 1, 1, 2);

        label = new QLabel(KPlato__SummaryTaskGeneralPanelBase);
        label->setObjectName("label");

        gridLayout->addWidget(label, 3, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        ui_priority = new QSpinBox(KPlato__SummaryTaskGeneralPanelBase);
        ui_priority->setObjectName("ui_priority");

        horizontalLayout->addWidget(ui_priority);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        gridLayout->addLayout(horizontalLayout, 3, 2, 1, 1);


        verticalLayout->addLayout(gridLayout);

        line = new QFrame(KPlato__SummaryTaskGeneralPanelBase);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

#if QT_CONFIG(shortcut)
        namelabel->setBuddy(namefield);
        leaderlabel->setBuddy(leaderfield);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(namefield, leaderfield);
        QWidget::setTabOrder(leaderfield, chooseLeader);
        QWidget::setTabOrder(chooseLeader, wbsfield);

        retranslateUi(KPlato__SummaryTaskGeneralPanelBase);

        QMetaObject::connectSlotsByName(KPlato__SummaryTaskGeneralPanelBase);
    } // setupUi

    void retranslateUi(QWidget *KPlato__SummaryTaskGeneralPanelBase)
    {
#if QT_CONFIG(tooltip)
        wbslabel->setToolTip(tr2i18n("Work Breakdown Structure", nullptr));
#endif // QT_CONFIG(tooltip)
        wbslabel->setText(tr2i18n("WBS:", nullptr));
        wbsfield->setText(QString());
#if QT_CONFIG(whatsthis)
        namelabel->setWhatsThis(tr2i18n("The name of the Task.", nullptr));
#endif // QT_CONFIG(whatsthis)
        namelabel->setText(tr2i18n("Name:", nullptr));
#if QT_CONFIG(whatsthis)
        namefield->setWhatsThis(tr2i18n("The name of the Task.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        leaderlabel->setWhatsThis(tr2i18n("The person responsible for this task.\n"
"\n"
"This is not limited to persons available in a resource group but can be anyone. You can even directly access your address book with the Choose button.", nullptr));
#endif // QT_CONFIG(whatsthis)
        leaderlabel->setText(tr2i18n("Responsible:", nullptr));
#if QT_CONFIG(whatsthis)
        leaderfield->setWhatsThis(tr2i18n("The person responsible for this task.\n"
"\n"
"This is not limited to persons available in a resource group but can be anyone. You can even directly access your address book with the Choose button.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        chooseLeader->setToolTip(tr2i18n("Insert a person from your address book", nullptr));
#endif // QT_CONFIG(tooltip)
        chooseLeader->setText(tr2i18n("Choose...", nullptr));
#if QT_CONFIG(shortcut)
        chooseLeader->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        label->setText(tr2i18n("Priority:", nullptr));
        (void)KPlato__SummaryTaskGeneralPanelBase;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class SummaryTaskGeneralPanelBase: public Ui_SummaryTaskGeneralPanelBase {};
} // namespace Ui
} // namespace KPlato

#endif // KPTSUMMARYTASKGENERALPANELBASE_H

