/****************************************************************************
** Meta object code from reading C++ file 'KoPartAdaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/main/KoPartAdaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KoPartAdaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KoPartAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto KoPartAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN13KoPartAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KoPartAdaptor",
        "D-Bus Interface",
        "org.kde.calligra.document",
        "url",
        "",
        "openUrl",
        "isLoading",
        "isModified",
        "viewCount",
        "view",
        "idx",
        "save",
        "saveAs",
        "setOutputMimeType",
        "mimetype",
        "documentInfoAuthorName",
        "documentInfoEmail",
        "documentInfoCompanyName",
        "documentInfoTitle",
        "documentInfoAbstract",
        "documentInfoKeywords",
        "documentInfoSubject",
        "documentInfoTelephone",
        "documentInfoTelephoneWork",
        "documentInfoTelephoneHome",
        "documentInfoFax",
        "documentInfoCountry",
        "documentInfoPostalCode",
        "documentInfoCity",
        "documentInfoStreet",
        "documentInfoInitial",
        "documentInfoAuthorPosition",
        "setDocumentInfoAuthorName",
        "text",
        "setDocumentInfoEmail",
        "setDocumentInfoCompanyName",
        "setDocumentInfoTelephone",
        "setDocumentInfoTelephoneWork",
        "setDocumentInfoTelephoneHome",
        "setDocumentInfoFax",
        "setDocumentInfoCountry",
        "setDocumentInfoTitle",
        "setDocumentInfoPostalCode",
        "setDocumentInfoCity",
        "setDocumentInfoStreet",
        "setDocumentInfoAbstract",
        "setDocumentInfoInitial",
        "setDocumentInfoKeywords",
        "setDocumentInfoSubject",
        "setDocumentInfoAuthorPosition"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'url'
        QtMocHelpers::SlotData<QString()>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'openUrl'
        QtMocHelpers::SlotData<void(const QString &)>(5, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'isLoading'
        QtMocHelpers::SlotData<bool()>(6, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool),
        // Slot 'isModified'
        QtMocHelpers::SlotData<bool()>(7, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool),
        // Slot 'viewCount'
        QtMocHelpers::SlotData<int()>(8, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Int),
        // Slot 'view'
        QtMocHelpers::SlotData<QString(int)>(9, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'save'
        QtMocHelpers::SlotData<void()>(11, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'saveAs'
        QtMocHelpers::SlotData<void(const QString &)>(12, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'setOutputMimeType'
        QtMocHelpers::SlotData<void(const QByteArray &)>(13, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QByteArray, 14 },
        }}),
        // Slot 'documentInfoAuthorName'
        QtMocHelpers::SlotData<QString() const>(15, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoEmail'
        QtMocHelpers::SlotData<QString() const>(16, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoCompanyName'
        QtMocHelpers::SlotData<QString() const>(17, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoTitle'
        QtMocHelpers::SlotData<QString() const>(18, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoAbstract'
        QtMocHelpers::SlotData<QString() const>(19, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoKeywords'
        QtMocHelpers::SlotData<QString() const>(20, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoSubject'
        QtMocHelpers::SlotData<QString() const>(21, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoTelephone'
        QtMocHelpers::SlotData<QString() const>(22, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoTelephoneWork'
        QtMocHelpers::SlotData<QString() const>(23, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoTelephoneHome'
        QtMocHelpers::SlotData<QString() const>(24, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoFax'
        QtMocHelpers::SlotData<QString() const>(25, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoCountry'
        QtMocHelpers::SlotData<QString() const>(26, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoPostalCode'
        QtMocHelpers::SlotData<QString() const>(27, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoCity'
        QtMocHelpers::SlotData<QString() const>(28, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoStreet'
        QtMocHelpers::SlotData<QString() const>(29, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoInitial'
        QtMocHelpers::SlotData<QString() const>(30, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentInfoAuthorPosition'
        QtMocHelpers::SlotData<QString() const>(31, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'setDocumentInfoAuthorName'
        QtMocHelpers::SlotData<void(const QString &)>(32, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoEmail'
        QtMocHelpers::SlotData<void(const QString &)>(34, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoCompanyName'
        QtMocHelpers::SlotData<void(const QString &)>(35, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoTelephone'
        QtMocHelpers::SlotData<void(const QString &)>(36, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoTelephoneWork'
        QtMocHelpers::SlotData<void(const QString &)>(37, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoTelephoneHome'
        QtMocHelpers::SlotData<void(const QString &)>(38, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoFax'
        QtMocHelpers::SlotData<void(const QString &)>(39, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoCountry'
        QtMocHelpers::SlotData<void(const QString &)>(40, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoTitle'
        QtMocHelpers::SlotData<void(const QString &)>(41, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoPostalCode'
        QtMocHelpers::SlotData<void(const QString &)>(42, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoCity'
        QtMocHelpers::SlotData<void(const QString &)>(43, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoStreet'
        QtMocHelpers::SlotData<void(const QString &)>(44, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoAbstract'
        QtMocHelpers::SlotData<void(const QString &)>(45, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoInitial'
        QtMocHelpers::SlotData<void(const QString &)>(46, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoKeywords'
        QtMocHelpers::SlotData<void(const QString &)>(47, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoSubject'
        QtMocHelpers::SlotData<void(const QString &)>(48, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'setDocumentInfoAuthorPosition'
        QtMocHelpers::SlotData<void(const QString &)>(49, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<KoPartAdaptor, qt_meta_tag_ZN13KoPartAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject KoPartAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KoPartAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KoPartAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KoPartAdaptorE_t>.metaTypes,
    nullptr
} };

void KoPartAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KoPartAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QString _r = _t->url();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->openUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: { bool _r = _t->isLoading();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->isModified();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { int _r = _t->viewCount();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 5: { QString _r = _t->view((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->save(); break;
        case 7: _t->saveAs((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->setOutputMimeType((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 9: { QString _r = _t->documentInfoAuthorName();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { QString _r = _t->documentInfoEmail();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 11: { QString _r = _t->documentInfoCompanyName();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 12: { QString _r = _t->documentInfoTitle();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 13: { QString _r = _t->documentInfoAbstract();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 14: { QString _r = _t->documentInfoKeywords();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 15: { QString _r = _t->documentInfoSubject();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 16: { QString _r = _t->documentInfoTelephone();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 17: { QString _r = _t->documentInfoTelephoneWork();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 18: { QString _r = _t->documentInfoTelephoneHome();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 19: { QString _r = _t->documentInfoFax();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 20: { QString _r = _t->documentInfoCountry();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 21: { QString _r = _t->documentInfoPostalCode();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 22: { QString _r = _t->documentInfoCity();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 23: { QString _r = _t->documentInfoStreet();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 24: { QString _r = _t->documentInfoInitial();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 25: { QString _r = _t->documentInfoAuthorPosition();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->setDocumentInfoAuthorName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 27: _t->setDocumentInfoEmail((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 28: _t->setDocumentInfoCompanyName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 29: _t->setDocumentInfoTelephone((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 30: _t->setDocumentInfoTelephoneWork((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 31: _t->setDocumentInfoTelephoneHome((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 32: _t->setDocumentInfoFax((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 33: _t->setDocumentInfoCountry((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 34: _t->setDocumentInfoTitle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 35: _t->setDocumentInfoPostalCode((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 36: _t->setDocumentInfoCity((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 37: _t->setDocumentInfoStreet((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 38: _t->setDocumentInfoAbstract((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 39: _t->setDocumentInfoInitial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 40: _t->setDocumentInfoKeywords((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 41: _t->setDocumentInfoSubject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: _t->setDocumentInfoAuthorPosition((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KoPartAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KoPartAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KoPartAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int KoPartAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 43)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 43;
    }
    return _id;
}
QT_WARNING_POP
