/********************************************************************************
** Form generated from reading UI file 'PerformanceView.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PERFORMANCEVIEW_H
#define UI_PERFORMANCEVIEW_H

#include <KChartChart>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_PerformanceView
{
public:
    QVBoxLayout *verticalLayout_7;
    QWidget *widget;
    QHBoxLayout *horizontalLayout;
    QSplitter *splitter_2;
    QTreeView *treeView;
    QSplitter *splitter;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_6;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_4;
    QSpacerItem *horizontalSpacer_2;
    QFrame *frame;
    QHBoxLayout *horizontalLayout_2;
    QLabel *costSPI;
    QLabel *costCPI;
    QSpacerItem *horizontalSpacer_3;
    KChart::Chart *costChartPi;
    QFrame *line;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_8;
    QSpacerItem *horizontalSpacer_4;
    QFrame *frame_2;
    QHBoxLayout *horizontalLayout_3;
    QLabel *costBCWS;
    QLabel *costBCWP;
    QLabel *costACWP;
    QSpacerItem *horizontalSpacer_5;
    KChart::Chart *costChartWork;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_5;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_6;
    QSpacerItem *horizontalSpacer_6;
    QFrame *frame_4;
    QHBoxLayout *horizontalLayout_9;
    QLabel *effortSPI;
    QLabel *effortCPI;
    QSpacerItem *horizontalSpacer_7;
    KChart::Chart *effortChartPi;
    QFrame *line_2;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_7;
    QSpacerItem *horizontalSpacer_8;
    QFrame *frame_3;
    QHBoxLayout *horizontalLayout_5;
    QLabel *effortBCWS;
    QLabel *effortBCWP;
    QLabel *effortACWP;
    QSpacerItem *horizontalSpacer_9;
    KChart::Chart *effortChartWork;

    void setupUi(QWidget *PerformanceView)
    {
        if (PerformanceView->objectName().isEmpty())
            PerformanceView->setObjectName("PerformanceView");
        PerformanceView->resize(1046, 568);
        verticalLayout_7 = new QVBoxLayout(PerformanceView);
        verticalLayout_7->setObjectName("verticalLayout_7");
        widget = new QWidget(PerformanceView);
        widget->setObjectName("widget");
        horizontalLayout = new QHBoxLayout(widget);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        splitter_2 = new QSplitter(widget);
        splitter_2->setObjectName("splitter_2");
        splitter_2->setOrientation(Qt::Horizontal);
        treeView = new QTreeView(splitter_2);
        treeView->setObjectName("treeView");
        treeView->setRootIsDecorated(false);
        treeView->setItemsExpandable(false);
        treeView->setExpandsOnDoubleClick(false);
        splitter_2->addWidget(treeView);
        treeView->header()->setVisible(false);
        splitter = new QSplitter(splitter_2);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Horizontal);
        groupBox_2 = new QGroupBox(splitter);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_6 = new QVBoxLayout(groupBox_2);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_2);

        frame = new QFrame(groupBox_2);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Sunken);
        horizontalLayout_2 = new QHBoxLayout(frame);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        costSPI = new QLabel(frame);
        costSPI->setObjectName("costSPI");
        costSPI->setMinimumSize(QSize(20, 0));
        costSPI->setFrameShadow(QFrame::Plain);

        horizontalLayout_2->addWidget(costSPI);

        costCPI = new QLabel(frame);
        costCPI->setObjectName("costCPI");
        costCPI->setMinimumSize(QSize(20, 0));

        horizontalLayout_2->addWidget(costCPI);


        horizontalLayout_4->addWidget(frame);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_3);


        verticalLayout_3->addLayout(horizontalLayout_4);

        costChartPi = new KChart::Chart(groupBox_2);
        costChartPi->setObjectName("costChartPi");

        verticalLayout_3->addWidget(costChartPi);


        verticalLayout_6->addLayout(verticalLayout_3);

        line = new QFrame(groupBox_2);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_6->addWidget(line);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_4);

        frame_2 = new QFrame(groupBox_2);
        frame_2->setObjectName("frame_2");
        frame_2->setFrameShape(QFrame::StyledPanel);
        frame_2->setFrameShadow(QFrame::Sunken);
        horizontalLayout_3 = new QHBoxLayout(frame_2);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        costBCWS = new QLabel(frame_2);
        costBCWS->setObjectName("costBCWS");
        costBCWS->setMinimumSize(QSize(20, 0));

        horizontalLayout_3->addWidget(costBCWS);

        costBCWP = new QLabel(frame_2);
        costBCWP->setObjectName("costBCWP");
        costBCWP->setMinimumSize(QSize(20, 0));

        horizontalLayout_3->addWidget(costBCWP);

        costACWP = new QLabel(frame_2);
        costACWP->setObjectName("costACWP");
        costACWP->setMinimumSize(QSize(20, 0));

        horizontalLayout_3->addWidget(costACWP);


        horizontalLayout_8->addWidget(frame_2);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_5);


        verticalLayout->addLayout(horizontalLayout_8);

        costChartWork = new KChart::Chart(groupBox_2);
        costChartWork->setObjectName("costChartWork");

        verticalLayout->addWidget(costChartWork);


        verticalLayout_6->addLayout(verticalLayout);

        splitter->addWidget(groupBox_2);
        groupBox_3 = new QGroupBox(splitter);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_5 = new QVBoxLayout(groupBox_3);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_6);

        frame_4 = new QFrame(groupBox_3);
        frame_4->setObjectName("frame_4");
        frame_4->setFrameShape(QFrame::StyledPanel);
        frame_4->setFrameShadow(QFrame::Sunken);
        horizontalLayout_9 = new QHBoxLayout(frame_4);
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        effortSPI = new QLabel(frame_4);
        effortSPI->setObjectName("effortSPI");
        effortSPI->setMinimumSize(QSize(20, 0));
        effortSPI->setFrameShadow(QFrame::Plain);

        horizontalLayout_9->addWidget(effortSPI);

        effortCPI = new QLabel(frame_4);
        effortCPI->setObjectName("effortCPI");
        effortCPI->setMinimumSize(QSize(20, 0));

        horizontalLayout_9->addWidget(effortCPI);


        horizontalLayout_6->addWidget(frame_4);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_7);


        verticalLayout_2->addLayout(horizontalLayout_6);

        effortChartPi = new KChart::Chart(groupBox_3);
        effortChartPi->setObjectName("effortChartPi");

        verticalLayout_2->addWidget(effortChartPi);


        verticalLayout_5->addLayout(verticalLayout_2);

        line_2 = new QFrame(groupBox_3);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_5->addWidget(line_2);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_8);

        frame_3 = new QFrame(groupBox_3);
        frame_3->setObjectName("frame_3");
        frame_3->setFrameShape(QFrame::StyledPanel);
        frame_3->setFrameShadow(QFrame::Sunken);
        horizontalLayout_5 = new QHBoxLayout(frame_3);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        effortBCWS = new QLabel(frame_3);
        effortBCWS->setObjectName("effortBCWS");
        effortBCWS->setMinimumSize(QSize(20, 0));

        horizontalLayout_5->addWidget(effortBCWS);

        effortBCWP = new QLabel(frame_3);
        effortBCWP->setObjectName("effortBCWP");
        effortBCWP->setMinimumSize(QSize(20, 0));

        horizontalLayout_5->addWidget(effortBCWP);

        effortACWP = new QLabel(frame_3);
        effortACWP->setObjectName("effortACWP");
        effortACWP->setMinimumSize(QSize(20, 0));

        horizontalLayout_5->addWidget(effortACWP);


        horizontalLayout_7->addWidget(frame_3);

        horizontalSpacer_9 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_9);


        verticalLayout_4->addLayout(horizontalLayout_7);

        effortChartWork = new KChart::Chart(groupBox_3);
        effortChartWork->setObjectName("effortChartWork");

        verticalLayout_4->addWidget(effortChartWork);


        verticalLayout_5->addLayout(verticalLayout_4);

        splitter->addWidget(groupBox_3);
        splitter_2->addWidget(splitter);

        horizontalLayout->addWidget(splitter_2);


        verticalLayout_7->addWidget(widget);


        retranslateUi(PerformanceView);

        QMetaObject::connectSlotsByName(PerformanceView);
    } // setupUi

    void retranslateUi(QWidget *PerformanceView)
    {
        PerformanceView->setWindowTitle(QString());
        groupBox_2->setTitle(tr2i18n("Cost", nullptr));
        costSPI->setText(QString());
        costCPI->setText(QString());
        costBCWS->setText(QString());
        costBCWP->setText(QString());
        costACWP->setText(QString());
        groupBox_3->setTitle(tr2i18n("Effort", nullptr));
        effortSPI->setText(QString());
        effortCPI->setText(QString());
        effortBCWS->setText(QString());
        effortBCWP->setText(QString());
        effortACWP->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class PerformanceView: public Ui_PerformanceView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PERFORMANCEVIEW_H

