/****************************************************************************
** Meta object code from reading C++ file 'kptnode.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/kernel/kptnode.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptnode.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato4NodeE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::Node::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato4NodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::Node",
        "slotStandardWorktimeChanged",
        "",
        "KPlato::StandardWorktime*",
        "setPriority",
        "priority",
        "State",
        "State_None",
        "State_Started",
        "State_StartedLate",
        "State_StartedEarly",
        "State_Finished",
        "State_FinishedLate",
        "State_FinishedEarly",
        "State_Running",
        "State_RunningLate",
        "State_RunningEarly",
        "State_ReadyToStart",
        "State_NotReadyToStart",
        "State_NotScheduled",
        "State_Late",
        "State_Baselined",
        "State_Scheduling",
        "State_Error",
        "Properties",
        "NameProperty",
        "LeaderProperty",
        "DescriptionProperty",
        "TypeProperty",
        "RunningAccountProperty",
        "StartupCostProperty",
        "StartupAccountProperty",
        "ShutdownCostProperty",
        "ShutdownAccountProperty",
        "CompletionEntryProperty",
        "CompletionStartedProperty",
        "CompletionFinishedProperty",
        "CompletionStartTimeProperty",
        "CompletionFinishTimeProperty",
        "CompletionPercentageProperty",
        "CompletionRemainingEffortProperty",
        "CompletionActualEffortProperty",
        "CompletionUsedEffortProperty",
        "CompletionNoteProperty",
        "ResourceRequestProperty",
        "ConstraintTypeProperty",
        "StartConstraintProperty",
        "EndConstraintProperty",
        "PriorityProperty",
        "EstimateProperty",
        "EstimateRiskProperty",
        "EstimateOptimisticProperty",
        "EstimatePessimisticProperty"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotStandardWorktimeChanged'
        QtMocHelpers::SlotData<void(KPlato::StandardWorktime *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'setPriority'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(6, 6, QMC::EnumFlags{}).add({
            {    7, State::State_None },
            {    8, State::State_Started },
            {    9, State::State_StartedLate },
            {   10, State::State_StartedEarly },
            {   11, State::State_Finished },
            {   12, State::State_FinishedLate },
            {   13, State::State_FinishedEarly },
            {   14, State::State_Running },
            {   15, State::State_RunningLate },
            {   16, State::State_RunningEarly },
            {   17, State::State_ReadyToStart },
            {   18, State::State_NotReadyToStart },
            {   19, State::State_NotScheduled },
            {   20, State::State_Late },
            {   21, State::State_Baselined },
            {   22, State::State_Scheduling },
            {   23, State::State_Error },
        }),
        // enum 'Properties'
        QtMocHelpers::EnumData<enum Properties>(24, 24, QMC::EnumFlags{}).add({
            {   25, Properties::NameProperty },
            {   26, Properties::LeaderProperty },
            {   27, Properties::DescriptionProperty },
            {   28, Properties::TypeProperty },
            {   29, Properties::RunningAccountProperty },
            {   30, Properties::StartupCostProperty },
            {   31, Properties::StartupAccountProperty },
            {   32, Properties::ShutdownCostProperty },
            {   33, Properties::ShutdownAccountProperty },
            {   34, Properties::CompletionEntryProperty },
            {   35, Properties::CompletionStartedProperty },
            {   36, Properties::CompletionFinishedProperty },
            {   37, Properties::CompletionStartTimeProperty },
            {   38, Properties::CompletionFinishTimeProperty },
            {   39, Properties::CompletionPercentageProperty },
            {   40, Properties::CompletionRemainingEffortProperty },
            {   41, Properties::CompletionActualEffortProperty },
            {   42, Properties::CompletionUsedEffortProperty },
            {   43, Properties::CompletionNoteProperty },
            {   44, Properties::ResourceRequestProperty },
            {   45, Properties::ConstraintTypeProperty },
            {   46, Properties::StartConstraintProperty },
            {   47, Properties::EndConstraintProperty },
            {   48, Properties::PriorityProperty },
            {   49, Properties::EstimateProperty },
            {   50, Properties::EstimateRiskProperty },
            {   51, Properties::EstimateOptimisticProperty },
            {   52, Properties::EstimatePessimisticProperty },
        }),
    };
    return QtMocHelpers::metaObjectData<Node, qt_meta_tag_ZN6KPlato4NodeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::Node::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato4NodeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato4NodeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato4NodeE_t>.metaTypes,
    nullptr
} };

void KPlato::Node::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Node *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotStandardWorktimeChanged((*reinterpret_cast<std::add_pointer_t<KPlato::StandardWorktime*>>(_a[1]))); break;
        case 1: _t->setPriority((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::Node::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::Node::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato4NodeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KPlato::Node::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
