/****************************************************************************
** Meta object code from reading C++ file 'kpttaskgeneralpanel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/kpttaskgeneralpanel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kpttaskgeneralpanel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato20TaskGeneralPanelImplE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::TaskGeneralPanelImpl::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato20TaskGeneralPanelImplE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::TaskGeneralPanelImpl",
        "obligatedFieldsFilled",
        "",
        "schedulingTypeChanged",
        "changed",
        "setSchedulingType",
        "type",
        "changeLeader",
        "setEstimationType",
        "setOptimistic",
        "value",
        "setPessimistic",
        "enableDateTime",
        "scheduleType",
        "estimationTypeChanged",
        "setEstimate",
        "duration",
        "setEstimateType",
        "checkAllFieldsFilled",
        "startDateChanged",
        "startTimeChanged",
        "time",
        "endDateChanged",
        "endTimeChanged",
        "scheduleTypeChanged",
        "setStartTime",
        "setEndTime",
        "setStartDateTime",
        "dt",
        "setEndDateTime",
        "setStartDate",
        "date",
        "setEndDate",
        "setRisktype",
        "r",
        "calendarChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'obligatedFieldsFilled'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'schedulingTypeChanged'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'changed'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSchedulingType'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'changeLeader'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setEstimationType'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setOptimistic'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'setPessimistic'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'enableDateTime'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'estimationTypeChanged'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setEstimate'
        QtMocHelpers::SlotData<void(double)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 16 },
        }}),
        // Slot 'setEstimateType'
        QtMocHelpers::SlotData<void(int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'checkAllFieldsFilled'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startDateChanged'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startTimeChanged'
        QtMocHelpers::SlotData<void(const QTime &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QTime, 21 },
        }}),
        // Slot 'endDateChanged'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'endTimeChanged'
        QtMocHelpers::SlotData<void(const QTime &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QTime, 21 },
        }}),
        // Slot 'scheduleTypeChanged'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'setStartTime'
        QtMocHelpers::SlotData<void(const QTime &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QTime, 21 },
        }}),
        // Slot 'setEndTime'
        QtMocHelpers::SlotData<void(const QTime &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QTime, 21 },
        }}),
        // Slot 'setStartDateTime'
        QtMocHelpers::SlotData<void(const QDateTime &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDateTime, 28 },
        }}),
        // Slot 'setEndDateTime'
        QtMocHelpers::SlotData<void(const QDateTime &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDateTime, 28 },
        }}),
        // Slot 'setStartDate'
        QtMocHelpers::SlotData<void(const QDate &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDate, 31 },
        }}),
        // Slot 'setEndDate'
        QtMocHelpers::SlotData<void(const QDate &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDate, 31 },
        }}),
        // Slot 'setRisktype'
        QtMocHelpers::SlotData<void(int)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'calendarChanged'
        QtMocHelpers::SlotData<void(int)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaskGeneralPanelImpl, qt_meta_tag_ZN6KPlato20TaskGeneralPanelImplE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::TaskGeneralPanelImpl::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20TaskGeneralPanelImplE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20TaskGeneralPanelImplE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato20TaskGeneralPanelImplE_t>.metaTypes,
    nullptr
} };

void KPlato::TaskGeneralPanelImpl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskGeneralPanelImpl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->obligatedFieldsFilled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->schedulingTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->changed(); break;
        case 3: _t->setSchedulingType((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->changeLeader(); break;
        case 5: _t->setEstimationType((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->setOptimistic((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->setPessimistic((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->enableDateTime((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->estimationTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->setEstimate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->setEstimateType((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->checkAllFieldsFilled(); break;
        case 13: _t->startDateChanged(); break;
        case 14: _t->startTimeChanged((*reinterpret_cast<std::add_pointer_t<QTime>>(_a[1]))); break;
        case 15: _t->endDateChanged(); break;
        case 16: _t->endTimeChanged((*reinterpret_cast<std::add_pointer_t<QTime>>(_a[1]))); break;
        case 17: _t->scheduleTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->setStartTime((*reinterpret_cast<std::add_pointer_t<QTime>>(_a[1]))); break;
        case 19: _t->setEndTime((*reinterpret_cast<std::add_pointer_t<QTime>>(_a[1]))); break;
        case 20: _t->setStartDateTime((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1]))); break;
        case 21: _t->setEndDateTime((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1]))); break;
        case 22: _t->setStartDate((*reinterpret_cast<std::add_pointer_t<QDate>>(_a[1]))); break;
        case 23: _t->setEndDate((*reinterpret_cast<std::add_pointer_t<QDate>>(_a[1]))); break;
        case 24: _t->setRisktype((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->calendarChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TaskGeneralPanelImpl::*)(bool )>(_a, &TaskGeneralPanelImpl::obligatedFieldsFilled, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskGeneralPanelImpl::*)(int )>(_a, &TaskGeneralPanelImpl::schedulingTypeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskGeneralPanelImpl::*)()>(_a, &TaskGeneralPanelImpl::changed, 2))
            return;
    }
}

const QMetaObject *KPlato::TaskGeneralPanelImpl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::TaskGeneralPanelImpl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20TaskGeneralPanelImplE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui_TaskGeneralPanelBase"))
        return static_cast< Ui_TaskGeneralPanelBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int KPlato::TaskGeneralPanelImpl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 26;
    }
    return _id;
}

// SIGNAL 0
void KPlato::TaskGeneralPanelImpl::obligatedFieldsFilled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::TaskGeneralPanelImpl::schedulingTypeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KPlato::TaskGeneralPanelImpl::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN6KPlato16TaskGeneralPanelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::TaskGeneralPanel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato16TaskGeneralPanelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::TaskGeneralPanel",
        "estimationTypeChanged",
        "",
        "type",
        "scheduleTypeChanged",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'estimationTypeChanged'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'scheduleTypeChanged'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaskGeneralPanel, qt_meta_tag_ZN6KPlato16TaskGeneralPanelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::TaskGeneralPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TaskGeneralPanelImpl::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16TaskGeneralPanelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16TaskGeneralPanelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato16TaskGeneralPanelE_t>.metaTypes,
    nullptr
} };

void KPlato::TaskGeneralPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskGeneralPanel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->estimationTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->scheduleTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::TaskGeneralPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::TaskGeneralPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato16TaskGeneralPanelE_t>.strings))
        return static_cast<void*>(this);
    return TaskGeneralPanelImpl::qt_metacast(_clname);
}

int KPlato::TaskGeneralPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TaskGeneralPanelImpl::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
