/********************************************************************************
** Form generated from reading UI file 'kptviewlisteditcategory.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTVIEWLISTEDITCATEGORY_H
#define UI_KPTVIEWLISTEDITCATEGORY_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "KComboBox"

namespace KPlato {

class Ui_EditCategoryPanel
{
public:
    QFormLayout *formLayout;
    QLabel *afterLabel;
    KComboBox *insertAfter;
    QLabel *categoryLabel;
    QLineEdit *viewname;
    QLabel *tooltipLabel;
    QLineEdit *tooltip;

    void setupUi(QWidget *KPlato__EditCategoryPanel)
    {
        if (KPlato__EditCategoryPanel->objectName().isEmpty())
            KPlato__EditCategoryPanel->setObjectName("KPlato__EditCategoryPanel");
        KPlato__EditCategoryPanel->resize(351, 108);
        formLayout = new QFormLayout(KPlato__EditCategoryPanel);
        formLayout->setContentsMargins(0, 0, 0, 0);
        formLayout->setObjectName("formLayout");
        afterLabel = new QLabel(KPlato__EditCategoryPanel);
        afterLabel->setObjectName("afterLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, afterLabel);

        insertAfter = new KComboBox(KPlato__EditCategoryPanel);
        insertAfter->setObjectName("insertAfter");
        insertAfter->setInsertPolicy(QComboBox::InsertAlphabetically);
        insertAfter->setAutoCompletion(false);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, insertAfter);

        categoryLabel = new QLabel(KPlato__EditCategoryPanel);
        categoryLabel->setObjectName("categoryLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, categoryLabel);

        viewname = new QLineEdit(KPlato__EditCategoryPanel);
        viewname->setObjectName("viewname");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, viewname);

        tooltipLabel = new QLabel(KPlato__EditCategoryPanel);
        tooltipLabel->setObjectName("tooltipLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, tooltipLabel);

        tooltip = new QLineEdit(KPlato__EditCategoryPanel);
        tooltip->setObjectName("tooltip");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, tooltip);

#if QT_CONFIG(shortcut)
        afterLabel->setBuddy(insertAfter);
        categoryLabel->setBuddy(viewname);
        tooltipLabel->setBuddy(tooltip);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__EditCategoryPanel);

        QMetaObject::connectSlotsByName(KPlato__EditCategoryPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__EditCategoryPanel)
    {
#if QT_CONFIG(tooltip)
        afterLabel->setToolTip(tr2i18n("The category is placed after this category", nullptr));
#endif // QT_CONFIG(tooltip)
        afterLabel->setText(tr2i18n("After", nullptr));
#if QT_CONFIG(tooltip)
        categoryLabel->setToolTip(tr2i18n("The name of the category", nullptr));
#endif // QT_CONFIG(tooltip)
        categoryLabel->setText(tr2i18n("Name:", nullptr));
#if QT_CONFIG(tooltip)
        tooltipLabel->setToolTip(tr2i18n("The description of the category", nullptr));
#endif // QT_CONFIG(tooltip)
        tooltipLabel->setText(tr2i18n("Description:", nullptr));
        (void)KPlato__EditCategoryPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class EditCategoryPanel: public Ui_EditCategoryPanel {};
} // namespace Ui
} // namespace KPlato

#endif // KPTVIEWLISTEDITCATEGORY_H

