/********************************************************************************
** Form generated from reading UI file 'MilestoneGanttChartOptionsPanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MILESTONEGANTTCHARTOPTIONSPANEL_H
#define UI_MILESTONEGANTTCHARTOPTIONSPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcolorcombo.h"

namespace KPlato {

class Ui_MilestoneGanttChartOptionsPanel
{
public:
    QVBoxLayout *verticalLayout_3;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout_4;
    QRadioButton *timeLineHide;
    QRadioButton *timeLineForeground;
    QRadioButton *timeLineBackground;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_3;
    QSpinBox *timeLineInterval;
    QFrame *line;
    QGridLayout *gridLayout_3;
    QCheckBox *timeLineUseCustom;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QSpinBox *timeLineStroke;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    KColorCombo *timeLineColor;
    QGroupBox *groupBox_4;
    QVBoxLayout *verticalLayout;
    QCheckBox *showRowSeparators;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_4;
    QComboBox *freedays;
    QSpacerItem *horizontalSpacer;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QCheckBox *showTaskName;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPlato__MilestoneGanttChartOptionsPanel)
    {
        if (KPlato__MilestoneGanttChartOptionsPanel->objectName().isEmpty())
            KPlato__MilestoneGanttChartOptionsPanel->setObjectName("KPlato__MilestoneGanttChartOptionsPanel");
        KPlato__MilestoneGanttChartOptionsPanel->resize(349, 413);
        verticalLayout_3 = new QVBoxLayout(KPlato__MilestoneGanttChartOptionsPanel);
        verticalLayout_3->setObjectName("verticalLayout_3");
        groupBox_3 = new QGroupBox(KPlato__MilestoneGanttChartOptionsPanel);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_2 = new QVBoxLayout(groupBox_3);
        verticalLayout_2->setObjectName("verticalLayout_2");
        gridLayout_4 = new QGridLayout();
        gridLayout_4->setObjectName("gridLayout_4");
        timeLineHide = new QRadioButton(groupBox_3);
        timeLineHide->setObjectName("timeLineHide");

        gridLayout_4->addWidget(timeLineHide, 0, 0, 1, 1);

        timeLineForeground = new QRadioButton(groupBox_3);
        timeLineForeground->setObjectName("timeLineForeground");

        gridLayout_4->addWidget(timeLineForeground, 0, 1, 1, 1);

        timeLineBackground = new QRadioButton(groupBox_3);
        timeLineBackground->setObjectName("timeLineBackground");

        gridLayout_4->addWidget(timeLineBackground, 0, 2, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_3 = new QLabel(groupBox_3);
        label_3->setObjectName("label_3");

        horizontalLayout_3->addWidget(label_3);

        timeLineInterval = new QSpinBox(groupBox_3);
        timeLineInterval->setObjectName("timeLineInterval");

        horizontalLayout_3->addWidget(timeLineInterval);


        gridLayout_4->addLayout(horizontalLayout_3, 1, 0, 1, 2);


        verticalLayout_2->addLayout(gridLayout_4);

        line = new QFrame(groupBox_3);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line);

        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName("gridLayout_3");
        timeLineUseCustom = new QCheckBox(groupBox_3);
        timeLineUseCustom->setObjectName("timeLineUseCustom");

        gridLayout_3->addWidget(timeLineUseCustom, 0, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label_2 = new QLabel(groupBox_3);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        timeLineStroke = new QSpinBox(groupBox_3);
        timeLineStroke->setObjectName("timeLineStroke");

        horizontalLayout->addWidget(timeLineStroke);


        gridLayout_3->addLayout(horizontalLayout, 1, 0, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label = new QLabel(groupBox_3);
        label->setObjectName("label");

        horizontalLayout_2->addWidget(label);

        timeLineColor = new KColorCombo(groupBox_3);
        timeLineColor->setObjectName("timeLineColor");

        horizontalLayout_2->addWidget(timeLineColor);


        gridLayout_3->addLayout(horizontalLayout_2, 1, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout_3);


        verticalLayout_3->addWidget(groupBox_3);

        groupBox_4 = new QGroupBox(KPlato__MilestoneGanttChartOptionsPanel);
        groupBox_4->setObjectName("groupBox_4");
        verticalLayout = new QVBoxLayout(groupBox_4);
        verticalLayout->setObjectName("verticalLayout");
        showRowSeparators = new QCheckBox(groupBox_4);
        showRowSeparators->setObjectName("showRowSeparators");

        verticalLayout->addWidget(showRowSeparators);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_4 = new QLabel(groupBox_4);
        label_4->setObjectName("label_4");

        horizontalLayout_4->addWidget(label_4);

        freedays = new QComboBox(groupBox_4);
        freedays->setObjectName("freedays");
        freedays->setMinimumSize(QSize(200, 0));

        horizontalLayout_4->addWidget(freedays);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_4);


        verticalLayout_3->addWidget(groupBox_4);

        groupBox = new QGroupBox(KPlato__MilestoneGanttChartOptionsPanel);
        groupBox->setObjectName("groupBox");
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        showTaskName = new QCheckBox(groupBox);
        showTaskName->setObjectName("showTaskName");

        gridLayout_2->addWidget(showTaskName, 0, 0, 1, 1);


        verticalLayout_3->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(20, 15, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        label_3->setBuddy(timeLineInterval);
        label_2->setBuddy(timeLineStroke);
        label->setBuddy(timeLineColor);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__MilestoneGanttChartOptionsPanel);

        QMetaObject::connectSlotsByName(KPlato__MilestoneGanttChartOptionsPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__MilestoneGanttChartOptionsPanel)
    {
#if QT_CONFIG(tooltip)
        KPlato__MilestoneGanttChartOptionsPanel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        groupBox_3->setTitle(tr2i18n("Timeline", nullptr));
#if QT_CONFIG(tooltip)
        timeLineHide->setToolTip(tr2i18n("Do not show the timeline", nullptr));
#endif // QT_CONFIG(tooltip)
        timeLineHide->setText(tr2i18n("Hide", nullptr));
#if QT_CONFIG(tooltip)
        timeLineForeground->setToolTip(tr2i18n("Show timeline in the foreground", nullptr));
#endif // QT_CONFIG(tooltip)
        timeLineForeground->setText(tr2i18n("Foreground", nullptr));
#if QT_CONFIG(tooltip)
        timeLineBackground->setToolTip(tr2i18n("Show timeline in the background", nullptr));
#endif // QT_CONFIG(tooltip)
        timeLineBackground->setText(tr2i18n("Background", nullptr));
#if QT_CONFIG(tooltip)
        label_3->setToolTip(tr2i18n("Update interval in minutes, 0 disables update", nullptr));
#endif // QT_CONFIG(tooltip)
        label_3->setText(tr2i18n("Interval:", nullptr));
#if QT_CONFIG(tooltip)
        timeLineInterval->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        timeLineUseCustom->setText(tr2i18n("Use custom line", nullptr));
#if QT_CONFIG(tooltip)
        label_2->setToolTip(tr2i18n("Line width", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(tr2i18n("Stroke:", nullptr));
#if QT_CONFIG(tooltip)
        timeLineStroke->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label->setToolTip(tr2i18n("Line color", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(tr2i18n("Color:", nullptr));
#if QT_CONFIG(tooltip)
        timeLineColor->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        groupBox_4->setTitle(tr2i18n("Decorations", nullptr));
        showRowSeparators->setText(tr2i18n("Show Row Separators", nullptr));
        label_4->setText(tr2i18n("Freedays:", nullptr));
        groupBox->setTitle(tr2i18n("Additional text information", nullptr));
        showTaskName->setText(tr2i18n("Task name", nullptr));
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class MilestoneGanttChartOptionsPanel: public Ui_MilestoneGanttChartOptionsPanel {};
} // namespace Ui
} // namespace KPlato

#endif // MILESTONEGANTTCHARTOPTIONSPANEL_H

