/********************************************************************************
** Form generated from reading UI file 'kpttaskdescriptionpanelbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTTASKDESCRIPTIONPANELBASE_H
#define UI_KPTTASKDESCRIPTIONPANELBASE_H

#include <KToolBar>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "KLineEdit"
#include "RichTextWidget.h"

namespace KPlato {

class Ui_TaskDescriptionPanelBase
{
public:
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *namelabel;
    KLineEdit *namefield;
    KToolBar *toolbar;
    KPlato::RichTextWidget *descriptionfield;

    void setupUi(QWidget *KPlato__TaskDescriptionPanelBase)
    {
        if (KPlato__TaskDescriptionPanelBase->objectName().isEmpty())
            KPlato__TaskDescriptionPanelBase->setObjectName("KPlato__TaskDescriptionPanelBase");
        KPlato__TaskDescriptionPanelBase->resize(600, 400);
        horizontalLayout = new QHBoxLayout(KPlato__TaskDescriptionPanelBase);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        namelabel = new QLabel(KPlato__TaskDescriptionPanelBase);
        namelabel->setObjectName("namelabel");
        namelabel->setWordWrap(false);

        gridLayout->addWidget(namelabel, 0, 0, 1, 1);

        namefield = new KLineEdit(KPlato__TaskDescriptionPanelBase);
        namefield->setObjectName("namefield");
        namefield->setReadOnly(true);

        gridLayout->addWidget(namefield, 0, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        toolbar = new KToolBar(KPlato__TaskDescriptionPanelBase);
        toolbar->setObjectName("toolbar");
        toolbar->setMinimumSize(QSize(0, 11));

        verticalLayout->addWidget(toolbar);

        descriptionfield = new KPlato::RichTextWidget(KPlato__TaskDescriptionPanelBase);
        descriptionfield->setObjectName("descriptionfield");

        verticalLayout->addWidget(descriptionfield);


        horizontalLayout->addLayout(verticalLayout);

#if QT_CONFIG(shortcut)
        namelabel->setBuddy(namefield);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__TaskDescriptionPanelBase);

        QMetaObject::connectSlotsByName(KPlato__TaskDescriptionPanelBase);
    } // setupUi

    void retranslateUi(QWidget *KPlato__TaskDescriptionPanelBase)
    {
#if QT_CONFIG(whatsthis)
        namelabel->setWhatsThis(tr2i18n("The name of the Task.", nullptr));
#endif // QT_CONFIG(whatsthis)
        namelabel->setText(tr2i18n("Name:", nullptr));
#if QT_CONFIG(whatsthis)
        namefield->setWhatsThis(tr2i18n("The name of the Task.", nullptr));
#endif // QT_CONFIG(whatsthis)
        (void)KPlato__TaskDescriptionPanelBase;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class TaskDescriptionPanelBase: public Ui_TaskDescriptionPanelBase {};
} // namespace Ui
} // namespace KPlato

#endif // KPTTASKDESCRIPTIONPANELBASE_H

