/****************************************************************************
** Meta object code from reading C++ file 'kptviewbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/kptviewbase.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptviewbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato10DockWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::DockWidget::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato10DockWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::DockWidget",
        "setShown",
        "",
        "show",
        "setLocation",
        "Qt::DockWidgetArea",
        "area"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setShown'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setLocation'
        QtMocHelpers::SlotData<void(Qt::DockWidgetArea)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DockWidget, qt_meta_tag_ZN6KPlato10DockWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::DockWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QDockWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato10DockWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato10DockWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato10DockWidgetE_t>.metaTypes,
    nullptr
} };

void KPlato::DockWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DockWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setShown((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->setLocation((*reinterpret_cast<std::add_pointer_t<Qt::DockWidgetArea>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::DockWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::DockWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato10DockWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QDockWidget::qt_metacast(_clname);
}

int KPlato::DockWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDockWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato20PrintingHeaderFooterE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::PrintingHeaderFooter::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato20PrintingHeaderFooterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::PrintingHeaderFooter",
        "changed",
        "",
        "KPlato::PrintingOptions",
        "slotChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void(const KPlato::PrintingOptions &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotChanged'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PrintingHeaderFooter, qt_meta_tag_ZN6KPlato20PrintingHeaderFooterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::PrintingHeaderFooter::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20PrintingHeaderFooterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20PrintingHeaderFooterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato20PrintingHeaderFooterE_t>.metaTypes,
    nullptr
} };

void KPlato::PrintingHeaderFooter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PrintingHeaderFooter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed((*reinterpret_cast<std::add_pointer_t<KPlato::PrintingOptions>>(_a[1]))); break;
        case 1: _t->slotChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PrintingHeaderFooter::*)(const KPlato::PrintingOptions & )>(_a, &PrintingHeaderFooter::changed, 0))
            return;
    }
}

const QMetaObject *KPlato::PrintingHeaderFooter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::PrintingHeaderFooter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato20PrintingHeaderFooterE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PrintingHeaderFooter"))
        return static_cast< Ui::PrintingHeaderFooter*>(this);
    return QWidget::qt_metacast(_clname);
}

int KPlato::PrintingHeaderFooter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KPlato::PrintingHeaderFooter::changed(const KPlato::PrintingOptions & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN6KPlato14PrintingDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::PrintingDialog::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato14PrintingDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::PrintingDialog",
        "changed",
        "",
        "KPlato::PrintingOptions",
        "opt",
        "setPrintingOptions",
        "setPrinterPageLayout",
        "KoPageLayout",
        "pagelayout",
        "startPrinting",
        "KoPrintJob::RemovePolicy",
        "removePolicy"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void(const KPlato::PrintingOptions &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setPrintingOptions'
        QtMocHelpers::SlotData<void(const KPlato::PrintingOptions &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setPrinterPageLayout'
        QtMocHelpers::SlotData<void(const KoPageLayout &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'startPrinting'
        QtMocHelpers::SlotData<void(KoPrintJob::RemovePolicy)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'startPrinting'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PrintingDialog, qt_meta_tag_ZN6KPlato14PrintingDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::PrintingDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<KoPrintingDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14PrintingDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14PrintingDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato14PrintingDialogE_t>.metaTypes,
    nullptr
} };

void KPlato::PrintingDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PrintingDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed((*reinterpret_cast<std::add_pointer_t<KPlato::PrintingOptions>>(_a[1]))); break;
        case 1: _t->setPrintingOptions((*reinterpret_cast<std::add_pointer_t<KPlato::PrintingOptions>>(_a[1]))); break;
        case 2: _t->setPrinterPageLayout((*reinterpret_cast<std::add_pointer_t<KoPageLayout>>(_a[1]))); break;
        case 3: _t->startPrinting((*reinterpret_cast<std::add_pointer_t<KoPrintJob::RemovePolicy>>(_a[1]))); break;
        case 4: _t->startPrinting(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PrintingDialog::*)(const KPlato::PrintingOptions & )>(_a, &PrintingDialog::changed, 0))
            return;
    }
}

const QMetaObject *KPlato::PrintingDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::PrintingDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato14PrintingDialogE_t>.strings))
        return static_cast<void*>(this);
    return KoPrintingDialog::qt_metacast(_clname);
}

int KPlato::PrintingDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KoPrintingDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void KPlato::PrintingDialog::changed(const KPlato::PrintingOptions & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN6KPlato8ViewBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ViewBase::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato8ViewBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ViewBase",
        "guiActivated",
        "",
        "KPlato::ViewBase*",
        "requestPopupMenu",
        "QPoint",
        "optionsModified",
        "projectChanged",
        "KPlato::Project*",
        "project",
        "readWriteChanged",
        "expandAll",
        "collapseAll",
        "openDocument",
        "QUrl",
        "url",
        "openTaskDescription",
        "setPrintingOptions",
        "KPlato::PrintingOptions",
        "opt",
        "setGuiActive",
        "activate",
        "setProject",
        "setScheduleManager",
        "KPlato::ScheduleManager*",
        "sm",
        "slotUpdateReadWrite",
        "slotHeaderContextMenuRequested",
        "pos",
        "slotEditCopy",
        "slotEditCut",
        "slotEditPaste",
        "slotRefreshView",
        "setPageLayout",
        "KoPageLayout",
        "layout",
        "slotOptions",
        "slotOptionsFinished",
        "result"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'guiActivated'
        QtMocHelpers::SignalData<void(KPlato::ViewBase *, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::Bool, 2 },
        }}),
        // Signal 'requestPopupMenu'
        QtMocHelpers::SignalData<void(const QString &, const QPoint &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { 0x80000000 | 5, 2 },
        }}),
        // Signal 'optionsModified'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'projectChanged'
        QtMocHelpers::SignalData<void(KPlato::Project *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'readWriteChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'expandAll'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'collapseAll'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openDocument'
        QtMocHelpers::SignalData<void(const QUrl &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'openTaskDescription'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setPrintingOptions'
        QtMocHelpers::SlotData<void(const KPlato::PrintingOptions &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'setGuiActive'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Slot 'setProject'
        QtMocHelpers::SlotData<void(KPlato::Project *)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'setScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'slotUpdateReadWrite'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotHeaderContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 28 },
        }}),
        // Slot 'slotEditCopy'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditCut'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditPaste'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRefreshView'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPageLayout'
        QtMocHelpers::SlotData<void(const KoPageLayout &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Slot 'slotOptions'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotOptionsFinished'
        QtMocHelpers::SlotData<void(int)>(37, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 38 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ViewBase, qt_meta_tag_ZN6KPlato8ViewBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ViewBase::staticMetaObject = { {
    QMetaObject::SuperData::link<KoView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8ViewBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8ViewBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato8ViewBaseE_t>.metaTypes,
    nullptr
} };

void KPlato::ViewBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ViewBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->guiActivated((*reinterpret_cast<std::add_pointer_t<KPlato::ViewBase*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->requestPopupMenu((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 2: _t->optionsModified(); break;
        case 3: _t->projectChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1]))); break;
        case 4: _t->readWriteChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->expandAll(); break;
        case 6: _t->collapseAll(); break;
        case 7: _t->openDocument((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 8: _t->openTaskDescription((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->setPrintingOptions((*reinterpret_cast<std::add_pointer_t<KPlato::PrintingOptions>>(_a[1]))); break;
        case 10: _t->setGuiActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->setProject((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1]))); break;
        case 12: _t->setScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 13: _t->slotUpdateReadWrite((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->slotHeaderContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 15: _t->slotEditCopy(); break;
        case 16: _t->slotEditCut(); break;
        case 17: _t->slotEditPaste(); break;
        case 18: _t->slotRefreshView(); break;
        case 19: _t->setPageLayout((*reinterpret_cast<std::add_pointer_t<KoPageLayout>>(_a[1]))); break;
        case 20: _t->slotOptions(); break;
        case 21: _t->slotOptionsFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ViewBase* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ViewBase::*)(KPlato::ViewBase * , bool )>(_a, &ViewBase::guiActivated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewBase::*)(const QString & , const QPoint & )>(_a, &ViewBase::requestPopupMenu, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewBase::*)()>(_a, &ViewBase::optionsModified, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewBase::*)(KPlato::Project * )>(_a, &ViewBase::projectChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewBase::*)(bool )>(_a, &ViewBase::readWriteChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewBase::*)()>(_a, &ViewBase::expandAll, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewBase::*)()>(_a, &ViewBase::collapseAll, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewBase::*)(const QUrl & )>(_a, &ViewBase::openDocument, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewBase::*)(bool )>(_a, &ViewBase::openTaskDescription, 8))
            return;
    }
}

const QMetaObject *KPlato::ViewBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ViewBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8ViewBaseE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ViewActionLists"))
        return static_cast< ViewActionLists*>(this);
    return KoView::qt_metacast(_clname);
}

int KPlato::ViewBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KoView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void KPlato::ViewBase::guiActivated(KPlato::ViewBase * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KPlato::ViewBase::requestPopupMenu(const QString & _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KPlato::ViewBase::optionsModified()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KPlato::ViewBase::projectChanged(KPlato::Project * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KPlato::ViewBase::readWriteChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KPlato::ViewBase::expandAll()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void KPlato::ViewBase::collapseAll()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KPlato::ViewBase::openDocument(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void KPlato::ViewBase::openTaskDescription(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN6KPlato22TreeViewPrintingDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::TreeViewPrintingDialog::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato22TreeViewPrintingDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::TreeViewPrintingDialog"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreeViewPrintingDialog, qt_meta_tag_ZN6KPlato22TreeViewPrintingDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::TreeViewPrintingDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<PrintingDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22TreeViewPrintingDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22TreeViewPrintingDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato22TreeViewPrintingDialogE_t>.metaTypes,
    nullptr
} };

void KPlato::TreeViewPrintingDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreeViewPrintingDialog *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::TreeViewPrintingDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::TreeViewPrintingDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22TreeViewPrintingDialogE_t>.strings))
        return static_cast<void*>(this);
    return PrintingDialog::qt_metacast(_clname);
}

int KPlato::TreeViewPrintingDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PrintingDialog::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato12TreeViewBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::TreeViewBase::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato12TreeViewBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::TreeViewBase",
        "contextMenuRequested",
        "",
        "QModelIndex",
        "QPoint",
        "pos",
        "QModelIndexList",
        "headerContextMenuRequested",
        "moveAfterLastColumn",
        "moveBeforeFirstColumn",
        "editAfterLastColumn",
        "editBeforeFirstColumn",
        "dropAllowed",
        "index",
        "dropIndicatorPosition",
        "QDragMoveEvent*",
        "event",
        "slotExpand",
        "slotCollapse",
        "closeEditor",
        "QWidget*",
        "editor",
        "QAbstractItemDelegate::EndEditHint",
        "hint",
        "slotCurrentChanged",
        "current",
        "previous",
        "slotHeaderContextMenuRequested",
        "doContextExpanded",
        "doExpanded"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'contextMenuRequested'
        QtMocHelpers::SignalData<void(const QModelIndex &, const QPoint &, const QModelIndexList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 2 },
        }}),
        // Signal 'headerContextMenuRequested'
        QtMocHelpers::SignalData<void(const QPoint &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'moveAfterLastColumn'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'moveBeforeFirstColumn'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'editAfterLastColumn'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'editBeforeFirstColumn'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'dropAllowed'
        QtMocHelpers::SignalData<void(const QModelIndex &, int, QDragMoveEvent *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 13 }, { QMetaType::Int, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'slotExpand'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCollapse'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'closeEditor'
        QtMocHelpers::SlotData<void(QWidget *, QAbstractItemDelegate::EndEditHint)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotCurrentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 25 }, { 0x80000000 | 3, 26 },
        }}),
        // Slot 'slotHeaderContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Slot 'doContextExpanded'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'doExpanded'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreeViewBase, qt_meta_tag_ZN6KPlato12TreeViewBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::TreeViewBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12TreeViewBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12TreeViewBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato12TreeViewBaseE_t>.metaTypes,
    nullptr
} };

void KPlato::TreeViewBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreeViewBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->contextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[3]))); break;
        case 1: _t->headerContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 2: _t->moveAfterLastColumn((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 3: _t->moveBeforeFirstColumn((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 4: _t->editAfterLastColumn((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 5: _t->editBeforeFirstColumn((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 6: _t->dropAllowed((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDragMoveEvent*>>(_a[3]))); break;
        case 7: _t->slotExpand(); break;
        case 8: _t->slotCollapse(); break;
        case 9: _t->closeEditor((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAbstractItemDelegate::EndEditHint>>(_a[2]))); break;
        case 10: _t->slotCurrentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 11: _t->slotHeaderContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 12: _t->doContextExpanded(); break;
        case 13: _t->doExpanded(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreeViewBase::*)(const QModelIndex & , const QPoint & , const QModelIndexList & )>(_a, &TreeViewBase::contextMenuRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeViewBase::*)(const QPoint & )>(_a, &TreeViewBase::headerContextMenuRequested, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeViewBase::*)(const QModelIndex & )>(_a, &TreeViewBase::moveAfterLastColumn, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeViewBase::*)(const QModelIndex & )>(_a, &TreeViewBase::moveBeforeFirstColumn, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeViewBase::*)(const QModelIndex & )>(_a, &TreeViewBase::editAfterLastColumn, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeViewBase::*)(const QModelIndex & )>(_a, &TreeViewBase::editBeforeFirstColumn, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeViewBase::*)(const QModelIndex & , int , QDragMoveEvent * )>(_a, &TreeViewBase::dropAllowed, 6))
            return;
    }
}

const QMetaObject *KPlato::TreeViewBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::TreeViewBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato12TreeViewBaseE_t>.strings))
        return static_cast<void*>(this);
    return QTreeView::qt_metacast(_clname);
}

int KPlato::TreeViewBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void KPlato::TreeViewBase::contextMenuRequested(const QModelIndex & _t1, const QPoint & _t2, const QModelIndexList & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void KPlato::TreeViewBase::headerContextMenuRequested(const QPoint & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KPlato::TreeViewBase::moveAfterLastColumn(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlato::TreeViewBase::moveBeforeFirstColumn(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KPlato::TreeViewBase::editAfterLastColumn(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KPlato::TreeViewBase::editBeforeFirstColumn(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KPlato::TreeViewBase::dropAllowed(const QModelIndex & _t1, int _t2, QDragMoveEvent * _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3);
}
namespace {
struct qt_meta_tag_ZN6KPlato28DoubleTreeViewPrintingDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::DoubleTreeViewPrintingDialog::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato28DoubleTreeViewPrintingDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::DoubleTreeViewPrintingDialog"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DoubleTreeViewPrintingDialog, qt_meta_tag_ZN6KPlato28DoubleTreeViewPrintingDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::DoubleTreeViewPrintingDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<PrintingDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato28DoubleTreeViewPrintingDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato28DoubleTreeViewPrintingDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato28DoubleTreeViewPrintingDialogE_t>.metaTypes,
    nullptr
} };

void KPlato::DoubleTreeViewPrintingDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DoubleTreeViewPrintingDialog *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::DoubleTreeViewPrintingDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::DoubleTreeViewPrintingDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato28DoubleTreeViewPrintingDialogE_t>.strings))
        return static_cast<void*>(this);
    return PrintingDialog::qt_metacast(_clname);
}

int KPlato::DoubleTreeViewPrintingDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PrintingDialog::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato18DoubleTreeViewBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::DoubleTreeViewBase::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato18DoubleTreeViewBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::DoubleTreeViewBase",
        "contextMenuRequested",
        "",
        "QModelIndex",
        "index",
        "QPoint",
        "pos",
        "QModelIndexList",
        "headerContextMenuRequested",
        "masterHeaderContextMenuRequested",
        "slaveHeaderContextMenuRequested",
        "currentChanged",
        "current",
        "previous",
        "selectionChanged",
        "dropAllowed",
        "dropIndicatorPosition",
        "QDragMoveEvent*",
        "event",
        "edit",
        "slotExpand",
        "slotCollapse",
        "slotSelectionChanged",
        "QItemSelection",
        "sel",
        "slotToRightView",
        "slotToLeftView",
        "slotEditToRightView",
        "slotEditToLeftView",
        "slotRightHeaderContextMenuRequested",
        "slotLeftHeaderContextMenuRequested",
        "slotLeftSortIndicatorChanged",
        "logicalIndex",
        "Qt::SortOrder",
        "order",
        "slotRightSortIndicatorChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'contextMenuRequested'
        QtMocHelpers::SignalData<void(const QModelIndex &, const QPoint &, const QModelIndexList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 2 },
        }}),
        // Signal 'headerContextMenuRequested'
        QtMocHelpers::SignalData<void(const QPoint &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'masterHeaderContextMenuRequested'
        QtMocHelpers::SignalData<void(const QPoint &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'slaveHeaderContextMenuRequested'
        QtMocHelpers::SignalData<void(const QPoint &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'currentChanged'
        QtMocHelpers::SignalData<void(const QModelIndex &, const QModelIndex &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 12 }, { 0x80000000 | 3, 13 },
        }}),
        // Signal 'selectionChanged'
        QtMocHelpers::SignalData<void(const QModelIndexList &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'dropAllowed'
        QtMocHelpers::SignalData<void(const QModelIndex &, int, QDragMoveEvent *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'edit'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotExpand'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCollapse'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 23, 24 }, { 0x80000000 | 23, 2 },
        }}),
        // Slot 'slotToRightView'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(25, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotToLeftView'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotEditToRightView'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotEditToLeftView'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotRightHeaderContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(29, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotLeftHeaderContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(30, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotLeftSortIndicatorChanged'
        QtMocHelpers::SlotData<void(int, Qt::SortOrder)>(31, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { 0x80000000 | 33, 34 },
        }}),
        // Slot 'slotRightSortIndicatorChanged'
        QtMocHelpers::SlotData<void(int, Qt::SortOrder)>(35, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { 0x80000000 | 33, 34 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DoubleTreeViewBase, qt_meta_tag_ZN6KPlato18DoubleTreeViewBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::DoubleTreeViewBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QSplitter::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18DoubleTreeViewBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18DoubleTreeViewBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato18DoubleTreeViewBaseE_t>.metaTypes,
    nullptr
} };

void KPlato::DoubleTreeViewBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DoubleTreeViewBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->contextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[3]))); break;
        case 1: _t->headerContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 2: _t->masterHeaderContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 3: _t->slaveHeaderContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 4: _t->currentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 5: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 6: _t->dropAllowed((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDragMoveEvent*>>(_a[3]))); break;
        case 7: _t->edit((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 8: _t->slotExpand(); break;
        case 9: _t->slotCollapse(); break;
        case 10: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 11: _t->slotToRightView((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 12: _t->slotToLeftView((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 13: _t->slotEditToRightView((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 14: _t->slotEditToLeftView((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 15: _t->slotRightHeaderContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 16: _t->slotLeftHeaderContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 17: _t->slotLeftSortIndicatorChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[2]))); break;
        case 18: _t->slotRightSortIndicatorChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DoubleTreeViewBase::*)(const QModelIndex & , const QPoint & , const QModelIndexList & )>(_a, &DoubleTreeViewBase::contextMenuRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DoubleTreeViewBase::*)(const QPoint & )>(_a, &DoubleTreeViewBase::headerContextMenuRequested, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DoubleTreeViewBase::*)(const QPoint & )>(_a, &DoubleTreeViewBase::masterHeaderContextMenuRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DoubleTreeViewBase::*)(const QPoint & )>(_a, &DoubleTreeViewBase::slaveHeaderContextMenuRequested, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DoubleTreeViewBase::*)(const QModelIndex & , const QModelIndex & )>(_a, &DoubleTreeViewBase::currentChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DoubleTreeViewBase::*)(const QModelIndexList & )>(_a, &DoubleTreeViewBase::selectionChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DoubleTreeViewBase::*)(const QModelIndex & , int , QDragMoveEvent * )>(_a, &DoubleTreeViewBase::dropAllowed, 6))
            return;
    }
}

const QMetaObject *KPlato::DoubleTreeViewBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::DoubleTreeViewBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18DoubleTreeViewBaseE_t>.strings))
        return static_cast<void*>(this);
    return QSplitter::qt_metacast(_clname);
}

int KPlato::DoubleTreeViewBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSplitter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void KPlato::DoubleTreeViewBase::contextMenuRequested(const QModelIndex & _t1, const QPoint & _t2, const QModelIndexList & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void KPlato::DoubleTreeViewBase::headerContextMenuRequested(const QPoint & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KPlato::DoubleTreeViewBase::masterHeaderContextMenuRequested(const QPoint & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlato::DoubleTreeViewBase::slaveHeaderContextMenuRequested(const QPoint & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KPlato::DoubleTreeViewBase::currentChanged(const QModelIndex & _t1, const QModelIndex & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void KPlato::DoubleTreeViewBase::selectionChanged(const QModelIndexList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KPlato::DoubleTreeViewBase::dropAllowed(const QModelIndex & _t1, int _t2, QDragMoveEvent * _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
