/********************************************************************************
** Form generated from reading UI file 'KoPageLayoutWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KOPAGELAYOUTWIDGET_H
#define UI_KOPAGELAYOUTWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "KoUnitDoubleSpinBox.h"

QT_BEGIN_NAMESPACE

class Ui_KoPageLayoutWidget
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout;
    QLabel *unitsLabel;
    QComboBox *units;
    QLabel *stylesLabel;
    QComboBox *pageStyle;
    QLabel *label_2;
    QComboBox *sizes;
    QLabel *label_3;
    QHBoxLayout *hboxLayout;
    KoUnitDoubleSpinBox *width;
    QLabel *label;
    KoUnitDoubleSpinBox *height;
    QLabel *label1;
    QRadioButton *portrait;
    QRadioButton *landscape;
    QLabel *facingPageLabel;
    QRadioButton *singleSided;
    QRadioButton *facingPages;
    QLabel *directionLabel;
    QComboBox *textDirection;
    QGroupBox *margins;
    QFormLayout *formLayout;
    QLabel *label2;
    KoUnitDoubleSpinBox *topMargin;
    QLabel *leftLabel;
    KoUnitDoubleSpinBox *bindingEdgeMargin;
    QLabel *rightLabel;
    KoUnitDoubleSpinBox *pageEdgeMargin;
    QLabel *label_12;
    KoUnitDoubleSpinBox *bottomMargin;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *KoPageLayoutWidget)
    {
        if (KoPageLayoutWidget->objectName().isEmpty())
            KoPageLayoutWidget->setObjectName("KoPageLayoutWidget");
        KoPageLayoutWidget->resize(341, 417);
        vboxLayout = new QVBoxLayout(KoPageLayoutWidget);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName("vboxLayout");
        gridLayout = new QGridLayout();
#ifndef UI_Q_OS_MAC
        gridLayout->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout->setObjectName("gridLayout");
        unitsLabel = new QLabel(KoPageLayoutWidget);
        unitsLabel->setObjectName("unitsLabel");
        unitsLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(unitsLabel, 0, 0, 1, 1);

        units = new QComboBox(KoPageLayoutWidget);
        units->setObjectName("units");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(units->sizePolicy().hasHeightForWidth());
        units->setSizePolicy(sizePolicy);

        gridLayout->addWidget(units, 0, 1, 1, 2);

        stylesLabel = new QLabel(KoPageLayoutWidget);
        stylesLabel->setObjectName("stylesLabel");
        stylesLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(stylesLabel, 1, 0, 1, 1);

        pageStyle = new QComboBox(KoPageLayoutWidget);
        pageStyle->setObjectName("pageStyle");

        gridLayout->addWidget(pageStyle, 1, 1, 1, 2);

        label_2 = new QLabel(KoPageLayoutWidget);
        label_2->setObjectName("label_2");
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_2, 2, 0, 1, 1);

        sizes = new QComboBox(KoPageLayoutWidget);
        sizes->setObjectName("sizes");

        gridLayout->addWidget(sizes, 2, 1, 1, 2);

        label_3 = new QLabel(KoPageLayoutWidget);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_3, 3, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        width = new KoUnitDoubleSpinBox(KoPageLayoutWidget);
        width->setObjectName("width");

        hboxLayout->addWidget(width);

        label = new QLabel(KoPageLayoutWidget);
        label->setObjectName("label");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);

        hboxLayout->addWidget(label);

        height = new KoUnitDoubleSpinBox(KoPageLayoutWidget);
        height->setObjectName("height");

        hboxLayout->addWidget(height);


        gridLayout->addLayout(hboxLayout, 3, 1, 1, 2);

        label1 = new QLabel(KoPageLayoutWidget);
        label1->setObjectName("label1");
        label1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label1, 4, 0, 1, 1);

        portrait = new QRadioButton(KoPageLayoutWidget);
        portrait->setObjectName("portrait");

        gridLayout->addWidget(portrait, 4, 1, 1, 1);

        landscape = new QRadioButton(KoPageLayoutWidget);
        landscape->setObjectName("landscape");

        gridLayout->addWidget(landscape, 4, 2, 1, 1);

        facingPageLabel = new QLabel(KoPageLayoutWidget);
        facingPageLabel->setObjectName("facingPageLabel");
        facingPageLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(facingPageLabel, 5, 0, 1, 1);

        singleSided = new QRadioButton(KoPageLayoutWidget);
        singleSided->setObjectName("singleSided");

        gridLayout->addWidget(singleSided, 5, 1, 1, 1);

        facingPages = new QRadioButton(KoPageLayoutWidget);
        facingPages->setObjectName("facingPages");

        gridLayout->addWidget(facingPages, 5, 2, 1, 1);

        directionLabel = new QLabel(KoPageLayoutWidget);
        directionLabel->setObjectName("directionLabel");
        directionLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(directionLabel, 6, 0, 1, 1);

        textDirection = new QComboBox(KoPageLayoutWidget);
        textDirection->addItem(QString());
        textDirection->addItem(QString());
        textDirection->addItem(QString());
        textDirection->setObjectName("textDirection");
        sizePolicy.setHeightForWidth(textDirection->sizePolicy().hasHeightForWidth());
        textDirection->setSizePolicy(sizePolicy);

        gridLayout->addWidget(textDirection, 6, 1, 1, 2);

        gridLayout->setColumnStretch(1, 1);
        gridLayout->setColumnStretch(2, 1);

        vboxLayout->addLayout(gridLayout);

        margins = new QGroupBox(KoPageLayoutWidget);
        margins->setObjectName("margins");
        formLayout = new QFormLayout(margins);
        formLayout->setObjectName("formLayout");
        label2 = new QLabel(margins);
        label2->setObjectName("label2");
        label2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label2);

        topMargin = new KoUnitDoubleSpinBox(margins);
        topMargin->setObjectName("topMargin");
        topMargin->setMaximum(999.990000000000009);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, topMargin);

        leftLabel = new QLabel(margins);
        leftLabel->setObjectName("leftLabel");
        leftLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, leftLabel);

        bindingEdgeMargin = new KoUnitDoubleSpinBox(margins);
        bindingEdgeMargin->setObjectName("bindingEdgeMargin");
        bindingEdgeMargin->setMaximum(999.990000000000009);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, bindingEdgeMargin);

        rightLabel = new QLabel(margins);
        rightLabel->setObjectName("rightLabel");
        rightLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, rightLabel);

        pageEdgeMargin = new KoUnitDoubleSpinBox(margins);
        pageEdgeMargin->setObjectName("pageEdgeMargin");
        pageEdgeMargin->setMaximum(999.990000000000009);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, pageEdgeMargin);

        label_12 = new QLabel(margins);
        label_12->setObjectName("label_12");
        label_12->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, label_12);

        bottomMargin = new KoUnitDoubleSpinBox(margins);
        bottomMargin->setObjectName("bottomMargin");
        bottomMargin->setMaximum(999.990000000000009);

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, bottomMargin);


        vboxLayout->addWidget(margins);

        spacerItem = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        vboxLayout->addItem(spacerItem);

#if QT_CONFIG(shortcut)
        unitsLabel->setBuddy(units);
        stylesLabel->setBuddy(pageStyle);
        label_2->setBuddy(sizes);
        label_3->setBuddy(width);
        label->setBuddy(height);
        directionLabel->setBuddy(textDirection);
        label2->setBuddy(topMargin);
        leftLabel->setBuddy(bindingEdgeMargin);
        rightLabel->setBuddy(pageEdgeMargin);
        label_12->setBuddy(bottomMargin);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(units, sizes);
        QWidget::setTabOrder(sizes, width);
        QWidget::setTabOrder(width, height);
        QWidget::setTabOrder(height, portrait);
        QWidget::setTabOrder(portrait, landscape);
        QWidget::setTabOrder(landscape, singleSided);
        QWidget::setTabOrder(singleSided, facingPages);
        QWidget::setTabOrder(facingPages, textDirection);
        QWidget::setTabOrder(textDirection, topMargin);
        QWidget::setTabOrder(topMargin, bindingEdgeMargin);
        QWidget::setTabOrder(bindingEdgeMargin, pageEdgeMargin);
        QWidget::setTabOrder(pageEdgeMargin, bottomMargin);

        retranslateUi(KoPageLayoutWidget);

        QMetaObject::connectSlotsByName(KoPageLayoutWidget);
    } // setupUi

    void retranslateUi(QWidget *KoPageLayoutWidget)
    {
        unitsLabel->setText(tr2i18n("Unit:", nullptr));
        stylesLabel->setText(tr2i18n("Follow style:", nullptr));
        label_2->setText(tr2i18n("Size:", nullptr));
        label_3->setText(tr2i18n("Width:", nullptr));
        label->setText(tr2i18n("Height:", nullptr));
        label1->setText(tr2i18n("Orientation:", nullptr));
        portrait->setText(tr2i18n("Portrait", nullptr));
        landscape->setText(tr2i18n("Landscape", nullptr));
        facingPageLabel->setText(tr2i18n("Facing pages:", nullptr));
        singleSided->setText(tr2i18n("Single sided", nullptr));
        facingPages->setText(tr2i18n("Facing pages", nullptr));
        directionLabel->setText(tr2i18n("Text direction:", nullptr));
        textDirection->setItemText(0, tr2i18n("Automatic", nullptr));
        textDirection->setItemText(1, tr2i18n("Left to right", nullptr));
        textDirection->setItemText(2, tr2i18n("Right to left", nullptr));

        margins->setTitle(tr2i18n("Margins", nullptr));
        label2->setText(tr2i18n("Top:", nullptr));
        leftLabel->setText(tr2i18n("Binding edge:", nullptr));
        rightLabel->setText(tr2i18n("Page edge:", nullptr));
        label_12->setText(tr2i18n("Bottom:", nullptr));
        (void)KoPageLayoutWidget;
    } // retranslateUi

};

namespace Ui {
    class KoPageLayoutWidget: public Ui_KoPageLayoutWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KOPAGELAYOUTWIDGET_H

