/* This file is part of the KDE project
 * SPDX-FileCopyrightText: 2009 Boudewijn Rempt
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */
#ifndef KO_ENCRYPTION_CHECKER
#define KO_ENCRYPTION_CHECKER

#include "kostore_export.h"

namespace KoEncryptionChecker
{

KOSTORE_EXPORT bool isEncryptionSupported();

}

#endif
