/****************************************************************************
** Meta object code from reading C++ file 'view.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../calligraplan-4.0.1/src/workpackage/view.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'view.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KPlatoWork4ViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlatoWork::View::qt_create_metaobjectdata<qt_meta_tag_ZN10KPlatoWork4ViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlatoWork::View",
        "currentScheduleManagerChanged",
        "",
        "KPlato::ScheduleManager*",
        "sm",
        "openInternalDocument",
        "KoStore*",
        "sigUpdateReadWrite",
        "viewDocument",
        "KPlato::Document*",
        "doc",
        "slotEditCut",
        "slotEditCopy",
        "slotEditPaste",
        "slotConfigure",
        "slotPopupMenu",
        "menuname",
        "QPoint",
        "pos",
        "slotTaskProgress",
        "slotTaskCompletion",
        "slotCurrentChanged",
        "index",
        "slotProgressChanged",
        "value",
        "slotEditDocument",
        "slotViewDocument",
        "slotRemoveDocument",
        "slotSendPackage",
        "slotTaskDescription",
        "slotRemoveCurrentPackage",
        "slotRemoveSelectedPackages",
        "slotSelectionChanged",
        "slotViewList",
        "slotViewGantt"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentScheduleManagerChanged'
        QtMocHelpers::SignalData<void(KPlato::ScheduleManager *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'openInternalDocument'
        QtMocHelpers::SignalData<void(KoStore *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Signal 'sigUpdateReadWrite'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'viewDocument'
        QtMocHelpers::SignalData<void(KPlato::Document *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotEditCut'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditCopy'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditPaste'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotConfigure'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPopupMenu'
        QtMocHelpers::SlotData<void(const QString &, const QPoint &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotTaskProgress'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotTaskCompletion'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCurrentChanged'
        QtMocHelpers::SlotData<void(int)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 22 },
        }}),
        // Slot 'slotProgressChanged'
        QtMocHelpers::SlotData<void(int)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'slotEditDocument'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEditDocument'
        QtMocHelpers::SlotData<void(KPlato::Document *)>(25, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotViewDocument'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRemoveDocument'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSendPackage'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotTaskDescription'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRemoveCurrentPackage'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRemoveSelectedPackages'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotViewList'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotViewGantt'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<View, qt_meta_tag_ZN10KPlatoWork4ViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlatoWork::View::staticMetaObject = { {
    QMetaObject::SuperData::link<QStackedWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork4ViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork4ViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KPlatoWork4ViewE_t>.metaTypes,
    nullptr
} };

void KPlatoWork::View::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<View *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentScheduleManagerChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 1: _t->openInternalDocument((*reinterpret_cast<std::add_pointer_t<KoStore*>>(_a[1]))); break;
        case 2: _t->sigUpdateReadWrite((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->viewDocument((*reinterpret_cast<std::add_pointer_t<KPlato::Document*>>(_a[1]))); break;
        case 4: _t->slotEditCut(); break;
        case 5: _t->slotEditCopy(); break;
        case 6: _t->slotEditPaste(); break;
        case 7: _t->slotConfigure(); break;
        case 8: _t->slotPopupMenu((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 9: _t->slotTaskProgress(); break;
        case 10: _t->slotTaskCompletion(); break;
        case 11: _t->slotCurrentChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->slotProgressChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->slotEditDocument(); break;
        case 14: _t->slotEditDocument((*reinterpret_cast<std::add_pointer_t<KPlato::Document*>>(_a[1]))); break;
        case 15: _t->slotViewDocument(); break;
        case 16: _t->slotRemoveDocument(); break;
        case 17: _t->slotSendPackage(); break;
        case 18: _t->slotTaskDescription(); break;
        case 19: _t->slotRemoveCurrentPackage(); break;
        case 20: _t->slotRemoveSelectedPackages(); break;
        case 21: _t->slotSelectionChanged(); break;
        case 22: _t->slotViewList(); break;
        case 23: _t->slotViewGantt(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (View::*)(KPlato::ScheduleManager * )>(_a, &View::currentScheduleManagerChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (View::*)(KoStore * )>(_a, &View::openInternalDocument, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (View::*)(bool )>(_a, &View::sigUpdateReadWrite, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (View::*)(KPlato::Document * )>(_a, &View::viewDocument, 3))
            return;
    }
}

const QMetaObject *KPlatoWork::View::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlatoWork::View::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KPlatoWork4ViewE_t>.strings))
        return static_cast<void*>(this);
    return QStackedWidget::qt_metacast(_clname);
}

int KPlatoWork::View::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStackedWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void KPlatoWork::View::currentScheduleManagerChanged(KPlato::ScheduleManager * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlatoWork::View::openInternalDocument(KoStore * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KPlatoWork::View::sigUpdateReadWrite(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlatoWork::View::viewDocument(KPlato::Document * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
