/****************************************************************************
** Meta object code from reading C++ file 'KoPageLayoutWidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/widgets/KoPageLayoutWidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KoPageLayoutWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18KoPageLayoutWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto KoPageLayoutWidget::qt_create_metaobjectdata<qt_meta_tag_ZN18KoPageLayoutWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KoPageLayoutWidget",
        "layoutChanged",
        "",
        "KoPageLayout",
        "layout",
        "unitChanged",
        "KoUnit",
        "unit",
        "setPageLayout",
        "setTextAreaAvailable",
        "available",
        "setApplyToDocument",
        "apply",
        "sizeChanged",
        "row",
        "slotUnitChanged",
        "facingPagesChanged",
        "optionsChanged",
        "marginsChanged",
        "orientationChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'layoutChanged'
        QtMocHelpers::SignalData<void(const KoPageLayout &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'unitChanged'
        QtMocHelpers::SignalData<void(const KoUnit &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setPageLayout'
        QtMocHelpers::SlotData<void(const KoPageLayout &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setTextAreaAvailable'
        QtMocHelpers::SlotData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'setApplyToDocument'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'sizeChanged'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'slotUnitChanged'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'facingPagesChanged'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'optionsChanged'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'marginsChanged'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'orientationChanged'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KoPageLayoutWidget, qt_meta_tag_ZN18KoPageLayoutWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KoPageLayoutWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KoPageLayoutWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KoPageLayoutWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18KoPageLayoutWidgetE_t>.metaTypes,
    nullptr
} };

void KoPageLayoutWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KoPageLayoutWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->layoutChanged((*reinterpret_cast<std::add_pointer_t<KoPageLayout>>(_a[1]))); break;
        case 1: _t->unitChanged((*reinterpret_cast<std::add_pointer_t<KoUnit>>(_a[1]))); break;
        case 2: _t->setPageLayout((*reinterpret_cast<std::add_pointer_t<KoPageLayout>>(_a[1]))); break;
        case 3: _t->setTextAreaAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->setApplyToDocument((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->sizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->slotUnitChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->facingPagesChanged(); break;
        case 8: _t->optionsChanged(); break;
        case 9: _t->marginsChanged(); break;
        case 10: _t->orientationChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KoPageLayoutWidget::*)(const KoPageLayout & )>(_a, &KoPageLayoutWidget::layoutChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KoPageLayoutWidget::*)(const KoUnit & )>(_a, &KoPageLayoutWidget::unitChanged, 1))
            return;
    }
}

const QMetaObject *KoPageLayoutWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KoPageLayoutWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KoPageLayoutWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int KoPageLayoutWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void KoPageLayoutWidget::layoutChanged(const KoPageLayout & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KoPageLayoutWidget::unitChanged(const KoUnit & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
