/********************************************************************************
** Form generated from reading UI file 'kpttaskprogresspanelbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTTASKPROGRESSPANELBASE_H
#define UI_KPTTASKPROGRESSPANELBASE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "KComboBox"
#include "kptusedefforteditor.h"

namespace KPlato {

class Ui_TaskProgressPanelBase
{
public:
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_3;
    QLineEdit *taskName;
    QFrame *frame;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *started;
    QDateTimeEdit *startTime;
    QLabel *label_4;
    QDateTimeEdit *finishTime;
    QCheckBox *finished;
    QSpacerItem *spacerItem;
    QWidget *editModeWidget;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label;
    QComboBox *editmode;
    QFrame *ui_resourceFrame;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *hboxLayout;
    QSpinBox *ui_year;
    QPushButton *prevWeekBtn;
    KComboBox *weekNumber;
    QPushButton *nextWeekBtn;
    QSpacerItem *spacerItem1;
    QWidget *addResourceWidget;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_2;
    QComboBox *ui_resourceCombo;
    UsedEffortEditor *resourceTable;
    QFrame *ui_entryFrame;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QPushButton *addEntryBtn;
    QPushButton *removeEntryBtn;
    QSpacerItem *spacerItem2;
    CompletionEntryEditor *entryTable;

    void setupUi(QWidget *KPlato__TaskProgressPanelBase)
    {
        if (KPlato__TaskProgressPanelBase->objectName().isEmpty())
            KPlato__TaskProgressPanelBase->setObjectName("KPlato__TaskProgressPanelBase");
        KPlato__TaskProgressPanelBase->resize(887, 427);
        verticalLayout_3 = new QVBoxLayout(KPlato__TaskProgressPanelBase);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label_3 = new QLabel(KPlato__TaskProgressPanelBase);
        label_3->setObjectName("label_3");

        horizontalLayout_5->addWidget(label_3);

        taskName = new QLineEdit(KPlato__TaskProgressPanelBase);
        taskName->setObjectName("taskName");
        taskName->setReadOnly(true);

        horizontalLayout_5->addWidget(taskName);


        verticalLayout_3->addLayout(horizontalLayout_5);

        frame = new QFrame(KPlato__TaskProgressPanelBase);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        horizontalLayout_2 = new QHBoxLayout(frame);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        started = new QCheckBox(frame);
        started->setObjectName("started");

        horizontalLayout_2->addWidget(started);

        startTime = new QDateTimeEdit(frame);
        startTime->setObjectName("startTime");
        startTime->setCalendarPopup(true);

        horizontalLayout_2->addWidget(startTime);

        label_4 = new QLabel(frame);
        label_4->setObjectName("label_4");

        horizontalLayout_2->addWidget(label_4);

        finishTime = new QDateTimeEdit(frame);
        finishTime->setObjectName("finishTime");
        finishTime->setCalendarPopup(true);

        horizontalLayout_2->addWidget(finishTime);

        finished = new QCheckBox(frame);
        finished->setObjectName("finished");

        horizontalLayout_2->addWidget(finished);

        spacerItem = new QSpacerItem(36, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(spacerItem);

        editModeWidget = new QWidget(frame);
        editModeWidget->setObjectName("editModeWidget");
        horizontalLayout_3 = new QHBoxLayout(editModeWidget);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_3->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(editModeWidget);
        label->setObjectName("label");

        horizontalLayout_3->addWidget(label);

        editmode = new QComboBox(editModeWidget);
        editmode->addItem(QString());
        editmode->addItem(QString());
        editmode->setObjectName("editmode");

        horizontalLayout_3->addWidget(editmode);


        horizontalLayout_2->addWidget(editModeWidget);


        verticalLayout_3->addWidget(frame);

        ui_resourceFrame = new QFrame(KPlato__TaskProgressPanelBase);
        ui_resourceFrame->setObjectName("ui_resourceFrame");
        ui_resourceFrame->setFrameShape(QFrame::StyledPanel);
        ui_resourceFrame->setFrameShadow(QFrame::Raised);
        verticalLayout_2 = new QVBoxLayout(ui_resourceFrame);
        verticalLayout_2->setObjectName("verticalLayout_2");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        ui_year = new QSpinBox(ui_resourceFrame);
        ui_year->setObjectName("ui_year");
        ui_year->setMinimum(1900);
        ui_year->setMaximum(2999);

        hboxLayout->addWidget(ui_year);

        prevWeekBtn = new QPushButton(ui_resourceFrame);
        prevWeekBtn->setObjectName("prevWeekBtn");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("go-previous");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        prevWeekBtn->setIcon(icon);

        hboxLayout->addWidget(prevWeekBtn);

        weekNumber = new KComboBox(ui_resourceFrame);
        weekNumber->setObjectName("weekNumber");

        hboxLayout->addWidget(weekNumber);

        nextWeekBtn = new QPushButton(ui_resourceFrame);
        nextWeekBtn->setObjectName("nextWeekBtn");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("go-next");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        nextWeekBtn->setIcon(icon1);

        hboxLayout->addWidget(nextWeekBtn);

        spacerItem1 = new QSpacerItem(16, 25, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem1);

        addResourceWidget = new QWidget(ui_resourceFrame);
        addResourceWidget->setObjectName("addResourceWidget");
        horizontalLayout_4 = new QHBoxLayout(addResourceWidget);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(0, 0, 0, 0);
        label_2 = new QLabel(addResourceWidget);
        label_2->setObjectName("label_2");

        horizontalLayout_4->addWidget(label_2);

        ui_resourceCombo = new QComboBox(addResourceWidget);
        ui_resourceCombo->setObjectName("ui_resourceCombo");

        horizontalLayout_4->addWidget(ui_resourceCombo);


        hboxLayout->addWidget(addResourceWidget);


        verticalLayout_2->addLayout(hboxLayout);

        resourceTable = new UsedEffortEditor(ui_resourceFrame);
        resourceTable->setObjectName("resourceTable");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(resourceTable->sizePolicy().hasHeightForWidth());
        resourceTable->setSizePolicy(sizePolicy);
        resourceTable->setTabKeyNavigation(false);
        resourceTable->horizontalHeader()->setDefaultSectionSize(70);
        resourceTable->horizontalHeader()->setStretchLastSection(true);

        verticalLayout_2->addWidget(resourceTable);


        verticalLayout_3->addWidget(ui_resourceFrame);

        ui_entryFrame = new QFrame(KPlato__TaskProgressPanelBase);
        ui_entryFrame->setObjectName("ui_entryFrame");
        ui_entryFrame->setFrameShape(QFrame::StyledPanel);
        ui_entryFrame->setFrameShadow(QFrame::Raised);
        verticalLayout = new QVBoxLayout(ui_entryFrame);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        addEntryBtn = new QPushButton(ui_entryFrame);
        addEntryBtn->setObjectName("addEntryBtn");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        addEntryBtn->setIcon(icon2);

        horizontalLayout->addWidget(addEntryBtn);

        removeEntryBtn = new QPushButton(ui_entryFrame);
        removeEntryBtn->setObjectName("removeEntryBtn");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        removeEntryBtn->setIcon(icon3);

        horizontalLayout->addWidget(removeEntryBtn);

        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(spacerItem2);


        verticalLayout->addLayout(horizontalLayout);

        entryTable = new CompletionEntryEditor(ui_entryFrame);
        entryTable->setObjectName("entryTable");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(entryTable->sizePolicy().hasHeightForWidth());
        entryTable->setSizePolicy(sizePolicy1);
        entryTable->setMinimumSize(QSize(0, 100));
        entryTable->setTabKeyNavigation(false);

        verticalLayout->addWidget(entryTable);


        verticalLayout_3->addWidget(ui_entryFrame);

#if QT_CONFIG(shortcut)
        label_4->setBuddy(finishTime);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(started, startTime);
        QWidget::setTabOrder(startTime, finishTime);
        QWidget::setTabOrder(finishTime, finished);
        QWidget::setTabOrder(finished, editmode);
        QWidget::setTabOrder(editmode, ui_year);
        QWidget::setTabOrder(ui_year, prevWeekBtn);
        QWidget::setTabOrder(prevWeekBtn, weekNumber);
        QWidget::setTabOrder(weekNumber, nextWeekBtn);
        QWidget::setTabOrder(nextWeekBtn, ui_resourceCombo);
        QWidget::setTabOrder(ui_resourceCombo, addEntryBtn);
        QWidget::setTabOrder(addEntryBtn, removeEntryBtn);
        QWidget::setTabOrder(removeEntryBtn, entryTable);
        QWidget::setTabOrder(entryTable, taskName);
        QWidget::setTabOrder(taskName, resourceTable);

        retranslateUi(KPlato__TaskProgressPanelBase);

        QMetaObject::connectSlotsByName(KPlato__TaskProgressPanelBase);
    } // setupUi

    void retranslateUi(QWidget *KPlato__TaskProgressPanelBase)
    {
        label_3->setText(tr2i18n("Task:", nullptr));
        started->setText(tr2i18n("Started:", nullptr));
#if QT_CONFIG(shortcut)
        started->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        label_4->setText(tr2i18n("Finished:", nullptr));
        finished->setText(QString());
#if QT_CONFIG(shortcut)
        finished->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        label->setText(tr2i18n("Edit mode:", nullptr));
        editmode->setItemText(0, tr2i18n("Per resource", nullptr));
        editmode->setItemText(1, tr2i18n("Per task", nullptr));

        prevWeekBtn->setText(tr2i18n("Prev", nullptr));
        nextWeekBtn->setText(tr2i18n("Next", nullptr));
        label_2->setText(tr2i18n("Add resource:", nullptr));
        addEntryBtn->setText(tr2i18n("Add Entry", nullptr));
        removeEntryBtn->setText(tr2i18n("Remove Entry", nullptr));
        (void)KPlato__TaskProgressPanelBase;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class TaskProgressPanelBase: public Ui_TaskProgressPanelBase {};
} // namespace Ui
} // namespace KPlato

#endif // KPTTASKPROGRESSPANELBASE_H

