/********************************************************************************
** Form generated from reading UI file 'kptmilestoneprogresspanelbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTMILESTONEPROGRESSPANELBASE_H
#define UI_KPTMILESTONEPROGRESSPANELBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_MilestoneProgressPanelBase
{
public:
    QVBoxLayout *vboxLayout;
    QCheckBox *finished;
    QDateTimeEdit *finishTime;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *KPlato__MilestoneProgressPanelBase)
    {
        if (KPlato__MilestoneProgressPanelBase->objectName().isEmpty())
            KPlato__MilestoneProgressPanelBase->setObjectName("KPlato__MilestoneProgressPanelBase");
        KPlato__MilestoneProgressPanelBase->resize(321, 90);
        vboxLayout = new QVBoxLayout(KPlato__MilestoneProgressPanelBase);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName("vboxLayout");
        finished = new QCheckBox(KPlato__MilestoneProgressPanelBase);
        finished->setObjectName("finished");

        vboxLayout->addWidget(finished);

        finishTime = new QDateTimeEdit(KPlato__MilestoneProgressPanelBase);
        finishTime->setObjectName("finishTime");
        finishTime->setCalendarPopup(true);

        vboxLayout->addWidget(finishTime);

        spacerItem = new QSpacerItem(321, 31, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem);


        retranslateUi(KPlato__MilestoneProgressPanelBase);

        QMetaObject::connectSlotsByName(KPlato__MilestoneProgressPanelBase);
    } // setupUi

    void retranslateUi(QWidget *KPlato__MilestoneProgressPanelBase)
    {
        finished->setText(tr2i18n("Finished", nullptr));
        (void)KPlato__MilestoneProgressPanelBase;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class MilestoneProgressPanelBase: public Ui_MilestoneProgressPanelBase {};
} // namespace Ui
} // namespace KPlato

#endif // KPTMILESTONEPROGRESSPANELBASE_H

