/********************************************************************************
** Form generated from reading UI file 'KoConfigDocumentPage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KOCONFIGDOCUMENTPAGE_H
#define UI_KOCONFIGDOCUMENTPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KoConfigDocumentPage
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QCheckBox *kcfg_CreateBackupFile;
    QLabel *label;
    QLabel *label_2;
    QSpinBox *kcfg_AutoSave;
    QSpacerItem *verticalSpacer;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *KoConfigDocumentPage)
    {
        if (KoConfigDocumentPage->objectName().isEmpty())
            KoConfigDocumentPage->setObjectName("KoConfigDocumentPage");
        KoConfigDocumentPage->resize(400, 300);
        verticalLayout = new QVBoxLayout(KoConfigDocumentPage);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(KoConfigDocumentPage);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        kcfg_CreateBackupFile = new QCheckBox(groupBox);
        kcfg_CreateBackupFile->setObjectName("kcfg_CreateBackupFile");

        gridLayout->addWidget(kcfg_CreateBackupFile, 1, 1, 1, 1);

        label = new QLabel(groupBox);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        kcfg_AutoSave = new QSpinBox(groupBox);
        kcfg_AutoSave->setObjectName("kcfg_AutoSave");
        kcfg_AutoSave->setMaximum(60);

        gridLayout->addWidget(kcfg_AutoSave, 0, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 2, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 2, 1, 1);


        verticalLayout->addWidget(groupBox);


        retranslateUi(KoConfigDocumentPage);

        QMetaObject::connectSlotsByName(KoConfigDocumentPage);
    } // setupUi

    void retranslateUi(QWidget *KoConfigDocumentPage)
    {
        KoConfigDocumentPage->setWindowTitle(QString());
        groupBox->setTitle(QString());
        kcfg_CreateBackupFile->setText(QString());
        label->setText(tr2i18n("Autosave interval:", nullptr));
        label_2->setText(tr2i18n("Create backup file:", nullptr));
        kcfg_AutoSave->setSpecialValueText(QString());
        kcfg_AutoSave->setSuffix(QString());
    } // retranslateUi

};

namespace Ui {
    class KoConfigDocumentPage: public Ui_KoConfigDocumentPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KOCONFIGDOCUMENTPAGE_H

