/********************************************************************************
** Form generated from reading UI file 'ScheduleManagerInfoDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCHEDULEMANAGERINFODIALOG_H
#define UI_SCHEDULEMANAGERINFODIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ScheduleManagerInfoDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTreeView *treeView;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ScheduleManagerInfoDialog)
    {
        if (ScheduleManagerInfoDialog->objectName().isEmpty())
            ScheduleManagerInfoDialog->setObjectName("ScheduleManagerInfoDialog");
        ScheduleManagerInfoDialog->resize(678, 300);
        ScheduleManagerInfoDialog->setWindowTitle(QString::fromUtf8(""));
        verticalLayout = new QVBoxLayout(ScheduleManagerInfoDialog);
        verticalLayout->setObjectName("verticalLayout");
        treeView = new QTreeView(ScheduleManagerInfoDialog);
        treeView->setObjectName("treeView");
        treeView->setAlternatingRowColors(true);
        treeView->setRootIsDecorated(false);
        treeView->setItemsExpandable(false);
        treeView->setWordWrap(true);
        treeView->setExpandsOnDoubleClick(false);

        verticalLayout->addWidget(treeView);

        buttonBox = new QDialogButtonBox(ScheduleManagerInfoDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Abort|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(ScheduleManagerInfoDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, ScheduleManagerInfoDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, ScheduleManagerInfoDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(ScheduleManagerInfoDialog);
    } // setupUi

    void retranslateUi(QDialog *ScheduleManagerInfoDialog)
    {
        (void)ScheduleManagerInfoDialog;
    } // retranslateUi

};

namespace Ui {
    class ScheduleManagerInfoDialog: public Ui_ScheduleManagerInfoDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SCHEDULEMANAGERINFODIALOG_H

