/********************************************************************************
** Form generated from reading UI file 'kpttaskgeneralpanelbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTTASKGENERALPANELBASE_H
#define UI_KPTTASKGENERALPANELBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTimeEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "KComboBox"
#include "KLineEdit"
#include "kptdurationspinbox.h"

namespace KPlato {

class Ui_TaskGeneralPanelBase
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *wbslabel;
    QLabel *wbsfield;
    QLabel *namelabel;
    KLineEdit *namefield;
    QLabel *leaderlabel;
    QHBoxLayout *hboxLayout;
    KLineEdit *leaderfield;
    QPushButton *chooseLeader;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QSpinBox *ui_priority;
    QSpacerItem *horizontalSpacer_3;
    QGroupBox *schedulingGroup;
    QGridLayout *gridLayout1;
    QFrame *frame;
    QGridLayout *gridLayout2;
    QLabel *textLabel3;
    KComboBox *scheduleType;
    QTimeEdit *scheduleStartTime;
    QTimeEdit *scheduleEndTime;
    QDateEdit *scheduleStartDate;
    QDateEdit *scheduleEndDate;
    QSpacerItem *horizontalSpacer;
    QFrame *frame_2;
    QGridLayout *gridLayout_3;
    QLabel *textLabel3_2;
    QSpinBox *pessimisticValue;
    QSpacerItem *horizontalSpacer_2;
    QLabel *textLabel3_2_3;
    KComboBox *calendarCombo;
    KComboBox *risk;
    QLabel *textLabel1_2;
    QSpinBox *optimisticValue;
    QLabel *textLabel2_2;
    DurationSpinBox *estimate;
    KComboBox *estimateType;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPlato__TaskGeneralPanelBase)
    {
        if (KPlato__TaskGeneralPanelBase->objectName().isEmpty())
            KPlato__TaskGeneralPanelBase->setObjectName("KPlato__TaskGeneralPanelBase");
        KPlato__TaskGeneralPanelBase->resize(526, 316);
        verticalLayout = new QVBoxLayout(KPlato__TaskGeneralPanelBase);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        wbslabel = new QLabel(KPlato__TaskGeneralPanelBase);
        wbslabel->setObjectName("wbslabel");
        wbslabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        wbslabel->setWordWrap(false);

        gridLayout->addWidget(wbslabel, 0, 0, 1, 1);

        wbsfield = new QLabel(KPlato__TaskGeneralPanelBase);
        wbsfield->setObjectName("wbsfield");
        wbsfield->setFrameShape(QFrame::StyledPanel);
        wbsfield->setFrameShadow(QFrame::Sunken);
        wbsfield->setWordWrap(false);

        gridLayout->addWidget(wbsfield, 0, 1, 1, 1);

        namelabel = new QLabel(KPlato__TaskGeneralPanelBase);
        namelabel->setObjectName("namelabel");
        namelabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        namelabel->setWordWrap(false);

        gridLayout->addWidget(namelabel, 1, 0, 1, 1);

        namefield = new KLineEdit(KPlato__TaskGeneralPanelBase);
        namefield->setObjectName("namefield");

        gridLayout->addWidget(namefield, 1, 1, 1, 1);

        leaderlabel = new QLabel(KPlato__TaskGeneralPanelBase);
        leaderlabel->setObjectName("leaderlabel");
        leaderlabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        leaderlabel->setWordWrap(false);

        gridLayout->addWidget(leaderlabel, 2, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        leaderfield = new KLineEdit(KPlato__TaskGeneralPanelBase);
        leaderfield->setObjectName("leaderfield");

        hboxLayout->addWidget(leaderfield);

        chooseLeader = new QPushButton(KPlato__TaskGeneralPanelBase);
        chooseLeader->setObjectName("chooseLeader");

        hboxLayout->addWidget(chooseLeader);


        gridLayout->addLayout(hboxLayout, 2, 1, 1, 1);

        label = new QLabel(KPlato__TaskGeneralPanelBase);
        label->setObjectName("label");

        gridLayout->addWidget(label, 3, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        ui_priority = new QSpinBox(KPlato__TaskGeneralPanelBase);
        ui_priority->setObjectName("ui_priority");

        horizontalLayout->addWidget(ui_priority);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);


        gridLayout->addLayout(horizontalLayout, 3, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        schedulingGroup = new QGroupBox(KPlato__TaskGeneralPanelBase);
        schedulingGroup->setObjectName("schedulingGroup");
        gridLayout1 = new QGridLayout(schedulingGroup);
        gridLayout1->setObjectName("gridLayout1");
        frame = new QFrame(schedulingGroup);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::NoFrame);
        frame->setFrameShadow(QFrame::Raised);
        gridLayout2 = new QGridLayout(frame);
        gridLayout2->setObjectName("gridLayout2");
        gridLayout2->setContentsMargins(0, 0, 0, 0);
        textLabel3 = new QLabel(frame);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3->setWordWrap(false);

        gridLayout2->addWidget(textLabel3, 0, 0, 1, 1);

        scheduleType = new KComboBox(frame);
        scheduleType->addItem(QString());
        scheduleType->addItem(QString());
        scheduleType->addItem(QString());
        scheduleType->addItem(QString());
        scheduleType->addItem(QString());
        scheduleType->addItem(QString());
        scheduleType->addItem(QString());
        scheduleType->setObjectName("scheduleType");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(scheduleType->sizePolicy().hasHeightForWidth());
        scheduleType->setSizePolicy(sizePolicy);

        gridLayout2->addWidget(scheduleType, 0, 1, 1, 1);

        scheduleStartTime = new QTimeEdit(frame);
        scheduleStartTime->setObjectName("scheduleStartTime");

        gridLayout2->addWidget(scheduleStartTime, 0, 3, 1, 1);

        scheduleEndTime = new QTimeEdit(frame);
        scheduleEndTime->setObjectName("scheduleEndTime");

        gridLayout2->addWidget(scheduleEndTime, 1, 3, 1, 1);

        scheduleStartDate = new QDateEdit(frame);
        scheduleStartDate->setObjectName("scheduleStartDate");
        scheduleStartDate->setCalendarPopup(true);

        gridLayout2->addWidget(scheduleStartDate, 0, 2, 1, 1);

        scheduleEndDate = new QDateEdit(frame);
        scheduleEndDate->setObjectName("scheduleEndDate");
        scheduleEndDate->setCalendarPopup(true);

        gridLayout2->addWidget(scheduleEndDate, 1, 2, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout2->addItem(horizontalSpacer, 0, 4, 1, 1);


        gridLayout1->addWidget(frame, 0, 0, 1, 1);

        frame_2 = new QFrame(schedulingGroup);
        frame_2->setObjectName("frame_2");
        frame_2->setFrameShape(QFrame::NoFrame);
        frame_2->setFrameShadow(QFrame::Raised);
        gridLayout_3 = new QGridLayout(frame_2);
        gridLayout_3->setObjectName("gridLayout_3");
        textLabel3_2 = new QLabel(frame_2);
        textLabel3_2->setObjectName("textLabel3_2");
        textLabel3_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3_2->setWordWrap(false);

        gridLayout_3->addWidget(textLabel3_2, 0, 0, 1, 1);

        pessimisticValue = new QSpinBox(frame_2);
        pessimisticValue->setObjectName("pessimisticValue");
        pessimisticValue->setMaximum(999);

        gridLayout_3->addWidget(pessimisticValue, 1, 6, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_2, 1, 7, 1, 1);

        textLabel3_2_3 = new QLabel(frame_2);
        textLabel3_2_3->setObjectName("textLabel3_2_3");
        textLabel3_2_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3_2_3->setWordWrap(false);

        gridLayout_3->addWidget(textLabel3_2_3, 1, 0, 1, 1);

        calendarCombo = new KComboBox(frame_2);
        calendarCombo->setObjectName("calendarCombo");

        gridLayout_3->addWidget(calendarCombo, 0, 5, 1, 2);

        risk = new KComboBox(frame_2);
        risk->addItem(QString());
        risk->addItem(QString());
        risk->addItem(QString());
        risk->setObjectName("risk");
        sizePolicy.setHeightForWidth(risk->sizePolicy().hasHeightForWidth());
        risk->setSizePolicy(sizePolicy);
        risk->setEditable(false);

        gridLayout_3->addWidget(risk, 1, 1, 1, 1);

        textLabel1_2 = new QLabel(frame_2);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel1_2->setWordWrap(false);

        gridLayout_3->addWidget(textLabel1_2, 1, 2, 1, 1);

        optimisticValue = new QSpinBox(frame_2);
        optimisticValue->setObjectName("optimisticValue");
        optimisticValue->setMinimum(-99);
        optimisticValue->setMaximum(0);

        gridLayout_3->addWidget(optimisticValue, 1, 3, 1, 1);

        textLabel2_2 = new QLabel(frame_2);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel2_2->setWordWrap(false);

        gridLayout_3->addWidget(textLabel2_2, 1, 5, 1, 1);

        estimate = new DurationSpinBox(frame_2);
        estimate->setObjectName("estimate");

        gridLayout_3->addWidget(estimate, 0, 3, 1, 1);

        estimateType = new KComboBox(frame_2);
        estimateType->addItem(QString());
        estimateType->addItem(QString());
        estimateType->addItem(QString());
        estimateType->setObjectName("estimateType");
        sizePolicy.setHeightForWidth(estimateType->sizePolicy().hasHeightForWidth());
        estimateType->setSizePolicy(sizePolicy);
        estimateType->setEditable(false);

        gridLayout_3->addWidget(estimateType, 0, 1, 1, 2);


        gridLayout1->addWidget(frame_2, 1, 0, 1, 1);


        verticalLayout->addWidget(schedulingGroup);

        verticalSpacer = new QSpacerItem(20, 42, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        namelabel->setBuddy(namefield);
        leaderlabel->setBuddy(leaderfield);
        textLabel3->setBuddy(scheduleType);
        textLabel3_2->setBuddy(estimateType);
        textLabel3_2_3->setBuddy(risk);
        textLabel1_2->setBuddy(optimisticValue);
        textLabel2_2->setBuddy(pessimisticValue);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(namefield, leaderfield);
        QWidget::setTabOrder(leaderfield, chooseLeader);
        QWidget::setTabOrder(chooseLeader, scheduleType);
        QWidget::setTabOrder(scheduleType, scheduleStartDate);
        QWidget::setTabOrder(scheduleStartDate, scheduleStartTime);
        QWidget::setTabOrder(scheduleStartTime, scheduleEndDate);
        QWidget::setTabOrder(scheduleEndDate, scheduleEndTime);
        QWidget::setTabOrder(scheduleEndTime, calendarCombo);
        QWidget::setTabOrder(calendarCombo, risk);
        QWidget::setTabOrder(risk, optimisticValue);
        QWidget::setTabOrder(optimisticValue, pessimisticValue);

        retranslateUi(KPlato__TaskGeneralPanelBase);

        QMetaObject::connectSlotsByName(KPlato__TaskGeneralPanelBase);
    } // setupUi

    void retranslateUi(QWidget *KPlato__TaskGeneralPanelBase)
    {
#if QT_CONFIG(tooltip)
        wbslabel->setToolTip(tr2i18n("Work Breakdown Structure", nullptr));
#endif // QT_CONFIG(tooltip)
        wbslabel->setText(tr2i18n("WBS:", nullptr));
        wbsfield->setText(QString());
#if QT_CONFIG(whatsthis)
        namelabel->setWhatsThis(tr2i18n("The name of the Task.", nullptr));
#endif // QT_CONFIG(whatsthis)
        namelabel->setText(tr2i18n("N&ame:", nullptr));
#if QT_CONFIG(whatsthis)
        namefield->setWhatsThis(tr2i18n("The name of the Task.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        leaderlabel->setWhatsThis(tr2i18n("The person responsible for this task.\n"
"        ", nullptr));
#endif // QT_CONFIG(whatsthis)
        leaderlabel->setText(tr2i18n("Responsi&ble:", nullptr));
#if QT_CONFIG(tooltip)
        chooseLeader->setToolTip(tr2i18n("Insert a person from your address book", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        chooseLeader->setWhatsThis(tr2i18n("Insert a person from your address book.", nullptr));
#endif // QT_CONFIG(whatsthis)
        chooseLeader->setText(tr2i18n("Choose...", nullptr));
#if QT_CONFIG(shortcut)
        chooseLeader->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        label->setText(tr2i18n("Priority:", nullptr));
#if QT_CONFIG(whatsthis)
        schedulingGroup->setWhatsThis(tr2i18n("Scheduling Configuration. These settings affect the actual scheduling of the task.\n"
"\n"
"The estimation can be either effort based or duration based. If it is effort based, the final duration will depend on the resources assigned to the task. For duration based estimation, the assigned resources do not affect the fixed duration of the task, but only the costs.", nullptr));
#endif // QT_CONFIG(whatsthis)
        schedulingGroup->setTitle(tr2i18n("Timing", nullptr));
        textLabel3->setText(tr2i18n("&Schedule:", nullptr));
        scheduleType->setItemText(0, tr2i18n("As Soon as Possible", nullptr));
        scheduleType->setItemText(1, tr2i18n("As Late as Possible", nullptr));
        scheduleType->setItemText(2, tr2i18n("Must Start On", nullptr));
        scheduleType->setItemText(3, tr2i18n("Must Finish On", nullptr));
        scheduleType->setItemText(4, tr2i18n("Start Not Earlier Than", nullptr));
        scheduleType->setItemText(5, tr2i18n("Finish Not Later Than", nullptr));
        scheduleType->setItemText(6, tr2i18n("Fixed Interval", nullptr));

        textLabel3_2->setText(tr2i18n("Esti&mate:", nullptr));
        pessimisticValue->setSuffix(tr2i18n("%", nullptr));
#if QT_CONFIG(tooltip)
        textLabel3_2_3->setToolTip(tr2i18n("Risk controls the PERT distribution used when calculating the actual estimate for this task.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        textLabel3_2_3->setWhatsThis(tr2i18n("<p>Risk controls the PERT distribution used when calculating the actual estimate for the task.\n"
"<b>None</b> means the Expected estimate is used as is.\n"
"<b>Low risk</b> means that a normal distribution is used.\n"
"<b>High risk</b> means that the estimate will be slightly pessimistic compared to Low risk.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel3_2_3->setText(tr2i18n("Risk:", nullptr));
        risk->setItemText(0, tr2i18n("None", nullptr));
        risk->setItemText(1, tr2i18n("Low", nullptr));
        risk->setItemText(2, tr2i18n("High", nullptr));

#if QT_CONFIG(tooltip)
        risk->setToolTip(tr2i18n("Risk controls the PERT distribution used when calculating the actual estimate for this task.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        risk->setWhatsThis(tr2i18n("<p>Risk controls the PERT distribution used when calculating the actual estimate for the task.\n"
"<b>None</b> means the Expected estimate is used as is.\n"
"<b>Low risk</b> means that a normal distribution is used.\n"
"<b>High risk</b> means that the estimate will be slightly pessimistic compared to Low risk.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1_2->setText(tr2i18n("Optimisti&c:", nullptr));
        optimisticValue->setSuffix(tr2i18n("%", nullptr));
        textLabel2_2->setText(tr2i18n("&Pessimistic:", nullptr));
        estimateType->setItemText(0, tr2i18n("Effort", nullptr));
        estimateType->setItemText(1, tr2i18n("Duration", nullptr));
        estimateType->setItemText(2, tr2i18n("Milestone", nullptr));

        (void)KPlato__TaskGeneralPanelBase;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class TaskGeneralPanelBase: public Ui_TaskGeneralPanelBase {};
} // namespace Ui
} // namespace KPlato

#endif // KPTTASKGENERALPANELBASE_H

