/****************************************************************************
** Meta object code from reading C++ file 'kptview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../calligraplan-4.0.1/src/plan/kptview.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato4ViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::View::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato4ViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::View",
        "currentScheduleManagerChanged",
        "",
        "KPlato::ScheduleManager*",
        "sm",
        "taskModulesChanged",
        "modules",
        "workPackagesAvailable",
        "slotUpdate",
        "slotCreateNewProject",
        "slotEditCut",
        "slotEditCopy",
        "slotEditPaste",
        "slotRefreshView",
        "slotViewSelector",
        "show",
        "slotProjectEdit",
        "slotDefineWBS",
        "slotCurrencyConfig",
        "slotCreateView",
        "slotRenameNode",
        "KPlato::Node*",
        "node",
        "name",
        "slotPopupMenuRequested",
        "menuname",
        "QPoint",
        "pos",
        "slotPopupMenu",
        "KPlato::ViewListItem*",
        "item",
        "addViewListItem",
        "const KPlato::ViewListItem*",
        "parent",
        "index",
        "removeViewListItem",
        "slotOpenReportFile",
        "slotSelectionChanged",
        "slotUpdateViewInfo",
        "itm",
        "loadWorkPackage",
        "KPlato::Project*",
        "project",
        "QList<QUrl>",
        "urls",
        "slotPublishWorkpackages",
        "QList<KPlato::Node*>",
        "nodes",
        "KPlato::Resource*",
        "resource",
        "mailTo",
        "slotCreateTemplate",
        "slotGuiActivated",
        "KPlato::ViewBase*",
        "view",
        "slotViewActivated",
        "slotPlugScheduleActions",
        "slotViewSchedule",
        "QAction*",
        "act",
        "slotScheduleAdded",
        "const KPlato::ScheduleManager*",
        "slotScheduleRemoved",
        "slotScheduleSwapped",
        "from",
        "to",
        "slotScheduleCalculated",
        "manager",
        "slotProjectWorktime",
        "slotOpenCurrentNode",
        "slotOpenNode",
        "slotOpenProjectDescription",
        "slotConnectNode",
        "slotCurrentChanged",
        "slotSelectDefaultView",
        "slotInsertResourcesFile",
        "file",
        "slotInsertFile",
        "slotUpdateSharedResources",
        "slotWorkPackageLoaded",
        "createReportView",
        "QDomDocument",
        "doc",
        "saveTaskModule",
        "QUrl",
        "url",
        "removeTaskModule",
        "slotCreateReportTemplate",
        "slotActionDestroyed",
        "o",
        "slotViewListItemRemoved",
        "slotViewListItemInserted",
        "slotProjectEditFinished",
        "result",
        "slotProjectDescriptionFinished",
        "slotProjectWorktimeFinished",
        "slotDefineWBSFinished",
        "slotCurrencyConfigFinished",
        "slotInsertFileFinished",
        "slotReportDesignFinished",
        "slotOpenReportFileFinished",
        "slotCreateViewFinished",
        "openWorkPackageMergeDialog",
        "workPackageMergeDialogFinished",
        "slotRemoveCommands",
        "initiateViews",
        "slotViewScheduleManager"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentScheduleManagerChanged'
        QtMocHelpers::SignalData<void(KPlato::ScheduleManager *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'taskModulesChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 6 },
        }}),
        // Signal 'workPackagesAvailable'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotUpdate'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCreateNewProject'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditCut'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditCopy'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditPaste'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRefreshView'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotViewSelector'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'slotProjectEdit'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDefineWBS'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCurrencyConfig'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCreateView'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRenameNode'
        QtMocHelpers::SlotData<void(KPlato::Node *, const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { QMetaType::QString, 23 },
        }}),
        // Slot 'slotPopupMenuRequested'
        QtMocHelpers::SlotData<void(const QString &, const QPoint &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 }, { 0x80000000 | 26, 27 },
        }}),
        // Slot 'slotPopupMenu'
        QtMocHelpers::SlotData<void(const QString &, const QPoint &, KPlato::ViewListItem *)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 }, { 0x80000000 | 26, 27 }, { 0x80000000 | 29, 30 },
        }}),
        // Slot 'addViewListItem'
        QtMocHelpers::SlotData<void(const KPlato::ViewListItem *, const KPlato::ViewListItem *, int)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 30 }, { 0x80000000 | 32, 33 }, { QMetaType::Int, 34 },
        }}),
        // Slot 'removeViewListItem'
        QtMocHelpers::SlotData<void(const KPlato::ViewListItem *)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 30 },
        }}),
        // Slot 'slotOpenReportFile'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotUpdateViewInfo'
        QtMocHelpers::SlotData<void(KPlato::ViewListItem *)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 39 },
        }}),
        // Slot 'loadWorkPackage'
        QtMocHelpers::SlotData<void(KPlato::Project *, const QList<QUrl> &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 42 }, { 0x80000000 | 43, 44 },
        }}),
        // Slot 'slotPublishWorkpackages'
        QtMocHelpers::SlotData<void(const QList<KPlato::Node*> &, KPlato::Resource *, bool)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 46, 47 }, { 0x80000000 | 48, 49 }, { QMetaType::Bool, 50 },
        }}),
        // Slot 'slotCreateTemplate'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotGuiActivated'
        QtMocHelpers::SlotData<void(KPlato::ViewBase *, bool)>(52, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 53, 54 }, { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotViewActivated'
        QtMocHelpers::SlotData<void(KPlato::ViewListItem *, KPlato::ViewListItem *)>(55, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 29, 2 }, { 0x80000000 | 29, 2 },
        }}),
        // Slot 'slotPlugScheduleActions'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotViewSchedule'
        QtMocHelpers::SlotData<void(QAction *)>(57, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 58, 59 },
        }}),
        // Slot 'slotScheduleAdded'
        QtMocHelpers::SlotData<void(const KPlato::ScheduleManager *)>(60, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 61, 2 },
        }}),
        // Slot 'slotScheduleRemoved'
        QtMocHelpers::SlotData<void(const KPlato::ScheduleManager *)>(62, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 61, 2 },
        }}),
        // Slot 'slotScheduleSwapped'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *, KPlato::ScheduleManager *)>(63, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 64 }, { 0x80000000 | 3, 65 },
        }}),
        // Slot 'slotScheduleCalculated'
        QtMocHelpers::SlotData<void(KPlato::Project *, KPlato::ScheduleManager *)>(66, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 41, 42 }, { 0x80000000 | 3, 67 },
        }}),
        // Slot 'slotProjectWorktime'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotOpenCurrentNode'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotOpenNode'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(70, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'slotOpenProjectDescription'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotConnectNode'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCurrentChanged'
        QtMocHelpers::SlotData<void(int)>(73, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotSelectDefaultView'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotInsertResourcesFile'
        QtMocHelpers::SlotData<void(const QString &)>(75, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 76 },
        }}),
        // Slot 'slotInsertFile'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotUpdateSharedResources'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotWorkPackageLoaded'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'createReportView'
        QtMocHelpers::SlotData<void(const QDomDocument &)>(80, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 81, 82 },
        }}),
        // Slot 'saveTaskModule'
        QtMocHelpers::SlotData<void(const QUrl &, KPlato::Project *)>(83, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 84, 85 }, { 0x80000000 | 41, 42 },
        }}),
        // Slot 'removeTaskModule'
        QtMocHelpers::SlotData<void(const QUrl &)>(86, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 84, 85 },
        }}),
        // Slot 'slotCreateReportTemplate'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotActionDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(88, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 89 },
        }}),
        // Slot 'slotViewListItemRemoved'
        QtMocHelpers::SlotData<void(KPlato::ViewListItem *)>(90, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'slotViewListItemInserted'
        QtMocHelpers::SlotData<void(KPlato::ViewListItem *, KPlato::ViewListItem *, int)>(91, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 29, 30 }, { 0x80000000 | 29, 33 }, { QMetaType::Int, 34 },
        }}),
        // Slot 'slotProjectEditFinished'
        QtMocHelpers::SlotData<void(int)>(92, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 93 },
        }}),
        // Slot 'slotProjectDescriptionFinished'
        QtMocHelpers::SlotData<void(int)>(94, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 93 },
        }}),
        // Slot 'slotProjectWorktimeFinished'
        QtMocHelpers::SlotData<void(int)>(95, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 93 },
        }}),
        // Slot 'slotDefineWBSFinished'
        QtMocHelpers::SlotData<void(int)>(96, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 93 },
        }}),
        // Slot 'slotCurrencyConfigFinished'
        QtMocHelpers::SlotData<void(int)>(97, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 93 },
        }}),
        // Slot 'slotInsertFileFinished'
        QtMocHelpers::SlotData<void(int)>(98, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 93 },
        }}),
        // Slot 'slotReportDesignFinished'
        QtMocHelpers::SlotData<void(int)>(99, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 93 },
        }}),
        // Slot 'slotOpenReportFileFinished'
        QtMocHelpers::SlotData<void(int)>(100, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 93 },
        }}),
        // Slot 'slotCreateViewFinished'
        QtMocHelpers::SlotData<void(int)>(101, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 93 },
        }}),
        // Slot 'openWorkPackageMergeDialog'
        QtMocHelpers::SlotData<void()>(102, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'workPackageMergeDialogFinished'
        QtMocHelpers::SlotData<void(int)>(103, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 93 },
        }}),
        // Slot 'slotRemoveCommands'
        QtMocHelpers::SlotData<void()>(104, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'initiateViews'
        QtMocHelpers::SlotData<void()>(105, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotViewScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(106, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<View, qt_meta_tag_ZN6KPlato4ViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::View::staticMetaObject = { {
    QMetaObject::SuperData::link<KoView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato4ViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato4ViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato4ViewE_t>.metaTypes,
    nullptr
} };

void KPlato::View::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<View *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentScheduleManagerChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 1: _t->taskModulesChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 2: _t->workPackagesAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->slotUpdate(); break;
        case 4: _t->slotCreateNewProject(); break;
        case 5: _t->slotEditCut(); break;
        case 6: _t->slotEditCopy(); break;
        case 7: _t->slotEditPaste(); break;
        case 8: _t->slotRefreshView(); break;
        case 9: _t->slotViewSelector((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->slotProjectEdit(); break;
        case 11: _t->slotDefineWBS(); break;
        case 12: _t->slotCurrencyConfig(); break;
        case 13: _t->slotCreateView(); break;
        case 14: _t->slotRenameNode((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 15: _t->slotPopupMenuRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 16: _t->slotPopupMenu((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[3]))); break;
        case 17: _t->addViewListItem((*reinterpret_cast<std::add_pointer_t<const KPlato::ViewListItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<const KPlato::ViewListItem*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 18: _t->removeViewListItem((*reinterpret_cast<std::add_pointer_t<const KPlato::ViewListItem*>>(_a[1]))); break;
        case 19: _t->slotOpenReportFile(); break;
        case 20: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 21: _t->slotUpdateViewInfo((*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[1]))); break;
        case 22: _t->loadWorkPackage((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[2]))); break;
        case 23: _t->slotPublishWorkpackages((*reinterpret_cast<std::add_pointer_t<QList<KPlato::Node*>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 24: _t->slotCreateTemplate(); break;
        case 25: _t->slotGuiActivated((*reinterpret_cast<std::add_pointer_t<KPlato::ViewBase*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 26: _t->slotViewActivated((*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[2]))); break;
        case 27: _t->slotPlugScheduleActions(); break;
        case 28: _t->slotViewSchedule((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 29: _t->slotScheduleAdded((*reinterpret_cast<std::add_pointer_t<const KPlato::ScheduleManager*>>(_a[1]))); break;
        case 30: _t->slotScheduleRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::ScheduleManager*>>(_a[1]))); break;
        case 31: _t->slotScheduleSwapped((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[2]))); break;
        case 32: _t->slotScheduleCalculated((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[2]))); break;
        case 33: _t->slotProjectWorktime(); break;
        case 34: _t->slotOpenCurrentNode(); break;
        case 35: _t->slotOpenNode((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 36: _t->slotOpenProjectDescription(); break;
        case 37: _t->slotConnectNode(); break;
        case 38: _t->slotCurrentChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 39: _t->slotSelectDefaultView(); break;
        case 40: _t->slotInsertResourcesFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 41: _t->slotInsertFile(); break;
        case 42: _t->slotUpdateSharedResources(); break;
        case 43: _t->slotWorkPackageLoaded(); break;
        case 44: _t->createReportView((*reinterpret_cast<std::add_pointer_t<QDomDocument>>(_a[1]))); break;
        case 45: _t->saveTaskModule((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[2]))); break;
        case 46: _t->removeTaskModule((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 47: _t->slotCreateReportTemplate(); break;
        case 48: _t->slotActionDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 49: _t->slotViewListItemRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[1]))); break;
        case 50: _t->slotViewListItemInserted((*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ViewListItem*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 51: _t->slotProjectEditFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 52: _t->slotProjectDescriptionFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 53: _t->slotProjectWorktimeFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 54: _t->slotDefineWBSFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 55: _t->slotCurrencyConfigFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 56: _t->slotInsertFileFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 57: _t->slotReportDesignFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 58: _t->slotOpenReportFileFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 59: _t->slotCreateViewFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 60: _t->openWorkPackageMergeDialog(); break;
        case 61: _t->workPackageMergeDialogFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 62: _t->slotRemoveCommands(); break;
        case 63: _t->initiateViews(); break;
        case 64: _t->slotViewScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ViewBase* >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (View::*)(KPlato::ScheduleManager * )>(_a, &View::currentScheduleManagerChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (View::*)(const QStringList & )>(_a, &View::taskModulesChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (View::*)(bool )>(_a, &View::workPackagesAvailable, 2))
            return;
    }
}

const QMetaObject *KPlato::View::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::View::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato4ViewE_t>.strings))
        return static_cast<void*>(this);
    return KoView::qt_metacast(_clname);
}

int KPlato::View::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KoView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 65)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 65;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 65)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 65;
    }
    return _id;
}

// SIGNAL 0
void KPlato::View::currentScheduleManagerChanged(KPlato::ScheduleManager * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::View::taskModulesChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KPlato::View::workPackagesAvailable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
