/********************************************************************************
** Form generated from reading UI file 'CreateReportTemplateDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CREATEREPORTTEMPLATEDIALOG_H
#define UI_CREATEREPORTTEMPLATEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_CreateReportTemplateDialog
{
public:
    QFormLayout *formLayout_2;
    QDialogButtonBox *buttonBox;
    QWidget *mainWidget;
    QFormLayout *formLayout;
    QLabel *label;
    QComboBox *templates;
    QLabel *label_2;
    KUrlRequester *fileName;
    QCheckBox *openAssistant;
    QSpacerItem *verticalSpacer;

    void setupUi(QDialog *CreateReportTemplateDialog)
    {
        if (CreateReportTemplateDialog->objectName().isEmpty())
            CreateReportTemplateDialog->setObjectName("CreateReportTemplateDialog");
        CreateReportTemplateDialog->resize(435, 245);
        formLayout_2 = new QFormLayout(CreateReportTemplateDialog);
        formLayout_2->setObjectName("formLayout_2");
        buttonBox = new QDialogButtonBox(CreateReportTemplateDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, buttonBox);

        mainWidget = new QWidget(CreateReportTemplateDialog);
        mainWidget->setObjectName("mainWidget");
        formLayout = new QFormLayout(mainWidget);
        formLayout->setObjectName("formLayout");
        label = new QLabel(mainWidget);
        label->setObjectName("label");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label);

        templates = new QComboBox(mainWidget);
        templates->setObjectName("templates");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, templates);

        label_2 = new QLabel(mainWidget);
        label_2->setObjectName("label_2");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_2);

        fileName = new KUrlRequester(mainWidget);
        fileName->setObjectName("fileName");
        fileName->setMode(KFile::File|KFile::LocalOnly);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, fileName);

        openAssistant = new QCheckBox(mainWidget);
        openAssistant->setObjectName("openAssistant");
        openAssistant->setChecked(true);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, openAssistant);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        formLayout->setItem(3, QFormLayout::ItemRole::FieldRole, verticalSpacer);


        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, mainWidget);

#if QT_CONFIG(shortcut)
        label->setBuddy(templates);
        label_2->setBuddy(fileName);
#endif // QT_CONFIG(shortcut)

        retranslateUi(CreateReportTemplateDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, CreateReportTemplateDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, CreateReportTemplateDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(CreateReportTemplateDialog);
    } // setupUi

    void retranslateUi(QDialog *CreateReportTemplateDialog)
    {
        CreateReportTemplateDialog->setWindowTitle(tr2i18n("Dialog", nullptr));
#if QT_CONFIG(whatsthis)
        mainWidget->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
        label->setText(tr2i18n("Base template:", nullptr));
        label_2->setText(tr2i18n("File name:", nullptr));
        openAssistant->setText(tr2i18n("Open assistant", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CreateReportTemplateDialog: public Ui_CreateReportTemplateDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CREATEREPORTTEMPLATEDIALOG_H

