/****************************************************************************
** Meta object code from reading C++ file 'MilestoneGanttView.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/MilestoneGanttView.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MilestoneGanttView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato31MilestoneGanttChartOptionsPanelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::MilestoneGanttChartOptionsPanel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato31MilestoneGanttChartOptionsPanelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::MilestoneGanttChartOptionsPanel",
        "slotOk",
        "",
        "setDefault"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotOk'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setDefault'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MilestoneGanttChartOptionsPanel, qt_meta_tag_ZN6KPlato31MilestoneGanttChartOptionsPanelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::MilestoneGanttChartOptionsPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato31MilestoneGanttChartOptionsPanelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato31MilestoneGanttChartOptionsPanelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato31MilestoneGanttChartOptionsPanelE_t>.metaTypes,
    nullptr
} };

void KPlato::MilestoneGanttChartOptionsPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MilestoneGanttChartOptionsPanel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotOk(); break;
        case 1: _t->setDefault(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *KPlato::MilestoneGanttChartOptionsPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::MilestoneGanttChartOptionsPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato31MilestoneGanttChartOptionsPanelE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int KPlato::MilestoneGanttChartOptionsPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato32MilestoneGanttViewSettingsDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::MilestoneGanttViewSettingsDialog::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato32MilestoneGanttViewSettingsDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::MilestoneGanttViewSettingsDialog",
        "slotOk",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotOk'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MilestoneGanttViewSettingsDialog, qt_meta_tag_ZN6KPlato32MilestoneGanttViewSettingsDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::MilestoneGanttViewSettingsDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemViewSettupDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato32MilestoneGanttViewSettingsDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato32MilestoneGanttViewSettingsDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato32MilestoneGanttViewSettingsDialogE_t>.metaTypes,
    nullptr
} };

void KPlato::MilestoneGanttViewSettingsDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MilestoneGanttViewSettingsDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotOk(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *KPlato::MilestoneGanttViewSettingsDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::MilestoneGanttViewSettingsDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato32MilestoneGanttViewSettingsDialogE_t>.strings))
        return static_cast<void*>(this);
    return ItemViewSettupDialog::qt_metacast(_clname);
}

int KPlato::MilestoneGanttViewSettingsDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemViewSettupDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato19MilestoneKGanttViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::MilestoneKGanttView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato19MilestoneKGanttViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::MilestoneKGanttView",
        "slotProjectCalculated",
        "",
        "KPlato::ScheduleManager*",
        "sm"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotProjectCalculated'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MilestoneKGanttView, qt_meta_tag_ZN6KPlato19MilestoneKGanttViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::MilestoneKGanttView::staticMetaObject = { {
    QMetaObject::SuperData::link<NodeGanttViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19MilestoneKGanttViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19MilestoneKGanttViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato19MilestoneKGanttViewE_t>.metaTypes,
    nullptr
} };

void KPlato::MilestoneKGanttView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MilestoneKGanttView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotProjectCalculated((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KPlato::MilestoneKGanttView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::MilestoneKGanttView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato19MilestoneKGanttViewE_t>.strings))
        return static_cast<void*>(this);
    return NodeGanttViewBase::qt_metacast(_clname);
}

int KPlato::MilestoneKGanttView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = NodeGanttViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato18MilestoneGanttViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::MilestoneGanttView::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato18MilestoneGanttViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::MilestoneGanttView",
        "setScheduleManager",
        "",
        "KPlato::ScheduleManager*",
        "sm",
        "setShowTaskName",
        "on",
        "setShowProgress",
        "setShowPositiveFloat",
        "setShowCriticalTasks",
        "setShowNoInformation",
        "slotEditCopy",
        "slotContextMenuRequested",
        "QModelIndex",
        "QPoint",
        "pos",
        "slotGanttHeaderContextMenuRequested",
        "pt",
        "slotDateTimeGridChanged",
        "slotOptions",
        "ganttActions",
        "itemDoubleClicked",
        "QPersistentModelIndex",
        "idx",
        "slotOpenCurrentNode",
        "slotOpenNode",
        "KPlato::Node*",
        "node",
        "slotTaskProgress",
        "slotOpenProjectDescription",
        "slotTaskDescription",
        "slotOpenTaskDescription",
        "slotDocuments",
        "slotTaskEditFinished",
        "result",
        "slotSummaryTaskEditFinished",
        "slotTaskProgressFinished",
        "slotMilestoneProgressFinished",
        "slotTaskDescriptionFinished",
        "slotDocumentsFinished",
        "slotSelectionChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setScheduleManager'
        QtMocHelpers::SlotData<void(KPlato::ScheduleManager *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setShowTaskName'
        QtMocHelpers::SlotData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setShowProgress'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setShowPositiveFloat'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setShowCriticalTasks'
        QtMocHelpers::SlotData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setShowNoInformation'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'slotEditCopy'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotContextMenuRequested'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QPoint &)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 2 }, { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotGanttHeaderContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 17 },
        }}),
        // Slot 'slotDateTimeGridChanged'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotOptions'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'ganttActions'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'itemDoubleClicked'
        QtMocHelpers::SlotData<void(const QPersistentModelIndex &)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotOpenCurrentNode'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenNode'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'slotTaskProgress'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenProjectDescription'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTaskDescription'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenTaskDescription'
        QtMocHelpers::SlotData<void(bool)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotDocuments'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTaskEditFinished'
        QtMocHelpers::SlotData<void(int)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'slotSummaryTaskEditFinished'
        QtMocHelpers::SlotData<void(int)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'slotTaskProgressFinished'
        QtMocHelpers::SlotData<void(int)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'slotMilestoneProgressFinished'
        QtMocHelpers::SlotData<void(int)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'slotTaskDescriptionFinished'
        QtMocHelpers::SlotData<void(int)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'slotDocumentsFinished'
        QtMocHelpers::SlotData<void(int)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MilestoneGanttView, qt_meta_tag_ZN6KPlato18MilestoneGanttViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::MilestoneGanttView::staticMetaObject = { {
    QMetaObject::SuperData::link<ViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18MilestoneGanttViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18MilestoneGanttViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato18MilestoneGanttViewE_t>.metaTypes,
    nullptr
} };

void KPlato::MilestoneGanttView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MilestoneGanttView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setScheduleManager((*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[1]))); break;
        case 1: _t->setShowTaskName((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->setShowProgress((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->setShowPositiveFloat((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->setShowCriticalTasks((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->setShowNoInformation((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->slotEditCopy(); break;
        case 7: _t->slotContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 8: _t->slotGanttHeaderContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 9: _t->slotDateTimeGridChanged(); break;
        case 10: _t->slotOptions(); break;
        case 11: _t->ganttActions(); break;
        case 12: _t->itemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QPersistentModelIndex>>(_a[1]))); break;
        case 13: _t->slotOpenCurrentNode(); break;
        case 14: _t->slotOpenNode((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 15: _t->slotTaskProgress(); break;
        case 16: _t->slotOpenProjectDescription(); break;
        case 17: _t->slotTaskDescription(); break;
        case 18: _t->slotOpenTaskDescription((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->slotDocuments(); break;
        case 20: _t->slotTaskEditFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->slotSummaryTaskEditFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 22: _t->slotTaskProgressFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->slotMilestoneProgressFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->slotTaskDescriptionFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->slotDocumentsFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->slotSelectionChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KPlato::MilestoneGanttView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::MilestoneGanttView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato18MilestoneGanttViewE_t>.strings))
        return static_cast<void*>(this);
    return ViewBase::qt_metacast(_clname);
}

int KPlato::MilestoneGanttView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    return _id;
}
QT_WARNING_POP
