/****************************************************************************
** Meta object code from reading C++ file 'kptbuiltinschedulerplugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../calligraplan-4.0.1/src/plan/kptbuiltinschedulerplugin.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptbuiltinschedulerplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato22BuiltinSchedulerPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::BuiltinSchedulerPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato22BuiltinSchedulerPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::BuiltinSchedulerPlugin",
        "sigCalculationStarted",
        "",
        "KPlato::Project*",
        "KPlato::ScheduleManager*",
        "sigCalculationFinished",
        "slotStarted",
        "KPlato::SchedulerThread*",
        "job",
        "slotFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sigCalculationStarted'
        QtMocHelpers::SignalData<void(KPlato::Project *, KPlato::ScheduleManager *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 },
        }}),
        // Signal 'sigCalculationFinished'
        QtMocHelpers::SignalData<void(KPlato::Project *, KPlato::ScheduleManager *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 },
        }}),
        // Slot 'slotStarted'
        QtMocHelpers::SlotData<void(KPlato::SchedulerThread *)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotFinished'
        QtMocHelpers::SlotData<void(KPlato::SchedulerThread *)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BuiltinSchedulerPlugin, qt_meta_tag_ZN6KPlato22BuiltinSchedulerPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::BuiltinSchedulerPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<SchedulerPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22BuiltinSchedulerPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22BuiltinSchedulerPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato22BuiltinSchedulerPluginE_t>.metaTypes,
    nullptr
} };

void KPlato::BuiltinSchedulerPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BuiltinSchedulerPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigCalculationStarted((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[2]))); break;
        case 1: _t->sigCalculationFinished((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::ScheduleManager*>>(_a[2]))); break;
        case 2: _t->slotStarted((*reinterpret_cast<std::add_pointer_t<KPlato::SchedulerThread*>>(_a[1]))); break;
        case 3: _t->slotFinished((*reinterpret_cast<std::add_pointer_t<KPlato::SchedulerThread*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::ScheduleManager* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::SchedulerThread* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::SchedulerThread* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BuiltinSchedulerPlugin::*)(KPlato::Project * , KPlato::ScheduleManager * )>(_a, &BuiltinSchedulerPlugin::sigCalculationStarted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BuiltinSchedulerPlugin::*)(KPlato::Project * , KPlato::ScheduleManager * )>(_a, &BuiltinSchedulerPlugin::sigCalculationFinished, 1))
            return;
    }
}

const QMetaObject *KPlato::BuiltinSchedulerPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::BuiltinSchedulerPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22BuiltinSchedulerPluginE_t>.strings))
        return static_cast<void*>(this);
    return SchedulerPlugin::qt_metacast(_clname);
}

int KPlato::BuiltinSchedulerPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SchedulerPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void KPlato::BuiltinSchedulerPlugin::sigCalculationStarted(KPlato::Project * _t1, KPlato::ScheduleManager * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KPlato::BuiltinSchedulerPlugin::sigCalculationFinished(KPlato::Project * _t1, KPlato::ScheduleManager * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN6KPlato15KPlatoSchedulerE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::KPlatoScheduler::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato15KPlatoSchedulerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::KPlatoScheduler",
        "stopScheduling",
        "",
        "haltScheduling",
        "cancelScheduling",
        "SchedulingContext&",
        "context",
        "slotProgress",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'stopScheduling'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'haltScheduling'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cancelScheduling'
        QtMocHelpers::SlotData<void(SchedulingContext &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotProgress'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KPlatoScheduler, qt_meta_tag_ZN6KPlato15KPlatoSchedulerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::KPlatoScheduler::staticMetaObject = { {
    QMetaObject::SuperData::link<SchedulerThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15KPlatoSchedulerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15KPlatoSchedulerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato15KPlatoSchedulerE_t>.metaTypes,
    nullptr
} };

void KPlato::KPlatoScheduler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KPlatoScheduler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stopScheduling(); break;
        case 1: _t->haltScheduling(); break;
        case 2: _t->cancelScheduling((*reinterpret_cast<std::add_pointer_t<SchedulingContext&>>(_a[1]))); break;
        case 3: _t->slotProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::KPlatoScheduler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::KPlatoScheduler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato15KPlatoSchedulerE_t>.strings))
        return static_cast<void*>(this);
    return SchedulerThread::qt_metacast(_clname);
}

int KPlato::KPlatoScheduler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SchedulerThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
QT_WARNING_POP
