/********************************************************************************
** Form generated from reading UI file 'kptscheduleeditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTSCHEDULEEDITOR_H
#define UI_KPTSCHEDULEEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_SchedulingRange
{
public:
    QHBoxLayout *horizontalLayout_2;
    QFrame *frame;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QDateTimeEdit *targetStartTime;
    QDateTimeEdit *targetEndTime;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *KPlato__SchedulingRange)
    {
        if (KPlato__SchedulingRange->objectName().isEmpty())
            KPlato__SchedulingRange->setObjectName("KPlato__SchedulingRange");
        KPlato__SchedulingRange->resize(495, 48);
        horizontalLayout_2 = new QHBoxLayout(KPlato__SchedulingRange);
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        frame = new QFrame(KPlato__SchedulingRange);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Sunken);
        horizontalLayout = new QHBoxLayout(frame);
        horizontalLayout->setSpacing(3);
        horizontalLayout->setContentsMargins(3, 3, 3, 3);
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(frame);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        targetStartTime = new QDateTimeEdit(frame);
        targetStartTime->setObjectName("targetStartTime");
        targetStartTime->setCalendarPopup(true);

        horizontalLayout->addWidget(targetStartTime);

        targetEndTime = new QDateTimeEdit(frame);
        targetEndTime->setObjectName("targetEndTime");
        targetEndTime->setCalendarPopup(true);

        horizontalLayout->addWidget(targetEndTime);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        horizontalLayout_2->addWidget(frame);

#if QT_CONFIG(shortcut)
        label->setBuddy(targetStartTime);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__SchedulingRange);

        QMetaObject::connectSlotsByName(KPlato__SchedulingRange);
    } // setupUi

    void retranslateUi(QWidget *KPlato__SchedulingRange)
    {
        label->setText(tr2i18n("Scheduling range:", nullptr));
        (void)KPlato__SchedulingRange;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class SchedulingRange: public Ui_SchedulingRange {};
} // namespace Ui
} // namespace KPlato

#endif // KPTSCHEDULEEDITOR_H

