/********************************************************************************
** Form generated from reading UI file 'ResourceAllocationPanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RESOURCEALLOCATIONPANEL_H
#define UI_RESOURCEALLOCATIONPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_ResourceAllocationPanel
{
public:
    QHBoxLayout *horizontalLayout_3;
    QTreeView *resourcesView;
    QTabWidget *tabWidget;
    QWidget *tab;
    QHBoxLayout *horizontalLayout_2;
    QTreeView *alternativsView;
    QWidget *tab_2;
    QHBoxLayout *horizontalLayout;
    QTreeView *requiredView;

    void setupUi(QWidget *KPlato__ResourceAllocationPanel)
    {
        if (KPlato__ResourceAllocationPanel->objectName().isEmpty())
            KPlato__ResourceAllocationPanel->setObjectName("KPlato__ResourceAllocationPanel");
        KPlato__ResourceAllocationPanel->resize(909, 400);
        KPlato__ResourceAllocationPanel->setMinimumSize(QSize(600, 300));
        horizontalLayout_3 = new QHBoxLayout(KPlato__ResourceAllocationPanel);
        horizontalLayout_3->setSpacing(3);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        resourcesView = new QTreeView(KPlato__ResourceAllocationPanel);
        resourcesView->setObjectName("resourcesView");
        resourcesView->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContentsOnFirstShow);
        resourcesView->setRootIsDecorated(false);
        resourcesView->setItemsExpandable(false);
        resourcesView->setSortingEnabled(true);
        resourcesView->setExpandsOnDoubleClick(false);

        horizontalLayout_3->addWidget(resourcesView);

        tabWidget = new QTabWidget(KPlato__ResourceAllocationPanel);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        horizontalLayout_2 = new QHBoxLayout(tab);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        alternativsView = new QTreeView(tab);
        alternativsView->setObjectName("alternativsView");
        alternativsView->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContentsOnFirstShow);
        alternativsView->setRootIsDecorated(false);
        alternativsView->setItemsExpandable(false);
        alternativsView->setSortingEnabled(true);
        alternativsView->setExpandsOnDoubleClick(false);

        horizontalLayout_2->addWidget(alternativsView);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        horizontalLayout = new QHBoxLayout(tab_2);
        horizontalLayout->setObjectName("horizontalLayout");
        requiredView = new QTreeView(tab_2);
        requiredView->setObjectName("requiredView");
        requiredView->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContentsOnFirstShow);
        requiredView->setRootIsDecorated(false);
        requiredView->setItemsExpandable(false);
        requiredView->setExpandsOnDoubleClick(false);

        horizontalLayout->addWidget(requiredView);

        tabWidget->addTab(tab_2, QString());

        horizontalLayout_3->addWidget(tabWidget);


        retranslateUi(KPlato__ResourceAllocationPanel);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KPlato__ResourceAllocationPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__ResourceAllocationPanel)
    {
        KPlato__ResourceAllocationPanel->setWindowTitle(QString());
        tabWidget->setTabText(tabWidget->indexOf(tab), tr2i18n("Alternatives", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), tr2i18n("Required", nullptr));
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class ResourceAllocationPanel: public Ui_ResourceAllocationPanel {};
} // namespace Ui
} // namespace KPlato

#endif // RESOURCEALLOCATIONPANEL_H

