#ifndef WRITEODF_H
#define WRITEODF_H
#include "writeodf/odfwriter.h"
namespace writeodf {
class group_change_marks;
class group_chart_data_label;
class group_chart_title;
class group_common_anim_add_accum_attlist;
class group_common_anim_named_target_attlist;
class group_common_anim_spline_mode_attlist;
class group_common_background_color_attlist;
class group_common_border_attlist;
class group_common_break_attlist;
class group_common_contour_attlist;
class group_common_current_value_attlist;
class group_common_data_style_attlist;
class group_common_db_object_description;
class group_common_db_object_title;
class group_common_db_table_style_name;
class group_common_decimal_places_attlist;
class group_common_disabled_attlist;
class group_common_draw_area_attlist;
class group_common_draw_circle_ellipse_attlist;
class group_common_draw_data_attlist;
class group_common_draw_name_attlist;
class group_common_draw_points_attlist;
class group_common_draw_style_name_attlist;
class group_common_draw_z_index_attlist;
class group_common_endsync_timing_attlist;
class group_common_field_display_value_formula_attlist;
class group_common_field_fixed_attlist;
class group_common_field_name_attlist;
class group_common_form_control_attlist;
class group_common_form_relative_image_position_attlist;
class group_common_format_source_attlist;
class group_common_in_content_meta_attlist;
class group_common_linked_cell;
class group_common_maxlength_attlist;
class group_common_num_format_prefix_suffix_attlist;
class group_common_padding_attlist;
class group_common_presentation_effect_attlist;
class group_common_printable_attlist;
class group_common_rotation_angle_attlist;
class group_common_shadow_attlist;
class group_common_spin_button;
class group_common_stock_marker_attlist;
class group_common_style_header_footer_attlist;
class group_common_svg_gradient_attlist;
class group_common_table_cell_address_attlist;
class group_common_table_range_attlist;
class group_common_text_align;
class group_common_title_attlist;
class group_common_value_attlist;
class group_common_vertical_margin_attlist;
class group_common_vertical_rel_attlist;
class group_config_config_item_map_entry;
class group_db_character_set;
class group_db_command;
class group_db_component_collection;
class group_db_filter_statement;
class group_db_order_statement;
class group_db_query_collection;
class group_db_table_filter_pattern;
class group_dc_date;
class group_dr3d_scene;
class group_draw_caption_attlist;
class group_draw_glue_point;
class group_draw_page;
class group_for;
class group_form_properties;
class group_label;
class group_list_source_type;
class group_number_am_pm;
class group_number_hours;
class group_number_number;
class group_number_text;
class group_office_automatic_styles;
class group_office_body;
class group_office_dde_source;
class group_office_document_common_attrs;
class group_office_font_face_decls;
class group_office_master_styles;
class group_office_scripts;
class group_office_styles;
class group_presentation_animation_elements;
class group_presentation_notes;
class group_presentation_sound;
class group_selected;
class group_style_background_image;
class group_style_graphic_fill_properties_attlist;
class group_style_list_level_properties;
class group_style_page_layout_content;
class group_style_text_properties;
class group_svg_stop;
class group_table_cell_address;
class group_table_database_source_sql;
class group_table_decls;
class group_table_dependencies;
class group_table_filter_and;
class group_table_filter_or;
class group_table_linked_source_attlist;
class group_table_table_column;
class group_target_location;
class group_text_alphabetical_index;
class group_text_bibliography;
class group_text_bookmark_ref_content;
class group_text_h;
class group_text_illustration_index;
class group_text_index_entry_chapter;
class group_text_index_entry_span;
class group_text_index_entry_text;
class group_text_index_scope_attr;
class group_text_index_title;
class group_text_list;
class group_text_list_level_style_attr;
class group_text_note_class;
class group_text_number;
class group_text_outline_level;
class group_text_relative_tab_stop_position_attr;
class group_text_soft_page_break;
class group_text_table_of_content;
class group_text_user_index;
class anim_param;
class chart_axis;
class chart_floor;
class chart_grid;
class chart_stock_loss_marker;
class config_config_item;
class config_config_item_map_indexed;
class db_auto_increment;
class db_column_definitions;
class db_connection_data;
class db_data_source;
class db_database_description;
class db_index;
class db_index_columns;
class db_key;
class db_key_columns;
class db_login;
class db_schema_definition;
class db_table_exclude_filter;
class dc_date;
class dc_language;
class draw_enhanced_geometry;
class draw_fill_image;
class draw_glue_point;
class draw_handle;
class draw_param;
class form_list_value_1;
class form_list_value_3;
class form_list_value_5;
class form_properties;
class meta_auto_reload;
class meta_print_date;
class meta_template;
class number_am_pm;
class number_currency_symbol;
class number_embedded_text;
class number_minutes;
class number_text;
class office_body;
class office_document_content;
class office_font_face_decls;
class office_script;
class presentation_animation_group;
class presentation_date_time;
class presentation_dim;
class presentation_footer;
class presentation_header;
class presentation_hide_shape;
class presentation_play;
class presentation_show;
class presentation_show_text;
class script_event_listener;
class style_column_sep;
class style_default_page_layout;
class style_drawing_page_properties;
class style_font_face;
class style_footnote_sep;
class style_presentation_page_layout;
class style_ruby_properties;
class style_tab_stops;
class style_table_column_properties;
class style_table_row_properties;
class svg_font_face_format;
class svg_font_face_src;
class svg_linearGradient;
class svg_stop;
class table_background;
class table_calculation_settings;
class table_cell_range_source;
class table_data_pilot_display_info;
class table_data_pilot_field_reference;
class table_data_pilot_group_member;
class table_data_pilot_layout_info;
class table_data_pilot_member;
class table_data_pilot_sort_info;
class table_data_pilot_subtotals;
class table_data_pilot_tables;
class table_database_ranges;
class table_database_source_sql;
class table_dependencies;
class table_desc;
class table_error_macro;
class table_filter_condition;
class table_filter_set_item;
class table_highlighted_range;
class table_insertion_cut_off;
class table_label_range;
class table_named_expression;
class table_named_range;
class table_operation;
class table_scenario;
class table_sort;
class table_sort_groups;
class table_source_range_address;
class table_subtotal_field;
class table_subtotal_rules;
class table_table_header_columns;
class table_table_template;
class table_title;
class text_alphabetical_index_auto_mark_file;
class text_author_initials;
class text_bookmark_end;
class text_creator;
class text_editing_cycles;
class text_file_name;
class text_hidden_text;
class text_index_entry_bibliography;
class text_index_entry_link_end;
class text_index_entry_page_number;
class text_index_entry_tab_stop;
class text_index_source_style;
class text_initial_creator;
class text_keywords;
class text_list_style;
class text_note;
class text_note_citation;
class text_note_continuation_notice_forward;
class text_outline_style;
class text_page;
class text_page_sequence;
class text_page_variable_set;
class text_reference_mark;
class text_reference_mark_start;
class text_ruby;
class text_ruby_text;
class text_script;
class text_section_source;
class text_sender_company;
class text_sender_email;
class text_sender_firstname;
class text_sender_lastname;
class text_sender_phone_work;
class text_sender_postal_code;
class text_sender_street;
class text_sequence_decls;
class text_sheet_name;
class text_sort_key;
class text_subject;
class text_template_name;
class text_toc_mark;
class text_user_field_decls;
class text_variable_decls;
class group_common_anim_target_attlist;
class group_common_auto_reorder_attlist;
class group_common_border_line_width_attlist;
class group_common_convert_empty_attlist;
class group_common_db_table_name_attlist;
class group_common_delay_for_repeat;
class group_common_draw_caption_id_attlist;
class group_common_draw_gradient_attlist;
class group_common_draw_layer_name_attlist;
class group_common_draw_position_attlist;
class group_common_draw_size_attlist;
class group_common_editable_attlist;
class group_common_field_display_value_none_attlist;
class group_common_form_visual_effect_attlist;
class group_common_keep_with_next_attlist;
class group_common_num_format_attlist;
class group_common_page_number_attlist;
class group_common_readonly_attlist;
class group_common_spline_anim_value_attlist;
class group_common_svg_font_face_xlink_attlist;
class group_common_table_change_attlist;
class group_common_text_anchor_attlist;
class group_common_value_type_attlist;
class group_common_writing_mode_attlist;
class group_current_selected;
class group_db_apply_command;
class group_db_component;
class group_db_is_first_row_header_line;
class group_db_show_deleted;
class group_dr3d_scene_attlist;
class group_draw_layer_set;
class group_form_property_name;
class group_image_data;
class group_list_source;
class group_number_and_text;
class group_number_seconds;
class group_office_binary_data;
class group_office_document;
class group_office_forms;
class group_office_settings;
class group_size;
class group_style_columns;
class group_style_map;
class group_svg_title;
class group_table_columns_and_groups;
class group_table_database_source_table;
class group_table_filter;
class group_table_table_row;
class group_text_id;
class group_text_illustration_index_source_attrs;
class group_text_index_entry_page_number;
class group_text_index_name;
class group_text_index_title_template;
class group_text_list_style;
class group_text_object_index;
class group_text_section;
class group_text_tracked_changes;
class chart_categories;
class chart_domain;
class chart_error_indicator;
class chart_mean_value;
class chart_regression_curve;
class chart_stock_gain_marker;
class db_character_set;
class db_connection_resource;
class db_delimiter;
class db_file_based_database;
class db_forms;
class db_indices;
class db_keys;
class db_server_database;
class db_table_definitions;
class db_update_table;
class draw_equation;
class draw_gradient;
class draw_stroke_dash;
class form_connection_resource;
class form_item;
class form_list_value;
class form_list_value_4;
class form_option;
class math_math;
class meta_editing_cycles;
class number_boolean;
class number_currency_style;
class number_hours;
class number_percentage_style;
class number_text_content;
class office_binary_data;
class office_document_styles;
class office_event_listeners;
class presentation_date_time_decl;
class presentation_footer_decl;
class presentation_hide_text;
class presentation_placeholder;
class presentation_show_shape;
class style_background_image;
class style_column;
class style_text_properties;
class svg_desc;
class svg_font_face_uri;
class svg_title;
class table_cell_address;
class table_data_pilot_field;
class table_data_pilot_groups;
class table_data_pilot_members;
class table_database_source_query;
class table_dependency;
class table_iteration;
class table_named_expressions;
class table_sort_by;
class table_source_service;
class table_table_columns;
class table_table_rows;
class table_target_range_address;
class text_alphabetical_index_mark_end;
class text_alphabetical_index_source;
class text_bibliography_source;
class text_chapter;
class text_conditional_text;
class text_illustration_index_source;
class text_index_entry_link_start;
class text_index_entry_text;
class text_linenumbering_configuration;
class text_list_level_style_image;
class text_measure;
class text_note_continuation_notice_backward;
class text_notes_configuration;
class text_object_index_source;
class text_reference_mark_end;
class text_sender_country;
class text_sender_initials;
class text_sender_position;
class text_sender_title;
class text_sequence_decl;
class text_soft_page_break;
class text_tab;
class text_toc_mark_end;
class text_tracked_changes;
class text_user_index_mark_end;
class xforms_model;
class group_animation_element;
class group_change_mark_attr;
class group_chart_title_attlist;
class group_common_calendar_attlist;
class group_common_dde_connection_decl_attlist;
class group_common_draw_circle_ellipse_pos_attlist;
class group_common_draw_path_data_attlist;
class group_common_draw_viewbox_attlist;
class group_common_field_formula_attlist;
class group_common_margin_attlist;
class group_common_presentation_header_footer_attlist;
class group_common_style_direction_attlist;
class group_common_table_template_attlist;
class group_common_vertical_pos_attlist;
class group_db_delimiter;
class group_dc_creator;
class group_draw_frame;
class group_office_annotation;
class group_office_event_listeners;
class group_presentation_shape_attlist;
class group_shape_instance;
class group_style_header_footer_properties;
class group_svg_desc;
class group_table_database_source_query;
class group_table_filter_condition;
class group_table_named_expressions;
class group_table_table;
class group_text_common_ref_content;
class group_text_decls;
class group_text_index_body;
class group_text_index_source_styles;
class group_text_notes_configuration;
class group_text_table_index;
class group_xml_id;
class chart_chart;
class chart_data_point;
class chart_series;
class db_columns;
class db_driver_settings;
class db_index_column;
class db_order_statement;
class db_reports;
class db_table_filter;
class db_table_include_filter;
class db_table_representations;
class db_table_settings;
class db_table_type_filter;
class draw_area_polygon;
class draw_contour_polygon;
class draw_floating_frame;
class draw_layer;
class draw_object_ole;
class form_list_property;
class form_list_value_6;
class meta_creation_date;
class meta_document_statistic;
class number_boolean_style;
class number_day;
class number_era;
class number_month;
class number_number_style;
class number_text_style;
class number_week_of_year;
class office_dde_source;
class office_image;
class office_meta;
class office_scripts;
class presentation_event_listener;
class presentation_sound;
class style_columns;
class style_drop_cap;
class style_list_level_label_alignment;
class style_map;
class style_page_layout;
class style_tab_stop;
class svg_font_face_name;
class table_body;
class table_content_validation;
class table_data_pilot_group;
class table_data_pilot_subtotal;
class table_database_range;
class table_dde_link;
class table_detective;
class table_even_columns;
class table_filter;
class table_filter_or;
class table_first_row;
class table_last_column;
class table_null_date;
class table_odd_rows;
class table_subtotal_rule;
class table_table_column_group;
class table_table_row;
class table_table_source;
class text_bookmark;
class text_bookmark_start;
class text_change_end;
class text_changed_region;
class text_dde_connection;
class text_dde_connection_decls;
class text_description;
class text_execute_macro;
class text_hidden_paragraph;
class text_index_entry_span;
class text_linenumbering_separator;
class text_list_level_style_bullet;
class text_reference_ref;
class text_s;
class text_sender_city;
class text_sender_phone_private;
class text_table_of_content_source;
class text_toc_mark_start;
class text_user_index_mark_start;
class text_variable_decl;
class group_button_type;
class group_column_controls;
class group_common_anim_set_values_attlist;
class group_common_background_transparency_attlist;
class group_common_begin_end_timing_attlist;
class group_common_dr3d_transform_attlist;
class group_common_draw_rel_size_attlist;
class group_common_field_num_format_attlist;
class group_common_horizontal_margin_attlist;
class group_common_repeat;
class group_common_source_cell_range;
class group_common_text_spreadsheet_shape_attlist;
class group_common_value_and_type_attlist;
class group_db_columns;
class group_dr3d_light;
class group_form_connection_resource;
class group_form_form;
class group_number_minutes;
class group_office_meta;
class group_styles;
class group_table_deletions;
class group_table_rows_and_groups;
class group_text_index_entry_tab_stop;
class group_text_p;
class chart_equation;
class chart_label_separator;
class chart_plot_area;
class chart_subtitle;
class chart_title;
class db_data_source_setting;
class db_data_source_settings;
class db_key_column;
class db_table_definition;
class db_table_representation;
class db_table_type;
class dc_description;
class dc_title;
class draw_area_rectangle;
class draw_contour_path;
class draw_hatch;
class draw_image_map;
class draw_opacity;
class form_column;
class meta_editing_duration;
class meta_hyperlink_behaviour;
class meta_keyword;
class meta_user_defined;
class number_day_of_week;
class number_time_style;
class office_automatic_styles;
class office_document;
class office_document_settings;
class office_forms;
class presentation_header_decl;
class presentation_settings;
class style_graphic_properties;
class style_list_level_properties;
class style_page_layout_properties;
class style_table_properties;
class svg_radialGradient;
class table_change_track_table_cell;
class table_content_validations;
class table_cut_offs;
class table_data_pilot_table;
class table_dde_links;
class table_deletions;
class table_even_rows;
class table_first_column;
class table_last_row;
class table_movement_cut_off;
class table_source_cell_range;
class table_table_header_rows;
class table_tracked_changes;
class text_author_name;
class text_bibliography_configuration;
class text_bibliography_mark;
class text_change;
class text_character_count;
class text_index_source_styles;
class text_index_title_template;
class text_line_break;
class text_list_level_style_number;
class text_note_ref;
class text_numbered_paragraph;
class text_outline_level_style;
class text_page_continuation;
class text_page_number;
class text_paragraph_count;
class text_printed_by;
class text_sender_state_or_province;
class text_sequence_ref;
class text_table_count;
class text_table_index_source;
class text_table_of_content_entry_template;
class text_user_index_mark;
class group_common_anim_values_attlist;
class group_common_data_field_attlist;
class group_common_db_object_name;
class group_common_field_data_style_name_attlist;
class group_common_field_description_attlist;
class group_common_number_attlist;
class group_common_office_annotation_name_attlist;
class group_common_tab_attlist;
class group_config_config_item_set;
class group_draw_param;
class group_dropdown;
class group_header_footer_content;
class group_math_math;
class group_paragraph_attrs;
class group_region_content;
class group_style_style_content;
class group_table_functions;
class group_text_alphabetical_index_mark_attrs;
class group_text_illustration_index_entry_content;
class chart_data_label;
class chart_legend;
class chart_symbol_image;
class db_component;
class db_data_source_setting_value;
class db_table_setting;
class dc_subject;
class draw_applet;
class meta_generator;
class meta_printed_by;
class number_fraction;
class number_quarter;
class number_seconds;
class office_change_info;
class office_database;
class office_drawing;
class office_presentation;
class office_spreadsheet;
class style_default_style;
class style_footer_left;
class style_header_footer_properties;
class style_header_style;
class style_paragraph_properties;
class style_region_left;
class style_section_properties;
class style_table_cell_properties;
class table_change_deletion;
class table_database_source_table;
class table_error_message;
class table_help_message;
class table_label_ranges;
class table_odd_columns;
class table_table_row_group;
class text_alphabetical_index_mark;
class text_bookmark_ref;
class text_creation_date;
class text_dde_connection_decl;
class text_editing_duration;
class text_image_count;
class text_index_entry_chapter;
class text_modification_date;
class text_object_count;
class text_object_index_entry_template;
class text_page_count;
class text_placeholder;
class text_print_time;
class text_sequence;
class text_table_formula;
class text_table_index_entry_template;
class text_text_input;
class text_title;
class text_user_field_decl;
class text_user_field_input;
class text_user_index_entry_template;
class text_variable_get;
class text_variable_set;
class group_common_anim_attlist;
class group_common_db_default_value;
class group_common_field_database_table;
class group_db_query;
class group_form_control_attlist;
class group_shape;
class group_table_table_cell_attlist;
class group_target_frame;
class group_text_list_item_content;
class anim_command;
class chart_footer;
class chart_wall;
class config_config_item_map_named;
class db_application_connection_settings;
class db_column_definition;
class db_filter_statement;
class db_query;
class db_table_filter_pattern;
class dr3d_light;
class draw_a;
class draw_page;
class meta_initial_creator;
class number_number;
class number_year;
class office_annotation_end;
class office_document_meta;
class office_settings;
class presentation_notes;
class style_footer;
class style_handout_master;
class style_header_left;
class style_region_center;
class style_style;
class table_consolidation;
class table_data_pilot_level;
class table_filter_and;
class table_shapes;
class text_alphabetical_index_entry_template;
class text_change_start;
class text_database_display;
class text_database_next;
class text_database_row_select;
class text_list_header;
class text_modification_time;
class text_number;
class text_print_date;
class text_user_defined;
class text_variable_input;
class group_common_basic_timing_attlist;
class group_common_form_control_content;
class group_common_section_attlist;
class group_config_items;
class group_paragraph_content;
class anim_audio;
class chart_stock_range_line;
class config_config_item_set;
class db_component_collection;
class db_query_collection;
class draw_plugin;
class form_button;
class form_combobox;
class form_file;
class form_form;
class form_frame;
class form_grid;
class form_image;
class form_list_value_2;
class form_property;
class form_text;
class meta_date_string;
class number_scientific_number;
class office_chart;
class office_styles;
class presentation_animations;
class style_footer_style;
class style_master_page;
class svg_definition_src;
class table_table;
class table_table_column;
class text_alphabetical_index;
class text_bibliography;
class text_creation_time;
class text_database_row_number;
class text_illustration_index;
class text_list_item;
class text_object_index;
class text_page_variable_get;
class text_table_of_content;
class text_user_field_get;
class text_user_index_source;
class group_common_draw_id_attlist;
class group_common_timing_attlist;
class group_paragraph_content_or_hyperlink;
class anim_animate;
class anim_animateMotion;
class anim_iterate;
class anim_seq;
class anim_transitionFilter;
class db_column;
class dc_creator;
class dr3d_extrude;
class dr3d_scene;
class draw_area_circle;
class draw_g;
class draw_layer_set;
class draw_marker;
class draw_object;
class form_checkbox;
class form_fixed_text;
class form_generic_control;
class form_image_frame;
class form_radio;
class number_date_style;
class office_master_styles;
class style_chart_properties;
class style_region_right;
class text_alphabetical_index_mark_start;
class text_database_name;
class text_illustration_index_entry_template;
class text_list;
class text_meta_field;
class text_p;
class text_span;
class text_time;
class text_word_count;
class group_common_draw_shape_with_styles_attlist;
class group_common_numeric_control_attlist;
class group_table_change_track_table_cell;
class group_text_content;
class anim_animateTransform;
class anim_set;
class db_queries;
class dr3d_rotate;
class draw_page_thumbnail;
class draw_text_box;
class form_formatted_text;
class form_listbox;
class form_password;
class form_time;
class style_header;
class table_cell_content_deletion;
class text_bibliography_entry_template;
class text_note_body;
class text_section;
class text_table_index;
class group_common_draw_shape_with_text_and_styles_attlist;
class group_index_content_main;
class group_table_table_cell_content;
class anim_par;
class dr3d_cube;
class draw_frame;
class form_hidden;
class form_textarea;
class office_annotation;
class table_table_cell;
class text_date;
class text_expression;
class text_h;
class text_index_title;
class text_meta;
class text_sender_fax;
class group_draw_text;
class anim_animateColor;
class dr3d_sphere;
class draw_circle;
class draw_control;
class draw_ellipse;
class draw_line;
class draw_path;
class draw_polyline;
class draw_regular_polygon;
class form_number;
class office_text;
class table_covered_table_cell;
class table_previous;
class text_index_body;
class text_ruby_base;
class group_office_change_info;
class draw_caption;
class draw_custom_shape;
class draw_measure;
class draw_rect;
class form_value_range;
class table_deletion;
class table_movement;
class text_deletion;
class text_insertion;
class config_config_item_map_entry;
class draw_image;
class form_date;
class table_insertion;
class text_format_change;
class draw_connector;
class table_cell_content_change;
class text_user_index;
class draw_polygon;
class text_a;
class group_change_marks {
private:
    OdfWriter& xml;
public:
    group_change_marks(OdfWriter& x) :xml(x) {
    }
    text_change add_text_change(const QString& text_change_id);
    text_change_end add_text_change_end(const QString& text_change_id);
    text_change_start add_text_change_start(const QString& text_change_id);
};
class group_chart_data_label {
private:
    OdfWriter& xml;
public:
    group_chart_data_label(OdfWriter& x) :xml(x) {
    }
    chart_data_label add_chart_data_label();
};
class group_chart_title {
private:
    OdfWriter& xml;
public:
    group_chart_title(OdfWriter& x) :xml(x) {
    }
    chart_title add_chart_title();
};
class group_common_anim_add_accum_attlist {
private:
    OdfWriter& xml;
public:
    group_common_anim_add_accum_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_smil_accumulate(const T& value) {
        xml.addAttribute("smil:accumulate", value);
    }
    template<class T>
    void set_smil_additive(const T& value) {
        xml.addAttribute("smil:additive", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_anim_named_target_attlist {
private:
    OdfWriter& xml;
public:
    group_common_anim_named_target_attlist(OdfWriter& x, const QString& smil_attributeName) :xml(x) {
        xml.addAttribute("smil:attributeName", smil_attributeName);
    }
    group_common_anim_named_target_attlist(OdfWriter& x) :xml(x) {}
    void set_smil_attributeName(const QString& value) {
        xml.addAttribute("smil:attributeName", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_anim_spline_mode_attlist {
private:
    OdfWriter& xml;
public:
    group_common_anim_spline_mode_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_smil_calcMode(const T& value) {
        xml.addAttribute("smil:calcMode", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_background_color_attlist {
private:
    OdfWriter& xml;
public:
    group_common_background_color_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_fo_background_color(const T& value) {
        xml.addAttribute("fo:background-color", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_border_attlist {
private:
    OdfWriter& xml;
public:
    group_common_border_attlist(OdfWriter& x) :xml(x) {
    }
    void set_fo_border(const QString& value) {
        xml.addAttribute("fo:border", value);
    }
    void set_fo_border_bottom(const QString& value) {
        xml.addAttribute("fo:border-bottom", value);
    }
    void set_fo_border_left(const QString& value) {
        xml.addAttribute("fo:border-left", value);
    }
    void set_fo_border_right(const QString& value) {
        xml.addAttribute("fo:border-right", value);
    }
    void set_fo_border_top(const QString& value) {
        xml.addAttribute("fo:border-top", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_break_attlist {
private:
    OdfWriter& xml;
public:
    group_common_break_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_fo_break_after(const T& value) {
        xml.addAttribute("fo:break-after", value);
    }
    template<class T>
    void set_fo_break_before(const T& value) {
        xml.addAttribute("fo:break-before", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_contour_attlist {
private:
    OdfWriter& xml;
public:
    group_common_contour_attlist(OdfWriter& x, const QString& draw_recreate_on_edit) :xml(x) {
        xml.addAttribute("draw:recreate-on-edit", draw_recreate_on_edit);
    }
    group_common_contour_attlist(OdfWriter& x) :xml(x) {}
    template<class T>
    void set_draw_recreate_on_edit(const T& value) {
        xml.addAttribute("draw:recreate-on-edit", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_current_value_attlist {
private:
    OdfWriter& xml;
public:
    group_common_current_value_attlist(OdfWriter& x) :xml(x) {
    }
    void set_form_current_value(const QString& value) {
        xml.addAttribute("form:current-value", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_data_style_attlist {
private:
    OdfWriter& xml;
public:
    group_common_data_style_attlist(OdfWriter& x, const QString& style_name) :xml(x) {
        xml.addAttribute("style:name", style_name);
    }
    group_common_data_style_attlist(OdfWriter& x) :xml(x) {}
    void set_number_country(const QString& value) {
        xml.addAttribute("number:country", value);
    }
    void set_number_language(const QString& value) {
        xml.addAttribute("number:language", value);
    }
    void set_number_rfc_language_tag(const QString& value) {
        xml.addAttribute("number:rfc-language-tag", value);
    }
    void set_number_script(const QString& value) {
        xml.addAttribute("number:script", value);
    }
    void set_number_title(const QString& value) {
        xml.addAttribute("number:title", value);
    }
    void set_number_transliteration_country(const QString& value) {
        xml.addAttribute("number:transliteration-country", value);
    }
    void set_number_transliteration_format(const QString& value) {
        xml.addAttribute("number:transliteration-format", value);
    }
    void set_number_transliteration_language(const QString& value) {
        xml.addAttribute("number:transliteration-language", value);
    }
    template<class T>
    void set_number_transliteration_style(const T& value) {
        xml.addAttribute("number:transliteration-style", value);
    }
    void set_style_display_name(const QString& value) {
        xml.addAttribute("style:display-name", value);
    }
    void set_style_name(const QString& value) {
        xml.addAttribute("style:name", value);
    }
    template<class T>
    void set_style_volatile(const T& value) {
        xml.addAttribute("style:volatile", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_db_object_description {
private:
    OdfWriter& xml;
public:
    group_common_db_object_description(OdfWriter& x) :xml(x) {
    }
    void set_db_description(const QString& value) {
        xml.addAttribute("db:description", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_db_object_title {
private:
    OdfWriter& xml;
public:
    group_common_db_object_title(OdfWriter& x) :xml(x) {
    }
    void set_db_title(const QString& value) {
        xml.addAttribute("db:title", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_db_table_style_name {
private:
    OdfWriter& xml;
public:
    group_common_db_table_style_name(OdfWriter& x) :xml(x) {
    }
    void set_db_default_row_style_name(const QString& value) {
        xml.addAttribute("db:default-row-style-name", value);
    }
    void set_db_style_name(const QString& value) {
        xml.addAttribute("db:style-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_decimal_places_attlist {
private:
    OdfWriter& xml;
public:
    group_common_decimal_places_attlist(OdfWriter& x) :xml(x) {
    }
    void set_number_decimal_places(qint64 value) {
        xml.addAttribute("number:decimal-places", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_disabled_attlist {
private:
    OdfWriter& xml;
public:
    group_common_disabled_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_form_disabled(const T& value) {
        xml.addAttribute("form:disabled", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_area_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_area_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_draw_nohref(const T& value) {
        xml.addAttribute("draw:nohref", value);
    }
    void set_office_name(const QString& value) {
        xml.addAttribute("office:name", value);
    }
    template<class T>
    void set_office_target_frame_name(const T& value) {
        xml.addAttribute("office:target-frame-name", value);
    }
    void set_xlink_href(const QUrl& value) {
        xml.addAttribute("xlink:href", value);
    }
    template<class T>
    void set_xlink_show(const T& value) {
        xml.addAttribute("xlink:show", value);
    }
    template<class T>
    void set_xlink_type(const T& value) {
        xml.addAttribute("xlink:type", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_circle_ellipse_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_circle_ellipse_attlist(OdfWriter& x) :xml(x) {
    }
    void set_draw_end_angle(const QString& value) {
        xml.addAttribute("draw:end-angle", value);
    }
    template<class T>
    void set_draw_kind(const T& value) {
        xml.addAttribute("draw:kind", value);
    }
    void set_draw_start_angle(const QString& value) {
        xml.addAttribute("draw:start-angle", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_data_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_data_attlist(OdfWriter& x, const QUrl& xlink_href) :xml(x) {
        xml.addAttribute("xlink:href", xlink_href);
        xml.addAttribute("xlink:type", "simple");
    }
    group_common_draw_data_attlist(OdfWriter& x) :xml(x) {}
    template<class T>
    void set_xlink_actuate(const T& value) {
        xml.addAttribute("xlink:actuate", value);
    }
    void set_xlink_href(const QUrl& value) {
        xml.addAttribute("xlink:href", value);
    }
    template<class T>
    void set_xlink_show(const T& value) {
        xml.addAttribute("xlink:show", value);
    }
    template<class T>
    void set_xlink_type(const T& value) {
        xml.addAttribute("xlink:type", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_name_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_name_attlist(OdfWriter& x) :xml(x) {
    }
    void set_draw_name(const QString& value) {
        xml.addAttribute("draw:name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_points_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_points_attlist(OdfWriter& x, const QString& draw_points) :xml(x) {
        xml.addAttribute("draw:points", draw_points);
    }
    group_common_draw_points_attlist(OdfWriter& x) :xml(x) {}
    void set_draw_points(const QString& value) {
        xml.addAttribute("draw:points", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_style_name_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_style_name_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_draw_class_names(const T& value) {
        xml.addAttribute("draw:class-names", value);
    }
    void set_draw_style_name(const QString& value) {
        xml.addAttribute("draw:style-name", value);
    }
    template<class T>
    void set_presentation_class_names(const T& value) {
        xml.addAttribute("presentation:class-names", value);
    }
    void set_presentation_style_name(const QString& value) {
        xml.addAttribute("presentation:style-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_z_index_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_z_index_attlist(OdfWriter& x) :xml(x) {
    }
    void set_draw_z_index(quint64 value) {
        xml.addAttribute("draw:z-index", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_endsync_timing_attlist {
private:
    OdfWriter& xml;
public:
    group_common_endsync_timing_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_smil_endsync(const T& value) {
        xml.addAttribute("smil:endsync", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_field_display_value_formula_attlist {
private:
    OdfWriter& xml;
public:
    group_common_field_display_value_formula_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_text_display(const T& value) {
        xml.addAttribute("text:display", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_field_fixed_attlist {
private:
    OdfWriter& xml;
public:
    group_common_field_fixed_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_text_fixed(const T& value) {
        xml.addAttribute("text:fixed", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_field_name_attlist {
private:
    OdfWriter& xml;
public:
    group_common_field_name_attlist(OdfWriter& x, const QString& text_name) :xml(x) {
        xml.addAttribute("text:name", text_name);
    }
    group_common_field_name_attlist(OdfWriter& x) :xml(x) {}
    void set_text_name(const QString& value) {
        xml.addAttribute("text:name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_form_control_attlist {
private:
    OdfWriter& xml;
public:
    group_common_form_control_attlist(OdfWriter& x) :xml(x) {
    }
    void set_form_control_implementation(const QString& value) {
        xml.addAttribute("form:control-implementation", value);
    }
    void set_form_name(const QString& value) {
        xml.addAttribute("form:name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_form_relative_image_position_attlist {
private:
    OdfWriter& xml;
public:
    group_common_form_relative_image_position_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_form_image_align(const T& value) {
        xml.addAttribute("form:image-align", value);
    }
    template<class T>
    void set_form_image_position(const T& value) {
        xml.addAttribute("form:image-position", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_format_source_attlist {
private:
    OdfWriter& xml;
public:
    group_common_format_source_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_number_format_source(const T& value) {
        xml.addAttribute("number:format-source", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_in_content_meta_attlist {
private:
    OdfWriter& xml;
public:
    group_common_in_content_meta_attlist(OdfWriter& x, const QString& xhtml_about, const QString& xhtml_property) :xml(x) {
        xml.addAttribute("xhtml:about", xhtml_about);
        xml.addAttribute("xhtml:property", xhtml_property);
    }
    group_common_in_content_meta_attlist(OdfWriter& x) :xml(x) {}
    template<class T>
    void set_xhtml_about(const T& value) {
        xml.addAttribute("xhtml:about", value);
    }
    void set_xhtml_content(const QString& value) {
        xml.addAttribute("xhtml:content", value);
    }
    void set_xhtml_datatype(const QString& value) {
        xml.addAttribute("xhtml:datatype", value);
    }
    template<class T>
    void set_xhtml_property(const T& value) {
        xml.addAttribute("xhtml:property", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_linked_cell {
private:
    OdfWriter& xml;
public:
    group_common_linked_cell(OdfWriter& x) :xml(x) {
    }
    void set_form_linked_cell(const QString& value) {
        xml.addAttribute("form:linked-cell", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_maxlength_attlist {
private:
    OdfWriter& xml;
public:
    group_common_maxlength_attlist(OdfWriter& x) :xml(x) {
    }
    void set_form_max_length(quint64 value) {
        xml.addAttribute("form:max-length", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_num_format_prefix_suffix_attlist {
private:
    OdfWriter& xml;
public:
    group_common_num_format_prefix_suffix_attlist(OdfWriter& x) :xml(x) {
    }
    void set_style_num_prefix(const QString& value) {
        xml.addAttribute("style:num-prefix", value);
    }
    void set_style_num_suffix(const QString& value) {
        xml.addAttribute("style:num-suffix", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_padding_attlist {
private:
    OdfWriter& xml;
public:
    group_common_padding_attlist(OdfWriter& x) :xml(x) {
    }
    void set_fo_padding(const QString& value) {
        xml.addAttribute("fo:padding", value);
    }
    void set_fo_padding_bottom(const QString& value) {
        xml.addAttribute("fo:padding-bottom", value);
    }
    void set_fo_padding_left(const QString& value) {
        xml.addAttribute("fo:padding-left", value);
    }
    void set_fo_padding_right(const QString& value) {
        xml.addAttribute("fo:padding-right", value);
    }
    void set_fo_padding_top(const QString& value) {
        xml.addAttribute("fo:padding-top", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_presentation_effect_attlist {
private:
    OdfWriter& xml;
public:
    group_common_presentation_effect_attlist(OdfWriter& x, const QString& draw_shape_id) :xml(x) {
        xml.addAttribute("draw:shape-id", draw_shape_id);
    }
    group_common_presentation_effect_attlist(OdfWriter& x) :xml(x) {}
    void set_draw_shape_id(const QString& value) {
        xml.addAttribute("draw:shape-id", value);
    }
    void set_presentation_delay(Duration value) {
        xml.addAttribute("presentation:delay", value);
    }
    template<class T>
    void set_presentation_direction(const T& value) {
        xml.addAttribute("presentation:direction", value);
    }
    template<class T>
    void set_presentation_effect(const T& value) {
        xml.addAttribute("presentation:effect", value);
    }
    void set_presentation_path_id(const QString& value) {
        xml.addAttribute("presentation:path-id", value);
    }
    template<class T>
    void set_presentation_speed(const T& value) {
        xml.addAttribute("presentation:speed", value);
    }
    void set_presentation_start_scale(const QString& value) {
        xml.addAttribute("presentation:start-scale", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_printable_attlist {
private:
    OdfWriter& xml;
public:
    group_common_printable_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_form_printable(const T& value) {
        xml.addAttribute("form:printable", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_rotation_angle_attlist {
private:
    OdfWriter& xml;
public:
    group_common_rotation_angle_attlist(OdfWriter& x) :xml(x) {
    }
    void set_style_rotation_angle(const QString& value) {
        xml.addAttribute("style:rotation-angle", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_shadow_attlist {
private:
    OdfWriter& xml;
public:
    group_common_shadow_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_style_shadow(const T& value) {
        xml.addAttribute("style:shadow", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_spin_button {
private:
    OdfWriter& xml;
public:
    group_common_spin_button(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_form_spin_button(const T& value) {
        xml.addAttribute("form:spin-button", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_stock_marker_attlist {
private:
    OdfWriter& xml;
public:
    group_common_stock_marker_attlist(OdfWriter& x) :xml(x) {
    }
    void set_chart_style_name(const QString& value) {
        xml.addAttribute("chart:style-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_style_header_footer_attlist {
private:
    OdfWriter& xml;
public:
    group_common_style_header_footer_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_style_display(const T& value) {
        xml.addAttribute("style:display", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_svg_gradient_attlist {
private:
    OdfWriter& xml;
public:
    group_common_svg_gradient_attlist(OdfWriter& x, const QString& draw_name) :xml(x) {
        xml.addAttribute("draw:name", draw_name);
    }
    group_common_svg_gradient_attlist(OdfWriter& x) :xml(x) {}
    void set_draw_display_name(const QString& value) {
        xml.addAttribute("draw:display-name", value);
    }
    void set_draw_name(const QString& value) {
        xml.addAttribute("draw:name", value);
    }
    void set_svg_gradientTransform(const QString& value) {
        xml.addAttribute("svg:gradientTransform", value);
    }
    template<class T>
    void set_svg_gradientUnits(const T& value) {
        xml.addAttribute("svg:gradientUnits", value);
    }
    template<class T>
    void set_svg_spreadMethod(const T& value) {
        xml.addAttribute("svg:spreadMethod", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_table_cell_address_attlist {
private:
    OdfWriter& xml;
public:
    group_common_table_cell_address_attlist(OdfWriter& x, qint64 table_column, qint64 table_row, qint64 table_table) :xml(x) {
        xml.addAttribute("table:column", table_column);
        xml.addAttribute("table:row", table_row);
        xml.addAttribute("table:table", table_table);
    }
    group_common_table_cell_address_attlist(OdfWriter& x) :xml(x) {}
    void set_table_column(qint64 value) {
        xml.addAttribute("table:column", value);
    }
    void set_table_row(qint64 value) {
        xml.addAttribute("table:row", value);
    }
    void set_table_table(qint64 value) {
        xml.addAttribute("table:table", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_table_range_attlist : public group_common_table_cell_address_attlist {
private:
    OdfWriter& xml;
public:
    group_common_table_range_attlist(OdfWriter& x) :group_common_table_cell_address_attlist(x), xml(x) {
    }
    void set_table_end_column(qint64 value) {
        xml.addAttribute("table:end-column", value);
    }
    void set_table_end_row(qint64 value) {
        xml.addAttribute("table:end-row", value);
    }
    void set_table_end_table(qint64 value) {
        xml.addAttribute("table:end-table", value);
    }
    void set_table_start_column(qint64 value) {
        xml.addAttribute("table:start-column", value);
    }
    void set_table_start_row(qint64 value) {
        xml.addAttribute("table:start-row", value);
    }
    void set_table_start_table(qint64 value) {
        xml.addAttribute("table:start-table", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_text_align {
private:
    OdfWriter& xml;
public:
    group_common_text_align(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_fo_text_align(const T& value) {
        xml.addAttribute("fo:text-align", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_title_attlist {
private:
    OdfWriter& xml;
public:
    group_common_title_attlist(OdfWriter& x) :xml(x) {
    }
    void set_form_title(const QString& value) {
        xml.addAttribute("form:title", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_value_attlist {
private:
    OdfWriter& xml;
public:
    group_common_value_attlist(OdfWriter& x) :xml(x) {
    }
    void set_form_value(const QString& value) {
        xml.addAttribute("form:value", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_vertical_margin_attlist {
private:
    OdfWriter& xml;
public:
    group_common_vertical_margin_attlist(OdfWriter& x) :xml(x) {
    }
    void set_fo_margin_bottom(const QString& value) {
        xml.addAttribute("fo:margin-bottom", value);
    }
    void set_fo_margin_top(const QString& value) {
        xml.addAttribute("fo:margin-top", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_vertical_rel_attlist {
private:
    OdfWriter& xml;
public:
    group_common_vertical_rel_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_style_vertical_rel(const T& value) {
        xml.addAttribute("style:vertical-rel", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_config_config_item_map_entry {
private:
    OdfWriter& xml;
public:
    group_config_config_item_map_entry(OdfWriter& x) :xml(x) {
    }
    config_config_item_map_entry add_config_config_item_map_entry();
};
class group_db_character_set {
private:
    OdfWriter& xml;
public:
    group_db_character_set(OdfWriter& x) :xml(x) {
    }
    db_character_set add_db_character_set();
};
class group_db_command {
private:
    OdfWriter& xml;
public:
    group_db_command(OdfWriter& x, const QString& db_command) :xml(x) {
        xml.addAttribute("db:command", db_command);
    }
    group_db_command(OdfWriter& x) :xml(x) {}
    void set_db_command(const QString& value) {
        xml.addAttribute("db:command", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_db_component_collection {
private:
    OdfWriter& xml;
public:
    group_db_component_collection(OdfWriter& x) :xml(x) {
    }
    db_component_collection add_db_component_collection(const QString& db_name);
};
class group_db_filter_statement {
private:
    OdfWriter& xml;
public:
    group_db_filter_statement(OdfWriter& x) :xml(x) {
    }
    db_filter_statement add_db_filter_statement(const QString& db_command);
};
class group_db_order_statement {
private:
    OdfWriter& xml;
public:
    group_db_order_statement(OdfWriter& x) :xml(x) {
    }
    db_order_statement add_db_order_statement(const QString& db_command);
};
class group_db_query_collection {
private:
    OdfWriter& xml;
public:
    group_db_query_collection(OdfWriter& x) :xml(x) {
    }
    db_query_collection add_db_query_collection(const QString& db_name);
};
class group_db_table_filter_pattern {
private:
    OdfWriter& xml;
public:
    group_db_table_filter_pattern(OdfWriter& x) :xml(x) {
    }
    db_table_filter_pattern add_db_table_filter_pattern();
};
class group_dc_date {
private:
    OdfWriter& xml;
public:
    group_dc_date(OdfWriter& x) :xml(x) {
    }
    dc_date add_dc_date();
};
class group_dr3d_scene {
private:
    OdfWriter& xml;
public:
    group_dr3d_scene(OdfWriter& x) :xml(x) {
    }
    dr3d_scene add_dr3d_scene();
};
class group_draw_caption_attlist {
private:
    OdfWriter& xml;
public:
    group_draw_caption_attlist(OdfWriter& x) :xml(x) {
    }
    void set_draw_caption_point_x(const QString& value) {
        xml.addAttribute("draw:caption-point-x", value);
    }
    void set_draw_caption_point_y(const QString& value) {
        xml.addAttribute("draw:caption-point-y", value);
    }
    void set_draw_corner_radius(const QString& value) {
        xml.addAttribute("draw:corner-radius", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_draw_glue_point {
private:
    OdfWriter& xml;
public:
    group_draw_glue_point(OdfWriter& x) :xml(x) {
    }
    draw_glue_point add_draw_glue_point(const QString& draw_escape_direction, quint64 draw_id, const QString& svg_x, const QString& svg_y);
};
class group_draw_page {
private:
    OdfWriter& xml;
public:
    group_draw_page(OdfWriter& x) :xml(x) {
    }
    draw_page add_draw_page(const QString& draw_master_page_name);
};
class group_for {
private:
    OdfWriter& xml;
public:
    group_for(OdfWriter& x) :xml(x) {
    }
    void set_form_for(const QString& value) {
        xml.addAttribute("form:for", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_form_properties {
private:
    OdfWriter& xml;
public:
    group_form_properties(OdfWriter& x) :xml(x) {
    }
    form_properties add_form_properties();
};
class group_label {
private:
    OdfWriter& xml;
public:
    group_label(OdfWriter& x) :xml(x) {
    }
    void set_form_label(const QString& value) {
        xml.addAttribute("form:label", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_list_source_type {
private:
    OdfWriter& xml;
public:
    group_list_source_type(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_form_list_source_type(const T& value) {
        xml.addAttribute("form:list-source-type", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_number_am_pm {
private:
    OdfWriter& xml;
public:
    group_number_am_pm(OdfWriter& x) :xml(x) {
    }
    number_am_pm add_number_am_pm();
};
class group_number_hours {
private:
    OdfWriter& xml;
public:
    group_number_hours(OdfWriter& x) :xml(x) {
    }
    number_hours add_number_hours();
};
class group_number_number {
private:
    OdfWriter& xml;
public:
    group_number_number(OdfWriter& x) :xml(x) {
    }
    number_number add_number_number();
};
class group_number_text {
private:
    OdfWriter& xml;
public:
    group_number_text(OdfWriter& x) :xml(x) {
    }
    number_text add_number_text();
};
class group_office_automatic_styles {
private:
    OdfWriter& xml;
public:
    group_office_automatic_styles(OdfWriter& x) :xml(x) {
    }
    office_automatic_styles add_office_automatic_styles();
};
class group_office_body {
private:
    OdfWriter& xml;
public:
    group_office_body(OdfWriter& x) :xml(x) {
    }
    office_body add_office_body();
};
class group_office_dde_source {
private:
    OdfWriter& xml;
public:
    group_office_dde_source(OdfWriter& x) :xml(x) {
    }
    office_dde_source add_office_dde_source(const QString& office_dde_application, const QString& office_dde_item, const QString& office_dde_topic);
};
class group_office_document_common_attrs {
private:
    OdfWriter& xml;
public:
    group_office_document_common_attrs(OdfWriter& x) :xml(x) {
        xml.addAttribute("office:version", "1.2");
    }
    template<class T>
    void set_grddl_transformation(const T& value) {
        xml.addAttribute("grddl:transformation", value);
    }
    template<class T>
    void set_office_version(const T& value) {
        xml.addAttribute("office:version", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_office_font_face_decls {
private:
    OdfWriter& xml;
public:
    group_office_font_face_decls(OdfWriter& x) :xml(x) {
    }
    office_font_face_decls add_office_font_face_decls();
};
class group_office_master_styles {
private:
    OdfWriter& xml;
public:
    group_office_master_styles(OdfWriter& x) :xml(x) {
    }
    office_master_styles add_office_master_styles();
};
class group_office_scripts {
private:
    OdfWriter& xml;
public:
    group_office_scripts(OdfWriter& x) :xml(x) {
    }
    office_scripts add_office_scripts();
};
class group_office_styles {
private:
    OdfWriter& xml;
public:
    group_office_styles(OdfWriter& x) :xml(x) {
    }
    office_styles add_office_styles();
};
class group_presentation_animation_elements {
private:
    OdfWriter& xml;
public:
    group_presentation_animation_elements(OdfWriter& x) :xml(x) {
    }
    presentation_dim add_presentation_dim(const QString& draw_color, const QString& draw_shape_id);
    presentation_hide_shape add_presentation_hide_shape(const QString& draw_shape_id);
    presentation_hide_text add_presentation_hide_text(const QString& draw_shape_id);
    presentation_play add_presentation_play(const QString& draw_shape_id);
    presentation_show_shape add_presentation_show_shape(const QString& draw_shape_id);
    presentation_show_text add_presentation_show_text(const QString& draw_shape_id);
};
class group_presentation_notes {
private:
    OdfWriter& xml;
public:
    group_presentation_notes(OdfWriter& x) :xml(x) {
    }
    presentation_notes add_presentation_notes();
};
class group_presentation_sound {
private:
    OdfWriter& xml;
public:
    group_presentation_sound(OdfWriter& x) :xml(x) {
    }
    presentation_sound add_presentation_sound(const QUrl& xlink_href);
};
class group_selected {
private:
    OdfWriter& xml;
public:
    group_selected(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_form_selected(const T& value) {
        xml.addAttribute("form:selected", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_style_background_image {
private:
    OdfWriter& xml;
public:
    group_style_background_image(OdfWriter& x) :xml(x) {
    }
    style_background_image add_style_background_image();
};
class group_style_graphic_fill_properties_attlist {
private:
    OdfWriter& xml;
public:
    group_style_graphic_fill_properties_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_draw_fill(const T& value) {
        xml.addAttribute("draw:fill", value);
    }
    void set_draw_fill_color(const QString& value) {
        xml.addAttribute("draw:fill-color", value);
    }
    void set_draw_fill_gradient_name(const QString& value) {
        xml.addAttribute("draw:fill-gradient-name", value);
    }
    void set_draw_fill_hatch_name(const QString& value) {
        xml.addAttribute("draw:fill-hatch-name", value);
    }
    template<class T>
    void set_draw_fill_hatch_solid(const T& value) {
        xml.addAttribute("draw:fill-hatch-solid", value);
    }
    void set_draw_fill_image_height(const QString& value) {
        xml.addAttribute("draw:fill-image-height", value);
    }
    void set_draw_fill_image_name(const QString& value) {
        xml.addAttribute("draw:fill-image-name", value);
    }
    template<class T>
    void set_draw_fill_image_ref_point(const T& value) {
        xml.addAttribute("draw:fill-image-ref-point", value);
    }
    void set_draw_fill_image_ref_point_x(const QString& value) {
        xml.addAttribute("draw:fill-image-ref-point-x", value);
    }
    void set_draw_fill_image_ref_point_y(const QString& value) {
        xml.addAttribute("draw:fill-image-ref-point-y", value);
    }
    void set_draw_fill_image_width(const QString& value) {
        xml.addAttribute("draw:fill-image-width", value);
    }
    void set_draw_gradient_step_count(quint64 value) {
        xml.addAttribute("draw:gradient-step-count", value);
    }
    void set_draw_opacity(const QString& value) {
        xml.addAttribute("draw:opacity", value);
    }
    void set_draw_opacity_name(const QString& value) {
        xml.addAttribute("draw:opacity-name", value);
    }
    void set_draw_secondary_fill_color(const QString& value) {
        xml.addAttribute("draw:secondary-fill-color", value);
    }
    template<class T>
    void set_draw_tile_repeat_offset(const T& value) {
        xml.addAttribute("draw:tile-repeat-offset", value);
    }
    template<class T>
    void set_style_repeat(const T& value) {
        xml.addAttribute("style:repeat", value);
    }
    template<class T>
    void set_svg_fill_rule(const T& value) {
        xml.addAttribute("svg:fill-rule", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_style_list_level_properties {
private:
    OdfWriter& xml;
public:
    group_style_list_level_properties(OdfWriter& x) :xml(x) {
    }
    style_list_level_properties add_style_list_level_properties();
};
class group_style_page_layout_content {
private:
    OdfWriter& xml;
public:
    group_style_page_layout_content(OdfWriter& x) :xml(x) {
    }
    style_footer_style add_style_footer_style();
    style_header_style add_style_header_style();
    style_page_layout_properties add_style_page_layout_properties();
};
class group_style_text_properties {
private:
    OdfWriter& xml;
public:
    group_style_text_properties(OdfWriter& x) :xml(x) {
    }
    style_text_properties add_style_text_properties();
};
class group_svg_stop {
private:
    OdfWriter& xml;
public:
    group_svg_stop(OdfWriter& x) :xml(x) {
    }
    svg_stop add_svg_stop(const QString& svg_offset);
};
class group_table_cell_address {
private:
    OdfWriter& xml;
public:
    group_table_cell_address(OdfWriter& x) :xml(x) {
    }
    table_cell_address add_table_cell_address(qint64 table_column, qint64 table_row, qint64 table_table);
};
class group_table_database_source_sql {
private:
    OdfWriter& xml;
public:
    group_table_database_source_sql(OdfWriter& x) :xml(x) {
    }
    table_database_source_sql add_table_database_source_sql(const QString& table_database_name, const QString& table_sql_statement);
};
class group_table_decls {
private:
    OdfWriter& xml;
public:
    group_table_decls(OdfWriter& x) :xml(x) {
    }
    table_calculation_settings add_table_calculation_settings();
    table_content_validations add_table_content_validations();
    table_label_ranges add_table_label_ranges();
};
class group_table_dependencies {
private:
    OdfWriter& xml;
public:
    group_table_dependencies(OdfWriter& x) :xml(x) {
    }
    table_dependencies add_table_dependencies();
};
class group_table_filter_and {
private:
    OdfWriter& xml;
public:
    group_table_filter_and(OdfWriter& x) :xml(x) {
    }
    table_filter_and add_table_filter_and();
};
class group_table_filter_or {
private:
    OdfWriter& xml;
public:
    group_table_filter_or(OdfWriter& x) :xml(x) {
    }
    table_filter_or add_table_filter_or();
};
class group_table_linked_source_attlist {
private:
    OdfWriter& xml;
public:
    group_table_linked_source_attlist(OdfWriter& x, const QUrl& xlink_href) :xml(x) {
        xml.addAttribute("xlink:href", xlink_href);
        xml.addAttribute("xlink:type", "simple");
    }
    group_table_linked_source_attlist(OdfWriter& x) :xml(x) {}
    void set_table_filter_name(const QString& value) {
        xml.addAttribute("table:filter-name", value);
    }
    void set_table_filter_options(const QString& value) {
        xml.addAttribute("table:filter-options", value);
    }
    void set_table_refresh_delay(Duration value) {
        xml.addAttribute("table:refresh-delay", value);
    }
    template<class T>
    void set_xlink_actuate(const T& value) {
        xml.addAttribute("xlink:actuate", value);
    }
    void set_xlink_href(const QUrl& value) {
        xml.addAttribute("xlink:href", value);
    }
    template<class T>
    void set_xlink_type(const T& value) {
        xml.addAttribute("xlink:type", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_table_table_column {
private:
    OdfWriter& xml;
public:
    group_table_table_column(OdfWriter& x) :xml(x) {
    }
    table_table_column add_table_table_column();
};
class group_target_location {
private:
    OdfWriter& xml;
public:
    group_target_location(OdfWriter& x) :xml(x) {
    }
    void set_xlink_href(const QUrl& value) {
        xml.addAttribute("xlink:href", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_text_alphabetical_index {
private:
    OdfWriter& xml;
public:
    group_text_alphabetical_index(OdfWriter& x) :xml(x) {
    }
    text_alphabetical_index add_text_alphabetical_index(const QString& text_name);
};
class group_text_bibliography {
private:
    OdfWriter& xml;
public:
    group_text_bibliography(OdfWriter& x) :xml(x) {
    }
    text_bibliography add_text_bibliography(const QString& text_name);
};
class group_text_bookmark_ref_content {
private:
    OdfWriter& xml;
public:
    group_text_bookmark_ref_content(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_text_reference_format(const T& value) {
        xml.addAttribute("text:reference-format", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_text_h {
private:
    OdfWriter& xml;
public:
    group_text_h(OdfWriter& x) :xml(x) {
    }
    text_h add_text_h(quint64 text_outline_level);
};
class group_text_illustration_index {
private:
    OdfWriter& xml;
public:
    group_text_illustration_index(OdfWriter& x) :xml(x) {
    }
    text_illustration_index add_text_illustration_index(const QString& text_name);
};
class group_text_index_entry_chapter {
private:
    OdfWriter& xml;
public:
    group_text_index_entry_chapter(OdfWriter& x) :xml(x) {
    }
    text_index_entry_chapter add_text_index_entry_chapter();
};
class group_text_index_entry_span {
private:
    OdfWriter& xml;
public:
    group_text_index_entry_span(OdfWriter& x) :xml(x) {
    }
    text_index_entry_span add_text_index_entry_span();
};
class group_text_index_entry_text {
private:
    OdfWriter& xml;
public:
    group_text_index_entry_text(OdfWriter& x) :xml(x) {
    }
    text_index_entry_text add_text_index_entry_text();
};
class group_text_index_scope_attr {
private:
    OdfWriter& xml;
public:
    group_text_index_scope_attr(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_text_index_scope(const T& value) {
        xml.addAttribute("text:index-scope", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_text_index_title {
private:
    OdfWriter& xml;
public:
    group_text_index_title(OdfWriter& x) :xml(x) {
    }
    text_index_title add_text_index_title(const QString& text_name);
};
class group_text_list {
private:
    OdfWriter& xml;
public:
    group_text_list(OdfWriter& x) :xml(x) {
    }
    text_list add_text_list();
};
class group_text_list_level_style_attr {
private:
    OdfWriter& xml;
public:
    group_text_list_level_style_attr(OdfWriter& x, quint64 text_level) :xml(x) {
        xml.addAttribute("text:level", text_level);
    }
    group_text_list_level_style_attr(OdfWriter& x) :xml(x) {}
    void set_text_level(quint64 value) {
        xml.addAttribute("text:level", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_text_note_class {
private:
    OdfWriter& xml;
public:
    group_text_note_class(OdfWriter& x, const QString& text_note_class) :xml(x) {
        xml.addAttribute("text:note-class", text_note_class);
    }
    group_text_note_class(OdfWriter& x) :xml(x) {}
    template<class T>
    void set_text_note_class(const T& value) {
        xml.addAttribute("text:note-class", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_text_number {
private:
    OdfWriter& xml;
public:
    group_text_number(OdfWriter& x) :xml(x) {
    }
    text_number add_text_number();
};
class group_text_outline_level {
private:
    OdfWriter& xml;
public:
    group_text_outline_level(OdfWriter& x) :xml(x) {
    }
    void set_text_outline_level(quint64 value) {
        xml.addAttribute("text:outline-level", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_text_relative_tab_stop_position_attr {
private:
    OdfWriter& xml;
public:
    group_text_relative_tab_stop_position_attr(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_text_relative_tab_stop_position(const T& value) {
        xml.addAttribute("text:relative-tab-stop-position", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_text_soft_page_break {
private:
    OdfWriter& xml;
public:
    group_text_soft_page_break(OdfWriter& x) :xml(x) {
    }
    text_soft_page_break add_text_soft_page_break();
};
class group_text_table_of_content {
private:
    OdfWriter& xml;
public:
    group_text_table_of_content(OdfWriter& x) :xml(x) {
    }
    text_table_of_content add_text_table_of_content(const QString& text_name);
};
class group_text_user_index {
private:
    OdfWriter& xml;
public:
    group_text_user_index(OdfWriter& x) :xml(x) {
    }
    text_user_index add_text_user_index(const QString& text_name);
};
class group_common_anim_target_attlist {
private:
    OdfWriter& xml;
public:
    group_common_anim_target_attlist(OdfWriter& x) :xml(x) {
    }
    void set_anim_sub_item(const QString& value) {
        xml.addAttribute("anim:sub-item", value);
    }
    void set_smil_targetElement(const QString& value) {
        xml.addAttribute("smil:targetElement", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_auto_reorder_attlist {
private:
    OdfWriter& xml;
public:
    group_common_auto_reorder_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_number_automatic_order(const T& value) {
        xml.addAttribute("number:automatic-order", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_border_line_width_attlist {
private:
    OdfWriter& xml;
public:
    group_common_border_line_width_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_style_border_line_width(const T& value) {
        xml.addAttribute("style:border-line-width", value);
    }
    template<class T>
    void set_style_border_line_width_bottom(const T& value) {
        xml.addAttribute("style:border-line-width-bottom", value);
    }
    template<class T>
    void set_style_border_line_width_left(const T& value) {
        xml.addAttribute("style:border-line-width-left", value);
    }
    template<class T>
    void set_style_border_line_width_right(const T& value) {
        xml.addAttribute("style:border-line-width-right", value);
    }
    template<class T>
    void set_style_border_line_width_top(const T& value) {
        xml.addAttribute("style:border-line-width-top", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_convert_empty_attlist {
private:
    OdfWriter& xml;
public:
    group_common_convert_empty_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_form_convert_empty_to_null(const T& value) {
        xml.addAttribute("form:convert-empty-to-null", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_db_table_name_attlist {
private:
    OdfWriter& xml;
public:
    group_common_db_table_name_attlist(OdfWriter& x, const QString& db_name) :xml(x) {
        xml.addAttribute("db:name", db_name);
    }
    group_common_db_table_name_attlist(OdfWriter& x) :xml(x) {}
    void set_db_catalog_name(const QString& value) {
        xml.addAttribute("db:catalog-name", value);
    }
    void set_db_name(const QString& value) {
        xml.addAttribute("db:name", value);
    }
    void set_db_schema_name(const QString& value) {
        xml.addAttribute("db:schema-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_delay_for_repeat {
private:
    OdfWriter& xml;
public:
    group_common_delay_for_repeat(OdfWriter& x) :xml(x) {
    }
    void set_form_delay_for_repeat(Duration value) {
        xml.addAttribute("form:delay-for-repeat", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_caption_id_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_caption_id_attlist(OdfWriter& x) :xml(x) {
    }
    void set_draw_caption_id(const QString& value) {
        xml.addAttribute("draw:caption-id", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_gradient_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_gradient_attlist(OdfWriter& x, const QString& draw_style) :xml(x) {
        xml.addAttribute("draw:style", draw_style);
    }
    group_common_draw_gradient_attlist(OdfWriter& x) :xml(x) {}
    void set_draw_angle(const QString& value) {
        xml.addAttribute("draw:angle", value);
    }
    void set_draw_border(const QString& value) {
        xml.addAttribute("draw:border", value);
    }
    void set_draw_cx(const QString& value) {
        xml.addAttribute("draw:cx", value);
    }
    void set_draw_cy(const QString& value) {
        xml.addAttribute("draw:cy", value);
    }
    void set_draw_display_name(const QString& value) {
        xml.addAttribute("draw:display-name", value);
    }
    void set_draw_name(const QString& value) {
        xml.addAttribute("draw:name", value);
    }
    template<class T>
    void set_draw_style(const T& value) {
        xml.addAttribute("draw:style", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_layer_name_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_layer_name_attlist(OdfWriter& x) :xml(x) {
    }
    void set_draw_layer(const QString& value) {
        xml.addAttribute("draw:layer", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_position_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_position_attlist(OdfWriter& x) :xml(x) {
    }
    void set_svg_x(const QString& value) {
        xml.addAttribute("svg:x", value);
    }
    void set_svg_y(const QString& value) {
        xml.addAttribute("svg:y", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_size_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_size_attlist(OdfWriter& x) :xml(x) {
    }
    void set_svg_height(const QString& value) {
        xml.addAttribute("svg:height", value);
    }
    void set_svg_width(const QString& value) {
        xml.addAttribute("svg:width", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_editable_attlist {
private:
    OdfWriter& xml;
public:
    group_common_editable_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_style_editable(const T& value) {
        xml.addAttribute("style:editable", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_field_display_value_none_attlist {
private:
    OdfWriter& xml;
public:
    group_common_field_display_value_none_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_text_display(const T& value) {
        xml.addAttribute("text:display", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_form_visual_effect_attlist {
private:
    OdfWriter& xml;
public:
    group_common_form_visual_effect_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_form_visual_effect(const T& value) {
        xml.addAttribute("form:visual-effect", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_keep_with_next_attlist {
private:
    OdfWriter& xml;
public:
    group_common_keep_with_next_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_fo_keep_with_next(const T& value) {
        xml.addAttribute("fo:keep-with-next", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_num_format_attlist {
private:
    OdfWriter& xml;
public:
    group_common_num_format_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_style_num_format(const T& value) {
        xml.addAttribute("style:num-format", value);
    }
    template<class T>
    void set_style_num_letter_sync(const T& value) {
        xml.addAttribute("style:num-letter-sync", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_page_number_attlist {
private:
    OdfWriter& xml;
public:
    group_common_page_number_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_style_page_number(const T& value) {
        xml.addAttribute("style:page-number", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_readonly_attlist {
private:
    OdfWriter& xml;
public:
    group_common_readonly_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_form_readonly(const T& value) {
        xml.addAttribute("form:readonly", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_spline_anim_value_attlist {
private:
    OdfWriter& xml;
public:
    group_common_spline_anim_value_attlist(OdfWriter& x) :xml(x) {
    }
    void set_smil_keySplines(const QString& value) {
        xml.addAttribute("smil:keySplines", value);
    }
    void set_smil_keyTimes(const QString& value) {
        xml.addAttribute("smil:keyTimes", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_svg_font_face_xlink_attlist {
private:
    OdfWriter& xml;
public:
    group_common_svg_font_face_xlink_attlist(OdfWriter& x, const QUrl& xlink_href) :xml(x) {
        xml.addAttribute("xlink:href", xlink_href);
        xml.addAttribute("xlink:type", "simple");
    }
    group_common_svg_font_face_xlink_attlist(OdfWriter& x) :xml(x) {}
    template<class T>
    void set_xlink_actuate(const T& value) {
        xml.addAttribute("xlink:actuate", value);
    }
    void set_xlink_href(const QUrl& value) {
        xml.addAttribute("xlink:href", value);
    }
    template<class T>
    void set_xlink_type(const T& value) {
        xml.addAttribute("xlink:type", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_table_change_attlist {
private:
    OdfWriter& xml;
public:
    group_common_table_change_attlist(OdfWriter& x, const QString& table_id) :xml(x) {
        xml.addAttribute("table:id", table_id);
    }
    group_common_table_change_attlist(OdfWriter& x) :xml(x) {}
    template<class T>
    void set_table_acceptance_state(const T& value) {
        xml.addAttribute("table:acceptance-state", value);
    }
    void set_table_id(const QString& value) {
        xml.addAttribute("table:id", value);
    }
    void set_table_rejecting_change_id(const QString& value) {
        xml.addAttribute("table:rejecting-change-id", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_text_anchor_attlist {
private:
    OdfWriter& xml;
public:
    group_common_text_anchor_attlist(OdfWriter& x) :xml(x) {
    }
    void set_calligra_anchor_type(const QString& value) {
        xml.addAttribute("calligra:anchor-type", value);
    }
    void set_text_anchor_page_number(quint64 value) {
        xml.addAttribute("text:anchor-page-number", value);
    }
    template<class T>
    void set_text_anchor_type(const T& value) {
        xml.addAttribute("text:anchor-type", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_value_type_attlist {
private:
    OdfWriter& xml;
public:
    group_common_value_type_attlist(OdfWriter& x, const QString& office_value_type) :xml(x) {
        xml.addAttribute("office:value-type", office_value_type);
    }
    group_common_value_type_attlist(OdfWriter& x) :xml(x) {}
    template<class T>
    void set_office_value_type(const T& value) {
        xml.addAttribute("office:value-type", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_writing_mode_attlist {
private:
    OdfWriter& xml;
public:
    group_common_writing_mode_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_style_writing_mode(const T& value) {
        xml.addAttribute("style:writing-mode", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_current_selected {
private:
    OdfWriter& xml;
public:
    group_current_selected(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_form_current_selected(const T& value) {
        xml.addAttribute("form:current-selected", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_db_apply_command {
private:
    OdfWriter& xml;
public:
    group_db_apply_command(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_db_apply_command(const T& value) {
        xml.addAttribute("db:apply-command", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_db_component {
private:
    OdfWriter& xml;
public:
    group_db_component(OdfWriter& x) :xml(x) {
    }
    db_component add_db_component(const QString& db_name);
};
class group_db_is_first_row_header_line {
private:
    OdfWriter& xml;
public:
    group_db_is_first_row_header_line(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_db_is_first_row_header_line(const T& value) {
        xml.addAttribute("db:is-first-row-header-line", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_db_show_deleted {
private:
    OdfWriter& xml;
public:
    group_db_show_deleted(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_db_show_deleted(const T& value) {
        xml.addAttribute("db:show-deleted", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_dr3d_scene_attlist {
private:
    OdfWriter& xml;
public:
    group_dr3d_scene_attlist(OdfWriter& x) :xml(x) {
    }
    void set_dr3d_ambient_color(const QString& value) {
        xml.addAttribute("dr3d:ambient-color", value);
    }
    void set_dr3d_distance(const QString& value) {
        xml.addAttribute("dr3d:distance", value);
    }
    void set_dr3d_focal_length(const QString& value) {
        xml.addAttribute("dr3d:focal-length", value);
    }
    template<class T>
    void set_dr3d_lighting_mode(const T& value) {
        xml.addAttribute("dr3d:lighting-mode", value);
    }
    template<class T>
    void set_dr3d_projection(const T& value) {
        xml.addAttribute("dr3d:projection", value);
    }
    template<class T>
    void set_dr3d_shade_mode(const T& value) {
        xml.addAttribute("dr3d:shade-mode", value);
    }
    void set_dr3d_shadow_slant(const QString& value) {
        xml.addAttribute("dr3d:shadow-slant", value);
    }
    void set_dr3d_vpn(const QString& value) {
        xml.addAttribute("dr3d:vpn", value);
    }
    void set_dr3d_vrp(const QString& value) {
        xml.addAttribute("dr3d:vrp", value);
    }
    void set_dr3d_vup(const QString& value) {
        xml.addAttribute("dr3d:vup", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_draw_layer_set {
private:
    OdfWriter& xml;
public:
    group_draw_layer_set(OdfWriter& x) :xml(x) {
    }
    draw_layer_set add_draw_layer_set();
};
class group_form_property_name {
private:
    OdfWriter& xml;
public:
    group_form_property_name(OdfWriter& x, const QString& form_property_name) :xml(x) {
        xml.addAttribute("form:property-name", form_property_name);
    }
    group_form_property_name(OdfWriter& x) :xml(x) {}
    void set_form_property_name(const QString& value) {
        xml.addAttribute("form:property-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_image_data {
private:
    OdfWriter& xml;
public:
    group_image_data(OdfWriter& x) :xml(x) {
    }
    void set_form_image_data(const QUrl& value) {
        xml.addAttribute("form:image-data", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_list_source {
private:
    OdfWriter& xml;
public:
    group_list_source(OdfWriter& x) :xml(x) {
    }
    void set_form_list_source(const QString& value) {
        xml.addAttribute("form:list-source", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_number_and_text : public group_number_number, public group_number_text {
private:
    OdfWriter& xml;
public:
    group_number_and_text(OdfWriter& x) :group_number_number(x), group_number_text(x), xml(x) {
    }
};
class group_number_seconds {
private:
    OdfWriter& xml;
public:
    group_number_seconds(OdfWriter& x) :xml(x) {
    }
    number_seconds add_number_seconds();
};
class group_office_binary_data {
private:
    OdfWriter& xml;
public:
    group_office_binary_data(OdfWriter& x) :xml(x) {
    }
    office_binary_data add_office_binary_data();
};
class group_office_document {
private:
    OdfWriter& xml;
public:
    group_office_document(OdfWriter& x) :xml(x) {
    }
    office_document add_office_document(const QString& office_mimetype);
};
class group_office_forms {
private:
    OdfWriter& xml;
public:
    group_office_forms(OdfWriter& x) :xml(x) {
    }
    office_forms add_office_forms();
};
class group_office_settings {
private:
    OdfWriter& xml;
public:
    group_office_settings(OdfWriter& x) :xml(x) {
    }
    office_settings add_office_settings();
};
class group_size {
private:
    OdfWriter& xml;
public:
    group_size(OdfWriter& x) :xml(x) {
    }
    void set_form_size(quint64 value) {
        xml.addAttribute("form:size", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_style_columns {
private:
    OdfWriter& xml;
public:
    group_style_columns(OdfWriter& x) :xml(x) {
    }
    style_columns add_style_columns(quint64 fo_column_count);
};
class group_style_map {
private:
    OdfWriter& xml;
public:
    group_style_map(OdfWriter& x) :xml(x) {
    }
    style_map add_style_map(const QString& style_apply_style_name, const QString& style_condition);
};
class group_svg_title {
private:
    OdfWriter& xml;
public:
    group_svg_title(OdfWriter& x) :xml(x) {
    }
    svg_title add_svg_title();
};
class group_table_columns_and_groups : public group_table_table_column {
private:
    OdfWriter& xml;
public:
    group_table_columns_and_groups(OdfWriter& x) :group_table_table_column(x), xml(x) {
    }
    table_table_column_group add_table_table_column_group();
    table_table_columns add_table_table_columns();
    table_table_header_columns add_table_table_header_columns();
};
class group_table_database_source_table {
private:
    OdfWriter& xml;
public:
    group_table_database_source_table(OdfWriter& x) :xml(x) {
    }
    table_database_source_query add_table_database_source_query(const QString& table_database_name, const QString& table_query_name);
};
class group_table_filter {
private:
    OdfWriter& xml;
public:
    group_table_filter(OdfWriter& x) :xml(x) {
    }
    table_filter add_table_filter();
};
class group_table_table_row {
private:
    OdfWriter& xml;
public:
    group_table_table_row(OdfWriter& x) :xml(x) {
    }
    table_table_row add_table_table_row();
};
class group_text_id {
private:
    OdfWriter& xml;
public:
    group_text_id(OdfWriter& x, const QString& text_id) :xml(x) {
        xml.addAttribute("text:id", text_id);
    }
    group_text_id(OdfWriter& x) :xml(x) {}
    void set_text_id(const QString& value) {
        xml.addAttribute("text:id", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_text_illustration_index_source_attrs : public group_text_index_scope_attr, public group_text_relative_tab_stop_position_attr {
private:
    OdfWriter& xml;
public:
    group_text_illustration_index_source_attrs(OdfWriter& x) :group_text_index_scope_attr(x), group_text_relative_tab_stop_position_attr(x), xml(x) {
    }
    template<class T>
    void set_text_caption_sequence_format(const T& value) {
        xml.addAttribute("text:caption-sequence-format", value);
    }
    void set_text_caption_sequence_name(const QString& value) {
        xml.addAttribute("text:caption-sequence-name", value);
    }
    template<class T>
    void set_text_use_caption(const T& value) {
        xml.addAttribute("text:use-caption", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_text_index_entry_page_number {
private:
    OdfWriter& xml;
public:
    group_text_index_entry_page_number(OdfWriter& x) :xml(x) {
    }
    text_index_entry_page_number add_text_index_entry_page_number();
};
class group_text_index_name {
private:
    OdfWriter& xml;
public:
    group_text_index_name(OdfWriter& x, const QString& text_index_name) :xml(x) {
        xml.addAttribute("text:index-name", text_index_name);
    }
    group_text_index_name(OdfWriter& x) :xml(x) {}
    void set_text_index_name(const QString& value) {
        xml.addAttribute("text:index-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_text_index_title_template {
private:
    OdfWriter& xml;
public:
    group_text_index_title_template(OdfWriter& x) :xml(x) {
    }
    text_index_title_template add_text_index_title_template();
};
class group_text_list_style {
private:
    OdfWriter& xml;
public:
    group_text_list_style(OdfWriter& x) :xml(x) {
    }
    text_list_style add_text_list_style(const QString& style_name);
};
class group_text_object_index {
private:
    OdfWriter& xml;
public:
    group_text_object_index(OdfWriter& x) :xml(x) {
    }
    text_object_index add_text_object_index(const QString& text_name);
};
class group_text_section {
private:
    OdfWriter& xml;
public:
    group_text_section(OdfWriter& x) :xml(x) {
    }
    text_section add_text_section(const QString& text_name);
};
class group_text_tracked_changes {
private:
    OdfWriter& xml;
public:
    group_text_tracked_changes(OdfWriter& x) :xml(x) {
    }
    text_tracked_changes add_text_tracked_changes();
};
class group_animation_element {
private:
    OdfWriter& xml;
public:
    group_animation_element(OdfWriter& x) :xml(x) {
    }
    anim_animate add_anim_animate(const QString& smil_attributeName);
    anim_animateColor add_anim_animateColor(const QString& smil_attributeName);
    anim_animateMotion add_anim_animateMotion(const QString& smil_attributeName);
    anim_animateTransform add_anim_animateTransform(const QString& smil_attributeName, const QString& svg_type);
    anim_audio add_anim_audio();
    anim_command add_anim_command(const QString& anim_command);
    anim_iterate add_anim_iterate();
    anim_par add_anim_par();
    anim_seq add_anim_seq();
    anim_set add_anim_set(const QString& smil_attributeName);
    anim_transitionFilter add_anim_transitionFilter(const QString& smil_type);
};
class group_change_mark_attr {
private:
    OdfWriter& xml;
public:
    group_change_mark_attr(OdfWriter& x, const QString& text_change_id) :xml(x) {
        xml.addAttribute("text:change-id", text_change_id);
    }
    group_change_mark_attr(OdfWriter& x) :xml(x) {}
    void set_text_change_id(const QString& value) {
        xml.addAttribute("text:change-id", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_chart_title_attlist : public group_common_draw_position_attlist {
private:
    OdfWriter& xml;
public:
    group_chart_title_attlist(OdfWriter& x) :group_common_draw_position_attlist(x), xml(x) {
    }
    void set_chart_style_name(const QString& value) {
        xml.addAttribute("chart:style-name", value);
    }
    void set_table_cell_range(const QString& value) {
        xml.addAttribute("table:cell-range", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_calendar_attlist {
private:
    OdfWriter& xml;
public:
    group_common_calendar_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_number_calendar(const T& value) {
        xml.addAttribute("number:calendar", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_dde_connection_decl_attlist {
private:
    OdfWriter& xml;
public:
    group_common_dde_connection_decl_attlist(OdfWriter& x, const QString& office_dde_application, const QString& office_dde_item, const QString& office_dde_topic) :xml(x) {
        xml.addAttribute("office:dde-application", office_dde_application);
        xml.addAttribute("office:dde-item", office_dde_item);
        xml.addAttribute("office:dde-topic", office_dde_topic);
    }
    group_common_dde_connection_decl_attlist(OdfWriter& x) :xml(x) {}
    template<class T>
    void set_office_automatic_update(const T& value) {
        xml.addAttribute("office:automatic-update", value);
    }
    void set_office_dde_application(const QString& value) {
        xml.addAttribute("office:dde-application", value);
    }
    void set_office_dde_item(const QString& value) {
        xml.addAttribute("office:dde-item", value);
    }
    void set_office_dde_topic(const QString& value) {
        xml.addAttribute("office:dde-topic", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_circle_ellipse_pos_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_circle_ellipse_pos_attlist(OdfWriter& x, const QString& svg_cx, const QString& svg_cy) :xml(x) {
        xml.addAttribute("svg:cx", svg_cx);
        xml.addAttribute("svg:cy", svg_cy);
    }
    group_common_draw_circle_ellipse_pos_attlist(OdfWriter& x) :xml(x) {}
    void set_svg_cx(const QString& value) {
        xml.addAttribute("svg:cx", value);
    }
    void set_svg_cy(const QString& value) {
        xml.addAttribute("svg:cy", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_path_data_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_path_data_attlist(OdfWriter& x, const QString& svg_d) :xml(x) {
        xml.addAttribute("svg:d", svg_d);
    }
    group_common_draw_path_data_attlist(OdfWriter& x) :xml(x) {}
    void set_svg_d(const QString& value) {
        xml.addAttribute("svg:d", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_viewbox_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_viewbox_attlist(OdfWriter& x, const QString& svg_viewBox) :xml(x) {
        xml.addAttribute("svg:viewBox", svg_viewBox);
    }
    group_common_draw_viewbox_attlist(OdfWriter& x) :xml(x) {}
    template<class T>
    void set_svg_viewBox(const T& value) {
        xml.addAttribute("svg:viewBox", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_field_formula_attlist {
private:
    OdfWriter& xml;
public:
    group_common_field_formula_attlist(OdfWriter& x) :xml(x) {
    }
    void set_text_formula(const QString& value) {
        xml.addAttribute("text:formula", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_margin_attlist {
private:
    OdfWriter& xml;
public:
    group_common_margin_attlist(OdfWriter& x) :xml(x) {
    }
    void set_fo_margin(const QString& value) {
        xml.addAttribute("fo:margin", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_presentation_header_footer_attlist {
private:
    OdfWriter& xml;
public:
    group_common_presentation_header_footer_attlist(OdfWriter& x) :xml(x) {
    }
    void set_presentation_use_date_time_name(const QString& value) {
        xml.addAttribute("presentation:use-date-time-name", value);
    }
    void set_presentation_use_footer_name(const QString& value) {
        xml.addAttribute("presentation:use-footer-name", value);
    }
    void set_presentation_use_header_name(const QString& value) {
        xml.addAttribute("presentation:use-header-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_style_direction_attlist {
private:
    OdfWriter& xml;
public:
    group_common_style_direction_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_style_direction(const T& value) {
        xml.addAttribute("style:direction", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_table_template_attlist {
private:
    OdfWriter& xml;
public:
    group_common_table_template_attlist(OdfWriter& x, const QString& table_style_name) :xml(x) {
        xml.addAttribute("table:style-name", table_style_name);
    }
    group_common_table_template_attlist(OdfWriter& x) :xml(x) {}
    void set_table_paragraph_style_name(const QString& value) {
        xml.addAttribute("table:paragraph-style-name", value);
    }
    void set_table_style_name(const QString& value) {
        xml.addAttribute("table:style-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_vertical_pos_attlist {
private:
    OdfWriter& xml;
public:
    group_common_vertical_pos_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_style_vertical_pos(const T& value) {
        xml.addAttribute("style:vertical-pos", value);
    }
    void set_svg_y(const QString& value) {
        xml.addAttribute("svg:y", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_db_delimiter {
private:
    OdfWriter& xml;
public:
    group_db_delimiter(OdfWriter& x) :xml(x) {
    }
    db_delimiter add_db_delimiter();
};
class group_dc_creator {
private:
    OdfWriter& xml;
public:
    group_dc_creator(OdfWriter& x) :xml(x) {
    }
    dc_creator add_dc_creator();
};
class group_draw_frame {
private:
    OdfWriter& xml;
public:
    group_draw_frame(OdfWriter& x) :xml(x) {
    }
    draw_frame add_draw_frame();
};
class group_office_annotation {
private:
    OdfWriter& xml;
public:
    group_office_annotation(OdfWriter& x) :xml(x) {
    }
    office_annotation add_office_annotation();
};
class group_office_event_listeners {
private:
    OdfWriter& xml;
public:
    group_office_event_listeners(OdfWriter& x) :xml(x) {
    }
    office_event_listeners add_office_event_listeners();
};
class group_presentation_shape_attlist {
private:
    OdfWriter& xml;
public:
    group_presentation_shape_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_presentation_class(const T& value) {
        xml.addAttribute("presentation:class", value);
    }
    template<class T>
    void set_presentation_placeholder(const T& value) {
        xml.addAttribute("presentation:placeholder", value);
    }
    template<class T>
    void set_presentation_user_transformed(const T& value) {
        xml.addAttribute("presentation:user-transformed", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_shape_instance : public group_dr3d_scene, public group_draw_frame {
private:
    OdfWriter& xml;
public:
    group_shape_instance(OdfWriter& x) :group_dr3d_scene(x), group_draw_frame(x), xml(x) {
    }
    draw_caption add_draw_caption();
    draw_circle add_draw_circle();
    draw_connector add_draw_connector(const QString& svg_viewBox);
    draw_control add_draw_control(const QString& draw_control);
    draw_custom_shape add_draw_custom_shape();
    draw_ellipse add_draw_ellipse();
    draw_g add_draw_g();
    draw_line add_draw_line(const QString& svg_x1, const QString& svg_x2, const QString& svg_y1, const QString& svg_y2);
    draw_measure add_draw_measure(const QString& svg_x1, const QString& svg_x2, const QString& svg_y1, const QString& svg_y2);
    draw_page_thumbnail add_draw_page_thumbnail();
    draw_path add_draw_path(const QString& svg_d, const QString& svg_viewBox);
    draw_polygon add_draw_polygon(const QString& draw_points, const QString& svg_viewBox);
    draw_polyline add_draw_polyline(const QString& draw_points, const QString& svg_viewBox);
    draw_rect add_draw_rect();
    draw_regular_polygon add_draw_regular_polygon(quint64 draw_corners);
};
class group_style_header_footer_properties {
private:
    OdfWriter& xml;
public:
    group_style_header_footer_properties(OdfWriter& x) :xml(x) {
    }
    style_header_footer_properties add_style_header_footer_properties();
};
class group_svg_desc {
private:
    OdfWriter& xml;
public:
    group_svg_desc(OdfWriter& x) :xml(x) {
    }
    svg_desc add_svg_desc();
};
class group_table_database_source_query {
private:
    OdfWriter& xml;
public:
    group_table_database_source_query(OdfWriter& x) :xml(x) {
    }
    table_database_source_table add_table_database_source_table(const QString& table_database_name, const QString& table_database_table_name);
};
class group_table_filter_condition {
private:
    OdfWriter& xml;
public:
    group_table_filter_condition(OdfWriter& x) :xml(x) {
    }
    table_filter_condition add_table_filter_condition(quint64 table_field_number, const QString& table_operator, const QString& table_value);
};
class group_table_named_expressions {
private:
    OdfWriter& xml;
public:
    group_table_named_expressions(OdfWriter& x) :xml(x) {
    }
    table_named_expressions add_table_named_expressions();
};
class group_table_table {
private:
    OdfWriter& xml;
public:
    group_table_table(OdfWriter& x) :xml(x) {
    }
    table_table add_table_table();
};
class group_text_common_ref_content {
private:
    OdfWriter& xml;
public:
    group_text_common_ref_content(OdfWriter& x) :xml(x) {
    }
    void set_text_ref_name(const QString& value) {
        xml.addAttribute("text:ref-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_text_decls {
private:
    OdfWriter& xml;
public:
    group_text_decls(OdfWriter& x) :xml(x) {
    }
    text_alphabetical_index_auto_mark_file add_text_alphabetical_index_auto_mark_file(const QUrl& xlink_href);
    text_dde_connection_decls add_text_dde_connection_decls();
    text_sequence_decls add_text_sequence_decls();
    text_user_field_decls add_text_user_field_decls();
    text_variable_decls add_text_variable_decls();
};
class group_text_index_body {
private:
    OdfWriter& xml;
public:
    group_text_index_body(OdfWriter& x) :xml(x) {
    }
    text_index_body add_text_index_body();
};
class group_text_index_source_styles {
private:
    OdfWriter& xml;
public:
    group_text_index_source_styles(OdfWriter& x) :xml(x) {
    }
    text_index_source_styles add_text_index_source_styles(quint64 text_outline_level);
};
class group_text_notes_configuration {
private:
    OdfWriter& xml;
public:
    group_text_notes_configuration(OdfWriter& x) :xml(x) {
    }
    text_notes_configuration add_text_notes_configuration(const QString& text_note_class);
};
class group_text_table_index {
private:
    OdfWriter& xml;
public:
    group_text_table_index(OdfWriter& x) :xml(x) {
    }
    text_table_index add_text_table_index(const QString& text_name);
};
class group_xml_id {
private:
    OdfWriter& xml;
public:
    group_xml_id(OdfWriter& x, const QString& xml_id) :xml(x) {
        xml.addAttribute("xml:id", xml_id);
    }
    group_xml_id(OdfWriter& x) :xml(x) {}
    void set_xml_id(const QString& value) {
        xml.addAttribute("xml:id", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_button_type {
private:
    OdfWriter& xml;
public:
    group_button_type(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_form_button_type(const T& value) {
        xml.addAttribute("form:button-type", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_column_controls {
private:
    OdfWriter& xml;
public:
    group_column_controls(OdfWriter& x) :xml(x) {
    }
    form_checkbox add_form_checkbox(const QString& xml_id);
    form_combobox add_form_combobox(const QString& xml_id);
    form_date add_form_date(const QString& xml_id);
    form_formatted_text add_form_formatted_text(const QString& xml_id);
    form_listbox add_form_listbox(const QString& xml_id);
    form_number add_form_number(const QString& xml_id);
    form_text add_form_text(const QString& xml_id);
    form_textarea add_form_textarea(const QString& xml_id);
    form_time add_form_time(const QString& xml_id);
};
class group_common_anim_set_values_attlist {
private:
    OdfWriter& xml;
public:
    group_common_anim_set_values_attlist(OdfWriter& x) :xml(x) {
    }
    void set_smil_to(const QString& value) {
        xml.addAttribute("smil:to", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_background_transparency_attlist {
private:
    OdfWriter& xml;
public:
    group_common_background_transparency_attlist(OdfWriter& x) :xml(x) {
    }
    void set_style_background_transparency(const QString& value) {
        xml.addAttribute("style:background-transparency", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_begin_end_timing_attlist {
private:
    OdfWriter& xml;
public:
    group_common_begin_end_timing_attlist(OdfWriter& x) :xml(x) {
    }
    void set_smil_begin(const QString& value) {
        xml.addAttribute("smil:begin", value);
    }
    void set_smil_end(const QString& value) {
        xml.addAttribute("smil:end", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_dr3d_transform_attlist {
private:
    OdfWriter& xml;
public:
    group_common_dr3d_transform_attlist(OdfWriter& x) :xml(x) {
    }
    void set_dr3d_transform(const QString& value) {
        xml.addAttribute("dr3d:transform", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_rel_size_attlist : public group_common_draw_size_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_rel_size_attlist(OdfWriter& x) :group_common_draw_size_attlist(x), xml(x) {
    }
    template<class T>
    void set_style_rel_height(const T& value) {
        xml.addAttribute("style:rel-height", value);
    }
    template<class T>
    void set_style_rel_width(const T& value) {
        xml.addAttribute("style:rel-width", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_field_num_format_attlist : public group_common_num_format_attlist {
private:
    OdfWriter& xml;
public:
    group_common_field_num_format_attlist(OdfWriter& x) :group_common_num_format_attlist(x), xml(x) {
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_horizontal_margin_attlist {
private:
    OdfWriter& xml;
public:
    group_common_horizontal_margin_attlist(OdfWriter& x) :xml(x) {
    }
    void set_fo_margin_left(const QString& value) {
        xml.addAttribute("fo:margin-left", value);
    }
    void set_fo_margin_right(const QString& value) {
        xml.addAttribute("fo:margin-right", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_repeat {
private:
    OdfWriter& xml;
public:
    group_common_repeat(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_form_repeat(const T& value) {
        xml.addAttribute("form:repeat", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_source_cell_range {
private:
    OdfWriter& xml;
public:
    group_common_source_cell_range(OdfWriter& x) :xml(x) {
    }
    void set_form_source_cell_range(const QString& value) {
        xml.addAttribute("form:source-cell-range", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_text_spreadsheet_shape_attlist : public group_common_text_anchor_attlist {
private:
    OdfWriter& xml;
public:
    group_common_text_spreadsheet_shape_attlist(OdfWriter& x) :group_common_text_anchor_attlist(x), xml(x) {
    }
    void set_table_end_cell_address(const QString& value) {
        xml.addAttribute("table:end-cell-address", value);
    }
    void set_table_end_x(const QString& value) {
        xml.addAttribute("table:end-x", value);
    }
    void set_table_end_y(const QString& value) {
        xml.addAttribute("table:end-y", value);
    }
    template<class T>
    void set_table_table_background(const T& value) {
        xml.addAttribute("table:table-background", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_value_and_type_attlist {
private:
    OdfWriter& xml;
public:
    group_common_value_and_type_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_office_boolean_value(const T& value) {
        xml.addAttribute("office:boolean-value", value);
    }
    void set_office_currency(const QString& value) {
        xml.addAttribute("office:currency", value);
    }
    template<class T>
    void set_office_date_value(const T& value) {
        xml.addAttribute("office:date-value", value);
    }
    void set_office_string_value(const QString& value) {
        xml.addAttribute("office:string-value", value);
    }
    void set_office_time_value(Duration value) {
        xml.addAttribute("office:time-value", value);
    }
    void set_office_value(double value) {
        xml.addAttribute("office:value", value);
    }
    template<class T>
    void set_office_value_type(const T& value) {
        xml.addAttribute("office:value-type", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_db_columns {
private:
    OdfWriter& xml;
public:
    group_db_columns(OdfWriter& x) :xml(x) {
    }
    db_columns add_db_columns();
};
class group_dr3d_light {
private:
    OdfWriter& xml;
public:
    group_dr3d_light(OdfWriter& x) :xml(x) {
    }
    dr3d_light add_dr3d_light(const QString& dr3d_direction);
};
class group_form_connection_resource {
private:
    OdfWriter& xml;
public:
    group_form_connection_resource(OdfWriter& x) :xml(x) {
    }
    form_connection_resource add_form_connection_resource(const QUrl& xlink_href);
};
class group_form_form {
private:
    OdfWriter& xml;
public:
    group_form_form(OdfWriter& x) :xml(x) {
    }
    form_form add_form_form();
};
class group_number_minutes {
private:
    OdfWriter& xml;
public:
    group_number_minutes(OdfWriter& x) :xml(x) {
    }
    number_minutes add_number_minutes();
};
class group_office_meta {
private:
    OdfWriter& xml;
public:
    group_office_meta(OdfWriter& x) :xml(x) {
    }
    office_meta add_office_meta();
};
class group_styles : public group_text_list_style {
private:
    OdfWriter& xml;
public:
    group_styles(OdfWriter& x) :group_text_list_style(x), xml(x) {
    }
    number_boolean_style add_number_boolean_style(const QString& style_name);
    number_currency_style add_number_currency_style(const QString& style_name);
    number_date_style add_number_date_style(const QString& style_name);
    number_number_style add_number_number_style(const QString& style_name);
    number_percentage_style add_number_percentage_style(const QString& style_name);
    number_text_style add_number_text_style(const QString& style_name);
    number_time_style add_number_time_style(const QString& style_name);
    style_style add_style_style(const QString& style_name);
};
class group_table_deletions {
private:
    OdfWriter& xml;
public:
    group_table_deletions(OdfWriter& x) :xml(x) {
    }
    table_deletions add_table_deletions();
};
class group_table_rows_and_groups : public group_table_table_row, public group_text_soft_page_break {
private:
    OdfWriter& xml;
public:
    group_table_rows_and_groups(OdfWriter& x) :group_table_table_row(x), group_text_soft_page_break(x), xml(x) {
    }
    table_table_header_rows add_table_table_header_rows();
    table_table_row_group add_table_table_row_group();
    table_table_rows add_table_table_rows();
};
class group_text_index_entry_tab_stop {
private:
    OdfWriter& xml;
public:
    group_text_index_entry_tab_stop(OdfWriter& x) :xml(x) {
    }
    text_index_entry_tab_stop add_text_index_entry_tab_stop();
};
class group_text_p {
private:
    OdfWriter& xml;
public:
    group_text_p(OdfWriter& x) :xml(x) {
    }
    text_p add_text_p();
};
class group_common_anim_values_attlist : public group_common_anim_set_values_attlist {
private:
    OdfWriter& xml;
public:
    group_common_anim_values_attlist(OdfWriter& x) :group_common_anim_set_values_attlist(x), xml(x) {
    }
    void set_anim_formula(const QString& value) {
        xml.addAttribute("anim:formula", value);
    }
    void set_smil_by(const QString& value) {
        xml.addAttribute("smil:by", value);
    }
    void set_smil_from(const QString& value) {
        xml.addAttribute("smil:from", value);
    }
    void set_smil_values(const QString& value) {
        xml.addAttribute("smil:values", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_data_field_attlist {
private:
    OdfWriter& xml;
public:
    group_common_data_field_attlist(OdfWriter& x) :xml(x) {
    }
    void set_form_data_field(const QString& value) {
        xml.addAttribute("form:data-field", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_db_object_name {
private:
    OdfWriter& xml;
public:
    group_common_db_object_name(OdfWriter& x, const QString& db_name) :xml(x) {
        xml.addAttribute("db:name", db_name);
    }
    group_common_db_object_name(OdfWriter& x) :xml(x) {}
    void set_db_name(const QString& value) {
        xml.addAttribute("db:name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_field_data_style_name_attlist {
private:
    OdfWriter& xml;
public:
    group_common_field_data_style_name_attlist(OdfWriter& x) :xml(x) {
    }
    void set_style_data_style_name(const QString& value) {
        xml.addAttribute("style:data-style-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_field_description_attlist {
private:
    OdfWriter& xml;
public:
    group_common_field_description_attlist(OdfWriter& x) :xml(x) {
    }
    void set_text_description(const QString& value) {
        xml.addAttribute("text:description", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_number_attlist {
private:
    OdfWriter& xml;
public:
    group_common_number_attlist(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_number_grouping(const T& value) {
        xml.addAttribute("number:grouping", value);
    }
    void set_number_min_integer_digits(qint64 value) {
        xml.addAttribute("number:min-integer-digits", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_office_annotation_name_attlist {
private:
    OdfWriter& xml;
public:
    group_common_office_annotation_name_attlist(OdfWriter& x, const QString& office_name) :xml(x) {
        xml.addAttribute("office:name", office_name);
    }
    group_common_office_annotation_name_attlist(OdfWriter& x) :xml(x) {}
    void set_office_name(const QString& value) {
        xml.addAttribute("office:name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_tab_attlist {
private:
    OdfWriter& xml;
public:
    group_common_tab_attlist(OdfWriter& x) :xml(x) {
    }
    void set_form_tab_index(quint64 value) {
        xml.addAttribute("form:tab-index", value);
    }
    template<class T>
    void set_form_tab_stop(const T& value) {
        xml.addAttribute("form:tab-stop", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_config_config_item_set {
private:
    OdfWriter& xml;
public:
    group_config_config_item_set(OdfWriter& x) :xml(x) {
    }
    config_config_item_set add_config_config_item_set(const QString& config_name);
};
class group_draw_param {
private:
    OdfWriter& xml;
public:
    group_draw_param(OdfWriter& x) :xml(x) {
    }
    draw_param add_draw_param();
};
class group_dropdown {
private:
    OdfWriter& xml;
public:
    group_dropdown(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_form_dropdown(const T& value) {
        xml.addAttribute("form:dropdown", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_header_footer_content : public group_change_marks, public group_table_table, public group_text_alphabetical_index, public group_text_bibliography, public group_text_decls, public group_text_h, public group_text_illustration_index, public group_text_index_title, public group_text_list, public group_text_object_index, public group_text_p, public group_text_section, public group_text_table_index, public group_text_table_of_content, public group_text_tracked_changes, public group_text_user_index {
private:
    OdfWriter& xml;
public:
    group_header_footer_content(OdfWriter& x) :group_change_marks(x), group_table_table(x), group_text_alphabetical_index(x), group_text_bibliography(x), group_text_decls(x), group_text_h(x), group_text_illustration_index(x), group_text_index_title(x), group_text_list(x), group_text_object_index(x), group_text_p(x), group_text_section(x), group_text_table_index(x), group_text_table_of_content(x), group_text_tracked_changes(x), group_text_user_index(x), xml(x) {
    }
    style_region_center add_style_region_center();
    style_region_left add_style_region_left();
    style_region_right add_style_region_right();
};
class group_math_math {
private:
    OdfWriter& xml;
public:
    group_math_math(OdfWriter& x) :xml(x) {
    }
    math_math add_math_math();
};
class group_paragraph_attrs : public group_common_in_content_meta_attlist, public group_xml_id {
private:
    OdfWriter& xml;
public:
    group_paragraph_attrs(OdfWriter& x) :group_common_in_content_meta_attlist(x), group_xml_id(x), xml(x) {
    }
    template<class T>
    void set_text_class_names(const T& value) {
        xml.addAttribute("text:class-names", value);
    }
    void set_text_cond_style_name(const QString& value) {
        xml.addAttribute("text:cond-style-name", value);
    }
    void set_text_id(const QString& value) {
        xml.addAttribute("text:id", value);
    }
    void set_text_style_name(const QString& value) {
        xml.addAttribute("text:style-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_region_content : public group_text_p {
private:
    OdfWriter& xml;
public:
    group_region_content(OdfWriter& x) :group_text_p(x), xml(x) {
    }
};
class group_style_style_content : public group_style_text_properties {
private:
    OdfWriter& xml;
public:
    group_style_style_content(OdfWriter& x) :group_style_text_properties(x), xml(x) {
    }
    style_chart_properties add_style_chart_properties();
    style_drawing_page_properties add_style_drawing_page_properties();
    style_graphic_properties add_style_graphic_properties();
    style_paragraph_properties add_style_paragraph_properties();
    style_ruby_properties add_style_ruby_properties();
    style_section_properties add_style_section_properties();
    style_table_cell_properties add_style_table_cell_properties();
    style_table_column_properties add_style_table_column_properties();
    style_table_properties add_style_table_properties();
    style_table_row_properties add_style_table_row_properties();
    template<class T>
    void set_style_family(const T& value) {
        xml.addAttribute("style:family", value);
    }
};
class group_table_functions : public group_table_named_expressions {
private:
    OdfWriter& xml;
public:
    group_table_functions(OdfWriter& x) :group_table_named_expressions(x), xml(x) {
    }
    table_consolidation add_table_consolidation(const QString& table_function, const QString& table_source_cell_range_addresses, const QString& table_target_cell_address);
    table_data_pilot_tables add_table_data_pilot_tables();
    table_database_ranges add_table_database_ranges();
    table_dde_links add_table_dde_links();
};
class group_text_alphabetical_index_mark_attrs {
private:
    OdfWriter& xml;
public:
    group_text_alphabetical_index_mark_attrs(OdfWriter& x) :xml(x) {
    }
    void set_text_key1(const QString& value) {
        xml.addAttribute("text:key1", value);
    }
    void set_text_key1_phonetic(const QString& value) {
        xml.addAttribute("text:key1-phonetic", value);
    }
    void set_text_key2(const QString& value) {
        xml.addAttribute("text:key2", value);
    }
    void set_text_key2_phonetic(const QString& value) {
        xml.addAttribute("text:key2-phonetic", value);
    }
    template<class T>
    void set_text_main_entry(const T& value) {
        xml.addAttribute("text:main-entry", value);
    }
    void set_text_string_value_phonetic(const QString& value) {
        xml.addAttribute("text:string-value-phonetic", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_text_illustration_index_entry_content : public group_text_index_entry_chapter, public group_text_index_entry_page_number, public group_text_index_entry_span, public group_text_index_entry_tab_stop, public group_text_index_entry_text {
private:
    OdfWriter& xml;
public:
    group_text_illustration_index_entry_content(OdfWriter& x, const QString& text_style_name) :group_text_index_entry_chapter(x), group_text_index_entry_page_number(x), group_text_index_entry_span(x), group_text_index_entry_tab_stop(x), group_text_index_entry_text(x), xml(x) {
        xml.addAttribute("text:style-name", text_style_name);
    }
    group_text_illustration_index_entry_content(OdfWriter& x) :group_text_index_entry_chapter(x), group_text_index_entry_page_number(x), group_text_index_entry_span(x), group_text_index_entry_tab_stop(x), group_text_index_entry_text(x), xml(x) {}
    void set_text_style_name(const QString& value) {
        xml.addAttribute("text:style-name", value);
    }
};
class group_common_anim_attlist : public group_xml_id {
private:
    OdfWriter& xml;
public:
    group_common_anim_attlist(OdfWriter& x) :group_xml_id(x), xml(x) {
    }
    void set_anim_id(const QString& value) {
        xml.addAttribute("anim:id", value);
    }
    void set_presentation_group_id(const QString& value) {
        xml.addAttribute("presentation:group-id", value);
    }
    void set_presentation_master_element(const QString& value) {
        xml.addAttribute("presentation:master-element", value);
    }
    template<class T>
    void set_presentation_node_type(const T& value) {
        xml.addAttribute("presentation:node-type", value);
    }
    template<class T>
    void set_presentation_preset_class(const T& value) {
        xml.addAttribute("presentation:preset-class", value);
    }
    void set_presentation_preset_id(const QString& value) {
        xml.addAttribute("presentation:preset-id", value);
    }
    void set_presentation_preset_sub_type(const QString& value) {
        xml.addAttribute("presentation:preset-sub-type", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_db_default_value : public group_common_value_and_type_attlist {
private:
    OdfWriter& xml;
public:
    group_common_db_default_value(OdfWriter& x) :group_common_value_and_type_attlist(x), xml(x) {
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_field_database_table : public group_form_connection_resource {
private:
    OdfWriter& xml;
public:
    group_common_field_database_table(OdfWriter& x, const QString& text_table_name) :group_form_connection_resource(x), xml(x) {
        xml.addAttribute("text:table-name", text_table_name);
    }
    group_common_field_database_table(OdfWriter& x) :group_form_connection_resource(x), xml(x) {}
    void set_text_database_name(const QString& value) {
        xml.addAttribute("text:database-name", value);
    }
    void set_text_table_name(const QString& value) {
        xml.addAttribute("text:table-name", value);
    }
    template<class T>
    void set_text_table_type(const T& value) {
        xml.addAttribute("text:table-type", value);
    }
};
class group_db_query {
private:
    OdfWriter& xml;
public:
    group_db_query(OdfWriter& x) :xml(x) {
    }
    db_query add_db_query(const QString& db_command, const QString& db_name);
};
class group_form_control_attlist : public group_common_form_control_attlist, public group_xml_id {
private:
    OdfWriter& xml;
public:
    group_form_control_attlist(OdfWriter& x, const QString& xml_id) :group_common_form_control_attlist(x), group_xml_id(x, xml_id), xml(x) {
    }
    group_form_control_attlist(OdfWriter& x) :group_common_form_control_attlist(x), group_xml_id(x), xml(x) {}
    void set_form_id(const QString& value) {
        xml.addAttribute("form:id", value);
    }
    void set_xforms_bind(const QString& value) {
        xml.addAttribute("xforms:bind", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_shape : public group_shape_instance {
private:
    OdfWriter& xml;
public:
    group_shape(OdfWriter& x) :group_shape_instance(x), xml(x) {
    }
    draw_a add_draw_a(const QUrl& xlink_href);
};
class group_table_table_cell_attlist : public group_common_in_content_meta_attlist, public group_common_value_and_type_attlist, public group_xml_id {
private:
    OdfWriter& xml;
public:
    group_table_table_cell_attlist(OdfWriter& x) :group_common_in_content_meta_attlist(x), group_common_value_and_type_attlist(x), group_xml_id(x), xml(x) {
    }
    void set_table_content_validation_name(const QString& value) {
        xml.addAttribute("table:content-validation-name", value);
    }
    void set_table_formula(const QString& value) {
        xml.addAttribute("table:formula", value);
    }
    void set_table_number_columns_repeated(quint64 value) {
        xml.addAttribute("table:number-columns-repeated", value);
    }
    template<class T>
    void set_table_protect(const T& value) {
        xml.addAttribute("table:protect", value);
    }
    template<class T>
    void set_table_protected(const T& value) {
        xml.addAttribute("table:protected", value);
    }
    void set_table_style_name(const QString& value) {
        xml.addAttribute("table:style-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_target_frame {
private:
    OdfWriter& xml;
public:
    group_target_frame(OdfWriter& x) :xml(x) {
    }
    template<class T>
    void set_office_target_frame(const T& value) {
        xml.addAttribute("office:target-frame", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_text_list_item_content : public group_text_h, public group_text_list, public group_text_number, public group_text_p, public group_text_soft_page_break {
private:
    OdfWriter& xml;
public:
    group_text_list_item_content(OdfWriter& x) :group_text_h(x), group_text_list(x), group_text_number(x), group_text_p(x), group_text_soft_page_break(x), xml(x) {
    }
};
class group_common_basic_timing_attlist : public group_common_begin_end_timing_attlist {
private:
    OdfWriter& xml;
public:
    group_common_basic_timing_attlist(OdfWriter& x) :group_common_begin_end_timing_attlist(x), xml(x) {
    }
    void set_smil_dur(const QString& value) {
        xml.addAttribute("smil:dur", value);
    }
    template<class T>
    void set_smil_fill(const T& value) {
        xml.addAttribute("smil:fill", value);
    }
    template<class T>
    void set_smil_fillDefault(const T& value) {
        xml.addAttribute("smil:fillDefault", value);
    }
    template<class T>
    void set_smil_repeatCount(const T& value) {
        xml.addAttribute("smil:repeatCount", value);
    }
    void set_smil_repeatDur(const QString& value) {
        xml.addAttribute("smil:repeatDur", value);
    }
    template<class T>
    void set_smil_restart(const T& value) {
        xml.addAttribute("smil:restart", value);
    }
    template<class T>
    void set_smil_restartDefault(const T& value) {
        xml.addAttribute("smil:restartDefault", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_form_control_content : public group_form_properties, public group_office_event_listeners {
private:
    OdfWriter& xml;
public:
    group_common_form_control_content(OdfWriter& x) :group_form_properties(x), group_office_event_listeners(x), xml(x) {
    }
};
class group_common_section_attlist : public group_xml_id {
private:
    OdfWriter& xml;
public:
    group_common_section_attlist(OdfWriter& x, const QString& text_name) :group_xml_id(x), xml(x) {
        xml.addAttribute("text:name", text_name);
    }
    group_common_section_attlist(OdfWriter& x) :group_xml_id(x), xml(x) {}
    void set_text_name(const QString& value) {
        xml.addAttribute("text:name", value);
    }
    template<class T>
    void set_text_protected(const T& value) {
        xml.addAttribute("text:protected", value);
    }
    void set_text_protection_key(const QString& value) {
        xml.addAttribute("text:protection-key", value);
    }
    void set_text_protection_key_digest_algorithm(const QUrl& value) {
        xml.addAttribute("text:protection-key-digest-algorithm", value);
    }
    void set_text_style_name(const QString& value) {
        xml.addAttribute("text:style-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_config_items : public group_config_config_item_set {
private:
    OdfWriter& xml;
public:
    group_config_items(OdfWriter& x) :group_config_config_item_set(x), xml(x) {
    }
    config_config_item add_config_config_item(const QString& config_name, const QString& config_type);
    config_config_item_map_indexed add_config_config_item_map_indexed(const QString& config_name);
    config_config_item_map_named add_config_config_item_map_named(const QString& config_name);
};
class group_paragraph_content : public group_change_marks, public group_office_annotation, public group_shape, public group_text_soft_page_break {
private:
    OdfWriter& xml;
public:
    group_paragraph_content(OdfWriter& x) :group_change_marks(x), group_office_annotation(x), group_shape(x), group_text_soft_page_break(x), xml(x) {
    }
    office_annotation_end add_office_annotation_end(const QString& office_name);
    presentation_date_time add_presentation_date_time();
    presentation_footer add_presentation_footer();
    presentation_header add_presentation_header();
    text_alphabetical_index_mark add_text_alphabetical_index_mark(const QString& text_string_value);
    text_alphabetical_index_mark_end add_text_alphabetical_index_mark_end(const QString& text_id);
    text_alphabetical_index_mark_start add_text_alphabetical_index_mark_start(const QString& text_id);
    text_author_initials add_text_author_initials();
    text_author_name add_text_author_name();
    text_bibliography_mark add_text_bibliography_mark(const QString& text_bibliography_type);
    text_bookmark add_text_bookmark(const QString& text_name);
    text_bookmark_end add_text_bookmark_end(const QString& text_name);
    text_bookmark_ref add_text_bookmark_ref();
    text_bookmark_start add_text_bookmark_start(const QString& text_name);
    text_chapter add_text_chapter(const QString& text_display, quint64 text_outline_level);
    text_character_count add_text_character_count();
    text_conditional_text add_text_conditional_text(const QString& text_condition, const QString& text_string_value_if_false, const QString& text_string_value_if_true);
    text_creation_date add_text_creation_date();
    text_creation_time add_text_creation_time();
    text_creator add_text_creator();
    text_database_display add_text_database_display(const QString& text_column_name, const QString& text_table_name);
    text_database_name add_text_database_name(const QString& text_table_name);
    text_database_next add_text_database_next(const QString& text_table_name);
    text_database_row_number add_text_database_row_number(const QString& text_table_name);
    text_database_row_select add_text_database_row_select(const QString& text_table_name);
    text_date add_text_date();
    text_dde_connection add_text_dde_connection(const QString& text_connection_name);
    text_description add_text_description();
    text_editing_cycles add_text_editing_cycles();
    text_editing_duration add_text_editing_duration();
    text_execute_macro add_text_execute_macro();
    text_expression add_text_expression();
    text_file_name add_text_file_name();
    text_hidden_paragraph add_text_hidden_paragraph(const QString& text_condition);
    text_hidden_text add_text_hidden_text(const QString& text_condition, const QString& text_string_value);
    text_image_count add_text_image_count();
    text_initial_creator add_text_initial_creator();
    text_keywords add_text_keywords();
    text_line_break add_text_line_break();
    text_measure add_text_measure(const QString& text_kind);
    text_meta add_text_meta();
    text_meta_field add_text_meta_field(const QString& xml_id);
    text_modification_date add_text_modification_date();
    text_modification_time add_text_modification_time();
    text_note add_text_note(const QString& text_note_class);
    text_note_ref add_text_note_ref(const QString& text_note_class);
    text_object_count add_text_object_count();
    text_page_continuation add_text_page_continuation(const QString& text_select_page);
    text_page_count add_text_page_count();
    text_page_number add_text_page_number();
    text_page_variable_get add_text_page_variable_get();
    text_page_variable_set add_text_page_variable_set();
    text_paragraph_count add_text_paragraph_count();
    text_placeholder add_text_placeholder(const QString& text_placeholder_type);
    text_print_date add_text_print_date();
    text_print_time add_text_print_time();
    text_printed_by add_text_printed_by();
    text_reference_mark add_text_reference_mark(const QString& text_name);
    text_reference_mark_end add_text_reference_mark_end(const QString& text_name);
    text_reference_mark_start add_text_reference_mark_start(const QString& text_name);
    text_reference_ref add_text_reference_ref();
    text_ruby add_text_ruby();
    text_s add_text_s();
    text_script add_text_script();
    text_sender_city add_text_sender_city();
    text_sender_company add_text_sender_company();
    text_sender_country add_text_sender_country();
    text_sender_email add_text_sender_email();
    text_sender_fax add_text_sender_fax();
    text_sender_firstname add_text_sender_firstname();
    text_sender_initials add_text_sender_initials();
    text_sender_lastname add_text_sender_lastname();
    text_sender_phone_private add_text_sender_phone_private();
    text_sender_phone_work add_text_sender_phone_work();
    text_sender_position add_text_sender_position();
    text_sender_postal_code add_text_sender_postal_code();
    text_sender_state_or_province add_text_sender_state_or_province();
    text_sender_street add_text_sender_street();
    text_sender_title add_text_sender_title();
    text_sequence add_text_sequence(const QString& text_name);
    text_sequence_ref add_text_sequence_ref();
    text_sheet_name add_text_sheet_name();
    text_span add_text_span();
    text_subject add_text_subject();
    text_tab add_text_tab();
    text_table_count add_text_table_count();
    text_table_formula add_text_table_formula();
    text_template_name add_text_template_name();
    text_text_input add_text_text_input();
    text_time add_text_time();
    text_title add_text_title();
    text_toc_mark add_text_toc_mark(const QString& text_string_value);
    text_toc_mark_end add_text_toc_mark_end(const QString& text_id);
    text_toc_mark_start add_text_toc_mark_start(const QString& text_id);
    text_user_defined add_text_user_defined(const QString& text_name);
    text_user_field_get add_text_user_field_get(const QString& text_name);
    text_user_field_input add_text_user_field_input(const QString& text_name);
    text_user_index_mark add_text_user_index_mark(const QString& text_index_name, const QString& text_string_value);
    text_user_index_mark_end add_text_user_index_mark_end(const QString& text_id);
    text_user_index_mark_start add_text_user_index_mark_start(const QString& text_id, const QString& text_index_name);
    text_variable_get add_text_variable_get(const QString& text_name);
    text_variable_input add_text_variable_input(const QString& office_value_type, const QString& text_name);
    text_variable_set add_text_variable_set(const QString& text_name);
    text_word_count add_text_word_count();
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_id_attlist : public group_xml_id {
private:
    OdfWriter& xml;
public:
    group_common_draw_id_attlist(OdfWriter& x) :group_xml_id(x), xml(x) {
    }
    void set_draw_id(const QString& value) {
        xml.addAttribute("draw:id", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_timing_attlist : public group_common_basic_timing_attlist {
private:
    OdfWriter& xml;
public:
    group_common_timing_attlist(OdfWriter& x) :group_common_basic_timing_attlist(x), xml(x) {
    }
    void set_smil_accelerate(double value) {
        xml.addAttribute("smil:accelerate", value);
    }
    template<class T>
    void set_smil_autoReverse(const T& value) {
        xml.addAttribute("smil:autoReverse", value);
    }
    void set_smil_decelerate(double value) {
        xml.addAttribute("smil:decelerate", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_paragraph_content_or_hyperlink : public group_paragraph_content {
private:
    OdfWriter& xml;
public:
    group_paragraph_content_or_hyperlink(OdfWriter& x) :group_paragraph_content(x), xml(x) {
    }
    text_a add_text_a(const QUrl& xlink_href);
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_draw_shape_with_styles_attlist : public group_common_draw_id_attlist, public group_common_draw_layer_name_attlist, public group_common_draw_name_attlist, public group_common_draw_style_name_attlist, public group_common_draw_z_index_attlist, public group_common_text_spreadsheet_shape_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_shape_with_styles_attlist(OdfWriter& x) :group_common_draw_id_attlist(x), group_common_draw_layer_name_attlist(x), group_common_draw_name_attlist(x), group_common_draw_style_name_attlist(x), group_common_draw_z_index_attlist(x), group_common_text_spreadsheet_shape_attlist(x), xml(x) {
    }
    void set_draw_transform(const QString& value) {
        xml.addAttribute("draw:transform", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_common_numeric_control_attlist : public group_common_convert_empty_attlist, public group_common_data_field_attlist, public group_common_disabled_attlist, public group_common_maxlength_attlist, public group_common_printable_attlist, public group_common_readonly_attlist, public group_common_tab_attlist, public group_common_title_attlist, public group_form_control_attlist {
private:
    OdfWriter& xml;
public:
    group_common_numeric_control_attlist(OdfWriter& x, const QString& xml_id) :group_common_convert_empty_attlist(x), group_common_data_field_attlist(x), group_common_disabled_attlist(x), group_common_maxlength_attlist(x), group_common_printable_attlist(x), group_common_readonly_attlist(x), group_common_tab_attlist(x), group_common_title_attlist(x), group_form_control_attlist(x, xml_id), xml(x) {
    }
    group_common_numeric_control_attlist(OdfWriter& x) :group_common_convert_empty_attlist(x), group_common_data_field_attlist(x), group_common_disabled_attlist(x), group_common_maxlength_attlist(x), group_common_printable_attlist(x), group_common_readonly_attlist(x), group_common_tab_attlist(x), group_common_title_attlist(x), group_form_control_attlist(x), xml(x) {}
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_table_change_track_table_cell {
private:
    OdfWriter& xml;
public:
    group_table_change_track_table_cell(OdfWriter& x) :xml(x) {
    }
    table_change_track_table_cell add_table_change_track_table_cell();
};
class group_text_content : public group_change_marks, public group_shape, public group_table_table, public group_text_alphabetical_index, public group_text_bibliography, public group_text_h, public group_text_illustration_index, public group_text_list, public group_text_object_index, public group_text_p, public group_text_section, public group_text_soft_page_break, public group_text_table_index, public group_text_table_of_content, public group_text_user_index {
private:
    OdfWriter& xml;
public:
    group_text_content(OdfWriter& x) :group_change_marks(x), group_shape(x), group_table_table(x), group_text_alphabetical_index(x), group_text_bibliography(x), group_text_h(x), group_text_illustration_index(x), group_text_list(x), group_text_object_index(x), group_text_p(x), group_text_section(x), group_text_soft_page_break(x), group_text_table_index(x), group_text_table_of_content(x), group_text_user_index(x), xml(x) {
    }
    text_numbered_paragraph add_text_numbered_paragraph(const QString& text_list_id);
};
class group_common_draw_shape_with_text_and_styles_attlist : public group_common_draw_shape_with_styles_attlist {
private:
    OdfWriter& xml;
public:
    group_common_draw_shape_with_text_and_styles_attlist(OdfWriter& x) :group_common_draw_shape_with_styles_attlist(x), xml(x) {
    }
    void set_draw_text_style_name(const QString& value) {
        xml.addAttribute("draw:text-style-name", value);
    }
    void addTextNode(const QString& data) {
        xml.addTextNode(data);
    }
};
class group_index_content_main : public group_text_content, public group_text_index_title {
private:
    OdfWriter& xml;
public:
    group_index_content_main(OdfWriter& x) :group_text_content(x), group_text_index_title(x), xml(x) {
    }
};
class group_table_table_cell_content : public group_office_annotation, public group_text_content {
private:
    OdfWriter& xml;
public:
    group_table_table_cell_content(OdfWriter& x) :group_office_annotation(x), group_text_content(x), xml(x) {
    }
    table_cell_range_source add_table_cell_range_source(quint64 table_last_column_spanned, quint64 table_last_row_spanned, const QString& table_name, const QUrl& xlink_href);
    table_detective add_table_detective();
};
class group_draw_text : public group_text_list, public group_text_p {
private:
    OdfWriter& xml;
public:
    group_draw_text(OdfWriter& x) :group_text_list(x), group_text_p(x), xml(x) {
    }
};
class group_office_change_info {
private:
    OdfWriter& xml;
public:
    group_office_change_info(OdfWriter& x) :xml(x) {
    }
    office_change_info add_office_change_info();
};
}
#endif
