/****************************************************************************
** Meta object code from reading C++ file 'kptperteditor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/kptperteditor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptperteditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato10PertEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::PertEditor::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato10PertEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::PertEditor",
        "executeCommand",
        "",
        "KUndo2Command*",
        "slotNodeAdded",
        "KPlato::Node*",
        "slotNodeRemoved",
        "slotNodeMoved",
        "slotNodeChanged",
        "slotRelationAdded",
        "KPlato::Relation*",
        "rel",
        "slotRelationRemoved",
        "dispAvailableTasks",
        "parent",
        "selectedTask",
        "addTaskInRequiredList",
        "QTreeWidgetItem*",
        "currentItem",
        "removeTaskFromRequiredList",
        "slotUpdate",
        "slotCurrentTaskChanged",
        "curr",
        "prev",
        "slotAvailableChanged",
        "item",
        "slotRequiredChanged",
        "QModelIndex",
        "index",
        "slotAddClicked",
        "slotRemoveClicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'executeCommand'
        QtMocHelpers::SignalData<void(KUndo2Command *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotNodeAdded'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(4, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Slot 'slotNodeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Slot 'slotNodeMoved'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Slot 'slotNodeChanged'
        QtMocHelpers::SlotData<void(KPlato::Node *)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Slot 'slotRelationAdded'
        QtMocHelpers::SlotData<void(KPlato::Relation *)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotRelationRemoved'
        QtMocHelpers::SlotData<void(KPlato::Relation *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'dispAvailableTasks'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'dispAvailableTasks'
        QtMocHelpers::SlotData<void(KPlato::Node *, KPlato::Node *)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 14 }, { 0x80000000 | 5, 15 },
        }}),
        // Slot 'dispAvailableTasks'
        QtMocHelpers::SlotData<void(KPlato::Relation *)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'addTaskInRequiredList'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'removeTaskFromRequiredList'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUpdate'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCurrentTaskChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, QTreeWidgetItem *)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 22 }, { 0x80000000 | 17, 23 },
        }}),
        // Slot 'slotAvailableChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 25 },
        }}),
        // Slot 'slotRequiredChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'slotAddClicked'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRemoveClicked'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PertEditor, qt_meta_tag_ZN6KPlato10PertEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::PertEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<ViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato10PertEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato10PertEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato10PertEditorE_t>.metaTypes,
    nullptr
} };

void KPlato::PertEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PertEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->executeCommand((*reinterpret_cast<std::add_pointer_t<KUndo2Command*>>(_a[1]))); break;
        case 1: _t->slotNodeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 2: _t->slotNodeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 3: _t->slotNodeMoved((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 4: _t->slotNodeChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1]))); break;
        case 5: _t->slotRelationAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Relation*>>(_a[1]))); break;
        case 6: _t->slotRelationRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Relation*>>(_a[1]))); break;
        case 7: _t->dispAvailableTasks(); break;
        case 8: _t->dispAvailableTasks((*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPlato::Node*>>(_a[2]))); break;
        case 9: _t->dispAvailableTasks((*reinterpret_cast<std::add_pointer_t<KPlato::Relation*>>(_a[1]))); break;
        case 10: _t->addTaskInRequiredList((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        case 11: _t->removeTaskFromRequiredList(); break;
        case 12: _t->slotUpdate(); break;
        case 13: _t->slotCurrentTaskChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[2]))); break;
        case 14: _t->slotAvailableChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        case 15: _t->slotRequiredChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 16: _t->slotAddClicked(); break;
        case 17: _t->slotRemoveClicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPlato::Node* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PertEditor::*)(KUndo2Command * )>(_a, &PertEditor::executeCommand, 0))
            return;
    }
}

const QMetaObject *KPlato::PertEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::PertEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato10PertEditorE_t>.strings))
        return static_cast<void*>(this);
    return ViewBase::qt_metacast(_clname);
}

int KPlato::PertEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void KPlato::PertEditor::executeCommand(KUndo2Command * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
