/********************************************************************************
** Form generated from reading UI file 'WelcomeView.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WELCOMEVIEW_H
#define UI_WELCOMEVIEW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_WelcomeView
{
public:
    QGridLayout *gridLayout;
    QWidget *widget_2;
    QVBoxLayout *verticalLayout_4;
    QWidget *newProjectWidget;
    QVBoxLayout *verticalLayout_10;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *newProjectBtn;
    QSpacerItem *horizontalSpacer_6;
    QLabel *label_2;
    QTreeView *projectTemplates;
    QWidget *widget;
    QVBoxLayout *verticalLayout;
    QWidget *openProjectWidget;
    QVBoxLayout *verticalLayout_9;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_4;
    QPushButton *openProjectBtn;
    QSpacerItem *horizontalSpacer;
    QLabel *recentProjectLabel;
    QTreeView *recentProjects;
    QWidget *widget_6;
    QVBoxLayout *verticalLayout_8;
    QWidget *informationWidget;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_5;
    QPushButton *introduction;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer_2;
    QWidget *widget_3;
    QVBoxLayout *verticalLayout_7;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_6;
    QLabel *contextHelp;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QLabel *onlineLabel;
    QLabel *calligraPlanLabel;
    QLabel *userDocLabel;
    QLabel *forumLabel;
    QLabel *commuityLabel;
    QSpacerItem *verticalSpacer;
    QWidget *widget_5;
    QHBoxLayout *horizontalLayout_5;
    QHBoxLayout *horizontalLayout_4;
    QSpacerItem *horizontalSpacer_8;
    QPushButton *createResourceFileBtn;
    QSpacerItem *horizontalSpacer_7;

    void setupUi(QWidget *WelcomeView)
    {
        if (WelcomeView->objectName().isEmpty())
            WelcomeView->setObjectName("WelcomeView");
        WelcomeView->resize(888, 442);
        WelcomeView->setFocusPolicy(Qt::StrongFocus);
        gridLayout = new QGridLayout(WelcomeView);
        gridLayout->setObjectName("gridLayout");
        widget_2 = new QWidget(WelcomeView);
        widget_2->setObjectName("widget_2");
        verticalLayout_4 = new QVBoxLayout(widget_2);
        verticalLayout_4->setObjectName("verticalLayout_4");
        newProjectWidget = new QWidget(widget_2);
        newProjectWidget->setObjectName("newProjectWidget");
        verticalLayout_10 = new QVBoxLayout(newProjectWidget);
        verticalLayout_10->setObjectName("verticalLayout_10");
        verticalLayout_10->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer_2 = new QSpacerItem(13, 14, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        newProjectBtn = new QPushButton(newProjectWidget);
        newProjectBtn->setObjectName("newProjectBtn");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        newProjectBtn->setIcon(icon);

        horizontalLayout_2->addWidget(newProjectBtn);

        horizontalSpacer_6 = new QSpacerItem(13, 17, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_6);


        verticalLayout_10->addLayout(horizontalLayout_2);

        label_2 = new QLabel(newProjectWidget);
        label_2->setObjectName("label_2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy);
        label_2->setAlignment(Qt::AlignCenter);

        verticalLayout_10->addWidget(label_2);


        verticalLayout_4->addWidget(newProjectWidget);

        projectTemplates = new QTreeView(widget_2);
        projectTemplates->setObjectName("projectTemplates");
        projectTemplates->setEditTriggers(QAbstractItemView::NoEditTriggers);
        projectTemplates->setAlternatingRowColors(true);
        projectTemplates->setTextElideMode(Qt::ElideMiddle);
        projectTemplates->setRootIsDecorated(false);
        projectTemplates->header()->setVisible(false);

        verticalLayout_4->addWidget(projectTemplates);


        gridLayout->addWidget(widget_2, 0, 0, 1, 1);

        widget = new QWidget(WelcomeView);
        widget->setObjectName("widget");
        verticalLayout = new QVBoxLayout(widget);
        verticalLayout->setObjectName("verticalLayout");
        openProjectWidget = new QWidget(widget);
        openProjectWidget->setObjectName("openProjectWidget");
        verticalLayout_9 = new QVBoxLayout(openProjectWidget);
        verticalLayout_9->setObjectName("verticalLayout_9");
        verticalLayout_9->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_4);

        openProjectBtn = new QPushButton(openProjectWidget);
        openProjectBtn->setObjectName("openProjectBtn");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        openProjectBtn->setIcon(icon1);

        horizontalLayout->addWidget(openProjectBtn);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_9->addLayout(horizontalLayout);

        recentProjectLabel = new QLabel(openProjectWidget);
        recentProjectLabel->setObjectName("recentProjectLabel");
        sizePolicy.setHeightForWidth(recentProjectLabel->sizePolicy().hasHeightForWidth());
        recentProjectLabel->setSizePolicy(sizePolicy);
        recentProjectLabel->setAlignment(Qt::AlignCenter);

        verticalLayout_9->addWidget(recentProjectLabel);


        verticalLayout->addWidget(openProjectWidget);

        recentProjects = new QTreeView(widget);
        recentProjects->setObjectName("recentProjects");
        recentProjects->setEditTriggers(QAbstractItemView::NoEditTriggers);
        recentProjects->setAlternatingRowColors(true);
        recentProjects->setTextElideMode(Qt::ElideMiddle);
        recentProjects->setRootIsDecorated(false);
        recentProjects->setItemsExpandable(false);
        recentProjects->header()->setVisible(false);

        verticalLayout->addWidget(recentProjects);


        gridLayout->addWidget(widget, 0, 1, 1, 1);

        widget_6 = new QWidget(WelcomeView);
        widget_6->setObjectName("widget_6");
        verticalLayout_8 = new QVBoxLayout(widget_6);
        verticalLayout_8->setObjectName("verticalLayout_8");
        informationWidget = new QWidget(widget_6);
        informationWidget->setObjectName("informationWidget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(informationWidget->sizePolicy().hasHeightForWidth());
        informationWidget->setSizePolicy(sizePolicy1);
        informationWidget->setMinimumSize(QSize(0, 0));
        verticalLayout_5 = new QVBoxLayout(informationWidget);
        verticalLayout_5->setSpacing(0);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_5);

        introduction = new QPushButton(informationWidget);
        introduction->setObjectName("introduction");
        introduction->setFocusPolicy(Qt::NoFocus);
        QIcon icon2;
        iconThemeName = QString::fromUtf8("emblem-information");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        introduction->setIcon(icon2);
        introduction->setFlat(true);

        horizontalLayout_3->addWidget(introduction);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);


        verticalLayout_5->addLayout(horizontalLayout_3);

        verticalSpacer_2 = new QSpacerItem(20, 22, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout_5->addItem(verticalSpacer_2);


        verticalLayout_8->addWidget(informationWidget);

        widget_3 = new QWidget(widget_6);
        widget_3->setObjectName("widget_3");
        verticalLayout_7 = new QVBoxLayout(widget_3);
        verticalLayout_7->setObjectName("verticalLayout_7");
        verticalLayout_7->setContentsMargins(0, 0, 0, 0);
        groupBox_2 = new QGroupBox(widget_3);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_6 = new QVBoxLayout(groupBox_2);
        verticalLayout_6->setObjectName("verticalLayout_6");
        contextHelp = new QLabel(groupBox_2);
        contextHelp->setObjectName("contextHelp");
        contextHelp->setScaledContents(true);
        contextHelp->setAlignment(Qt::AlignHCenter|Qt::AlignTop);
        contextHelp->setWordWrap(true);

        verticalLayout_6->addWidget(contextHelp);


        verticalLayout_7->addWidget(groupBox_2);

        groupBox = new QGroupBox(widget_3);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        onlineLabel = new QLabel(groupBox);
        onlineLabel->setObjectName("onlineLabel");
        onlineLabel->setAlignment(Qt::AlignCenter);

        verticalLayout_2->addWidget(onlineLabel);

        calligraPlanLabel = new QLabel(groupBox);
        calligraPlanLabel->setObjectName("calligraPlanLabel");
        calligraPlanLabel->setAlignment(Qt::AlignCenter);

        verticalLayout_2->addWidget(calligraPlanLabel);

        userDocLabel = new QLabel(groupBox);
        userDocLabel->setObjectName("userDocLabel");
        userDocLabel->setAlignment(Qt::AlignCenter);
        userDocLabel->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(userDocLabel);

        forumLabel = new QLabel(groupBox);
        forumLabel->setObjectName("forumLabel");
        forumLabel->setAlignment(Qt::AlignCenter);
        forumLabel->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(forumLabel);

        commuityLabel = new QLabel(groupBox);
        commuityLabel->setObjectName("commuityLabel");
        commuityLabel->setAlignment(Qt::AlignCenter);
        commuityLabel->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(commuityLabel);


        verticalLayout_7->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(20, 175, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(verticalSpacer);


        verticalLayout_8->addWidget(widget_3);


        gridLayout->addWidget(widget_6, 0, 2, 2, 1);

        widget_5 = new QWidget(WelcomeView);
        widget_5->setObjectName("widget_5");
        horizontalLayout_5 = new QHBoxLayout(widget_5);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setSizeConstraint(QLayout::SetDefaultConstraint);
        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_8);

        createResourceFileBtn = new QPushButton(widget_5);
        createResourceFileBtn->setObjectName("createResourceFileBtn");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("user-group-new");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        createResourceFileBtn->setIcon(icon3);

        horizontalLayout_4->addWidget(createResourceFileBtn);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_7);


        horizontalLayout_5->addLayout(horizontalLayout_4);


        gridLayout->addWidget(widget_5, 1, 0, 1, 2);


        retranslateUi(WelcomeView);

        QMetaObject::connectSlotsByName(WelcomeView);
    } // setupUi

    void retranslateUi(QWidget *WelcomeView)
    {
#if QT_CONFIG(whatsthis)
        newProjectBtn->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
        newProjectBtn->setText(tr2i18n("New Project", nullptr));
        label_2->setText(tr2i18n("<html><head/><body><p align=\"center\"><span style=\" font-size:8pt; font-style:italic;\">Project templates:</span></p></body></html>", nullptr));
#if QT_CONFIG(whatsthis)
        openProjectBtn->setWhatsThis(tr2i18n("<html><head/><body><p><span style=\" font-weight:600;\">Open Project</span></p><p>Displays a file dialog enabling you to open an existing project.</p></body></html>", "@info:whatsthis"));
#endif // QT_CONFIG(whatsthis)
        openProjectBtn->setText(tr2i18n("Open Project", nullptr));
        recentProjectLabel->setText(tr2i18n("<html><head/><body><p align=\"center\"><span style=\" font-size:8pt; font-style:italic;\">Recent projects:</span></p></body></html>", nullptr));
        introduction->setText(tr2i18n("Information", nullptr));
        groupBox_2->setTitle(QString());
        contextHelp->setText(QString());
        groupBox->setTitle(QString());
        onlineLabel->setText(QString());
#if QT_CONFIG(tooltip)
        calligraPlanLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        calligraPlanLabel->setStatusTip(tr2i18n("https://calligra.org/plan", "@info:status"));
#endif // QT_CONFIG(statustip)
        calligraPlanLabel->setText(tr2i18n("<html><head/><body><p><a href=\"https://calligra.org/plan\"><span style=\" text-decoration: underline; color:#2980b9;\">Home</span></a></p></body></html>", nullptr));
#if QT_CONFIG(tooltip)
        userDocLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        userDocLabel->setStatusTip(tr2i18n("https://docs.kde.org", "@info:status"));
#endif // QT_CONFIG(statustip)
        userDocLabel->setText(tr2i18n("<a href=\"https://docs.kde.org/index.php?application=calligraplan\">User documentation</a>", "@action:button"));
#if QT_CONFIG(tooltip)
        forumLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        forumLabel->setStatusTip(tr2i18n("https://forum.kde.org/viewforum.php?f=203", "@info:status"));
#endif // QT_CONFIG(statustip)
        forumLabel->setText(tr2i18n("<a href=\"https://forum.kde.org/viewforum.php?f=203\">Community forum</a>", "@action:button"));
#if QT_CONFIG(tooltip)
        commuityLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        commuityLabel->setStatusTip(tr2i18n("https://userbase.kde.org/Plan", "@info:status"));
#endif // QT_CONFIG(statustip)
        commuityLabel->setText(tr2i18n("<a href=\"https://userbase.kde.org/Plan\">Community</a>", "@action:button"));
        createResourceFileBtn->setText(tr2i18n("Create Shared Resources", nullptr));
        (void)WelcomeView;
    } // retranslateUi

};

namespace Ui {
    class WelcomeView: public Ui_WelcomeView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // WELCOMEVIEW_H

