/********************************************************************************
** Form generated from reading UI file 'kptpertresult.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTPERTRESULT_H
#define UI_KPTPERTRESULT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kptitemmodelbase.h"

namespace KPlato {

class Ui_PertResult
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *label_1;
    QLabel *scheduleName;
    QLabel *label;
    QLabel *label_3;
    QLabel *totalFloat;
    QSpacerItem *spacerItem;
    DoubleTreeViewBase *treeWidgetTaskResult;

    void setupUi(QWidget *KPlato__PertResult)
    {
        if (KPlato__PertResult->objectName().isEmpty())
            KPlato__PertResult->setObjectName("KPlato__PertResult");
        KPlato__PertResult->resize(407, 285);
        KPlato__PertResult->setContextMenuPolicy(Qt::NoContextMenu);
        vboxLayout = new QVBoxLayout(KPlato__PertResult);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        label_1 = new QLabel(KPlato__PertResult);
        label_1->setObjectName("label_1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_1->sizePolicy().hasHeightForWidth());
        label_1->setSizePolicy(sizePolicy);
        label_1->setFrameShape(QFrame::NoFrame);
        label_1->setFrameShadow(QFrame::Plain);
        label_1->setMargin(1);
        label_1->setIndent(0);

        hboxLayout->addWidget(label_1);

        scheduleName = new QLabel(KPlato__PertResult);
        scheduleName->setObjectName("scheduleName");
        sizePolicy.setHeightForWidth(scheduleName->sizePolicy().hasHeightForWidth());
        scheduleName->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(scheduleName);

        label = new QLabel(KPlato__PertResult);
        label->setObjectName("label");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);
        label->setMinimumSize(QSize(20, 0));

        hboxLayout->addWidget(label);

        label_3 = new QLabel(KPlato__PertResult);
        label_3->setObjectName("label_3");
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setFrameShape(QFrame::NoFrame);
        label_3->setFrameShadow(QFrame::Raised);

        hboxLayout->addWidget(label_3);

        totalFloat = new QLabel(KPlato__PertResult);
        totalFloat->setObjectName("totalFloat");
        sizePolicy.setHeightForWidth(totalFloat->sizePolicy().hasHeightForWidth());
        totalFloat->setSizePolicy(sizePolicy);
        totalFloat->setFrameShape(QFrame::NoFrame);
        totalFloat->setFrameShadow(QFrame::Plain);

        hboxLayout->addWidget(totalFloat);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);


        vboxLayout->addLayout(hboxLayout);

        treeWidgetTaskResult = new DoubleTreeViewBase(KPlato__PertResult);
        treeWidgetTaskResult->setObjectName("treeWidgetTaskResult");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(treeWidgetTaskResult->sizePolicy().hasHeightForWidth());
        treeWidgetTaskResult->setSizePolicy(sizePolicy2);
        treeWidgetTaskResult->setFrameShape(QFrame::NoFrame);
        treeWidgetTaskResult->setFrameShadow(QFrame::Raised);

        vboxLayout->addWidget(treeWidgetTaskResult);


        retranslateUi(KPlato__PertResult);

        QMetaObject::connectSlotsByName(KPlato__PertResult);
    } // setupUi

    void retranslateUi(QWidget *KPlato__PertResult)
    {
        label_1->setText(tr2i18n("Project Schedule:", nullptr));
        scheduleName->setText(tr2i18n("...", nullptr));
        label->setText(QString());
        label_3->setText(tr2i18n("Total Float:", nullptr));
        totalFloat->setText(tr2i18n("...", nullptr));
        (void)KPlato__PertResult;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class PertResult: public Ui_PertResult {};
} // namespace Ui
} // namespace KPlato

#endif // KPTPERTRESULT_H

