/********************************************************************************
** Form generated from reading UI file 'ConfigProjectPanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGPROJECTPANEL_H
#define UI_CONFIGPROJECTPANEL_H

#include <KToolBar>
#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "KLineEdit"
#include "KTextEdit"
#include "krichtextedit.h"
#include "krichtextwidget.h"

namespace KPlato {

class Ui_ConfigProjectPanel
{
public:
    QHBoxLayout *horizontalLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *kcfg_SharedResourcesFile;
    QPushButton *resourceFileBrowseBtn;
    QLabel *srflabel;
    QLabel *leaderlabel;
    QCheckBox *kcfg_UseSharedResources;
    QLabel *usrlabel;
    KLineEdit *kcfg_Manager;
    QSpacerItem *verticalSpacer;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout;
    KToolBar *toolbar;
    KRichTextWidget *kcfg_ProjectDescription;

    void setupUi(QWidget *KPlato__ConfigProjectPanel)
    {
        if (KPlato__ConfigProjectPanel->objectName().isEmpty())
            KPlato__ConfigProjectPanel->setObjectName("KPlato__ConfigProjectPanel");
        KPlato__ConfigProjectPanel->resize(547, 435);
        horizontalLayout = new QHBoxLayout(KPlato__ConfigProjectPanel);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        tabWidget = new QTabWidget(KPlato__ConfigProjectPanel);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        gridLayout = new QGridLayout(tab);
        gridLayout->setObjectName("gridLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        kcfg_SharedResourcesFile = new QLineEdit(tab);
        kcfg_SharedResourcesFile->setObjectName("kcfg_SharedResourcesFile");

        horizontalLayout_2->addWidget(kcfg_SharedResourcesFile);

        resourceFileBrowseBtn = new QPushButton(tab);
        resourceFileBrowseBtn->setObjectName("resourceFileBrowseBtn");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        resourceFileBrowseBtn->setIcon(icon);

        horizontalLayout_2->addWidget(resourceFileBrowseBtn);


        gridLayout->addLayout(horizontalLayout_2, 2, 1, 1, 1);

        srflabel = new QLabel(tab);
        srflabel->setObjectName("srflabel");

        gridLayout->addWidget(srflabel, 2, 0, 1, 1);

        leaderlabel = new QLabel(tab);
        leaderlabel->setObjectName("leaderlabel");
        leaderlabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        leaderlabel->setWordWrap(false);

        gridLayout->addWidget(leaderlabel, 0, 0, 1, 1);

        kcfg_UseSharedResources = new QCheckBox(tab);
        kcfg_UseSharedResources->setObjectName("kcfg_UseSharedResources");

        gridLayout->addWidget(kcfg_UseSharedResources, 1, 1, 1, 1);

        usrlabel = new QLabel(tab);
        usrlabel->setObjectName("usrlabel");

        gridLayout->addWidget(usrlabel, 1, 0, 1, 1);

        kcfg_Manager = new KLineEdit(tab);
        kcfg_Manager->setObjectName("kcfg_Manager");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(kcfg_Manager->sizePolicy().hasHeightForWidth());
        kcfg_Manager->setSizePolicy(sizePolicy);

        gridLayout->addWidget(kcfg_Manager, 0, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 3, 0, 1, 1);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout = new QVBoxLayout(tab_2);
        verticalLayout->setObjectName("verticalLayout");
        toolbar = new KToolBar(tab_2);
        toolbar->setObjectName("toolbar");
        toolbar->setMinimumSize(QSize(0, 10));

        verticalLayout->addWidget(toolbar);

        kcfg_ProjectDescription = new KRichTextWidget(tab_2);
        kcfg_ProjectDescription->setObjectName("kcfg_ProjectDescription");

        verticalLayout->addWidget(kcfg_ProjectDescription);

        tabWidget->addTab(tab_2, QString());

        horizontalLayout->addWidget(tabWidget);

#if QT_CONFIG(shortcut)
        leaderlabel->setBuddy(kcfg_Manager);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__ConfigProjectPanel);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KPlato__ConfigProjectPanel);
    } // setupUi

    void retranslateUi(QWidget *KPlato__ConfigProjectPanel)
    {
        resourceFileBrowseBtn->setText(tr2i18n("Browse...", nullptr));
        srflabel->setText(tr2i18n("Shared resource file:", nullptr));
#if QT_CONFIG(whatsthis)
        leaderlabel->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
        leaderlabel->setText(tr2i18n("&Manager:", nullptr));
        kcfg_UseSharedResources->setText(QString());
        usrlabel->setText(tr2i18n("Use shared resources:", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_Manager->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
        tabWidget->setTabText(tabWidget->indexOf(tab), tr2i18n("General", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), tr2i18n("Description", nullptr));
        (void)KPlato__ConfigProjectPanel;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class ConfigProjectPanel: public Ui_ConfigProjectPanel {};
} // namespace Ui
} // namespace KPlato

#endif // CONFIGPROJECTPANEL_H

