// This file is generated by kconfig_compiler_kf6 from portfoliosettings.kcfg.
// All changes you do to this file will be lost.
#ifndef PORTFOLIOSETTINGS_H
#define PORTFOLIOSETTINGS_H

#include <ConfigSkeleton.h>

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class PLANPORTFOLIO_EXPORT PortfolioSettings : public ConfigSkeleton
{
  public:

    static PortfolioSettings *self();
    ~PortfolioSettings() override;

    /**
      Set Path to the documentation
    */
    static
    void setDocumentationPath( const QString & v )
    {
      if (!self()->isDocumentationPathImmutable())
        self()->mDocumentationPath = v;
    }

    /**
      Get Path to the documentation
    */
    static
    QString documentationPath()
    {
      return self()->mDocumentationPath;
    }

    /**
      Is Path to the documentation Immutable
    */
    static
    bool isDocumentationPathImmutable()
    {
      return self()->isImmutable( QStringLiteral( "DocumentationPath" ) );
    }

    /**
      Set Path to the context help documentation
    */
    static
    void setContextPath( const QString & v )
    {
      if (!self()->isContextPathImmutable())
        self()->mContextPath = v;
    }

    /**
      Get Path to the context help documentation
    */
    static
    QString contextPath()
    {
      return self()->mContextPath;
    }

    /**
      Is Path to the context help documentation Immutable
    */
    static
    bool isContextPathImmutable()
    {
      return self()->isImmutable( QStringLiteral( "ContextPath" ) );
    }

    /**
      Set The context help documentation must have been translated to the language
    */
    static
    void setContextLanguage( const QString & v )
    {
      if (!self()->isContextLanguageImmutable())
        self()->mContextLanguage = v;
    }

    /**
      Get The context help documentation must have been translated to the language
    */
    static
    QString contextLanguage()
    {
      return self()->mContextLanguage;
    }

    /**
      Is The context help documentation must have been translated to the language Immutable
    */
    static
    bool isContextLanguageImmutable()
    {
      return self()->isImmutable( QStringLiteral( "ContextLanguage" ) );
    }

    /**
      Set Autosave interval
    */
    static
    void setAutoSave( int v )
    {
      if (!self()->isAutoSaveImmutable())
        self()->mAutoSave = v;
    }

    /**
      Get Autosave interval
    */
    static
    int autoSave()
    {
      return self()->mAutoSave;
    }

    /**
      Is Autosave interval Immutable
    */
    static
    bool isAutoSaveImmutable()
    {
      return self()->isImmutable( QStringLiteral( "AutoSave" ) );
    }

    /**
      Set Create a backup file
    */
    static
    void setCreateBackupFile( bool v )
    {
      if (!self()->isCreateBackupFileImmutable())
        self()->mCreateBackupFile = v;
    }

    /**
      Get Create a backup file
    */
    static
    bool createBackupFile()
    {
      return self()->mCreateBackupFile;
    }

    /**
      Is Create a backup file Immutable
    */
    static
    bool isCreateBackupFileImmutable()
    {
      return self()->isImmutable( QStringLiteral( "CreateBackupFile" ) );
    }

  protected:
    PortfolioSettings();
    friend class PortfolioSettingsHelper;


    // Documentation
    QString mDocumentationPath;
    QString mContextPath;
    QString mContextLanguage;

    // Interface
    int mAutoSave;
    bool mCreateBackupFile;
};

#endif

