/****************************************************************************
** Meta object code from reading C++ file 'kptworkpackagemergedialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/kptworkpackagemergedialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptworkpackagemergedialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato21WorkPackageMergePanelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::WorkPackageMergePanel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato21WorkPackageMergePanelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::WorkPackageMergePanel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WorkPackageMergePanel, qt_meta_tag_ZN6KPlato21WorkPackageMergePanelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::WorkPackageMergePanel::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato21WorkPackageMergePanelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato21WorkPackageMergePanelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato21WorkPackageMergePanelE_t>.metaTypes,
    nullptr
} };

void KPlato::WorkPackageMergePanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WorkPackageMergePanel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KPlato::WorkPackageMergePanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::WorkPackageMergePanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato21WorkPackageMergePanelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::WorkPackageMergePanel"))
        return static_cast< Ui::WorkPackageMergePanel*>(this);
    return QWidget::qt_metacast(_clname);
}

int KPlato::WorkPackageMergePanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6KPlato22WorkPackageMergeDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::WorkPackageMergeDialog::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato22WorkPackageMergeDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::WorkPackageMergeDialog",
        "executeCommand",
        "",
        "KUndo2Command*",
        "cmd",
        "terminateWorkPackage",
        "const KPlato::Package*",
        "package",
        "slotMerge",
        "slotReject",
        "slotApply",
        "slotBack"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'executeCommand'
        QtMocHelpers::SignalData<void(KUndo2Command *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'terminateWorkPackage'
        QtMocHelpers::SignalData<void(const KPlato::Package *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotMerge'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotReject'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotApply'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotBack'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WorkPackageMergeDialog, qt_meta_tag_ZN6KPlato22WorkPackageMergeDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::WorkPackageMergeDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<KoDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22WorkPackageMergeDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22WorkPackageMergeDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato22WorkPackageMergeDialogE_t>.metaTypes,
    nullptr
} };

void KPlato::WorkPackageMergeDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WorkPackageMergeDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->executeCommand((*reinterpret_cast<std::add_pointer_t<KUndo2Command*>>(_a[1]))); break;
        case 1: _t->terminateWorkPackage((*reinterpret_cast<std::add_pointer_t<const KPlato::Package*>>(_a[1]))); break;
        case 2: _t->slotMerge(); break;
        case 3: _t->slotReject(); break;
        case 4: _t->slotApply(); break;
        case 5: _t->slotBack(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WorkPackageMergeDialog::*)(KUndo2Command * )>(_a, &WorkPackageMergeDialog::executeCommand, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkPackageMergeDialog::*)(const KPlato::Package * )>(_a, &WorkPackageMergeDialog::terminateWorkPackage, 1))
            return;
    }
}

const QMetaObject *KPlato::WorkPackageMergeDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::WorkPackageMergeDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato22WorkPackageMergeDialogE_t>.strings))
        return static_cast<void*>(this);
    return KoDialog::qt_metacast(_clname);
}

int KPlato::WorkPackageMergeDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KoDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void KPlato::WorkPackageMergeDialog::executeCommand(KUndo2Command * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::WorkPackageMergeDialog::terminateWorkPackage(const KPlato::Package * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
