/****************************************************************************
** Meta object code from reading C++ file 'kptaccount.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/kernel/kptaccount.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kptaccount.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato8AccountsE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::Accounts::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato8AccountsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::Accounts",
        "accountAdded",
        "",
        "const KPlato::Account*",
        "accountToBeAdded",
        "accountRemoved",
        "accountToBeRemoved",
        "changed",
        "KPlato::Account*",
        "defaultAccountChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'accountAdded'
        QtMocHelpers::SignalData<void(const KPlato::Account *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'accountToBeAdded'
        QtMocHelpers::SignalData<void(const KPlato::Account *, int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::Int, 2 },
        }}),
        // Signal 'accountRemoved'
        QtMocHelpers::SignalData<void(const KPlato::Account *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'accountToBeRemoved'
        QtMocHelpers::SignalData<void(const KPlato::Account *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'changed'
        QtMocHelpers::SignalData<void(KPlato::Account *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Signal 'defaultAccountChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Accounts, qt_meta_tag_ZN6KPlato8AccountsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::Accounts::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8AccountsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8AccountsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato8AccountsE_t>.metaTypes,
    nullptr
} };

void KPlato::Accounts::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Accounts *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accountAdded((*reinterpret_cast<std::add_pointer_t<const KPlato::Account*>>(_a[1]))); break;
        case 1: _t->accountToBeAdded((*reinterpret_cast<std::add_pointer_t<const KPlato::Account*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->accountRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::Account*>>(_a[1]))); break;
        case 3: _t->accountToBeRemoved((*reinterpret_cast<std::add_pointer_t<const KPlato::Account*>>(_a[1]))); break;
        case 4: _t->changed((*reinterpret_cast<std::add_pointer_t<KPlato::Account*>>(_a[1]))); break;
        case 5: _t->defaultAccountChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Accounts::*)(const KPlato::Account * )>(_a, &Accounts::accountAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Accounts::*)(const KPlato::Account * , int )>(_a, &Accounts::accountToBeAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Accounts::*)(const KPlato::Account * )>(_a, &Accounts::accountRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Accounts::*)(const KPlato::Account * )>(_a, &Accounts::accountToBeRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Accounts::*)(KPlato::Account * )>(_a, &Accounts::changed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Accounts::*)()>(_a, &Accounts::defaultAccountChanged, 5))
            return;
    }
}

const QMetaObject *KPlato::Accounts::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::Accounts::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato8AccountsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KPlato::Accounts::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void KPlato::Accounts::accountAdded(const KPlato::Account * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KPlato::Accounts::accountToBeAdded(const KPlato::Account * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KPlato::Accounts::accountRemoved(const KPlato::Account * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KPlato::Accounts::accountToBeRemoved(const KPlato::Account * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KPlato::Accounts::changed(KPlato::Account * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KPlato::Accounts::defaultAccountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
