/********************************************************************************
** Form generated from reading UI file 'kpttaskcostpanelbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTTASKCOSTPANELBASE_H
#define UI_KPTTASKCOSTPANELBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "KLineEdit"

namespace KPlato {

class Ui_TaskCostPanelBase
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *runningGroup;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel3_2_2;
    QComboBox *runningAccount;
    QSpacerItem *spacerItem;
    QGroupBox *startupGroup;
    QGridLayout *gridLayout;
    QLabel *textLabel1;
    KLineEdit *startupCost;
    QLabel *textLabel3;
    QComboBox *startupAccount;
    QSpacerItem *spacerItem1;
    QGroupBox *shutdownGroup;
    QGridLayout *gridLayout1;
    QLabel *textLabel2;
    KLineEdit *shutdownCost;
    QLabel *textLabel3_3;
    QComboBox *shutdownAccount;
    QSpacerItem *spacerItem2;
    QSpacerItem *spacerItem3;

    void setupUi(QWidget *KPlato__TaskCostPanelBase)
    {
        if (KPlato__TaskCostPanelBase->objectName().isEmpty())
            KPlato__TaskCostPanelBase->setObjectName("KPlato__TaskCostPanelBase");
        KPlato__TaskCostPanelBase->resize(250, 249);
        vboxLayout = new QVBoxLayout(KPlato__TaskCostPanelBase);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName("vboxLayout");
        runningGroup = new QGroupBox(KPlato__TaskCostPanelBase);
        runningGroup->setObjectName("runningGroup");
        hboxLayout = new QHBoxLayout(runningGroup);
        hboxLayout->setObjectName("hboxLayout");
        textLabel3_2_2 = new QLabel(runningGroup);
        textLabel3_2_2->setObjectName("textLabel3_2_2");
        textLabel3_2_2->setWordWrap(false);
        textLabel3_2_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        hboxLayout->addWidget(textLabel3_2_2);

        runningAccount = new QComboBox(runningGroup);
        runningAccount->setObjectName("runningAccount");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(runningAccount->sizePolicy().hasHeightForWidth());
        runningAccount->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(runningAccount);

        spacerItem = new QSpacerItem(128, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);


        vboxLayout->addWidget(runningGroup);

        startupGroup = new QGroupBox(KPlato__TaskCostPanelBase);
        startupGroup->setObjectName("startupGroup");
        gridLayout = new QGridLayout(startupGroup);
        gridLayout->setObjectName("gridLayout");
        textLabel1 = new QLabel(startupGroup);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);
        textLabel1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        startupCost = new KLineEdit(startupGroup);
        startupCost->setObjectName("startupCost");

        gridLayout->addWidget(startupCost, 0, 1, 1, 1);

        textLabel3 = new QLabel(startupGroup);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);
        textLabel3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(textLabel3, 1, 0, 1, 1);

        startupAccount = new QComboBox(startupGroup);
        startupAccount->setObjectName("startupAccount");
        sizePolicy.setHeightForWidth(startupAccount->sizePolicy().hasHeightForWidth());
        startupAccount->setSizePolicy(sizePolicy);

        gridLayout->addWidget(startupAccount, 1, 1, 1, 1);

        spacerItem1 = new QSpacerItem(91, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem1, 1, 2, 1, 1);


        vboxLayout->addWidget(startupGroup);

        shutdownGroup = new QGroupBox(KPlato__TaskCostPanelBase);
        shutdownGroup->setObjectName("shutdownGroup");
        gridLayout1 = new QGridLayout(shutdownGroup);
        gridLayout1->setObjectName("gridLayout1");
        textLabel2 = new QLabel(shutdownGroup);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);
        textLabel2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout1->addWidget(textLabel2, 0, 0, 1, 1);

        shutdownCost = new KLineEdit(shutdownGroup);
        shutdownCost->setObjectName("shutdownCost");

        gridLayout1->addWidget(shutdownCost, 0, 1, 1, 1);

        textLabel3_3 = new QLabel(shutdownGroup);
        textLabel3_3->setObjectName("textLabel3_3");
        textLabel3_3->setWordWrap(false);
        textLabel3_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout1->addWidget(textLabel3_3, 1, 0, 1, 1);

        shutdownAccount = new QComboBox(shutdownGroup);
        shutdownAccount->setObjectName("shutdownAccount");
        sizePolicy.setHeightForWidth(shutdownAccount->sizePolicy().hasHeightForWidth());
        shutdownAccount->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(shutdownAccount, 1, 1, 1, 1);

        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout1->addItem(spacerItem2, 1, 2, 1, 1);


        vboxLayout->addWidget(shutdownGroup);

        spacerItem3 = new QSpacerItem(151, 33, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem3);

#if QT_CONFIG(shortcut)
        textLabel3_2_2->setBuddy(runningAccount);
        textLabel1->setBuddy(startupCost);
        textLabel3->setBuddy(startupAccount);
        textLabel2->setBuddy(shutdownCost);
        textLabel3_3->setBuddy(shutdownAccount);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__TaskCostPanelBase);

        QMetaObject::connectSlotsByName(KPlato__TaskCostPanelBase);
    } // setupUi

    void retranslateUi(QWidget *KPlato__TaskCostPanelBase)
    {
        runningGroup->setTitle(tr2i18n("Running", nullptr));
        textLabel3_2_2->setText(tr2i18n("Account:", nullptr));
        startupGroup->setTitle(tr2i18n("Startup", nullptr));
        textLabel1->setText(tr2i18n("Cost:", nullptr));
        textLabel3->setText(tr2i18n("Account:", nullptr));
        shutdownGroup->setTitle(tr2i18n("Shutdown", nullptr));
        textLabel2->setText(tr2i18n("Cost:", nullptr));
        textLabel3_3->setText(tr2i18n("Account:", nullptr));
        (void)KPlato__TaskCostPanelBase;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class TaskCostPanelBase: public Ui_TaskCostPanelBase {};
} // namespace Ui
} // namespace KPlato

#endif // KPTTASKCOSTPANELBASE_H

