/****************************************************************************
** Meta object code from reading C++ file 'ResourceItemModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/models/ResourceItemModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ResourceItemModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato17ResourceItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::ResourceItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato17ResourceItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::ResourceItemModel",
        "setGroupsEnabled",
        "",
        "enable",
        "setTeamsEnabled",
        "setRequiredEnabled",
        "slotResourceChanged",
        "KPlato::Resource*",
        "slotResourceGroupChanged",
        "KPlato::ResourceGroup*",
        "slotResourceGroupToBeAdded",
        "resource",
        "row",
        "slotResourceGroupAdded",
        "group",
        "slotResourceGroupToBeRemoved",
        "slotResourceGroupRemoved",
        "slotResourceToBeAdded",
        "KPlato::Project*",
        "project",
        "slotResourceAdded",
        "slotResourceToBeRemoved",
        "slotResourceRemoved",
        "slotResourceTeamToBeAdded",
        "slotResourceTeamAdded",
        "slotResourceTeamToBeRemoved",
        "team",
        "slotResourceTeamRemoved",
        "slotCalendarChanged",
        "KPlato::Calendar*",
        "cal",
        "slotDataArrived",
        "KIO::Job*",
        "job",
        "data",
        "slotJobFinished",
        "KJob*"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setGroupsEnabled'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setTeamsEnabled'
        QtMocHelpers::SlotData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setRequiredEnabled'
        QtMocHelpers::SlotData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'slotResourceChanged'
        QtMocHelpers::SlotData<void(KPlato::Resource *)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Slot 'slotResourceGroupChanged'
        QtMocHelpers::SlotData<void(KPlato::ResourceGroup *)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Slot 'slotResourceGroupToBeAdded'
        QtMocHelpers::SlotData<void(KPlato::Resource *, int)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'slotResourceGroupAdded'
        QtMocHelpers::SlotData<void(KPlato::ResourceGroup *)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 14 },
        }}),
        // Slot 'slotResourceGroupToBeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Resource *, int, KPlato::ResourceGroup *)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 11 }, { QMetaType::Int, 12 }, { 0x80000000 | 9, 14 },
        }}),
        // Slot 'slotResourceGroupRemoved'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotResourceToBeAdded'
        QtMocHelpers::SlotData<void(KPlato::Project *, int)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'slotResourceAdded'
        QtMocHelpers::SlotData<void(KPlato::Resource *)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 11 },
        }}),
        // Slot 'slotResourceToBeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Project *, int, KPlato::Resource *)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { QMetaType::Int, 12 }, { 0x80000000 | 7, 11 },
        }}),
        // Slot 'slotResourceRemoved'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotResourceTeamToBeAdded'
        QtMocHelpers::SlotData<void(KPlato::Resource *, int)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'slotResourceTeamAdded'
        QtMocHelpers::SlotData<void(KPlato::Resource *)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 11 },
        }}),
        // Slot 'slotResourceTeamToBeRemoved'
        QtMocHelpers::SlotData<void(KPlato::Resource *, int, KPlato::Resource *)>(25, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 11 }, { QMetaType::Int, 12 }, { 0x80000000 | 7, 26 },
        }}),
        // Slot 'slotResourceTeamRemoved'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCalendarChanged'
        QtMocHelpers::SlotData<void(KPlato::Calendar *)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'slotDataArrived'
        QtMocHelpers::SlotData<void(KIO::Job *, const QByteArray &)>(31, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 32, 33 }, { QMetaType::QByteArray, 34 },
        }}),
        // Slot 'slotJobFinished'
        QtMocHelpers::SlotData<void(KJob *)>(35, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 36, 33 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ResourceItemModel, qt_meta_tag_ZN6KPlato17ResourceItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::ResourceItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ItemModelBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17ResourceItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17ResourceItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato17ResourceItemModelE_t>.metaTypes,
    nullptr
} };

void KPlato::ResourceItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResourceItemModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setGroupsEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->setTeamsEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->setRequiredEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->slotResourceChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 4: _t->slotResourceGroupChanged((*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[1]))); break;
        case 5: _t->slotResourceGroupToBeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->slotResourceGroupAdded((*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[1]))); break;
        case 7: _t->slotResourceGroupToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KPlato::ResourceGroup*>>(_a[3]))); break;
        case 8: _t->slotResourceGroupRemoved(); break;
        case 9: _t->slotResourceToBeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 10: _t->slotResourceAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 11: _t->slotResourceToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Project*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[3]))); break;
        case 12: _t->slotResourceRemoved(); break;
        case 13: _t->slotResourceTeamToBeAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 14: _t->slotResourceTeamAdded((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1]))); break;
        case 15: _t->slotResourceTeamToBeRemoved((*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KPlato::Resource*>>(_a[3]))); break;
        case 16: _t->slotResourceTeamRemoved(); break;
        case 17: _t->slotCalendarChanged((*reinterpret_cast<std::add_pointer_t<KPlato::Calendar*>>(_a[1]))); break;
        case 18: _t->slotDataArrived((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 19: _t->slotJobFinished((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::ResourceItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::ResourceItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17ResourceItemModelE_t>.strings))
        return static_cast<void*>(this);
    return ItemModelBase::qt_metacast(_clname);
}

int KPlato::ResourceItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ItemModelBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    return _id;
}
QT_WARNING_POP
