/********************************************************************************
** Form generated from reading UI file 'kptconfigtaskpanelbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPTCONFIGTASKPANELBASE_H
#define UI_KPTCONFIGTASKPANELBASE_H

#include <KToolBar>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "klineedit.h"
#include "kptdurationspinbox.h"
#include "kptmycombobox_p.h"
#include "krichtextedit.h"
#include "krichtextwidget.h"
#include "ktextedit.h"

namespace KPlato {

class Ui_ConfigTaskPanelBase
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QGridLayout *gridLayout;
    QLabel *leaderlabel;
    QDateTimeEdit *kcfg_ConstraintStartTime;
    KComboBox *kcfg_EstimateType;
    QSpacerItem *verticalSpacer;
    QLabel *textLabel2_2;
    QLabel *textLabel1_3;
    KComboBox *kcfg_StartTimeUsage;
    QFrame *line;
    QLabel *textLabel3_7;
    QLabel *textLabel3_5;
    QHBoxLayout *horizontalLayout;
    KPlato::DurationSpinBox *kcfg_ExpectedEstimate;
    KPlato::MyComboBox *kcfg_Unit;
    QSpinBox *kcfg_PessimisticRatio;
    QFrame *line_2;
    QLabel *textLabel3_6;
    QLabel *textLabel3_8;
    QLabel *textLabel3_3;
    QDateTimeEdit *kcfg_ConstraintEndTime;
    KComboBox *kcfg_EndTimeUsage;
    KComboBox *kcfg_ConstraintType;
    QLabel *textLabel3_4;
    QSpinBox *kcfg_OptimisticRatio;
    QHBoxLayout *hboxLayout;
    KLineEdit *kcfg_Leader;
    QPushButton *chooseLeader;
    QLabel *textLabel3;
    QLabel *label;
    QComboBox *kcfg_Risk;
    QWidget *widget;
    QVBoxLayout *verticalLayout_2;
    KToolBar *toolbar;
    KRichTextWidget *kcfg_Description;

    void setupUi(QWidget *KPlato__ConfigTaskPanelBase)
    {
        if (KPlato__ConfigTaskPanelBase->objectName().isEmpty())
            KPlato__ConfigTaskPanelBase->setObjectName("KPlato__ConfigTaskPanelBase");
        KPlato__ConfigTaskPanelBase->resize(410, 492);
        verticalLayout = new QVBoxLayout(KPlato__ConfigTaskPanelBase);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        tabWidget = new QTabWidget(KPlato__ConfigTaskPanelBase);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        gridLayout = new QGridLayout(tab);
        gridLayout->setObjectName("gridLayout");
        leaderlabel = new QLabel(tab);
        leaderlabel->setObjectName("leaderlabel");
        leaderlabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        leaderlabel->setWordWrap(false);

        gridLayout->addWidget(leaderlabel, 0, 0, 1, 1);

        kcfg_ConstraintStartTime = new QDateTimeEdit(tab);
        kcfg_ConstraintStartTime->setObjectName("kcfg_ConstraintStartTime");
        kcfg_ConstraintStartTime->setCalendarPopup(true);

        gridLayout->addWidget(kcfg_ConstraintStartTime, 4, 1, 1, 1);

        kcfg_EstimateType = new KComboBox(tab);
        kcfg_EstimateType->addItem(QString());
        kcfg_EstimateType->addItem(QString());
        kcfg_EstimateType->setObjectName("kcfg_EstimateType");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(kcfg_EstimateType->sizePolicy().hasHeightForWidth());
        kcfg_EstimateType->setSizePolicy(sizePolicy);
        kcfg_EstimateType->setEditable(false);

        gridLayout->addWidget(kcfg_EstimateType, 8, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 13, 0, 1, 1);

        textLabel2_2 = new QLabel(tab);
        textLabel2_2->setObjectName("textLabel2_2");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textLabel2_2->sizePolicy().hasHeightForWidth());
        textLabel2_2->setSizePolicy(sizePolicy1);
        textLabel2_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel2_2->setWordWrap(false);

        gridLayout->addWidget(textLabel2_2, 12, 0, 1, 1);

        textLabel1_3 = new QLabel(tab);
        textLabel1_3->setObjectName("textLabel1_3");
        sizePolicy1.setHeightForWidth(textLabel1_3->sizePolicy().hasHeightForWidth());
        textLabel1_3->setSizePolicy(sizePolicy1);
        textLabel1_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel1_3->setWordWrap(false);

        gridLayout->addWidget(textLabel1_3, 11, 0, 1, 1);

        kcfg_StartTimeUsage = new KComboBox(tab);
        kcfg_StartTimeUsage->addItem(QString());
        kcfg_StartTimeUsage->addItem(QString());
        kcfg_StartTimeUsage->addItem(QString());
        kcfg_StartTimeUsage->setObjectName("kcfg_StartTimeUsage");
        sizePolicy.setHeightForWidth(kcfg_StartTimeUsage->sizePolicy().hasHeightForWidth());
        kcfg_StartTimeUsage->setSizePolicy(sizePolicy);

        gridLayout->addWidget(kcfg_StartTimeUsage, 3, 1, 1, 1);

        line = new QFrame(tab);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line, 7, 0, 1, 2);

        textLabel3_7 = new QLabel(tab);
        textLabel3_7->setObjectName("textLabel3_7");
        sizePolicy1.setHeightForWidth(textLabel3_7->sizePolicy().hasHeightForWidth());
        textLabel3_7->setSizePolicy(sizePolicy1);
        textLabel3_7->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3_7->setWordWrap(false);

        gridLayout->addWidget(textLabel3_7, 8, 0, 1, 1);

        textLabel3_5 = new QLabel(tab);
        textLabel3_5->setObjectName("textLabel3_5");
        sizePolicy1.setHeightForWidth(textLabel3_5->sizePolicy().hasHeightForWidth());
        textLabel3_5->setSizePolicy(sizePolicy1);
        textLabel3_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3_5->setWordWrap(false);

        gridLayout->addWidget(textLabel3_5, 3, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        kcfg_ExpectedEstimate = new KPlato::DurationSpinBox(tab);
        kcfg_ExpectedEstimate->setObjectName("kcfg_ExpectedEstimate");

        horizontalLayout->addWidget(kcfg_ExpectedEstimate);

        kcfg_Unit = new KPlato::MyComboBox(tab);
        kcfg_Unit->setObjectName("kcfg_Unit");

        horizontalLayout->addWidget(kcfg_Unit);


        gridLayout->addLayout(horizontalLayout, 9, 1, 1, 1);

        kcfg_PessimisticRatio = new QSpinBox(tab);
        kcfg_PessimisticRatio->setObjectName("kcfg_PessimisticRatio");
        kcfg_PessimisticRatio->setMaximum(999);

        gridLayout->addWidget(kcfg_PessimisticRatio, 12, 1, 1, 1);

        line_2 = new QFrame(tab);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line_2, 1, 0, 1, 2);

        textLabel3_6 = new QLabel(tab);
        textLabel3_6->setObjectName("textLabel3_6");
        sizePolicy1.setHeightForWidth(textLabel3_6->sizePolicy().hasHeightForWidth());
        textLabel3_6->setSizePolicy(sizePolicy1);
        textLabel3_6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3_6->setWordWrap(false);

        gridLayout->addWidget(textLabel3_6, 5, 0, 1, 1);

        textLabel3_8 = new QLabel(tab);
        textLabel3_8->setObjectName("textLabel3_8");
        sizePolicy1.setHeightForWidth(textLabel3_8->sizePolicy().hasHeightForWidth());
        textLabel3_8->setSizePolicy(sizePolicy1);
        textLabel3_8->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3_8->setWordWrap(false);

        gridLayout->addWidget(textLabel3_8, 9, 0, 1, 1);

        textLabel3_3 = new QLabel(tab);
        textLabel3_3->setObjectName("textLabel3_3");
        sizePolicy1.setHeightForWidth(textLabel3_3->sizePolicy().hasHeightForWidth());
        textLabel3_3->setSizePolicy(sizePolicy1);
        textLabel3_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3_3->setWordWrap(false);

        gridLayout->addWidget(textLabel3_3, 4, 0, 1, 1);

        kcfg_ConstraintEndTime = new QDateTimeEdit(tab);
        kcfg_ConstraintEndTime->setObjectName("kcfg_ConstraintEndTime");
        kcfg_ConstraintEndTime->setCalendarPopup(true);

        gridLayout->addWidget(kcfg_ConstraintEndTime, 6, 1, 1, 1);

        kcfg_EndTimeUsage = new KComboBox(tab);
        kcfg_EndTimeUsage->addItem(QString());
        kcfg_EndTimeUsage->addItem(QString());
        kcfg_EndTimeUsage->addItem(QString());
        kcfg_EndTimeUsage->setObjectName("kcfg_EndTimeUsage");
        sizePolicy.setHeightForWidth(kcfg_EndTimeUsage->sizePolicy().hasHeightForWidth());
        kcfg_EndTimeUsage->setSizePolicy(sizePolicy);

        gridLayout->addWidget(kcfg_EndTimeUsage, 5, 1, 1, 1);

        kcfg_ConstraintType = new KComboBox(tab);
        kcfg_ConstraintType->addItem(QString());
        kcfg_ConstraintType->addItem(QString());
        kcfg_ConstraintType->addItem(QString());
        kcfg_ConstraintType->addItem(QString());
        kcfg_ConstraintType->addItem(QString());
        kcfg_ConstraintType->addItem(QString());
        kcfg_ConstraintType->addItem(QString());
        kcfg_ConstraintType->setObjectName("kcfg_ConstraintType");
        sizePolicy.setHeightForWidth(kcfg_ConstraintType->sizePolicy().hasHeightForWidth());
        kcfg_ConstraintType->setSizePolicy(sizePolicy);

        gridLayout->addWidget(kcfg_ConstraintType, 2, 1, 1, 1);

        textLabel3_4 = new QLabel(tab);
        textLabel3_4->setObjectName("textLabel3_4");
        sizePolicy1.setHeightForWidth(textLabel3_4->sizePolicy().hasHeightForWidth());
        textLabel3_4->setSizePolicy(sizePolicy1);
        textLabel3_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3_4->setWordWrap(false);

        gridLayout->addWidget(textLabel3_4, 6, 0, 1, 1);

        kcfg_OptimisticRatio = new QSpinBox(tab);
        kcfg_OptimisticRatio->setObjectName("kcfg_OptimisticRatio");
        kcfg_OptimisticRatio->setMinimum(-99);
        kcfg_OptimisticRatio->setMaximum(0);

        gridLayout->addWidget(kcfg_OptimisticRatio, 11, 1, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        kcfg_Leader = new KLineEdit(tab);
        kcfg_Leader->setObjectName("kcfg_Leader");
        sizePolicy1.setHeightForWidth(kcfg_Leader->sizePolicy().hasHeightForWidth());
        kcfg_Leader->setSizePolicy(sizePolicy1);

        hboxLayout->addWidget(kcfg_Leader);

        chooseLeader = new QPushButton(tab);
        chooseLeader->setObjectName("chooseLeader");

        hboxLayout->addWidget(chooseLeader);


        gridLayout->addLayout(hboxLayout, 0, 1, 1, 1);

        textLabel3 = new QLabel(tab);
        textLabel3->setObjectName("textLabel3");
        sizePolicy1.setHeightForWidth(textLabel3->sizePolicy().hasHeightForWidth());
        textLabel3->setSizePolicy(sizePolicy1);
        textLabel3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3->setWordWrap(false);

        gridLayout->addWidget(textLabel3, 2, 0, 1, 1);

        label = new QLabel(tab);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 10, 0, 1, 1);

        kcfg_Risk = new QComboBox(tab);
        kcfg_Risk->addItem(QString());
        kcfg_Risk->addItem(QString());
        kcfg_Risk->addItem(QString());
        kcfg_Risk->setObjectName("kcfg_Risk");

        gridLayout->addWidget(kcfg_Risk, 10, 1, 1, 1);

        gridLayout->setColumnStretch(1, 1);
        tabWidget->addTab(tab, QString());
        widget = new QWidget();
        widget->setObjectName("widget");
        verticalLayout_2 = new QVBoxLayout(widget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        toolbar = new KToolBar(widget);
        toolbar->setObjectName("toolbar");
        toolbar->setMinimumSize(QSize(0, 10));

        verticalLayout_2->addWidget(toolbar);

        kcfg_Description = new KRichTextWidget(widget);
        kcfg_Description->setObjectName("kcfg_Description");

        verticalLayout_2->addWidget(kcfg_Description);

        tabWidget->addTab(widget, QString());

        verticalLayout->addWidget(tabWidget);

#if QT_CONFIG(shortcut)
        leaderlabel->setBuddy(kcfg_Leader);
        textLabel2_2->setBuddy(kcfg_PessimisticRatio);
        textLabel1_3->setBuddy(kcfg_OptimisticRatio);
        textLabel3_7->setBuddy(kcfg_EstimateType);
        textLabel3_5->setBuddy(kcfg_StartTimeUsage);
        textLabel3_6->setBuddy(kcfg_EndTimeUsage);
        textLabel3_8->setBuddy(kcfg_ExpectedEstimate);
        textLabel3_3->setBuddy(kcfg_ConstraintStartTime);
        textLabel3_4->setBuddy(kcfg_ConstraintEndTime);
        textLabel3->setBuddy(kcfg_ConstraintType);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(kcfg_Leader, chooseLeader);
        QWidget::setTabOrder(chooseLeader, kcfg_ConstraintType);
        QWidget::setTabOrder(kcfg_ConstraintType, kcfg_StartTimeUsage);
        QWidget::setTabOrder(kcfg_StartTimeUsage, kcfg_ConstraintStartTime);
        QWidget::setTabOrder(kcfg_ConstraintStartTime, kcfg_EndTimeUsage);
        QWidget::setTabOrder(kcfg_EndTimeUsage, kcfg_ConstraintEndTime);
        QWidget::setTabOrder(kcfg_ConstraintEndTime, kcfg_EstimateType);
        QWidget::setTabOrder(kcfg_EstimateType, kcfg_ExpectedEstimate);
        QWidget::setTabOrder(kcfg_ExpectedEstimate, kcfg_Unit);
        QWidget::setTabOrder(kcfg_Unit, kcfg_OptimisticRatio);
        QWidget::setTabOrder(kcfg_OptimisticRatio, kcfg_PessimisticRatio);
        QWidget::setTabOrder(kcfg_PessimisticRatio, kcfg_Description);

        retranslateUi(KPlato__ConfigTaskPanelBase);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KPlato__ConfigTaskPanelBase);
    } // setupUi

    void retranslateUi(QWidget *KPlato__ConfigTaskPanelBase)
    {
#if QT_CONFIG(whatsthis)
        leaderlabel->setWhatsThis(tr2i18n("The person responsible for this task.\n"
"\n"
"This is not limited to persons available in a resource group but can be anyone. You can even directly access your address book with the Choose button.", nullptr));
#endif // QT_CONFIG(whatsthis)
        leaderlabel->setText(tr2i18n("Responsible:", nullptr));
        kcfg_EstimateType->setItemText(0, tr2i18n("Effort", nullptr));
        kcfg_EstimateType->setItemText(1, tr2i18n("Duration", nullptr));

        textLabel2_2->setText(tr2i18n("Pessimistic estimate:", nullptr));
        textLabel1_3->setText(tr2i18n("Optimistic estimate:", nullptr));
        kcfg_StartTimeUsage->setItemText(0, tr2i18n("Current date and time", nullptr));
        kcfg_StartTimeUsage->setItemText(1, tr2i18n("Current date, specified time", nullptr));
        kcfg_StartTimeUsage->setItemText(2, tr2i18n("Specified date and time", nullptr));

        textLabel3_7->setText(tr2i18n("Estimate type:", nullptr));
        textLabel3_5->setText(tr2i18n("Constraint Start time usage:", nullptr));
        kcfg_PessimisticRatio->setSuffix(tr2i18n("%", nullptr));
        textLabel3_6->setText(tr2i18n("Constraint End time usage:", nullptr));
        textLabel3_8->setText(tr2i18n("Expected estimate:", nullptr));
        textLabel3_3->setText(tr2i18n("Constraint start time:", nullptr));
        kcfg_EndTimeUsage->setItemText(0, tr2i18n("Current date and time", nullptr));
        kcfg_EndTimeUsage->setItemText(1, tr2i18n("Current date, specified time", nullptr));
        kcfg_EndTimeUsage->setItemText(2, tr2i18n("Specified date and time", nullptr));

        kcfg_ConstraintType->setItemText(0, tr2i18n("As Soon as Possible", nullptr));
        kcfg_ConstraintType->setItemText(1, tr2i18n("As Late as Possible", nullptr));
        kcfg_ConstraintType->setItemText(2, tr2i18n("Must Start On", nullptr));
        kcfg_ConstraintType->setItemText(3, tr2i18n("Must Finish On", nullptr));
        kcfg_ConstraintType->setItemText(4, tr2i18n("Start Not Earlier Than", nullptr));
        kcfg_ConstraintType->setItemText(5, tr2i18n("Finish Not Later Than", nullptr));
        kcfg_ConstraintType->setItemText(6, tr2i18n("Fixed Interval", nullptr));

        textLabel3_4->setText(tr2i18n("Constraint end time:", nullptr));
        kcfg_OptimisticRatio->setSuffix(tr2i18n("%", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_Leader->setWhatsThis(tr2i18n("The person responsible for this task.\n"
"\n"
"This is not limited to persons available in a resource group but can be anyone. You can even directly access your address book with the Choose button.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        chooseLeader->setToolTip(tr2i18n("Insert a person from your address book", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        chooseLeader->setWhatsThis(tr2i18n("Insert a person from your address book.", nullptr));
#endif // QT_CONFIG(whatsthis)
        chooseLeader->setText(tr2i18n("Choose...", nullptr));
#if QT_CONFIG(shortcut)
        chooseLeader->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        textLabel3->setText(tr2i18n("Scheduling constraint:", nullptr));
        label->setText(tr2i18n("Risk:", nullptr));
        kcfg_Risk->setItemText(0, tr2i18n("None", nullptr));
        kcfg_Risk->setItemText(1, tr2i18n("Low", nullptr));
        kcfg_Risk->setItemText(2, tr2i18n("High", nullptr));

        tabWidget->setTabText(tabWidget->indexOf(tab), tr2i18n("General", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(widget), tr2i18n("Description", nullptr));
        (void)KPlato__ConfigTaskPanelBase;
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class ConfigTaskPanelBase: public Ui_ConfigTaskPanelBase {};
} // namespace Ui
} // namespace KPlato

#endif // KPTCONFIGTASKPANELBASE_H

