/****************************************************************************
** Meta object code from reading C++ file 'kpttaskprogresspanel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../calligraplan-4.0.1/src/libs/ui/kpttaskprogresspanel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kpttaskprogresspanel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KPlato21TaskProgressPanelImplE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::TaskProgressPanelImpl::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato21TaskProgressPanelImplE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::TaskProgressPanelImpl",
        "changed",
        "",
        "slotChanged",
        "slotEditmodeChanged",
        "idx",
        "slotStartedChanged",
        "state",
        "slotFinishedChanged",
        "slotPercentFinishedChanged",
        "value",
        "slotStartTimeChanged",
        "dt",
        "slotFinishTimeChanged",
        "slotEntryChanged",
        "slotSelectionChanged",
        "QItemSelection",
        "sel",
        "slotPrevWeekBtnClicked",
        "slotNextWeekBtnClicked",
        "updateResourceCombo",
        "slotEffortChanged",
        "date",
        "slotCalculateEffort",
        "slotFillWeekNumbers",
        "year",
        "updateFinishedDateTime"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotChanged'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditmodeChanged'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'slotStartedChanged'
        QtMocHelpers::SlotData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'slotFinishedChanged'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'slotPercentFinishedChanged'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'slotStartTimeChanged'
        QtMocHelpers::SlotData<void(const QDateTime &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDateTime, 12 },
        }}),
        // Slot 'slotFinishTimeChanged'
        QtMocHelpers::SlotData<void(const QDateTime &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDateTime, 12 },
        }}),
        // Slot 'slotEntryChanged'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotPrevWeekBtnClicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNextWeekBtnClicked'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateResourceCombo'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEffortChanged'
        QtMocHelpers::SlotData<void(const QDate &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDate, 22 },
        }}),
        // Slot 'slotCalculateEffort'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotFillWeekNumbers'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'updateFinishedDateTime'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaskProgressPanelImpl, qt_meta_tag_ZN6KPlato21TaskProgressPanelImplE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::TaskProgressPanelImpl::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato21TaskProgressPanelImplE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato21TaskProgressPanelImplE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato21TaskProgressPanelImplE_t>.metaTypes,
    nullptr
} };

void KPlato::TaskProgressPanelImpl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskProgressPanelImpl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: _t->slotChanged(); break;
        case 2: _t->slotEditmodeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->slotStartedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->slotFinishedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->slotPercentFinishedChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->slotStartTimeChanged((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1]))); break;
        case 7: _t->slotFinishTimeChanged((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1]))); break;
        case 8: _t->slotEntryChanged(); break;
        case 9: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 10: _t->slotPrevWeekBtnClicked(); break;
        case 11: _t->slotNextWeekBtnClicked(); break;
        case 12: _t->updateResourceCombo(); break;
        case 13: _t->slotEffortChanged((*reinterpret_cast<std::add_pointer_t<QDate>>(_a[1]))); break;
        case 14: _t->slotCalculateEffort(); break;
        case 15: _t->slotFillWeekNumbers((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->updateFinishedDateTime(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TaskProgressPanelImpl::*)()>(_a, &TaskProgressPanelImpl::changed, 0))
            return;
    }
}

const QMetaObject *KPlato::TaskProgressPanelImpl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::TaskProgressPanelImpl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato21TaskProgressPanelImplE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui_TaskProgressPanelBase"))
        return static_cast< Ui_TaskProgressPanelBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int KPlato::TaskProgressPanelImpl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void KPlato::TaskProgressPanelImpl::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN6KPlato17TaskProgressPanelE_t {};
} // unnamed namespace

template <> constexpr inline auto KPlato::TaskProgressPanel::qt_create_metaobjectdata<qt_meta_tag_ZN6KPlato17TaskProgressPanelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPlato::TaskProgressPanel",
        "slotWeekNumberChanged",
        "",
        "slotEntryAdded",
        "date",
        "slotEntryRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotWeekNumberChanged'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotEntryAdded'
        QtMocHelpers::SlotData<void(const QDate &)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QDate, 4 },
        }}),
        // Slot 'slotEntryRemoved'
        QtMocHelpers::SlotData<void(const QDate &)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QDate, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaskProgressPanel, qt_meta_tag_ZN6KPlato17TaskProgressPanelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPlato::TaskProgressPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TaskProgressPanelImpl::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17TaskProgressPanelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17TaskProgressPanelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KPlato17TaskProgressPanelE_t>.metaTypes,
    nullptr
} };

void KPlato::TaskProgressPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskProgressPanel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotWeekNumberChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->slotEntryAdded((*reinterpret_cast<std::add_pointer_t<QDate>>(_a[1]))); break;
        case 2: _t->slotEntryRemoved((*reinterpret_cast<std::add_pointer_t<QDate>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KPlato::TaskProgressPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPlato::TaskProgressPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KPlato17TaskProgressPanelE_t>.strings))
        return static_cast<void*>(this);
    return TaskProgressPanelImpl::qt_metacast(_clname);
}

int KPlato::TaskProgressPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TaskProgressPanelImpl::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
