/********************************************************************************
** Form generated from reading UI file 'standardworktimedialogbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STANDARDWORKTIMEDIALOGBASE_H
#define UI_STANDARDWORKTIMEDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KPlato {

class Ui_StandardWorktimeDialogBase
{
public:
    QGridLayout *gridLayout;
    QLabel *textLabel1;
    QDoubleSpinBox *year;
    QLabel *textLabel2;
    QDoubleSpinBox *month;
    QLabel *textLabe3;
    QDoubleSpinBox *week;
    QLabel *textLabel4;
    QDoubleSpinBox *day;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPlato__StandardWorktimeDialogBase)
    {
        if (KPlato__StandardWorktimeDialogBase->objectName().isEmpty())
            KPlato__StandardWorktimeDialogBase->setObjectName("KPlato__StandardWorktimeDialogBase");
        KPlato__StandardWorktimeDialogBase->resize(241, 108);
        gridLayout = new QGridLayout(KPlato__StandardWorktimeDialogBase);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName("gridLayout");
        textLabel1 = new QLabel(KPlato__StandardWorktimeDialogBase);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);
        textLabel1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        year = new QDoubleSpinBox(KPlato__StandardWorktimeDialogBase);
        year->setObjectName("year");
        year->setMinimum(1.000000000000000);
        year->setMaximum(8784.000000000000000);
        year->setValue(1760.000000000000000);

        gridLayout->addWidget(year, 0, 1, 1, 1);

        textLabel2 = new QLabel(KPlato__StandardWorktimeDialogBase);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);
        textLabel2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(textLabel2, 1, 0, 1, 1);

        month = new QDoubleSpinBox(KPlato__StandardWorktimeDialogBase);
        month->setObjectName("month");
        month->setMinimum(1.000000000000000);
        month->setMaximum(744.000000000000000);
        month->setSingleStep(1.000000000000000);
        month->setValue(176.000000000000000);

        gridLayout->addWidget(month, 1, 1, 1, 1);

        textLabe3 = new QLabel(KPlato__StandardWorktimeDialogBase);
        textLabe3->setObjectName("textLabe3");
        textLabe3->setWordWrap(false);
        textLabe3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(textLabe3, 2, 0, 1, 1);

        week = new QDoubleSpinBox(KPlato__StandardWorktimeDialogBase);
        week->setObjectName("week");
        week->setMinimum(1.000000000000000);
        week->setMaximum(168.000000000000000);
        week->setValue(40.000000000000000);

        gridLayout->addWidget(week, 2, 1, 1, 1);

        textLabel4 = new QLabel(KPlato__StandardWorktimeDialogBase);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);
        textLabel4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(textLabel4, 3, 0, 1, 1);

        day = new QDoubleSpinBox(KPlato__StandardWorktimeDialogBase);
        day->setObjectName("day");
        day->setMinimum(1.000000000000000);
        day->setMaximum(24.000000000000000);
        day->setValue(8.000000000000000);

        gridLayout->addWidget(day, 3, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 4, 0, 1, 2);

        gridLayout->setColumnStretch(1, 1);
#if QT_CONFIG(shortcut)
        textLabel1->setBuddy(year);
        textLabel2->setBuddy(month);
        textLabe3->setBuddy(week);
        textLabel4->setBuddy(day);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KPlato__StandardWorktimeDialogBase);

        QMetaObject::connectSlotsByName(KPlato__StandardWorktimeDialogBase);
    } // setupUi

    void retranslateUi(QWidget *KPlato__StandardWorktimeDialogBase)
    {
#if QT_CONFIG(whatsthis)
        KPlato__StandardWorktimeDialogBase->setWhatsThis(tr2i18n("These values are used if your estimate is not in hours.\n"
"Let us say Hours per day is 8. If you estimate that a task needs 3 days effort to complete, this is converted to 24 hours when the task is scheduled. The actual time it will take to complete is of course dependent on the availability of the person (or persons) that is assigned to the task.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1->setText(tr2i18n("Hours per year:", nullptr));
        textLabel2->setText(tr2i18n("Hours per month:", nullptr));
        textLabe3->setText(tr2i18n("Hours per week:", nullptr));
#if QT_CONFIG(whatsthis)
        textLabel4->setWhatsThis(tr2i18n("These values are used when you estimate the effort needed to complete a task.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel4->setText(tr2i18n("Hours per day:", nullptr));
    } // retranslateUi

};

} // namespace KPlato

namespace KPlato {
namespace Ui {
    class StandardWorktimeDialogBase: public Ui_StandardWorktimeDialogBase {};
} // namespace Ui
} // namespace KPlato

#endif // STANDARDWORKTIMEDIALOGBASE_H

